/*
 * Main.java
 *
 * only Xerces and Saxon is supported
 */

package at.ac.tuwien.asgaard.delta;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.misc.SplashWindow;
import at.ac.tuwien.asgaard.delta.util.EnvironmentProperties;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.util.Arguments;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.JHelpUtils;
import at.ac.tuwien.asgaard.util.Log;

import java.io.*;
import java.util.Locale;

import javax.swing.*;

/**
 * Main class of the DELTA. Parses the command line arguments and "loads" the main window
 * of the app while showing a splash screen.
 * @author  Peter VOTRUBA
 * @version 2.7.3
 */
public final class Main {

	/** Main method of the main class
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		String preferencesFilename = Preferences.DEFAULT_USER_PREFERENCES_FILE, projectFilename = null;
		boolean defaultPrefs = false;

		try {
			System.setSecurityManager(null); // makes DELTA probably faster AND there should not be problems with JFileChooser anymore
		} catch (SecurityException exc) {
			Log.trace("Couldn't remove SecurityManager!"); // doesn't really matter
		}
		Locale.setDefault(Locale.ENGLISH);

		PrintStream logWriter = null;
		try {
			Arguments arguments = new Arguments();
			arguments.parseArgumentTokens(args, new char[]{'p', 'P', 'm','l','r','h','?','L'});

			boolean badParam = false;
			for (char c = (char)arguments.getArgument(); c != (char) -1; c = (char) arguments.getArgument()) {
				switch (c) {
					case 'p':
						projectFilename = arguments.getStringParameter();
						if (projectFilename != null)
							System.out.println("Using \""+projectFilename+"\" as startup project.\n");
						else
							System.err.println("  -p needs a parameter (<projectfile>)!");
						break;
					case 'P':
						preferencesFilename = arguments.getStringParameter();
						if (preferencesFilename != null)
							System.out.println("Using \""+preferencesFilename+"\" as preferences file.");
						 else
							System.err.println("  -P needs a parameter (<preferencesfile>)!");
						break;
					case 'd':
						defaultPrefs = true;
						System.out.println("Using default preferences.\n");
						break;
					case 'm': {
						Reader reader = new BufferedReader(
								new InputStreamReader(Main.class.getResourceAsStream("gui/macros/macros.dtd")));
						FileUtils.pipeReaderToWriter(reader, new OutputStreamWriter(System.out));
						System.exit(0);
						break; }
					case 'l': {
						Reader reader = new BufferedReader(
								new InputStreamReader(Main.class.getResourceAsStream("xmlBeans/xmlLanguage/xmlLanguage.dtd")));
						FileUtils.pipeReaderToWriter(reader, new OutputStreamWriter(System.out));
						System.out.println();
						System.exit(0); }
						break;
					case 'r': {
						Reader reader = new BufferedReader(
								new InputStreamReader(Main.class.getResourceAsStream("xmlBeans/project.dtd")));
						FileUtils.pipeReaderToWriter(reader, new OutputStreamWriter(System.out));
						System.exit(0); }
						break;
					case 'h':
					case '?':
						showHelp();
						System.exit(0);
						break;
					case 'L':
					case 'D':
					case 'S':
					case 'T':
					case 'X':
						at.ac.tuwien.asgaard.delta.util.Transform.main(args);
						System.exit(0);
						break;
					default:
						badParam = true;
						System.out.println("unknown option: '" + c + '\'');
				}
			}
			String rest;
			while ((rest = arguments.getlistFiles()) != null)
				if ((rest != null) && (rest.trim().length() != 0)) {
					badParam = true;
					System.out.println("unknown argument: \"" + rest + '\"');
				}

			if (badParam) {
				showHelp();
				System.exit(1);
			}

			SplashWindow.showSplash();

		// checkXerces();
		// checkSaxon();
			System.setProperty("javax.xml.parsers.SAXParserFactory", "org.apache.xerces.jaxp.SAXParserFactoryImpl");
			System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
			System.setProperty("org.xml.sax.driver", "org.apache.xerces.parsers.SAXParser");
			System.setProperty("javax.xml.transform.TransformerFactory", "net.sf.saxon.TransformerFactoryImpl");

			System.out.println();

			if (preferencesFilename.indexOf('/') == -1)
				preferencesFilename = FileUtils.currentDir + '/' + preferencesFilename;

			final Preferences preferences = new Preferences(preferencesFilename, true, !defaultPrefs);

			final Preferences stateProperties = new Preferences(FileUtils.currentDir + '/' + Preferences.STATE_PROPERTIES_FILE,
											  false, preferences.loadPreferences);

			String looknfeel = System.getProperty("swing.defaultlaf");
			try {
				UIManager.getLookAndFeel();
			} catch (Throwable t) {
				System.err.println("Couldn't set Look'n'Feel \"" + looknfeel + "\", use default LaF instead.");
				looknfeel = UIManager.getCrossPlatformLookAndFeelClassName();
				UIManager.setLookAndFeel(looknfeel);
			}
			if (looknfeel == null) {
				looknfeel = stateProperties.getString(Preferences.KEY_APPEARANCE_PREVIOUS_LAF);
				if (looknfeel != null)
					try {
						UIManager.setLookAndFeel(looknfeel);
						System.out.println("Using \"" + looknfeel + "\" Look'n'Feel.");
					} catch (Throwable t) {
						System.err.println("Couldn't set previous Look'n'Feel \"" + looknfeel + "\", use default LaF instead.");
						looknfeel = UIManager.getCrossPlatformLookAndFeelClassName();
					}
			}
			stateProperties.putString(Preferences.KEY_APPEARANCE_PREVIOUS_LAF, looknfeel);

			if (EnvironmentProperties.isIsJhelpAvailable())
				JHelpUtils.startHelpsetLoader("at/ac/tuwien/asgaard/delta/help/delta.hs");
			else
				System.out.println("\njhelp.jar is missing - help will not be available.\n");

			if (preferences.getBoolean("general.log")) {
				final String logFile = FileUtils.currentDir + '/' + preferences.getString("general.log.file");
				System.out.println("Redirect output to \"" + logFile + "\" !");
				try {
					logWriter = new PrintStream(new FileOutputStream(logFile));
					System.setOut(logWriter);
					System.setErr(logWriter);
				} catch (Exception exc) {
					System.out.println("  Couldn't open logfile \"" + logFile + "\" !");
				}
			}

			MainController.init(preferences, stateProperties, projectFilename);
			SplashWindow.closeSplash();
			MainController.startApp();
			System.out.println();
		} catch (Throwable t) {
			System.err.println("\nFatal Error:");
			if (t instanceof ExceptionInInitializerError) {
				System.err.println("  ExceptionInInitializerError:");
				t = ((ExceptionInInitializerError)t).getException();
			}
			t.printStackTrace(System.err);
			SplashWindow.closeSplash();
			JOptionPane.showMessageDialog(null, "Fatal Error!\n" + t.getMessage() + "\n(" + t.getClass().getName() + ')', "Fatal Error", JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		} finally {
			if (logWriter != null) {
				logWriter.flush();
				logWriter.close();
			}
		}
	}

	private static void showHelp() {
		System.out.println("Document Exploration and Linking Tool (with Addons) 2.7.3 - #452, by Peter Votruba");
		System.out.println("Usage:");
		System.out.println("  java -jar delta.jar [options]");
		System.out.println("  options:");
		System.out.println("    -p <projectfile>");
		System.out.println("    -P <preferencesfile>");
		System.out.println("    -d ... use default preferences");
		System.out.println("    -h ... show this help");
		System.out.println("    -m ... print Macros DTD");
		System.out.println("    -l ... print XML-Language DTD");
		System.out.println("    -r ... print pRoject DTD");
		System.out.println("    -L <sourceDTD> [<link enriched DTD>] ... add DELT/A-Links to DTD");
		System.out.println("    -D <sourceDTD> [<HTML result>] ... transform DTD to HTML documentation");
		System.out.println("    -S <sourceDTD> [<HTML result>] ... transform DTD to syntax-highlighted source (HTML)");
		System.out.println("    -T <sourceXML> <targetXML> <XSL file> ... transform <sourceXML> to <targetXML> using <XSL file>");
        System.out.println("    -X <sourceXML> <targetHTML> ... transform XML file <sourceXML> to formatted source <targetHTML>");
		System.out.println();
	}

    /*private static void checkXerces() {
        try {
            Class.forName("org.apache.xerces.parsers.SAXParser");
            Class.forName("org.apache.xerces.jaxp.SAXParserFactoryImpl");
            Class.forName("org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
            System.setProperty("javax.xml.parsers.SAXParserFactory", "org.apache.xerces.jaxp.SAXParserFactoryImpl");
            System.setProperty("javax.xml.parsers.DocumentBuilderFactory", "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl");
            System.setProperty("org.xml.sax.driver", "org.apache.xerces.parsers.SAXParser");
            System.out.println("Apache's \"Xerces\" XML-Parser found.");
        } catch (Throwable t) {
            throw new NoClassDefFoundError("Couldn't find Apache's \"Xerces\" XML-Parser!");
        }
    }

    private static void checkSaxon() {
        try {
            // Saxon 6.x
            //        Class.forName("com.icl.saxon.TransformerFactoryImpl");
            //        System.setProperty("javax.xml.transform.TransformerFactory", "com.icl.saxon.TransformerFactoryImpl");
            //     System.out.println("\"" + com.icl.saxon.Version.getProductName() + "\" XSLT stylesheet processor found.");
            // Saxon 7.x -> works only with Java 1.4.x
            Class.forName("net.sf.saxon.TransformerFactoryImpl");
            System.setProperty("javax.xml.transform.TransformerFactory", "net.sf.saxon.TransformerFactoryImpl");
            System.out.println('\"' + net.sf.saxon.Version.getProductTitle() + "\" XSLT stylesheet processor found.");
        } catch (Throwable t) {
            throw new NoClassDefFoundError("Couldn't find Saxon XSLT stylesheet processor!");
        }
    }*/

	private Main() {} // this class should not be instantiated
}
