/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 16.05.2003
 * Time: 11:43:10
 */
package at.ac.tuwien.asgaard.delta.gui;

import at.ac.tuwien.asgaard.delta.gui.actions.EmptyAction;
import at.ac.tuwien.asgaard.delta.gui.actions.edit.*;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FileCloseAllAction;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FileExitAction;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FileSaveAllAction;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FileSwapSidesAction;
import at.ac.tuwien.asgaard.delta.gui.actions.help.*;
import at.ac.tuwien.asgaard.delta.gui.actions.macros.*;
import at.ac.tuwien.asgaard.delta.gui.actions.options.OptionsLanguageShowPropertiesDialogAction;
import at.ac.tuwien.asgaard.delta.gui.actions.options.OptionsPreferencesShowDialogAction;
import at.ac.tuwien.asgaard.delta.gui.actions.options.OptionsProcessTemplatesShowDialogAction;
import at.ac.tuwien.asgaard.delta.gui.actions.project.*;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.*;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideAwareUndoManager;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.xml.StyleInfo;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.xmlBeans.DefaultSAXErrorHandler;
import at.ac.tuwien.asgaard.delta.xmlBeans.Project;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.ProcessTemplate;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.ProcessTemplates;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.StepTemplate;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.View;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.ButtonPopupMenuHandler;
import at.ac.tuwien.asgaard.swing.PopupMenuHandler;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.selectable.SelectableAction;
import at.ac.tuwien.asgaard.swing.selectable.SelectableRadioButtonMenuItem;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.DocTypeFileFilter;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelperXML4J;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.undo.*;

/**
 * Class MainController ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class MainController implements GUIConstants {
	public static MainFrame.DeltaSide[] deltaSides;
	public static DeltaSideController[] deltaSideController;

	public static List<XmlLanguage> xmlLanguages;
	public static ProcessTemplates processTemplates;

	public static MainFrame mainFrame;
	public static Project currentProject;
	public static at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process currentProcess;
	public static at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.ProcessTemplate currentProcessTemplate;
	public static int currentProcessStep = 0;
	private static View currentProcessView;

	private static int maxRecentFiles;
	public static int removeDeadLinks, changeDocType;
	public static boolean askBeforeRemoveHTMLLinks, askBeforeRemoveXMLNodes, askBeforeRemoveXMLLinks,
							askBeforeRemoveXMLSubtree, askBeforeRemoveXMLAttributes;
	static boolean macroSourceViewNeedsUpdate = true, macroTextViewNeedsUpdate = true;
	public static List<File> recentXHTMLFiles,recentDirs, recentMacrosFiles, recentProjectFiles;
	public static Action aOpenInMenu, aFileSaveAll, aFileCloseAll, aFileSwapSides;

	public static Action aProjectNew, aProjectClose, aProjectShowPropertiesDialog,
			aMenuProjectReopen, aMenuProjectProcessTemplates, aMenuProjectSteps;
	public static ProjectOpenAction aProjectOpen;

	public static Action aOptionsShowPreferencesDialog, aOptionsShowProcessTemplatesDialog;
	public static OptionsLanguageShowPropertiesDialogAction aOptionsLanguageShowPropertiesDialog;

	public static FileExitAction aFileExit;
	public static EditUndoAction aEditUndo;
	public static EditRedoAction aEditRedo;
	public static Action aEditCut, aEditCopy, aEditPasteInto, aEditPasteBefore, aEditPasteAfter, aEditPasteReplace,
							aEditFind, aEditFindNext, aEditFindPrevious, aEditHighlightSearchResults;
	public static MacrosOpenAction aMacrosOpen;
	public static MacrosCloseAction aMacrosClose;
	public static Action aMacrosReload, aMacrosOpenAsXML, aMacrosOpenXMLInMacrosView, aMacrosGoBack, aMacrosGoForward, aMacrosReopen, aMacroInsert, aMacrosShowEditor;
	public static SelectableAction aMacrosSwitchToStructureView, aMacrosSwitchToSearchView, aMacrosSwitchToInsertableView, aMacroSwitchToTreeView, aMacroSwitchToSourceView, aMacroSwitchToTextView;

	// Help-Menu actions
	public static Action aHelpShowHelp, aHelpWWWOpenDELTASite, aHelpWWWOpenDELTAManual,
							aHelpWWWOpenAsgaardSite, aHelpCheckVersion, aHelpRefreshGUI, aHelpResetLayout, aHelpShowAbout;
	public static Action aXMLLanguagesSubmenu, aTemplates, aNewFromOtherLang;
	public static final File XML_LANGUAGES_BASE_DIR = new File(FileUtils.currentDirFile, "xmlLanguages");
	private static XMLLanguageDialog xmlLanguageDialog;

	public static XMLClipBoard xmlClipBoard;

	public static Preferences preferences, stateProperties;

	public static InsertableXMLNodesDialog insertableElementsDialog;
	public static Action aEditShowInsertableElementsDialog;
	public static AllElementsDialog allElementsDialog;
	public static Action aEditShowAllElementsDialog;

	static ActionListener recentMacrosOpenActionListener, recentProjectOpenActionListener;

	public static final	SelectableAction aMacroInsertModeInto	 = new MacroSwitchToInsertModeIntoAction();
	public static final	SelectableAction aMacroInsertModeBefore	 = new MacroSwitchToInsertModeBeforeAction();
	public static final	SelectableAction aMacroInsertModeAfter	 = new MacroSwitchToInsertModeAfterAction();
	public static final	SelectableAction aMacroInsertModeReplace = new MacroSwitchToInsertModeReplaceAction();

	public static Action aMacroSearchInsertUp;
	public static Action aMacroSearchInsertDown;

	public static SideAwareUndoManager undoManager;
	public static final boolean AUTO_SAVE_AFTER_DOCTYPE_ADAPTION = true; // todo move to prefs

	public static boolean ADVANCED_MODE; // = Boolean.valueOf(System.getProperty("light")).booleanValue();
	public static final String PROCESS_TEMPLATES_XML_FILE = "processTemplates.xml";
	public static final File PROCESS_TEMPLATES_FILE = new File(FileUtils.currentDirFile, PROCESS_TEMPLATES_XML_FILE);

	private MainController() { } // this class should not be instantiated

	public static void init(Preferences preferences, Preferences stateProperties, String projectFilename) {
		MainController.preferences = preferences;
		MainController.stateProperties = stateProperties;

		activeSideIndex = EMPTY;

		ADVANCED_MODE = preferences.getBoolean(Preferences.KEY_ADVANCED_MODE);

		System.setProperty("swing.aatext", "true");
		UIManager.put("MenuItemUI", "at.ac.tuwien.asgaard.swing.plaf.LocalAcceleratorMenuItemUI");
		UIManager.put("CheckBoxMenuItemUI", "at.ac.tuwien.asgaard.swing.plaf.LocalAcceleratorCheckBoxMenuItemUI");
		UIManager.put("RadioButtonMenuItemUI", "at.ac.tuwien.asgaard.swing.plaf.LocalAcceleratorRadioButtonMenuItemUI");
		UIManager.put("TreeUI", "at.ac.tuwien.asgaard.swing.plaf.NoToggleSelectionTreeUI");
		UIManager.put("ToolTipUI", "at.ac.tuwien.asgaard.swing.plaf.ExtendedToolTipUI");
		UIManager.put("ToolTip.background", GlobalObjects.INACTIVE_SIDE_BACKGROUND);
		ToolTipManager.sharedInstance().setReshowDelay(750);  // increase reshow delay, default is 500 ms
		Toolkit.getDefaultToolkit().setDynamicLayout(preferences.getBoolean(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT));

		fetchProperties();

		recentXHTMLFiles   = stateProperties.getFileList(Preferences.KEY_RECENT_XHTML_FILES);
		recentMacrosFiles  = stateProperties.getFileList(Preferences.KEY_RECENT_MACROS_FILES);
		recentProjectFiles = stateProperties.getFileList(Preferences.KEY_RECENT_PROJECT_FILES);
		recentDirs		   = stateProperties.getFileList(Preferences.KEY_RECENT_DIRS);

		undoManager = new SideAwareUndoManager();
		xmlLanguages = loadXMLLanguages();
		processTemplates = loadProcessTemplates();

		deltaSideController = new DeltaSideController[] {new DeltaSideController(LEFT), new DeltaSideController(RIGHT)};
		mainFrame = new MainFrame(preferences, stateProperties);
		deltaSides = mainFrame.deltaSides;

		initActions();

		deltaSideController[LEFT].init();
		deltaSideController[RIGHT].init();

		mainFrame.installActions();
		installListeners();
		deltaSideController[LEFT].postInit();
		deltaSideController[RIGHT].postInit();

		buildRecentMacrosFilesMenus();
		buildRecentProjectFilesMenu();
		buildRecentDirsMenus();
		buildProcessTemplatesMenu();

		loadLastFiles(projectFilename);
	}

	public static void buildProcessTemplatesMenu() {
		mainFrame.jMenuProjectProcessTemplates.removeAll();
		if (processTemplates.getProcessTemplateList().isEmpty())
			mainFrame.jMenuProjectProcessTemplates.add(EMPTY_MENU_ACTION);
		else
			for (int i = 0; i < processTemplates.getProcessTemplateList().size(); i++) {
				ProcessTemplate processTemplate = processTemplates.getProcessTemplate(i);
			//	System.out.println("Found process-template: " + processTemplate.getName());
				Action action = new ProjectNewProcessBasedOnTemplateAction(processTemplate, i, PROCESS_TEMPLATES_FILE);
				mainFrame.jMenuProjectProcessTemplates.add(action);
			}
		mainFrame.statusBar.attachToMenuElement(mainFrame.jMenuProjectProcessTemplates);

		if (currentProcess != null) {
			String templateName = currentProcess.getTemplate();
			for (int i = 0; i < processTemplates.getProcessTemplateList().size(); i++) {
				ProcessTemplate temp = processTemplates.getProcessTemplate(i);
				if (temp.getName().equalsIgnoreCase(templateName)) {
					currentProcessTemplate = temp;
					break;
				}
			}

			if (currentProcessTemplate == null) {
				mainFrame.jMenuProjectSteps.setEnabled(false);
				JOptionPane.showMessageDialog(mainFrame, "Process template \"" + templateName + "\" of current process not found!");
			} else {
				mainFrame.jMenuProjectSteps.setEnabled(true);
				buildProcessStepMenu(currentProcessStep, currentProcessView, false);
			}
		}
	}

	private static void fetchProperties() {
		maxRecentFiles = preferences.getInt(Preferences.KEY_RECENT_FILES_MAX_COUNT);
		removeDeadLinks = preferences.getInt(Preferences.KEY_REMOVE_DEAD_LINKS);
		askBeforeRemoveHTMLLinks = preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_HTML_LINKS);

		askBeforeRemoveXMLNodes = preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_NODES);
		askBeforeRemoveXMLLinks = preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_LINKS);
		askBeforeRemoveXMLSubtree = preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_SUBTREE);
		askBeforeRemoveXMLAttributes = preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES);

		changeDocType = preferences.getInt(Preferences.KEY_XML_AUTO_CHANGE_DOCTYPE);

		StyleInfo.fetchProperties(preferences);
	}

	private static void initActions() {
		aFileSaveAll	 = new FileSaveAllAction();
		aFileCloseAll	 = new FileCloseAllAction();
		aFileSwapSides	 = new FileSwapSidesAction();
		aFileExit = new FileExitAction();

		aProjectNew	 = new ProjectNewAction();
		aProjectOpen = new ProjectOpenAction();

		aMenuProjectReopen = new EmptyAction("Reopen", MainImages.REOPEN_ICON, 'R', "Reopen Project",
										 "List of previously opened projects.");
		aMenuProjectProcessTemplates = new EmptyAction("New Process based on Template", MainImages.PROCESS_ICON, 'T', "New Transformation Process", "List of defined transformation process templates.");
		aMenuProjectSteps = new EmptyAction("Process Steps/Views", MainImages.APPLICABLES_ICON, 'S', "Change process step/view", "Change current process step/view.");

		aOpenInMenu = new EmptyAction("Open in", MainImages.OPENED_FOLDER_ICON, 'I', "Open File in recent directory",
									  "Open File in recent directory.");

		aProjectClose = new ProjectCloseAction();
		aProjectShowPropertiesDialog = new ProjectShowPropertiesDialogAction();

		aOptionsLanguageShowPropertiesDialog = new OptionsLanguageShowPropertiesDialogAction();
		aOptionsShowPreferencesDialog = new OptionsPreferencesShowDialogAction();
		aOptionsShowProcessTemplatesDialog = new OptionsProcessTemplatesShowDialogAction();

		aEditUndo						  =	new	EditUndoAction();
		aEditRedo						  =	new	EditRedoAction();
		aEditCut						  =	new	EditCutAction();
		aEditCopy						  =	new	EditCopyAction();
		aEditPasteInto					  =	new	EditPasteIntoAction();
		aEditPasteBefore				  =	new	EditPasteBeforeAction();
		aEditPasteAfter					  =	new	EditPasteAfterAction();
		aEditPasteReplace				  =	new	EditPasteReplaceAction();
		aEditFind						  =	new	EditFindAction();
		aEditFindNext					  =	new	EditFindNextAction();
		aEditFindPrevious				  =	new	EditFindPreviousAction();
		aEditHighlightSearchResults		  =	new	EditHighlightSearchResultsAction();
		aEditShowInsertableElementsDialog =	new	EditShowInsertableElementsDialogAction();
		aEditShowAllElementsDialog		  =	new	EditShowAllElementsDialogAction();

		aMacrosOpen					  =	new	MacrosOpenAction();
		aMacrosReopen				  =	new	EmptyAction("Reopen", MainImages.REOPEN_ICON, 'E', "Reopen Macros file",
				"List of previously opened Macros files.");
		aMacrosReload				  =	new	MacrosReloadAction();
		aMacrosOpenAsXML			  =	new	MacrosOpenAsXMLAction();
		aMacrosOpenXMLInMacrosView	  =	new	MacrosOpenXMLInMacrosViewAction();
		aMacrosClose				  =	new	MacrosCloseAction();
		aMacrosGoBack				  =	new	MacrosGoBackAction();
		aMacrosGoForward			  =	new	MacrosGoForwardAction();
		aMacroInsert				  =	new	MacroInsertAction();
		aMacrosShowEditor			  =	new	MacroShowEditorAction();
		aMacrosSwitchToStructureView  =	new	MacrosSwitchToStructureViewAction();
		aMacrosSwitchToSearchView	  =	new	MacrosSwitchToSearchViewAction();
		aMacrosSwitchToInsertableView =	new	MacrosSwitchToSearchInsertableAction();
		aMacroSwitchToTreeView		  =	new	MacroSwitchToTreeViewAction();
		aMacroSwitchToSourceView	  =	new	MacroSwitchToSourceViewAction();
		aMacroSwitchToTextView		  =	new	MacroSwitchToTextViewAction();
		aMacroSearchInsertUp		  =	new	MacrosSearchForInsertLocationUpAction();
		aMacroSearchInsertDown		  =	new	MacrosSearchForInsertLocationDownAction();

		aHelpShowHelp  = new HelpShowHelpAction();
		aHelpWWWOpenDELTASite  = new HelpWWWOpenDELTASiteAction();
		aHelpWWWOpenDELTAManual = new HelpWWWOpenDELTAManualAction();
		aHelpWWWOpenAsgaardSite = new HelpWWWOpenAsgaardSiteAction();
		aHelpCheckVersion  = new HelpCheckVersionAction();
		aHelpRefreshGUI  = new HelpRefreshGUIAction();
		aHelpResetLayout  = new HelpResetLayoutAction();
		aHelpShowAbout  = new HelpShowAboutAction();

		aTemplates = new EmptyAction("New based on Template", MainImages.NEW_FROM_TEMPLATE_ICON, 'T', "Create new XML file based on Template",
									 "Create new XML file based on Template of current XML language or from any other XML language");
		aNewFromOtherLang = new EmptyAction("New from other XML-Lang.", MainImages.XML_LANGUAGE_ICON, 'O',
									 "Create new XML file from other XML-Language",
									 "Create new empty XML file or new XML file based on Template from other XML-Language");
	}

	public static void startApp() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				mainFrame.setVisible(true);
				mainFrame.initLayout();
			}
		});
	}

	private static void loadLastFiles(String projectFilename) {
		if ((projectFilename == null) && preferences.getBoolean(Preferences.KEY_OPEN_LAST_PROJECT))
			projectFilename = stateProperties.getFilePath(Preferences.KEY_LAST_PROJECT_FILE);
		if (projectFilename != null) {
			if (projectFilename.endsWith(".process")) {
				File projectFile = FileUtils.toExistingAbsoluteFile(projectFilename, FileUtils.currentDirFile);
				try {
					at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process process =
							at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process.unmarshal(projectFile, true);
					setDELTAProcess(process, true);
				} catch (IOException exc) {
					Log.error("I/O-Error while loading DELT/A-Process XML-File \"" + projectFile.getAbsolutePath() + "\": " +
							  exc.getMessage());
				}
			} else {
				if (!projectFilename.endsWith(".project"))
					projectFilename += ".project";
				Project project = Project.loadSilent(FileUtils.toAbsoluteFile(projectFilename));
				if (project != null)
					setDELTAProject(project, true, true);
			}
		} else
			if (preferences.getBoolean(Preferences.KEY_OPEN_LAST_FILES))
				loadFiles(stateProperties.getFile(Preferences.KEY_LAST_LEFT_FILE),
					  stateProperties.getFile(Preferences.KEY_LAST_RIGHT_FILE),
					  stateProperties.getFile(Preferences.KEY_LAST_MACROS_FILE), true);
	}

	private static void loadFiles(final File leftFile, final File rightFile, final File macrosFile, final boolean onStartUp) {
		System.out.println("load multiple files:");
		System.out.println("  leftFile:  "+leftFile);
		System.out.println("  rightFile: "+ rightFile);
		System.out.println("  macroFile: "+ macrosFile);

		// first check if left or right need to be saved and save it
		if (!MiscUtils.equals(leftFile, deltaSides[LEFT].deltaView.getFile()))
			deltaSideController[LEFT].saveIfChanged(false);
		if (!MiscUtils.equals(rightFile, deltaSides[RIGHT].deltaView.getFile()))
			deltaSideController[RIGHT].saveIfChanged(false);

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (!MiscUtils.equals(leftFile, deltaSides[LEFT].deltaView.getFile()))
					if (leftFile == null) {
						deltaSideController[LEFT].aClose.doCloseFile(false, false);
						deltaSideController[LEFT].unsetCurrentXMLLanguage();
					} else
						deltaSideController[LEFT].aOpen.doOpenFile(leftFile, onStartUp, false);
				else
					deltaSideController[LEFT].aSwitchToViewer.setSelected(true);
				if (!MiscUtils.equals(rightFile, deltaSides[RIGHT].deltaView.getFile()))
					if (rightFile == null) {
						deltaSideController[RIGHT].aClose.doCloseFile(false, false);
						deltaSideController[RIGHT].unsetCurrentXMLLanguage();
					} else
						deltaSideController[RIGHT].aOpen.doOpenFile(rightFile, onStartUp, false);
				else
					deltaSideController[RIGHT].aSwitchToViewer.setSelected(true);
				if (!MiscUtils.equals(macrosFile, mainFrame.macrosView.getFile()))
					if (macrosFile == null)
						aMacrosClose.doCloseFile(false);
					else
						aMacrosOpen.doOpenFileOnStartUp(macrosFile);
				else {
					aMacrosSwitchToStructureView.setSelected(true);
					aMacroSwitchToTreeView.setSelected(true);
				}
			}
		});
	}

	public static void setDELTAProcess(
			at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process newProcess,
			boolean loadFiles) {
		if (currentProcess != null) {
			addRecentFile(recentProjectFiles, currentProcess.getFile());
			buildRecentProjectFilesMenu();
			if (addRecentFile(recentDirs, currentProcess.getFile().getParentFile()))
				buildRecentDirsMenus();
		}
		if (currentProject != null) {
			addRecentFile(recentProjectFiles, currentProject.getProjectFile());
			buildRecentProjectFilesMenu();
			if (addRecentFile(recentDirs, currentProject.getProjectFile().getParentFile()))
				buildRecentDirsMenus();
		}
		currentProject = null;

		aProjectShowPropertiesDialog.setEnabled(true);
		aProjectClose.setEnabled(true);
		int processStep = convertStringToInt(newProcess.getSteps().getActiveStep(), 1, 1, newProcess.getSteps().getStepList().size());

		String processName = newProcess.getName();
		mainFrame.projectNameLabel.setText(processName + " ("+ processStep+")");
		mainFrame.projectNameLabel.setIcon(MainImages.PROCESS_ICON);
		String toolTipText = "<html>Active Process: <b>" + processName + "</b><br>Active step: #" +
							 processStep + " - " + newProcess.getSteps().getStep(processStep-1).getName();
		String description = newProcess.getDescription();
		if (description != null)
			toolTipText = toolTipText + "<br>&nbsp;&nbsp;" + description;
		mainFrame.projectNameLabel.setToolTipText(toolTipText);

		currentProcess = newProcess;

		String templateName = currentProcess.getTemplate();
		for (int i = 0; i < processTemplates.getProcessTemplateList().size(); i++) {
			ProcessTemplate temp = processTemplates.getProcessTemplate(i);
			if (temp.getName().equalsIgnoreCase(templateName)) {
				currentProcessTemplate = temp;
				break;
			}
		}

		currentProcessStep = processStep;

		if (currentProcessTemplate == null) {
			mainFrame.jMenuProjectSteps.setEnabled(false);
			JOptionPane.showMessageDialog(mainFrame, "Process template \""+ templateName+"\" of current process not found!");
		} else {
			mainFrame.jMenuProjectSteps.setEnabled(true);
			buildProcessStepMenu(currentProcessStep, currentProcessView, loadFiles);
		}
	}

	private static void buildProcessStepMenu(int activeProcessStep, View activeView, boolean load) {
		mainFrame.jMenuProjectSteps.removeAll();
		int processStepCount = currentProcess.getSteps().getStepList().size();
		int shownSteps, maxSteps;
		if (currentProcessTemplate == null)
			shownSteps = maxSteps = processStepCount;
		else {
			maxSteps = currentProcessTemplate.getStepTemplateList().size();
			shownSteps = Math.min(processStepCount + 1, maxSteps);
		}
		ButtonGroup buttonGroup = new ButtonGroup();
		for (int i = 1; i <= shownSteps; i++) {
			SelectableAction action =
					new ProjectChangeProcessStepAction(i, currentProcess, currentProcessTemplate,
													   ((i > processStepCount) && (processStepCount < maxSteps)));
			if ((activeView == null) && (i == activeProcessStep))
				if (load)
					action.setSelected(true);
				else
					action.setSelectedState(true);
			buttonGroup.add(mainFrame.jMenuProjectSteps.add(new SelectableRadioButtonMenuItem(action)));
		}
		
		if (currentProcessTemplate != null)
		if (currentProcessTemplate.getViewList().size() > 0) {
			int maxStepIndex = currentProcess.getSteps().getStepList().size();
			List<View> views = new LinkedList<View>();
			for (int i = 0; i < currentProcessTemplate.getViewList().size(); i++) {
				View view = currentProcessTemplate.getView(i);
				int leftSideStepIndex = convertStringToInt(view.getLeftFile().getStepIndex(), -1, 0, maxStepIndex);
				int rightSideStepIndex = convertStringToInt(view.getRightFile().getStepIndex(), -1, 0, maxStepIndex);
				if ((leftSideStepIndex != -1) && (rightSideStepIndex != -1))
					views.add(view);
			}
			if (!views.isEmpty()) {
				mainFrame.jMenuProjectSteps.addSeparator();
				for (View view : views) {
					SelectableAction action = new ProjectChangeProcessViewAction(view);
					if (view == activeView)
						if (load)
							action.setSelected(true);
						else
							action.setSelectedState(true);
					buttonGroup.add(mainFrame.jMenuProjectSteps.add(new SelectableRadioButtonMenuItem(action)));
				}
			}
		}
		mainFrame.statusBar.attachToMenuElement(mainFrame.jMenuProjectSteps);
	}

	private static int convertStringToInt(String s, int def, int min, int max) {
		int result;
		try {
			result = Integer.parseInt(s);
			if ((result > max) || (result < min)) {
			//	System.err.println(result + " is not in the range ["+min+"..."+max+"]!");
				result = def;
			}
			return result;
		} catch (NumberFormatException e) {
		//	System.err.println(s + " is not a valid int!");
			return def;
		}
	}

	// is called by ProjectChangeProcessStepAction
	public static void setDELTAProcessStep(int processStep) {
		currentProcessView = null;
		File leftFile;
		if (processStep == 1)
			leftFile = FileUtils.toExistingAbsoluteFile(currentProcess.getSourceFile(), currentProcess.getFile());
		else
			leftFile =
					FileUtils.toExistingAbsoluteFile(currentProcess.getSteps().getStep(processStep - 2).getTargetFile(), currentProcess.getFile());

		File macroFile,rightFile;
		if (processStep <= currentProcess.getSteps().getStepList().size()) {
			rightFile = FileUtils
					.toExistingAbsoluteFile(currentProcess.getSteps().getStep(processStep - 1).getTargetFile(), currentProcess.getFile());
			macroFile = FileUtils
					.toExistingAbsoluteFile(currentProcess.getSteps().getStep(processStep - 1).getMacroFile(), currentProcess.getFile());

		//	loadFiles(leftFile, rightFile, macroFile, false);
		} else { // new step
			macroFile = FileUtils
					.toExistingAbsoluteFile(currentProcessTemplate.getStepTemplate(processStep - 1).getMacroFile(), PROCESS_TEMPLATES_FILE);

			// ask for new right file
			StepTemplate stepTemplate = currentProcessTemplate.getStepTemplate(processStep-1);
			String targetLanguageName = stepTemplate.getTargetLanguage();

			if (targetLanguageName.equalsIgnoreCase("HTML")) {
				MainController.deltaSideController[RIGHT].setTypeHTML();
				rightFile = MainController.deltaSideController[RIGHT].aOpen.doOpenFile(null, false, true, false, true,
																					   "Open '" + targetLanguageName + "' File (target) ..."); // no cancel button
			} else {
				XmlLanguage targetLanguage = getXmlLanguageByName(targetLanguageName);
				if (targetLanguage == null) {
					JOptionPane.showMessageDialog(mainFrame, "XML-Language for first step not defined: '" + targetLanguageName + "'!");
					return;
				}
				doNotCheckProcessWhenFileChanged = true;
				MainController.deltaSideController[RIGHT].setCurrentXMLLanguage(targetLanguage, false, false);
				doNotCheckProcessWhenFileChanged = false;

				JFileChooser fileChooser = new DeltaFileChooser(
						"Choose existing or enter new '" + targetLanguageName + "' File (target of "+ processStep+". step) ... ", false);

				fileChooser.setAcceptAllFileFilterUsed(true);
				fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
				fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
				fileChooser.setFileFilter(targetLanguage.getFileFilter());

				if (fileChooser.showSaveDialog(mainFrame) == JFileChooser.CANCEL_OPTION)
					return; // user cancelled
				rightFile = fileChooser.getSelectedFile();

				at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step newStep =
						at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step.newInstance();
				newStep.setName(stepTemplate.getName());
				newStep.setTargetFile(FileUtils.toRelativeFile(rightFile, currentProcess.getFile()));
				if (macroFile != null)
					newStep.setMacroFile(FileUtils.toRelativeFile(macroFile, currentProcess.getFile()));
				currentProcess.getSteps().addStep(newStep);

				// create new XML file, if file does not exist (if file exists, loadFiles() below will load it)
				if (!rightFile.exists() || !targetLanguage.getFileFilter().accept(rightFile)) {
					deltaSideController[RIGHT].aXMLNewEmptyFile.createNewEmptyXMLFile(rightFile);
				//	rightFile = null; // hack so that loadFiles() below don't load it
				}
			}

			// re-build process-steps menu
			buildProcessStepMenu(processStep, currentProcessView, false);
		}

		String processName = currentProcess.getName();
		mainFrame.projectNameLabel.setText(processName + " (" + processStep + ")");
		String toolTipText = "<html>Active Process: <b>" + processName + "</b><br>Active step: #" +
							 processStep + " - " + currentProcess.getSteps().getStep(processStep - 1).getName();
		String description = currentProcess.getDescription();
		if (description != null)
			toolTipText = toolTipText + "<br>&nbsp;&nbsp;" + description;
		mainFrame.projectNameLabel.setToolTipText(toolTipText);

		loadFiles(leftFile, rightFile, macroFile, false);

		if (currentProcessStep != processStep) {
			currentProcessStep = processStep;
			currentProcess.getSteps().setActiveStep(String.valueOf(currentProcessStep));
			try {
				currentProcess.marshal(currentProcess.getFile());
			} catch (IOException exc) {
				System.err.println("Could not write current process step to process file '" + currentProcess.getFile() + "'!");
				exc.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame, "Could not write current process step to process file '" +
														 currentProcess.getFile() + "':\n  " + exc.getMessage(), "I/O error",
																												 JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public static void setDELTAProcessView(View view) {
		String processName = currentProcess.getName();
		mainFrame.projectNameLabel.setText(processName + " (view)");
		String toolTipText = "<html>Active Process: <b>" + processName + "</b><br>Active view: " + view.getName();
		String description = currentProcess.getDescription();
		if (description != null)
			toolTipText = toolTipText + "<br>&nbsp;&nbsp;" + description;
		mainFrame.projectNameLabel.setToolTipText(toolTipText);

		currentProcessView = view;
		int maxStepIndex = currentProcess.getSteps().getStepList().size();
		int leftSideStepIndex = convertStringToInt(view.getLeftFile().getStepIndex(), 0, 0, maxStepIndex);
		int rightSideStepIndex = convertStringToInt(view.getRightFile().getStepIndex(), 0, 0, maxStepIndex);
		String leftFileName, rightFileName;
		if (leftSideStepIndex == 0) leftFileName = currentProcess.getSourceFile();
		else leftFileName = currentProcess.getSteps().getStep(leftSideStepIndex-1).getTargetFile();
		if (rightSideStepIndex == 0) rightFileName = currentProcess.getSourceFile();
		else rightFileName = currentProcess.getSteps().getStep(rightSideStepIndex-1).getTargetFile();
		File leftFile = FileUtils.toExistingAbsoluteFile(leftFileName, currentProcess.getFile());
		File rightFile = FileUtils.toExistingAbsoluteFile(rightFileName, currentProcess.getFile());
		// don't load macro file
		loadFiles(leftFile, rightFile, null, false);
	}

	public static void setDELTAProject(Project newProject, boolean loadProjectFiles, boolean onStartUp) {
		Log.trace("set new Project: " + newProject);
		mainFrame.resetLayout();

		if (currentProject != null) {
			addRecentFile(recentProjectFiles, currentProject.getProjectFile());
			buildRecentProjectFilesMenu();
			if (addRecentFile(recentDirs, currentProject.getProjectFile().getParentFile()))
				buildRecentDirsMenus();
		}
		if (currentProcess != null) {
			addRecentFile(recentProjectFiles, currentProcess.getFile());
			buildRecentProjectFilesMenu();
			if (addRecentFile(recentDirs, currentProcess.getFile().getParentFile()))
				buildRecentDirsMenus();
			mainFrame.jMenuProjectSteps.removeAll();
			mainFrame.jMenuProjectSteps.setEnabled(false);
		}

		if (newProject != null) {
            if (newProject.getMacrosFile() == null)
                mainFrame.hideMacrosPart();
			boolean replaceChangedFiles = (!MiscUtils.equals(newProject.getLeftSideFile(), deltaSides[LEFT].deltaView.getFile()) && deltaSides[LEFT].deltaView.contentHasChanged())
					|| (!MiscUtils.equals(newProject.getRightSideFile(), deltaSides[RIGHT].deltaView.getFile()) && deltaSides[RIGHT].deltaView.contentHasChanged())
					|| !MiscUtils.equals(newProject.getMacrosFile(), mainFrame.macrosView.getFile());
			currentProject = newProject;
			if (!loadProjectFiles || (!replaceChangedFiles || saveChangedFiles(true))) {
				aProjectShowPropertiesDialog.setEnabled(true);
				aProjectClose.setEnabled(true);
				if (loadProjectFiles)
					loadFiles(newProject.getLeftSideFile(), newProject.getRightSideFile(), newProject.getMacrosFile(), onStartUp);
				String projectName = newProject.getName();
				mainFrame.projectNameLabel.setText(projectName);
				mainFrame.projectNameLabel.setIcon(MainImages.PROJECT_ICON);
				String toolTipText = "<html>Active Project: <b>" + projectName + "</b>";
				String description = newProject.getDescription();
				if (description != null)
					toolTipText = toolTipText + "<br>&nbsp;&nbsp;" + description;
				mainFrame.projectNameLabel.setToolTipText(toolTipText);
			}
		} else if ((currentProject != null) || (currentProcess != null))
			if (saveChangedFiles(true)) {
				aProjectShowPropertiesDialog.setEnabled(false);
				aProjectClose.setEnabled(false);
				mainFrame.projectNameLabel.setText("-");
				mainFrame.projectNameLabel.setIcon(null);
				mainFrame.projectNameLabel.setToolTipText("No active project/process.");
			//	if (currentProject.getMacrosFileName() != null)
					aMacrosClose.doCloseFile(false);
			//	boolean shouldCloseLeftFile = currentProject.getLeftSideFileName() != null;
			//	boolean shouldCloseRightFile = currentProject.getRightSideFileName() != null;
				currentProject = null;
			//	if (shouldCloseLeftFile)
					deltaSideController[LEFT].aClose.doCloseFile(false, true);
			//	if (shouldCloseRightFile)
					deltaSideController[RIGHT].aClose.doCloseFile(false, true);

			}
		currentProcess = null;
	}

	public static boolean saveChangedFiles(boolean allowCancel) {
		if (deltaSides[GUIConstants.LEFT].deltaView.contentHasChanged() ||
			deltaSides[GUIConstants.RIGHT].deltaView.contentHasChanged()) {
			switch (preferences.getInt(Preferences.KEY_AUTO_SAVE)) {
				case AUTO_SAVE_ASK:
					String msg = "The content of ";
					if (deltaSides[GUIConstants.LEFT].deltaView.contentHasChanged())
						msg = msg + "the left file\n\"" + deltaSides[GUIConstants.LEFT].deltaView.getFilePath() + "\"\n";
					if (deltaSides[GUIConstants.RIGHT].deltaView.contentHasChanged()) {
						if (deltaSides[GUIConstants.LEFT].deltaView.contentHasChanged())
							msg = msg + " and ";
						msg = msg + "the right file\n\"" + deltaSides[GUIConstants.RIGHT].deltaView.getFilePath() + "\"\n";
					}
					msg += " has been changed.";
					switch (JOptionPane.showConfirmDialog(mainFrame, msg + "\n\nDo you want to save the changes ?",
														  "Save changed files?", (allowCancel ? JOptionPane.YES_NO_CANCEL_OPTION : JOptionPane.YES_NO_OPTION), JOptionPane.WARNING_MESSAGE)) {
						case JOptionPane.YES_OPTION:
						// mainFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
							if (deltaSides[GUIConstants.LEFT].deltaView.contentHasChanged())
								deltaSideController[GUIConstants.LEFT].aSave.doSaveFileInSameThread();
							if (deltaSides[GUIConstants.RIGHT].deltaView.contentHasChanged())
								deltaSideController[GUIConstants.RIGHT].aSave.doSaveFileInSameThread();
							return true;
						case JOptionPane.NO_OPTION:
							return true;
						case JOptionPane.CANCEL_OPTION:
							return false;
					}
					break;
				case AUTO_SAVE_YES:
				// mainFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
					if (deltaSides[GUIConstants.LEFT].deltaView.contentHasChanged())
						deltaSideController[GUIConstants.LEFT].aSave.doSaveFileInSameThread();
					if (deltaSides[GUIConstants.RIGHT].deltaView.contentHasChanged())
						deltaSideController[GUIConstants.RIGHT].aSave.doSaveFileInSameThread();
					return true;
			}
		}
		return true;
	}

	private static ProcessTemplates loadProcessTemplates() {
		at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process.setPrototype(
				new at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process() {
					{
						setDocType("process", "-//Asgaard//DTD DELT/A Process//EN",
								   "http://www.ifs.tuwien.ac.at/~votruba/DELTA/dtd/transformationProcessProject.dtd");
					//	setDocType("process", "-//Asgaard//DTD DELT/A Process//EN", "http://www.asgaard.tuwien.ac.at/~peter/DELTA/dtd/transformationProcessProject.dtd");
					}
				});
		at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process.setErrorHandler(DefaultSAXErrorHandler.SINGLETON);
		at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process.setEntityResolver(new EntityResolver() {
			public InputSource resolveEntity(String publicId, String systemId) {
				return new InputSource(new BufferedReader(new InputStreamReader(
						at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process.class.getResourceAsStream("transformationProcessProject.dtd"))));
			}
		});

		ProcessTemplates.setPrototype(new ProcessTemplates() {
			{
				setDocType("process-templates", "-//Asgaard//DTD DELT/A Process-Templates//EN", "http://www.ifs.tuwien.ac.at/~votruba/DELTA/dtd/transformationProcessTemplates.dtd");
//				setDocType("process-templates", "-//Asgaard//DTD DELT/A Process-Templates//EN", "http://www.asgaard.tuwien.ac.at/~peter/DELTA/dtd/transformationProcessTemplates.dtd");
			}
		});

		ProcessTemplates.setErrorHandler(DefaultSAXErrorHandler.SINGLETON);
		ProcessTemplates.setEntityResolver(new EntityResolver() {
			public InputSource resolveEntity(String publicId, String systemId) {
				return new InputSource(new BufferedReader(new InputStreamReader(
						ProcessTemplates.class.getResourceAsStream("transformationProcessTemplates.dtd"))));
			}
		});
		try {
			ProcessTemplates processTemplates = ProcessTemplates.unmarshal(PROCESS_TEMPLATES_FILE, true);
			List<ProcessTemplate> processTemplateList = processTemplates.getProcessTemplateList();
			/*Collections.sort(processTemplateList, new Comparator<ProcessTemplate>() {
				public int compare(ProcessTemplate template1, ProcessTemplate template2) {
					return template1.getName().compareToIgnoreCase(template2.getName());
				}
			});*/
			return processTemplates;
		} catch (IOException exc) {
			System.err.println("Couldn't parse \"" + PROCESS_TEMPLATES_XML_FILE + "\":\n  " + exc.getMessage());
		}

		return ProcessTemplates.newInstance();
	}

	// need to be public because it is used by Macro Editor
	public static List<XmlLanguage> loadXMLLanguages() {
//		Log.trace("MainController#loadXMLLanguages: begin");
		Set<XmlLanguage> result = new TreeSet<XmlLanguage>(XML_LANGUAGE_COMPARATOR);
		if (!XML_LANGUAGES_BASE_DIR.exists()) {
			if (XML_LANGUAGES_BASE_DIR.mkdir())
				Log.trace(XML_LANGUAGES_BASE_DIR.getAbsolutePath() + " was created ...");
			else
				throw new RuntimeException("Could not create XML languages directory: " + XML_LANGUAGES_BASE_DIR);
		} else if (XML_LANGUAGES_BASE_DIR.isFile() || XML_LANGUAGES_BASE_DIR.isHidden())
			throw new RuntimeException('\"' + XML_LANGUAGES_BASE_DIR.toString() + "\" exists, but is not a directory!");
		File dirs[] = XML_LANGUAGES_BASE_DIR.listFiles(new FileFilter() {
			public boolean accept(File file) {
				return file.isDirectory();
			}
		});
		XmlLanguage.setEntityResolver(new EntityResolver() {
			public InputSource resolveEntity(String publicId, String systemId) {
				return new InputSource(new BufferedReader(new InputStreamReader(
						XmlLanguage.class.getResourceAsStream("xmlLanguage.dtd"))));
			}
		});
		XmlLanguage.setPrototype(new XmlLanguage() {
			{
				setDocType("xmlLanguage", "-//Asgaard//DTD DELT/A XML-Languages//EN", "http://www.asgaard.tuwien.ac.at/~peter/DELTA/dtd/xmlLanguage3.dtd");
			}

			public String toString() {
				return getName();
			}
		});
		// set SAX error-handler
		XmlLanguage.setErrorHandler(DefaultSAXErrorHandler.SINGLETON);
//		Log.trace("... looping through all dirs");
		for (int i = 0; i < dirs.length; i++) {
			File files[] = dirs[i].listFiles(new FileFilter() {
				public boolean accept(File file) {
					return file.getPath().endsWith("lang");
				}
			});
//			Log.trace("... looping through all lang files in "+dirs[i]);
			for (int j = 0; j < files.length; j++) {
				try {
//					Log.trace("... unmarshalling "+ files[j]);
					XmlLanguage xmlLanguage = XmlLanguage.unmarshal(files[j], true);
//					Log.trace("... adding "+ files[j] + " to list");
					URL url = FileUtils.toURL(xmlLanguage.getIconFileName(), files[j]);
					if (url != null) {
						Icon icon = new ImageIcon(url);
						if (icon.getIconWidth() < 10) icon = XMLLanguageDialog.DEFAULT_LANGUAGE_ICON;
						xmlLanguage.setIcon(icon);
					} else
						xmlLanguage.setIcon(XMLLanguageDialog.DEFAULT_LANGUAGE_ICON);
					if (xmlLanguage.getExt().charAt(0) != '.') xmlLanguage.setExt("." + xmlLanguage.getExt());
					xmlLanguage.setFileFilter(new DocTypeFileFilter(xmlLanguage));
					DeltaFileChooser.FILE_VIEW.getExtensionToLanguagesMap().add(xmlLanguage.getExt(), xmlLanguage);
					if (result.add(xmlLanguage))
						System.out.println("Found definition for XML-Language \""+xmlLanguage.getName()+"\" ...");
					else
						System.out.println("Found double definition for XML-Language \""+xmlLanguage.getName()+"\" ...\n  take last one: " +
										   files[j].getAbsolutePath());
				} catch (Exception exc) {
					System.err.println("Couldn't parse \"" + files[j].getAbsolutePath() + "\":\n  " + exc.getMessage());
				//	exc.printStackTrace();
				}
			}
		}
//		Log.trace("MainController#loadXMLLanguages: end");
		return new ArrayList<XmlLanguage>(result);
	}
	public static XmlLanguage getXmlLanguageByName(String name) {
		for (XmlLanguage xmlLanguage : xmlLanguages)
			if (xmlLanguage.getName().equalsIgnoreCase(name))
				return xmlLanguage;
		return null;
	}

	private static int activeSideIndex = EMPTY;
	private static boolean refreshedLastTime;
	public static boolean setActiveSideIndex(int sideIndex) {
		return setActiveSideIndex(sideIndex, true);
	}
	public static boolean setActiveSideIndex(int sideIndex, boolean resetAll) {
		if (sideIndex == EMPTY) return false;

		if (!refreshedLastTime || (activeSideIndex != sideIndex)) {
			if ((sideIndex != LEFT) && (sideIndex != RIGHT)) {		// should never happen
				System.err.println("Bad side index: "+ sideIndex);
				return false;
			}
			refreshedLastTime = resetAll;
			deltaSideController[1 - sideIndex].setInactive(); // do not use activeSideIndex here, because it is 3 (=EMPTY) at start up
			if (deltaSideController[sideIndex].type != EMPTY) {
				activeSideIndex = sideIndex;
				DeltaSideController activeSideController = deltaSideController[activeSideIndex];
				DeltaSideController inactiveSideController = deltaSideController[1 - activeSideIndex];
				activeSideController.setActive();

				if ((activeSideController.currentXMLLanguage != null) && (activeSideController.currentXMLLanguage.getExt().equals(".macros")))
					aMacrosOpenXMLInMacrosView.setEnabled(mainFrame.macrosView.isLoaded());
				else aMacrosOpenXMLInMacrosView.setEnabled(false);

				if (resetAll) {
					mainFrame.macrosView.setDTDHelper((activeSideController.type == XML) ? activeSideController.dtdHelper : inactiveSideController.dtdHelper); // macrosView.setDTDHelper() resets all macros listitems
					resetMacroInsertStates();
					/*boolean linksSelectedOnBothSides = (activeSideController.thisSide.deltaView.getCurrentDELTALinkID(false) >= LinkIDConstants.FIRST_ID)
							&& (inactiveSideController.thisSide.deltaView.getCurrentDELTALinkID(false) >= LinkIDConstants.FIRST_ID);
					aMacroInsert.setEnabled(activeSideController.isTypeXML()
									&& activeSideController.dtdHelper.isMacroInsertable(mainFrame.macrosView.getMacroXMLElement()) && (!mainFrame.macrosView.isMacroLink() || (!linksSelectedOnBothSides && inactiveSideController.thisSide.deltaView.acceptsNewDELTALink(false)))
							|| (activeSideController.isTypeHTML()
									&& inactiveSideController.dtdHelper.isMacroInsertable(mainFrame.macrosView.getMacroXMLElement()) && activeSideController.thisSide.deltaView.acceptsNewDELTALink(false) && !linksSelectedOnBothSides)); // mainFrame.macrosView.isMacroLink() && ))*/

					XMLTreeNode selectedTreeNode;
					if (activeSideController.isTypeXML() &&
							((selectedTreeNode = activeSideController.thisSide.xmlView.getSelectedTreeNode()) != null)) {
						Node xmlNode = selectedTreeNode.getXMLNode();
						aEditCopy.setEnabled((xmlNode.getNodeType() != Node.DOCUMENT_NODE)
								   && (xmlNode.getNodeType() != Node.DOCUMENT_TYPE_NODE));
						aEditCut.setEnabled(activeSideController.dtdHelper.isNodeRemoveable());
					} else {
						aEditCopy.setEnabled(false);
						aEditCut.setEnabled(false);
					}

					boolean dtd = activeSideController.dtdHelper.isLoaded();
					if (dtd) {
						aEditShowInsertableElementsDialog.setEnabled(true);
						if (insertableElementsDialog != null)
							if (insertableElementsDialog.showAgain())
								insertableElementsDialog.refreshLists(activeSideController.dtdHelper);
						aEditShowAllElementsDialog.setEnabled(true);
						if (allElementsDialog != null)
							if (allElementsDialog.showAgain())
								allElementsDialog.refreshList(activeSideController.dtdHelper);
						aMacroSearchInsertDown.setEnabled(mainFrame.macrosView.isMacroSelected());
						aMacroSearchInsertUp.setEnabled(mainFrame.macrosView.isMacroSelected());
						resetPasteActions();
					} else {
						aEditShowInsertableElementsDialog.setEnabled(false);
						if (insertableElementsDialog != null)
							insertableElementsDialog.hideTemporary();
						aEditShowAllElementsDialog.setEnabled(false);
						if (allElementsDialog != null)
							allElementsDialog.hideTemporary();
						aMacroSearchInsertDown.setEnabled(false);
						aMacroSearchInsertUp.setEnabled(false);
						aEditPasteInto.setEnabled(false);
						aEditPasteBefore.setEnabled(false);
						aEditPasteAfter.setEnabled(false);
						aEditPasteReplace.setEnabled(false);
					}
				}
			} else {
				activeSideIndex = EMPTY;
				aEditFindNext.setEnabled(false);
                aEditFindPrevious.setEnabled(false);
				mainFrame.macrosView.setDTDHelper(null);
				aMacroInsert.setEnabled(false);
				aMacroSearchInsertDown.setEnabled(false);
				aMacroSearchInsertUp.setEnabled(false);
				aEditShowInsertableElementsDialog.setEnabled(false);
				aEditShowAllElementsDialog.setEnabled(false);
				aEditPasteInto.setEnabled(false);
				aEditPasteBefore.setEnabled(false);
				aEditPasteAfter.setEnabled(false);
				aEditPasteReplace.setEnabled(false);
			}

			if (mainFrame.macrosView.isLoaded()) {
				mainFrame.macroInfoTree.repaint();
				mainFrame.jLabelMacroPreview.setIcon(mainFrame.macrosView.getCurrentItemIcon());
			}

			return true;
		}
		return false;
	}

	public static int getActiveSideIndex() {
		return activeSideIndex;
	}

	public static MainFrame.DeltaSide getActiveSide() {
		if (activeSideIndex == EMPTY)
			return deltaSides[LEFT]; // todo
		return deltaSides[activeSideIndex];
	}

	public static MainFrame.DeltaSide getInactiveSide() {
		if (activeSideIndex == EMPTY)
			return deltaSides[LEFT]; // todo
		return deltaSides[1 - activeSideIndex];
	}

	public static DeltaSideController getActiveSideController() {
		if (activeSideIndex == EMPTY)
			return deltaSideController[LEFT]; // todo
		return deltaSideController[activeSideIndex];
	}

	public static DeltaSideController getInactiveSideController() {
		if (activeSideIndex == EMPTY)
			return deltaSideController[LEFT]; // todo
		return deltaSideController[1 - activeSideIndex];
	}

	public static void resetPasteActions() {
		DeltaSideController thisSideController = getActiveSideController();
		XMLTreeNode selectedTreeNode = thisSideController.thisSide.xmlView.getSelectedTreeNode();
		if ((xmlClipBoard != null) && (selectedTreeNode != null)) {
			aEditPasteInto.setEnabled(((xmlClipBoard.clipBoardMode == XMLClipBoard.ClipBoardMode.COPY)
													  || !selectedTreeNode.isNodeAncestor(xmlClipBoard.treeNode))
													 && thisSideController.dtdHelper.isNodeInsertableInto(xmlClipBoard.xmlNode));
			aEditPasteBefore.setEnabled(thisSideController.dtdHelper.isNodeInsertableBefore(xmlClipBoard.xmlNode));
			aEditPasteAfter.setEnabled(thisSideController.dtdHelper.isNodeInsertableAfter(xmlClipBoard.xmlNode));
			aEditPasteReplace.setEnabled(thisSideController.dtdHelper.canBeReplacedByNode(xmlClipBoard.xmlNode));
		} else {
			aEditPasteInto.setEnabled(false);
			aEditPasteBefore.setEnabled(false);
			aEditPasteAfter.setEnabled(false);
			aEditPasteReplace.setEnabled(false);
		}
	}

	private static void installListeners() {
		mainFrame.addWindowListener(createMainWindowListener());
		mainFrame.macrosView.addActionListener(aMacroInsert);
		mainFrame.macrosView.addChangeListener(new MacrosChangeListener());
		deltaSideController[LEFT].installListeners();
		deltaSideController[RIGHT].installListeners();

		mainFrame.macroInsertModeComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				if (mainFrame.macrosView.isLoaded()) {
					mainFrame.macrosView.resetMacroListItems();
					mainFrame.macroInfoTree.repaint();
					mainFrame.jLabelMacroPreview.setIcon(mainFrame.macrosView.getCurrentItemIcon());
					aMacroInsert.setEnabled((!mainFrame.macrosView.isMacroLink()
											 || getInactiveSide().deltaView.acceptsNewDELTALink(true, false)) // selection of getInactiveSide() should either be a link or a link should be insertable there (for a XML-File with a *_DELTA.dtd this is always true)
											&& mainFrame.macrosView.isMacroInsertable());
				}
			}
		});
		mainFrame.jButtonMacrosReopen.addMouseListener(
				new ButtonPopupMenuHandler(mainFrame.jButtonMacrosReopen, mainFrame.jPopupMenuMacrosReopen));
		mainFrame.projectNameLabel.addMouseListener(new PopupMenuHandler(mainFrame.jMenuProjectSteps.getPopupMenu()));
		mainFrame.projectNameLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				if (evt.getClickCount() == 2)
					aProjectShowPropertiesDialog.actionPerformed(null);
			}
		});
		mainFrame.clipBoardLabel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				if ((xmlClipBoard != null) && (evt.getClickCount() == 2))
					deltaSides[xmlClipBoard.origin].xmlView.centerAndSelectTreeNode(xmlClipBoard.treeNode);
			}
		});
		recentMacrosOpenActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				aMacrosOpen.doOpenFile(new File(evt.getActionCommand()));
			}
		};
		recentProjectOpenActionListener = new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				aProjectOpen.doOpenProjectFile(new File(evt.getActionCommand()));
			}
		};

		if (MainController.ADVANCED_MODE)
		mainFrame.validModeCheckBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent evt) {
                boolean validModeOn = mainFrame.validModeCheckBox.isSelected();
                mainFrame.validModeCheckBox.setIcon(validModeOn ? MainImages.VALID_MODE_ON_ICON : MainImages.VALID_MODE_OFF_ICON);
                mainFrame.validModeCheckBox.setToolTipText("Valid Mode: " + (validModeOn ? "\"On\"" : "\"Off\""));
                int new_state = validModeOn ? DTDHelper.STRICT_CHECK : DTDHelper.NO_CHECK;
                preferences.putInt(Preferences.KEY_XML_DTD_STATE, new_state);
                deltaSideController[LEFT].dtdHelper.setState(new_state);
                deltaSideController[RIGHT].dtdHelper.setState(new_state);
				int activeSideIndex = getActiveSideIndex();
				deltaSides[LEFT].xmlView.reselectCurrentTreeNode();
                deltaSides[RIGHT].xmlView.reselectCurrentTreeNode();
				if (validModeOn) {
					List invalidNodes = deltaSideController[LEFT].dtdHelper.getAllInvalidNodes(deltaSides[LEFT].xmlView.getDocumentNode()); //thisSide.xmlView.getDocNode());
					if (!invalidNodes.isEmpty()) {
						deltaSides[LEFT].xmlView.highlightTreeNodes(deltaSides[LEFT].xmlView.convertXMLNodeListToTreeNodeList(invalidNodes),
															preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE));
						JOptionPane.showMessageDialog(mainFrame, "Document is NOT valid according to its DTD!\n\nInvalid Nodes - either due to missing or bad attributes\nor due to content model violation - are highlighted.",
													  "Validation", JOptionPane.ERROR_MESSAGE);
					}
					invalidNodes = deltaSideController[RIGHT].dtdHelper.getAllInvalidNodes(deltaSides[RIGHT].xmlView.getDocumentNode()); //thisSide.xmlView.getDocNode());
					if (!invalidNodes.isEmpty()) {
						deltaSides[RIGHT].xmlView.highlightTreeNodes(deltaSides[RIGHT].xmlView.convertXMLNodeListToTreeNodeList(invalidNodes),
																  preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE));
						JOptionPane.showMessageDialog(mainFrame, "Document is NOT valid according to its DTD!\n\nInvalid Nodes - either due to missing or bad attributes\nor due to content model violation - are highlighted.",
													  "Validation", JOptionPane.ERROR_MESSAGE);
					}
				}
				setActiveSideIndex(activeSideIndex);
            }
        });
	}

	public static void savePreferences() {
		File file = deltaSides[LEFT].deltaView.getFile();
		if (file != null) {
			stateProperties.putFile(Preferences.KEY_LAST_LEFT_FILE, file, true);
			addRecentFile(recentXHTMLFiles, file);
			addRecentFile(recentDirs, file.getParentFile());
		} else
			stateProperties.remove(Preferences.KEY_LAST_LEFT_FILE);
		file = deltaSides[RIGHT].deltaView.getFile();
		if (file != null) {
			stateProperties.putFile(Preferences.KEY_LAST_RIGHT_FILE, file, true);
			addRecentFile(recentXHTMLFiles, file);
			addRecentFile(recentDirs, file.getParentFile());
		} else
			stateProperties.remove(Preferences.KEY_LAST_RIGHT_FILE);
		file = mainFrame.macrosView.getFile();
		if (file != null) {
			stateProperties.putFile(Preferences.KEY_LAST_MACROS_FILE, file, true);
			addRecentFile(recentMacrosFiles, file);
			addRecentFile(recentDirs, file.getParentFile());
		} else
			stateProperties.remove(Preferences.KEY_LAST_MACROS_FILE);
		if ((currentProject != null) || (currentProcess != null)) {
			if (currentProject != null) file = currentProject.getProjectFile();
			else file = currentProcess.getFile();
			stateProperties.putFile(Preferences.KEY_LAST_PROJECT_FILE, file, true);
			addRecentFile(recentProjectFiles, file);
			addRecentFile(recentDirs, file.getParentFile());
		} else
			stateProperties.remove(Preferences.KEY_LAST_PROJECT_FILE);

		stateProperties.putFileList(Preferences.KEY_RECENT_XHTML_FILES, recentXHTMLFiles, true);
		stateProperties.putFileList(Preferences.KEY_RECENT_MACROS_FILES, recentMacrosFiles, true);
		stateProperties.putFileList(Preferences.KEY_RECENT_PROJECT_FILES, recentProjectFiles, true);
		stateProperties.putFileList(Preferences.KEY_RECENT_DIRS, recentDirs, true);

		stateProperties.putInt(Preferences.KEY_LAST_ACTIVE_SIDE, activeSideIndex);
		deltaSideController[LEFT].storeProperties();
		deltaSideController[RIGHT].storeProperties();
		mainFrame.macrosView.storeProperties();
		if (preferences.getBoolean(Preferences.KEY_LAYOUT_RESTORE))
			mainFrame.storeLayout(stateProperties);
		try {
			preferences.save();
			stateProperties.save();
		} catch (Throwable t) {
			System.err.println("Couldn't save Prefernces!");
			t.printStackTrace(System.err);
		}
	}

	public static void enableMacrosActions() {
		aMacrosOpen.setEnabled(true);
		aMacrosReopen.setEnabled(true);
		boolean macrosFileLoaded = mainFrame.macrosView.getFile() != null;
		aMacrosReload.setEnabled(macrosFileLoaded);

		aMacrosOpenAsXML.setEnabled(macrosFileLoaded);
		XmlLanguage currentXMLLanguage = getActiveSideController().currentXMLLanguage;
		if ((currentXMLLanguage != null) && (currentXMLLanguage.getExt().equals(".macros")))
			aMacrosOpenXMLInMacrosView.setEnabled(macrosFileLoaded);

		aMacrosClose.setEnabled(macrosFileLoaded);
		aMacrosShowEditor.setEnabled(macrosFileLoaded);
		if (getInactiveSideController().isTypeXML())
			aMacroInsert.setEnabled((!mainFrame.macrosView.isMacroLink()
									 || getInactiveSide().deltaView.acceptsNewDELTALink(true, false)) // selection of getInactiveSide() should either be a link or a link should be insertable there (for a XML-File with a *_DELTA.dtd this is always true)
									&& mainFrame.macrosView.isMacroInsertable());
		else
			aMacroInsert.setEnabled(false);
		aMacrosGoBack.setEnabled(mainFrame.macrosView.canGoBack());
		aMacrosGoForward.setEnabled(mainFrame.macrosView.canGoForward());
		aMacrosSwitchToSearchView.setEnabled(macrosFileLoaded);
		aMacrosSwitchToInsertableView.setEnabled(macrosFileLoaded);
		aMacroSwitchToTreeView.setEnabled(macrosFileLoaded);
		aMacroSwitchToSourceView.setEnabled(macrosFileLoaded);
		aMacroSwitchToTextView.setEnabled(macrosFileLoaded);

		aMacroSearchInsertDown.setEnabled(getActiveSideController().isTypeXML() && mainFrame.macrosView.isMacroSelected());
		aMacroSearchInsertUp.setEnabled(getActiveSideController().isTypeXML() && mainFrame.macrosView.isMacroSelected());

		aFileCloseAll.setEnabled(
				deltaSides[LEFT].deltaView.isLoaded() || deltaSides[RIGHT].deltaView.isLoaded() || mainFrame.macrosView.isLoaded());

		mainFrame.jMenuProjectProcessTemplates.setEnabled(!processTemplates.getProcessTemplateList().isEmpty());
		mainFrame.jMenuProjectSteps.setEnabled(currentProcess != null);
	}

	public static void disableMacrosActions() {
		aMacrosOpen.setEnabled(false);
		aMacrosReopen.setEnabled(false);
		aMacrosReload.setEnabled(false);
		aMacrosOpenAsXML.setEnabled(false);
		aMacrosOpenXMLInMacrosView.setEnabled(false);
		aMacrosClose.setEnabled(false);
		aMacroInsert.setEnabled(false);
		aMacrosGoBack.setEnabled(false);
		aMacrosGoForward.setEnabled(false);
		aMacrosShowEditor.setEnabled(false);
		aMacrosSwitchToSearchView.setEnabled(false);
		aMacrosSwitchToInsertableView.setEnabled(false);
		aMacroSwitchToTreeView.setEnabled(false);
		aMacroSwitchToSourceView.setEnabled(false);
		aMacroSwitchToTextView.setEnabled(false);
		aMacroSearchInsertDown.setEnabled(false);
		aMacroSearchInsertUp.setEnabled(false);
		aFileCloseAll.setEnabled(false);

		mainFrame.jMenuProjectProcessTemplates.setEnabled(false);
		mainFrame.jMenuProjectSteps.setEnabled(false);
	}

	// should be called on New, Open, SaveAs and Close of XHTML and Macros files
	public static void fileChanged(int viewIndex) {
		aFileSwapSides.setEnabled(deltaSides[LEFT].deltaView.isLoaded() && deltaSides[RIGHT].deltaView.isLoaded());
		if (currentProject != null) {
			File oldFile, newFile;
			if (viewIndex == MACROS) {
				oldFile = currentProject.getMacrosFile();
				newFile = mainFrame.macrosView.getFile();
			} else {
				if (viewIndex == LEFT)
					oldFile = currentProject.getLeftSideFile();
				else
					oldFile = currentProject.getRightSideFile();
				newFile = deltaSides[viewIndex].deltaView.getFile();
			}
			if (!MiscUtils.equals(oldFile, newFile)) { // don't use != because in recentFileList are not the same File object as in Project
				Log.trace(SIDE_NAMES[viewIndex] + " file of project changed: " + oldFile + " -> " + newFile);
				int whatShouldIDoWithTheProject = preferences.getInt(Preferences.KEY_PROJECT_ON_CHANGE);
				if (whatShouldIDoWithTheProject == Preferences.PROJECT_CHANGE_ASK) {
					/*whatShouldIDoWithTheProject = JOptionPane.showOptionDialog(mainFrame,
											   "The name of the " + SIDE_NAMES[viewIndex]
											   + " file changed, compared to the active project.\n\nPlease choose one of the following actions:\n"
											   + "UNSET the project, CLOSE the entire project, ADJUST the project entry or SAVE the project under a new name\n",
											   "Project change", JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE, null,
											   new String[]{"Unset", "Close", "Adjust", "Save As..."}, "Unset");
					if (whatShouldIDoWithTheProject == JOptionPane.CLOSED_OPTION) // user cancelled input
						whatShouldIDoWithTheProject = Preferences.PROJECT_CHANGE_UNSET;*/
					int ask = JOptionPane.showConfirmDialog(mainFrame,
												  "The name of the " + SIDE_NAMES[viewIndex] +
												  " file changed, compared to the active project.\nAdjust and keep the project?",
												  "Project change", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
					whatShouldIDoWithTheProject = (ask == JOptionPane.YES_OPTION) ? Preferences.PROJECT_CHANGE_ADJUST :
												  Preferences.PROJECT_CHANGE_UNSET;
				}
				switch (whatShouldIDoWithTheProject) {
					case Preferences.PROJECT_CHANGE_CLOSE:
						Log.trace("project close ...");
						if (viewIndex != LEFT)
							deltaSideController[LEFT].aClose.doCloseFile(false, false);
						if (viewIndex != RIGHT)
							deltaSideController[RIGHT].aClose.doCloseFile(false, false);
						if (viewIndex != MACROS)
							aMacrosClose.doCloseFile(false);
					case Preferences.PROJECT_CHANGE_UNSET:
						aProjectShowPropertiesDialog.setEnabled(false);
						aProjectClose.setEnabled(false);
						mainFrame.projectNameLabel.setText("-");
						mainFrame.projectNameLabel.setIcon(null);
						mainFrame.projectNameLabel.setToolTipText("No active project/process.");

						addRecentFile(recentProjectFiles, currentProject.getProjectFile());
						buildRecentProjectFilesMenu();
						if (addRecentFile(recentDirs, currentProject.getProjectFile().getParentFile()))
							buildRecentDirsMenus();

						currentProject = null;
						break;
					case Preferences.PROJECT_CHANGE_ADJUST:
						Log.trace("project adjust ...");
						switch (viewIndex) {
							case LEFT:
								currentProject.setLeftSideFile(newFile);
								break;
							case RIGHT:
								currentProject.setRightSideFile(newFile);
								break;
							case MACROS:
								currentProject.setMacrosFile(newFile);
						}
						try {
							currentProject.save();
						} catch (IOException exc) {
							exc.printStackTrace();
							JOptionPane.showMessageDialog(mainFrame,
											"Couldn't save DELT/A-Project File!\n\n" + exc.getMessage(), "I/O Error",
											JOptionPane.ERROR_MESSAGE);
						}
						break;
					case Preferences.PROJECT_CHANGE_SAVEAS:
						Log.trace("project save as ...");
						Project newProject = new Project(currentProject);
						switch (viewIndex) {
							case LEFT:
								newProject.setLeftSideFile(newFile);
								break;
							case RIGHT:
								newProject.setRightSideFile(newFile);
								break;
							case MACROS:
								newProject.setMacrosFile(newFile);
						}
						if (ProjectPropertiesDialog.saveProjectAs(newProject, mainFrame))
							setDELTAProject(newProject, true, false);
						break;
				}
			}
		} else if ((currentProcess != null) && !doNotCheckProcessWhenFileChanged) {
			File oldFile, newFile;
			if (viewIndex == MACROS) {
				oldFile = FileUtils.toAbsoluteFile(currentProcess.getSteps().getStep(currentProcessStep-1).getMacroFile(), currentProcess.getFile());
				newFile = mainFrame.macrosView.getFile();
			} else {
				if (viewIndex == LEFT) {
					String fileName;
					if (currentProcessView == null)
						if (currentProcessStep == 1)
							fileName = currentProcess.getSourceFile();
						else
							fileName = currentProcess.getSteps().getStep(currentProcessStep - 2).getTargetFile();
					else {
						int _currentProcessStep = convertStringToInt(currentProcessView.getLeftFile().getStepIndex(), 0, 0,
																 currentProcess.getSteps().getStepList().size());
						if (_currentProcessStep == 0)
							fileName = currentProcess.getSourceFile();
						else
							fileName = currentProcess.getSteps().getStep(_currentProcessStep - 1).getTargetFile();
					}
					oldFile = FileUtils.toAbsoluteFile(fileName, currentProcess.getFile());
				} else {
					// RIGHT side
					String fileName;
					if (currentProcessView == null)
						fileName = currentProcess.getSteps().getStep(currentProcessStep-1).getTargetFile();
					else {
						int _currentProcessStep = convertStringToInt(currentProcessView.getRightFile().getStepIndex(), 0, 0,
																	 currentProcess.getSteps().getStepList().size());
						if (_currentProcessStep == 0)
							fileName = currentProcess.getSourceFile();
						else
							fileName = currentProcess.getSteps().getStep(_currentProcessStep - 1).getTargetFile();
					}
					oldFile = FileUtils.toAbsoluteFile(fileName, currentProcess.getFile());
				}
				newFile = deltaSides[viewIndex].deltaView.getFile();
			}
			if (!MiscUtils.equals(oldFile, newFile)) {
				int ask = JOptionPane.showConfirmDialog(mainFrame,
														"The name of the " + SIDE_NAMES[viewIndex] +
														" file changed, compared to the active process.\nAdjust and keep the project?",
														"Process change", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
				if (ask == JOptionPane.YES_OPTION) {
					String newFileRelativePath = FileUtils.toRelativeFile(newFile, currentProcess.getFile());
					Log.trace("adjust process with" + newFileRelativePath);
					switch (viewIndex) {
						case LEFT:
							if (currentProcessStep == 1)
								currentProcess.setSourceFile(newFileRelativePath);
							else
								currentProcess.getSteps().getStep(currentProcessStep - 2).setTargetFile(newFileRelativePath);
							break;
						case RIGHT:
							currentProcess.getSteps().getStep(currentProcessStep-1).setTargetFile(newFileRelativePath);
							break;
						case MACROS:
							currentProcess.getSteps().getStep(currentProcessStep-1).setMacroFile(newFileRelativePath);
					}
					try {
						currentProcess.marshal(currentProcess.getFile());
					} catch (Exception exc) {
						exc.printStackTrace();
						JOptionPane.showMessageDialog(mainFrame,
													  "Couldn't save DELT/A-Process File!\n\n" + exc.getMessage(), "I/O Error",
													  JOptionPane.ERROR_MESSAGE);
					}
				} else {
					mainFrame.jMenuProjectSteps.setEnabled(false);
					aProjectShowPropertiesDialog.setEnabled(false);
					aProjectClose.setEnabled(false);
					mainFrame.projectNameLabel.setText("-");
					mainFrame.projectNameLabel.setIcon(null);
					mainFrame.projectNameLabel.setToolTipText("No active project/process.");

					addRecentFile(recentProjectFiles, currentProcess.getFile());
					buildRecentProjectFilesMenu();
					if (addRecentFile(recentDirs, currentProcess.getFile().getParentFile()))
						buildRecentDirsMenus();

					currentProcess = null;
					currentProcessTemplate = null;
					currentProcessStep = -1;
					currentProcessView = null;
				}
			} else
				if (viewIndex != MACROS) {
					updateHighestLinkIDofProcess(viewIndex);
				}
		}

		if (viewIndex != MACROS) {
			boolean dtd = getActiveSideController().dtdHelper.isLoaded();
			if (dtd) {
				aEditShowInsertableElementsDialog.setEnabled(true);
				aEditShowAllElementsDialog.setEnabled(true);
				aMacroSearchInsertDown.setEnabled(mainFrame.macrosView.isMacroSelected());
				aMacroSearchInsertUp.setEnabled(mainFrame.macrosView.isMacroSelected());
			} else {
				aEditShowInsertableElementsDialog.setEnabled(false);
				if (insertableElementsDialog != null)
					insertableElementsDialog.hideTemporary();
				aEditShowAllElementsDialog.setEnabled(false);
				if (allElementsDialog != null)
					allElementsDialog.hideTemporary();
				aMacroSearchInsertDown.setEnabled(false);
				aMacroSearchInsertUp.setEnabled(false);
			}
		}
	}

	public static int getProcessMaxLinkID() {
		if (currentProcess != null)
			try {
				return Integer.parseInt(currentProcess.getHighestLinkID());
			} catch (NumberFormatException e) {
				return -1;
			}
		return -1;
	}

	public static void updateHighestLinkIDofProcess(int viewIndex) {
		// update highest link ID of process
		int highestLinkID = 1; // has to be greater 0 and <= count of steps
		try {
			highestLinkID = Integer.parseInt(currentProcess.getHighestLinkID());
			int currentMaxDELTALinkID = deltaSides[viewIndex].deltaView.getMaxDELTALinkID();
			if (currentMaxDELTALinkID > highestLinkID) {
				currentProcess.setHighestLinkID(String.valueOf(currentMaxDELTALinkID));
				try {
					currentProcess.marshal(currentProcess.getFile());
					System.out.println("new highst link-ID "+ currentMaxDELTALinkID+" written to process file.");
				} catch (IOException exc) {
					System.err.println(
							"Could not write current highest link-ID to process file '" + currentProcess.getFile() + "'!");
					exc.printStackTrace();
//					JOptionPane.showMessageDialog(mainFrame, "Could not write current highest link-ID to process file '" +
//															 currentProcess.getFile() + "':\n  " + exc.getMessage(),
//												  "I/O error",
//												  JOptionPane.ERROR_MESSAGE);
				}
			} else if (currentMaxDELTALinkID < highestLinkID)
				deltaSides[viewIndex].deltaView.setExternalMaxDELTALinkID(highestLinkID);
		} catch (NumberFormatException e) {
			System.err.println("highest link-ID of process not a valid number!");
		}
	}

	// Event listener methods
	private static WindowListener createMainWindowListener() {
		return new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				aFileExit.quit();
			}
		};
	}

	/** Adds the recent <i>file</i> to <i>recentFiles</i>
	 * and to ArrayList <i>recentFiles</i>
	 * @param file
	 * @param recentFiles
	 */
	public static boolean addRecentFile(final List<File> recentFiles, File file) {
		if (file == null) return false;
		file = FileUtils.getAbsoluteFile(file);

		final int offset = recentFiles.indexOf(file);
		// Check whether this file is already in the file list
		if (offset == -1) {
			recentFiles.add(0, file); // Not present - add it at the beginning
			if (recentFiles.size() > maxRecentFiles)
				recentFiles.remove(maxRecentFiles); // List was full - remove the last (oldest) item
			return true;
		}
		if (offset == 0)
			return false;

		// Already present: move this item to the front of the list
		recentFiles.remove(offset);
		recentFiles.add(0, file);
		return true;
	}

	private static void buildRecentProjectFilesMenu() {
		mainFrame.jMenuProjectReopen.removeAll();
		final int size = Math.min(recentProjectFiles.size(), maxRecentFiles);
		if (size == 0)
			mainFrame.jMenuProjectReopen.add(EMPTY_MENU_ACTION);
		else {
			for (int i = 1; i <= size; i++) {
				final File file = recentProjectFiles.get(i - 1);
				final String fileName = file.getName();
				final String filePath = file.getAbsolutePath();
				final Action action = new AbstractAction("" + i + ' ' + fileName) {
					{
						if (fileName.endsWith(".process")) putValue(SMALL_ICON, MainImages.PROCESS_ICON);
						else putValue(SMALL_ICON, MainImages.PROJECT_ICON);
						putValue(SHORT_DESCRIPTION, "Reopen project file \"" + fileName + '\"'); // when mouse over - tool-tip
						putValue(StatusBar.DESCRIPTION_HOVER,
								 "Open previously opened project file \"" + filePath + '\"'); // when mouse over - statusbar
					}
					public void actionPerformed(ActionEvent evt) {
						mainFrame.statusBar.unsetHoverMessageFrom(this);
						recentProjectOpenActionListener.actionPerformed(evt);
					}
				};
				action.putValue(Action.MNEMONIC_KEY, new Integer('0' + i));
				JMenuItem mi = mainFrame.jMenuProjectReopen.add(action);
				mi.setActionCommand(filePath);
			}
			mainFrame.statusBar.attachToMenuElement(mainFrame.jMenuProjectReopen);
		}
	}

	public static void buildRecentMacrosFilesMenus() {
		_buildRecentFilesMenus(mainFrame.jMenuMacrosReopen, mainFrame.jPopupMenuMacrosReopen, recentMacrosFiles, recentMacrosOpenActionListener);
	}

	private static final Action EMPTY_MENU_ACTION = new EmptyAction("empty", "empty list",
																	"empty list", false);

	static void _buildRecentFilesMenus(final JMenu fileListMenu, final JPopupMenu fileListPopupMenu,
									   final List recentFiles, final ActionListener menuActionListener) {
		fileListMenu.removeAll();
		fileListPopupMenu.removeAll();
		final int size = Math.min(recentFiles.size(), maxRecentFiles);
		if (size == 0) {
			fileListMenu.add(EMPTY_MENU_ACTION);
			fileListPopupMenu.add(EMPTY_MENU_ACTION);
		} else {
			for (int i = 1; i <= recentFiles.size(); i++) { // attention: do not use variable "size", because recentFiles.size() may change in another thread during this loop
				final File file = (File) recentFiles.get(i - 1);
				final String fileName = file.getName();
				final String filePath = file.getAbsolutePath();
				final Action action = new AbstractAction("" + i + ' ' + fileName) {
					{
						// putValue(ACTION_COMMAND_KEY, file.getAbsolutePath()); // MenuItem gets only NAME, SMALL_ICON, SHORT_DESCRIPTION and MNEMONIC_KEY from Action in Java 1.3!
						putValue(SHORT_DESCRIPTION, "Reopen \"" + fileName + '\"'); // when mouse over - tool-tip
						putValue(StatusBar.DESCRIPTION_HOVER, "Open previously opened file \"" + filePath + '\"'); // when mouse over - statusbar
					}
					public void actionPerformed(ActionEvent evt) {
						mainFrame.statusBar.unsetHoverMessageFrom(this);
						menuActionListener.actionPerformed(evt);
					}
				};
				action.putValue(Action.MNEMONIC_KEY, new Integer('0' + i));
				JMenuItem mi = fileListMenu.add(action);
				mi.setActionCommand(filePath);
				mi = fileListPopupMenu.add(action);
				mi.setActionCommand(filePath);
			}
			mainFrame.statusBar.attachToMenuElement(fileListPopupMenu);
			mainFrame.statusBar.attachToMenuElement(fileListMenu);
		}
	}

	public static void buildRecentDirsMenus() {
		deltaSideController[LEFT].buildRecentDirsMenu();
		deltaSideController[RIGHT].buildRecentDirsMenu();
		_buildRecentDirsMenus(mainFrame.jMenuOpenMacrosInDir, recentMacrosOpenActionListener);
		_buildRecentDirsMenus(mainFrame.jMenuOpenProjectInDir, recentProjectOpenActionListener);
	}
	static void _buildRecentDirsMenus(final JMenu fileListMenu, final ActionListener menuActionListener) {
		fileListMenu.removeAll();
		final int size = Math.min(recentDirs.size(), maxRecentFiles);
		if (size == 0)
			fileListMenu.add(EMPTY_MENU_ACTION);
		else {
			for (int i = 1; i <= size; i++) {
				final File file = recentDirs.get(i - 1);
				final String dirName = file.getName();
				final String dirPath = FileUtils.getAbsolutePath(file); //file.getAbsolutePath();
				final Action action = new AbstractAction("" + i + ' ' + dirName) {  // show dirName or dirPath ?
					{
						putValue(ACTION_COMMAND_KEY, dirPath);
						putValue(SHORT_DESCRIPTION, "Open in directory \"" + dirName + '\"'); // when mouse over - tool-tip
						putValue(StatusBar.DESCRIPTION_HOVER, "Open file in directory \"" + dirPath + '\"'); // when mouse over - statusbar
					}
					public void actionPerformed(ActionEvent evt) {
						mainFrame.statusBar.unsetHoverMessageFrom(this);
						menuActionListener.actionPerformed(evt);
					}
				};
				action.putValue(Action.MNEMONIC_KEY, new Integer('0' + i));
				fileListMenu.add(action);
			}
			mainFrame.statusBar.attachToMenuElement(fileListMenu);
		}
	}

	public static void preferencesChanged(Set<String> changedProperties) {
		if (!changedProperties.isEmpty()) {
			fetchProperties();

			deltaSides[LEFT].htmlView.refetchProperties(changedProperties);
			deltaSides[LEFT].xmlView.refetchProperties(changedProperties);
			deltaSides[LEFT].xmlAttributesTable.refetchProperties();
			int dtdHelperState = preferences.getInt(Preferences.KEY_XML_DTD_STATE);
			deltaSideController[LEFT].dtdHelper.setState(dtdHelperState);
			deltaSides[RIGHT].htmlView.refetchProperties(changedProperties);
			deltaSides[RIGHT].xmlView.refetchProperties(changedProperties);
			deltaSides[RIGHT].xmlAttributesTable.refetchProperties();
			deltaSideController[RIGHT].dtdHelper.setState(dtdHelperState);
			mainFrame.macrosView.refetchProperties(changedProperties);
			mainFrame.macroInfoTree.refetchProperties(changedProperties);

			boolean clearRecentFiles = changedProperties.contains(Preferences.KEY_CLEAR_RECENT_FILES);
			if (clearRecentFiles || changedProperties.contains(Preferences.KEY_RECENT_FILES_MAX_COUNT)) {
				if (clearRecentFiles) {
					recentXHTMLFiles.clear();
					recentMacrosFiles.clear();
					recentProjectFiles.clear();
					recentDirs.clear();
				}
				deltaSideController[LEFT].buildRecentFilesMenus();
				deltaSideController[RIGHT].buildRecentFilesMenus();
				buildRecentProjectFilesMenu();
				buildRecentMacrosFilesMenus();
				buildRecentDirsMenus();
			}
			if (MainController.ADVANCED_MODE)
				mainFrame.validModeCheckBox.setSelected(preferences.getInt(Preferences.KEY_XML_DTD_STATE) > DTDHelper.NO_CHECK);
			if (changedProperties.contains(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT))
				Toolkit.getDefaultToolkit().setDynamicLayout(preferences.getBoolean(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT));

			int index = activeSideIndex;
			deltaSides[LEFT].xmlView.reselectCurrentTreeNode();
            deltaSides[RIGHT].xmlView.reselectCurrentTreeNode();
			setActiveSideIndex(index);

			mainFrame.refetchProperties();
			autoChangeInsertMode = preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_AUTO_SWITCH);
			alwaysChangeInsertModeToDefault = preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_SWITCH_BACK);
			resetMacroInsertModes(getActiveSideController().dtdHelper);
		}
	}

	public static XMLLanguageDialog getXMLLanguageDialog() {
		if (xmlLanguageDialog == null)
			xmlLanguageDialog = new XMLLanguageDialog(mainFrame, xmlLanguages);
		return xmlLanguageDialog;
	}

	public static File[] getIconDirs(File baseDir) {
		File[] result = baseDir.listFiles(new FileFilter() {
			public boolean accept(File file) {
				return isIconDir(file);
			}
		});
		if (result == null) result = new File[0];
		return result;
	}

	public static boolean isIconDir(File file) {
		if (file.isDirectory()) {
			File[] files = file.listFiles(new FileFilter() {
				public boolean accept(File file) {
					return file.getPath().endsWith(".gif");
				}
			});
			return ((files != null) && (files.length > 0));
		} 
		return false;
	}

	/**
	 * Resets the undo manager.
	 */
	public static void resetUndoManager(int sideIndex) {
        undoManager.discardEdits(sideIndex);
		aEditUndo.updateAction();
		aEditRedo.updateAction();
	}

	public static boolean alwaysChangeInsertModeToDefault = false;
	public static boolean autoChangeInsertMode = true;
	public static void resetMacroInsertModes(DTDHelperXML4J dtdHelper) {
		if (mainFrame.macrosView.isLoaded()) {
			Element macroElement = mainFrame.macrosView.getMacroXMLElement();
			if (macroElement != null) {
				aMacroInsertModeInto.setEnabled(dtdHelper.getLastInsertIntoIndexForMacro(macroElement) != -1);
				aMacroInsertModeBefore.setEnabled(dtdHelper.getInsertBeforeIndexForMacro(macroElement) != -1);
				aMacroInsertModeAfter.setEnabled(dtdHelper.getInsertAfterIndexForMacro(macroElement) != -1);
				aMacroInsertModeReplace.setEnabled(dtdHelper.canBeReplacedByMacro(macroElement));
			} else {
				aMacroInsertModeInto.setEnabled(false);
				aMacroInsertModeBefore.setEnabled(false);
				aMacroInsertModeAfter.setEnabled(false);
				aMacroInsertModeReplace.setEnabled(false);
			}
			if (autoChangeInsertMode)
				if (((Action) mainFrame.macroInsertModeComboBox.getSelectedItem()).isEnabled()) {
					if (alwaysChangeInsertModeToDefault && aMacroInsertModeInto.isEnabled())
						mainFrame.macroInsertModeComboBox.setSelectedIndex(0);
				} else if (aMacroInsertModeInto.isEnabled()) mainFrame.macroInsertModeComboBox.setSelectedIndex(0);
				else if (aMacroInsertModeAfter.isEnabled()) mainFrame.macroInsertModeComboBox.setSelectedIndex(2);
				else if (aMacroInsertModeBefore.isEnabled()) mainFrame.macroInsertModeComboBox.setSelectedIndex(3);
				else if (aMacroInsertModeReplace.isEnabled()) mainFrame.macroInsertModeComboBox.setSelectedIndex(5);
				else if (alwaysChangeInsertModeToDefault)
					mainFrame.macroInsertModeComboBox.setSelectedIndex(0); // keep user-selected-insert-mode if user switch to HTML side
			mainFrame.macroInsertModeComboBox.repaint();
		}
	}

	private static boolean resetMacroInsertStates() {
		DeltaSideController activeSideController = getActiveSideController();
		if (activeSideController.type == HTML) activeSideController = activeSideController.otherSideController;
		DeltaSideController inactiveSideController = activeSideController.otherSideController; //getInactiveSideController();
		resetMacroInsertModes(activeSideController.dtdHelper);
		boolean macroSelected = mainFrame.macrosView.isMacroSelected();
		if (macroSelected) {
			boolean linksSelectedOnBothSides =
					(activeSideController.thisSide.deltaView.getCurrentDELTALinkID(true) >= LinkIDConstants.FIRST_ID) &&
					(inactiveSideController.thisSide.deltaView.getCurrentDELTALinkID(true) >= LinkIDConstants.FIRST_ID);
            aMacroInsert.setEnabled(((Action)mainFrame.macroInsertModeComboBox.getSelectedItem()).isEnabled()
                && (!mainFrame.macrosView.isMacroLink() ||
                    (!linksSelectedOnBothSides && inactiveSideController.thisSide.deltaView.acceptsNewDELTALink(false, false))));

			aMacroSearchInsertDown.setEnabled(getActiveSideController().dtdHelper.isLoaded());
			aMacroSearchInsertUp.setEnabled(getActiveSideController().dtdHelper.isLoaded());

			mainFrame.macroInfoTree.load(mainFrame.macrosView.getCurrentItem());
			mainFrame.jLabelMacroPreview.setToolTipText(mainFrame.macrosView.getCurrentItem().getToolTipTextVerbose());
			macroSourceViewNeedsUpdate = macroTextViewNeedsUpdate = true;
		} else {
			aMacroInsert.setEnabled(false);
			aMacroSearchInsertDown.setEnabled(false);
			aMacroSearchInsertUp.setEnabled(false);
			mainFrame.macroInfoTree.clear();
			mainFrame.jLabelMacroPreview.setToolTipText("");
		}
		return macroSelected;
	}

	public static void updateUndoActions() {
		aEditUndo.updateAction();
		aEditRedo.updateAction();
	}

	public static String getCurrentUserName() {
		String userName = preferences.getString(Preferences.KEY_USER_NAME);
		if (userName == null) userName = System.getProperty(Preferences.KEY_USER_NAME, "DELT/A-User");
		return userName;
	}

	private static List  sharedXPathList;
	public static List getSharedXPathList() {
		if (sharedXPathList == null) sharedXPathList = stateProperties.getStringList(Preferences.KEY_COMBOBOX_XPATH);
		return sharedXPathList;
	}

	private static List sharedTextValueList;
	public static List getSharedTextValueList() {
		if (sharedTextValueList == null) sharedTextValueList = stateProperties.getStringList(Preferences.KEY_COMBOBOX_EDIT_TEXT);
		return sharedTextValueList;
	}

	public static void swapSides() {
		DeltaSideController deltaSideControllerLeft = deltaSideController[GUIConstants.LEFT];
		deltaSideController[GUIConstants.LEFT] = deltaSideController[GUIConstants.RIGHT];
		deltaSideController[GUIConstants.RIGHT] = deltaSideControllerLeft;
		deltaSideController[LEFT].thisSideIndex = 1 - deltaSideController[LEFT].thisSideIndex;
		deltaSideController[RIGHT].thisSideIndex = 1 - deltaSideController[RIGHT].thisSideIndex;
	}

	private static boolean doNotCheckProcessWhenFileChanged;
	public static void setCreatingNewProcess(boolean active) {
		doNotCheckProcessWhenFileChanged = active;
	}

	static class UndoHandler implements UndoableEditListener {
		private int sideIndex;
		public UndoHandler(int sideIndex) {
			this.sideIndex = sideIndex;
		}

		public void undoableEditHappened(UndoableEditEvent evt) {
			UndoableEdit edit = evt.getEdit();
			Log.trace(SIDE_NAMES[sideIndex]+" undoableEditHappened: "+edit);
			if (edit instanceof SideUndoableEdit)
				((SideUndoableEdit) edit).setOriginSideIndex(sideIndex);
			undoManager.addEdit(edit);
			updateUndoActions();
		}
	}

	private static class MacrosChangeListener implements ChangeListener {
		public void stateChanged(ChangeEvent evt) {
			aMacrosGoBack.setEnabled(mainFrame.macrosView.canGoBack()); // canGoBack() is always true, except for the first time after (re)loading
			aMacrosGoForward.setEnabled(mainFrame.macrosView.canGoForward());
			boolean macroSelected = resetMacroInsertStates();
			mainFrame.jLabelMacroPreview.setIcon(mainFrame.macrosView.getCurrentItemIcon());
			aMacroSwitchToTreeView.setEnabled(macroSelected);
			aMacroSwitchToSourceView.setEnabled(macroSelected);
			aMacroSwitchToTextView.setEnabled(macroSelected);
			aMacroSwitchToTreeView.setSelected(true);
			aMacroSearchInsertDown.setEnabled(getActiveSideController().isTypeXML() && macroSelected);
			aMacroSearchInsertUp.setEnabled(getActiveSideController().isTypeXML() && macroSelected);
		}
	}

	public static final class XMLClipBoard {
		public XMLClipBoard(XMLTreeNode treeNode, int origin, ClipBoardMode clipBoardMode) {
			this.treeNode = treeNode;
			xmlNode = treeNode.getXMLNode();
			this.origin = origin;
			this.clipBoardMode = clipBoardMode;
			treeNode.setClipBoardMode(clipBoardMode);
		}

		public XMLTreeNode treeNode; // needed only to remove treNode after cut
		public Node xmlNode;
		public int origin; // LEFT or RIGHT side
		public ClipBoardMode clipBoardMode;

		public final static class ClipBoardMode {
			public final static ClipBoardMode COPY = new ClipBoardMode();
			public final static ClipBoardMode CUT = new ClipBoardMode();
			private ClipBoardMode() { }
		}
	}

	public static final Comparator<XmlLanguage> XML_LANGUAGE_COMPARATOR = new Comparator<XmlLanguage>() {
		public int compare(XmlLanguage lang1, XmlLanguage lang2) {
			return lang1.getName().compareToIgnoreCase(lang2.getName());
		}
	};

}

