/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 16.05.2003
 * Time: 11:18:59
 */
package at.ac.tuwien.asgaard.delta.gui;

import at.ac.tuwien.asgaard.delta.gui.actions.EmptyAction;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FilePrintAction;
import at.ac.tuwien.asgaard.delta.gui.actions.file.FilePrintPreviewAction;
import at.ac.tuwien.asgaard.delta.gui.html.HTMLView;
import at.ac.tuwien.asgaard.delta.gui.html.SimpleHTMLView;
import at.ac.tuwien.asgaard.delta.gui.images.LazyResourceLoader;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosInsertableView;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosSearchView;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosView;
import at.ac.tuwien.asgaard.delta.gui.misc.AutoXPathBar;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.gui.misc.XPathExpressionEditDialog;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.gui.xml.SimpleXMLTree;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLAttributesTable;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLView;
import at.ac.tuwien.asgaard.delta.gui.xml.images.XMLImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.*;
import at.ac.tuwien.asgaard.swing.autoComplete.AutoCompleteComboBox;
import at.ac.tuwien.asgaard.swing.models.SimpleComboBoxModel;
import at.ac.tuwien.asgaard.swing.selectable.SelectableAction;
import at.ac.tuwien.asgaard.swing.selectable.SelectableCheckBoxMenuItem;
import at.ac.tuwien.asgaard.swing.selectable.SelectableRadioButtonMenuItem;
import at.ac.tuwien.asgaard.swing.selectable.SelectableToggleButton;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import javax.swing.*;
import javax.swing.border.*;

/**
 * Class MainFrame ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public final class MainFrame extends JFrame implements GUIConstants {
	public final DeltaSide[] deltaSides;
	public StatusBar statusBar;
	private JSplitPane jSplitMain, jSplitSides, jSplitMacros;
	public JLabel jLabelMacrosFileName;
	JButton jButtonMacrosReopen;
	public MacrosView macrosView;
	JLabel jLabelMacroPreview;
	private CardPanel cardPanelMacros, cardPanelMacroInfo;
	SimpleXMLTree macroInfoTree;
	private SimpleHTMLView jEditorPaneMacroSource;
	private JTextArea jTextAreaMacroSource;

	JMenu jMenuMacrosReopen, jMenuOpenMacrosInDir;
	JMenu jMenuProjectReopen, jMenuOpenProjectInDir, jMenuProject, jMenuProjectSteps, jMenuProjectProcessTemplates;

	JPopupMenu jPopupMenuMacrosReopen;
	public Action pasteSubMenuAction;
	private JToolBar macrosToolBarLeft, macrosToolBarRight;
	private final Preferences preferences, stateProperties;
	public JLabel projectNameLabel, clipBoardLabel;
	JToggleButton validModeCheckBox;
	public static final String CLIPBOARD_LABEL_TOOLTIP = "<html>Last (XML-)clipboard action: COPY or CUT";
	public JComboBox macroInsertModeComboBox;
	public ExtendedButtonGroup macroInsertModeButtonGroup;

	private static boolean toolbarsFloatable, toolbarsRollover, splitpanesContinuous;
	private LineBorder sideBorderActive, sideBorderInactive;

	MainFrame(Preferences preferences, Preferences stateProperties) {
		super("Document Exploration and Linking Tool (with Addons) v2.7.3" + (MainController.ADVANCED_MODE ? " (Advanced Mode)" : ""));
		Log.trace("enter: MainFrame.MainFrame()");
		this.preferences = preferences;
		this.stateProperties = stateProperties;
		setIconImage(LazyResourceLoader.getImageRelativeToClass(LazyResourceLoader.APP_ICON));
		fetchProperties();
		deltaSides = new DeltaSide[2];
		buildGUI();
	// initLayout();

		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		Log.trace("exit:  MainFrame.MainFrame()");
	}

	private void fetchProperties() {
//		Object o = preferences.get(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER);
//		if (o == null) toolbarsRollover = GlobalConstants.JAVA_1_5;
//		else toolbarsRollover = Boolean.valueOf((String) o).booleanValue();
		toolbarsRollover = preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER);
		toolbarsFloatable = preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_FLOATABLE);
		splitpanesContinuous = preferences.getBoolean(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT);
		Color color = preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_ACTIVE);
		if ((sideBorderActive == null) || !sideBorderActive.getLineColor().equals(color))
			sideBorderActive = new LineBorder(color, SIDE_BORDER_THICKNESS, false);
		color = preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_INACTIVE);
		if ((sideBorderInactive == null) || !sideBorderInactive.getLineColor().equals(color))
			sideBorderInactive = new LineBorder(color, SIDE_BORDER_THICKNESS, false);
		color = preferences.getColor(Preferences.KEY_COLOR_SELECTION);
		UIManager.put("MenuItem.selectionBackground", color);
		UIManager.put("Menu.selectionBackground", color);
		UIManager.put("RadioButtonMenuItem.selectionBackground", color);
		UIManager.put("ComboBox.selectionBackground", color);
		UIManager.put("TextField.selectionBackground", color);
		UIManager.put("TextArea.selectionBackground", color);
		UIManager.put("TextPane.selectionBackground", color);
		UIManager.put("EditorPane.selectionBackground", color);
		UIManager.put("Spinner.selectionBackground", color);
		UIManager.put("List.selectionBackground", color);
		UIManager.put("Table.selectionBackground", color);
	}

	void refetchProperties() {
		boolean leftActive = (deltaSides[LEFT].getBorder() == sideBorderActive);
		boolean rightActive = (deltaSides[RIGHT].getBorder() == sideBorderActive);
		fetchProperties();
		//... //todo
		if (leftActive)
			deltaSides[LEFT].setBorder(sideBorderActive);
		else
			deltaSides[LEFT].setBorder(sideBorderInactive);
		if (rightActive)
			deltaSides[RIGHT].setBorder(sideBorderActive);
		else
			deltaSides[RIGHT].setBorder(sideBorderInactive);
	}

	void installActions() {
		fillMacrosToolBarsAndPopupMenus();
		deltaSides[LEFT].fillToolBarsAndPopupMenus();
		deltaSides[RIGHT].fillToolBarsAndPopupMenus();
		initMenuBar();

		jSplitMain.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).getParent().put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_MASK), "startResize");
		deltaSides[LEFT].xmlSplit.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).getParent().put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_MASK), "startResize");
		deltaSides[RIGHT].xmlSplit.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).getParent().put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_MASK), "startResize");
		jSplitSides.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).getParent().put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_MASK), "startResize");
		jSplitMacros.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).getParent().put(KeyStroke.getKeyStroke(KeyEvent.VK_F2, InputEvent.CTRL_MASK), "startResize");
	}

	private void buildGUI() {
		Log.trace("enter: MainFrame.buildGUI()");

		statusBar = new StatusBar();
		statusBar.setFont(GlobalObjects.BOLD_FIXED_FONT);

		pasteSubMenuAction = new EmptyAction("Paste (XML)", MainImages.PASTE_ICON, 'P', false);

		deltaSides[LEFT] = new DeltaSide(LEFT);
		deltaSides[RIGHT] = new DeltaSide(RIGHT);
		Container contentPane = rootPane.getContentPane();
		jSplitSides = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, splitpanesContinuous,
				deltaSides[LEFT], deltaSides[RIGHT]);
		jSplitSides.setOneTouchExpandable(true);
		jSplitSides.setResizeWeight(0.5);

		jSplitMain = new JSplitPane(JSplitPane.VERTICAL_SPLIT, splitpanesContinuous,
				jSplitSides, createMacrosPart());
		jSplitMain.setOneTouchExpandable(true);
		jSplitMain.setResizeWeight(0.75);

		JPanel southPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.FILL, TableLayout.PREFERRED, 40.0, 150.0}, {20.0}}));
		southPanel.setBorder(new CompoundBorder(BorderFactory.createLoweredBevelBorder(), new EmptyBorder(1, 0, 1, 0)));

	//	if (MainController.ADVANCED_MODE) {
			boolean validModeOn = (preferences.getInt(Preferences.KEY_XML_DTD_STATE) > DTDHelper.NO_CHECK);
			validModeCheckBox = new JToggleButton(validModeOn ? MainImages.VALID_MODE_ON_ICON : MainImages.VALID_MODE_OFF_ICON, validModeOn);
			validModeCheckBox.setToolTipText("Valid Mode: " + (validModeOn ? "\"On\"" : "\"Off\""));
			validModeCheckBox.setFocusPainted(false);
			validModeCheckBox.setFocusable(false);
			//  validModeCheckBox.setContentAreaFilled(false);
			//  validModeCheckBox.setBorderPainted(true);
			//  validModeCheckBox.setBorder(BorderFactory.createLoweredBevelBorder());
		// validModeCheckBox.setText("");
			validModeCheckBox.setMargin(GlobalObjects.INSETS_ZERO);
			validModeCheckBox.setPreferredSize(new Dimension(16,14));
			JPanel validModePanel = new JPanel(new BorderLayout());
			validModePanel.setBorder(new CompoundBorder(new MatteBorder(0, 1, 0, 0, Color.GRAY), new EmptyBorder(1, 4, 1, 4)));
			validModePanel.add(new JLabel("Valid Mode: "), BorderLayout.WEST);
			validModePanel.add(validModeCheckBox);
			southPanel.add(validModePanel, new TableLayoutConstraints(1, 0));
	//	}

		clipBoardLabel = new JLabel("-");
		clipBoardLabel.setHorizontalAlignment(SwingConstants.CENTER);
		clipBoardLabel.setBorder(new MatteBorder(0, 1, 0, 1, Color.GRAY));
		clipBoardLabel.setFont(clipBoardLabel.getFont().deriveFont(Font.ITALIC)); //StyleInfo.ITALIC_FONT);
		clipBoardLabel.setToolTipText(CLIPBOARD_LABEL_TOOLTIP);

		projectNameLabel = new JLabel("-");
		projectNameLabel.setHorizontalAlignment(SwingConstants.CENTER);
		projectNameLabel.setToolTipText("No active project/process.");

		southPanel.add(statusBar, new TableLayoutConstraints());
		southPanel.add(clipBoardLabel, new TableLayoutConstraints(2, 0));
		southPanel.add(projectNameLabel, new TableLayoutConstraints(3, 0));
		contentPane.add(jSplitMain, BorderLayout.CENTER);
		contentPane.add(southPanel, BorderLayout.SOUTH);

		setJMenuBar(new JMenuBar());
		Log.trace("exit:  MainFrame.buildGUI()");
	}

	private Component createMacrosPart() {
		JPanel panelLeft = new JPanel(new BorderLayout());

		// create Macros Toolbar
		macrosToolBarLeft = new SmallToolBar("Macros View");
		panelLeft.add(macrosToolBarLeft, BorderLayout.NORTH);
		MacrosSearchView macrosSearchView = new MacrosSearchView(preferences, statusBar);
		MacrosInsertableView macrosInsertableView = new MacrosInsertableView();
		macrosView = new MacrosView(preferences, macrosSearchView, macrosInsertableView);
		cardPanelMacros = new CardPanel();
		cardPanelMacros.add(new JScrollPane(macrosView), CARD_MACROS_STRUCTURE);
		cardPanelMacros.add(macrosSearchView, CARD_MACROS_SEARCH);
		cardPanelMacros.add(macrosInsertableView, CARD_MACROS_INSERTABLE);
		panelLeft.add(cardPanelMacros, BorderLayout.CENTER);

		// create macro preview part
		macroInfoTree = new SimpleXMLTree(preferences);
		jEditorPaneMacroSource = new SimpleHTMLView();
		jTextAreaMacroSource = new JTextArea();
		jTextAreaMacroSource.setBackground(GlobalObjects.DEFAULT_BACKGROUND);
		jTextAreaMacroSource.setEditable(false);
		cardPanelMacroInfo = new CardPanel();
		cardPanelMacroInfo.add(new JScrollPane(macroInfoTree), CARD_XML);
		cardPanelMacroInfo.add(new JScrollPane(jEditorPaneMacroSource), CARD_SIMPLE_HTML);
		cardPanelMacroInfo.add(new JScrollPane(jTextAreaMacroSource), CARD_SOURCE_PLAIN);
		macrosToolBarRight = new SmallToolBar("Macro Preview");
		JPanel panelRight = new JPanel(new BorderLayout());
		panelRight.add(macrosToolBarRight, BorderLayout.NORTH);
		panelRight.add(cardPanelMacroInfo, BorderLayout.CENTER);

		jSplitMacros = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, splitpanesContinuous,
				panelLeft, panelRight);
		jSplitMacros.setOneTouchExpandable(true);
		jSplitMacros.setResizeWeight(0.75);
		return jSplitMacros;
	}

	private void fillMacrosToolBarsAndPopupMenus() {
		// init Macros Toolbar
		JLabel label = new JLabel("Macros View");
		label.setIcon(MainImages.MACRO_ICON);
	//	label.setFont(new Font("SansSerif", Font.BOLD, 12));
		macrosToolBarLeft.add(label);
		macrosToolBarLeft.add(Box.createHorizontalStrut(15));

		jLabelMacrosFileName = new JLabel();
		jLabelMacrosFileName.setFont(SAVED_FILENAME_FONT);
		macrosToolBarLeft.add(jLabelMacrosFileName);
		macrosToolBarLeft.add(Box.createGlue());
		macrosToolBarLeft.add(MainController.aMacrosOpen);
		jButtonMacrosReopen = macrosToolBarLeft.add(MainController.aMacrosReopen);
		jButtonMacrosReopen.setIcon(MainImages.DOWN_ICON);
		jButtonMacrosReopen.setPreferredSize(DOWN_BUTTON_SIZE);
		macrosToolBarLeft.add(Box.createHorizontalStrut(2));
		macrosToolBarLeft.add(MainController.aMacrosReload);
		if (MainController.ADVANCED_MODE) {
			macrosToolBarLeft.add(Box.createHorizontalStrut(8));
			macrosToolBarLeft.add(MainController.aMacrosOpenAsXML);
			macrosToolBarLeft.add(MainController.aMacrosOpenXMLInMacrosView);
		}
		macrosToolBarLeft.add(Box.createHorizontalStrut(15));
		macrosToolBarLeft.add(MainController.aMacrosGoBack);
		macrosToolBarLeft.add(MainController.aMacrosGoForward);
		if (MainController.ADVANCED_MODE) {
			macrosToolBarLeft.add(Box.createHorizontalStrut(30));
			macrosToolBarLeft.add(MainController.aMacrosShowEditor); 
		}

		macrosToolBarLeft.add(Box.createHorizontalStrut(30));
		JComponent buttonPanel = new JToolBar();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.setBorder(GlobalObjects.EMPTY_BORDER_0); // BorderFactory.createEtchedBorder());
		ButtonGroup buttonGroup = new ButtonGroup();
		JToggleButton toggleButton = new SelectableToggleButton(MainController.aMacrosSwitchToStructureView);
		toggleButton.setText("");
		toggleButton.setMargin(GlobalObjects.INSETS_ZERO);
		buttonPanel.add(toggleButton);
		buttonPanel.add(Box.createHorizontalStrut(2));
		buttonGroup.add(toggleButton);
		toggleButton = new SelectableToggleButton(MainController.aMacrosSwitchToSearchView);
		toggleButton.setText("");
		toggleButton.setMargin(GlobalObjects.INSETS_ZERO);
		buttonPanel.add(toggleButton);
		buttonPanel.add(Box.createHorizontalStrut(2));
		buttonGroup.add(toggleButton);
		toggleButton = new SelectableToggleButton(MainController.aMacrosSwitchToInsertableView);
		toggleButton.setText("");
		toggleButton.setMargin(GlobalObjects.INSETS_ZERO);
		buttonPanel.add(toggleButton);
		buttonGroup.add(toggleButton);
		macrosToolBarLeft.add(buttonPanel);

		statusBar.attachToToolBar(macrosToolBarLeft);

		// init Macro preview toolbar
		jLabelMacroPreview = new JLabel("Macro Preview ");
		macrosToolBarRight.add(jLabelMacroPreview);
		macrosToolBarRight.add(Box.createGlue());
		macrosToolBarRight.add(MainController.aMacroSearchInsertDown);
		macrosToolBarRight.add(MainController.aMacroSearchInsertUp);
		macrosToolBarRight.add(Box.createGlue());
		macroInsertModeComboBox = new BlockComboBox(new Object[]{MainController.aMacroInsertModeInto,
				BlockComboBox.SEPARATOR,
				MainController.aMacroInsertModeAfter,
				MainController.aMacroInsertModeBefore,
				BlockComboBox.SEPARATOR,
				MainController.aMacroInsertModeReplace});
		macroInsertModeComboBox.setMaximumSize(new Dimension(80, 22));
		macrosToolBarRight.add(macroInsertModeComboBox);
		macrosToolBarRight.add(MainController.aMacroInsert);
		statusBar.attachToToolBar(macrosToolBarRight);

		// init Macros PopupMenu
		JPopupMenu popupMenu = new JPopupMenu("Macro Preview");
		JMenu subMenu = new JMenu("View");
		subMenu.setIcon(MainImages.VIEW_ICON);
		subMenu.setMnemonic(KeyEvent.VK_V);
		buttonGroup = new ButtonGroup();
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToTreeView)));
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToSourceView)));
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToTextView)));
		popupMenu.add(subMenu);
		popupMenu.addSeparator();
		popupMenu.add(MainController.aMacroInsert);
		statusBar.attachToMenuElement(popupMenu);

		// install PopupMenu-MouseListener for Macros components
		MouseListener popupListener = new PopupMenuHandler(popupMenu);
		macroInfoTree.addMouseListener(popupListener);
		jEditorPaneMacroSource.addMouseListener(popupListener);
		jTextAreaMacroSource.addMouseListener(popupListener);
		jPopupMenuMacrosReopen = new JPopupMenu();
	}

	private void initMenuBar() {
		JMenuBar menuBar = rootPane.getJMenuBar();
		menuBar.add(createFileMenu());
		menuBar.add(createEditMenu());
		menuBar.add(createProjectMenu());
		menuBar.add(deltaSides[LEFT].createMenu());
		menuBar.add(deltaSides[RIGHT].createMenu());
		menuBar.add(createMacrosMenu());
//  menuBar.add(Box.createHorizontalGlue());  // space before Help-Menu ?
		menuBar.add(createOptionsMenu());
		menuBar.add(createHelpMenu());
		statusBar.attachToMenuBar(menuBar);
	}

	private JMenu createProjectMenu() {
		jMenuProject = new JMenu("Project");
		jMenuProject.setMnemonic(KeyEvent.VK_P);
		jMenuProject.add(MainController.aProjectNew);
		jMenuProjectProcessTemplates = new JMenu(MainController.aMenuProjectProcessTemplates);
		jMenuProject.add(jMenuProjectProcessTemplates);
		jMenuProject.add(MainController.aProjectOpen);
		jMenuProjectReopen = new JMenu(MainController.aMenuProjectReopen);
		jMenuProject.add(jMenuProjectReopen);
		jMenuOpenProjectInDir = new JMenu(MainController.aOpenInMenu);
		jMenuProject.add(jMenuOpenProjectInDir);
		jMenuProject.add(MainController.aProjectClose);
		jMenuProject.addSeparator();
		jMenuProject.add(MainController.aProjectShowPropertiesDialog);
		jMenuProject.addSeparator();
		jMenuProjectSteps = new JMenu(MainController.aMenuProjectSteps);
		jMenuProject.add(jMenuProjectSteps);
		jMenuProjectSteps.setEnabled(false);
		return jMenuProject;
	}

	private JMenu createEditMenu() {
		JMenu menu = new JMenu("Edit");
		menu.setMnemonic(KeyEvent.VK_E);
		menu.add(new JMenuItem(MainController.aEditUndo));
		menu.add(new JMenuItem(MainController.aEditRedo));
		menu.addSeparator();
		menu.add(new JMenuItem(MainController.aEditCut));
		menu.add(new JMenuItem(MainController.aEditCopy));
		JMenu subMenu = new JMenu(pasteSubMenuAction);
		subMenu.setIcon(MainImages.PASTE_ICON);
		subMenu.setMnemonic(KeyEvent.VK_P);
		subMenu.add(MainController.aEditPasteInto);
		subMenu.addSeparator();
		subMenu.add(MainController.aEditPasteBefore);
		subMenu.add(MainController.aEditPasteAfter);
		subMenu.addSeparator();
		subMenu.add(MainController.aEditPasteReplace);
		subMenu.setEnabled(false);
		menu.add(subMenu);
		menu.addSeparator();
		menu.add(new JMenuItem(MainController.aEditFind));
	// menu.add(new JMenuItem(MainController.aEditReplace)); // ......
		menu.add(new JMenuItem(MainController.aEditFindNext));
		menu.add(new JMenuItem(MainController.aEditFindPrevious));
		menu.add(new JMenuItem(MainController.aEditHighlightSearchResults)); // ......
		if (MainController.ADVANCED_MODE) {
			menu.addSeparator();
			menu.add(MainController.aEditShowInsertableElementsDialog);
			menu.add(MainController.aEditShowAllElementsDialog);
		}
		return menu;
	}

	private JMenu createFileMenu() {
		JMenu menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		menu.add(new JMenuItem(MainController.aFileSaveAll));
		menu.add(new JMenuItem(MainController.aFileCloseAll));
		menu.add(new JMenuItem(MainController.aFileSwapSides));

		menu.addSeparator();
		menu.add(new JMenuItem(new FilePrintPreviewAction()));
		menu.add(new JMenuItem(new FilePrintAction()));
		menu.addSeparator();
		menu.add(new JMenuItem(MainController.aFileExit));
		return menu;
	}

	private JMenu createMacrosMenu() {
		JMenu menu = new JMenu("Macro File");
		menu.setMnemonic(KeyEvent.VK_M);
		menu.add(MainController.aMacrosOpen);
		jMenuMacrosReopen = new JMenu(MainController.aMacrosReopen);
		menu.add(jMenuMacrosReopen);
		jMenuOpenMacrosInDir = new JMenu(MainController.aOpenInMenu);
		menu.add(jMenuOpenMacrosInDir);
		menu.add(MainController.aMacrosReload);
		menu.add(MainController.aMacrosClose);
		if (MainController.ADVANCED_MODE) {
			menu.addSeparator();
			menu.add(MainController.aMacrosOpenAsXML);
			menu.add(MainController.aMacrosOpenXMLInMacrosView);
			menu.addSeparator();
			menu.add(MainController.aMacrosShowEditor);
		}
		menu.addSeparator();
		JMenu subMenu = new JMenu("View");
		subMenu.setIcon(MainImages.VIEW_ICON);
		subMenu.setMnemonic(KeyEvent.VK_V);

		// Macro Preview submenu
		JMenu subMenuPreview = new JMenu("Macro Preview");
		subMenuPreview.setIcon(MainImages.VIEW_ICON);
		subMenuPreview.setBorder(INTENTED_BORDER);
		subMenuPreview.setMnemonic(KeyEvent.VK_P);
		ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(subMenuPreview.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToTreeView)));
		buttonGroup.add(subMenuPreview.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToSourceView)));
		buttonGroup.add(subMenuPreview.add(new SelectableRadioButtonMenuItem(MainController.aMacroSwitchToTextView)));
		subMenu.add(subMenuPreview);
		subMenu.addSeparator();
		buttonGroup = new ButtonGroup();
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacrosSwitchToStructureView)));
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacrosSwitchToSearchView)));
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacrosSwitchToInsertableView)));
		menu.add(subMenu);
		menu.addSeparator();
		menu.add(MainController.aMacroInsert);
		subMenu = new JMenu("Macro Insert Mode");
		subMenu.setIcon(MainImages.PREFERENCES_ICON);
		subMenu.setMnemonic(KeyEvent.VK_I);
		buttonGroup = new ButtonGroup();
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroInsertModeInto)));
		subMenu.addSeparator();
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroInsertModeAfter)));
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroInsertModeBefore)));
		subMenu.addSeparator();
		buttonGroup.add(subMenu.add(new SelectableRadioButtonMenuItem(MainController.aMacroInsertModeReplace)));
		menu.add(subMenu);
		return menu;
	}

	private JMenu createOptionsMenu() {
		JMenu menu = new JMenu("Options");
		menu.setMnemonic(KeyEvent.VK_O);
	//	if (MainController.ADVANCED_MODE) {
			menu.add(MainController.aOptionsLanguageShowPropertiesDialog);
			menu.addSeparator();
			menu.add(MainController.aOptionsShowProcessTemplatesDialog);
			menu.addSeparator();
	//	}
		menu.add(MainController.aOptionsShowPreferencesDialog);
		return menu;
	}

	private JMenu createHelpMenu() {
		JMenu menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);
		menu.add(MainController.aHelpShowHelp);
		JMenu subMenu = new JMenu("WWW Resources");
		subMenu.setIcon(MainImages.WWW_ICON);
		subMenu.setMnemonic(KeyEvent.VK_W);
		subMenu.add(MainController.aHelpWWWOpenDELTASite);
		subMenu.add(MainController.aHelpWWWOpenDELTAManual);
		subMenu.add(MainController.aHelpWWWOpenAsgaardSite);
		menu.add(subMenu);
		menu.addSeparator();
		menu.add(MainController.aHelpCheckVersion);
		menu.add(MainController.aHelpRefreshGUI);
		menu.add(MainController.aHelpResetLayout);
		menu.addSeparator();
		menu.add(MainController.aHelpShowAbout);
		return menu;
	}

	public void resetLayout() {  // FIXME: needs 2 or more invocations until layout is really resetted
		// rootPane.getContentPane().invalidate();
		applyLayout(false);
	}

	 void initLayout() {
// ??? do I need this?
		applyLayout(preferences.getBoolean(Preferences.KEY_LAYOUT_RESTORE));
	}

	private void applyLayout(boolean restoreLayout) {
		int winWidth, winHeight;
		if (restoreLayout) {// center window
			Rectangle scrnSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
			winWidth = stateProperties.getInt(Preferences.KEY_LAYOUT_WINDOW_SIZE_WIDTH,
								(int) (scrnSize.getWidth() * 0.95), restoreLayout);
			winHeight = stateProperties.getInt(Preferences.KEY_LAYOUT_WINDOW_SIZE_HEIGHT,
										(int) (scrnSize.getHeight() * 0.95), restoreLayout);
			int winPosX = stateProperties.getInt(Preferences.KEY_LAYOUT_WINDOW_POS_X,
										(scrnSize.width - winWidth) >> 1, restoreLayout);
			int winPosY = stateProperties.getInt(Preferences.KEY_LAYOUT_WINDOW_POS_Y,
										(scrnSize.height - winHeight) >> 1, restoreLayout);
			setBounds(winPosX, winPosY, winWidth, winHeight);
			invalidate();
		} else { winWidth = getWidth(); winHeight = getHeight();}

		int divLoc;

		jSplitMain.setSize(winWidth, winHeight - rootPane.getJMenuBar().getHeight() - statusBar.getHeight());
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_MAIN_DIV_LOC, -1)) != -1))
			jSplitMain.setDividerLocation(divLoc);
		else
			jSplitMain.setDividerLocation(0.75);
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_MAIN_DIV_LOC_LAST, -1)) != -1))
			jSplitMain.setLastDividerLocation(divLoc); // is only necessary if DividerLocation = minimum | maximum for the oneTouchButton
		jSplitMain.invalidate();

		jSplitSides.setSize(winWidth, jSplitMain.getDividerLocation());
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_SIDES_DIV_LOC, -1)) != -1))
			jSplitSides.setDividerLocation(divLoc);
		else
			jSplitSides.setDividerLocation(0.5);
		if (restoreLayout
				&& ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_SIDES_DIV_LOC_LAST, -1)) != -1))
			jSplitSides.setLastDividerLocation(divLoc);
		jSplitSides.revalidate();

		deltaSides[LEFT].xmlSplit.setSize(jSplitSides.getDividerLocation(), jSplitMain.getDividerLocation());
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_LEFT_DIV_LOC, -1)) != -1))
			deltaSides[LEFT].xmlSplit.setDividerLocation(divLoc);
		else
			deltaSides[LEFT].xmlSplit.setDividerLocation(0.75);
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_LEFT_DIV_LOC_LAST, -1)) != -1))
			deltaSides[LEFT].xmlSplit.setLastDividerLocation(divLoc);
		deltaSides[LEFT].xmlSplit.invalidate();

		deltaSides[RIGHT].xmlSplit.setSize(winWidth - jSplitSides.getDividerLocation(), jSplitMain.getDividerLocation());
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC, -1)) != -1))
			deltaSides[RIGHT].xmlSplit.setDividerLocation(divLoc);
		else
			deltaSides[RIGHT].xmlSplit.setDividerLocation(0.75);
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC_LAST, -1)) != -1))
			deltaSides[RIGHT].xmlSplit.setLastDividerLocation(divLoc);
		deltaSides[RIGHT].xmlSplit.invalidate();

		jSplitMacros.setSize(winWidth, jSplitMain.getHeight() - jSplitMain.getDividerLocation());
		if (restoreLayout
				&& ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_MACROS_DIV_LOC, -1)) != -1))
			jSplitMacros.setDividerLocation(divLoc);
		else
			jSplitMacros.setDividerLocation(0.75);
		if (restoreLayout && ((divLoc = stateProperties.getInt(Preferences.KEY_LAYOUT_SPLIT_MACROS_DIV_LOC_LAST, -1)) != -1))
			jSplitMacros.setLastDividerLocation(divLoc);
	}

	void storeLayout(Preferences stateProperties) {
		stateProperties.putInt(Preferences.KEY_LAYOUT_WINDOW_POS_X, getX());
		stateProperties.putInt(Preferences.KEY_LAYOUT_WINDOW_POS_Y, getY());
		stateProperties.putInt(Preferences.KEY_LAYOUT_WINDOW_SIZE_WIDTH, getWidth());
		stateProperties.putInt(Preferences.KEY_LAYOUT_WINDOW_SIZE_HEIGHT, getHeight());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_MAIN_DIV_LOC, jSplitMain.getDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_MAIN_DIV_LOC_LAST, jSplitMain.getLastDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_SIDES_DIV_LOC, jSplitSides.getDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_SIDES_DIV_LOC_LAST, jSplitSides.getLastDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_LEFT_DIV_LOC, deltaSides[LEFT].xmlSplit.getDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_LEFT_DIV_LOC_LAST,
				deltaSides[LEFT].xmlSplit.getLastDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC,
				deltaSides[RIGHT].xmlSplit.getDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC_LAST,
				deltaSides[RIGHT].xmlSplit.getLastDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_MACROS_DIV_LOC, jSplitMacros.getDividerLocation());
		stateProperties.putInt(Preferences.KEY_LAYOUT_SPLIT_MACROS_DIV_LOC_LAST, jSplitMacros.getLastDividerLocation());
	}

	public void ensureMacrosPartIsVisible() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				if (jSplitMain.getDividerLocation() >= jSplitMain.getMaximumDividerLocation()) {
					if (jSplitMain.getLastDividerLocation() >= jSplitMain.getMaximumDividerLocation())
						jSplitMain.setDividerLocation(0.75);
					else
						jSplitMain.setDividerLocation(jSplitMain.getLastDividerLocation());
					//	jSplitMain.revalidate();
				}
				if ((jSplitMacros.getDividerLocation() <= jSplitMacros.getMinimumDividerLocation()) ||
							(jSplitMacros.getDividerLocation() >= jSplitMacros.getMaximumDividerLocation())) {
					if ((jSplitMacros.getDividerLocation() <= jSplitMacros.getMinimumDividerLocation()) ||
							(jSplitMacros.getDividerLocation() >= jSplitMacros.getMaximumDividerLocation()))
						jSplitMacros.setDividerLocation(0.75);
					else
						jSplitMacros.setDividerLocation(jSplitMacros.getLastDividerLocation());
					//	jSplitMacros.revalidate();
				}
			}
		});
	}
	public void hideMacrosPart() {
		jSplitMain.setDividerLocation(1.0);
		jSplitMain.revalidate();  // todo does not work on load
	}

	public void switchToMacrosStructureView() {
		cardPanelMacros.switchToCard(CARD_MACROS_STRUCTURE);
		MainController.aMacrosGoBack.setEnabled(macrosView.canGoBack());
		MainController.aMacrosGoForward.setEnabled(macrosView.canGoForward());
	}

	public void switchToMacrosSearchView() {
		cardPanelMacros.switchToCard(CARD_MACROS_SEARCH);
		MainController.aMacrosGoBack.setEnabled(false);
		MainController.aMacrosGoForward.setEnabled(false);
	}

	public void switchToMacrosInsertableView() {
		cardPanelMacros.switchToCard(CARD_MACROS_INSERTABLE);
		MainController.aMacrosGoBack.setEnabled(false);
		MainController.aMacrosGoForward.setEnabled(false);
	}

	public void switchToMacroTreeView() {
		cardPanelMacroInfo.switchToCard(CARD_XML);
		macroInfoTree.requestFocusInWindow();
	}

	public void switchToMacroSourceView() {
		cardPanelMacroInfo.switchToCard(CARD_SIMPLE_HTML);
		if (MainController.macroSourceViewNeedsUpdate) {
			jEditorPaneMacroSource.setText(macrosView.getMacroSource());
			jEditorPaneMacroSource.setCaretPosition(0);
			MainController.macroSourceViewNeedsUpdate = false;
		}
	}

	public void switchToMacroTextView() {
		cardPanelMacroInfo.switchToCard(CARD_SOURCE_PLAIN);
		if (MainController.macroTextViewNeedsUpdate) {
			jTextAreaMacroSource.setText(macrosView.getMacroSourcePlain());
			jTextAreaMacroSource.setCaretPosition(0);
			MainController.macroTextViewNeedsUpdate = false;
		}
	}

	public void setStatusBarActiveMessage(Action action, boolean set) {
		if (set)
			statusBar.setActiveMessageFrom(action);
		else
			statusBar.unsetActiveMessageFrom(action);
	}

	public void refreshGUI() {
		try {
			rootPane.getContentPane().invalidate();
			rootPane.getContentPane().validate();
			rootPane.getContentPane().repaint();
			deltaSides[RIGHT].xmlView.reloadModel();
			deltaSides[RIGHT].xmlView.revalidate();
			deltaSides[RIGHT].xmlView.repaint();
			deltaSides[LEFT].xmlView.reloadModel();
			deltaSides[LEFT].xmlView.revalidate();
			deltaSides[LEFT].xmlView.repaint();
		} finally {
			statusBar.clearAll();
		}
	}

	public final class DeltaSide extends JPanel {
		JLabel xmlNodeLabel;
		public final DeltaSideController sideController;
		public DeltaSideView deltaView;
		public final HTMLView htmlView;
		public final XMLView xmlView;
		public final XMLAttributesTable xmlAttributesTable;
		public final JTextArea textOutput, xmlNodeInfoText;
		public final SimpleHTMLView htmlOutput;
		public final JLabel fileNameLabel, langNameLabel, filterStateLabel;
		final JSplitPane xmlSplit;
		final CardPanel mainPanel, xmlInfoPanel;
		public final JComboBox newAttributesComboBox;
		public final SimpleComboBoxModel newAttributesComboBoxModel;
		public final JFileChooser fileChooser;
		JMenu jMenuReopen, jMenuOpenInDir, jMenuTemplates, jMenuNewFromOtherLang;
		JPopupMenu jPopupMenuReopen, jPopupMenuNewFromTemplateOrOtherLang;
		public int sideIndex; // cannot be final because "SwapSides" features changes it
		JPopupMenu popupMenuXMLNodeTextArea;
		final JPanel xmlPanel;
		private final JToolBar toolBarMain, toolBarAttr, toolBarXMLNode;

		public final AutoCompleteComboBox xpathBarText;
		private JComponent xpathBar, treePathPanel, autoXPathPanel;
		private TreePathBar treePathBar;
		private AutoXPathBar autoXPathBar;

		public JButton jButtonReopen, jButtonNewFromTemplateOrOtherLang;

		// TODO could be removed, because these menus are not shown anymore, but they are still used in DeltaSideController
		public JMenu subMenuLangHighlight=new JMenu(), subMenuLangFilter = new JMenu(), subMenuLangFolding = new JMenu(), subMenuLangSummarize = new JMenu(), subMenuLangIconSets = new JMenu(), subMenuLangStylesheet = new JMenu(); // todo

		public JMenu popupSubMenuLangHighlight, popupSubMenuLangFilter, popupSubMenuLangFolding, popupSubMenuLangSummarize, popupSubMenuLangIconSets, popupSubMenuLangStylesheet;
		public final String langNameLabelDefaultToolTip;
		private JPopupMenu popupMenuXMLSource, popupMenuHTMLSource;

		JMenu menuXMLAttributeChangeTo;

		public DeltaSide(int sideIndex) {
			super(new BorderLayout());
			this.sideIndex = sideIndex;
			sideController = MainController.deltaSideController[sideIndex];

			String name = SIDE_NAMES[sideIndex] + " View";
			toolBarMain = new SmallToolBar(name);

			langNameLabel = new JLabel("");
			langNameLabel.setHorizontalAlignment(SwingConstants.LEFT);
		//	langNameLabel.setBorder(new CompoundBorder(LineBorder.createGrayLineBorder(), new EmptyBorder(0, 2, 0, 2)));
			langNameLabelDefaultToolTip = "<html>Current XML Language of <i>" + SIDE_NAMES[sideIndex] + "</i> side";
			langNameLabel.setToolTipText(langNameLabelDefaultToolTip);
			toolBarMain.add(langNameLabel);

			toolBarMain.add(Box.createHorizontalStrut(10));

			fileNameLabel = new JLabel();
			fileNameLabel.setFont(SAVED_FILENAME_FONT);
			toolBarMain.add(fileNameLabel);
			filterStateLabel = new JLabel();
			filterStateLabel.setFont(UNSAVED_FILENAME_FONT);
			toolBarMain.add(Box.createHorizontalStrut(5));
			toolBarMain.add(filterStateLabel);

			toolBarMain.add(Box.createGlue());
			this.add(toolBarMain, BorderLayout.NORTH);

			// create XML components
			xmlAttributesTable = new XMLAttributesTable(preferences, sideController.dtdHelper, true);
			xmlView = new XMLView(sideIndex, preferences, xmlAttributesTable, sideController.dtdHelper);
            xmlAttributesTable.setXmlView(xmlView);
			deltaView = xmlView; // to avoid a null-deltaView
			
			xpathBarText = new AutoCompleteComboBox(MainController.getSharedXPathList());
			xpathBarText.setAutoAddValue(true);
			xpathBarText.setRestrictValues(false);
			xpathBarText.setIgnoreCase(false); // case-sensitive

			xpathBarText.setToolTipText("XPath expression pointing to the current XML node");
			xpathBarText.setPreferredSize(new Dimension(xpathBarText.getWidth(), 22));
			xpathBar = new SmallToolBar("XPath: ", true); // new JPanel(new BorderLayout(2, 0));
		// JLabel xpathLabel = new JLabel(" XPath: ");
		// xpathLabel.setFont(SAVED_FILENAME_FONT);
		// xpathBar.add(xpathLabel, BorderLayout.WEST);
			xpathBar.add(xpathBarText);
			xpathBarText.setFont(SAVED_FILENAME_FONT);

			// create XML Attributes Panel
			JPanel panelAttr = new JPanel(new BorderLayout());
			panelAttr.setBackground(GlobalObjects.DEFAULT_BACKGROUND);
			toolBarAttr = new SmallToolBar("XML-Attributes");
			newAttributesComboBoxModel = new SimpleComboBoxModel();
			newAttributesComboBox = new JComboBox(newAttributesComboBoxModel);
			newAttributesComboBox.setEnabled(false);
			newAttributesComboBox.setToolTipText("Choose new attribute to add ...");
			newAttributesComboBox.setPreferredSize(new Dimension(31, 16));
			panelAttr.add(toolBarAttr, BorderLayout.SOUTH);

			JScrollPane scroll = new JScrollPane(xmlAttributesTable);
			scroll.setBackground(GlobalObjects.DEFAULT_BACKGROUND);
			panelAttr.add(scroll, BorderLayout.CENTER);

			// create XML Node Info TextArea
			xmlNodeInfoText = new JTextArea();
			xmlNodeInfoText.setEditable(false);
			xmlNodeInfoText.setLineWrap(true);
			xmlNodeInfoText.setWrapStyleWord(true);
			xmlNodeInfoText.setBackground(GlobalObjects.DEFAULT_BACKGROUND);

			// create CardPanel with "XML Node Info TextArea" and "XML Attributes Panel"
			xmlInfoPanel = new CardPanel();
			xmlInfoPanel.add(new JScrollPane(xmlNodeInfoText), CARD_XML_INFO);
			xmlInfoPanel.add(panelAttr, CARD_XML_ATTRS);

			// create "wrapper"-panel with ToolBar and xmlInfoPanel
			JPanel panelXMLNode = new JPanel(new BorderLayout());
			toolBarXMLNode = new SmallToolBar("XML Node");
			panelXMLNode.add(toolBarXMLNode, BorderLayout.NORTH);
			panelXMLNode.add(xmlInfoPanel, BorderLayout.CENTER);

			xmlPanel = new JPanel(new BorderLayout());
			JScrollPane scrollPaneXMLView = new JScrollPane(xmlView);
			scrollPaneXMLView.setBorder(null);
			xmlPanel.add(scrollPaneXMLView, BorderLayout.CENTER);

			xmlSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT, splitpanesContinuous, xmlPanel, panelXMLNode);
			xmlSplit.setOneTouchExpandable(true);
			xmlSplit.setResizeWeight(0.75);

			htmlOutput = new SimpleHTMLView();
			textOutput = new JTextArea();
			textOutput.setBackground(GlobalObjects.DEFAULT_BACKGROUND);
			textOutput.setEditable(false);
			htmlView = new HTMLView(preferences);

			mainPanel = new CardPanel();
			mainPanel.add(new JScrollPane(textOutput), CARD_SOURCE_PLAIN);
			mainPanel.add(new JScrollPane(htmlOutput), CARD_SIMPLE_HTML);
			mainPanel.add(xmlSplit, CARD_XML);
			JScrollPane scrollPaneHTMLView = new JScrollPane(htmlView);
			scrollPaneHTMLView.setBorder(null);
			mainPanel.add(scrollPaneHTMLView, CARD_HTML);
			this.add(mainPanel, BorderLayout.CENTER);

			fileChooser = new DeltaFileChooser(FileUtils.currentDirFile);
			setBorder(sideBorderInactive);
		}

		public XPathExpressionEditDialog xpathEditDialog;
		void fillToolBarsAndPopupMenus() {
			xpathBar.add(new AbstractInputDialog.SmallButton(sideController.aXMLEvaluateXPathExpression));
			Action xpathEditAction = new SimpleAction(MainImages.EDIT_ICON, "Show XPath edit dialog", KeyStroke.getKeyStroke(KeyEvent.VK_F2, 0)) {
				protected void simpleActionPerformed() {
					if (xpathEditDialog == null)
						xpathEditDialog = new XPathExpressionEditDialog(DeltaSide.this);

					xpathEditDialog.init(sideController.dtdHelper);

					String xpath = xpathEditDialog.showEditDialog(xpathBarText.getText());
					if (xpath != null)
						xpathBarText.setText(xpath);
					xpathBarText.requestFocus();
				}
			};
			xpathBar.add(new AbstractInputDialog.SmallButton(xpathEditAction));
			Action xpathClearAction = new SimpleAction(MainImages.CLEAR_ICON, "Clear XPath field",
													  KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0)) {
				protected void simpleActionPerformed() {
					xpathBarText.setText("");
					xpathBarText.requestFocusInWindow();
				}
			};
			xpathBar.add(new AbstractInputDialog.SmallButton(xpathClearAction));
			Action xpathRemoveBarAction = new SimpleAction(MainImages.CLOSE_ICON, "Remove XPath bar", KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK)) {
				protected void simpleActionPerformed() {
					sideController.aXMLShowXPathBar.setSelected(false);
				}
			};
			xpathBar.add(new AbstractInputDialog.SmallButton(xpathRemoveBarAction));

		// xpathBar.add(xpathButtonPanel, BorderLayout.EAST);
			xpathBarText.addActionListener(sideController.aXMLEvaluateXPathExpression);

			ActionMap xpathBarTextActionMap = xpathBarText.getActionMap();
			InputMap xpathBarTextInputMap = xpathBarText.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
			SwingUtils.addShortCut(xpathBarTextActionMap, xpathBarTextInputMap, xpathEditAction);
			SwingUtils.addShortCut(xpathBarTextActionMap, xpathBarTextInputMap, xpathClearAction);
			SwingUtils.addShortCut(xpathBarTextActionMap, xpathBarTextInputMap, xpathRemoveBarAction);
			statusBar.attachToToolBar(xpathBar);

			toolBarMain.add(sideController.aXMLNewEmptyFile);
			jButtonNewFromTemplateOrOtherLang = toolBarMain.add(MainController.aTemplates);
			jButtonNewFromTemplateOrOtherLang.setEnabled(false);
			jButtonNewFromTemplateOrOtherLang.setIcon(MainImages.DOWN_ICON);
			jButtonNewFromTemplateOrOtherLang.setPreferredSize(DOWN_BUTTON_SIZE);
			toolBarMain.add(Box.createHorizontalStrut(4));
			toolBarMain.add(sideController.aOpen);
			jButtonReopen = toolBarMain.add(sideController.aReopen);
			jButtonReopen.setIcon(MainImages.DOWN_ICON);
			jButtonReopen.setPreferredSize(DOWN_BUTTON_SIZE);
			toolBarMain.add(Box.createHorizontalStrut(2));
			toolBarMain.add(sideController.aReload);
			toolBarMain.add(Box.createHorizontalStrut(2));
			toolBarMain.add(sideController.aSave);
			toolBarMain.add(Box.createHorizontalStrut(2));
			toolBarMain.add(sideController.aSaveAs);
			toolBarMain.add(Box.createHorizontalStrut(4));
			toolBarMain.add(sideController.aGoBack);
			toolBarMain.add(sideController.aGoForward);
			statusBar.attachToToolBar(toolBarMain);

			toolBarAttr.add(new JLabel("Attribute: Insert "));
			toolBarAttr.add(newAttributesComboBox);
			toolBarAttr.add(sideController.aXMLAttributeAdd);
			toolBarAttr.add(Box.createHorizontalStrut(20));//createGlue());
			toolBarAttr.add(sideController.aXMLAttributeRemove);
			statusBar.attachToToolBar(toolBarAttr);
			toolBarXMLNode.add(new JLabel("XML Node: "));
			xmlNodeLabel = new JLabel();
			xmlNodeLabel.setFont(GlobalObjects.BOLD_ITALIC_FIXED_FONT);
			toolBarXMLNode.add(xmlNodeLabel);
			toolBarXMLNode.add(Box.createGlue());
			toolBarXMLNode.add(sideController.aXMLRemoveNode);
			statusBar.attachToToolBar(toolBarXMLNode);

// XML - PopupMenu
			JPopupMenu popupMenuXML = new JPopupMenu(SIDE_NAMES[sideIndex] + " side");
			JMenu subMenu = new JMenu("Edit");
			subMenu.setIcon(MainImages.EDIT_ICON);
			subMenu.setMnemonic(KeyEvent.VK_E);
			subMenu.add(MainController.aEditUndo);
			subMenu.add(MainController.aEditRedo);
			subMenu.addSeparator();
			subMenu.add(MainController.aEditCut);
			subMenu.add(MainController.aEditCopy);

			JMenu subMenuEdit = new JMenu(pasteSubMenuAction);
			subMenuEdit.add(MainController.aEditPasteInto);
			subMenuEdit.addSeparator();
			subMenuEdit.add(MainController.aEditPasteBefore);
			subMenuEdit.add(MainController.aEditPasteAfter);
			subMenuEdit.addSeparator();
			subMenuEdit.add(MainController.aEditPasteReplace);
			subMenu.add(subMenuEdit);
			popupMenuXML.add(subMenu);

			subMenu = new JMenu("Navigation");
			subMenu.setIcon(MainImages.TREE_VIEW_ICON);
			subMenu.setMnemonic(KeyEvent.VK_N);
			subMenu.add(sideController.aGoBack);
			subMenu.add(sideController.aGoForward);
			subMenu.addSeparator();
			subMenu.add(sideController.aXMLTreeGotoParent);
			subMenu.addSeparator();
			subMenu.add(sideController.aXMLTreeGotoNextSibling);
			subMenu.add(sideController.aXMLTreeGotoPreviousSibling);
			subMenu.addSeparator();
			subMenu.add(sideController.aXMLJumpToLineNumber);
			popupMenuXML.add(subMenu);

			subMenu = new JMenu("View");
			subMenu.setIcon(MainImages.VIEW_ICON);
			subMenu.setMnemonic(KeyEvent.VK_W);

			JMenu subMenuViews = new JMenu("Change View");
			subMenuViews.setIcon(MainImages.VIEW_ICON);
			subMenuViews.setBorder(INTENTED_BORDER);
			subMenuViews.setMnemonic(KeyEvent.VK_C);
			ButtonGroup buttonGroup = new ButtonGroup();
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToViewer)));
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToHTMLOutput)));
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToTextOutput)));
			subMenuViews.addSeparator();
			subMenuViews.add(sideController.aMaximzeViewSize).setBorder(INTENTED_BORDER);
			subMenuViews.add(sideController.aRestoreViewSize).setBorder(INTENTED_BORDER);
			subMenu.add(subMenuViews);
			subMenu.addSeparator();

			JMenu subMenuVis = new JMenu("DELT/A-Links Visualization");
			subMenuVis.setIcon(MainImages.LINKS_VIS_ICON);
			subMenuVis.setBorder(INTENTED_BORDER);
			subMenuVis.setMnemonic(KeyEvent.VK_V);
			subMenuVis.add(new SelectableCheckBoxMenuItem(sideController.aXMLDELTALinksVisualizationUp));
			subMenuVis.add(new SelectableCheckBoxMenuItem(sideController.aXMLDELTALinksVisualizationDown));
			subMenu.add(subMenuVis);

			subMenu.addSeparator();
			subMenu.add(sideController.aShowFormattedSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aShowPlainSource).setBorder(INTENTED_BORDER);
			subMenu.addSeparator();
			subMenu.add(sideController.aXMLShowFormattedDTDSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aXMLShowPlainDTDSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aXMLShowDTDReport).setBorder(INTENTED_BORDER);

			if (MainController.ADVANCED_MODE) {
				subMenu.addSeparator();
				subMenu.add(sideController.aXMLSetRootTreeNode).setBorder(INTENTED_BORDER);
			}
			subMenu.addSeparator();
			subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowTreePathBar));
			if (MainController.ADVANCED_MODE) {
				subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowXPathBar));
				subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowAutoXPathBar));
			}

			popupMenuXML.add(subMenu);

			if (MainController.ADVANCED_MODE) {
				subMenu = new JMenu("Applicable \"Views\"");
				subMenu.setIcon(MainImages.APPLICABLES_ICON);
				subMenu.setMnemonic(KeyEvent.VK_A);

				JMenu subMenuHighlight = new JMenu("Highlight");
				subMenuHighlight.setIcon(MainImages.HIGHLIGHT_ICON);
				subMenuHighlight.setBorder(INTENTED_BORDER);
				subMenuHighlight.setMnemonic(KeyEvent.VK_H);
				subMenuHighlight.add(sideController.aHighlightAllLinks);
				subMenuHighlight.add(sideController.aHighlightDeadLinks);
				subMenuHighlight.add(sideController.aHighlightSameLinks);
				subMenuHighlight.addSeparator();
				subMenuHighlight.add(sideController.aXMLHighlightXPath);
				subMenuHighlight.add(sideController.aXMLHighlightXQuery);
				subMenuHighlight.addSeparator();
				popupSubMenuLangHighlight = new JMenu("XML-Language pre-defined");
				popupSubMenuLangHighlight.setIcon(MainImages.XML_LANGUAGE_ICON);
				popupSubMenuLangHighlight.setMnemonic(KeyEvent.VK_X);
				popupSubMenuLangHighlight.setEnabled(false);
				subMenuHighlight.add(popupSubMenuLangHighlight);
				subMenu.add(subMenuHighlight);

				JMenu subMenuFilter = new JMenu("Filter");
				subMenuFilter.setIcon(MainImages.FILTER_ICON);
				subMenuFilter.setBorder(INTENTED_BORDER);
				subMenuFilter.setMnemonic(KeyEvent.VK_F);
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowDocumentNode));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowCommentNodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowTextNodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowCDATANodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowPINodes));
				subMenuFilter.addSeparator();
				subMenuFilter.add(sideController.aXMLTreeShowXPath).setBorder(INTENTED_BORDER);
				subMenuFilter.add(sideController.aXMLTreeHideXPath).setBorder(INTENTED_BORDER);
				subMenuFilter.addSeparator();
				subMenuFilter.add(sideController.aXMLTreeShowAllTreeNodes).setBorder(INTENTED_BORDER);
				subMenuFilter.addSeparator();
				popupSubMenuLangFilter = new JMenu("XML-Language pre-defined");
				popupSubMenuLangFilter.setIcon(MainImages.XML_LANGUAGE_ICON);
				popupSubMenuLangFilter.setBorder(INTENTED_BORDER);
				popupSubMenuLangFilter.setMnemonic(KeyEvent.VK_X);
				popupSubMenuLangFilter.setEnabled(false);
				subMenuFilter.add(popupSubMenuLangFilter);
				subMenu.add(subMenuFilter);

				JMenu subMenuFolding = new JMenu("Folding");
				subMenuFolding.setIcon(XMLImages.OPENED_ROOT_NODE_ICON);
				subMenuFolding.setBorder(INTENTED_BORDER);
				subMenuFolding.setMnemonic(KeyEvent.VK_O);
				subMenuFolding.add(sideController.aXMLTreeExpandAll);
				subMenuFolding.add(sideController.aXMLTreeCollapseAll);
				subMenuFolding.addSeparator();
				subMenuFolding.add(sideController.aXMLTreeExpandAllDescendants);
				subMenuFolding.add(sideController.aXMLTreeCollapseAllDescendants);
				subMenuFolding.addSeparator();
				subMenuFolding.add(sideController.aXMLTreeExpandAllXPath);
				subMenuFolding.add(sideController.aXMLTreeCollapseAllXPath);
				subMenuFolding.addSeparator();
				popupSubMenuLangFolding = new JMenu("XML-Language pre-defined");
				popupSubMenuLangFolding.setIcon(MainImages.XML_LANGUAGE_ICON);
				popupSubMenuLangFolding.setMnemonic(KeyEvent.VK_X);
				popupSubMenuLangFolding.setEnabled(false);
				subMenuFolding.add(popupSubMenuLangFolding);
				subMenu.add(subMenuFolding);

				JMenu subMenuSummarize = new JMenu("Summarize");
				subMenuSummarize.setIcon(MainImages.VIEW_ICON);
				subMenuSummarize.setBorder(INTENTED_BORDER);
				subMenuSummarize.setMnemonic(KeyEvent.VK_E);
				subMenuSummarize.add(sideController.aXMLTreeApplySummarizeXSL);
				subMenuSummarize.add(sideController.aXMLTreeUnloadSummarizeXSL);
				subMenuSummarize.addSeparator();
				popupSubMenuLangSummarize = new JMenu("XML-Language pre-defined");
				popupSubMenuLangSummarize.setIcon(MainImages.XML_LANGUAGE_ICON);
				popupSubMenuLangSummarize.setMnemonic(KeyEvent.VK_X);
				popupSubMenuLangSummarize.setEnabled(false);
				subMenuSummarize.add(popupSubMenuLangSummarize);
				subMenu.add(subMenuSummarize);

				popupSubMenuLangStylesheet = new JMenu("Apply Transformation");
				popupSubMenuLangStylesheet.setToolTipText("XML-Language pre-defined XSLT Stylesheets");
				popupSubMenuLangStylesheet.setIcon(MiscImages.XSL_FILE_ICON);
				popupSubMenuLangStylesheet.setBorder(INTENTED_BORDER);
				popupSubMenuLangStylesheet.setMnemonic(KeyEvent.VK_S);
				popupSubMenuLangStylesheet.setEnabled(false);
				subMenu.add(popupSubMenuLangStylesheet);

				popupSubMenuLangIconSets = new JMenu("Icon-Sets");
				popupSubMenuLangIconSets.setIcon(MiscImages.ICON_FILE_ICON);
				popupSubMenuLangIconSets.setBorder(INTENTED_BORDER);
				popupSubMenuLangIconSets.setMnemonic(KeyEvent.VK_I);
				subMenu.add(popupSubMenuLangIconSets);
				popupMenuXML.add(subMenu);

				PopupMenuHandler popupMenuHandler = new PopupMenuHandler(popupMenuXML);
				popupMenuHandler.addPopupMenu(popupSubMenuLangHighlight.getPopupMenu(), PopupMenuHandler.CTRL_MASK);
				popupMenuHandler.addPopupMenu(popupSubMenuLangFilter.getPopupMenu(), PopupMenuHandler.ALT_MASK);
				popupMenuHandler.addPopupMenu(popupSubMenuLangFolding.getPopupMenu(), PopupMenuHandler.SHIFT_MASK);
				popupMenuHandler.addPopupMenu(popupSubMenuLangSummarize.getPopupMenu(), PopupMenuHandler.CTRL_MASK | PopupMenuHandler.SHIFT_MASK);
				popupMenuHandler.addPopupMenu(popupSubMenuLangStylesheet.getPopupMenu(), PopupMenuHandler.CTRL_MASK | PopupMenuHandler.ALT_MASK);
				xmlView.addMouseListener(popupMenuHandler);
			} else
				xmlView.addMouseListener(new PopupMenuHandler(popupMenuXML));

			popupMenuXML.addSeparator();
			popupMenuXML.add(sideController.aXMLRemoveNode);
			popupMenuXML.add(sideController.aXMLDuplicateNode);
			if (MainController.ADVANCED_MODE) {
				popupMenuXML.add(sideController.aXMLAddRequiredContent);
				popupMenuXML.add(sideController.aXMLChangeTextValue);
			}
			popupMenuXML.addSeparator();
			popupMenuXML.add(sideController.aInsertNewDELTALink);
			if (MainController.ADVANCED_MODE)
				popupMenuXML.add(sideController.aSynchronizeLinks);
			popupMenuXML.add(sideController.aActivateLinks);
			if (MainController.ADVANCED_MODE) {
				popupMenuXML.addSeparator();
				popupMenuXML.add(sideController.aXMLCommentElement);
				popupMenuXML.add(sideController.aXMLValidateSubTree);
				popupMenuXML.addSeparator();
				popupMenuXML.add(sideController.aXMLGoToElementDefinitionInDTDReport);
				popupMenuXML.add(sideController.aXMLGoToElementDefinitionInDTDSource);
			}
			statusBar.attachToMenuElement(popupMenuXML);

			popupMenuXMLNodeTextArea = new JPopupMenu("XML-Node");
			popupMenuXMLNodeTextArea.add(sideController.aXMLChangeTextValue);
			popupMenuXMLNodeTextArea.setEnabled(false);
			statusBar.attachToMenuElement(popupMenuXMLNodeTextArea);
			xmlNodeInfoText.addMouseListener(new PopupMenuHandler(popupMenuXMLNodeTextArea));

			JPopupMenu xmlAttributesTablePopupMenu = new JPopupMenu("XML-Attributes");
			xmlAttributesTablePopupMenu.add(sideController.aXMLAttributeAdd);
			xmlAttributesTablePopupMenu.add(sideController.aXMLAttributeRemove);
			menuXMLAttributeChangeTo = new JMenu("Change Attribute to ...");
			menuXMLAttributeChangeTo.setIcon(MainImages.CONVERT_ICON); 
			menuXMLAttributeChangeTo.setMnemonic(KeyEvent.VK_C);
			xmlAttributesTablePopupMenu.add(menuXMLAttributeChangeTo);
			statusBar.attachToMenuElement(xmlAttributesTablePopupMenu);
			xmlAttributesTable.addMouseListener(new PopupMenuHandler(xmlAttributesTablePopupMenu));

// HTML Popup-Menu
			JPopupMenu popupMenuHTML = new JPopupMenu(SIDE_NAMES[sideIndex] + " side");
			if (MainController.ADVANCED_MODE) {
				subMenuViews = new JMenu("View");
				subMenuViews.setIcon(MainImages.VIEW_ICON);
				subMenuViews.setMnemonic(KeyEvent.VK_V);
				buttonGroup = new ButtonGroup();
				buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToViewer)));
				buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToHTMLOutput)));
				buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToTextOutput)));
				subMenuViews.addSeparator();
				subMenuViews.add(sideController.aShowFormattedSource).setBorder(INTENTED_BORDER);
				subMenuViews.add(sideController.aShowPlainSource).setBorder(INTENTED_BORDER);
				subMenuViews.addSeparator();
				subMenuViews.add(sideController.aMaximzeViewSize).setBorder(INTENTED_BORDER);
				subMenuViews.add(sideController.aRestoreViewSize).setBorder(INTENTED_BORDER);
				popupMenuHTML.add(subMenuViews);
				popupMenuHTML.addSeparator();
			}
			popupMenuHTML.add(sideController.aInsertNewDELTALink);

			/*JMenu subMenuInsertLinkHTML = new JMenu("... if selection contains Link, ..."); // todo
			ButtonGroup buttonGroupInsertLinkHTML = new ButtonGroup();
			buttonGroupInsertLinkHTML.add(subMenuInsertLinkHTML.add(new SelectableRadioButtonMenuItem(sideController.aHTMLCreateOverlappingLinksMode)));
			buttonGroupInsertLinkHTML.add(subMenuInsertLinkHTML.add(new SelectableRadioButtonMenuItem(sideController.aHTMLProvideLinkIDForOtherSideMode)));
			popupMenuHTML.add(subMenuInsertLinkHTML);*/

			popupMenuHTML.add(sideController.aSynchronizeLinks);
			popupMenuHTML.add(sideController.aHTMLRemoveLinks);
			popupMenuHTML.addSeparator();
			popupMenuHTML.add(sideController.aActivateLinks);
			popupMenuHTML.add(sideController.aHighlightAllLinks);
			popupMenuHTML.add(sideController.aHighlightDeadLinks);
			popupMenuHTML.add(sideController.aHighlightSameLinks);
			popupMenuHTML.addSeparator();
			popupMenuHTML.add(sideController.aHTMLShowInBrowser);
			statusBar.attachToMenuElement(popupMenuHTML);
			htmlView.addMouseListener(new PopupMenuHandler(popupMenuHTML));

			if (MainController.ADVANCED_MODE) {
// XML source menu
				popupMenuXMLSource = new JPopupMenu(SIDE_NAMES[sideIndex] + " side");
				buttonGroup = new ButtonGroup();
				buttonGroup.add(popupMenuXMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToViewer)));
				buttonGroup.add(popupMenuXMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToHTMLOutput)));
				buttonGroup.add(popupMenuXMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToTextOutput)));
//				popupMenuXMLSource.addSeparator();
//				popupMenuXMLSource.add(sideController.aShowFormattedSource).setBorder(INTENTED_BORDER);
//				popupMenuXMLSource.add(sideController.aShowPlainSource).setBorder(INTENTED_BORDER);
//				popupMenuXMLSource.addSeparator();
//				popupMenuXMLSource.add(sideController.aXMLShowFormattedDTDSource).setBorder(INTENTED_BORDER);
//				popupMenuXMLSource.add(sideController.aXMLShowPlainDTDSource).setBorder(INTENTED_BORDER);
//				popupMenuXMLSource.add(sideController.aXMLShowDTDReport).setBorder(INTENTED_BORDER);
				popupMenuXMLSource.addSeparator();
				popupMenuXMLSource.add(sideController.aSourceToBrowser).setBorder(INTENTED_BORDER);
				popupMenuXMLSource.add(sideController.aSourceExport).setBorder(INTENTED_BORDER);
				popupMenuXMLSource.addSeparator();
				popupMenuXMLSource.add(sideController.aMaximzeViewSize).setBorder(INTENTED_BORDER);
				popupMenuXMLSource.add(sideController.aRestoreViewSize).setBorder(INTENTED_BORDER);

				// HTML source menu
				popupMenuHTMLSource = new JPopupMenu(SIDE_NAMES[sideIndex] + " side");
				buttonGroup = new ButtonGroup();
				buttonGroup.add(popupMenuHTMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToViewer)));
				buttonGroup.add(popupMenuHTMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToHTMLOutput)));
				buttonGroup.add(popupMenuHTMLSource.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToTextOutput)));
				popupMenuHTMLSource.addSeparator();
				popupMenuHTMLSource.add(sideController.aSourceToBrowser).setBorder(INTENTED_BORDER);
				popupMenuHTMLSource.add(sideController.aSourceExport).setBorder(INTENTED_BORDER);
				popupMenuHTMLSource.addSeparator();
				popupMenuHTMLSource.add(sideController.aMaximzeViewSize).setBorder(INTENTED_BORDER);
				popupMenuHTMLSource.add(sideController.aRestoreViewSize).setBorder(INTENTED_BORDER);

				statusBar.attachToMenuElement(popupMenuXMLSource);
				MouseListener popupListener = new PopupMenuHandler(popupMenuXMLSource);
				textOutput.addMouseListener(popupListener);
				htmlOutput.addMouseListener(popupListener);

				statusBar.attachToMenuElement(popupMenuHTMLSource);
				popupListener = new PopupMenuHandler(popupMenuHTMLSource);
				textOutput.addMouseListener(popupListener);
				htmlOutput.addMouseListener(popupListener);

				popupMenuHTMLSource.setEnabled(false);
				popupMenuXMLSource.setEnabled(false);
			}

			jPopupMenuReopen = new JPopupMenu();
			jPopupMenuNewFromTemplateOrOtherLang = new JPopupMenu();
			langNameLabel.addMouseListener(sideController.focusChangeMouseListener);
			addFocusChangeMouseListener(this);
		}

		private void addFocusChangeMouseListener(Container container) {
			Component[] components = container.getComponents();
			for (int i = 0; i < components.length; i++) {
				Component comp = components[i];
				comp.addMouseListener(sideController.focusChangeMouseListener);
				if (comp instanceof Container)
					addFocusChangeMouseListener((Container) comp);
			}
		}

		private void addFocusChangeMouseListener(JMenu menu) {
			Component[] components = menu.getMenuComponents();
			for (int i = 0; i < components.length; i++) {
				Component comp = components[i];
				comp.addMouseListener(sideController.focusChangeMouseListener);
				if (comp instanceof JMenu)
					addFocusChangeMouseListener((JMenu) comp);
			}
		}

		JMenu createMenu() {
			JMenu menu = new JMenu(sideController.sideName + " File");
			menu.setMnemonic(sideController.sideName.charAt(0));
			menu.add(sideController.aXMLNewEmptyFile);
			jMenuTemplates = new JMenu(MainController.aTemplates);
			jMenuTemplates.setEnabled(false);
			menu.add(jMenuTemplates);
			jMenuNewFromOtherLang = new JMenu(MainController.aNewFromOtherLang);
			menu.add(jMenuNewFromOtherLang);
			menu.addSeparator();
			menu.add(sideController.aOpen);

			jMenuReopen = new JMenu(sideController.aReopen);
			menu.add(jMenuReopen);
			jMenuOpenInDir = new JMenu(MainController.aOpenInMenu);
			menu.add(jMenuOpenInDir);

			menu.add(sideController.aReload);
			menu.add(sideController.aRevert);
			menu.add(sideController.aClose);
			menu.addSeparator();
			menu.add(sideController.aSave);
			JMenuItem mi = menu.add(sideController.aSaveAs);
			mi.setDisplayedMnemonicIndex(5);
/*				menu.addSeparator();

		// XML / View submenu
			JMenu subMenu = new JMenu("View");
			subMenu.setIcon(MainImages.VIEW_ICON);
			subMenu.setMnemonic(KeyEvent.VK_W);
			JMenu subMenuViews = new JMenu("Change View");
			subMenuViews.setIcon(MainImages.VIEW_ICON);
			subMenuViews.setBorder(INTENTED_BORDER);
			subMenuViews.setMnemonic(KeyEvent.VK_C);
			ButtonGroup buttonGroup = new ButtonGroup();
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToViewer)));
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToHTMLOutput)));
			buttonGroup.add(subMenuViews.add(new SelectableRadioButtonMenuItem(sideController.aSwitchToTextOutput)));
			subMenuViews.addSeparator();

			subMenuViews.add(sideController.aMaximzeViewSize).setBorder(INTENTED_BORDER);
			subMenuViews.add(sideController.aRestoreViewSize).setBorder(INTENTED_BORDER);
			subMenu.add(subMenuViews);

			subMenu.addSeparator();
			subMenu.add(sideController.aShowFormattedSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aShowPlainSource).setBorder(INTENTED_BORDER);
			subMenu.addSeparator();
			subMenu.add(new MenuLabel("XML", 26));
			subMenu.add(sideController.aXMLShowFormattedDTDSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aXMLShowPlainDTDSource).setBorder(INTENTED_BORDER);
			subMenu.add(sideController.aXMLShowDTDReport).setBorder(INTENTED_BORDER);

			subMenu.addSeparator();

			JMenu subMenuVis = new JMenu("DELT/A-Links Visualization");
			subMenuVis.setIcon(MainImages.LINKS_VIS_ICON);
			subMenuVis.setBorder(INTENTED_BORDER);
			subMenuVis.setMnemonic(KeyEvent.VK_V);
			subMenuVis.add(new SelectableCheckBoxMenuItem(sideController.aXMLDELTALinksVisualizationUp));
			subMenuVis.add(new SelectableCheckBoxMenuItem(sideController.aXMLDELTALinksVisualizationDown));
			subMenu.add(subMenuVis);
			subMenu.addSeparator();
			subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowXPathBar));
			subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowTreePathBar));
			subMenu.add(new SelectableCheckBoxMenuItem(sideController.aXMLShowAutoXPathBar));

			menu.add(subMenu);*/
/*			menu.addSeparator();

			menu.add(sideController.aInsertNewDELTALink);
			if (!GlobalConstants.DELTA_LIGHT)
				menu.add(sideController.aSynchronizeLinks);
			menu.add(sideController.aActivateLinks);
			menu.addSeparator();
			menu.add(new MenuLabel("XML", 26));

			if (!GlobalConstants.DELTA_LIGHT) {
				subMenu = new JMenu("Applicable \"Views\"");
				subMenu.setIcon(MainImages.APPLICABLES_ICON);
				subMenu.setMnemonic(KeyEvent.VK_A);

				JMenu subMenuHighlight = new JMenu("Highlight");
				subMenuHighlight.setIcon(MainImages.HIGHLIGHT_ICON);
				subMenuHighlight.setBorder(INTENTED_BORDER);
				subMenuHighlight.setMnemonic(KeyEvent.VK_H);
				subMenuHighlight.add(sideController.aHighlightAllLinks);
				subMenuHighlight.add(sideController.aHighlightDeadLinks);
				subMenuHighlight.add(sideController.aHighlightSameLinks);
				subMenuHighlight.addSeparator();
				subMenuHighlight.add(sideController.aXMLHighlightXPath);
				subMenuHighlight.add(sideController.aXMLHighlightXQuery);
				subMenuHighlight.addSeparator();
				subMenuLangHighlight = new JMenu(new EmptyAction("XML-Language pre-defined", MainImages.XML_LANGUAGE_ICON, 'X', false));
				subMenuHighlight.add(subMenuLangHighlight);
				subMenu.add(subMenuHighlight);

				JMenu subMenuFilter = new JMenu("Filter");
				subMenuFilter.setIcon(MainImages.FILTER_ICON);
				subMenuFilter.setBorder(INTENTED_BORDER);
				subMenuFilter.setMnemonic(KeyEvent.VK_F);
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowDocumentNode));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowCommentNodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowTextNodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowCDATANodes));
				subMenuFilter.add(new SelectableCheckBoxMenuItem(sideController.aXMLTreeShowPINodes));
				subMenuFilter.addSeparator();
				subMenuFilter.add(sideController.aXMLTreeShowXPath).setBorder(INTENTED_BORDER);
				subMenuFilter.add(sideController.aXMLTreeHideXPath).setBorder(INTENTED_BORDER);
				subMenuFilter.addSeparator();
				subMenuFilter.add(sideController.aXMLTreeShowAllTreeNodes).setBorder(INTENTED_BORDER);
				subMenuFilter.addSeparator();
				subMenuLangFilter = new JMenu("XML-Language pre-defined");
				subMenuLangFilter.setIcon(MainImages.XML_LANGUAGE_ICON);
				subMenuLangFilter.setBorder(INTENTED_BORDER);
				subMenuLangFilter.setMnemonic(KeyEvent.VK_X);
				subMenuLangFilter.setEnabled(false);
				subMenuFilter.add(subMenuLangFilter);
				subMenu.add(subMenuFilter);

				JMenu subMenuFolding = new JMenu("Folding");
				subMenuFolding.setIcon(XMLImages.OPENED_ROOT_NODE_ICON);
				subMenuFolding.setBorder(INTENTED_BORDER);
				subMenuFolding.setMnemonic(KeyEvent.VK_O);
				subMenuFolding.add(sideController.aXMLTreeExpandAll);
				subMenuFolding.add(sideController.aXMLTreeCollapseAll);
				subMenuFolding.addSeparator();
				subMenuFolding.add(sideController.aXMLTreeExpandAllDescendants);
				subMenuFolding.add(sideController.aXMLTreeCollapseAllDescendants);
				subMenuFolding.addSeparator();
				subMenuFolding.add(sideController.aXMLTreeExpandAllXPath);
				subMenuFolding.add(sideController.aXMLTreeCollapseAllXPath);
				subMenuFolding.addSeparator();
				subMenuLangFolding = new JMenu("XML-Language pre-defined");
				subMenuLangFolding.setIcon(MainImages.XML_LANGUAGE_ICON);
				subMenuLangFolding.setMnemonic(KeyEvent.VK_X);
				subMenuLangFolding.setEnabled(false);
				subMenuFolding.add(subMenuLangFolding);
				subMenu.add(subMenuFolding);

				JMenu subMenuSummarize = new JMenu("Info-Summarize");
				subMenuSummarize.setIcon(MainImages.VIEW_ICON);
				subMenuSummarize.setBorder(INTENTED_BORDER);
				subMenuSummarize.setMnemonic(KeyEvent.VK_E);
				subMenuSummarize.add(sideController.aXMLTreeApplySummarizeXSL);
				subMenuSummarize.add(sideController.aXMLTreeUnloadSummarizeXSL);
				subMenuSummarize.addSeparator();
				subMenuLangSummarize = new JMenu("XML-Language pre-defined");
				subMenuLangSummarize.setIcon(MainImages.XML_LANGUAGE_ICON);
				subMenuLangSummarize.setMnemonic(KeyEvent.VK_X);
				subMenuLangSummarize.setEnabled(false);
				subMenuSummarize.add(subMenuLangSummarize);
				subMenu.add(subMenuSummarize);

				subMenuLangStylesheet = new JMenu("Apply Transformation");
				subMenuLangStylesheet.setToolTipText("XML-Language pre-defined XSLT Stylesheets");
				subMenuLangStylesheet.setIcon(MiscImages.XSL_FILE_ICON);
				subMenuLangStylesheet.setBorder(INTENTED_BORDER);
				subMenuLangStylesheet.setMnemonic(KeyEvent.VK_S);
				subMenuLangStylesheet.setEnabled(false);
				subMenu.add(subMenuLangStylesheet);

				subMenuLangIconSets = new JMenu("Icon-Sets");
				subMenuLangIconSets.setIcon(MiscImages.ICON_FILE_ICON);
				subMenuLangIconSets.setBorder(INTENTED_BORDER);
				subMenuLangIconSets.setMnemonic(KeyEvent.VK_I);
				subMenu.add(subMenuLangIconSets);

				menu.add(subMenu);
			}

			menu.add(sideController.aXMLRemoveNode);
			menu.add(sideController.aXMLDuplicateNode);
			if (!GlobalConstants.DELTA_LIGHT) {
				menu.add(sideController.aXMLAddRequiredContent);
				menu.add(sideController.aXMLChangeTextValue);
				menu.add(sideController.aXMLValidateSubTree);
				menu.add(sideController.aXMLSetRootTreeNode);
				menu.add(sideController.aXMLGoToElementDefinitionInDTDReport);
				menu.add(sideController.aXMLGoToElementDefinitionInDTDSource);
				menu.add(sideController.aXMLCommentElement);
			}
			menu.addSeparator();
			menu.add(new MenuLabel("HTML", 26));
			menu.add(sideController.aHTMLRemoveLinks);*/
			addFocusChangeMouseListener(menu);
			return menu;
		}

		// little helper methods
		public void switchToViewer() {
			if (sideController.isTypeXML())
				switchToXML();
			else
				switchToHTML();
		}

		void switchToXML() {
			deltaView = xmlView;
			mainPanel.switchToCard(CARD_XML);
			xmlView.requestFocusInWindow();
			if (MainController.ADVANCED_MODE) {
				popupMenuXMLSource.setEnabled(true);
				popupMenuHTMLSource.setEnabled(false);
			}
			visualizeFilterState(true, false);
		}

		void switchToHTML() {
			deltaView = htmlView;
			mainPanel.switchToCard(CARD_HTML);
			htmlView.requestFocusInWindow();
			if (MainController.ADVANCED_MODE) {
				popupMenuXMLSource.setEnabled(false);
				popupMenuHTMLSource.setEnabled(true);
			}
			visualizeFilterState(false, false);
			langNameLabel.setText("HTML");
			langNameLabel.setIcon(MainImages.HTML_ICON);
			langNameLabel.setToolTipText(langNameLabelDefaultToolTip);
		}

		public void switchToSimpleHTMLView() {
			mainPanel.switchToCard(CARD_SIMPLE_HTML);
			htmlOutput.requestFocusInWindow(); // to permit keyboard shortcuts
			sideController.aSourceToBrowser.setEnabled(true);
			sideController.aSourceExport.setEnabled(true);
		}

		public void switchToPlainSourceView() {
			mainPanel.switchToCard(CARD_SOURCE_PLAIN);
			textOutput.requestFocusInWindow(); // to permit keyboard shortcuts
			sideController.aSourceToBrowser.setEnabled(false);
			sideController.aSourceExport.setEnabled(false);
		}

		public void clearViews() {
			htmlOutput.setText(""); // NOT setText(null) !!!
			textOutput.setText("");
			mainPanel.switchToCard(CARD_SOURCE_PLAIN);
		// langNameLabel.setText("-");
		// langNameLabel.setIcon(null);
		// langNameLabel.setToolTipText(langNameLabelDefaultToolTip);
			visualizeFilterState(false, false);
		}

		void applyProperties() {

		}

		void visualizeActiveState(boolean active) {
	//		System.out.println(sideIndex+" "+active);
			if (active) {
				setBorder(sideBorderActive);
				deltaView.setBackground(Color.WHITE);
				if (deltaView != null) deltaView.requestFocusInWindow();
				else if (htmlOutput.isShowing()) htmlOutput.requestFocusInWindow();
				else if (textOutput.isShowing()) textOutput.requestFocusInWindow();
			} else {
				setBorder(sideBorderInactive);
				deltaView.setBackground(GlobalObjects.INACTIVE_SIDE_BACKGROUND);
				deltaView.repaint();
			}
		}

		public void visualizeFilterState(boolean show, boolean active) {
			if (show) {
				if (active) {
					filterStateLabel.setIcon(MainImages.FILTER_ICON);
					filterStateLabel.setToolTipText("Filter set!");
				} else {
					filterStateLabel.setIcon(null); //MainImages.FILTER_INACTIVE_ICON);
					filterStateLabel.setToolTipText("No filter set");
				}
			} else  {
				filterStateLabel.setIcon(null);
				filterStateLabel.setToolTipText("");
			}
		}

		public void ensureSideIsVisible() { // todo
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					if (jSplitMain.getDividerLocation() <= jSplitMain.getMinimumDividerLocation()) {
						if (jSplitMain.getLastDividerLocation() <= jSplitMain.getMinimumDividerLocation())
							jSplitMain.setDividerLocation(0.75);
						else jSplitMain.setDividerLocation(jSplitMain.getLastDividerLocation());
					//	jSplitMain.revalidate();
					}
					if (sideIndex == LEFT) {
						if (jSplitSides.getDividerLocation() <= jSplitSides.getMinimumDividerLocation())
							if (jSplitSides.getLastDividerLocation() <= jSplitSides.getMinimumDividerLocation())
								jSplitSides.setDividerLocation(0.5);
							else jSplitSides.setDividerLocation(jSplitSides.getLastDividerLocation());
					} else if (jSplitSides.getDividerLocation() >= jSplitSides.getMaximumDividerLocation())
						if (jSplitSides.getLastDividerLocation() >= jSplitSides.getMaximumDividerLocation())
							jSplitSides.setDividerLocation(0.5);
						else jSplitSides.setDividerLocation(jSplitSides.getLastDividerLocation());
					//	jSplitSides.revalidate();
					if (sideController.isTypeXML()) {
						if ((xmlSplit.getDividerLocation() <= xmlSplit.getMinimumDividerLocation()) ||
								(xmlSplit.getDividerLocation() >= xmlSplit.getMaximumDividerLocation()))
							if ((xmlSplit.getDividerLocation() <= xmlSplit.getMinimumDividerLocation()) ||
									(xmlSplit.getDividerLocation() >= xmlSplit.getMaximumDividerLocation()))
								xmlSplit.setDividerLocation(0.75);
							else xmlSplit.setDividerLocation(xmlSplit.getLastDividerLocation());
					//	xmlSplit.revalidate();
					}
				}
			});
		}
		public void hideSide() {
			jSplitSides.setDividerLocation((sideIndex == LEFT) ? 0.0 : 1.0);
			jSplitSides.revalidate();
		}

		public void maximizeView() {
			// real divider location = splitPaneUI.getDividerLocation(splitPane)
			// real maximum divider location = splitpane.getHeight() - splitpane.getDividerSize(),
			//   since splitpane.getMaximumDividerLocation() considers minimum and maximum sizes of the two components

			Insets insets = xmlSplit.getInsets();
			int dividerLocation = xmlSplit.getUI().getDividerLocation(xmlSplit);
			int dividerMaxLocation = (xmlSplit.getHeight() - xmlSplit.getDividerSize()) - insets.bottom;
			if (xmlView.isVisible() && (dividerLocation != dividerMaxLocation)) {  // (deltaView == xmlView)
				xmlSplit.setDividerLocation(dividerMaxLocation);
				xmlSplit.setLastDividerLocation(dividerLocation);
			} else {
				insets = jSplitSides.getInsets();
				dividerLocation = jSplitSides.getUI().getDividerLocation(jSplitSides);
				dividerMaxLocation = (jSplitSides.getWidth() - jSplitSides.getDividerSize()) - insets.right;
				if ((sideIndex == LEFT) && (dividerLocation != dividerMaxLocation)) {
					jSplitSides.setDividerLocation(dividerMaxLocation);
					jSplitSides.setLastDividerLocation(dividerLocation);
				} else if ((sideIndex == RIGHT) && (dividerLocation != insets.left)) {
					jSplitSides.setDividerLocation(insets.left);
					jSplitSides.setLastDividerLocation(dividerLocation);
				} else {
					insets = jSplitMain.getInsets();
					dividerLocation = jSplitMain.getUI().getDividerLocation(jSplitMain);
					dividerMaxLocation = (jSplitMain.getHeight() - jSplitMain.getDividerSize()) - insets.bottom;
					if (dividerLocation != dividerMaxLocation) {
						jSplitMain.setDividerLocation(dividerMaxLocation);
						jSplitMain.setLastDividerLocation(dividerLocation);
					}
				}
			}
		}

		public void restoreViewSize() {
			if (jSplitMain.getUI().getDividerLocation(jSplitMain)
					== (jSplitMain.getHeight() - jSplitMain.getDividerSize() - jSplitMain.getInsets().bottom))
				jSplitMain.setDividerLocation(jSplitMain.getLastDividerLocation());
			else if ((sideIndex == LEFT) && (jSplitSides.getUI().getDividerLocation(jSplitSides)
					== (jSplitSides.getWidth() - jSplitSides.getDividerSize() - jSplitSides.getInsets().right)))
				jSplitSides.setDividerLocation(jSplitSides.getLastDividerLocation());
			else if ((sideIndex == RIGHT) &&
					(jSplitSides.getUI().getDividerLocation(jSplitSides) == jSplitSides.getInsets().left))
				jSplitSides.setDividerLocation(jSplitSides.getLastDividerLocation());
			else if ((deltaView == xmlView) && (xmlSplit.getUI().getDividerLocation(xmlSplit)
					== (xmlSplit.getHeight() - xmlSplit.getDividerSize() - xmlSplit.getInsets().bottom)))
				xmlSplit.setDividerLocation(xmlSplit.getLastDividerLocation());
		}

		public void showXPathBar(boolean show) {
			if (show) {
				xmlPanel.add(xpathBar, BorderLayout.SOUTH);
				xpathBarText.requestFocus();
			} else {
				xmlPanel.remove(xpathBar);
				deltaView.requestFocusInWindow();
			}
			xmlPanel.revalidate();
		}

		public void showTreePathBar(boolean show) {
			if (treePathPanel == null) {
				treePathPanel = new JPanel(new BorderLayout(2, 0));
				JLabel treePathLabel = new JLabel(" Path to Node: ");
				treePathLabel.setFont(SAVED_FILENAME_FONT);
				treePathPanel.add(treePathLabel, BorderLayout.WEST);
				treePathPanel.add(getTreePathBar());
				Action treePathBarRemoveAction = new SimpleAction(MainImages.CLOSE_ICON, "Remove tree-path bar",
																  (KeyStroke)sideController.aXMLShowTreePathBar.getValue(SelectableAction.LOCAL_ACCELERATOR_KEY)) {
					protected void simpleActionPerformed() {
						sideController.aXMLShowTreePathBar.setSelected(false);
					}
				};
				treePathPanel.add(new AbstractInputDialog.SmallButton(treePathBarRemoveAction), BorderLayout.EAST);
			}
			if (show) {
				xmlPanel.add(treePathPanel, BorderLayout.NORTH);
				treePathPanel.revalidate();
				treePathPanel.repaint();
			} else
				xmlPanel.remove(treePathPanel);
			xmlPanel.validate();
			deltaView.requestFocusInWindow();
		}
		public TreePathBar getTreePathBar() {
			if (treePathBar == null)
				treePathBar = new TreePathBar();
			return treePathBar;
		}

        public AutoXPathBar getAutoXPathBar() {
            if (autoXPathBar == null)
                autoXPathBar = new AutoXPathBar();
            return autoXPathBar;
        }
		public void showAutoXPathBar(boolean show) {
			if (autoXPathPanel == null) {
				autoXPathPanel = new JPanel(new BorderLayout(2, 0));
				JLabel label = new JLabel(" Auto-XPath: ");
				label.setFont(SAVED_FILENAME_FONT);
				autoXPathPanel.add(label, BorderLayout.WEST);
				autoXPathPanel.add(getAutoXPathBar());
				Action treePathBarRemoveAction = new SimpleAction(MainImages.CLOSE_ICON, "Remove auto-XPath bar",
																  (KeyStroke) sideController.aXMLShowAutoXPathBar.getValue(SelectableAction.LOCAL_ACCELERATOR_KEY)) {
					protected void simpleActionPerformed() {
						sideController.aXMLShowAutoXPathBar.setSelected(false);
					}
				};
				autoXPathPanel.add(new AbstractInputDialog.SmallButton(treePathBarRemoveAction), BorderLayout.EAST);
			}
			if (show) {
				xmlPanel.add(autoXPathPanel, BorderLayout.NORTH);
				autoXPathPanel.revalidate();
				autoXPathPanel.repaint();
			} else
				xmlPanel.remove(autoXPathPanel);
			xmlPanel.validate();
			deltaView.requestFocusInWindow();
		}

		public void setXPathText(String xpath) {
			xpathBarText.setText(xpath);
		}

		public String getXPathExpression() {
            xpathBarText.storeInput();
			return xpathBarText.getText();
		}

		public void setFileNameLabel(File file) {
			setFileNameLabel(file.getName(), file.getAbsolutePath());
		}
		public void setFileNameLabel(String fileName, String filePath) {
		//	fileNameLabel.setBorder((fileName == null) ? null : MainFrame.FILENAME_BORDER);
			if (fileName != null) fileName = " " + fileName + " ";
			fileNameLabel.setText(fileName);
			fileNameLabel.setToolTipText(filePath);
			fileNameLabel.getParent().validate();//revalidate();
		}
	}


	public static final class SmallToolBar extends JToolBar {
		public SmallToolBar(String name) {
			this(name, false);
		}

		public SmallToolBar(String name, boolean showLabel) {
			super(name);
			setFloatable(toolbarsFloatable);
			setRollover(toolbarsRollover);
		//	putClientProperty("JToolBar.isRollover", Boolean.valueOf(toolbarsRollover));
			if (showLabel)
				super.add(new JLabel(name));
		}

		public Component add(Component component) {
			if (component instanceof AbstractButton) {
				AbstractButton button = (AbstractButton) component;
				if (button instanceof JToggleButton)
					button.setText("");
				button.setMargin(GlobalObjects.INSETS_ZERO);
			}
			return super.add(component);
		}
	}

	static final class MenuLabel extends JLabel {
		private final Color COLOR = new Color(-10066279); // blue

		MenuLabel(String text, final int indent) {
			super(text);
			setIcon(new Icon() {
				public int getIconHeight() {
					return MenuLabel.this.getHeight();
				}

				public int getIconWidth() {
					return indent;
				}

				public void paintIcon(Component c, Graphics g, int x, int y) {
				}
			});
			setForeground(COLOR);
		}
	}

	public static final String CARD_XML = "xml", CARD_HTML = "html",
		CARD_SIMPLE_HTML = "source-html", CARD_SOURCE_PLAIN = "source-text",
		CARD_MACROS_STRUCTURE = "macros-structure", CARD_MACROS_SEARCH = "macros-search", CARD_MACROS_INSERTABLE = "macros-insertable",
		CARD_XML_ATTRS = "table", CARD_XML_INFO = "text";

	public static final Font SAVED_FILENAME_FONT = GlobalObjects.DEFAULT_FONT; //new Font("Dialog", Font.PLAIN, 12);
	public static final Font UNSAVED_FILENAME_FONT = SAVED_FILENAME_FONT.deriveFont(Font.ITALIC);
	private static final Dimension DOWN_BUTTON_SIZE = new Dimension(16, 22);
	private static final int SIDE_BORDER_THICKNESS = 2;
	static final Border INTENTED_BORDER = new EmptyBorder(2, 12, 2, 2);

	public void swapSides() {
		DeltaSide deltaSideLeft = deltaSides[GUIConstants.LEFT];
		deltaSides[GUIConstants.LEFT] = deltaSides[GUIConstants.RIGHT];
		deltaSides[GUIConstants.RIGHT] = deltaSideLeft;
		deltaSides[LEFT].sideIndex = 1 - deltaSides[LEFT].sideIndex;
		deltaSides[RIGHT].sideIndex = 1 - deltaSides[RIGHT].sideIndex;
		int dividerLocation = jSplitSides.getDividerLocation();
		Component leftComponent = jSplitSides.getLeftComponent();
		Component rightComponent = jSplitSides.getRightComponent();
		jSplitSides.setLeftComponent(null);
		jSplitSides.setRightComponent(leftComponent);
		jSplitSides.setLeftComponent(rightComponent);
		jSplitSides.setDividerLocation(jSplitSides.getWidth() - dividerLocation);
		jSplitSides.revalidate();
	}
}

