/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 30.04.2003
 * Time: 12:03:37
 */
package at.ac.tuwien.asgaard.delta.gui.actions;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.Log;

import java.awt.event.*;

import javax.swing.*;

/**
 * Class SimpleAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public abstract class SimpleAction extends BaseAction {
    protected SimpleAction() {  }

    protected SimpleAction(String name, char mnemonicKey) {
        super(name);
        putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
    }
    protected SimpleAction(String name, Icon icon) {
        super(name, icon);
    }
	protected SimpleAction(String name, Icon icon, char mnemonicKey) {
		super(name, icon);
		putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip) {
		this(name, icon, mnemonicKey, toolTip, toolTip);
	}
	protected SimpleAction(String name, char mnemonicKey, String toolTip) {
		this(name, mnemonicKey);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
	}
	protected SimpleAction(String name, String toolTip) {
		super(name);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
	}
	protected SimpleAction(String name, String toolTip, String msgHover) {
		super(name);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, msgHover); // when mouse over - statusbar
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip, String msgHover) {
		this(name, icon, mnemonicKey);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, msgHover); // when mouse over - statusbar
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip, boolean enabled) {
		this(name, icon, mnemonicKey, toolTip, toolTip);
		this.enabled = enabled;
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip, String msgHover, boolean enabled) {
		this(name, icon, mnemonicKey, toolTip, msgHover);
		this.enabled = enabled;
	}
	protected SimpleAction(Icon icon) {
		putValue(Action.SMALL_ICON, icon);
	}
	protected SimpleAction(Icon icon, String toolTip) {
		putValue(Action.SMALL_ICON, icon);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
	}
	protected SimpleAction(Icon icon, String toolTip, boolean enabled) {
		putValue(Action.SMALL_ICON, icon);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
		this.enabled = enabled;
	}
	protected SimpleAction(Icon icon, String toolTip, KeyStroke shortcut) {
		this(icon, toolTip, toolTip, shortcut);
	}
	protected SimpleAction(Icon icon, String toolTip, String msgHover, KeyStroke shortcut) {
		this(icon, toolTip);
		putValue(StatusBar.DESCRIPTION_HOVER, msgHover);
		putValue(LOCAL_ACCELERATOR_KEY, shortcut);
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip, String msgHover, String msgActive) {
		this(name, icon, mnemonicKey);
		putValue(StatusBar.TOOL_TIP, toolTip); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, msgHover); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, msgActive);  // when clicked
	}
	protected SimpleAction(String name, Icon icon, char mnemonicKey, String toolTip, String msgHover, String msgActive, boolean enabled) {
		this(name, icon, mnemonicKey, toolTip, msgHover, msgActive);
		this.enabled = enabled;
	}

	public final void actionPerformed(ActionEvent evt) {
		if (enabled)
			try {
				MainController.mainFrame.statusBar.unsetHoverMessageFrom(this);
				MainController.mainFrame.statusBar.setActiveMessageFrom(this);
				simpleActionPerformed();
			} catch (Throwable t) {
				Log.error("Error in action: " + getValue(NAME));
				t.printStackTrace();
				JOptionPane.showMessageDialog(null, t.toString(), "Unexpected error occurred", JOptionPane.ERROR_MESSAGE);
			} finally {
				MainController.mainFrame.statusBar.unsetActiveMessageFrom(this);
			}
	}

	protected abstract void simpleActionPerformed();
	// I never need an ActionEvent...
}
