/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 30.04.2003
 * Time: 11:26:35
 */
package at.ac.tuwien.asgaard.delta.gui.actions.edit;

import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.*;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.*;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.Log;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.undo.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Set;

/**
 * Class FileSaveAllAction ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 */
public class EditPasteIntoAction extends SimpleAction {
	protected EditPasteIntoAction(String name, Icon icon, char mnemonicKey) {
		super(name, icon);
		putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
		this.enabled = false;
	}

	public EditPasteIntoAction() {
		this("Paste Into", MainImages.XML_INSERT_INTO_ICON, 'P');

		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK));
		putValue(SHORT_DESCRIPTION, "Paste Into from clipboard"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Paste clipboard content into current element."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Pasting from clipboard ...");  // when clicked
	}

	protected void simpleActionPerformed() {
		Log.trace("enter: EditPasteAction.simpleActionPerformed()");

		int activeSideIndex = MainController.getActiveSideIndex();
		boolean cut = (MainController.xmlClipBoard.clipBoardMode == MainController.XMLClipBoard.ClipBoardMode.CUT);

		CompoundEdit wrapperEdit = new PasteWrapperUndoableEdit(cut);
		MainController.deltaSideController[activeSideIndex].fireDirectUndoableEditHappened(wrapperEdit);

		MainController.xmlClipBoard.treeNode.setClipBoardMode(null);
		// MainController.gmtSides[MainController.xmlClipBoard.origin].xmlView.revalidate();
		MainController.deltaSides[MainController.xmlClipBoard.origin].xmlView.nodeChanged(MainController.xmlClipBoard.treeNode);

		paste(activeSideIndex);

		if (cut) {
			MainController.mainFrame.clipBoardLabel.setText("COPY");

			// don't need to check for dead links in other side, because the clipboard node is not "really" removed - only "moved"
			DeltaSideController otherSideController = MainController.deltaSideController[1 - MainController.xmlClipBoard.origin];
			MainFrame.DeltaSide removedSide = MainController.deltaSides[MainController.xmlClipBoard.origin], otherSide = otherSideController.thisSide;
			TreePath selPath = removedSide.xmlView.getSelectionPath();
			Set<Integer> removedDELTALinks = removedSide.xmlView.removeTreeNode(MainController.xmlClipBoard.treeNode);
			if (activeSideIndex != MainController.xmlClipBoard.origin) {
				if ((removedDELTALinks != null) && otherSide.deltaView.isLoaded()) {
					removedSide.deltaView.removeDELTALinksFromCollection(removedDELTALinks, true);   // remove links from gmtLinks which are in this XML view
					otherSide.deltaView.removeDELTALinksFromCollection(removedDELTALinks, false);  // remove links from gmtLinks which are not in the other DELTA view
					if (!removedDELTALinks.isEmpty())
						switch (MainController.removeDeadLinks) {
							case REMOVE_LINKS_ASK:
								if (JOptionPane.showConfirmDialog(mainFrame, "Do you want to remove " + removedDELTALinks.size()
																			 + " corresponding dead link(s) "
																			 + ((removedDELTALinks.size() > 10) ? ""
																				: "(IDs=" + removedDELTALinks.toString() + ") ")
																			 + "in the opposite file?",
																  "Remove dead links", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
										== JOptionPane.NO_OPTION)
									break;
							case REMOVE_LINKS_YES:
								otherSide.deltaView.removeDELTALinks(removedDELTALinks);
								break;
						}

					// removeTreeNode changed the selection of xmlClipboardOriginSideIndex xmlView and that changed the activeSide
					MainController.setActiveSideIndex(activeSideIndex);
				}
			} else
				removedSide.xmlView.centerAndSelectTreePath(selPath);

			MainController.xmlClipBoard.origin = activeSideIndex;
			TreePath treePath = MainController.deltaSides[MainController.xmlClipBoard.origin].xmlView.getSelectionPath();
			MainController.xmlClipBoard.treeNode = (XMLTreeNode) treePath.getLastPathComponent();
		}

		wrapperEdit.end();
		MainController.updateUndoActions();
	}

	protected void paste(int activeSideIndex) {
		MainController.deltaSides[activeSideIndex].xmlView.insertNodeInto(MainController.xmlClipBoard.xmlNode.cloneNode(true));
	}

	// needed for CUT and "replace-node-on-paste"
	private final class PasteWrapperUndoableEdit extends SideCompoundEdit {
		public PasteWrapperUndoableEdit(boolean cutOrCopy) {
			super(false);
			undoPresentationName = "Undo - " + (cutOrCopy ? "cut" : "copy") + " & paste";
			redoPresentationName = "Redo - " + (cutOrCopy ? "cut" : "copy") + " & paste";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof MacroInsertUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}

		// todo: reset MainController.xmlClipBoard.* in undo() and redo() 
	}
}
