/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 19.05.2003
 * Time: 16:11:35
 */
package at.ac.tuwien.asgaard.delta.gui.actions.macros;

import at.ac.tuwien.asgaard.delta.gui.*;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosListItem;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.MacroInsertUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideCompoundEdit;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;

import javax.swing.*;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Set;

/**
 * Class MacroInsertAction ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.3.0
 */
public class MacroInsertAction extends SimpleAction {
	public MacroInsertAction() {
		super("Insert Macro", MainImages.MACRO_INSERT_ICON, 'M',
			  "Insert Macro", "Insert the selected Macro into the current XML Element.", "Inserting macro ...", false);
		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
	}

	public void simpleActionPerformed() {
		int activeSideIndex = MainController.getActiveSideIndex();
		MacrosListItem.Element macrosListItem = (MacrosListItem.Element) mainFrame.macrosView.getCurrentItem();

		if (macrosListItem.isMacro() && ((MacrosListItem.Macro) macrosListItem).hasLink()) {
			if (MainController.deltaSideController[GUIConstants.LEFT].isTypeHTML() &&
				!MainController.deltaSides[GUIConstants.LEFT].deltaView.acceptsNewDELTALink(activeSideIndex == GUIConstants.LEFT, true)) {
				JOptionPane.showMessageDialog(MainController.mainFrame,
											  "To insert a link-macro, first select\ntext to be linked in the left side!",
											  "No text selected", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			if (MainController.deltaSideController[GUIConstants.RIGHT].isTypeHTML() &&
				!MainController.deltaSides[GUIConstants.RIGHT].deltaView.acceptsNewDELTALink(activeSideIndex == GUIConstants.RIGHT, true)) {
				JOptionPane.showMessageDialog(MainController.mainFrame,
											  "To insert a link-macro, first select\ntext to be linked in the right side!",
											  "No text selected", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
		}

		DeltaSideController thisSideController = MainController.deltaSideController[activeSideIndex];
		if (thisSideController.getType() == HTML) thisSideController = thisSideController.otherSideController;

		int insertablePosition;
		if (thisSideController.isTypeHTML() && MainController.aMacroInsert.isEnabled()) { //mainFrame.macrosView.isMacroLink()) {
			thisSideController = MainController.getInactiveSideController();
			insertablePosition = thisSideController.dtdHelper.getInsertIndexForMacro(macrosListItem.getXMLElement());
		} else
			insertablePosition = macrosListItem.getInsertablePosition();

		DeltaSideController otherSideController = thisSideController.otherSideController;
		MainFrame.DeltaSide thisSide = thisSideController.thisSide, otherSide = otherSideController.thisSide;

		thisSideController.aSwitchToViewer.setSelected(true);
		thisSide.deltaView.removeHighlights();

		String otherSelectedText = otherSide.deltaView.getSelectedText();
		int linkID, otherDELTALinkID = otherSide.deltaView.getCurrentDELTALinkID(true);

		CompoundEdit wrapperEdit = new MacroInsertWrapperUndoableEdit(macrosListItem.getName());
		thisSideController.fireDirectUndoableEditHappened(wrapperEdit);

		Set<Integer> deltaLinks = null;
		if (thisSideController.dtdHelper.getInsertMode() == DTDHelper.INSERT_MODE_REPLACE)
			deltaLinks = thisSide.xmlView.removeCurrentTreeNode();

		if (macrosListItem.isMacro())
			linkID = thisSideController.thisSide.xmlView.insertMacro((MacrosListItem.Macro) macrosListItem,
																	 insertablePosition, otherDELTALinkID, otherSelectedText,
																	 otherSide.deltaView.getSelectedNode());
		else {
			thisSideController.thisSide.xmlView.insertNode(macrosListItem.getName(), insertablePosition);
			linkID = LinkIDConstants.NO_LINK;
		}

		thisSideController.sourceViewNeedsUpdateXHTMLSource = thisSideController.textViewNeedsUpdateXHTMLSource = true;

		if ((linkID >= LinkIDConstants.FIRST_ID) && (otherDELTALinkID < LinkIDConstants.FIRST_ID)) { // was this a link macro?
			otherSideController.aSwitchToViewer.setSelected(true);
			otherSideController.thisSide.deltaView.removeHighlights();
			otherSideController.thisSide.deltaView.insertNewDELTALink(linkID);
			MainController.setActiveSideIndex(activeSideIndex);
			otherSideController.sourceViewNeedsUpdateXHTMLSource = otherSideController.textViewNeedsUpdateXHTMLSource = true;
		}
		if (deltaLinks != null) { // for INSERT_MODE_REPLACE: have links been removed?
			if (otherSide.deltaView.isLoaded()) {
				thisSide.deltaView
						.removeDELTALinksFromCollection(deltaLinks, true);   // remove links from gmtLinks which are in this XML view
				otherSide.deltaView.removeDELTALinksFromCollection(deltaLinks,
																   false);  // remove links from gmtLinks which are not in the other DELTA view

				if (!deltaLinks.isEmpty())
					switch (MainController.removeDeadLinks) {
						case REMOVE_LINKS_ASK:
							if (JOptionPane.showConfirmDialog(mainFrame, "Do you want to remove " + deltaLinks.size()
																		 + " corresponding dead link(s) "
																		 + ((deltaLinks.size() > 10) ? "" :
																			"(IDs=" + deltaLinks.toString() + ") ") +
																													"in the opposite file?",
															  "Remove dead links", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
								== JOptionPane.NO_OPTION)
								break;
						case REMOVE_LINKS_YES:
							otherSideController.disableActions(true);
							otherSide.deltaView.setEnabled(false);
							otherSide.deltaView.removeDELTALinks(deltaLinks);
							otherSide.deltaView.setEnabled(true);
							otherSideController.enableActions();
							break;
					}
			}
		}

		wrapperEdit.end();
		MainController.updateUndoActions();
	}


	private final class MacroInsertWrapperUndoableEdit extends SideCompoundEdit {
		public MacroInsertWrapperUndoableEdit(String macroName) {
			super(false);
			undoPresentationName = "Undo - Macro " + "\"" + macroName + "\" insertion";
			redoPresentationName = "Redo - Macro " + "\"" + macroName + "\" insertion";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof MacroInsertUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}
}
