/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 19.05.2003
 * Time: 15:43:00
 */
package at.ac.tuwien.asgaard.delta.gui.actions.macros;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.BaseAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.awt.event.*;
import java.io.File;
import java.io.PrintWriter;

import javax.swing.*;

/**
 * Class MacrosOpenAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class MacrosOpenAction extends BaseAction {
	private JFileChooser macrosFileChooser;

	public MacrosOpenAction() {
		super("Open...", MainImages.OPEN_ICON);

		putValue(MNEMONIC_KEY, new Integer('O'));
		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(StatusBar.TOOL_TIP, "Open Macros file"); // when mouse over - tool-tip
		putValue(StatusBar.DESCRIPTION_HOVER, "Open Macros file."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Opening Macros file ...");  // when clicked
	}

	public void actionPerformed(ActionEvent evt) {
		doOpenFile(null);
	}

	public void doOpenFile(File macrosFile) {
		mainFrame.setStatusBarActiveMessage(this, true);

		if ((macrosFile == null) || macrosFile.isDirectory()) {
			File dir = FileUtils.currentDirFile;
			if (macrosFile == null) {
				macrosFile = mainFrame.macrosView.getFile();
				if (macrosFile != null) dir = macrosFile.getParentFile();
			} else
				dir = macrosFile;
			if (macrosFileChooser == null)
				macrosFileChooser = new DeltaFileChooser(dir, "Open Macros File");
			else
				macrosFileChooser.setCurrentDirectory(dir);

			macrosFileChooser.resetChoosableFileFilters();
			XmlLanguage currentXMLLanguage = MainController.getActiveSideController().currentXMLLanguage;
			if (currentXMLLanguage != null) {
				macrosFileChooser.addChoosableFileFilter(DeltaFileChooser.MACROS_FILE_FILTER);
				macrosFileChooser.setFileFilter(new DeltaFileChooser.MacroFileFilter(currentXMLLanguage.getName()));
			} else
				macrosFileChooser.setFileFilter(DeltaFileChooser.MACROS_FILE_FILTER);

			if (macrosFileChooser.showOpenDialog(mainFrame) != JFileChooser.APPROVE_OPTION) {
				mainFrame.setStatusBarActiveMessage(this, false);
				return;
			}
			macrosFile = macrosFileChooser.getSelectedFile();
		}
		doOpenFile(macrosFile, false);
	}

	public void doOpenFileOnStartUp(File file) {
		mainFrame.setStatusBarActiveMessage(this, true);
		doOpenFile(file, true);
	}

	private void doOpenFile(final File macrosFile, final boolean onStartUp) {
		MainController.aMacrosSwitchToStructureView.setSelected(true);
		MainController.aMacroSwitchToTreeView.setSelected(true);
		MainController.disableMacrosActions();
		mainFrame.ensureMacrosPartIsVisible();
		
		final String filePath = macrosFile.getAbsolutePath();
		final File oldFile = mainFrame.macrosView.getFile();
		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			{
				ClassLoader loader = MacrosOpenAction.this.getClass().getClassLoader();
				Thread.currentThread().setContextClassLoader(loader);
			}
			public Object construct() {
				try {
					mainFrame.macrosView.load(macrosFile, "delta-link");
				} catch (Throwable err) {
					System.err.println("\nCouldn't load macros file \"" + filePath + "\": ");
					err.printStackTrace(new PrintWriter(System.err, true));
					return "Couldn't load macros file \"" + filePath + "\":\n\n"+err.getMessage();
				}
				return null;
			}
			public void finished() {
				Object errorMsg = get();
				if (errorMsg == null) {
					if (macrosFileChooser != null)
						macrosFileChooser.setCurrentDirectory(macrosFile.getParentFile());
					MainController.recentMacrosFiles.remove(macrosFile);
					if (oldFile != null) {
						MainController.addRecentFile(MainController.recentMacrosFiles, oldFile);
						MainController.buildRecentMacrosFilesMenus();
						if (MainController.addRecentFile(MainController.recentDirs, macrosFile.getParentFile()))
							MainController.buildRecentDirsMenus();
					} // else mainFrame.jLabelMacrosFileName.setBorder(MainFrame.FILENAME_BORDER);

					mainFrame.jLabelMacrosFileName.setText(macrosFile.getName());
					mainFrame.jLabelMacrosFileName.setToolTipText(macrosFile.getAbsolutePath());

					mainFrame.macrosView.revalidate();
					MainController.fileChanged(MACROS);
				} else {
					if (MainController.recentMacrosFiles.remove(macrosFile))
						MainController.buildRecentMacrosFilesMenus();
					if (filePath.equals(MainController.stateProperties.getString(Preferences.KEY_LAST_MACROS_FILE)))
						MainController.stateProperties.putString(Preferences.KEY_LAST_MACROS_FILE, "");
					if (!onStartUp)
						JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(),
													  "Error", JOptionPane.ERROR_MESSAGE);
				}
				MainController.enableMacrosActions();
				mainFrame.setStatusBarActiveMessage(MacrosOpenAction.this, false);
			}
		}.start();
	}
}
