/*
 * Project "Asbru Interpreter".
 * 
 * $Id$
 */
package at.ac.tuwien.asgaard.delta.gui.actions.project;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Steps;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.ProcessTemplate;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.StepTemplate;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.io.File;
import java.io.IOException;

import javax.swing.*;

/**
 * Class ProjectNewProcessBasedOnTemplateAction - todo javadoc.
 *
 * @author Peter Votruba
 * @version $Revision$
 */
public class ProjectNewProcessBasedOnTemplateAction extends SimpleAction {
	private final ProcessTemplate processTemplate;
	private final File baseFile;

	private JFileChooser processFileChooser; // re-use JFileChooser

	public ProjectNewProcessBasedOnTemplateAction(ProcessTemplate processTemplate, int nr, File baseFile) {
		this.processTemplate = processTemplate;
		this.baseFile = baseFile;

		String label = processTemplate.getName();
		putValue(NAME, "" + (nr + 1) + ' ' + label);
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Creating new process based on '" + label + "' ...");  // when clicked
		putValue(Action.SMALL_ICON, MainImages.TEMPLATE_ICON);
		putValue(Action.MNEMONIC_KEY, '1' + nr);
		String statusBarText = "Create new process based on Template \"" + label + '\"';
		putValue(StatusBar.DESCRIPTION_HOVER, statusBarText); // when mouse over - statusbar
		String description = processTemplate.getDescription();
		if ((description == null) || (description.trim().length() == 0))
			putValue(Action.SHORT_DESCRIPTION, statusBarText);
		else
			putValue(Action.SHORT_DESCRIPTION, statusBarText + ":\n  " + description); // when mouse over - tool-tip
	}

	protected void simpleActionPerformed() {
		// prevent at.ac.tuwien.asgaard.delta.gui.MainController.fileChanged()
		MainController.setCreatingNewProcess(true);
		createNewProcess();
		MainController.setCreatingNewProcess(false);
	}

	private void createNewProcess() {
		// left side: use source-language of template
		String sourceLanguageName = processTemplate.getSourceLanguage();
		if (sourceLanguageName.equalsIgnoreCase("HTML")) {
			MainController.deltaSideController[LEFT].setTypeHTML();
		} else {
			XmlLanguage sourceLanguage = MainController.getXmlLanguageByName(sourceLanguageName);
			if (sourceLanguage == null) {
				JOptionPane.showMessageDialog(mainFrame, "XML-Language for source file not defined: '"+sourceLanguageName +"'!");
				return;
			}
			MainController.deltaSideController[LEFT].setCurrentXMLLanguage(sourceLanguage, false, true);
		}
		File sourceFile = MainController.deltaSideController[LEFT].aOpen.doOpenFile(null, false, true, false, true, "Open '"+ sourceLanguageName +"' File (source of 1. step) ...");
		if (sourceFile == null)
			return; // user cancelled

		// right side: use target-language of first step
		StepTemplate firstStepTemplate = processTemplate.getStepTemplate(0);
		String targetLanguageName = firstStepTemplate.getTargetLanguage();
		File targetFile;
		if (targetLanguageName.equalsIgnoreCase("HTML")) {
			MainController.deltaSideController[RIGHT].setTypeHTML();
			targetFile = MainController.deltaSideController[RIGHT].aOpen.doOpenFile(null, false, true, false, true,
																					"Open '" + targetLanguageName + "' File (target of 1. step) ...");
			if (targetFile == null)
				return; // user cancelled
		} else {
			XmlLanguage targetLanguage = MainController.getXmlLanguageByName(targetLanguageName);
			if (targetLanguage == null) {
				JOptionPane.showMessageDialog(mainFrame, "XML-Language for first step not defined: '" + targetLanguageName + "'!");
				return;
			}
			MainController.deltaSideController[RIGHT].setCurrentXMLLanguage(targetLanguage, false, true);

			JFileChooser fileChooser = new DeltaFileChooser("Choose existing or enter new '" + targetLanguageName + "' File (target of 1. step) ... ", false);

			fileChooser.setAcceptAllFileFilterUsed(true);
			fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
			fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
			fileChooser.setFileFilter(targetLanguage.getFileFilter());

			if (fileChooser.showSaveDialog(mainFrame) == JFileChooser.CANCEL_OPTION)
				return; // user cancelled
			targetFile = fileChooser.getSelectedFile();
			if (targetFile.exists() && targetLanguage.getFileFilter().accept(targetFile))
				MainController.deltaSideController[RIGHT].aOpen.doOpenFile(targetFile, false, false, false);
			else
				MainController.deltaSideController[RIGHT].aXMLNewEmptyFile.createNewEmptyXMLFile(targetFile);
		}

		// macro-file
		String macroFilePath = firstStepTemplate.getMacroFile();
		File macroFile = null;
		if (macroFilePath != null) {
			macroFilePath = macroFilePath.trim();
			if (macroFilePath.length() > 0) {
				macroFile = FileUtils.toExistingAbsoluteFile(macroFilePath, baseFile);
				if (macroFile == null)
					JOptionPane.showMessageDialog(mainFrame, "Macro-File for first step does not exist:\n  '" + macroFilePath + "'!");
				else
					MainController.aMacrosOpen.doOpenFile(macroFile);
			}
		}

		if (processFileChooser == null) {
			processFileChooser = new DeltaFileChooser("Save transformation process ... ");
			processFileChooser.setFileFilter(DeltaFileChooser.PROCESS_FILE_FILTER);
		}
		if (processFileChooser.showSaveDialog(mainFrame) == JFileChooser.CANCEL_OPTION) {
			System.out.println("process file not saved!");
			return;
		}
		File processFile = processFileChooser.getSelectedFile();

		Process process = Process.newInstance();
		process.setName(processTemplate.getName());
		process.setTemplate(processTemplate.getName());
		process.setSourceFile(FileUtils.toRelativeFile(sourceFile, processFile));

		int leftMaxDELTALinkID = MainController.deltaSides[LEFT].deltaView.getMaxDELTALinkID();
		int rightMaxDELTALinkID = MainController.deltaSides[RIGHT].deltaView.getMaxDELTALinkID();
		int highestLinkID = Math.max(leftMaxDELTALinkID, rightMaxDELTALinkID);
		process.setHighestLinkID(String.valueOf(highestLinkID));

		at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step firstStep =
				at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step.newInstance();
		firstStep.setName(sourceLanguageName + " -> " + targetLanguageName);
		firstStep.setTargetFile(FileUtils.toRelativeFile(targetFile, processFile));
		if (macroFile != null)
			firstStep.setMacroFile(FileUtils.toRelativeFile(macroFile, processFile));
		Steps steps = Steps.newInstance();
		steps.setActiveStep("1");
		steps.addStep(firstStep);
		process.setSteps(steps);
		try {
			process.marshal(processFile);
			MainController.setDELTAProcess(process, false);
		} catch (IOException exc) {
			System.err.println("Could not write process file '"+ processFile+"':\n  "+exc.getMessage());
			JOptionPane.showMessageDialog(mainFrame, "Could not write process file '"+ processFile+"':\n  "+exc.getMessage(), "I/O Error", JOptionPane.ERROR_MESSAGE);
		}
	}
}
