/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 30.04.2003
 * Time: 11:26:35
 */
package at.ac.tuwien.asgaard.delta.gui.actions.project;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.xmlBeans.Project;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;

import org.xml.sax.SAXException;

import java.awt.event.*;
import java.io.File;
import java.io.IOException;

import javax.swing.*;

/**
 * Class ProjectOpenAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class ProjectOpenAction extends SimpleAction {
	private JFileChooser fileChooser;

	public ProjectOpenAction() {
		super("Open...", MainImages.PROJECT_OPEN_ICON, 'O');
		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(StatusBar.TOOL_TIP, "Open Project"); // when mouse over - tool-tip
		putValue(StatusBar.DESCRIPTION_HOVER, "Open project/process."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Open project/process...");  // when clicked
	}
	public void simpleActionPerformed() {
		initFileChooser();

		if (MainController.currentProject != null)
			fileChooser.setCurrentDirectory(MainController.currentProject.getProjectFile().getParentFile());
		else if (MainController.currentProcess != null)
			fileChooser.setCurrentDirectory(MainController.currentProcess.getFile().getParentFile());

		if (fileChooser.showOpenDialog(MainController.mainFrame) == JFileChooser.APPROVE_OPTION) {
			File projectFile = fileChooser.getSelectedFile();
			doOpenProjectFile(projectFile);
		}
	}

	private void initFileChooser() {
		if (fileChooser == null) {
			fileChooser = new DeltaFileChooser(FileUtils.currentDirFile, "Open Project/Process ...");
			fileChooser.addChoosableFileFilter(DeltaFileChooser.PROJECT_FILE_FILTER);
			fileChooser.addChoosableFileFilter(DeltaFileChooser.PROCESS_FILE_FILTER);
			fileChooser.setFileFilter(DeltaFileChooser.PROJECT_AND_PROCESS_FILE_FILTER);
		}
	}

	public void doOpenProjectFile(File projectFile) {
		if (projectFile.isDirectory()) {
			initFileChooser();
			fileChooser.setCurrentDirectory(projectFile);
			if (fileChooser.showOpenDialog(MainController.mainFrame) == JFileChooser.APPROVE_OPTION)
				projectFile = fileChooser.getSelectedFile();
			else
				return;
		}

		if (DeltaFileChooser.PROJECT_FILE_FILTER.accept(projectFile))
			try {
				Project project = Project.load(projectFile);
				MainController.setDELTAProject(project, true, false);
			} catch (IOException exc) {
				Log.error("I/O-Error while loading DELT/A-Project XML-File \"" + projectFile.getAbsolutePath() + "\": " + exc.getMessage());
			//	exc.printStackTrace();
				JOptionPane.showMessageDialog(MainController.mainFrame, "I/O-Error while loading DELT/A-Project XML-File \"" + projectFile.getAbsolutePath() + "\":\n\n" + exc.getMessage(), "I/O Error", JOptionPane.ERROR_MESSAGE);
			} catch (SAXException exc) {
				Log.error("Couldn't parse DELT/A-Project XML-File \"" + projectFile.getAbsolutePath() + "\": " + exc.getMessage());
			//	exc.printStackTrace();
				JOptionPane.showMessageDialog(MainController.mainFrame, "Couldn't parse DELT/A-Project XML-File \"" + projectFile.getAbsolutePath() + "\":\n\n" + exc.getMessage(), "Parse Error", JOptionPane.ERROR_MESSAGE);
			}
		else // opened file was process file
			try {
				at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process process = Process.unmarshal(projectFile, true);
				MainController.setDELTAProcess(process, true);
			} catch (IOException exc) {
				Log.error("I/O-Error while loading DELT/A-Process XML-File \"" + projectFile.getAbsolutePath() + "\": " +
						  exc.getMessage());
			//	exc.printStackTrace();
				JOptionPane.showMessageDialog(MainController.mainFrame, "I/O-Error while loading DELT/A-Process XML-File \"" +
																		projectFile.getAbsolutePath() + "\":\n\n" + exc.getMessage(),
																		"I/O Error", JOptionPane.ERROR_MESSAGE);
			}
	}
}
