package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.*;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.*;
import at.ac.tuwien.asgaard.swing.StatusBar;

import javax.swing.undo.*;

public class XHTMLInsertNewLinkAction extends SimpleBothSidesAction {
    public XHTMLInsertNewLinkAction(int thisSideIndex) {
		super(thisSideIndex, "Create new DELT/A link", MainImages.LINK_ICON);
        putValue(MNEMONIC_KEY, new Integer('N'));
        putValue(StatusBar.TOOL_TIP,
                "Create a new DELT/A link"); // when mouse over
        putValue(StatusBar.DESCRIPTION_HOVER,
                "Create a new DELT/A link with either a new link ID or the Link ID of the selected DELT/A link from opposite file."); // when mouse over - statusbar
        putValue(StatusBar.DESCRIPTION_ACTIVE,
                "Creating a new DELT/A link ...");  // when clicked
        enabled = false;
    }

    public void simpleActionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
		otherSideController.aSwitchToViewer.setSelected(true);

		DeltaSideView activeSideDeltaView = thisSide.deltaView, inactiveSideDELTAView = otherSide.deltaView;
		activeSideDeltaView.removeHighlights();
		inactiveSideDELTAView.removeHighlights();

		int activeSideDELTALinkID = activeSideDeltaView.getCurrentDELTALinkID(true);
		int inactiveSideDELTALinkID = inactiveSideDELTAView.getCurrentDELTALinkID(true);

		boolean activeSideLink = (activeSideDELTALinkID >= LinkIDConstants.FIRST_ID), inactiveSideLink = (inactiveSideDELTALinkID >= LinkIDConstants.FIRST_ID);
		int newDELTALinkID = (activeSideLink ? activeSideDELTALinkID :
				(inactiveSideLink ? inactiveSideDELTALinkID :
				   Math.max(Math.max(activeSideDeltaView.getMaxDELTALinkID(), inactiveSideDELTAView.getMaxDELTALinkID()), MainController.getProcessMaxLinkID()) + 1));

		int activeSideIndex = MainController.getActiveSideIndex();

		InsertLinksWrapperUndoableEdit wrapperEdit = new InsertLinksWrapperUndoableEdit();
		thisSideController.fireDirectUndoableEditHappened(wrapperEdit);

		if (!activeSideLink)
			activeSideDeltaView.insertNewDELTALink(newDELTALinkID);
		if (!inactiveSideLink)
			inactiveSideDELTAView.insertNewDELTALink(newDELTALinkID);

		wrapperEdit.end();
		MainController.updateUndoActions();

		if (!inactiveSideLink)
			MainController.setActiveSideIndex(activeSideIndex);
    }

	private final class InsertLinksWrapperUndoableEdit extends SideCompoundEdit {
		public InsertLinksWrapperUndoableEdit() {
			super(true);
			undoPresentationName = "Undo - Links Insertion";
			redoPresentationName = "Redo - Links Insertion";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof LinkInsertedUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}
}
