/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 15.05.2003
 * Time: 15:33:08
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

//import at.ac.tuwien.asgaard.xmlgrammar.dtd.DTDGrammar;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;

import net.socialchange.doctype.Doctype;
import net.socialchange.doctype.DoctypeChangerReader;
import net.socialchange.doctype.DoctypeGenerator;

import java.awt.event.*;
import java.io.File;

import javax.swing.*;

/**
 * Class XHTMLOpenFileAction ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.3.0
 */
public final class XHTMLOpenFileAction extends BothSidesAction {
	public XHTMLOpenFileAction(int sideIndex) {
		super(sideIndex, "Open", MainImages.OPEN_ICON);

		putValue(MNEMONIC_KEY, new Integer('O'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		putValue(StatusBar.TOOL_TIP, "Open HTML/XML file"); // when mouse over - tool-tip
		putValue(StatusBar.DESCRIPTION_HOVER, "Open HTML/XML file."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Opening HTML/XML file ...");  // when clicked
	}

	protected void actionPerformed() {
		doOpenFile(null, false, true, true);
	}

	public void doOpenFile(File file, boolean onStartUp, boolean revertToPreviousFileOnError, boolean allowCancel) {
		doOpenFile(file, onStartUp, revertToPreviousFileOnError, allowCancel, false, "Open HTML/XML File ...");
	}

	// returns the loaded file or null if the user cancelled file-chooser
	public File doOpenFile(File file, boolean onStartUp, boolean revertToPreviousFileOnError, boolean allowCancel,
						   boolean restrictFileFilter, String dialogTitle) {
		Log.trace("enter: XHTMLOpenFileAction.doOpenFile() file=" + file);
		mainFrame.setStatusBarActiveMessage(this, true);

		if (thisSideController.saveIfChanged(allowCancel)) {
			if ((file == null) || file.isDirectory()) {
				File dir = FileUtils.currentDirFile;
				if ((file == null)) {
					if (thisSide.deltaView.isLoaded())
						dir = thisSide.deltaView.getFile().getParentFile();
				} else
					dir = file;
				thisSide.fileChooser.setCurrentDirectory(dir);
				thisSide.fileChooser.setDialogTitle(dialogTitle);

				thisSide.fileChooser.resetChoosableFileFilters();

				boolean isTypeHTML = thisSide.langNameLabel.getIcon() == MainImages.HTML_ICON; // hack instead of thisSideController.isTypeHTML()
				if (thisSideController.currentXMLLanguage != null)
					thisSide.fileChooser.addChoosableFileFilter(thisSideController.currentXMLLanguage.getFileFilter()); 
				else if (isTypeHTML)
					thisSide.fileChooser.addChoosableFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
				if (!restrictFileFilter || ((thisSideController.currentXMLLanguage == null) && !isTypeHTML)) {
					for (XmlLanguage xmlLanguage : MainController.xmlLanguages)
						if (xmlLanguage != thisSideController.currentXMLLanguage)
							thisSide.fileChooser.addChoosableFileFilter(xmlLanguage.getFileFilter()); //new DocTypeFileFilter(xmlLanguage));
					if (!isTypeHTML)
						thisSide.fileChooser.addChoosableFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
					thisSide.fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
					thisSide.fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
					thisSide.fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_HTML_FILE_FILTER);
				}
				if (thisSideController.currentXMLLanguage != null)
					thisSide.fileChooser.setFileFilter(thisSideController.currentXMLLanguage.getFileFilter());
				else if (isTypeHTML)
					thisSide.fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);

				if (thisSide.fileChooser.showOpenDialog(mainFrame) == JFileChooser.CANCEL_OPTION) {
					mainFrame.setStatusBarActiveMessage(this, false);
					return null;
				}
				file = thisSide.fileChooser.getSelectedFile();
			}

			doOpenFile(file, false, true);
			return file;
		} else {
			mainFrame.setStatusBarActiveMessage(this, false);
			return null;
		}
	}

	public void doOpenFile(final File file, final boolean onStartUp, final boolean revertToPreviousFileOnError) {
		thisSideController.disableActions(true);
		thisSideController.aSwitchToViewer.setSelected(true);
		final File oldFile = thisSide.deltaView.getFile();
		final String filePath = file.getAbsolutePath();
		if (DeltaFileChooser.HTML_FILE_FILTER.accept(file))
			thisSideController.setTypeHTML();
		else
			thisSideController.setTypeXML();
		thisSide.ensureSideIsVisible();
		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			public Object construct() {
				try {
					if (thisSideController.getType() == HTML)
						thisSide.htmlView.load(file);
					else {
						DoctypeChangerReader dtReader = new DoctypeChangerReader(file, new DoctypeGenerator() {
							public Doctype generate(Doctype doctype) {
								// setCurrentXMLLanguageFromDocType returns true, if doctype matches a defined XML language
								// if doctype matches a defined XML language, the current XML language will be changed and the corresponding DTD will be loaded (both by setCurrentXMLLanguageFromDocType)
								// thus only if setCurrentXMLLanguageFromDocType returns false, the DTD file needs to be loaded

                                if (!thisSideController.setCurrentXMLLanguageFromDocType(doctype, file, true))
									thisSideController.dtdHelper.loadDTDFile(FileUtils.toAbsoluteFilePath((doctype != null) ? doctype.getSystemId() : null, file));
														
								return doctype;
							}
						});
						thisSide.xmlView.load(file, dtReader);
						if (dtReader.isDoctypeChanged())
							if (MainController.AUTO_SAVE_AFTER_DOCTYPE_ADAPTION)
								thisSideController.aSave.doSaveFileInSameThread();
							else
								thisSide.xmlView.contentHasChanged(true);
					}

					int externalSideMaxDELTALinkID = thisSide.deltaView.getMaxDELTALinkID();
					if (MainController.currentProcess != null)
						externalSideMaxDELTALinkID = Math.max(externalSideMaxDELTALinkID, MainController.getProcessMaxLinkID());
					otherSide.deltaView.setExternalMaxDELTALinkID(externalSideMaxDELTALinkID);
				} catch (Throwable err) {
					System.err.println("\nCouldn't load file \"" + filePath + "\"!\n\n" + err.getMessage());
					err.printStackTrace();
					return "Couldn't load file \"" + filePath + "\"!\n\n" + err.getMessage();
				}
				return null;
			}
			public void finished() {
				Object errorMsg = get();
				if (errorMsg == null) {
					MainController.addRecentFile(MainController.recentXHTMLFiles, file);
					thisSideController.buildRecentFilesMenus();
					otherSideController.buildRecentFilesMenus();
					if (MainController.addRecentFile(MainController.recentDirs, file.getParentFile()))
						MainController.buildRecentDirsMenus();

					thisSide.setFileNameLabel(file); 
					thisSideController.sourceViewNeedsUpdateXHTMLSource = thisSideController.sourceViewNeedsUpdateDTDSource = thisSideController.sourceViewNeedsUpdateDTDReport = thisSideController.textViewNeedsUpdateXHTMLSource = thisSideController.textViewNeedsUpdateDTDSource = true;
					MainController.fileChanged(thisSideIndex);
				} else {
					if (MainController.recentXHTMLFiles.remove(file)) {
						thisSideController.buildRecentFilesMenus();
						otherSideController.buildRecentFilesMenus();
					}
					if (filePath.equals(MainController.stateProperties.getString(Preferences.KEY_LAST_FILE[thisSideIndex])))
						MainController.stateProperties.putString(Preferences.KEY_LAST_FILE[thisSideIndex], "");
					if (!onStartUp)
						JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
					if (!revertToPreviousFileOnError || (oldFile == null)) {
						thisSideController.aClose.doCloseFile(false, false);
						thisSideController.unsetCurrentXMLLanguage();
					} else
						doOpenFile(oldFile, true, false);
				}
				
				MainController.resetUndoManager(thisSideIndex);
				thisSideController.enableActions();
				if (onStartUp)
					MainController.setActiveSideIndex(MainController.stateProperties.getInt(Preferences.KEY_LAST_ACTIVE_SIDE, LEFT));
				else
					thisSideController.resetXMLTreeActions();
				thisSide.deltaView.requestFocusInWindow();
				mainFrame.setStatusBarActiveMessage(XHTMLOpenFileAction.this, false);
				Log.trace("XHTMLOpenFileAction: finished loading " + file);
			}
		}.start();
	}
}
