/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XHTMLReloadFileAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:26:08
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

import at.ac.tuwien.asgaard.delta.gui.GUIConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.StatusBar;

import java.awt.event.*;

import javax.swing.*;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XHTMLReloadFileAction extends BothSidesAction {
	public XHTMLReloadFileAction(int thisSideIndex) {
		super(thisSideIndex, "Reload", MainImages.RELOAD_ICON);

		putValue(MNEMONIC_KEY, new Integer('R'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_R, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(StatusBar.TOOL_TIP, "Reload current file"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Reload currently open file."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Reloading current file ...");  // when clicked
		enabled = false;
	}

	protected void actionPerformed() {
		doReload(true);
	}

	// Note: does not reset XML language from DocType entry
	public void doReload(boolean checkIfChanged) {
		mainFrame.setStatusBarActiveMessage(this, true);
		if (thisSide.deltaView.contentHasChanged())
			switch (preferences.getInt(Preferences.KEY_AUTO_SAVE)) {
				case GUIConstants.AUTO_SAVE_ASK:
					switch (JOptionPane.showConfirmDialog(mainFrame, "The content of \"" + thisSide.deltaView.getFilePath() +
																	 "\" has been changed.\n\nDo you want to save it ?",
														  "Save", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE)) {
						case JOptionPane.YES_OPTION:
							thisSideController.aSave.doSaveFileInSameThread();
							break;
						case JOptionPane.NO_OPTION:
							break;
						case JOptionPane.CANCEL_OPTION:
							mainFrame.setStatusBarActiveMessage(this, false);
							return;
					}
					break;
				case GUIConstants.AUTO_SAVE_YES:
					thisSideController.aSave.doSaveFile();
			}

		thisSideController.disableActions(true);
		thisSideController.aSwitchToViewer.setSelected(true);
        thisSideController.resetXMLTreeActions();

		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			public Object construct() {
				String errorMsg = null;
				try {
					thisSide.deltaView.reload();

					int externalSideMaxDELTALinkID = thisSide.deltaView.getMaxDELTALinkID();
					if (MainController.currentProcess != null)
						externalSideMaxDELTALinkID = Math.max(externalSideMaxDELTALinkID, MainController.getProcessMaxLinkID());
					otherSide.deltaView.setExternalMaxDELTALinkID(externalSideMaxDELTALinkID);

					thisSideController.dtdHelper.loadDTDFile(thisSide.deltaView.getDTDURLString(), true);
				} catch (Exception exc) {
					System.err.println("\nCouldn't reload file \"" + thisSide.deltaView.getFilePath() + "\": ");
					exc.printStackTrace();
					errorMsg = exc.getMessage();
				}
				return errorMsg;
			}
			public void finished() {
				Object errorMsg = get();
				if (errorMsg == null) {
					thisSideController.aGoBack.setEnabled(false);
					thisSideController.aGoForward.setEnabled(false);
					otherSide.deltaView.removeHighlights();
					thisSideController.sourceViewNeedsUpdateXHTMLSource = thisSideController.sourceViewNeedsUpdateDTDSource = thisSideController.sourceViewNeedsUpdateDTDReport =
							thisSideController.textViewNeedsUpdateXHTMLSource = thisSideController.textViewNeedsUpdateDTDSource = true;
				} else
					JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
				MainController.resetUndoManager(thisSideIndex);
				thisSideController.enableActions();
				mainFrame.setStatusBarActiveMessage(XHTMLReloadFileAction.this, false);
				thisSide.deltaView.requestFocusInWindow();
			}
		}.start();
	}
}
