/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XHTMLReloadFileAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:26:08
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.awt.event.*;
import java.io.File;

import javax.swing.*;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XHTMLSaveFileAction extends BothSidesAction {
	public XHTMLSaveFileAction(int thisSideIndex) {
		super(thisSideIndex, "Save", MainImages.SAVE_ICON);

		putValue(MNEMONIC_KEY, new Integer('S'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		putValue(SHORT_DESCRIPTION, "Save file"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Save the currently open file."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Saving file ...");  // when clicked
		enabled = false;
	}

	protected void actionPerformed() {
		doSaveFile();
	}

	public void doSaveFile() {
		mainFrame.setStatusBarActiveMessage(this, true);
		thisSideController.disableActions(true);
		thisSideController.aSwitchToViewer.setSelected(true);
		final File file = thisSide.deltaView.getFile();

		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			public Object construct() {
				if (preferences.getBoolean(Preferences.KEY_BACKUP_ON_SAVE))
					try {
						FileUtils.createBackup(file);
					} catch (Exception e) {
						System.err.println("Couldn't backup up file \"" + file.getAbsolutePath() + "\": ");
						e.printStackTrace();
					}
				try {
					thisSide.deltaView.save();
				} catch (Throwable exc) {
					System.err.println("\nCouldn't save XML file \"" + file.getAbsolutePath() + "\": ");
					exc.printStackTrace(System.err);
					return "Couldn't save XML file \"" + file.getAbsolutePath() + "\"!\n\n" + exc.getMessage();
				}
				return null;
			}
			public void finished() {
				Object errorMsg = get();
				if (errorMsg != null)
					JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
                else {
				//	thisSideController.aSave.setEnabled(false); // ToThink: let "Save" be always enabled, to allow saving an unchanged XML file in a formatted style
                    MainController.aFileSaveAll.setEnabled(otherSide.deltaView.contentHasChanged());
                }
//  MainController.resetUndoManager(thisSideIndex);
				thisSideController.enableActions();
				mainFrame.setStatusBarActiveMessage(XHTMLSaveFileAction.this, false);
				thisSide.deltaView.requestFocusInWindow();
			}
		}.start();
	}

	public void doSaveFileInSameThread() {
		mainFrame.setStatusBarActiveMessage(this, true);
		thisSideController.disableActions(true);
		thisSideController.aSwitchToViewer.setSelected(true);
		final File file = thisSide.deltaView.getFile();

		if (preferences.getBoolean(Preferences.KEY_BACKUP_ON_SAVE))
			try {
				FileUtils.createBackup(file);
			} catch (Exception e) {
				System.err.println("Couldn't backup up file \"" + file.getAbsolutePath() + "\": ");
				e.printStackTrace();
			}
		try {
			thisSide.deltaView.save();
			//	thisSideController.aSave.setEnabled(false); // ToThink: let "Save" be always enabled, to allow saving an unchanged XML file in a formatted style
			MainController.aFileSaveAll.setEnabled(otherSide.deltaView.contentHasChanged());
		} catch (Throwable exc) {
			System.err.println("\nCouldn't save XML file \"" + file.getAbsolutePath() + "\": ");
			exc.printStackTrace(System.err);
			JOptionPane.showMessageDialog(mainFrame, "Couldn't save XML file \"" + file.getAbsolutePath() + "\"!\n\n" + exc.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}


		MainController.updateHighestLinkIDofProcess(thisSideIndex);

		thisSideController.enableActions();
		mainFrame.setStatusBarActiveMessage(XHTMLSaveFileAction.this, false);
		thisSide.deltaView.requestFocusInWindow();
	}
}
