/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XHTMLReloadFileAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:26:08
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.swing.StatusBar;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.KeyStroke;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XHTMLSaveFileAsAction extends BothSidesAction {
	public XHTMLSaveFileAsAction(int thisSideIndex) {
		super(thisSideIndex, "Save As...", MainImages.SAVE_AS_ICON);

		putValue(MNEMONIC_KEY, new Integer('A'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(SHORT_DESCRIPTION, "Save file as "); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Save the currently open file under a new name."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Saving file ...");  // when clicked
		this.enabled = false;
	}

	protected void actionPerformed() {
		doSaveFileAs();
	}

	private void doSaveFileAs() {
		mainFrame.setStatusBarActiveMessage(this, true);
		thisSideController.aSwitchToViewer.setSelected(true);

		thisSide.fileChooser.resetChoosableFileFilters();
		if (thisSideController.getType() == HTML) {
			thisSide.fileChooser.setDialogTitle("Save HTML File as ...");
			thisSide.fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
			thisSide.fileChooser.setCurrentDirectory(thisSide.deltaView.getFile().getParentFile());
		} else {
			if (thisSideController.currentXMLLanguage == null) {
				thisSide.fileChooser.setDialogTitle("Save XML File as ...");
				thisSide.fileChooser.setFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
				thisSide.fileChooser.setFileFilter(DeltaFileChooser.XML_FILE_FILTER);
				thisSide.fileChooser.setCurrentDirectory(thisSide.deltaView.getFile().getParentFile());
			} else {
				thisSide.fileChooser.setDialogTitle("Save \""+ thisSideController.currentXMLLanguage.getName() +"\" XML File as ...");
				thisSide.fileChooser.setCurrentDirectory(thisSideController.currentXMLLanguage.getFile().getParentFile());
				thisSide.fileChooser.setFileFilter(thisSideController.currentXMLLanguage.getFileFilter()); //new DocTypeFileFilter(thisSideController.currentXMLLanguage));
			}
		}

		if (thisSide.fileChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
			thisSideController.disableActions(true);

			final File oldFile = thisSide.deltaView.getFile();
			final File file = thisSide.fileChooser.getSelectedFile();
			final String xmlFilePath = file.getAbsolutePath();
			new at.ac.tuwien.asgaard.swing.SwingWorker() {
				public Object construct() {
					try {
						thisSide.deltaView.saveAs(file);
					} catch (Throwable exc) {
						System.err.println("\nCouldn't save file \"" + oldFile + "\" to \"" + xmlFilePath + "\": ");
						exc.printStackTrace(System.err);
						return "Couldn't save file \"" + oldFile + "\" to \"" + xmlFilePath + "\"!\n\n"+exc.getMessage();
					}
					return null;
				}
				public void finished() {
					Object errorMsg = get();
					if (errorMsg == null) {
						thisSide.setFileNameLabel(file); //.setText(file.getName());
					//	thisSide.fileNameLabel.setToolTipText(file.getAbsolutePath());

						MainController.addRecentFile(MainController.recentXHTMLFiles, oldFile);
						thisSideController.buildRecentFilesMenus();
						otherSideController.buildRecentFilesMenus();
						if (MainController.addRecentFile(MainController.recentDirs, file.getParentFile()))
							MainController.buildRecentDirsMenus();

						MainController.fileChanged(thisSideIndex);
					} else
						JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
//  MainController.resetUndoManager(thisSideIndex);
					thisSideController.enableActions();
					mainFrame.setStatusBarActiveMessage(XHTMLSaveFileAsAction.this, false);
					thisSide.deltaView.requestFocusInWindow();
				}
			}.start();
		} else {
			mainFrame.setStatusBarActiveMessage(this, false);
			thisSide.deltaView.requestFocusInWindow();
		}
	}
}
