/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XMLRemoveNodeAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:04:38
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.*;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.*;

import javax.swing.JOptionPane;
import javax.swing.undo.*;
import java.util.Set;

/**
 * Class XMLRemoveNodeAction ... todo
 */
public class XHTMLSynchronizeLinksAction extends SimpleBothSidesAction {
	public XHTMLSynchronizeLinksAction(int thisSideIndex) {
		super(thisSideIndex, "Synchronize DELT/A link(s)", MainImages.SYNCHRONIZE_LINKS_ICON);
		putValue(MNEMONIC_KEY, new Integer('S'));
		putValue(StatusBar.TOOL_TIP, "Synchronize selected DELT/A link(s)"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Replace the Link ID of the selected DELT/A link(s) with the Link ID of the selected link from the opposite file."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Replacing the Link ID of the selected DELT/A link(s) with the Link ID of the selected link from the opposite file ...");  // when clicked
		this.enabled = false;
	}

	public void simpleActionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
		thisSide.deltaView.removeHighlights();
		otherSide.deltaView.removeHighlights();

		SynchronizeLinksWrapperUndoableEdit wrapperEdit = new SynchronizeLinksWrapperUndoableEdit();
		thisSideController.fireDirectUndoableEditHappened(wrapperEdit);

		final Set<Integer> gmtLinks = thisSide.deltaView.synchronizeSelectedDELTALinks(otherSide.deltaView.getCurrentDELTALinkID(true));
		if (gmtLinks != null) {
			thisSide.deltaView.removeDELTALinksFromCollection(gmtLinks, true);   // remove links from gmtLinks which are in the XML view
			otherSide.deltaView.removeDELTALinksFromCollection(gmtLinks, false);  // remove links from gmtLinks which are not in the HTML view

			if (!gmtLinks.isEmpty())
				switch (MainController.removeDeadLinks) {
					case REMOVE_LINKS_ASK:
						if (JOptionPane.showConfirmDialog(mainFrame, "Do you want to remove " + gmtLinks.size() + " corresponding dead link(s) "
																	 + ((gmtLinks.size() > 10) ? "" : "(IDs=" + gmtLinks.toString() + ") ")
																	 + "in the opposite file?",
														  "Remove dead links", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
							== JOptionPane.NO_OPTION)
							break;
					case REMOVE_LINKS_YES:
						otherSide.deltaView.removeDELTALinks(gmtLinks);
						break;
				}
		}
		wrapperEdit.end();
		MainController.updateUndoActions();
	}

	private final class SynchronizeLinksWrapperUndoableEdit extends SideCompoundEdit {
		public SynchronizeLinksWrapperUndoableEdit() {
			super(true);
			undoPresentationName = "Undo - Link(s) Synchronization";
			redoPresentationName = "Redo - Link(s) Synchronization";
		}
		
		public boolean addEdit(UndoableEdit anEdit) {
			if ((anEdit instanceof LinksRemovedUndoableEdit) || (anEdit instanceof LinkIDChangedUndoableEdit))
				return super.addEdit(anEdit);
			return false;
		}
	}
}
