/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XMLRemoveNodeAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:04:38
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.html;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.SimpleBothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.MacroInsertUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideCompoundEdit;
import at.ac.tuwien.asgaard.swing.StatusBar;

import javax.swing.*;
import javax.swing.undo.UndoableEdit;
import java.util.Set;

/**
 * Class XMLRemoveNodeAction ... todo
 */
public class HTMLRemoveLinksAction extends SimpleBothSidesAction {
	public HTMLRemoveLinksAction(int thisSideIndex) {
		super(thisSideIndex, "Remove DELT/A link(s)", MainImages.REMOVE_HTML_LINK_ICON);

		putValue(MNEMONIC_KEY, new Integer('R'));
		putValue(StatusBar.TOOL_TIP,            "Remove selected DELT/A links"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER,   "Remove selected DELT/A link(s)."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE,  "Removing selected DELT/A link(s) ...");  // when clicked
		enabled = false;
	}

	public void simpleActionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
		thisSide.deltaView.removeHighlights();
		otherSide.deltaView.removeHighlights();

		if (MainController.askBeforeRemoveHTMLLinks &&
			(JOptionPane.showConfirmDialog(mainFrame, "Do you really want to remove the selected DELT/A-Links in the HTML file?", "Remove DELT/A-Links",
										   JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
			 == JOptionPane.NO_OPTION))
			return;

		RemoveLinksWrapperUndoableEdit wrapperEdit = new RemoveLinksWrapperUndoableEdit();
		thisSideController.fireDirectUndoableEditHappened(wrapperEdit);
		final Set<Integer> deltaLinks = thisSide.htmlView.removeSelectedDELTALinks();

		thisSide.deltaView.removeDELTALinksFromCollection(deltaLinks, true);   // remove link-ids from gmtLinks which are in the current view
		otherSide.deltaView.removeDELTALinksFromCollection(deltaLinks, false);  // remove link-ids from gmtLinks which are not in the other view

		if (!deltaLinks.isEmpty()) {
			switch (MainController.removeDeadLinks) {
				case REMOVE_LINKS_ASK:
					if (JOptionPane.showConfirmDialog(mainFrame, "Do you want to remove " + deltaLinks.size()
																 + " corresponding dead link(s) "
																 + ((deltaLinks.size() > 10) ? "" : "(IDs=" + deltaLinks.toString() + ") ")
																 + "in the opposite file?", "Remove dead links",
																							JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.NO_OPTION)
						break;
				case REMOVE_LINKS_YES:
					otherSide.deltaView.removeDELTALinks(deltaLinks);
					break;
			}
		}

		wrapperEdit.end();
		MainController.updateUndoActions();
	}

	private final class RemoveLinksWrapperUndoableEdit extends SideCompoundEdit {
		public RemoveLinksWrapperUndoableEdit() {
			super(true);
			undoPresentationName = "Undo - Links Removal";
			redoPresentationName = "Redo - Links Removal";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof MacroInsertUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}
}
