/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideController;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.util.EnvironmentProperties;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.Stylesheet;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.SwingWorker;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import org.apache.fop.apps.FOPException;

import javax.swing.*;
import javax.xml.transform.TransformerException;
import java.io.File;
import java.io.IOException;
import java.net.URL;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XMLApplyPredefinedStylesheet extends BothSidesAction {
	private Stylesheet stylesheetDef;

	public XMLApplyPredefinedStylesheet(int thisSideIndex, Stylesheet stylesheetDef, int nr) {
		super(thisSideIndex);

		this.stylesheetDef = stylesheetDef;
		String label = stylesheetDef.getLabel();

		putValue(NAME, "" + (nr + 1) + ' ' + label);
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Applying Stylesheet definition ..."); // when clicked

		Icon icon = MiscImages.XSL_FILE_ICON;
		URL url = FileUtils.toExistingURL(stylesheetDef.getIconFileName(), thisSideController.currentXMLLanguage.getFile());
		if (url != null) {
			icon = new ImageIcon(url);
			if (icon.getIconWidth() < 10) icon = MiscImages.XSL_FILE_ICON;
		}
		stylesheetDef.setIcon(icon);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.MNEMONIC_KEY, new Integer('1' + nr));
		String statusBarText = "Apply Stylesheet definition \"" + label + '\"';
		putValue(StatusBar.DESCRIPTION_HOVER, statusBarText); // when mouse over - statusbar
		String description = stylesheetDef.getDescription();
		if ((description == null) || (description.trim().length() == 0))
			putValue(Action.SHORT_DESCRIPTION, statusBarText);
		else
			putValue(Action.SHORT_DESCRIPTION, "<html>" +statusBarText + ":<br>&nbsp;&nbsp;" + description); // when mouse over - tool-tip
	}

	protected void actionPerformed() {
		mainFrame.statusBar.setActiveMessageFrom(this);
		final String outputType = stylesheetDef.getOutputType().intern();
		final String outputIn = stylesheetDef.getOutputIn().intern();
		final File xslFile = FileUtils.toAbsoluteFile(stylesheetDef.getFileName(), thisSideController.currentXMLLanguage.getFile());

		if (outputType == "pdf") {
			if (!EnvironmentProperties.isIsFopAvailable()) {
				JOptionPane.showMessageDialog(mainFrame, "fop.jar and batik.jar are missing!");
				mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
				return;
			}
			if (outputIn == "extraWindow") {
				try {
					final File foFile = File.createTempFile("Result", ".fo");
					XMLUtils.FOPUtils.transformDOM2FO(thisSide.xmlView.getDocumentNode(), xslFile, foFile);
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							try {
								XMLUtils.FOPUtils.getFOPreviewDialog().setVisible(true);
								XMLUtils.FOPUtils.getFOPreviewDialog().invalidate();
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										try {
											XMLUtils.FOPUtils.renderFO2PreviewDialog(foFile);
										} catch (IOException e) {
											e.printStackTrace();
										} catch (FOPException e) {
											e.printStackTrace();
										}
									}
								});
							} catch (IOException e) {
								e.printStackTrace();
							}
						}
					});
					foFile.deleteOnExit();
				} catch (IOException e) {
					e.printStackTrace();
				} catch (TransformerException e) {
					e.printStackTrace();
				}
			} else if (outputIn == "defaultApp") {
				final String pdfViewerPath = MainController.preferences.getFilePath(Preferences.KEY_PATH_TO_PDF_VIEWER);
				if (pdfViewerPath == null)
					JOptionPane.showMessageDialog(mainFrame, "Path to PDF viewer not defined!");
				else
					try {
						final File pdfFile = File.createTempFile("Result", ".pdf");
						XMLUtils.FOPUtils.transformDOM2PDF(thisSide.xmlView.getDocumentNode(), xslFile, pdfFile);
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								try {
									Runtime.getRuntime().exec(new String[]{pdfViewerPath, pdfFile.getAbsolutePath()}); //.waitFor();
								} catch (IOException e) {
									e.printStackTrace();
									JOptionPane.showMessageDialog(mainFrame, "Couldn't view pdf file!");
								} /*catch (InterruptedException e) {
									e.printStackTrace();  // should not happen
								}*/
							}
						});
						pdfFile.deleteOnExit();
					} catch (IOException e) {
						e.printStackTrace();
					} catch (TransformerException e) {
						e.printStackTrace();
					}
			} else if (outputIn == "newFile") {
				File pdfFile = DeltaFileChooser.showSaveFileChooser(mainFrame, "Choose PDF Result file", null, "pdf");
				if (pdfFile != null)
					try {
						XMLUtils.FOPUtils.transformDOM2PDF(thisSide.xmlView.getDocumentNode(), xslFile, pdfFile);
					} catch (IOException e) {
						e.printStackTrace();
					} catch (TransformerException e) {
						e.printStackTrace();
					}
			}
			mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
		} else
			new at.ac.tuwien.asgaard.swing.SwingWorker() {
				public Object construct() {
					try {
						return XMLUtils.transformDOM2String(thisSide.xmlView.getDocumentNode(), xslFile);
					} catch (Exception exc) {
						exc.printStackTrace();
						return exc;
					}
				}
				public void finished() {
					Object result = get();
					if (!(result instanceof String))
						JOptionPane.showMessageDialog(thisSide, "Could not apply stylesheet!\n   -> "+result, "Transformation Error", JOptionPane.ERROR_MESSAGE);
					else {
						final String resultString = (String) result;
						if (outputType == "text") {
							if (outputIn == "sameSide") {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										setTextContent(thisSideController, resultString);
										thisSideController.styleSheetSourceSideController = thisSideController;
										mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
									}
								});
							} else if (outputIn == "otherSide") {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										setTextContent(otherSideController, resultString);
										otherSideController.styleSheetSourceSideController = thisSideController;
										mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
									}
								});
							}
						} else if (outputType == "html") {
							if (outputIn == "sameSide") {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										setHTMLContent(thisSideController, resultString);
										thisSideController.styleSheetSourceSideController = thisSideController;
										mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
									}
								});
							} else if (outputIn == "otherSide") {
								SwingUtilities.invokeLater(new Runnable() {
									public void run() {
										setHTMLContent(otherSideController, resultString);
										otherSideController.styleSheetSourceSideController = thisSideController;
										mainFrame.statusBar.unsetActiveMessageFrom(XMLApplyPredefinedStylesheet.this);
									}
								});
							}
						}
					}
				}
			}.start();
	}

	private static void setHTMLContent(DeltaSideController sideController, String content) {
		sideController.thisSide.htmlOutput.setText(content);
		sideController.thisSide.htmlOutput.setCaretPosition(0);
		sideController.sourceViewNeedsUpdateXHTMLSource = true;
		sideController.sourceViewNeedsUpdateDTDSource = true;
		sideController.sourceViewNeedsUpdateDTDReport = true;
		sideController.aSwitchToHTMLOutput.setEnabled(true);
		sideController.aSwitchToHTMLOutput.setSelected(true);
	}
	private static void setTextContent(DeltaSideController sideController, String content) {
		sideController.thisSide.textOutput.setText(content);
		sideController.thisSide.textOutput.setCaretPosition(0);
		sideController.textViewNeedsUpdateXHTMLSource = true;
		sideController.textViewNeedsUpdateDTDSource = true;
		sideController.aSwitchToTextOutput.setEnabled(true);
		sideController.aSwitchToTextOutput.setSelected(true);
	}
}
