/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideCompoundEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.MacroInsertUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import at.ac.tuwien.asgaard.swing.StatusBar;

import org.w3c.dom.Comment;
import org.w3c.dom.Node;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import javax.swing.*;
import javax.swing.undo.UndoableEdit;

public class XMLCommentElementAction extends BothSidesAction {
	public XMLCommentElementAction(int thisSideIndex) {
		super(thisSideIndex, "Comment/Uncomment Element", MainImages.COMMENT_EDIT_ICON);

		putValue(MNEMONIC_KEY, new Integer('C'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(StatusBar.TOOL_TIP, "Comment/Uncomment current element"); // when mouse over - tool-tip
		putValue(StatusBar.DESCRIPTION_HOVER, "Comment/Uncomment current element."); // when mouse over - statusbar

		enabled = false;
	}

	protected void actionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
		XMLTreeNode selectedTreeNode = thisSide.xmlView.getSelectedTreeNode();
		Node xmlNode = selectedTreeNode.getXMLNode();
		SideCompoundEdit wrapperEdit = new ElementCommentWrapperUndoableEdit(xmlNode.getNodeName());
		thisSideController.fireDirectUndoableEditHappened(wrapperEdit);
		if ((xmlNode.getNodeType() == Node.ELEMENT_NODE) && thisSideController.dtdHelper.isNodeRemoveable())
			thisSide.xmlView.commentCurrentElement();
		else if (xmlNode.getNodeType() == Node.COMMENT_NODE)
			thisSide.xmlView.unCommentCurrentElement();
		wrapperEdit.end();
		MainController.updateUndoActions();
	}

	public void setEnabled(XMLTreeNode treeNode) {
		Node xmlNode = treeNode.getXMLNode();
		switch (xmlNode.getNodeType()) {
			case Node.ELEMENT_NODE:
				setEnabled(thisSideController.dtdHelper.isNodeRemoveable());
				break;
			case Node.COMMENT_NODE:
				setEnabled(XMLUtils.stringToElement(((Comment) xmlNode).getData()) != null);
				break;
			default: setEnabled(false);
		}
	}

	private static final class ElementCommentWrapperUndoableEdit extends SideCompoundEdit {
		public ElementCommentWrapperUndoableEdit(String elementName) {
			super(false);
			undoPresentationName = "Undo - comment \"" + elementName + "\"";
			redoPresentationName = "Redo - comment \"" + elementName + "\"";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof MacroInsertUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}
}
