/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 12.02.2004
 * Time: 17:02:30
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.SimpleBothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import org.w3c.dom.Attr;
import org.w3c.dom.Node;

import java.util.ArrayList;
import java.util.List;

import javax.swing.*;

/**
 * Class XMLEvaluateXPathExpressionAction ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class XMLEvaluateXPathExpressionAction extends SimpleBothSidesAction {
	public XMLEvaluateXPathExpressionAction(int sideIndex) {
		super(sideIndex, MainImages.EXECUTE_ICON, "Evaluate XPath expression");
	}
	protected XMLEvaluateXPathExpressionAction(int thisSideIndex, String name, Icon icon) {
		super(thisSideIndex, name, icon);
	}

	protected void simpleActionPerformed() {
		String xPathExpression = thisSide.getXPathExpression();
        if (xPathExpression == null) return;
		xPathExpression = xPathExpression.trim();
		if (xPathExpression.equals("")) return;

		boolean result = evaluateAndVisualizeXPathExpression(xPathExpression);
		if (!result) thisSide.xpathBarText.requestFocus();
	}

	public boolean evaluateAndVisualizeXPathExpression(String xPathExpression) {
		System.out.println("Evaluating XPath-Expression: " + xPathExpression);
		thisSideController.aSwitchToViewer.setSelected(true);
		try {
			List resultList = XMLUtils.evaluateXPath(xPathExpression, thisSide.xmlView.getSelectedTreeNode().getXMLNode());

			String resultString = null;
			List<XMLTreeNode> treeNodeResultList = new ArrayList<XMLTreeNode>();

			int valueCount = 0, nodeCount = 0;
			StringBuffer sb = new StringBuffer();
			for (int i = 0, size = resultList.size(); i < size; i++) {
				Object item = resultList.get(i);
				if ((item instanceof Attr)) {
					if (sb.length() > 0) sb.append(", ");
					sb.append(((Attr)item).getNodeValue());
					valueCount++;
				} else if (!(item instanceof Node)) {
					if (sb.length() > 0) sb.append(", ");
					sb.append(item);
					valueCount++;
				} else {
					Node xmlNode = (Node) item; nodeCount++;
					XMLTreeNode xmlTreeNode = thisSide.xmlView.getTreeNode4XMLNode(xmlNode);
					if (xmlTreeNode != null)
						treeNodeResultList.add(xmlTreeNode);
				}
			}
			if (valueCount > 0) {
				if (valueCount > 1) {sb.insert(0, '('); sb.append(')');}
				resultString = MiscUtils.wrapString(sb.toString(), "\n", 60, 5);
			}
			if (treeNodeResultList.isEmpty())
				if (!resultList.isEmpty())
					if (nodeCount > 0)
						if (valueCount > 0)
							JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n  "+ nodeCount+" Node(s) (hidden due to active filters)"
									+ "\n    " + valueCount + " Value(s): "+resultString,
									"Mixed result", JOptionPane.INFORMATION_MESSAGE);
						else
							JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    "+ nodeCount+" Node(s) (hidden due to active filters)",
												  "No visible result", JOptionPane.INFORMATION_MESSAGE);
					else
						JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    " + valueCount + " Value(s): " + resultString,
								"Value(s) as result", JOptionPane.INFORMATION_MESSAGE);
				else
					JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to no result.",
											  "No result", JOptionPane.INFORMATION_MESSAGE);
			else
				if (nodeCount == 1) {
					thisSide.xmlView.centerAndSelectTreeNode(treeNodeResultList.get(0));
					if (valueCount > 0)
						JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    1 Node"
								+ "\n  " + valueCount + " Value(s): " + resultString,
								"Mixed result", JOptionPane.INFORMATION_MESSAGE);
					/*else  // don't need to tell count of nodes if it is only one node
						JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    1 Node",
								"Node as result", JOptionPane.INFORMATION_MESSAGE);*/
				} else {
					thisSide.xmlView.highlightTreeNodes(treeNodeResultList);
					if (valueCount > 0)
						JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    " + nodeCount + " Node(s)"
								+ "\n  " + valueCount + " Value(s): " + resultString,
								"Mixed result", JOptionPane.INFORMATION_MESSAGE);
					else
						JOptionPane.showMessageDialog(mainFrame, "XPath-Expression evaluates to ...\n    " + nodeCount + " Node(s)",
								"Node(s) as result", JOptionPane.INFORMATION_MESSAGE);
				}

			return true;
		} catch (Exception exc) {
			String msg = exc.getMessage();
			JOptionPane.showMessageDialog(mainFrame,
										  "Couldn't evaluate XPath-Expression:\n\n" + xPathExpression + "\n\n" + msg,
										  "XPath Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}
}