/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import java.awt.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.net.URL;

import javax.swing.*;

/**
 * Class XMLJumpToElementDefinitionInDTDReportAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.4.0
 */
public class XMLJumpToElementDefinitionInDTDReportAction extends BothSidesAction {
	public XMLJumpToElementDefinitionInDTDReportAction(int sideIndex) {
		super(sideIndex, "Go to Definition (Report)", MainImages.GO_TO_DEFINITION_ICON);

		putValue(MNEMONIC_KEY, new Integer('G'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK));
		putValue(SHORT_DESCRIPTION, "Go to Definition in DTD - HTML Report"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Switch to DTD-Report view and go to definition of current element."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Switching to DTD-Report view ...");  // when clicked
		enabled = false;
	}

	protected void actionPerformed() {
		jumpToDefinition(thisSide.xmlView.getSelectedNode().getNodeName());
	}

	public void jumpToDefinition(final String elementName) {
		mainFrame.statusBar.setActiveMessageFrom(this);
		thisSideController.aSwitchToHTMLOutput.setEnabled(true);
		thisSideController.aSwitchToHTMLOutput.setSelected(true);
		if (thisSideController.sourceViewNeedsUpdateDTDReport) {
			thisSide.htmlOutput.setBackground(Color.WHITE);
			thisSide.htmlOutput.setText("Showing HTML report of large or remote DTD file may take a while ...");
			new at.ac.tuwien.asgaard.swing.SwingWorker() {
				{
					ClassLoader loader = XMLJumpToElementDefinitionInDTDReportAction.this.getClass().getClassLoader();
					Thread.currentThread().setContextClassLoader(loader);
				}
				public Object construct() {
					URL dtdurl = thisSide.xmlView.getDTDURL();
					if (dtdurl == null) return "No or bad DTD entry in DOCTYPE statement!";
					return XMLUtils.transformDTD2Report(dtdurl);
				}
				public void finished() {
					thisSide.htmlOutput.setText((String) get());
					thisSide.htmlOutput.setCaretPosition(0);
					thisSideController.sourceViewNeedsUpdateDTDReport = false;
					thisSideController.sourceViewNeedsUpdateXHTMLSource = thisSideController.sourceViewNeedsUpdateDTDSource = true;
					thisSide.htmlOutput.scrollToReference(elementName);
					mainFrame.statusBar.unsetActiveMessageFrom(XMLJumpToElementDefinitionInDTDReportAction.this);
				}
			}.start();
		} else {
			thisSide.htmlOutput.scrollToReference(elementName);
			mainFrame.statusBar.unsetActiveMessageFrom(this);
		}
	}
}
