/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XHTMLReloadFileAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:26:08
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.Dtd;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.awt.event.*;
import java.io.File;

import javax.swing.*;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XMLNewEmptyFileAction extends BothSidesAction {
	private final XmlLanguage xmlLanguage;

	public XMLNewEmptyFileAction(int thisSideIndex, XmlLanguage xmlLanguage) {
		super(thisSideIndex, "New empty XML-File...", MainImages.NEW_ICON);
		this.xmlLanguage = xmlLanguage;
		enabled = (xmlLanguage != null);
		if (enabled) {
			putValue(MNEMONIC_KEY, new Integer('N'));
			putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
			putValue(StatusBar.TOOL_TIP, "Create new empty XML file"); // when mouse over
			putValue(StatusBar.DESCRIPTION_HOVER, "Create new empty XML file of " +
												  (enabled ? "'" + xmlLanguage.getName() + "'" : "current") +
												  " XML language."); // when mouse over - statusbar
			putValue(StatusBar.DESCRIPTION_ACTIVE, "Creating new empty XML file ...");  // when clicked}
		}
	}

	private XmlLanguage getXMLLanguage() {
		return (xmlLanguage == null) ? thisSideController.currentXMLLanguage : xmlLanguage;
	}

	protected void actionPerformed() {
		if (xmlLanguage != null)
			thisSideController.setCurrentXMLLanguage(xmlLanguage, false, true);

		mainFrame.setStatusBarActiveMessage(this, true);
		thisSide.ensureSideIsVisible();
		if (thisSideController.saveIfChanged(true)) {
			thisSide.fileChooser.setCurrentDirectory(getXMLLanguage().getFile().getParentFile());
			thisSide.fileChooser.resetChoosableFileFilters();
			thisSide.fileChooser.setFileFilter(getXMLLanguage().getFileFilter());
			thisSide.fileChooser.setDialogTitle("Create new \"" + getXMLLanguage().getName() + "\" XML File as ...");
			if (thisSide.fileChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
				final File file = thisSide.fileChooser.getSelectedFile();
				_createNewEmptyXMLFile(file);
			} else
				mainFrame.setStatusBarActiveMessage(this, false);
		} else
			mainFrame.setStatusBarActiveMessage(this, false);
	}

	public void createNewEmptyXMLFile(final File file) {
		mainFrame.setStatusBarActiveMessage(this, true);
		if (thisSideController.saveIfChanged(false)) // no cancel button
			_createNewEmptyXMLFile(file);
		else
			mainFrame.setStatusBarActiveMessage(this, false);
	}

	private void _createNewEmptyXMLFile(final File file) {
		thisSideController.disableActions(true);
		thisSide.ensureSideIsVisible();
		thisSideController.setTypeXML();
		thisSideController.aSwitchToViewer.setSelected(true);

		final Dtd dtd = getXMLLanguage().getDtd();

		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			public Object construct() {
				try {
					File absoluteDTDFile = FileUtils.toAbsoluteFile(dtd.getNewFileName(), getXMLLanguage().getFile());
					String relativeDTDPath = FileUtils.toRelativeFile(absoluteDTDFile, file);
					thisSide.xmlView.createNewEmptyXMLFile(file, relativeDTDPath, dtd.getRootElement(), getXMLLanguage().getName(),
														   MainController.getCurrentUserName());
					otherSide.deltaView.setExternalMaxDELTALinkID(LinkIDConstants.FIRST_ID - 1);
				} catch (Throwable exc) {
					final String xmlFilePath = file.getAbsolutePath();
					System.err.println("\nCouldn't create XML file \"" + xmlFilePath + "\": ");
					exc.printStackTrace(System.err);
					return "Couldn't create file \"" + xmlFilePath + "\"!\n\n" + exc.getMessage();
				}
				return null;
			}

			public void finished() {
				Object errorMsg = get();
				if (errorMsg == null) {
					thisSide.setFileNameLabel(file);

					MainController.addRecentFile(MainController.recentXHTMLFiles, file);
					thisSideController.buildRecentFilesMenus();
					otherSideController.buildRecentFilesMenus();
					if (MainController.addRecentFile(MainController.recentDirs, file.getParentFile()))
						MainController.buildRecentDirsMenus();

					MainController.fileChanged(thisSideIndex);
				} else
					JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
				MainController.resetUndoManager(thisSideIndex);
				thisSideController.resetXMLTreeActions();
				thisSideController.enableActions();
				mainFrame.setStatusBarActiveMessage(XMLNewEmptyFileAction.this, false);
			}
		}.start();
	}
}
