/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XHTMLReloadFileAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:26:08
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.misc.XMLLanguageDialog;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLView;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.Dtd;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.Template;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import net.socialchange.doctype.Doctype;
import net.socialchange.doctype.DoctypeChangerReader;
import net.socialchange.doctype.DoctypeGenerator;

import java.io.File;
import java.net.URL;

import javax.swing.*;

/**
 * Class XHTMLReloadFileAction ... todo
 */
public class XMLNewFileFromTemplateAction extends BothSidesAction {
	private final Template template;
	private final XmlLanguage xmlLanguage;

	public XMLNewFileFromTemplateAction(int thisSideIndex, XmlLanguage xmlLanguage, Template template, int nr) {
		super(thisSideIndex);

		this.xmlLanguage = xmlLanguage;
		this.template = template;
		String label = template.getLabel();

		putValue(NAME, ""+(nr + 1) + ' ' + label);
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Creating new '"+ xmlLanguage.getName()+"' XML file...");  // when clicked

		Icon icon = XMLLanguageDialog.DEFAULT_TEMPLATE_ICON;
		URL url = FileUtils.toURL(template.getIconFileName(), xmlLanguage.getFile());
		if (url != null) {
			icon = new ImageIcon(url);
			if (icon.getIconWidth() < 10) icon = XMLLanguageDialog.DEFAULT_TEMPLATE_ICON;
		}
		template.setIcon(icon);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.MNEMONIC_KEY, new Integer('1' + nr));
		String statusBarText = "Create new '"+ xmlLanguage.getName()+"' XML file base on Template \"" + label + '\"';
		putValue(StatusBar.DESCRIPTION_HOVER, statusBarText); // when mouse over - statusbar
		String description = template.getDescription();
		if ((description == null) || (description.trim().length() == 0))
			putValue(Action.SHORT_DESCRIPTION, statusBarText);
		else
			putValue(Action.SHORT_DESCRIPTION, statusBarText + ":\n  " + description); // when mouse over - tool-tip
	}

	protected void actionPerformed() {
		thisSideController.setCurrentXMLLanguage(xmlLanguage, false, true);

		mainFrame.statusBar.unsetHoverMessageFrom(this);
		mainFrame.setStatusBarActiveMessage(this, true);
		if (thisSideController.saveIfChanged(true)) {
			thisSideController.setTypeXML();

            thisSide.fileChooser.setCurrentDirectory(
                    xmlLanguage.getFile().getParentFile());
            thisSide.fileChooser.resetChoosableFileFilters();
            final String langName = xmlLanguage.getName();
            thisSide.fileChooser.setFileFilter(xmlLanguage.getFileFilter());

            thisSide.fileChooser.setDialogTitle("Create new \"" + langName + "\" XML File as ...");
			if (thisSide.fileChooser.showSaveDialog(mainFrame) == JFileChooser.APPROVE_OPTION) {
				thisSideController.disableActions(true);
				thisSideController.aSwitchToViewer.setSelected(true);
				final File file = thisSide.fileChooser.getSelectedFile();
				final File oldFile = thisSide.deltaView.getFile();
				final String xmlFilePath = file.getAbsolutePath();
				final Action action = this;
                final Dtd dtd = xmlLanguage.getDtd();
                final String systemId = FileUtils.toRelativeFile(FileUtils.toAbsoluteFile(dtd.getNewFileName(), xmlLanguage.getFile()), file);

				new at.ac.tuwien.asgaard.swing.SwingWorker() {
					public Object construct() {
						try {
                            File sourceFile = FileUtils.toExistingAbsoluteFile(template.getFileName(), xmlLanguage.getFile().getParentFile());
                            if (sourceFile == null) return "Template \"" + template.getFileName() + "\" not found!";
                            FileUtils.copyFileToFile(sourceFile, file);
                            DoctypeChangerReader dtReader = new DoctypeChangerReader(file, new DoctypeGenerator() {
                                                            public Doctype generate(Doctype doctype) {
                                                                doctype.setSystemId(systemId);
                                                                return doctype; //new Doctype(dtd.getRootElement(), null, systemId, null);
                                                            }
                                                        });
                            thisSide.xmlView.load(file, dtReader, langName + " document created by " + MainController.getCurrentUserName() + " using DELT/A on " + XMLView.DATE_STRING);
                            if (dtReader.isDoctypeChanged())
                                thisSide.xmlView.contentHasChanged(true);
							
							otherSide.deltaView.setExternalMaxDELTALinkID(LinkIDConstants.FIRST_ID - 1);
                        } catch (Throwable exc) {
							System.err.println("\nCouldn't save file \"" + oldFile + "\" to \"" + xmlFilePath + "\": ");
							exc.printStackTrace(System.err);
							return "Couldn't save file \"" + oldFile + "\" to \"" + xmlFilePath + "\"!\n\n" + exc.getMessage();
						}
						return null;
					}
					public void finished() {
						Object errorMsg = get();
						if (errorMsg == null) {
							thisSide.setFileNameLabel(file);

							MainController.addRecentFile(MainController.recentXHTMLFiles, file);
							thisSideController.buildRecentFilesMenus();
							otherSideController.buildRecentFilesMenus();
							if (MainController.addRecentFile(MainController.recentDirs, file.getParentFile()))
								MainController.buildRecentDirsMenus();

							MainController.fileChanged(thisSideIndex);
						} else
							JOptionPane.showMessageDialog(mainFrame, errorMsg.toString(), "Error", JOptionPane.ERROR_MESSAGE);
						MainController.resetUndoManager(thisSideIndex);
						thisSideController.resetXMLTreeActions();
						thisSideController.enableActions();
						mainFrame.setStatusBarActiveMessage(action, false);
					}
				}.start();
			} else
				mainFrame.setStatusBarActiveMessage(this, false);
		} else
			mainFrame.setStatusBarActiveMessage(this, false);
	}
}
