/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XMLRemoveNodeAction
 * User: Peter
 * Date: 26.05.2003
 * Time: 03:04:38
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.SimpleBothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.MacroInsertUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideCompoundEdit;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.swing.StatusBar;

import java.awt.event.KeyEvent;
import java.util.Set;

import javax.swing.*;
import javax.swing.undo.UndoableEdit;

/**
 * Class XMLRemoveNodeAction ... todo
 */
public class XMLRemoveNodeAction extends SimpleBothSidesAction {
	public XMLRemoveNodeAction(int thisSideIndex) {
		super(thisSideIndex, "Remove Node", MainImages.REMOVE_XML_NODE_ICON);

		putValue(MNEMONIC_KEY, new Integer('R'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		putValue(StatusBar.TOOL_TIP, "Remove XML Node"); // when mouse over - tool-tip
		putValue(StatusBar.DESCRIPTION_HOVER, "Remove selected XML Node."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Removing node ...");  // when clicked
		enabled = false;
	}

	public void simpleActionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
		XMLTreeNode xmlTreeNode = thisSide.xmlView.getSelectedTreeNode();

		thisSide.deltaView.removeHighlights();
		otherSide.deltaView.removeHighlights();

		boolean xmlTreeNodeHasDELTALinkDescendant = xmlTreeNode.hasDELTALinkDescendant();
		if (MainController.askBeforeRemoveXMLLinks && xmlTreeNodeHasDELTALinkDescendant) {
			if (JOptionPane.showConfirmDialog(mainFrame, "Do you really want to remove DELT/A-Link(s) in the active file?", "Remove Node(s) / DELT/A-Link(s) ",
											  JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
				== JOptionPane.NO_OPTION)
				return;
		} else if (MainController.askBeforeRemoveXMLNodes
				   || (MainController.askBeforeRemoveXMLSubtree && !xmlTreeNode.isLeaf()))
			if (JOptionPane.showConfirmDialog(mainFrame, "Do you really want to remove the node '" + xmlTreeNode + '\''
														 + (xmlTreeNode.isLeaf() ? "" : " together with its children") + '?', "Remove Node(s)",
																															  JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
				== JOptionPane.NO_OPTION)
				return;

        RemoveNodeWrapperUndoableEdit wrapperEdit = new RemoveNodeWrapperUndoableEdit(xmlTreeNode, xmlTreeNodeHasDELTALinkDescendant);
        thisSideController.fireDirectUndoableEditHappened(wrapperEdit);

		final Set<Integer> removedDELTALinks = thisSide.xmlView.removeCurrentTreeNode();

		thisSide.deltaView.removeDELTALinksFromCollection(removedDELTALinks, true);   // remove links from gmtLinks which are in this XML view
		otherSide.deltaView.removeDELTALinksFromCollection(removedDELTALinks, false);  // remove links from gmtLinks which are not in the other DELTA view

		if (!removedDELTALinks.isEmpty()) {
            switch (MainController.removeDeadLinks) {
                case REMOVE_LINKS_ASK:
                    if (JOptionPane.showConfirmDialog(mainFrame, "Do you want to remove " + removedDELTALinks.size() + " corresponding dead link(s) "
                            + ((removedDELTALinks.size() > 10) ? "" : "(IDs=" + removedDELTALinks.toString() + ") ") + "in the opposite file?",
                            "Remove dead links", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
                            == JOptionPane.NO_OPTION)
                        break;
                case REMOVE_LINKS_YES:
                    otherSide.deltaView.removeDELTALinks(removedDELTALinks);
                    break;
            }
        }
		thisSide.deltaView.requestFocusInWindow();

        wrapperEdit.end();
        MainController.updateUndoActions();
	}

    private static final class RemoveNodeWrapperUndoableEdit extends SideCompoundEdit {
        public RemoveNodeWrapperUndoableEdit(XMLTreeNode xmlTreeNode, boolean affectsBothSides) {
			super(affectsBothSides);
			String shortDescription = xmlTreeNode.getShortDescription();
            undoPresentationName = "Undo - \"" + shortDescription + "\" removal";
            redoPresentationName = "Redo - \"" + shortDescription + "\" removal";
        }

        public boolean addEdit(UndoableEdit anEdit) {
            if (anEdit instanceof MacroInsertUndoableEdit)
                return super.addEdit(anEdit);
            return false;
        }
    }
}
