/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 19.05.2003
 * Time: 16:10:30
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.MainFrame;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.SwingWorker;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import javax.swing.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;

/**
 * Class XMLViewShowDTDReportAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.4.0
 */
public class XMLViewShowDTDReportAction extends BothSidesAction {
	public XMLViewShowDTDReportAction(int thisSideIndex) {
		super(thisSideIndex, "DTD - Show Report", MainImages.HTML_VIEW_ICON);

		putValue(MNEMONIC_KEY, new Integer('T'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F9, KeyEvent.ALT_MASK));
		putValue(SHORT_DESCRIPTION, "Show DTD report view"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Show DTD-Report view."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Showing DTD-Report view ...");  // when clicked
		enabled = false;
	}
	protected void actionPerformed() {
		final MainFrame mainFrame = MainController.mainFrame;
		mainFrame.statusBar.setActiveMessageFrom(this);
	// thisSide.switchToSimpleHTMLView();
		thisSideController.aSwitchToHTMLOutput.setEnabled(true);
		thisSideController.aSwitchToHTMLOutput.setSelected(true);
		thisSideController.aGoBack.setEnabled(true);
		thisSideController.aGoForward.setEnabled(true);
		if (thisSideController.sourceViewNeedsUpdateDTDReport) {
			thisSide.htmlOutput.setBackground(Color.WHITE);
			thisSide.htmlOutput.setText("Showing HTML report of large or remote DTD files may take a while ...");
			new at.ac.tuwien.asgaard.swing.SwingWorker() {
{
	ClassLoader loader = XMLViewShowDTDReportAction.this.getClass().getClassLoader();
	Thread.currentThread().setContextClassLoader(loader);
}
				public Object construct() {
					Log.trace("Constructing DTD-Report HTML file ...");
					URL dtdurl = thisSide.xmlView.getDTDURL();
					if (dtdurl == null) return "No or bad DTD entry in DOCTYPE statement!";
					File tmpFile = new File(FileUtils.currentDirFile, FileUtils.getFileName(dtdurl.getPath())+"-Report_tmp.html");
					if (!tmpFile.exists()) {
						tmpFile.deleteOnExit();
						try {
							XMLUtils.transformDTD2Report(dtdurl, new BufferedWriter(new FileWriter(tmpFile)));
						} catch (Exception e) {
							e.printStackTrace();
							return XMLUtils.HTML_ERROR_MSG;
						}
					} // todo put caching in own method of XMLUtils
				// return XMLUtils.transformDTD2Report(dtdurl);
					return tmpFile;
				}
				public void finished() {
					Log.trace("Showing DTD-Report HTML file ...");
					Object o = get();
					if (o instanceof String)
						thisSide.htmlOutput.setText((String)o);
					else
						try {
							thisSide.htmlOutput.setPage(((File)o).toURI().toURL());
						} catch (IOException e) { }
					thisSide.htmlOutput.setCaretPosition(0);
					thisSideController.sourceViewNeedsUpdateDTDReport = false;
					thisSideController.sourceViewNeedsUpdateXHTMLSource = thisSideController.sourceViewNeedsUpdateDTDSource = true;
					mainFrame.statusBar.unsetActiveMessageFrom(XMLViewShowDTDReportAction.this);
					Log.trace("Finished Constructing & Showing DTD-Report HTML file.");
				}
			}.start();
		} else
			mainFrame.statusBar.unsetActiveMessageFrom(this);
	}
}
