/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 19.05.2003
 * Time: 16:10:30
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.MainFrame;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.awt.event.KeyEvent;
import java.io.*;
import java.net.URL;

import javax.swing.*;

/**
 * Class MacroSwitchToTextViewAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class XMLViewShowPlainDTDSourceAction extends BothSidesAction {
	public XMLViewShowPlainDTDSourceAction(int thisSideIndex) {
		super(thisSideIndex, "DTD - Show Plain Source", MainImages.TEXT_VIEW_ICON);

		putValue(MNEMONIC_KEY, new Integer('S'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F9, KeyEvent.SHIFT_MASK));
		putValue(SHORT_DESCRIPTION, "Show DTD text source view"); // when mouse over
		putValue(StatusBar.DESCRIPTION_HOVER, "Show plain text DTD source view."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Showing plain text DTD source view ...");  // when clicked
		enabled = false;
	}
	protected void actionPerformed() {
		if (!thisSideController.isTypeXML()) return;
		final MainFrame mainFrame = MainController.mainFrame;
		mainFrame.statusBar.setActiveMessageFrom(this);
		thisSideController.aGoBack.setEnabled(false);
		thisSideController.aGoForward.setEnabled(false);
		thisSideController.aSwitchToTextOutput.setEnabled(true);
		thisSideController.aSwitchToTextOutput.setSelected(true);
		thisSide.textOutput.setText("Showing source of large or remote DTD files may take a while ...");
		new at.ac.tuwien.asgaard.swing.SwingWorker() {
			public Object construct() {
				StringWriter sw = new StringWriter();
				try {
					URL dtdurl = thisSide.xmlView.getDTDURL();
					if (dtdurl == null) return "No or bad DTD entry in DOCTYPE statement!";
					InputStream inputStream = dtdurl.openStream();
					FileUtils.pipeReaderToWriter(new BufferedReader(new InputStreamReader(inputStream)), sw);
					inputStream.close();
				} catch (IOException exc) {
					exc.printStackTrace();
					return "Couldn't show source of associated DTD!";
				}
				return sw.toString();
			}
			public void finished() {
				thisSide.textOutput.setText((String)get());
				thisSide.textOutput.setCaretPosition(0);
				thisSideController.textViewNeedsUpdateDTDSource = false;
				thisSideController.textViewNeedsUpdateXHTMLSource = true;
				mainFrame.statusBar.unsetActiveMessageFrom(XMLViewShowPlainDTDSourceAction.this);
			}
		}.start();

	}
}
