/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 08.07.2003
 * Time: 11:09:57
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml.view;

import at.ac.tuwien.asgaard.swing.selectable.RadioButtonAbstractAction;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.FilterDef;
import at.ac.tuwien.asgaard.delta.gui.misc.XMLLanguageDialog;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.DeltaSideController;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLView;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import java.net.URL;
import java.util.List;

/**
 * Class XMLTreeFilterApplyPredefined ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class XMLTreeFilterApplyPredefined extends RadioButtonAbstractAction {
	private final FilterDef filterDef;
	private final DeltaSideController thisSideController;

	public XMLTreeFilterApplyPredefined(int thisSideIndex, FilterDef filterDef, int nr) {
		this.filterDef = filterDef;
		thisSideController = MainController.deltaSideController[thisSideIndex];

		Icon icon = XMLLanguageDialog.DEFAULT_FILTER_ICON;
		URL url = FileUtils.toURL(filterDef.getIconFileName(), thisSideController.currentXMLLanguage.getFile());
		if (url != null) {
			icon = new ImageIcon(url);
			if (icon.getIconWidth() < 10) icon = XMLLanguageDialog.DEFAULT_FILTER_ICON;
		}
		filterDef.setIcon(icon);
		String label = filterDef.getLabel();

		putValue(NAME, ""+(nr + 1) + ' ' + label);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.MNEMONIC_KEY, new Integer('1' + nr));
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Applying Filter definition ...");  // when clicked

		String statusBarText = "Apply Filter definition \"" + label + '\"';
		putValue(StatusBar.DESCRIPTION_HOVER, statusBarText); // when mouse over - statusbar
		String description = filterDef.getDescription();
		if ((description == null) || (description.trim().length() == 0))
			putValue(Action.SHORT_DESCRIPTION, statusBarText);
		else
			putValue(Action.SHORT_DESCRIPTION, "<html>" +statusBarText + ":<br>&nbsp;&nbsp;" + description); // when mouse over - tool-tip
	}

	public void stateChanged(ChangeEvent evt) {
		MainController.mainFrame.statusBar.unsetHoverMessageFrom(this);
		thisSideController.aSwitchToViewer.setSelected(true);
		XMLView thisXMLView = thisSideController.thisSide.xmlView;
		try {
			List domNodeList = XMLUtils.evaluateXPath(filterDef.getXPathExpr(), thisXMLView.getSelectedTreeNode().getXMLNode()); //thisXMLView.getDocumentNode());

			thisXMLView.activateFilter(domNodeList, filterDef.getType().equalsIgnoreCase("show"));
			thisSideController.thisSide.visualizeFilterState(true, true);

			thisSideController.aGoBack.setEnabled(false);
			thisSideController.aGoForward.setEnabled(false);
		} catch (Exception exc) {
			JOptionPane.showMessageDialog(MainController.mainFrame,
										  "Couldn't evaluate XPath-Expression:\n\n" + filterDef.getXPathExpr() + "\n\n" + exc.getMessage(),
										  "XPath Error", JOptionPane.ERROR_MESSAGE);
		}
	}
}
