/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 26.06.2003
 * Time: 12:20:16
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml.view;

import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.BothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.misc.XPathExpressionEditDialog;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.*;
import at.ac.tuwien.asgaard.swing.autoComplete.AutoCompleteComboBoxEditDialog;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import javax.swing.*;
import java.util.List;
import java.awt.Component;

/**
 * Class XMLTreeFilterShowXPath provides ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class XMLTreeFilterShowXPath extends BothSidesAction {
    private AutoCompleteComboBoxEditDialog dialog;
	private XPathExpressionEditDialog xPathExpressionEditDialog;

	public XMLTreeFilterShowXPath(int thisSideIndex) {
        super(thisSideIndex, "Show only result of XPath ...", MainImages.EDIT_ICON);
        putValue(MNEMONIC_KEY, new Integer('S'));
        putValue(StatusBar.TOOL_TIP, "Show only result of XPath-Expression"); // when mouse over
        putValue(StatusBar.DESCRIPTION_HOVER, "Evaluate XPath-Expression and show only resulting tree nodes."); // when mouse over - statusbar
        enabled = false;
    }

	protected void actionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);

        if (dialog == null) {
			dialog = new AutoCompleteComboBoxEditDialog(MainController.mainFrame, "Filter Treenodes...", "XPath-Expression: ",
														MainController.getSharedXPathList()) {
				protected TextEditDialog getTextEditDialog(Component parentComponent, String title) {
					if (xPathExpressionEditDialog == null)
						xPathExpressionEditDialog = new XPathExpressionEditDialog(parentComponent, null);
					xPathExpressionEditDialog.init(thisSideController.dtdHelper);
					return xPathExpressionEditDialog;
				}
			};
			dialog.setIgnoreCase(false);
		}
		XMLTreeNode selectedTreeNode = thisSide.xmlView.getSelectedTreeNode();
		String xPathExpression = selectedTreeNode.getSimpleXPath();
		if (xPathExpression == null) xPathExpression = "/"; // DOCUMENT_NODE and DOCUMENT_TYPE_NODE nodes return null on getSimpleXPath()
		xPathExpression = dialog.showEditDialog(xPathExpression);
		if (xPathExpression != null) {
			xPathExpression = xPathExpression.trim();
			if (xPathExpression.equals(""))
				thisSide.xmlView.showAllTreeNodes();
			else
				try {
					List domNodeResultList = XMLUtils.evaluateXPath(xPathExpression, selectedTreeNode.getXMLNode());
					if (domNodeResultList.isEmpty())
                        JOptionPane.showMessageDialog(mainFrame, "No matching nodes found for XPath-Expression \n\"" + xPathExpression + "\"",
                                                      "No result", JOptionPane.INFORMATION_MESSAGE);
                    else
                        thisSide.xmlView.activateFilter(domNodeResultList, true);
					
					thisSideController.thisSide.visualizeFilterState(true, true);
					thisSideController.aGoBack.setEnabled(false);
					thisSideController.aGoForward.setEnabled(false);
					thisSideController.buildFilterMenus("<html>show only result of XPath:<br>  " + xPathExpression);
                    MainController.stateProperties.putStringList(Preferences.KEY_COMBOBOX_XPATH, dialog.getElements());
				} catch (Exception exc) {
					JOptionPane.showMessageDialog(mainFrame,
												  "Couldn't evaluate XPath-Expression:\n\n" + xPathExpression + "\n\n" + exc.getMessage(),
												  "XPath Error", JOptionPane.ERROR_MESSAGE);
				}
		}
	}
}
