/*
 * Project: Asgaard - Guideline Markup Tool
 * Class: XMLTreeHighlightXPath
 * User: Peter
 * Date: 18.06.2003
 * Time: 16:51:53
 */

package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml.view;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.gui.actions.xhtml.SimpleBothSidesAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import net.sf.saxon.Configuration;
import net.sf.saxon.om.DocumentInfo;
import net.sf.saxon.query.DynamicQueryContext;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;
import org.w3c.dom.Node;

import javax.swing.*;
import javax.swing.tree.TreeNode;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.List;

/**
 * Class XMLTreeHighlightXPath ... todo
 */
public class XMLTreeHighlightXQuery extends SimpleBothSidesAction {
    private TextEditDialog dialog;
    private Configuration config;

	public XMLTreeHighlightXQuery(int thisSideIndex) {
		super(thisSideIndex, "Highlight result of XQuery...", MainImages.HIGHLIGHT_ICON);
        putValue(MNEMONIC_KEY, new Integer('E'));
		putValue(LOCAL_ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.SHIFT_MASK | InputEvent.ALT_MASK | InputEvent.CTRL_MASK));
        putValue(StatusBar.TOOL_TIP, "Highlight result of XQuery"); // when mouse over
        putValue(StatusBar.DESCRIPTION_HOVER, "Evaluate XQuery and highlight resulting tree nodes."); // when mouse over - statusbar
		putValue(StatusBar.DESCRIPTION_ACTIVE, "Highlighting result of XQuery...");  // when clicked
		enabled = false;
	}

	protected void simpleActionPerformed() {
		thisSideController.aSwitchToViewer.setSelected(true);
        
        if (dialog == null)
            dialog = new TextEditDialog(MainController.mainFrame, "Highlight Treenodes using XQuery ...");

		
        String xqueryString = dialog.showEditDialog();
		if (xqueryString != null) {
			xqueryString = xqueryString.trim();
			if (xqueryString.equals(""))
                thisSide.xmlView.removeHighlights();
            else
				try {
				    if (config == null) {
				        config = new Configuration();
				        config.setHostLanguage(Configuration.XQUERY);
				    }
				    StaticQueryContext staticContext = new StaticQueryContext(config);
			        Source sourceInput = new DOMSource(thisSide.xmlView.getDocumentNode());
					
					DynamicQueryContext dynamicEnv = new DynamicQueryContext(config);
			        XQueryExpression exp = staticContext.compileQuery(xqueryString);
			        DocumentInfo docInfo = staticContext.buildDocument(sourceInput);

					/*QueryProcessor queryProcessor = new QueryProcessor(staticContext);
					XQueryExpression exp = queryProcessor.compileQuery(xqueryString);
					DynamicQueryContext dynamicEnv = new DynamicQueryContext();
					DocumentInfo docInfo = queryProcessor.buildDocument(sourceInput);*/

					dynamicEnv.setContextItem(docInfo);
			        List resultList = exp.evaluate(dynamicEnv);
			        
			        String resultString = null;
					List<XMLTreeNode> treeNodeResultList = thisSide.xmlView.convertXMLNodeListToTreeNodeList(resultList);

					int nodeCount = treeNodeResultList.size(), valueCount = 0;
					if (resultList.size() != nodeCount) {
						StringBuffer sb = new StringBuffer();
						for (int i = 0, size = resultList.size(); i < size; i++) {
							Object o = resultList.get(i);
							if (!(o instanceof Node)) {
								if (sb.length() > 0) sb.append(", ");
								sb.append(o);
								valueCount++;
								nodeCount--;
							}
						}
						if (valueCount > 0) {
							if (valueCount > 1) {
								sb.insert(0, '(');
								sb.append(')');
							}
							resultString = sb.toString();
						}
					}

					if (treeNodeResultList.isEmpty())
						if (!resultList.isEmpty())
							if (nodeCount > 0)
								if (valueCount > 0)
									JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n  "+ nodeCount+" Node(s) (hidden due to active filters)"
											+ "\n    " + valueCount + " Value(s): "+resultString,
											"Mixed result", JOptionPane.INFORMATION_MESSAGE);
								else
									JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    "+ nodeCount+" Node(s) (hidden due to active filters)",
														  "No visible result", JOptionPane.INFORMATION_MESSAGE);
							else
								JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    " + valueCount + " Value(s): " + resultString,
										"Value(s) as result", JOptionPane.INFORMATION_MESSAGE);
						else
							JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression \n\"" + xqueryString + "\" evaluates to no result.",
													  "No result", JOptionPane.INFORMATION_MESSAGE);
					else
						if (nodeCount == 1) {
							thisSide.xmlView.centerAndSelectTreeNode((TreeNode) treeNodeResultList.get(0));
							if (valueCount > 0)
								JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    1 Node"
										+ "\n  " + valueCount + " Value(s): " + resultString,
										"Mixed result", JOptionPane.INFORMATION_MESSAGE);
							else
								JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    1 Node",
										"Node as result", JOptionPane.INFORMATION_MESSAGE);
						} else {
							thisSide.xmlView.highlightTreeNodes(treeNodeResultList);
							if (valueCount > 0)
								JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    " + nodeCount + " Node(s)"
										+ "\n  " + valueCount + " Value(s): " + resultString,
										"Mixed result", JOptionPane.INFORMATION_MESSAGE);
							else
								JOptionPane.showMessageDialog(mainFrame, "XQuery-Expression evaluates to ...\n    " + nodeCount + " Node(s)",
										"Node(s) as result", JOptionPane.INFORMATION_MESSAGE);
						}
				} catch (XPathException err) {
	                int line = -1;
	                if (err.getLocator() != null) {
	                    line = err.getLocator().getLineNumber();
	                }
	                if (line == -1) {
	                    System.err.println("Failed to compile xquery: " + err.getMessage());
	                } else {
	                    System.err.println("Static error at line " + line + ":");
	                    System.err.println(err.getMessage());
	                }
					JOptionPane.showMessageDialog(mainFrame,
							"Couldn't compile XQuery-Expression:\n\n" + xqueryString,
							"XQuery Error", JOptionPane.ERROR_MESSAGE);
				} catch (TransformerException err) {
				    JOptionPane.showMessageDialog(mainFrame,
							"Couldn't evaluate XQuery-Expression:\n\n" + xqueryString,
							"XQuery Error", JOptionPane.ERROR_MESSAGE);
				}
		}
	}
}
