/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 08.07.2003
 * Time: 11:09:57
 */
package at.ac.tuwien.asgaard.delta.gui.actions.xhtml.xml.view;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideController;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.misc.XMLLanguageDialog;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.SummarizeDef;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.selectable.RadioButtonAbstractAction;
import at.ac.tuwien.asgaard.util.FileUtils;

import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.net.URL;

import javax.swing.*;
import javax.swing.event.*;

/**
 * Class XMLTreeFilterApplyPredefined ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class XMLTreeSummarizeApplyPredefinedAction extends RadioButtonAbstractAction {
	private final SummarizeDef summarizeDef;
	private final DeltaSideController thisSideController;

	public XMLTreeSummarizeApplyPredefinedAction(int thisSideIndex, SummarizeDef summarizeDef, int nr) {
		this.summarizeDef = summarizeDef;
		thisSideController = MainController.deltaSideController[thisSideIndex];

		Icon icon = XMLLanguageDialog.DEFAULT_SUMMARIZE_ICON;
		URL url = FileUtils.toURL(summarizeDef.getIconFileName(), thisSideController.currentXMLLanguage.getFile());
		if (url != null) {
			icon = new ImageIcon(url);
			if (icon.getIconWidth() < 10) icon = XMLLanguageDialog.DEFAULT_SUMMARIZE_ICON;
		}
		summarizeDef.setIcon(icon);
		String label = summarizeDef.getLabel();

		putValue(NAME, ""+(nr + 1) + ' ' + label);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.MNEMONIC_KEY, new Integer('1' + nr));
		String statusBarText = "Apply Summarize definition \"" + label + '\"';
		putValue(StatusBar.DESCRIPTION_HOVER, statusBarText); // when mouse over - statusbar
		String description = summarizeDef.getDescription();
		if ((description == null) || (description.trim().length() == 0))
			putValue(Action.SHORT_DESCRIPTION, statusBarText);
		else
			putValue(Action.SHORT_DESCRIPTION, "<html>" +statusBarText + ":<br>&nbsp;&nbsp;" + description); // when mouse over - tool-tip
	}

	public void stateChanged(ChangeEvent evt) { }

	public void actionPerformed(ActionEvent evt) {
		super.actionPerformed(evt);
		MainController.mainFrame.statusBar.unsetHoverMessageFrom(this);
		thisSideController.aSwitchToViewer.setSelected(true);
		File xslFile = FileUtils.toAbsoluteFile(summarizeDef.getXslFileName(), thisSideController.currentXMLLanguage.getFile());
		try {
			thisSideController.thisSide.xmlView.applySummarizeXSLFile(xslFile);
		} catch (IOException exc) {
			exc.printStackTrace();
			JOptionPane.showMessageDialog(MainController.mainFrame,
										  "Couldn't load XSL-File \"" + xslFile.getAbsolutePath() + "\":\n\n" + exc.getMessage(),
										  "IO Error", JOptionPane.ERROR_MESSAGE);
		}
	}
}
