package at.ac.tuwien.asgaard.delta.gui.html;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideView;
import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.delta.gui.html.imageMap.ImageMapEditorDialog;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaHyperlinkEvent;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.LinkIDChangedUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.LinkInsertedUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.LinksRemovedUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideAbstractUndoableEdit;
import at.ac.tuwien.asgaard.delta.util.GlobalConstants;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import org.w3c.dom.Node;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.text.DecimalFormat;
import java.util.*;
import java.util.List;
import java.util.Map;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.text.*;
import javax.swing.text.html.*;
import javax.swing.undo.*;

// Note: Element#getEndOffset() is the position of the last character of this element PLUS 1
//   => length of element = Element#getEndOffset() - Element#getStartOffset()

/**
 * HTMLView ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class HTMLView extends JTextPane implements DeltaSideView, Printable {
	public static final	int	LINKS_NO_CHECK			 = 0;
	public static final	int	LINKS_CHECK_ONACTIVATION = 1;
	public static final	int	LINKS_CHECK_ONHOVER		 = 2;
	public static final	int	LINKS_CHECK_ONLOAD		 = 3;

	private boolean prefUseLinkSeparator = true;
	private String prefLinkSeparator1, prefLinkSeparator2;

	private Color prefLinkDelimiterColor;
	String prefLinkTextColor = "#0080D0", prefLinkSetTextColor = "#0070C0";
	private static final int CHECK_URL_TIMEOUT = 2000;

	private	static final String	GMT_LINK_ID_PREFIX	 = "gmt:";
	private	static final String	DELTA_LINK_ID_PREFIX = "delta:";

	private static final DecimalFormat LINK_ID_FORMAT = new DecimalFormat(DELTA_LINK_ID_PREFIX + "0"); // no leading 0 anymore
	private static final SimpleAttributeSet attributeSetTagA;

//	private ExtendedHighlightPainter highlightPainter;

	private File htmlFile;
	private boolean contentHasChanged;
	private final Preferences preferences;

	private ExtendedHTMLEditorKit kit;
	private Element currentElement;
	private LinkedList<HistoryEntry> historyBackList, historyForwardList;
	private URL currentURL;

	private int prefCheckLinks;

	private Map<String, Boolean> viewableURLs;
	private List highlights, highlightedElements;
	private int currentDELTALinkID, currentHighlightIndex;
	private Color prefLinkHighlightColor;
	private boolean isLoading;

	private ImageMapEditorDialog imageMapEditorDialog;

	private static final String ID_LIST_SEPARATOR = ",";

	static final String LINK_SEPARATOR_CLASS_ATTR = "sep";
	static final String LINK_SEPARATOR_CLASS_ATTR_START = "sep1";
	static final String LINK_SEPARATOR_CLASS_ATTR_END = "sep2";
	protected ExtendedHTMLDocument doc;

	private int currentPage = 0; private double pageStartY, pageEndY;

	/** Creates new HTMLView */
	public HTMLView(Preferences preferences) {
		super();
		Log.trace("enter: HTMLView()");

		setEditable(false);
		setEnabled(false);
		setSelectedTextColor(Color.BLACK);
		isLoading = false;
		contentHasChanged = false;
		currentDELTALinkID = Integer.MIN_VALUE;
		searchStartPos = -1;

		kit = new ExtendedHTMLEditorKit(preferences, this);
		setEditorKit(kit);

		this.preferences = preferences;
		setProperties();
		setFocusTraversalKeysEnabled(false);

		setCaret(new DefaultCaret() {  // override DefaultCarets focusLost to hide only caret and not selection
			public void focusLost(FocusEvent evt) {
				this.setVisible(false); // not: setSelectionVisible(false);
			}
		});
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				if (doc == null) return;
				currentElement = null;
				if (SwingUtilities.isMiddleMouseButton(evt)) {
					if (evt.isControlDown()) {
						kit.showLinks = !kit.showLinks;
						removeHighlights();
						repaint();
					} else
						if (highlightedElements == null)
							scrollToRange(getCaret().getDot(), getCaret().getMark());
						else
							if ((evt.getModifiers() & InputEvent.SHIFT_MASK) == InputEvent.SHIFT_MASK)
								jumpToPreviousHighlight();
							else
								jumpToNextHighlight();
				} else {
					if (SwingUtilities.isLeftMouseButton(evt))
						removeHighlights();
					if (!isSelected()) {
						int pos = viewToModel(evt.getX(), evt.getY());
						if (pos >= 0) {
							currentElement = doc.getCharacterElement(pos);
							if (GlobalConstants.DEBUG) {
								AttributeSet as = currentElement.getAttributes();
								if (as.getAttribute(StyleConstants.NameAttribute) == HTML.Tag.IMG) {
								// ImageView view = kit.getView(currentElement);
									Object imgsrc = as.getAttribute(HTML.Attribute.SRC);
									System.out.println("clicked on image " + imgsrc ); // + " (alt=" + view.getAltText() + ")");  // DEBUG
									if (imageMapEditorDialog == null)
										imageMapEditorDialog = new ImageMapEditorDialog(HTMLView.this);
									imageMapEditorDialog.setImage(FileUtils.toAbsoluteFilePath(imgsrc.toString(), htmlFile));
									imageMapEditorDialog.setVisible(true);
								}
							}
						}
					}
				}
				currentDELTALinkID = Integer.MIN_VALUE;
			}
		});
		Log.trace("exit: HTMLView()");
	}

//  I/O Methods
	public void load(File htmlFile) throws IOException {
		Log.trace("enter: HTMLView.load()");
		if (doc == null) {
	//  setBackground(Color.white);
			viewableURLs = new HashMap<String, Boolean>();
			historyBackList = new LinkedList<HistoryEntry>();
			historyForwardList = new LinkedList<HistoryEntry>();
		}
		this.htmlFile = htmlFile;  // have to be set before contentHasChanged(false); is called
		load(htmlFile.toURI().toURL(), true);
		// Have to scroll after painted.
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// reset scrollbar
				scrollRectToVisible(GlobalObjects.TOP_RECT);
				// setBackground(kit.bgColor);
			}
		});
		historyBackList.clear();
		historyForwardList.clear();
		Log.trace("exit:  HTMLView.load()");
	}

	protected void load(URL url, boolean forceReload) throws IOException {
		if (forceReload || (currentURL == null) || !currentURL.sameFile(url)) {
			setEnabled(false);
			isLoading = true;
			// always create a new document
			doc = kit.createDocument();
			setDocument(doc);
			removeHighlights();
			try {
				InputStream in = getStream(url);
				doc.remove(0, doc.getLength());
				doc.setBase(url);  // do I really need that??? yes: doc.getBase() appears in javax.swing.text.html.HTMLEditorKit.LinkController.createHyperlinkEvent()
				kit.read(new BufferedReader(new InputStreamReader(in)), doc, 0);
				currentURL = url;
				contentHasChanged(false);
			} catch (Exception exc) {
				exc.printStackTrace();
				throw new IOException(exc.getMessage());
			} finally {
				isLoading = false;
				setEnabled(true);
			}
		}
	}

	public void reload() throws IOException {
		unselect();
		removeHighlights();
		final Point currentViewPos = ((JViewport) getParent()).getViewPosition();

		load(currentURL, true);

		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				((JViewport) getParent()).setViewPosition(currentViewPos);
			}
		});
	}

	public void save() throws IOException {
		saveAs(htmlFile);
	}

	public void saveAs(File newHTMLFile) throws IOException {
		setEnabled(false);
		try {
			Writer out = null;
			try {
				out = new BufferedWriter(new FileWriter(newHTMLFile));
				write(out);
			} finally {
				if (out != null) out.close();
			}
			htmlFile = newHTMLFile;
			contentHasChanged(false);
		} finally {
			setEnabled(true);
		}
	}

	public void close() {
		if (htmlFile != null) {
			try {
				doc.remove(0, doc.getLength());
			} catch (BadLocationException e) {}
			historyBackList.clear();
			historyForwardList.clear();
			setBackground(GlobalObjects.DEFAULT_BACKGROUND);
			if (highlights != null) highlights.clear();
			highlightedElements = null;
			currentHighlightIndex = -1;
			kit.deltaLinkMap.clear();
			kit.maxLinkID = LinkIDConstants.FIRST_ID - 1;
			htmlFile = null;
			contentHasChanged(false);
		}
	}

	public boolean print() {
		// TODO implement print
		PrinterJob printJob = PrinterJob.getPrinterJob();
		printJob.setPrintable(this);
		if (printJob.printDialog())
			try {
				printJob.print();
				return true;
			} catch (PrinterException pe) {
				System.out.println("Error printing: " + pe);
			}
		return false;
	}
	public int print(Graphics g, PageFormat fmt, int index) {
		Graphics2D g2d = (Graphics2D) g;

// Set default foreground color to black
		g2d.setColor(Color.black);

// For faster printing, turn off double buffering
		RepaintManager.currentManager(this).setDoubleBufferingEnabled(false);

// Shift Graphic to line up with beginning of print-imageable region
		g2d.translate(fmt.getImageableX(), fmt.getImageableY());

		View rootView = getUI().getRootView(this);

		if (index > currentPage) {
			currentPage = index;

			pageStartY += pageEndY;
			pageEndY = g2d.getClipBounds().getHeight();
		}

		g2d.translate(g2d.getClipBounds().getX(), g2d.getClipBounds().getY());

		Rectangle allocation = new Rectangle(0, (int) -pageStartY, getWidth(), getHeight());

		if (printView(g2d, allocation, rootView))
			return Printable.PAGE_EXISTS;
		else {
			currentPage = -1;

			pageStartY = 0.0;
			pageEndY = 0.0;

			return Printable.NO_SUCH_PAGE;
		}
	}

	protected boolean printView(Graphics2D g2d, Shape alloc, View view) {
		boolean pageExists = false;

		Rectangle clip = g2d.getClipBounds();

		Shape childAlloc;
		View childView;

		if (view.getViewCount() > 0) {
			for (int i = 0; i < view.getViewCount(); i++) {
				childAlloc = view.getChildAllocation(i, alloc);

				if (childAlloc != null) {
					childView = view.getView(i);

					if (printView(g2d, childAlloc, childView)) {
						pageExists = true;
					}
				}
			}
		} else {
// I
			if (alloc.getBounds().getMaxY() >= clip.getY()) {
				pageExists = true;
// II
				if ((alloc.getBounds().getHeight() > clip.getHeight()) && (alloc.intersects(clip))) {
				// System.out.println("Calling view.paint()... (1)");
					view.paint(g2d, alloc);
				} else {
// III
					if (alloc.getBounds().getY() >= clip.getY()) {
						if (alloc.getBounds().getMaxY() <= clip.getMaxY()) {
				//  System.out.println("Calling view.paint()... (2)");
				//  System.out.println(view.getClass().getName());
				//  System.out.println(view.getElement().getName());
							view.paint(g2d, alloc);
						} else {
// IV
							if (alloc.getBounds().getY() < pageEndY) {
								pageEndY = alloc.getBounds().getY();
							}
						}
					}
				}
			}
		}

		return pageExists;
	}

	public boolean isLoaded() {
		return htmlFile != null;
	}

	public boolean isLoading() {
		return isLoading;
	}

	protected InputStream getStream(URL page) throws IOException {
		URLConnection conn = page.openConnection();
		if (conn instanceof HttpURLConnection) {
			HttpURLConnection hconn = (HttpURLConnection) conn;
			hconn.setInstanceFollowRedirects(false);
			int response = hconn.getResponseCode();
			boolean redirect = (response >= 300 && response <= 399);
			// In the case of a redirect, we want to actually change the URL
			// that was input to the new, redirected URL
			if (redirect) {
				String loc = conn.getHeaderField("Location");
				if (loc.startsWith("http", 0))
					page = new URL(loc);
				else
					page = new URL(page, loc);
				return getStream(page);
			}
		}
		return conn.getInputStream();
	}

// DELTA - Link Methods
	public boolean removeDELTALink(int xmlLinkID) {
		unselect();
		List<Element> elements = kit.deltaLinkMap.get(xmlLinkID);
		if ((elements == null) || elements.isEmpty()) return false;
		Iterator<Element> iter = elements.iterator();
		for (int i = elements.size(); i > 0; i--) {
			Element element = iter.next();
            removeDELTALLink(element, true);
			iter.remove();
        }
		contentHasChanged(true);
        return true;
	}

	public void removeDELTALinks(Set<Integer> xmlLinks) {
		setEnabled(false);
		Iterator<Integer> iter = xmlLinks.iterator();
		for (int i = xmlLinks.size() - 1; i >= 0; i--)
            removeDELTALink(iter.next().intValue());
		setEnabled(true);
	}

	public Set<Integer> removeSelectedDELTALinks() {
		int start = getSelectionStart();
		int end = getSelectionEnd();
		unselect();
		if (start == end) // nothing selected?
			if (currentElement != null) {
				start = currentElement.getStartOffset();
				end = currentElement.getEndOffset();
				currentElement = null;
			} else
				return null;

		Set<Integer> result = new HashSet<Integer>();
		for (int pos = start; pos < end; ) { // Note: (pos < end) and not (pos <= end) because both getSelectionEnd() and getEndOffset() return the position of the next character
			Element linkElem = doc.getCharacterElement(pos);
			Integer deltaLinkID = removeDELTALLink(linkElem, true);
			if (deltaLinkID.intValue() >= LinkIDConstants.FIRST_ID) {
                result.add(deltaLinkID);
				kit.deltaLinkMap.remove(deltaLinkID, linkElem);
				if (kit.deltaLinkMap.isEmpty())
					break;
             }
			pos = linkElem.getEndOffset();
		}
		if (!result.isEmpty())
			contentHasChanged(true);
		return result;
	}

	private Integer removeDELTALLink(Element linkElem, boolean fireUndoableEditEvent) {
		Integer deltaLinkID = getDELTALinkID(linkElem);
		if (deltaLinkID.intValue() >= LinkIDConstants.FIRST_ID) {
			if (kit.deltaLinkMap.isEmpty())
				kit.maxLinkID = LinkIDConstants.FIRST_ID - 1;
			else
				while (!kit.deltaLinkMap.containsKey(kit.maxLinkID))
					kit.maxLinkID--;
			int startOffset = linkElem.getStartOffset();
			int endOffset = linkElem.getEndOffset();
			AttributeSet aTagAttrs = (AttributeSet) linkElem.getAttributes().getAttribute(HTML.Tag.A);
			doc.removeCharacterAttributes(startOffset, endOffset - startOffset, attributeSetTagA);
			String classAttrVal = (String) aTagAttrs.getAttribute(HTML.Attribute.CLASS);
			// System.out.println("rem linkElem: "+ Integer.toHexString(linkElem.hashCode()) +" startOffset: "+startOffset+", endOffset: "+endOffset+" - <"+ getText(startOffset, endOffset - startOffset)+">");
			boolean singleElementLink = MiscUtils.equals(classAttrVal, LINK_SEPARATOR_CLASS_ATTR);

			final Point currentViewPos = ((JViewport) getParent()).getViewPosition();

			if (MiscUtils.equals(classAttrVal, LINK_SEPARATOR_CLASS_ATTR_START) || singleElementLink)
				try {
					doc.remove(--startOffset, 1); // remove Link separator [
				} catch (BadLocationException exc) { }
			if (MiscUtils.equals(classAttrVal, LINK_SEPARATOR_CLASS_ATTR_END) || singleElementLink)
				try {
					if (singleElementLink) endOffset--;
					doc.remove(endOffset, 1); // remove Link separator ]
				} catch (BadLocationException exc) { }

			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					((JViewport) getParent()).setViewPosition(currentViewPos);
				}
			});

			if (fireUndoableEditEvent)
				SwingUtils.fireUndoableEditHappened(new HTMLLinkRemovedUndoableEdit(deltaLinkID.intValue(),
						startOffset, endOffset), this, listenerList);
		}
		return deltaLinkID;
	}

	/**
	 * <i>getCurrentDELTALinkID</i> returns the ID of the currently selected DELTA-Link -
	 * if nothing is selected, returns the ID of the DELTA-Link under the mouse cursor (or -1 if there is no DELTA-Link)
	 * @return LinkIDConstants.NO_LINK, if no DELTA-Link is selected;
	 * LinkIDConstants.BAD_LINK, if more than one DELTA-Link is selected or a HTML-Link is inside the selection
	 * else returns the ID of the currently selected DELTA-Link
	 * @param useCache
	 */
	public int getCurrentDELTALinkID(boolean useCache) {
		if (!useCache)
			return determineCurrentDELTALinkID();

		if (currentDELTALinkID == Integer.MIN_VALUE)
			currentDELTALinkID = determineCurrentDELTALinkID();
		return currentDELTALinkID;
	}

	/**
	 * <i>determineCurrentDELTALinkID</i> returns the ID of the currently selected DELTA-Link -
	 * if nothing is selected, returns the ID of the DELTA-Link under the mouse cursor (or -1 if there is no DELTA-Link)
	 * @return LinkIDConstants.NO_LINK, if no DELTA-Link is selected;
	 * LinkIDConstants.BAD_LINK, if more than one DELTA-Link is selected;
	 * LinkIDConstants.HTML_LINK if a HTML-Link is inside the selection
	 * else returns the ID of the currently selected DELTA-Link
	 */
	public int determineCurrentDELTALinkID() {
		int selStart = getSelectionStart();
		int selEnd = getSelectionEnd();
		if (selStart != selEnd) { // is something selected ?
			boolean htmlLinkEncountered = false;
			int lastLinkID = LinkIDConstants.NO_LINK;
			for (int pos = selStart; pos < selEnd;) {
				Element elem = doc.getCharacterElement(pos);
				AttributeSet attrs = (AttributeSet) elem.getAttributes().getAttribute(HTML.Tag.A);
				if ((attrs != null) && attrs.isDefined(HTML.Attribute.HREF)) {
					int linkID = getDELTALinkID(attrs).intValue();
					if (linkID >= LinkIDConstants.FIRST_ID) {
						if ((lastLinkID >= LinkIDConstants.FIRST_ID) && (linkID != lastLinkID))
							return LinkIDConstants.BAD_LINK;  // multiple links with differnt ids selected
						lastLinkID = linkID;
					} else
						htmlLinkEncountered = true;
				}
				pos = elem.getEndOffset();
			}
			if (lastLinkID != LinkIDConstants.NO_LINK)
				return lastLinkID; // selection with one or more links with same id
			else if (htmlLinkEncountered)
				return LinkIDConstants.HTML_LINK;
		} else if (currentElement != null)
			return getDELTALinkID(currentElement).intValue();
		return LinkIDConstants.NO_LINK; // nothing selected
	}

	static Integer getDELTALinkID(Element elem) {
		AttributeSet attrs = (AttributeSet) elem.getAttributes().getAttribute(HTML.Tag.A);
		return getDELTALinkID(attrs);
	}
	static SortedSet getDELTALinkIDs(Element elem) {
		AttributeSet attrs = (AttributeSet) elem.getAttributes().getAttribute(HTML.Tag.A);
		return getDELTALinkIDs(attrs);
	}

	static final SortedSet BAD_LINK_IDS = new TreeSet();
	static Integer getDELTALinkID(AttributeSet attrs) {
		if (attrs != null) {
			String idAttrVal = (String) attrs.getAttribute(HTML.Attribute.ID);
			if (idAttrVal != null)
				try {
					return parseIDList(idAttrVal).first();
				} catch (NumberFormatException exc) { }
		}
		return LinkIDConstants._NO_LINK;
	}
	static SortedSet getDELTALinkIDs(AttributeSet attrs) {
		if (attrs != null) {
			String idAttrVal = (String) attrs.getAttribute(HTML.Attribute.ID);
			if (idAttrVal != null)
				try {
					return parseIDList(idAttrVal);
				} catch (NumberFormatException exc) { }
		}
		return BAD_LINK_IDS;
	}

	static SortedSet<Integer> parseIDList(String idStr) throws NumberFormatException {
		if (idStr != null) {
			if (idStr.startsWith(DELTA_LINK_ID_PREFIX))
				idStr = idStr.substring(DELTA_LINK_ID_PREFIX.length());
			else if (idStr.startsWith(GMT_LINK_ID_PREFIX))
				idStr = idStr.substring(GMT_LINK_ID_PREFIX.length());
			else
				throw new NumberFormatException("Bad Format!");
			SortedSet<Integer> set = new TreeSet<Integer>();
			StringTokenizer st = new StringTokenizer(idStr, ID_LIST_SEPARATOR);
			while (st.hasMoreTokens()) {
				try {
					Integer integer = new Integer(st.nextToken());
					if (integer.intValue() >= 0)
						set.add(integer);
				} catch (NumberFormatException e) { }
			}
			if (set.size() > 0) return set;
		}
		throw new NumberFormatException("Bad Format!");
	}

	public Set<Integer> synchronizeSelectedDELTALinks(int newLinkID) {
		int selStart = getSelectionStart();
		int selEnd = getSelectionEnd();
		String linkName = LINK_ID_FORMAT.format(newLinkID);
		Set<Integer> result = null;
		if (selStart != selEnd) { // is something selected ?
			result = new HashSet<Integer>();
			for (int pos = selStart; pos < selEnd;) {
				Element elem = doc.getCharacterElement(pos);
				Integer oldID = synchronizeDELTALink(elem, newLinkID, linkName);
				if (oldID != null) {
					result.add(oldID);
					SwingUtils.fireUndoableEditHappened(new HTMLLinkIDChangedUndoableEdit(elem, newLinkID, oldID.intValue()), this, listenerList);
				}
				pos = elem.getEndOffset();
			}
		} else if (currentElement != null) {
			result = new HashSet<Integer>();
			Integer oldID = synchronizeDELTALink(currentElement, newLinkID, linkName);
			if (oldID != null) {
				result.add(oldID);
				SwingUtils.fireUndoableEditHappened(new HTMLLinkIDChangedUndoableEdit(currentElement, newLinkID, oldID.intValue()), this, listenerList);
			}
		}
		if ((result != null) && !result.isEmpty())
			contentHasChanged(true);
		return result;
	}

	private boolean linkMode; // todo - needs better name
	public void setLinkMode(boolean linkMode) {
		this.linkMode = linkMode;
	}
	public boolean getLinkMode() {
		return linkMode;
	}

	private Integer synchronizeDELTALink(Element elem, int newLinkID, String linkName) {
		MutableAttributeSet attrSet = (MutableAttributeSet) elem.getAttributes().getAttribute(HTML.Tag.A);
		if (attrSet != null) {
			String idAttrVal = (String) attrSet.getAttribute(HTML.Attribute.ID);
			if (idAttrVal != null) {
				try {
					Integer id = parseIDList(idAttrVal).first();
					int oldKey = id.intValue();
					if (oldKey == newLinkID) return null;
					attrSet.addAttribute(HTML.Attribute.HREF, linkName);
					attrSet.addAttribute(HTML.Attribute.ID, linkName);
					attrSet.addAttribute(HTML.Attribute.NAME, linkName);
					kit.deltaLinkMap.changeKey(oldKey, newLinkID, elem);
					if (newLinkID > kit.maxLinkID)
						kit.maxLinkID = newLinkID;
					else if (oldKey == kit.maxLinkID)
						while (!kit.deltaLinkMap.containsKey(kit.maxLinkID))
							kit.maxLinkID--;

					doc.applyCharacterChanges(elem.getStartOffset(), elem.getEndOffset(), null);
					removeHighlights();
					return id;
				} catch (NumberFormatException exc) { }
			}
		}
		return null;
	}

	public void removeDELTALinksFromCollection(Collection<Integer> deltaLinkIDs, boolean contained) {
		Iterator<Integer> iter = deltaLinkIDs.iterator();
		for (int i = deltaLinkIDs.size() - 1; i >= 0; i--) {
			if (contained == containsDELTALink(iter.next()))
				iter.remove();
		}
	}

	public boolean acceptsNewDELTALink(boolean activeSide, boolean includeHTMLView) {
		return !includeHTMLView || ((isSelected() && (getCurrentDELTALinkID(true) == LinkIDConstants.NO_LINK))
				|| (!activeSide && (getCurrentDELTALinkID(true) >= LinkIDConstants.FIRST_ID)));
	}

	public boolean containsDELTALink(int linkID) {
		if (htmlFile == null) return false;
		return kit.deltaLinkMap.containsKey(linkID);
	}

	public int getDELTALinkCount(int linkID) {
		if (htmlFile == null) return 0;
	//	return kit.deltaLinkMap.getListSize(linkID);
		// do not count <a> elements if they cover adjacent text
		List<Element> list = kit.deltaLinkMap.getList(linkID);
		int count = 0;
		if (list != null)
			for (Element element1 : list) {
				int endOffset = element1.getEndOffset();
				boolean countThisElement = true;
				for (Element element2 : list)
					if (element2.getStartOffset() == endOffset) {
						countThisElement = false;
						break;
					}
				if (countThisElement) count++;
			}
		return count;
	}

	public String getDELTALinkContext(int gmtLinkID) {
		List<Element> elements = kit.deltaLinkMap.get(gmtLinkID);
		if ((elements == null) || elements.isEmpty()) return null;
		int index = 0;
		if (highlightedElements == elements) index = currentHighlightIndex;
		Element linkElem = elements.get(index);
		int startOffset = linkElem.getStartOffset();
		int endOffset = linkElem.getEndOffset();
		try {
			return getText(startOffset, endOffset - startOffset);
		} catch (BadLocationException e) {
			return null;
		}
	}

	public Set getAllLinkIDs() {
		return kit.deltaLinkMap.keySet();
	}


	public Set<Integer> getSelectedLinkIDs() {
		int selStart = getSelectionStart();
		int selEnd = getSelectionEnd();
		Set<Integer> result = null;
		if (selStart < selEnd) { // is something selected ?
			result = new TreeSet<Integer>();
			for (int pos = selStart; pos < selEnd;) {
				Element elem = doc.getCharacterElement(pos);
				Integer gmtLinkID = getDELTALinkID(elem);
				if (gmtLinkID != LinkIDConstants._BAD_LINK)
					result.add(gmtLinkID);
				pos = elem.getEndOffset();
			}
		} else if (currentElement != null) {
			result = new HashSet<Integer>();
			Integer gmtLinkID = getDELTALinkID(currentElement);
			if (gmtLinkID != LinkIDConstants._BAD_LINK)
				result.add(gmtLinkID);
		}
		return result;
	}

	public void highlightDELTALinks(Collection<Integer> selectedLinkIDs) {
		List<Element> allElements = new ArrayList<Element>();
		for (Iterator<Integer> iterator = selectedLinkIDs.iterator(); iterator.hasNext();) {
			Integer gmtLinkID = iterator.next();
			List<Element> elements = kit.deltaLinkMap.get(gmtLinkID);
			if ((elements != null) && !elements.isEmpty())
				allElements.addAll(elements);
		}
		if (!allElements.isEmpty())
			highlightElements(allElements, prefLinkHighlightColor);
	}

	public void highlightAllDELTALinks() {
		highlightElements(kit.deltaLinkMap.getAll(), prefLinkHighlightColor);
	}

	public void highlightSameDELTALinks() {
		highlightElements(kit.deltaLinkMap.get(getCurrentDELTALinkID(true)), prefLinkHighlightColor);
	}

	public void highlightDeadDELTALinks(Collection<Integer> linksOfOtherSide) {
		Set<Integer> links = new HashSet<Integer>(kit.deltaLinkMap.keySet());
		links.removeAll(linksOfOtherSide);
		highlightDELTALinks(links);
	}

	public void jumpToDELTALink(int gmtLinkID) {
		if (htmlFile != null)
			highlightElements(kit.deltaLinkMap.get(gmtLinkID), prefLinkHighlightColor);
	}

	private void highlightElements(List elements, Color highlightColor) {
		// ??? should I remove the current selection before highliting text?
		  // no, if the user select the text first and then clicks on a link-element in the other side the selection would disappear
		  // yes, if the link to be highlighted is inside the current selection
	// unselect();

		if (elements == null) return;
		if (highlightedElements == elements)
			jumpToNextHighlight();
		else {
			if (!kit.showLinks) { // todo
				kit.showLinks = true;
				repaint();
			}
			DefaultHighlighter.DefaultHighlightPainter highlightPainter = new DefaultHighlighter.DefaultHighlightPainter(highlightColor);
			if (highlightedElements != null)
				removeHighlights();
			else /*if (highlightPainter == null)*/ {
			//	highlightPainter = new ExtendedHighlightPainter();
			//	((DefaultHighlighter) getHighlighter()).setDrawsLayeredHighlights(true);
				highlights = new LinkedList();
			}
		//	highlightPainter.setColor(highlightColor);
			highlightedElements = elements;
			int selectionStart = getSelectionStart(), selectionEnd = getSelectionEnd();
			int length = highlightedElements.size();

			// very bad bugfix to eliminate doubled element with incorrect endoffset - blame Sun's javax.swing.text authors
			int docLength = doc.getLength();

			for (int i = 0; i < length; i++) {
				Element linkElem = (Element) highlightedElements.get(i);
				int startOffset = linkElem.getStartOffset(), endOffset = linkElem.getEndOffset();

				if (endOffset == docLength) continue;

				// test if selection and link element are overlapping
				if (((selectionStart >= startOffset) && (selectionStart < endOffset))
						|| ((selectionEnd > startOffset) && (selectionEnd <= endOffset))
						|| ((startOffset >= selectionStart) && (startOffset < selectionEnd))
						|| ((endOffset > selectionStart) && (endOffset <= selectionEnd)))
					unselect();

				if (i == 0)   // scroll to the first occurence
					scrollToRange(startOffset, endOffset);
				Color c = getSelectionColor();
				setSelectionColor(highlightColor);
				try {
					highlights.add(getHighlighter().addHighlight(startOffset, endOffset, highlightPainter));
				} catch (BadLocationException exc) { }
				setSelectionColor(c);
			}
			currentHighlightIndex = 0;
		}
	}

	public void jumpToNextHighlight() {
		if (highlightedElements != null) {
			int size = highlightedElements.size();
			if (size > 1) {
				Element element;
				if ((currentHighlightIndex >= size - 1)) {
					currentHighlightIndex = 0;
					element = (Element) highlightedElements.get(currentHighlightIndex);
				} else {
					element = (Element) highlightedElements.get(currentHighlightIndex);
					int endOffset;
					do {
						endOffset = element.getEndOffset();
						currentHighlightIndex++;
						element = (Element) highlightedElements.get(currentHighlightIndex);
					} while ((endOffset == element.getStartOffset()) && (currentHighlightIndex < (size - 1)));
				}
				scrollToElement(element);
			}
		}
	}

	public void jumpToPreviousHighlight() {
		if (highlightedElements != null) {
			int size = highlightedElements.size();
			if (size > 1) {
				Element element;
				if (currentHighlightIndex == 0) {
					currentHighlightIndex = size - 1;
					element = (Element) highlightedElements.get(currentHighlightIndex);
				} else {
					element = (Element) highlightedElements.get(currentHighlightIndex);
					int startOffset;
					do {
						startOffset = element.getStartOffset();
						currentHighlightIndex--;
						element = (Element) highlightedElements.get(currentHighlightIndex);
					} while ((startOffset == element.getEndOffset()) && (currentHighlightIndex > 0));
				}
				scrollToElement(element);
			}
		}
	}

	public void removeHighlights() {
		if (highlightedElements != null) {
			//    getHighlighter().removeAllHighlights(); // this would hide the current selection
			for (int i = highlights.size() - 1; i >= 0; i--)
				getHighlighter().removeHighlight(highlights.get(i));
			highlights.clear();
			highlightedElements = null;
			currentHighlightIndex = -1;
		}
	}

	public void unselect() {
		setSelectionEnd(0);
        currentDELTALinkID = Integer.MIN_VALUE;
	// getCaret().setDot(getCaret().getDot()+1);
	// getCaret().moveDot(getCaret().getMark());
	}

	private void scrollToElement(Element element) {
		scrollToRange(element.getStartOffset(), element.getEndOffset());
	}

	private void scrollToRange(int startOffset, int endOffset) {
		if (startOffset == endOffset) return;
		Point previousViewPos = ((JViewport) getParent()).getViewPosition();
		startOffset = Math.min(startOffset, endOffset);
		endOffset = Math.max(startOffset, endOffset);
		try {
			Rectangle startBounds = ((TextUI) ui).modelToView(this, startOffset, Position.Bias.Forward),
					endBounds = (((endOffset - startOffset) > 1) ? ((TextUI) ui).modelToView(this, endOffset + 1, Position.Bias.Backward) : startBounds);
			Rectangle bounds = getVisibleRect();
			int w = 50;
			if (startBounds.y == endBounds.y) w = (endBounds.x - startBounds.x) + 5;
			if (startBounds.x > (bounds.width - w))
				bounds.x = startBounds.x - (bounds.width - w);
			else
				bounds.x = 0;
			if (startBounds.y + bounds.height < endBounds.y)
				bounds.y = startBounds.y;
			else
				bounds.y = Math.max(0, ((startBounds.y + endBounds.y) >> 1) - (bounds.height >> 1));
			scrollRectToVisible(bounds);
		} catch (Exception exc) { // BadLocationException or NullPointerException, because modelToView may return null
		} finally {
			if (!previousViewPos.equals(((JViewport) getParent()).getViewPosition()))
				historyBackList.addFirst(new HistoryEntry(currentURL, previousViewPos));  // todo: notify Main[Controller|Frame] of history-list change to en-/disable history buttons
			historyForwardList.clear();
		}
	}

	public void scrollToReference(String reference) {
	// Point previousViewPos = ((JViewport) getParent()).getViewPosition();
		for (HTMLDocument.Iterator iter = doc.getIterator(HTML.Tag.A);
			 iter.isValid(); iter.next()) {
			String name = (String) iter.getAttributes().getAttribute(HTML.Attribute.NAME);
			if ((name != null) && name.equals(reference))
				try {
					Rectangle bounds = ((TextUI) ui).modelToView(this, iter.getStartOffset(), Position.Bias.Forward);
					bounds.height = getVisibleRect().height;
					scrollRectToVisible(bounds);
		//  if (!previousViewPos.equals(((JViewport) getParent()).getViewPosition()))
		//  historyBackList.addFirst(new HistoryEntry(currentURL, previousViewPos));  // todo: notify Main[Controller|Frame] of history-list change to en-/disable history buttons
		//  historyForwardList.clear();
					break;  // exit for loop
				} catch (Exception exc) { } // BadLocationException or NullPointerException, because modelToView may return null
		}
	}

	public boolean goBack() {
		if (historyBackList.isEmpty()) return false;
		Point currentViewPos = ((JViewport) getParent()).getViewPosition();
		URL oldURL = currentURL;
		final HistoryEntry entry = historyBackList.removeFirst();
		if ((currentURL != null) && !currentURL.sameFile(entry.url))
			try {
				load(entry.url, false);
			} catch (IOException exc) {
				throw new RuntimeException(exc); // since 1.4
			}
		else
			currentURL = entry.url;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				((JViewport) getParent()).setViewPosition(entry.viewPos);
			}
		});
		historyForwardList.addFirst(new HistoryEntry(oldURL, currentViewPos));
		return (!historyBackList.isEmpty());
	}

	public boolean goForward() {
		if (historyForwardList.isEmpty()) return false;
		Point currentViewPos = ((JViewport) getParent()).getViewPosition();
		URL oldURL = currentURL;
		final HistoryEntry entry = (HistoryEntry) historyForwardList.removeFirst();
		if ((currentURL != null) && !currentURL.sameFile(entry.url))
			try {
				load(entry.url, false);
			} catch (IOException exc) {
				throw new RuntimeException(exc); // since 1.4
			}
		else
			currentURL = entry.url;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				((JViewport) getParent()).setViewPosition(entry.viewPos);
			}
		});
		historyBackList.addFirst(new HistoryEntry(oldURL, currentViewPos));
		return (!historyForwardList.isEmpty());
	}

	public boolean canGoBack() {
		return (!historyBackList.isEmpty());
	}

	public boolean canGoForward() {
		return (!historyForwardList.isEmpty());
	}

	protected static final class HistoryEntry {
		final URL url;
		final Point viewPos;

		HistoryEntry(URL url, Point viewPos) {
			this.url = url;
			this.viewPos = viewPos;
		}
	}


// hyperlink methods

	void checkURL(String hrefStr) {
		if (hrefStr == null) return;
		if (prefCheckLinks == LINKS_CHECK_ONLOAD)
			try {
				isViewable(new URL(currentURL, hrefStr), true);
			} catch (Exception exc) { }
	}

	private boolean isViewable(final URL url, boolean fullCheck) {
		String file = url.getFile();
		Boolean cachedValue = (Boolean) viewableURLs.get(file);
		if (cachedValue != null)
			return cachedValue.booleanValue();
		boolean result = true;
		if (fullCheck) {
			if (!(file.endsWith(".htm") || file.endsWith(".html")))
				result = false;
			else if (!currentURL.getFile().equals(file))
				result = CheckURL.check(url);
			viewableURLs.put(file, Boolean.valueOf(result));
		}
		return result;
	}

// event methods
	public void fireHyperlinkUpdate(HyperlinkEvent evt) {
		if (!kit.showLinks) return;
		URL url = evt.getURL();
		String desc = evt.getDescription();
		if ((url == null) && (desc.startsWith(DELTA_LINK_ID_PREFIX) || desc.startsWith(GMT_LINK_ID_PREFIX))) {
		// if (kit.showLinks)
			try {
				if (evt.getEventType() == HyperlinkEvent.EventType.ENTERED)
					setLinkCursor(GlobalObjects.DELTA_LINK_CURSOR);
				else if (evt.getEventType() == HyperlinkEvent.EventType.EXITED)
					setLinkCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				SortedSet<Integer> ids = parseIDList(desc);
				if (ids.size() == 1) {
					evt = new DeltaHyperlinkEvent(this, evt.getEventType(), ids.first());
				} else
					evt = new DeltaHyperlinkEvent(this, evt.getEventType(), ids, kit.getLastClickedLocation());
				super.fireHyperlinkUpdate(evt);
			} catch (IllegalArgumentException e) {
				System.err.println("HTMLView: bad DELT/A link id ('" + desc + "') encountered in '"+htmlFile.getName()+"'!");
			}
		} else {
			if ((url != null) && !isViewable(url, (prefCheckLinks >= LINKS_CHECK_ONHOVER))) {
				if (evt.getEventType() == HyperlinkEvent.EventType.ENTERED)
					setLinkCursor(GlobalObjects.LINK_ERROR_CURSOR);
				else if (evt.getEventType() == HyperlinkEvent.EventType.EXITED)
					setLinkCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				super.fireHyperlinkUpdate(new HyperlinkEvent(this, evt.getEventType(), url, null));
				return; // do nothing, if URL is not viewable
			}
			super.fireHyperlinkUpdate(evt);
			if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
				if (evt instanceof HTMLFrameHyperlinkEvent) {
					HTMLFrameHyperlinkEvent evt2 = (HTMLFrameHyperlinkEvent) evt;
					doc.processHTMLFrameHyperlinkEvent(evt2);
				} else if (isViewable(url, (prefCheckLinks == LINKS_CHECK_ONACTIVATION)))
					try {
						Point currentViewPos = ((JViewport) getParent()).getViewPosition();
						URL oldURL = currentURL;
						load(url, false);
						final String reference = url.getRef();
						// Have to scroll after painted.
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								if (reference != null)
									scrollToReference(reference);
								else
								// reset scrollbar
									scrollRectToVisible(new Rectangle(0, 0, 1, 1));
							}
						});
						historyBackList.addFirst(new HistoryEntry(oldURL, currentViewPos));
						historyForwardList.clear();
					} catch (Exception ex) {
						viewableURLs.put(url.getFile(), Boolean.FALSE);
						System.err.println("Could't load URL \"" + url + "\": " + ex);
					}
			}
		}
	}

	/** Notify all listeners that have registered interest for
	 notification contents changes. */
	public void contentHasChanged(boolean hasChanged) {
		contentHasChanged = hasChanged;
		SwingUtils.fireChangeEvent(listenerList, this);
	}

	/**
	 * Adds a change listener for notification of any changes.
	 *
	 * @param listener the change listener
	 */
	public void addChangeListener(ChangeListener listener) {
		listenerList.add(ChangeListener.class, listener);
	}

	/**
	 * Removes a change listener.
	 *
	 * @param listener the change listener
	 */
	public void removeChangeListener(ChangeListener listener) {
		listenerList.remove(ChangeListener.class, listener);
	}

// Getter/Setter Methods
	public boolean contentHasChanged() {
		return contentHasChanged;
	}

	public boolean isSelected() {
	// return (getCaret().getDot() != getCaret().getMark());
		int selStart = getSelectionStart();
		int selEnd = getSelectionEnd();
		if (selStart == selEnd) return false;
		String selText = "";
		try {
			selText = doc.getText(selStart, (selEnd - selStart));
		} catch (BadLocationException e) { }
		return (selText.trim().length() > 0);
	}

	public int getMaxDELTALinkID() {
		if (htmlFile == null) return LinkIDConstants.NO_LINK;
		return kit.maxLinkID;
	}

	public void setExternalMaxDELTALinkID(int externalMaxLinkID) { }  // HTMLView has nothing to do - this is for XMLView

	public void insertNewDELTALink(int newDELTALinkID) {
		int selStart = getSelectionStart();
		int selEnd = getSelectionEnd();
		unselect();

		try {
			int selLength = (selEnd - selStart);
			String selText = doc.getText(selStart, selLength);
			for (int i = 0; i < selLength && Character.isWhitespace(selText.charAt(i)); i++, selStart++) ;
			for (int i = selLength-1; i >= 0 && Character.isWhitespace(selText.charAt(i)); i--, selEnd--) ;
			if ((selEnd - selStart) < 1) return; // only whitespaces - should not happen
		} catch (BadLocationException e) { }

		insertNewDELTALink(newDELTALinkID, selStart, selEnd, true);
		contentHasChanged(true);
	}


	private void insertNewDELTALink(int newDELTALinkID, int startOffset, int endOffset, boolean fireUndoableEditEvent) {
		String linkName = LINK_ID_FORMAT.format(newDELTALinkID);
		SimpleAttributeSet aTagAttrSet = new SimpleAttributeSet();
		aTagAttrSet.addAttribute(HTML.Attribute.HREF, linkName);
		aTagAttrSet.addAttribute(HTML.Attribute.ID, linkName);
		aTagAttrSet.addAttribute(HTML.Attribute.COLOR, prefLinkTextColor); // todo this should not be needed anymore

        if (prefUseLinkSeparator) {
        //    aTagAttrSet.addAttribute(HTML.Attribute.CLASS, LINK_SEPARATOR_CLASS_ATTR);
			try {
			// System.out.println("ins startOffset: " + startOffset + ", endOffset: " + endOffset + " - <" + getText(startOffset, endOffset - startOffset) + ">");
				doc.insertString(startOffset, prefLinkSeparator1, null);
				doc.insertString(endOffset + 1, prefLinkSeparator2, null);
				MutableAttributeSet attr = new SimpleAttributeSet();
				StyleConstants.setForeground(attr, prefLinkDelimiterColor);
				doc.setCharacterAttributes(startOffset++, 1, attr, false);
				doc.setCharacterAttributes(endOffset++ + 1, 1, attr, false);
			} catch (Exception exc) { }
		}

		/*SimpleAttributeSet aTagSet = new SimpleAttributeSet();
		aTagSet.addAttribute(HTML.Tag.A, aTagAttrSet);*/
		doc.setLinkElementAttributes(startOffset, endOffset - startOffset, aTagAttrSet, newDELTALinkID, prefUseLinkSeparator);
		kit.maxLinkID = Math.max(kit.maxLinkID, newDELTALinkID);
        if (fireUndoableEditEvent)
            SwingUtils.fireUndoableEditHappened(new HTMLLinkInsertedUndoableEdit(newDELTALinkID, startOffset, endOffset), this, listenerList);
	}

	public String getDTDURLString() { return null; } // HTMLView don't need to provide DTD filepath

	public Node getSelectedNode() { return null; } // HTMLView has no XML-Nodes

	/**
	 * Adds an undo listener for notification of any changes.
	 * Undo/Redo operations performed on the <code>UndoableEdit</code>
	 * will cause the appropriate DocumentEvent to be fired to keep
	 * the view(s) in sync with the model.
	 *
	 * @param listener the <code>UndoableEditListener</code> to add
	 * @see Document#addUndoableEditListener
	 */
	public void addUndoableEditListener(UndoableEditListener listener) {
		listenerList.add(UndoableEditListener.class, listener);
	}

	/**
	 * Removes an undo listener.
	 *
	 * @param listener the <code>UndoableEditListener</code> to remove
	 * @see #addUndoableEditListener
	 */
	public void removeUndoableEditListener(UndoableEditListener listener) {
		listenerList.remove(UndoableEditListener.class, listener);
	}

	private String searchData, searchText;
	private int searchStartPos;
	private boolean searchDirUp, searchWholeWords, searchWrap=true, searchFound;

	public void findNext() {
        searchAgain(searchDirUp);
    }
    public void findPrevious() {
        searchAgain(!searchDirUp);
    }
    public void searchAgain(boolean searchDirUp) {
		int pos = getCaretPosition();

		int xStart, xFinish;
		while (true) {
			if (searchDirUp)
				xStart = searchData.lastIndexOf(searchText, pos - searchText.length());
			else
				xStart = searchData.indexOf(searchText, pos - searchStartPos + searchText.length());
			if (xStart < 0) {
				if (searchWrap && searchFound) {
					try {
						if (!searchDirUp) {
							searchData = doc.getText(0, pos);
							pos = searchStartPos = 0;
						} else
							searchData = doc.getText(pos, doc.getLength() - pos); // tothink
						continue;
					} catch (BadLocationException e) { }
				}
				JOptionPane.showMessageDialog(this, "Search Text not found", "Search", JOptionPane.INFORMATION_MESSAGE);
				return;
			}

			xFinish = xStart + searchText.length();

			if (searchWholeWords) {
				boolean s1 = xStart > 0;
				boolean b1 = s1 && !MiscUtils.isWordSeparator(searchData.charAt(xStart - 1));
				boolean s2 = xFinish < searchData.length();
				boolean b2 = s2 && !MiscUtils.isWordSeparator(searchData.charAt(xFinish));

				if (b1 || b2) { // Not a whole word
					if (searchDirUp && s1) { // Can continue up
						pos = xStart;
						continue;
					}
					if (!searchDirUp && s2) { // Can continue down
						pos = xFinish + 1;
						continue;
					}
					if (searchWrap && searchFound) {
						try {
							if (!searchDirUp)
								searchData = doc.getText(0, pos);
							else
								searchData = doc.getText(pos, doc.getLength() - pos);
							continue;
						} catch (BadLocationException e) { }
					}
					// Found, but not a whole word, and we cannot continue
					JOptionPane.showMessageDialog(this, "Search Text not found", "Search", JOptionPane.INFORMATION_MESSAGE);
					return;
				}
			}
			break;
		}

		if (!searchDirUp) {
			xStart += searchStartPos;
			xFinish += searchStartPos;
		}

		searchFound = true;
		if (searchDirUp) {
			setCaretPosition(xFinish);
			moveCaretPosition(xStart);
		} else
			select(xStart, xFinish);
	}

	public boolean canSearchAgain() {
		return searchData != null;  
	}

	public void search(String text, boolean onlySelection, boolean searchUp, boolean caseSensitive, boolean wholeWords, boolean wrap) {
		// todo
		int pos = getCaretPosition();
		if ((pos != searchStartPos) || (searchDirUp != searchUp))
			try {
				if (searchUp)
					searchData = doc.getText(0, pos);
				else
					searchData = doc.getText(pos, doc.getLength() - pos);
				if (!caseSensitive)
					searchData = searchData.toLowerCase();
				searchStartPos = pos;
				searchDirUp = searchUp;
			} catch (BadLocationException exc) {
				return;  // should not happen
			}
		searchText = (caseSensitive ? text : text.toLowerCase());
		searchWholeWords = (wholeWords && !MiscUtils.containsWordSeparators(searchText));
		searchWrap = wrap;
		searchFound = false;
		findNext();
	}

	public String getDocumentName() {
		if (doc == null)
			return "No Document loaded!";
		String title = doc.getTitle();
		if (title == null)
			return "Untitled";
		title = title.trim();
		if (title.length() > 0)
			return title;
		return "Untitled";
	}

	public File getFile() {
		return htmlFile;
	}

	public String getFilePath() {
		return ((htmlFile == null) ? null : htmlFile.getAbsolutePath());
	}

	public String getSourceHTML() {
		return XMLUtils.transformHTML2HTMLSource(new StringReader(getText()));
	}

	public String getSourcePlain() {
		return getText();
	}

	private void setProperties() {
		prefLinkHighlightColor = preferences.getColor(Preferences.KEY_COLOR_LINK_ACTIVATED_BG);
		prefLinkTextColor	   = SwingUtils.colorToHex(preferences.getColor(Preferences.KEY_COLOR_LINK_FG));
		prefLinkSetTextColor   = SwingUtils.colorToHex(preferences.getColor(Preferences.KEY_COLOR_LINK_HTML_OVERLAPPING_FG));
		prefLinkDelimiterColor = preferences.getColor(Preferences.KEY_COLOR_LINK_HTML_DELIM);
		prefCheckLinks		   = preferences.getInt(Preferences.KEY_HTML_CHECK_LINKS);
		setSelectionColor(preferences.getColor(Preferences.KEY_COLOR_SELECTION));
		prefUseLinkSeparator = preferences.getBoolean(Preferences.KEY_HTML_USE_LINK_SEPARATOR);
		prefLinkSeparator1 = String.valueOf(preferences.getChar(Preferences.KEY_HTML_LINK_SEPARATOR1, '['));
		prefLinkSeparator2 = String.valueOf(preferences.getChar(Preferences.KEY_HTML_LINK_SEPARATOR2, ']'));
	}

	public void refetchProperties(Set<String> changedProperties) {
		setProperties();
		kit.setProperties();
		repaint();
	}

	public int getCheckLinks() {
		return prefCheckLinks;
	}

	public void setCheckLinks(int checkLinks) {
		this.prefCheckLinks = checkLinks;
	}

	/**
	 * Sets the cursor to use over links.
	 *
	 * @since 1.3
	 */
	public void setLinkCursor(Cursor linkCursor) {
		kit.setLinkCursor(linkCursor);
	}

	public void write(Writer out) throws IOException {
		try {
			kit.write(out, doc, 0, doc.getLength());
		} catch (BadLocationException e) {
			throw new IOException(e.getMessage());
		}
	}

	/** This is used by viewToModel to avoid allocing a new array each
	 * time. */
	private static final Position.Bias[] bias = new Position.Bias[1];
	private static final Point point = new Point();

	public int viewToModel(Point pt) {
		int pos = ((TextUI) ui).viewToModel(this, pt, bias);
		if ((bias[0] == Position.Bias.Backward) && (pos > 0)) pos--;
		try {
			Rectangle rect = modelToView(pos);
			if ((rect != null)) {
				rect.width =15;
				rect.x -= 7;
				if (rect.contains(pt))
					return pos;
			}
		} catch (BadLocationException exc) { }
		return -1;
	}

	int viewToModel(int x, int y) {
		point.move(x, y);
		return viewToModel(point);
	}


	public void storeProperties() {
		// currently, HTMLView don't has any properties to store
	// preferences.putColor(Preferences.KEY_COLOR_LINK_HTML_SET, Color.decode(prefLinkSetTextColor));
	// preferences.putColor(Preferences.KEY_COLOR_LINK_HTML_DELIM, prefLinkDelimiterColor);
	}

	// overridden to prevent exceptions
	public Dimension getPreferredSize() {
		try {
			return super.getPreferredSize();
		} catch (Throwable t) { // to supress unpredictable swing/html errors
			return getParent().getParent().getBounds().getSize();
		}
	}

	public Rectangle getBounds() {
		try {
			return super.getBounds();
		} catch (Throwable t) { // supress unpredictable swing/html errors
			return getParent().getParent().getBounds();
		}
	}

	public boolean getScrollableTracksViewportWidth() {
		try {
			return super.getScrollableTracksViewportWidth();
		} catch (Exception e) { // supress unpredictable swing/html errors
			return false;
		}
	}

	public boolean getScrollableTracksViewportHeight() {
		try {
			return super.getScrollableTracksViewportHeight();
		} catch (Exception e) { // supress unpredictable swing/html errors
			return false;
		}
	}

	protected void paintComponent(Graphics g) {
		try {
			super.paintComponent(g);
		} catch (Exception exc) { }
	}

	public void doLayout() {
		try {
			super.doLayout();
		} catch (Exception e) { }
	}

	static {
		attributeSetTagA = new SimpleAttributeSet();
		attributeSetTagA.addAttribute(HTML.Tag.A, HTML.Tag.A);
	}

	private final class HTMLLinkInsertedUndoableEdit extends SideAbstractUndoableEdit implements LinkInsertedUndoableEdit {
		private int newDELTALinkID, startPos, endPos;

		public HTMLLinkInsertedUndoableEdit(int newDELTALinkID, int startPos, int endPos) {
			super(false);
			this.newDELTALinkID = newDELTALinkID;
			this.startPos = startPos;
			this.endPos = endPos;
			undoPresentationName = "Undo - Link insertion";
			redoPresentationName = "Redo - Link insertion";
		}

		public void undo() throws CannotUndoException {
			super.undo();
		// unselect();
			scrollToRange(startPos, endPos);
			for (int pos = startPos; pos < endPos;) {
				Element linkElem = doc.getCharacterElement(pos);
				if (removeDELTALLink(linkElem, false).intValue() == newDELTALinkID)
				    kit.deltaLinkMap.remove(newDELTALinkID, linkElem);
				pos = linkElem.getEndOffset();
			}
            if (prefUseLinkSeparator) {
                startPos--;
                endPos--;
            }
            select(startPos, endPos);
		}

		public void redo() throws CannotRedoException {
			super.redo();
			unselect();
			scrollToRange(startPos, endPos);
			insertNewDELTALink(newDELTALinkID, startPos, endPos, false);
            if (prefUseLinkSeparator) {
                startPos++;
                endPos++;
            }
		}
	}
	/*private final class HTMLLinksRemovedWrapperUndoableEdit extends SideCompoundEdit {
		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof HTMLLinkRemovedUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}*/
	private final class HTMLLinkRemovedUndoableEdit extends SideAbstractUndoableEdit implements LinksRemovedUndoableEdit {
		private int removedDELTALinkID, startPos, endPos;

		public HTMLLinkRemovedUndoableEdit(Element element) {
			this(getDELTALinkID(element).intValue(), element.getStartOffset(), element.getEndOffset());
		}
		public HTMLLinkRemovedUndoableEdit(int removedDELTALinkID, int startPos, int endPos) {
			super(false);
			this.removedDELTALinkID = removedDELTALinkID;
			this.startPos = startPos;
			this.endPos = endPos;
		}

		public void redo() throws CannotUndoException {
			super.redo();
			scrollToRange(startPos, endPos);
		// for (int pos = startPos; pos < endPos;) {
				Element linkElem = doc.getCharacterElement(startPos); //pos);
				removeDELTALLink(linkElem, false);
				kit.deltaLinkMap.remove(removedDELTALinkID, linkElem);
		//  pos = linkElem.getEndOffset();
		// }
            if (prefUseLinkSeparator) {
                startPos--;
                endPos--;
            }
            select(startPos, endPos);
		}
		public void undo() throws CannotRedoException {
			super.undo();
			unselect();
			scrollToRange(startPos, endPos);
			insertNewDELTALink(removedDELTALinkID, startPos, endPos, false);
            if (prefUseLinkSeparator) {
                startPos++;
                endPos++;
            }
		}
	}
	private final class HTMLLinkIDChangedUndoableEdit extends SideAbstractUndoableEdit implements LinkIDChangedUndoableEdit {
		private Element elem;
		private int newLinkID, oldLinkID;

		public HTMLLinkIDChangedUndoableEdit(Element elem, int newLinkID, int oldLinkID) {
			super(false);
			this.elem = elem;
			this.newLinkID = newLinkID;
			this.oldLinkID = oldLinkID;
			undoPresentationName = "Undo - Link Synchronization";
			redoPresentationName = "Redo - Link Synchronization";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			unselect();
			synchronizeDELTALink(elem, oldLinkID, LINK_ID_FORMAT.format(oldLinkID));
			scrollToElement(elem);
		}

		public void redo() throws CannotRedoException {
			super.redo();
			unselect();
			synchronizeDELTALink(elem, newLinkID, LINK_ID_FORMAT.format(newLinkID));
			scrollToElement(elem);
		}
	}

	private static final class ExtendedHighlightPainter
			extends DefaultHighlighter.DefaultHighlightPainter {
		private Color highlightColor;

		ExtendedHighlightPainter() {
			super(null);
		}

		public Color getColor() {
			return highlightColor;
		}

		void setColor(Color color) {
			highlightColor = color;
		}
	}

	private static final class CheckURL {
		private static boolean result;

		static boolean check(final URL url) {
			result = false;
			Thread checkURL = new Thread() {
				public void run() {
					try {
						URLConnection urlConnection = url.openConnection();
						InputStream inputStream = urlConnection.getInputStream();
						inputStream.read();
						inputStream.close();
						// or
						/* URLConnection connection =
						url.openConnection();
						if (connection instanceof HttpURLConnection) {
						HttpURLConnection httpConnection = (HttpURLConnection)connection;
						httpConnection.connect();
						int response = httpConnection.getResponseCode();
						System.out.println("Response: " + response);
						InputStream is = httpConnection.getInputStream();
						byte[] buffer = new byte [256];
						while (is.read (buffer) != -1) {}
						is.close();  */
						result = true;
					} catch (IOException e) {
						result = false;
					}
				}
			};
			checkURL.start();
			try {
				checkURL.join(CHECK_URL_TIMEOUT);
			} catch (InterruptedException e) { }
			return result;
		}
	}
}
