package at.ac.tuwien.asgaard.delta.gui.html;

import javax.swing.*;
import javax.swing.plaf.TextUI;
import javax.swing.text.*;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import java.awt.*;
import java.awt.event.FocusEvent;
import java.io.IOException;
import java.io.StringReader;
import java.io.Writer;
import java.util.LinkedList;

public class SimpleHTMLView extends JEditorPane {
	protected LinkedList historyBackList;
	protected LinkedList historyForwardList;

	public SimpleHTMLView() {
		super();
		setEditorKit(new HTMLEditorKit() {
			public Document createDefaultDocument() {
				Document defaultDocument = super.createDefaultDocument();
				defaultDocument.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
				return defaultDocument;
			}
			public void install(JEditorPane ep) {
				LinkController linkHandler = new HTMLEditorKit.LinkController();
				ep.addMouseListener(linkHandler);
				ep.addMouseMotionListener(linkHandler);
			}
//            public void deinstall(JEditorPane c) {}
		});
	//	setBackground(GlobalConstants.DEFAULT_BACKGROUND);
//  setMargin(GlobalConstants.INSETS_ZERO);
		setCaret(new DefaultCaret() {  // override DefaultCarets focusLost to hide only caret and not selection
			public void focusLost(FocusEvent evt) {
				setVisible(false); // not: setSelectionVisible(false);
			}
		});
		historyBackList = new LinkedList();
		historyForwardList = new LinkedList();
		setEditable(false);
	}

    public void setText(String text) {
		historyBackList.clear();
		historyForwardList.clear();
        EditorKit kit = getEditorKit();
        Document doc = kit.createDefaultDocument();
        try {
            kit.read(new StringReader(text), doc, 0);
        } catch (Exception exc) { exc.printStackTrace(); } // shouldn't occur
        setDocument(doc);
	// setBackground(Color.WHITE);
	// super.setText(text);
    }

	public void write(Writer out) throws IOException {
		try {
			new ExtendedHTMLEditorKit.ExtendedHTMLWriter(out, (HTMLDocument) getDocument()).write();
		} catch (BadLocationException e) {
			throw new IOException(e.getMessage());
		}
	}

/* public void fireHyperlinkUpdate(HyperlinkEvent evt) {
		if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
			scrollToReference(evt.getDescription().substring(1));
	}*/

	public void scrollToReference(String reference) {
		Point previousViewPos = ((JViewport) getParent()).getViewPosition();
		for (HTMLDocument.Iterator iter = ((HTMLDocument)getDocument()).getIterator(HTML.Tag.A);
                    iter.isValid(); iter.next()) {
			String name = (String) iter.getAttributes().getAttribute(HTML.Attribute.NAME);
			if ((name != null) && name.equals(reference))
				try {
					Rectangle bounds = ((TextUI) ui).modelToView(this, iter.getStartOffset(), Position.Bias.Forward);
					bounds.height = getVisibleRect().height;
					scrollRectToVisible(bounds);
					if (!previousViewPos.equals(((JViewport) getParent()).getViewPosition()))
						historyBackList.addFirst(previousViewPos);  // todo: notify Main[Controller|Frame] of history-list change to en-/disable history buttons
					historyForwardList.clear();
					break;  // exit for loop
				} catch (Exception exc) { } // BadLocationException or NullPointerException, because modelToView may return null
		}
	}

	public boolean goBack() {
		if (historyBackList.isEmpty()) return false;
		Point currentViewPos = ((JViewport) getParent()).getViewPosition();
		final Point historyPos = (Point)  historyBackList.removeFirst();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				((JViewport) getParent()).setViewPosition(historyPos);
			}
		});
		historyForwardList.addFirst(currentViewPos);
		return (!historyBackList.isEmpty());
	}

	public boolean goForward() {
		if (historyForwardList.isEmpty()) return false;
		Point currentViewPos = ((JViewport) getParent()).getViewPosition();
		final Point historyPos = (Point) historyForwardList.removeFirst();
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				((JViewport) getParent()).setViewPosition(historyPos);
			}
		});
		historyBackList.addFirst(currentViewPos);
		return (!historyForwardList.isEmpty());
	}

	public boolean canGoBack() {
		return (!historyBackList.isEmpty());
	}

	public boolean canGoForward() {
		return (!historyForwardList.isEmpty());
	}

	// overridden to prevent exceptions
	public Dimension getPreferredSize() {
		try {
			return super.getPreferredSize();
		} catch (Throwable t) { // to supress unpredictable swing/html errors
			return getParent().getParent().getBounds().getSize();
		}
	}

	public Rectangle getBounds() {
		try {
			return super.getBounds();
		} catch (Throwable t) { // supress unpredictable swing/html errors
			return getParent().getParent().getBounds();
		}
	}

	public boolean getScrollableTracksViewportWidth() {
		try {
			return super.getScrollableTracksViewportWidth();
		} catch (Exception e) { // supress unpredictable swing/html errors
			return false;
		}
	}

	public boolean getScrollableTracksViewportHeight() {
		try {
			return super.getScrollableTracksViewportHeight();
		} catch (Exception e) { // supress unpredictable swing/html errors
			return false;
		}
	}

	protected void paintComponent(Graphics g) {
		try {
			super.paintComponent(g);
		} catch (Exception exc) {
		}
	}

	public void doLayout() {
		try {
			super.doLayout();
		} catch (Exception e) {
		}
	}
}
