/*
 * Part of "Guideline Markup Tool"
 * Created by peter, 27.09.2004, 18:16:50
 */

package at.ac.tuwien.asgaard.delta.gui.images;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import java.util.Map;
import java.util.HashMap;
import java.awt.Image;
import java.net.URL;

/**
 * Class LazyResourceLoader ... todo
 * 
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class LazyResourceLoader {
	// Images
	public static final String APP_ICON = "deltaIcon.gif";
	public static final String PROJECT_FILE_ICON = "deltaIcon.gif";
	
	public static final String APP_LOGO = "at/ac/tuwien/asgaard/delta/gui/misc/images/SplashWindow.gif";

	private static final Map<String, ImageIcon> RESOURCES = new HashMap<String, ImageIcon>();

	/** Gets an icon using an absolute path. Removes leading / from path. */
	public static Icon getIcon(String path) {
		if (path.charAt(0) == '/')
			path = path.substring(1);
		return getIconRelativeToClass(path, null);
	}

	/** Gets an icon using a path relative to this class. */
	public static Icon getIconRelativeToClass(String path) {
		return getIconRelativeToClass(path, LazyResourceLoader.class);
	}
//	public Icon getIconRelativeToClass(String path) {
//		return getIconRelativeToClass(path, getClass());
//	}

	/** Gets an icon using a path relative to specified class. */
	public static Icon getIconRelativeToClass(String path, Class<?> clazz) {
		ImageIcon icon = RESOURCES.get(path);
		if (icon == null) {
			URL url;
			if (clazz == null)
				url = LazyResourceLoader.class.getClassLoader().getResource(path);
			else
				url = LazyResourceLoader.class.getResource(path);
			icon = new ImageIcon(url);
			RESOURCES.put(path, icon);
		}
		return icon;
	}

	/** Gets an image using an absolute path. Removes leading / from path. */
	public static Image getImage(String path) {
		if (path.charAt(0) == '/')
			path = path.substring(1);
		return getImageRelativeToClass(path, null);
	}

	/** Gets an image using a path relative to this class. */
	public static Image getImageRelativeToClass(String path) {
		return getImageRelativeToClass(path, LazyResourceLoader.class);
	}

	/** Gets an image using a path relative to specified class. */
	public static Image getImageRelativeToClass(String path, Class<?> clazz) {
		ImageIcon imageIcon = RESOURCES.get(path);
		if (imageIcon == null) {
			URL url;
			if (clazz == null)
				url = LazyResourceLoader.class.getClassLoader().getResource(path);
			else
				url = LazyResourceLoader.class.getResource(path);
			imageIcon = new ImageIcon(url);
			RESOURCES.put(path, imageIcon);
		}
		return imageIcon.getImage();
	}

	/** Gets an ImageIcon using an absolute path. Removes leading / from path. */
	public static ImageIcon getImageIcon(String path) {
		if (path.charAt(0) == '/')
			path = path.substring(1);
		return getImageIconRelativeToClass(path, null);
	}

	/** Gets an ImageIcon using a path relative to this class. */
	public Icon getImageIconRelativeToClass(String path) {
		return getImageIconRelativeToClass(path, getClass());
	}

	/** Gets an ImageIcon using a path relative to specified class. */
	public static ImageIcon getImageIconRelativeToClass(String path, Class<?> clazz) {
		ImageIcon imageIcon = RESOURCES.get(path);
		if (imageIcon == null) {
			URL url;
			if (clazz == null)
				url = LazyResourceLoader.class.getClassLoader().getResource(path);
			else
				url = LazyResourceLoader.class.getResource(path);
			imageIcon = new ImageIcon(url);
			RESOURCES.put(path, imageIcon);
		}
		return imageIcon;
	}
}
