package at.ac.tuwien.asgaard.delta.gui.macros;

import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.models.SimpleListModel;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Set;
import java.util.TreeSet;

public class MacrosInsertableView extends JPanel {
	private MacrosList macrosList;
	private SimpleListModel macrosListModel;
	private MacrosListItem currentItem;
	private MacrosListModel rootListModel;
	private boolean needReset;

	public MacrosInsertableView() {
		super(new BorderLayout());
		currentItem = null;
		macrosListModel = new SimpleListModel();
		macrosList = new MacrosList(macrosListModel);

		JScrollPane scrollPane = new JScrollPane(macrosList);
		macrosList.setLayoutOrientation(JList.VERTICAL_WRAP);
		macrosList.setVisibleRowCount(-1);

		JPanel dummy = new JPanel(new BorderLayout());
		dummy.setBorder(new TitledBorder(GlobalObjects.EMPTY_BORDER_0, "All Insertable Macros"));
		dummy.add(scrollPane);

		add(dummy);

		ToolTipManager.sharedInstance().registerComponent(macrosList);
	// setBackground(GlobalConstants.DEFAULT_BACKGROUND);
//		setBorder(new CompoundBorder(BorderFactory.createLoweredBevelBorder(),
		setBorder(new EmptyBorder(MacrosView.MACROS_LIST_MARGIN, MacrosView.MACROS_LIST_MARGIN, MacrosView.MACROS_LIST_MARGIN, MacrosView.MACROS_LIST_MARGIN));
		setMinimumSize(GlobalObjects.ZERO_SIZE);

		installListeners();
	}

	private void installListeners() {
		macrosList.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				int itemIndex = ((JList) evt.getSource()).locationToIndex(evt.getPoint());
				ListModel model = macrosList.getModel();
				if ((itemIndex < 0) || (itemIndex >= model.getSize())) return;
				if (SwingUtilities.isLeftMouseButton(evt)) {
					if (evt.getClickCount() == 2) {
						currentItem = (MacrosListItem) model.getElementAt(itemIndex);
						if (currentItem.isMacro())
							SwingUtils.fireActionEvent(listenerList, MacrosInsertableView.this);
					}
				} else if (evt.getClickCount() == 1)
					macrosList.setSelectedIndex(itemIndex);
			}
		});
		macrosList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
			// int itemIndex = macrosList.getMinSelectionIndex();
			// ListModel model = macrosList.getModel();
			// if ((itemIndex < 0) || (itemIndex >= model.getSize())) return;
				currentItem = (MacrosListItem) macrosList.getSelectedValue(); //(MacrosListItem) model.getElementAt(itemIndex);
				SwingUtils.fireChangeEvent(listenerList, MacrosInsertableView.this);
			}
		});
		macrosList.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
					if ((currentItem != null) && currentItem.isMacro())
						SwingUtils.fireActionEvent(listenerList, MacrosInsertableView.this);
					evt.consume();
				}
			}
		});
		addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent evt) {
				if (needReset)
					resetMacroListItems();
				SwingUtils.fireChangeEvent(listenerList, MacrosInsertableView.this);
			}
		});
	}

	void setRootMacrosListModel(MacrosListModel rootListModel) {
		this.rootListModel = rootListModel;
		resetMacroListItems();
	}

	public MacrosListItem getCurrentItem() {
		return currentItem;
	}


	public void resetMacroListItems() {
		if (macrosList.isShowing()) {
			needReset = false;
			Set result = new TreeSet(new MacrosListItem.ListItemComparator(false, true, true));
			if ((rootListModel != null))
				createResultSet(result, rootListModel);
			if ((rootListModel == null) || result.isEmpty()) {
				// macrosList.clear();
				macrosListModel.clear();
				macrosList.setBackground(MacrosView.DEFAULT_LIST_BACKGROUND);
			} else {
				Object selectedValue = macrosList.getSelectedValue();
			// macrosList.setModel(new SimpleListModel(new ArrayList(result)));
				macrosListModel.init(new ArrayList(result));
				macrosList.setBackground(MacrosView.SELECTED_LIST_BACKGROUND);

			// macrosList.setSelectedValue(selectedValue, true);
				boolean found = false;
				if (selectedValue != null)
					for (int i = 0, size = macrosListModel.getSize(); i < size; i++)
						if (selectedValue == macrosListModel.getElementAt(i)) {
							macrosList.setSelectedIndex(i);
							macrosList.ensureIndexIsVisible(i);
							repaint();  /** FIX-ME setSelectedIndex does not redraw all the time with the basic l&f**/
							found = true;
							break;
						}
				if (!found)
					macrosList.setSelectedIndex(0);
			}
		} else
			needReset = true;
	}

	private void createResultSet(Set result, MacrosListModel macrosListModel) {
		for (int i = 0, size = macrosListModel.getSize(); i < size; i++) {
			MacrosListItem macrosListItem = macrosListModel.getItemAt(i);
			if (macrosListItem.isGroup())
				createResultSet(result, ((MacrosListItem.Group) macrosListItem).getMacrosListModel());
			else if (macrosListItem.isInsertable())
				result.add(macrosListItem);
		}
	}


	// Eventlistener methods
	void addChangeListener(ChangeListener listener) {
		listenerList.add(ChangeListener.class, listener);
	}
	void removeChangeListener(ChangeListener listener) {
		listenerList.remove(ChangeListener.class, listener);
	}
	void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}
	void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}

	public void setSelectionColor(Color selectionColor) {
		macrosList.setSelectionBackground(selectionColor);
	}
}
