package at.ac.tuwien.asgaard.delta.gui.macros;

import at.ac.tuwien.asgaard.swing.models.EmptyListModel;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.event.MouseEvent;
import java.awt.*;

/**
 * Class MacrosList. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
final class MacrosList extends JList {
	MacrosListModel macrosListModel;

	MacrosList() {
		this(EmptyListModel.sharedInstance);
	}

	public MacrosList(ListModel dataModel) {
		super(dataModel);
		setCellRenderer(new MacrosListCellRenderer());
		setSelectionModel(new SingleListSelectionModel(true));
		setBackground(MacrosView.DEFAULT_LIST_BACKGROUND);
	}

	public String getToolTipText(MouseEvent evt) {
		if (macrosListModel == null) return null;
		int index = locationToIndex(evt.getPoint());
		if ((index == -1) || !getCellBounds(index, index).contains(evt.getPoint())) return null;
		MacrosListItem macrosListItem = (MacrosListItem) macrosListModel.getElementAt(index);
		if (macrosListItem == null) return null;
		if (evt.isAltDown())
			return macrosListItem.getToolTipTextVerbose();
		return macrosListItem.getToolTipText();
	}

	void clear() {
		setBackground(MacrosView.DEFAULT_LIST_BACKGROUND);
		setModel(EmptyListModel.sharedInstance);
	}

	public void setModel(MacrosListModel model) {
		super.setModel(model);
		macrosListModel = model;
		scrollRectToVisible(GlobalObjects.TOP_RECT);
	}

	MacrosListModel getMacrosListModel() {
		return macrosListModel;
	}

	void selectAndShowIndex(int itemIndex) {
		setSelectedIndex(itemIndex);
		ensureIndexIsVisible(itemIndex);
		repaint();
	}

	void setColors(Color title, Color listBackGround) {
		setBackground(listBackGround);
		JComponent comp = (JComponent) (getParent().getParent().getParent());
		TitledBorder tb = (TitledBorder) comp.getBorder();
	//	tb.setBorder(MacrosView.DEFAULT_LIST_BORDER);
		tb.setTitleColor(title);
		comp.repaint();
	}
}
