/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 19, 2002
 * Time: 3:21:28 AM
 */
package at.ac.tuwien.asgaard.delta.gui.macros;

import at.ac.tuwien.asgaard.delta.gui.macros.images.MacrosImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.SimpleListCellRenderer;
import at.ac.tuwien.asgaard.swing.SwingUtils;

import javax.swing.*;
import java.awt.*;

/**
 * Class MacrosListCellRenderer. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
final class MacrosListCellRenderer extends SimpleListCellRenderer {  

	public static final Color GROUP_LINK_FOREGROUND = new Color(0, 128, 0),
								NOT_STANDALONE_FOREGROUND = new Color(230, 0, 0);
	public static final Font  LABEL_FONT   = GlobalObjects.DEFAULT_FONT.deriveFont(Font.ITALIC);

	public Component getListCellRendererComponent(JList list, Object value, int index,
							boolean isSelected, boolean cellHasFocus) {
		int alignment = SwingConstants.LEFT;
		Icon icon;

		super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);

		MacrosListItem item = (MacrosListItem)value;
		//setToolTipText(item.getToolTipText());

		icon = item.getIcon();

		switch (item.getType()) {
			case MacrosListItem.TYPE_SEPARATOR:
				return new JSeparator();
			case MacrosListItem.TYPE_LABEL:
				setForeground(Color.BLUE);
				alignment = CENTER;
				setFont(LABEL_FONT);
				break;
			case MacrosListItem.TYPE_GROUP_LINK:
				setForeground(GROUP_LINK_FOREGROUND);
				break;
			case MacrosListItem.TYPE_GROUP:
				icon = (isSelected ? MacrosImages.OPENED_GROUP_ICON : MacrosImages.CLOSED_GROUP_ICON);
				break;
			case MacrosListItem.TYPE_ELEMENT:
			case MacrosListItem.TYPE_MACRO:
				MacrosListItem.Element elementItem = (MacrosListItem.Element)item;
				if (!elementItem.isInsertable())
					setBackground(SwingUtils.changeColor(getBackground(), -8));
				if (isSelected && !elementItem.isStandalone())
					setForeground(NOT_STANDALONE_FOREGROUND);
				break;
		}
		setIcon(icon);
		setHorizontalAlignment(alignment);

		return this;
	}
}