/*
 * MacrosSearchView.java
 *
 * Created on 20. M?rz 2002, 12:07
 */

package at.ac.tuwien.asgaard.delta.gui.macros;

import at.ac.tuwien.asgaard.delta.gui.macros.images.MacrosImages;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.CardPanel;
import at.ac.tuwien.asgaard.swing.RadioButtonsPanel;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.util.Log;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.*;
import java.util.TreeSet;

/**
 * Class MacrosSearchView.... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
public final class MacrosSearchView extends JPanel {
    private final Preferences properties;
	private MacrosListModel rootMacrosListModel;
	private MacrosListItem currentItem;

    private Action aSearch, aClear;

	/** Creates new form MacrosSearchView */
	public MacrosSearchView(Preferences properties, StatusBar statusBar) {
        super(new BorderLayout());
		Log.trace("enter: MacrosSearchView.MacrosSearchView()");

        this.properties = properties;
		currentItem = null;

        createActions();
        buildUI();
        installListeners();

        statusBar.attachToAbstractButton(jButtonSearch);
        statusBar.attachToAbstractButton(jButtonClear);
        statusBar.attachToAbstractButton(jToggleButtonAdvancedSearch);

        Log.trace("exit: MacrosSearchView.MacrosSearchView()");
	}

	void setRootMacrosListModel(MacrosListModel rootMacrosListModel) {
		this.rootMacrosListModel = rootMacrosListModel;
		jListSearchResults.clear();
        jTextFieldKeyword.setText("");
        currentItem = null;
	}

	void resetMacroListItems() {
		jCheckBoxIncInsertable.setEnabled(true);
		jCheckBoxInsertable.setEnabled(true);
		redrawList();
	}

	MacrosListItem getCurrentItem() {
		return currentItem;
	}

	private void standardSearch(TreeSet searchResults, MacrosListModel mlm, String keyWord, boolean ignoreCase, boolean searchOnlyForMacros, boolean searchInDescription) {
		for (int i = 0; i < mlm.getSize(); i++) {
			MacrosListItem item = mlm.getItemAt(i);
			if (!searchOnlyForMacros || (item.getType() == MacrosListItem.TYPE_MACRO)) {
				String name = item.getName(), description = item.getDescription();
				if (ignoreCase) {
					name = name.toLowerCase();
					if (searchInDescription) description = description.toLowerCase();
				}
				if ((name.indexOf(keyWord) > -1) ||         // todo: exactSearch && name.equals(keyWord)
				        (searchInDescription && (description.indexOf(keyWord) > -1)))
					searchResults.add(item);
			}
			if ((item.getType() == MacrosListItem.TYPE_GROUP) &&
					!((MacrosListItem.Group) item).isGroupRef())
				standardSearch(searchResults, ((MacrosListItem.Group) item).getMacrosListModel(), keyWord,
				               ignoreCase, searchOnlyForMacros, searchInDescription);
		}
	}

	private void redrawList() {
		jListSearchResults.revalidate();
		jListSearchResults.repaint();
	}


	private void doSearch() {
		if (jToggleButtonAdvancedSearch.isSelected()) {  // todo
		// advancedSearch(searchResults, rootMacrosListModel, keyWord, ignoreCase, jCheckBoxOnlyMacros.isSelected());
			jTextFieldName.requestFocusInWindow();
		} else
			doStandardSearch();
		if (jListSearchResults.getModel().getSize() == 0)
			JOptionPane.showMessageDialog(this, "No matching macro found!", "Macro search", JOptionPane.INFORMATION_MESSAGE);
	}

	private void doStandardSearch() {
		String keyWord = jTextFieldKeyword.getText().trim();
		jTextFieldKeyword.setText(keyWord);
		if (keyWord.length() == 0)
			jListSearchResults.clear();
		else {
			jListSearchResults.setBackground(Color.white);
			TreeSet searchResults = new TreeSet(new MacrosListItem.ListItemComparator(true, true, false));
			boolean ignoreCase = properties.getBoolean(Preferences.KEY_MACROS_SEARCH_IGNORE_CASE);
			if (ignoreCase)
				keyWord = keyWord.toLowerCase();
			standardSearch(searchResults, rootMacrosListModel, keyWord, ignoreCase, properties.getBoolean(Preferences.KEY_MACROS_SEARCH_ONLY_MACROS), false);
			if (searchResults.isEmpty())
				jListSearchResults.clear();
			else {
				jListSearchResults.setListData(searchResults.toArray());
                jListSearchResults.setSelectedIndex(0);
            }
		}
		jTextFieldKeyword.requestFocusInWindow();
	}

    private void advancedSearch(TreeSet searchResults, MacrosListModel mlm, String keyWord, boolean ignoreCase, boolean onlyMacros) {
        // todo
    }


// Eventlistener methods
	void addChangeListener(ChangeListener listener) {
		listenerList.add(ChangeListener.class, listener);
	}

	void removeChangeListener(ChangeListener listener) {
		listenerList.remove(ChangeListener.class, listener);
	}

    void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}

	void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}

    private void createActions() {
        aSearch = new AbstractAction("Search", MacrosImages.START_SEARCH_ICON) {
            {
                putValue(MNEMONIC_KEY, new Integer('S'));
                putValue(StatusBar.TOOL_TIP, "Starts the search"); // when mouse over - tool-tip
                putValue(StatusBar.DESCRIPTION_HOVER, "Starts the search."); // when mouse over - statusbar
                putValue(StatusBar.DESCRIPTION_ACTIVE, "Searching ...");  // when clicked
            }
            public void actionPerformed(ActionEvent evt) {
                doSearch();
            }
        };
        aClear = new AbstractAction("Clear", MacrosImages.CLEAR_SEARCH_ICON) {
            {
                putValue(MNEMONIC_KEY, new Integer('C'));
                putValue(StatusBar.TOOL_TIP, "Clears all fields"); // when mouse over - tool-tip
                putValue(StatusBar.DESCRIPTION_HOVER, "Clears all search fields."); // when mouse over - statusbar
                putValue(StatusBar.DESCRIPTION_ACTIVE, "Clearing ...");  // when clicked
            }
            public void actionPerformed(ActionEvent evt) {
                jTextFieldKeyword.setText("");
                jCheckBoxIncName.setSelected(true);
                jTextFieldName.setText("");
                jCheckBoxIncDescription.setSelected(false);
                jTextFieldDescription.setText("");
                jCheckBoxIncInsertable.setSelected(false);
                jCheckBoxInsertable.setSelected(false);
                jCheckBoxIncLink.setSelected(false);
                jCheckBoxLink.setSelected(false);
                jListSearchResults.clear();
                currentItem = null;
                SwingUtils.fireChangeEvent(listenerList, MacrosSearchView.this);
            }
        };
    }

    private void buildUI() {
        jListSearchResults = new MacrosList();
    //    jListSearchResults.setBorder(BorderFactory.createLineBorder(Color.gray));
        JScrollPane scrollPane = new JScrollPane(jListSearchResults);
		JPanel dummy = new JPanel(new BorderLayout());
		dummy.setBorder(new TitledBorder(GlobalObjects.EMPTY_BORDER_0, "Search Results"));
		dummy.add(scrollPane);
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, properties.getBoolean(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT),
                                              createMainPanel(), dummy);
		splitPane.setOneTouchExpandable(true);
		splitPane.setResizeWeight(0.7);
		add(splitPane);
        ToolTipManager.sharedInstance().registerComponent(jListSearchResults);
		setMinimumSize(GlobalObjects.ZERO_SIZE);
    }

    private JPanel createMainPanel() {
        JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(GlobalObjects.EMPTY_BORDER_2);
        panel.add(createSearchForms(), BorderLayout.CENTER);
        panel.add(createButtonPanel(), BorderLayout.SOUTH);
        return panel;
    }

    private Component createSearchForms() {
        jPanelSearchCards = new CardPanel();
        jPanelSearchCards.setBorder(new CompoundBorder(BorderFactory.createEtchedBorder(), GlobalObjects.EMPTY_BORDER_5));
        jPanelSearchCards.add(createStandardSearchForm(), "standard");
        jPanelSearchCards.add(createAdvancedSearchForm(), "advanced");
        return jPanelSearchCards;
    }

    private Component createStandardSearchForm() {
        final double[][] size = new double[][] {{TableLayout.PREFERRED, TableLayout.FILL},
                                                {TableLayout.PREFERRED}};
        JPanel panel = new JPanel(new TableLayout(size));
        JLabel label = new JLabel("Keyword: ");
        label.setDisplayedMnemonic(KeyEvent.VK_K);
        jTextFieldKeyword = new JTextField();
        label.setLabelFor(jTextFieldKeyword);
        panel.add(label, "0, 0");
        panel.add(jTextFieldKeyword, "1, 0");
        return panel;
    }

    private Component createAdvancedSearchForm() {
        final double[][] size = new double[][] {{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL},
                                                {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}};
        JPanel panel = new JPanel(new TableLayout(size));

        jCheckBoxIncName = new JCheckBox();
		jCheckBoxIncName.setSelected(true);
        panel.add(jCheckBoxIncName, new TableLayoutConstraints());
        panel.add(new JLabel("Name: "), new TableLayoutConstraints(1, 0));
        jTextFieldName = new JTextField();
        panel.add(jTextFieldName, new TableLayoutConstraints(2, 0));

        jCheckBoxIncDescription = new JCheckBox();
        panel.add(jCheckBoxIncDescription, new TableLayoutConstraints(0, 1));
        panel.add(new JLabel("Description: "), new TableLayoutConstraints(1, 1));
        jTextFieldDescription = new JTextField();
        panel.add(jTextFieldDescription, new TableLayoutConstraints(2, 1));

        JPanel macroSearchOptions = new JPanel(new TableLayout(
                 new double[][] {{TableLayout.PREFERRED, TableLayout.PREFERRED, 20.0, TableLayout.PREFERRED, TableLayout.PREFERRED},
                                 {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		final EmptyBorder TITLE_BORDER_MARGIN = new EmptyBorder(0, 2, 2, 2);
		macroSearchOptions.setBorder(new CompoundBorder(new TitledBorder("Macro: "), TITLE_BORDER_MARGIN));
        jCheckBoxIncInsertable = new JCheckBox();
        jCheckBoxInsertable = new JCheckBox("is insertable:");
        jCheckBoxInsertable.setMnemonic(KeyEvent.VK_I);
        jCheckBoxInsertable.setDisplayedMnemonicIndex(3);
        jCheckBoxInsertable.setHorizontalTextPosition(SwingConstants.LEFT);
        jCheckBoxInsertable.setSelected(true);

        jCheckBoxIncLink = new JCheckBox();
        jCheckBoxLink = new JCheckBox("has Link:");
        jCheckBoxLink.setMnemonic(KeyEvent.VK_L);
        jCheckBoxLink.setHorizontalTextPosition(SwingConstants.LEFT);
        jCheckBoxLink.setSelected(true);

        jCheckBoxIncInput = new JCheckBox();
        jCheckBoxInput = new JCheckBox("needs input:");
        jCheckBoxInput.setMnemonic(KeyEvent.VK_N);
        jCheckBoxInput.setHorizontalTextPosition(SwingConstants.LEFT);
        jCheckBoxInput.setSelected(true);

        jCheckBoxIncStandalone = new JCheckBox();
        jCheckBoxStandalone = new JCheckBox("is standalone:");
        jCheckBoxStandalone.setHorizontalTextPosition(SwingConstants.LEFT);
        jCheckBoxStandalone.setSelected(true);

        jCheckBoxIncSelection = new JCheckBox();
        jCheckBoxSelection = new JCheckBox("takes selection:");
        jCheckBoxSelection.setHorizontalTextPosition(SwingConstants.LEFT);
        jCheckBoxSelection.setSelected(true);

        macroSearchOptions.add(jCheckBoxIncInsertable, new TableLayoutConstraints());
        macroSearchOptions.add(jCheckBoxInsertable, new TableLayoutConstraints(1, 0));
        macroSearchOptions.add(jCheckBoxIncLink, new TableLayoutConstraints(0, 1));
        macroSearchOptions.add(jCheckBoxLink, new TableLayoutConstraints(1, 1));
        macroSearchOptions.add(jCheckBoxIncStandalone, new TableLayoutConstraints(3, 1));
        macroSearchOptions.add(jCheckBoxStandalone, new TableLayoutConstraints(4, 1));
        macroSearchOptions.add(jCheckBoxIncInput, new TableLayoutConstraints(0, 2));
        macroSearchOptions.add(jCheckBoxInput, new TableLayoutConstraints(1, 2));
        macroSearchOptions.add(jCheckBoxIncSelection, new TableLayoutConstraints(3, 2));
        macroSearchOptions.add(jCheckBoxSelection, new TableLayoutConstraints(4, 2));

        panel.add(macroSearchOptions, new TableLayoutConstraints(1, 2, 2, 2));

        jCheckBoxIncInGroup = new JCheckBox();
        panel.add(jCheckBoxIncInGroup, new TableLayoutConstraints(0, 3, 0, 3, TableLayout.FULL, TableLayout.TOP));

        JPanel groupSearchOptions = new JPanel(new TableLayout(
					new double[][] {{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL},
									{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		groupSearchOptions.setBorder(new CompoundBorder(new TitledBorder("Group: "), TITLE_BORDER_MARGIN));

		jCheckBoxIncGroupName = new JCheckBox();
		jCheckBoxIncGroupName.setSelected(true);
		groupSearchOptions.add(jCheckBoxIncGroupName, new TableLayoutConstraints());
		groupSearchOptions.add(new JLabel("Name: "), new TableLayoutConstraints(1, 0));
		jTextFieldGroupName = new JTextField();
		groupSearchOptions.add(jTextFieldGroupName, new TableLayoutConstraints(2, 0));

		jCheckBoxIncGroupDescription = new JCheckBox();
		groupSearchOptions.add(jCheckBoxIncGroupDescription, new TableLayoutConstraints(0, 1));
		groupSearchOptions.add(new JLabel("Description: "), new TableLayoutConstraints(1, 1));
		jTextFieldGroupDescription = new JTextField();
		groupSearchOptions.add(jTextFieldGroupDescription, new TableLayoutConstraints(2, 1));
		jCheckBoxIncSubgroups = new JCheckBox("Include Subgroups: ");
		jCheckBoxIncSubgroups.setHorizontalTextPosition(SwingConstants.LEFT);
		groupSearchOptions.add(jCheckBoxIncSubgroups, new TableLayoutConstraints(1, 2, 2, 2));

		panel.add(groupSearchOptions, new TableLayoutConstraints(1, 3, 2, 3));

		andOrSearchPanel = new RadioButtonsPanel();
		andOrSearchPanel.add("And", KeyEvent.VK_A);
		andOrSearchPanel.add("Or", KeyEvent.VK_O);
		andOrSearchPanel.setSelected(0);
		panel.add(andOrSearchPanel, new TableLayoutConstraints(0, 5, 2, 5));

        return new JScrollPane(panel);
    }

    private JComponent createButtonPanel() {
        jToggleButtonAdvancedSearch = new JToggleButton("Advanced Search");
		jToggleButtonAdvancedSearch.setMnemonic(KeyEvent.VK_A);
		// TODO: keep disabled since it is not finished yet
	//	jToggleButtonAdvancedSearch.setEnabled(false);

        jButtonSearch = new JButton(aSearch);
        jButtonClear = new JButton(aClear);
		jToggleButtonAdvancedSearch.setMargin(GlobalObjects.BUTTON_MARGIN);
        jButtonSearch.setMargin(GlobalObjects.BUTTON_MARGIN);
        jButtonClear.setMargin(GlobalObjects.BUTTON_MARGIN);

        Box buttonPanel = Box.createHorizontalBox();
    //    buttonPanel.add(jToggleButtonAdvancedSearch);
        buttonPanel.add(Box.createHorizontalGlue());
        buttonPanel.add(jButtonClear);
        buttonPanel.add(jButtonSearch);
        buttonPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);

        return buttonPanel;
    }

    private void installListeners() {
        jListSearchResults.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                int itemIndex = ((JList) evt.getSource()).locationToIndex(evt.getPoint());
                ListModel model = jListSearchResults.getModel();
                if ((itemIndex < 0) || (itemIndex >= model.getSize())) return;
                if (SwingUtilities.isLeftMouseButton(evt)) {
                    if (evt.getClickCount() == 2) {
                        currentItem = (MacrosListItem) model.getElementAt(itemIndex);
                        if (currentItem.isMacro())
                            SwingUtils.fireActionEvent(listenerList, MacrosSearchView.this);
                    }
                } else if (evt.getClickCount() == 1)
                    jListSearchResults.setSelectedIndex(itemIndex);
            }
        });
        jListSearchResults.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
                int itemIndex = jListSearchResults.getMinSelectionIndex();
                ListModel model = jListSearchResults.getModel();
                if ((itemIndex < 0) || (itemIndex >= model.getSize())) return;
                currentItem = (MacrosListItem) model.getElementAt(itemIndex);
                SwingUtils.fireChangeEvent(listenerList, MacrosSearchView.this);
            }
        });
        jListSearchResults.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent evt) {
                if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
                    if ((currentItem != null) && currentItem.isMacro())
                        SwingUtils.fireActionEvent(listenerList, MacrosSearchView.this);
                    evt.consume();
                }
            }
        });
        addComponentListener(new ComponentAdapter() {
            public void componentShown(ComponentEvent evt) {
                if (jToggleButtonAdvancedSearch.isSelected())
                    jTextFieldName.requestFocusInWindow();
                else
                    jTextFieldKeyword.requestFocusInWindow();
                //   if ((currentItem == null) || !getSynchronizeStructureView())
                SwingUtils.fireChangeEvent(listenerList, MacrosSearchView.this);
            }
        });
		jToggleButtonAdvancedSearch.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                jPanelSearchCards.switchToCard(jToggleButtonAdvancedSearch.isSelected() ? "advanced" : "standard");
                if (jToggleButtonAdvancedSearch.isSelected())
                    jTextFieldName.requestFocusInWindow();
                else
                    jTextFieldKeyword.requestFocusInWindow();
            }
        });

        jTextFieldKeyword.addActionListener(aSearch);
        jTextFieldName.addActionListener(aSearch);
        jTextFieldDescription.addActionListener(aSearch);
        jTextFieldGroupName.addActionListener(aSearch);
        jTextFieldGroupDescription.addActionListener(aSearch);
    }

	private MacrosList jListSearchResults;
	private JToggleButton jToggleButtonAdvancedSearch;
	private JButton jButtonClear, jButtonSearch;
	private CardPanel jPanelSearchCards;

    private JTextField jTextFieldKeyword, jTextFieldName, jTextFieldDescription, jTextFieldGroupName, jTextFieldGroupDescription;
    private JCheckBox jCheckBoxIncName, jCheckBoxIncDescription, jCheckBoxIncGroupName, jCheckBoxIncGroupDescription;
    private JCheckBox jCheckBoxIncInGroup, jCheckBoxIncSubgroups;

    private JCheckBox jCheckBoxLink, jCheckBoxIncLink;
    private JCheckBox jCheckBoxInsertable, jCheckBoxIncInsertable;
    private JCheckBox jCheckBoxStandalone, jCheckBoxIncStandalone;
    private JCheckBox jCheckBoxInput, jCheckBoxIncInput;
    private JCheckBox jCheckBoxSelection, jCheckBoxIncSelection;
	private RadioButtonsPanel andOrSearchPanel;
}
