package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.*;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLView;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.BlockComboBox;
import at.ac.tuwien.asgaard.swing.SimpleListCellRenderer;
import at.ac.tuwien.asgaard.swing.ExtendedList;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;
import at.ac.tuwien.asgaard.swing.selectable.RadioButtonAbstractAction;
import at.ac.tuwien.asgaard.swing.selectable.SelectableAction;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.BevelBorder;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.util.Set;

/**
 * Class AllElementsDialog ... todo
 * 
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class AllElementsDialog extends JDialog {
	private     final ExtendedList    elementsList;
	private   XMLView    xmlView;
	private   String    lastDTDURLString;
	private     final BlockComboBox    elementInsertModeComboBox;
	private DTDHelper dtdHelper;

	private     final Action    insertAction;
	private     final SelectableAction insertModeInto, insertModeBefore, insertModeAfter, insertModeReplace, insertModeConvert, insertModeWrap;

	private static final Font    NOT_SELECTED_FONT  = GlobalObjects.DEFAULT_FONT.deriveFont(Font.ITALIC);
	private boolean shouldShowAgain;

	public static final boolean REFRESH_ALL_ELEMENTS = false; // todo

/*	private static class ListItemInfo {
		String name;
		boolean insertable;
	}
	private ListItemInfo items[];*/

	public AllElementsDialog() {
		super(MainController.mainFrame, "All Elements of current XML-Language", false);
		elementsList = new ExtendedList(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				if (REFRESH_ALL_ELEMENTS) {
					if (!insertAction.isEnabled())
						setText("<html><s>" + value + "</s>");
				} else if (!isSelected)
					setFont(NOT_SELECTED_FONT);
				else if (!insertAction.isEnabled())
					setText("<html><s>" + value + "</s>");
				return this;
			}
		});
		elementsList.setLayoutOrientation(JList.VERTICAL_WRAP);
		elementsList.setVisibleRowCount(-1);
		elementsList.setSelectionModel(new SingleListSelectionModel());
		elementsList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				elementsList.ensureIndexIsVisible(elementsList.getSelectedIndex());
				if (!REFRESH_ALL_ELEMENTS) refresh();
			}
		});
		JToolBar toolBar  = new MainFrame.SmallToolBar("Elements");

		Action  searchUpAction = new SimpleAction("Search Up", MainImages.SEARCH_UP_ICON, 'U',
				"Search upwards for insertable position in current XML file for selected node") {
			protected void simpleActionPerformed() {
				xmlView.searchForNextNodeInsertionPosition(elementsList.getSelectedValue().toString(), false, false);
				MainController.aEditFindNext.setEnabled(true);
				MainController.aEditFindPrevious.setEnabled(true);
			}
		};
		Action searchDownAction = new SimpleAction("Search Down", MainImages.SEARCH_DOWN_ICON, 'D',
				"Search downwards for insertable position in current XML file for selected node") {
			protected void simpleActionPerformed() {
				xmlView.searchForNextNodeInsertionPosition(elementsList.getSelectedValue().toString(), false, true);
				MainController.aEditFindNext.setEnabled(true);
				MainController.aEditFindPrevious.setEnabled(true);
			}
		};
		Action closeAction = new SimpleAction("Close", 'C') {
			protected void simpleActionPerformed() {
				setVisible(false);
			}
		};

		Action highlightOccurrences = new SimpleAction("Highlight Occurrences", MainImages.HIGHLIGHT_SEARCH_ICON, 'H', "Highlight all occurrences of selected element in current XML file") {
			protected void simpleActionPerformed() {
				MainController.getActiveSideController().aXMLEvaluateXPathExpression.evaluateAndVisualizeXPathExpression("//"+ elementsList.getSelectedValue().toString());
			}
		};
		Action jumpToDefinition = new SimpleAction("Go to definition", MainImages.GO_TO_DEFINITION_ICON, 'D',
													   "Go to definition of selected element") {
			protected void simpleActionPerformed() {
				MainController.getActiveSideController().aXMLGoToElementDefinitionInDTDReport.jumpToDefinition(elementsList.getSelectedValue().toString());
			}
		};


		insertAction = new SimpleAction(MainImages.MACRO_INSERT_ICON, "Inserts selected element using selected Insert-Mode") {
			protected void simpleActionPerformed() {
				int insertMode = elementInsertModeComboBox.getSelectedIndexTrimmed();
				InsertableXMLNodesDialog.insertNode(elementsList.getSelectedValue().toString(), insertMode);
			}
		};
		insertModeInto = new RadioButtonAbstractAction("Insert Into", MainImages.XML_INSERT_INTO_ICON, 'I',
				"\"Insert Element\"-Action will insert current element as child of selected Element", true) {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.isNodeInsertableInto(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		insertModeBefore = new RadioButtonAbstractAction("Insert Before", MainImages.XML_INSERT_BEFORE_ICON, 'B',
				"\"Insert Element\"-Action will insert current element before selected Element") {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.isNodeInsertableBefore(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		insertModeAfter = new RadioButtonAbstractAction("Insert After", MainImages.XML_INSERT_AFTER_ICON, 'A',
				"\"Insert Element\"-Action will insert current element after selected Element") {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.isNodeInsertableAfter(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		insertModeReplace = new RadioButtonAbstractAction("Replace", MainImages.XML_INSERT_REPLACE_ICON, 'R',
				"\"Insert Element\"-Action will replace selected Element") {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.canBeReplacedByNode(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		insertModeConvert = new RadioButtonAbstractAction("Convert", MainImages.XML_INSERT_REPLACE_ICON, 'C',
														  "\"Insert Element\"-Action will convert selected Element") {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.canBeConvertedToElement(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		insertModeWrap = new RadioButtonAbstractAction("Wrap", MainImages.XML_INSERT_REPLACE_ICON, 'W',
														      "\"Insert Element\"-Action will wrap selected Element") {
			public void stateChanged(ChangeEvent e) {
				insertAction.setEnabled(dtdHelper.canBeWrappedByElement(elementsList.getSelectedValue().toString()));
				elementsList.repaint();
			}
		};
		elementsList.addActionListener(insertAction);

		toolBar.add(searchDownAction);
		toolBar.add(searchUpAction);
		toolBar.add(Box.createGlue());
		toolBar.add(highlightOccurrences);
		toolBar.add(Box.createHorizontalStrut(10));
		toolBar.add(jumpToDefinition);
		toolBar.add(Box.createGlue());
		elementInsertModeComboBox = new BlockComboBox(new Object[]{insertModeInto,
																   BlockComboBox.SEPARATOR,
																   insertModeAfter, insertModeBefore,
																   BlockComboBox.SEPARATOR,
																   insertModeReplace, insertModeConvert, insertModeWrap});
		elementInsertModeComboBox.setMaximumSize(new Dimension(80, 22));
		toolBar.add(elementInsertModeComboBox);
		toolBar.add(insertAction);
		MainController.mainFrame.statusBar.attachToToolBar(toolBar);
		JPanel borderPanel = new JPanel(new BorderLayout());
		borderPanel.setBorder(new CompoundBorder(GlobalObjects.EMPTY_BORDER_2, new BevelBorder(BevelBorder.LOWERED, Color.LIGHT_GRAY, Color.DARK_GRAY)));//BorderFactory.createLoweredBevelBorder()));
		borderPanel.add(new JScrollPane(elementsList));
		JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		bottomPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
		bottomPanel.add(new JButton(closeAction));
		JPanel panel = new JPanel(new BorderLayout());
		panel.add(borderPanel);
		panel.add(toolBar, BorderLayout.NORTH);
	// panel.add(bottomPanel, BorderLayout.SOUTH);
		getContentPane().add(panel);
		setSize(400, 300);
		rootPane.getActionMap().put("closeAction", closeAction);
		rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "closeAction");
	}

	public void showDialog(Component relativeToComponent, DTDHelper dtdHelper, XMLView xmlView) {
		setLocationRelativeTo(relativeToComponent);
		this.xmlView = xmlView;
		this.dtdHelper = dtdHelper;

		String dtdURLString = xmlView.getDTDURLString();
		if (!MiscUtils.equals(lastDTDURLString, dtdURLString)) {
			lastDTDURLString = dtdURLString;
			Set elements = dtdHelper.getAllElementsSet();
			elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
			elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
			String[] result = new String[elements.size()];
			elements.toArray(result);
			elementsList.setListData(result);
		}
		elementsList.setSelectedIndex(0);
		setVisible(true);
		shouldShowAgain = false;
	}

	public boolean showAgain() {
		if (shouldShowAgain) {
			elementsList.setSelectedIndex(0);
			setVisible(true);
			shouldShowAgain = false;
			return false;
		}
		return true;
	}
	public void hideTemporary() {
		if (isVisible()) {
			shouldShowAgain = true;
			setVisible(false);
		}
	}

	public void refreshList(DTDHelper dtdHelper) {
		Set elements = dtdHelper.getAllElementsSet();
		elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
		elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
		String[] result = new String[elements.size()];
		elements.toArray(result);
		elementsList.setListData(result);
		elementsList.setSelectedIndex(0);
	}

	public void targetElementChanged(DTDHelper dtdHelper) {
		this.dtdHelper = dtdHelper;
		refresh();
	}

	private void refresh() {
		Object selectedValue = elementsList.getSelectedValue();
		if (selectedValue == null) return;
		
		String newNodeName = selectedValue.toString();
		insertModeInto.setEnabled(dtdHelper.getLastInsertIntoIndex(newNodeName) != -1);
		insertModeBefore.setEnabled(dtdHelper.getInsertBeforeIndex(newNodeName) != -1);
		insertModeAfter.setEnabled(dtdHelper.getInsertAfterIndex(newNodeName) != -1);
		insertModeReplace.setEnabled(dtdHelper.canBeReplacedByNode(newNodeName));
		insertModeConvert.setEnabled(dtdHelper.canBeConvertedToElement(newNodeName));
		insertModeWrap.setEnabled(dtdHelper.canBeWrappedByElement(newNodeName));
		boolean enabled = ((Action) elementInsertModeComboBox.getSelectedItem()).isEnabled();
		insertAction.setEnabled(enabled);
		elementsList.repaint();
		if (MainController.autoChangeInsertMode)
			if (enabled) {
				if (MainController.alwaysChangeInsertModeToDefault && insertModeInto.isEnabled())
					elementInsertModeComboBox.setSelectedIndex(0);
			} else if (insertModeInto.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(0);
			else if (insertModeBefore.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(3);
			else if (insertModeAfter.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(4);
			else if (insertModeReplace.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(5);
			else if (insertModeConvert.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(6);
			else if (insertModeWrap.isEnabled())
				elementInsertModeComboBox.setSelectedIndex(7);
			else if (MainController.alwaysChangeInsertModeToDefault)
				elementInsertModeComboBox.setSelectedIndex(0);
		elementInsertModeComboBox.repaint();
	}
}