/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */
package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.AutoXPathDef;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.AutoXPathDefs;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.ButtonScrollPanel;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.LineBorder;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

/**
 * Class AutoXPathBar ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class AutoXPathBar extends ButtonScrollPanel {
    private Node node;

	public AutoXPathBar() {
		super(new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0)));
	}

	public void refresh(AutoXPathDefs autoXPathDefs, Node currentNode) {
		contentComponent.removeAll();
		if (autoXPathDefs != null) {
			java.util.List autoXPathDefList = autoXPathDefs.getAutoXPathDefList();
			for (int i = 0, size = autoXPathDefList.size(); i < size; i++) {
				AutoXPathDef autoXPathDef = (AutoXPathDef) autoXPathDefList.get(i);
				String xpath = autoXPathDef.getXpath();
				String resultString;
                Object result = XMLUtils.evaluateXPathSingleSilently(xpath, currentNode);
                if (result == null) continue;
                final Node resultNode = (result instanceof Node) ? (Node) result : null;
                if (result instanceof Node) {
                    if (resultNode.getNodeType() == Node.ELEMENT_NODE)
                        resultString = resultNode.getNodeName();
                    else
                        resultString = resultNode.getNodeValue().trim();
                } else
                    resultString = result.toString().trim();
				if (resultString.length() == 0) continue;
				JPanel pairPanel = new JPanel(new BorderLayout(2, 0));
				pairPanel.setBorder(new CompoundBorder(new LineBorder(Color.GRAY), GlobalObjects.EMPTY_BORDER_2));
				pairPanel.add(new JLabel(autoXPathDef.getLabel()), BorderLayout.WEST);
				JLabel resultLabel = new JLabel(MiscUtils.truncateSring(resultString, 10, 5));
				resultLabel.setFont(GlobalObjects.BOLD_ITALIC_FIXED_FONT);
				String toolTipText = "<html>\"<b><i>" + resultString + "</i></b>\"";
                if (resultNode != null) {
					toolTipText += "<br>... double-click to jump to corresponding element";
					resultLabel.addMouseListener(new MouseAdapter() {
						public void mouseClicked(MouseEvent evt) {
							if (evt.getClickCount() == 2) {
								node = (resultNode instanceof Attr) ? ((Attr) resultNode).getOwnerElement() : resultNode;
								SwingUtils.fireActionEvent(listenerList, AutoXPathBar.this);
							}
						}
					});
				}
				resultLabel.setToolTipText(toolTipText);
				pairPanel.add(resultLabel, BorderLayout.CENTER);
				contentComponent.add(pairPanel);
			}
			contentComponent.revalidate();
			contentComponent.repaint();
			revalidate();
			repaint();
			resetScrollButtons();
		}
	}

    public Node getNode() {
        return node;
    }

    public void addActionListener(ActionListener listener) {
        listenerList.add(ActionListener.class, listener);
    }
}