package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.images.LazyResourceLoader;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.util.DefaultFileFilter;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.ListMap;
import at.ac.tuwien.asgaard.util.xml.AnyXMLFileFilter;
import at.ac.tuwien.asgaard.util.xml.DocTypeFileFilter;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.util.List;

import javax.swing.*;
import javax.swing.filechooser.*;

public final class DeltaFileChooser extends JFileChooser implements PropertyChangeListener {
	public static final	FileFilter XML_HTML_FILE_FILTER	= new XMLorHTMLFileFilter();
	public static final	FileFilter HTML_FILE_FILTER		= new HTMLFileFilter();
	public static final	FileFilter XML_FILE_FILTER		= new DefaultFileFilter(".xml",		"XML");
	public static final	FileFilter ANY_XML_FILE_FILTER	= new AnyXMLFileFilter();
	public static final	FileFilter MACROS_FILE_FILTER	= new DefaultFileFilter(".macros", 	"DELT/A-Macro");
	public static final	FileFilter PROJECT_FILE_FILTER	= new DefaultFileFilter(".project",	"DELT/A-Project");
	public static final	FileFilter PROCESS_FILE_FILTER	= new DefaultFileFilter(".process",	"Transformation Process");
	public static final	FileFilter PROJECT_AND_PROCESS_FILE_FILTER	= new ProjectAndProcessFileFilter();
	public static final	FileFilter DTD_FILE_FILTER		= new DefaultFileFilter(".dtd",		"DTD");
	public static final	FileFilter XSL_FILE_FILTER		= new XSLFileFilter();
	public static final	FileFilter GIF_FILE_FILTER		= new DefaultFileFilter(".gif",		"GIF");
	public static final	FileFilter IMAGE_FILE_FILTER	= new ImageFileFilter();

	private boolean askToOverwrite;

	private  File  selectedFile   = null;
	private  File  oldDirectory;
	public static final DeltaFileView FILE_VIEW   = new DeltaFileView();

	public static File showOpenFileChooser(Component parent, String title, File baseDir, String extension) {
		DeltaFileChooser deltaFileChooser = new DeltaFileChooser(baseDir, title);
		if (extension == null) deltaFileChooser.setAcceptAllFileFilterUsed(true);
		else deltaFileChooser.addChoosableFileFilter(new DefaultFileFilter(extension));
		int result = deltaFileChooser.showOpenDialog(parent);
		if (result == JFileChooser.APPROVE_OPTION)
			return deltaFileChooser.getSelectedFile();
		return null;
	}
	public static File showSaveFileChooser(Component parent, String title, File baseDir, String extension) {
		DeltaFileChooser deltaFileChooser = new DeltaFileChooser(baseDir, title);
		if (extension == null) deltaFileChooser.setAcceptAllFileFilterUsed(true);
		else deltaFileChooser.addChoosableFileFilter(new DefaultFileFilter(extension));
		int result = deltaFileChooser.showSaveDialog(parent);
		if (result == JFileChooser.APPROVE_OPTION)
			return deltaFileChooser.getSelectedFile();
		return null;
	}

	public DeltaFileChooser() {
		this(FileUtils.currentDirFile);
	}

	public DeltaFileChooser(String dialogTitle) {
		this(FileUtils.currentDirFile);
		setDialogTitle(dialogTitle);
	}

	public DeltaFileChooser(String dialogTitle, boolean askToOverwrite) {
		this(FileUtils.currentDirFile);
		setDialogTitle(dialogTitle);
		this.askToOverwrite = askToOverwrite;
	}

	public DeltaFileChooser(File currentDirectory, String dialogTitle) {
		this(currentDirectory);
		setDialogTitle(dialogTitle);
	}

	public DeltaFileChooser(File currentDirectory) {
		super((currentDirectory == null) ? FileUtils.currentDirFile : currentDirectory);
		askToOverwrite = true;
		setAcceptAllFileFilterUsed(false);
		oldDirectory = currentDirectory;
		setFileView(FILE_VIEW);
		addPropertyChangeListener(JFileChooser.DIRECTORY_CHANGED_PROPERTY, this);
		addPropertyChangeListener(JFileChooser.FILE_FILTER_CHANGED_PROPERTY, this);
	}

	public boolean isAskToOverwrite() {
		return askToOverwrite;
	}
	public void setAskToOverwrite(boolean askToOverwrite) {
		this.askToOverwrite = askToOverwrite;
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if  (evt.getPropertyName() == JFileChooser.DIRECTORY_CHANGED_PROPERTY)
			setSelectedFile(null);
		else
			FILE_VIEW.setFileFilter(getFileFilter());
	}

	public void approveSelection() {
		selectedFile = super.getSelectedFile();
		boolean openDialog = (getDialogType() != JFileChooser.SAVE_DIALOG);
		if (selectedFile == null || (!selectedFile.isFile() && (openDialog || selectedFile.exists()))) {
			JOptionPane.showMessageDialog(getParent(), "Please choose an existent File!",
										  (openDialog ? "Open File" : "Save File"), JOptionPane.INFORMATION_MESSAGE);
			// Return - causes the dialog to stay posted
			return ;
		}
		String fileName = selectedFile.getName();
		if (openDialog) {
			if (!selectedFile.canRead()) {
				JOptionPane.showMessageDialog(getParent(),
											  "You do not have permission to read from \"" + fileName + "\"!",
											  "File not readable", JOptionPane.ERROR_MESSAGE    );
				// Return - causes the dialog to stay posted
				return ;
			}
		} else { // save dialog
			FileFilter fileFilter = getFileFilter();
			if (fileFilter instanceof DefaultFileFilter) {
				String ext = ((DefaultFileFilter) fileFilter).getExtension();
				if (!fileName.endsWith(ext)) {
					fileName  = fileName + ext;
					selectedFile = new File(getCurrentDirectory(), fileName);
				}
			} else if (fileFilter == HTML_FILE_FILTER) {
				if (!fileName.endsWith(".html") && !fileName.endsWith(".htm")) {
					fileName  = fileName + ".html";
					selectedFile = new File(getCurrentDirectory(), fileName);
				}
			}
			if (selectedFile.exists()) {
				if ((askToOverwrite || !fileFilter.accept(selectedFile)) &&
						JOptionPane.showConfirmDialog(getParent(),
												  "\"" + fileName + "\" already exists. Do wish to overwrite it?",
												  "Overwrite?", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
						!= JOptionPane.YES_OPTION)
					return;
				if (!selectedFile.canWrite()   ) {
					JOptionPane.showMessageDialog(getParent(),
												  "You do not have permission to write to \"" + fileName + "\"!",
												  "File not Writable", JOptionPane.ERROR_MESSAGE   );
					// Return - causes the dialog to stay posted
					return ;
				}
			}
		}
		// No file selected, or file can be written - let OK action continue
		super.approveSelection();
	}

	public void cancelSelection() {
		setCurrentDirectory(oldDirectory);
		super.cancelSelection();
	}

	public File getSelectedFile() {
		if (selectedFile == null)
			return super.getSelectedFile();
		File f = selectedFile;
		selectedFile = null;
		return f;
	}

	public int showDialog(Component parent, String approveButtonText) {
		if (!getCurrentDirectory().exists())
			super.setCurrentDirectory(oldDirectory);
		else
			oldDirectory = getCurrentDirectory();
		FILE_VIEW.setFileFilter(getFileFilter());
		return super.showDialog(parent, approveButtonText);
	}

	public void setCurrentDirectory(File dir) {
		try {
			if (!dir.isDirectory()) dir = dir.getParentFile();
			if (!dir.exists()) dir = FileUtils.currentDirFile;
			super.setCurrentDirectory(dir); // todo: BUG in JFileChooser!?!
		} catch (Throwable err) {
			System.err.println("BUG in JFileChooser(!?!) at setCurrentDirectory - please complain to Sun");
			err.printStackTrace();
			try {
				super.setCurrentDirectory(null);
				rescanCurrentDirectory();
				super.setCurrentDirectory(dir);
			} catch (Throwable e) {  }
		}
	}

	public static final class DeltaFileView extends FileView {
		private final ListMap<String,XmlLanguage> extensionToLanguagesMap;
		private FileFilter fileFilter;

		private DeltaFileView() {
			extensionToLanguagesMap = new ListMap<String, XmlLanguage>();
		}

		void setFileFilter(FileFilter fileFilter) {
			this.fileFilter = fileFilter;
		}

		public Icon getIcon(File file) {
			if ((file != null) && (file.getName().length() > 3)) { //(file.getParent() != null) && file.isFile()) { // getParent() == null, if file points to a (floppy) drive
				if (file.isDirectory()) return null;

				if (fileFilter instanceof DocTypeFileFilter)
					return ((DocTypeFileFilter) fileFilter).getXmlLanguage().getIcon();

				String ext = FileUtils.getFileExt(file).toLowerCase();
				if ((fileFilter == XML_FILE_FILTER) || (fileFilter == ANY_XML_FILE_FILTER) || (fileFilter == XML_HTML_FILE_FILTER)) {
					if (HTML_FILE_FILTER.accept(file))
						return MiscImages.HTML_FILE_ICON;
					List<XmlLanguage> xmlLanguages = extensionToLanguagesMap.get(ext);
					if (xmlLanguages != null)
						for (XmlLanguage xmlLanguage : xmlLanguages)
							if (xmlLanguage.getFileFilter().accept(file))
								return xmlLanguage.getIcon();
					return MiscImages.XML_FILE_ICON;
				}
				if (fileFilter == HTML_FILE_FILTER)  
					return MiscImages.HTML_FILE_ICON;
				if ((fileFilter == MACROS_FILE_FILTER) || (fileFilter instanceof MacroFileFilter))  //(ext.equals("macros")) //(name.endsWith(".macros"))
					return MiscImages.MACROS_FILE_ICON;
				if (ext.equals(".project"))
					return LazyResourceLoader.getIconRelativeToClass(LazyResourceLoader.PROJECT_FILE_ICON);
				if (ext.equals(".process"))
					return MainImages.PROCESS_ICON;
				if (ext.equals(".lang"))
					return MiscImages.LANG_FILE_ICON;
				if (fileFilter == DTD_FILE_FILTER)
					return MiscImages.DTD_FILE_ICON;
				if (fileFilter == XSL_FILE_FILTER  )
					return MiscImages.XSL_FILE_ICON;
				if (IMAGE_FILE_FILTER.accept(file)  ) {
					Icon icon = new ImageIcon(file.getAbsolutePath());
					if  ((icon.getIconWidth() <= 16) && (icon.getIconHeight() <= 16))
						return icon;
					return MiscImages.ICON_FILE_ICON;
				}
			}
			return null;
		}

		public ListMap<String, XmlLanguage> getExtensionToLanguagesMap() {
			return extensionToLanguagesMap;
		}
	}

	private static final class ProjectAndProcessFileFilter extends FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory()) return true;  // to show directories in FileChooser when this filter is active
			String ext = FileUtils.getFileExt(file).toLowerCase();
			return (ext.equals(".process") || ext.equals(".project"));
		}

		public String getDescription() {
			return "Project/Process Files (*.project | *.process)";
		}
	}

	private static final class HTMLFileFilter extends FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory()) return true;  // to show directories in FileChooser when this filter is active
			String ext = FileUtils.getFileExt(file).toLowerCase();
			return (ext.equals(".html") || ext.equals(".htm"));
		}

		public String getDescription() {
			return "HTML Files (*.html | *.htm)";
		}
	}

	private static final class XSLFileFilter extends FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory()) return true;
			String ext = FileUtils.getFileExt(file).toLowerCase();
			return (ext.equals(".xsl") || ext.equals(".xslt"));
		}

		public String getDescription() {
			return "eXtensible Stylesheet Language (*.xsl | *.xslt)";
		}
	}

	private static final class XMLorHTMLFileFilter extends FileFilter {
		public boolean accept(File file) {
			return HTML_FILE_FILTER.accept(file) || ANY_XML_FILE_FILTER.accept(file);
		}

		public String getDescription() {
			return "All DELT/A Files (XML / HTML)";
		}
	}

	private static final class ImageFileFilter extends FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory()) return true;
			String ext = FileUtils.getFileExt(file).toLowerCase();
			return (ext.equals(".gif") || ext.equals(".png") || ext.equals(".jpeg") || ext.equals(".jpg"));
		}

		public String getDescription() {
			return "All Image Files";
		}
	}

	public static final class MacroFileFilter extends AnyXMLFileFilter {
		private String langID;

		public MacroFileFilter(String langID) {
			this.langID = langID;
			xmlFileExtensionsSet.add(".macros");
		}

		protected ContentHandler getContentHandler() {
			return new AnyXMLContentHandler() {
				public void startElement(String uri, String localName, String qName, Attributes attributes) {
					String langIDAttrValue = attributes.getValue("lang-id");
					successful = localName.equals("gmt-macros") && (langIDAttrValue != null) && langID.equalsIgnoreCase(langIDAttrValue);
					continueParse = false;
				}
			};
		}

		public String getDescription() {
			return "'"+langID + "' Macro file (*.macros)";
		}
	}
}


