package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.*;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.MacroInsertUndoableEdit;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.SideCompoundEdit;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.ExtendedList;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;
import java.awt.*;
import java.awt.event.*;
import java.util.Set;

public class InsertableXMLNodesDialog extends JDialog {
    private final JTabbedPane tabs;
    private DTDHelper dtdHelper;
    private final boolean listNeedsRefresh[];
    private final ExtendedList insertableIntoList, insertableBeforeList, insertableAfterList,
								insertableReplaceList, insertableConvertList, insertableWrapList;
	private final Action insertAction;
    private int tabIndex = 0;
    private JList[] lists;
	private boolean shouldShowAgain;
	private Preferences preferences;

	private static final int INIT_WIDTH = 331; // 264; // without node-count    //261; // without icons on tabs
	private static final int INIT_HEIGHT = 249;

	private static boolean prefShowCountOnTabs = true;

	private static final int INSERT_INTO_LIST_INDEX    = 0;
	private static final int INSERT_AFTER_LIST_INDEX   = 1;
	private static final int INSERT_BEFORE_LIST_INDEX  = 2;
	private static final int INSERT_REPLACE_LIST_INDEX = 3;
	private static final int INSERT_CONVERT_LIST_INDEX = 4;
	private static final int INSERT_WRAP_LIST_INDEX    = 5;
	private static final String[] TAB_TITLES = {"As Child", "After", "Before", "Replace", "Convert", "Wrap"};

	public InsertableXMLNodesDialog(Preferences preferences) {
        super(MainController.mainFrame, "Insertable Nodes", false);
		this.preferences = preferences;
		prefShowCountOnTabs = preferences.getBoolean(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT);
        listNeedsRefresh = new boolean[6];

        JPanel contentPane = new JPanel(new BorderLayout());
        contentPane.setBorder(GlobalObjects.EMPTY_BORDER_2);
        contentPane.setLayout(new TableLayout(new double[][] {{0.5, 0.5}, {TableLayout.FILL, TableLayout.PREFERRED}}, 0, 5));
        rootPane.setContentPane(contentPane);

		insertableIntoList   = new ExtendedList();
		insertableBeforeList  = new ExtendedList();
		insertableAfterList   = new ExtendedList();
		insertableReplaceList = new ExtendedList();
		insertableConvertList = new ExtendedList();
		insertableWrapList   = new ExtendedList();
        lists = new JList[6];
		lists[INSERT_INTO_LIST_INDEX]  = insertableIntoList;
		lists[INSERT_BEFORE_LIST_INDEX]  = insertableBeforeList;
		lists[INSERT_AFTER_LIST_INDEX]  = insertableAfterList;
		lists[INSERT_REPLACE_LIST_INDEX] = insertableReplaceList;
		lists[INSERT_CONVERT_LIST_INDEX] = insertableConvertList;
		lists[INSERT_WRAP_LIST_INDEX]  = insertableWrapList;

        tabs = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
		tabs.addTab(TAB_TITLES[INSERT_INTO_LIST_INDEX],	   MainImages.XML_INSERT_INTO_ICON,	   new BorderPanel(new JScrollPane(insertableIntoList)), 	null);
		tabs.addTab(TAB_TITLES[INSERT_AFTER_LIST_INDEX],   MainImages.XML_INSERT_AFTER_ICON,   new BorderPanel(new JScrollPane(insertableAfterList)), 	null);
		tabs.addTab(TAB_TITLES[INSERT_BEFORE_LIST_INDEX],  MainImages.XML_INSERT_BEFORE_ICON,  new BorderPanel(new JScrollPane(insertableBeforeList)), 	null);
		tabs.addTab(TAB_TITLES[INSERT_REPLACE_LIST_INDEX], MainImages.XML_INSERT_REPLACE_ICON, new BorderPanel(new JScrollPane(insertableReplaceList)),	null);
		tabs.addTab(TAB_TITLES[INSERT_CONVERT_LIST_INDEX], MainImages.XML_INSERT_REPLACE_ICON, new BorderPanel(new JScrollPane(insertableConvertList)),	null);
		tabs.addTab(TAB_TITLES[INSERT_WRAP_LIST_INDEX],	   MainImages.XML_INSERT_REPLACE_ICON, new BorderPanel(new JScrollPane(insertableWrapList)), 	null);
		tabs.setMnemonicAt(INSERT_INTO_LIST_INDEX, 	  'C');
		tabs.setMnemonicAt(INSERT_BEFORE_LIST_INDEX,  'B');
		tabs.setMnemonicAt(INSERT_AFTER_LIST_INDEX,	  'A');
		tabs.setMnemonicAt(INSERT_REPLACE_LIST_INDEX, 'R');
		tabs.setMnemonicAt(INSERT_CONVERT_LIST_INDEX, 'V');
		tabs.setMnemonicAt(INSERT_WRAP_LIST_INDEX, 	  'W');
        tabs.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
				if (prefShowCountOnTabs) insertAction.setEnabled(lists[tabs.getSelectedIndex()].getModel().getSize() != 0);
				else refreshCurrentList();
                Object selectedValue = lists[tabIndex].getSelectedValue();
                tabIndex = tabs.getSelectedIndex();
                int selectedIndex = lists[tabIndex].getSelectedIndex();
                lists[tabIndex].setSelectedValue(selectedValue, true);
                if (lists[tabIndex].getSelectedIndex() == -1)
                    lists[tabIndex].setSelectedIndex(selectedIndex);
            }
        });

		insertAction = new SimpleAction("Insert", MainImages.MACRO_INSERT_ICON, 'I') {
				protected void simpleActionPerformed() {
					int tabIndex = tabs.getSelectedIndex();
					insertNode(lists[tabIndex].getSelectedValue().toString(), tabIndex);
				}
			};
		Action closeAction = new SimpleAction("Close", 'E') {
            protected void simpleActionPerformed() {
				setVisible(false);
            }
        };
        insertableIntoList.addActionListener(insertAction);
        insertableBeforeList.addActionListener(insertAction);
        insertableAfterList.addActionListener(insertAction);
        insertableReplaceList.addActionListener(insertAction);
        insertableConvertList.addActionListener(insertAction);
        insertableWrapList.addActionListener(insertAction);

        contentPane.add(tabs, new TableLayoutConstraints(0, 0, 1, 0, TableLayout.FULL, TableLayout.FULL));
        contentPane.add(new JButton(insertAction), new TableLayoutConstraints(0, 1, 0, 1, TableLayout.LEFT, TableLayout.FULL));
        contentPane.add(new JButton(closeAction), new TableLayoutConstraints(1, 1, 1, 1, TableLayout.RIGHT, TableLayout.FULL));

		rootPane.getActionMap().put("closeAction", closeAction);
		rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "closeAction");

		Rectangle scrnSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
		setBounds(scrnSize.width - INIT_WIDTH, scrnSize.height - INIT_HEIGHT, INIT_WIDTH, INIT_HEIGHT);

		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				if (evt.getID() == ComponentEvent.COMPONENT_RESIZED) {
					final int width = getWidth(), height = getHeight();
					if ((width < INIT_WIDTH) && (height < INIT_HEIGHT))
						setSize(INIT_WIDTH, INIT_HEIGHT);
					else if (width < INIT_WIDTH)
						setSize(INIT_WIDTH, height);
					else if (height < INIT_HEIGHT)
						setSize(width, INIT_HEIGHT);
				}
			}
		});
    }

    public void showDialog(DTDHelper dtdHelper) {
        refreshLists(dtdHelper);
		setVisible(true);
		shouldShowAgain = false;
    }
	public boolean showAgain() {
		if (shouldShowAgain) {
			refreshLists(dtdHelper);
			setVisible(true);
			shouldShowAgain = false;
			return false;
		}
		return true;
	}
	public void hideTemporary() {
		if (isVisible()) {
			shouldShowAgain = true;
			setVisible(false);
		}
	}

    public void refreshLists(DTDHelper dtdHelper) {
        this.dtdHelper = dtdHelper;
		prefShowCountOnTabs = preferences.getBoolean(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT);
        listNeedsRefresh[0] = listNeedsRefresh[1] = listNeedsRefresh[2] = listNeedsRefresh[3] = listNeedsRefresh[4] = listNeedsRefresh[5] = true;
      if (prefShowCountOnTabs)
			 for (int i=0; i<6; i++) refreshList(i);
		else
			refreshCurrentList();
		insertAction.setEnabled(lists[tabs.getSelectedIndex()].getModel().getSize() != 0);
    }

    private void refreshCurrentList() {
        int selectedIndex = tabs.getSelectedIndex();
		refreshList(selectedIndex);
	}

	private void refreshList(int listIndex) {
		switch (listIndex) {
			case INSERT_INTO_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
                    Set elements = dtdHelper.getInsertableIntoElementSet();
                    elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
					elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
					insertableIntoList.init(elements);
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableIntoList.getModel().getSize() + ")");
				break;
			case INSERT_BEFORE_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
                    Set elements = dtdHelper.getInsertableBeforeElementSet();
                    elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
					elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
					insertableBeforeList.init(elements);
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableBeforeList.getModel().getSize() + ")");
				break;
			case INSERT_AFTER_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
                    Set elements = dtdHelper.getInsertableAfterElementSet();
                    elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
					elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
					insertableAfterList.init(elements);
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableAfterList.getModel().getSize() + ")");
				break;
			case INSERT_REPLACE_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
                    Set elements = dtdHelper.getInsertableReplaceElementSet();
                    elements.remove(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
					elements.remove(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
					insertableReplaceList.init(elements);
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableReplaceList.getModel().getSize() + ")");
				break;
			case INSERT_CONVERT_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
					insertableConvertList.init(dtdHelper.getInsertableConvertElementSet());
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableConvertList.getModel().getSize() + ")");
				break;
			case INSERT_WRAP_LIST_INDEX:
				if (listNeedsRefresh[listIndex]) {
					insertableWrapList.init(dtdHelper.getInsertableWrapElementSet());
					listNeedsRefresh[listIndex] = false;
				}
				if (prefShowCountOnTabs)
					tabs.setTitleAt(listIndex, TAB_TITLES[listIndex] + " (" + insertableWrapList.getModel().getSize() + ")");
				break;
		}
	}

	public static void insertNode(String newNodeName, int insertMode) {
		switch (insertMode) {
			case INSERT_INTO_LIST_INDEX:
				MainController.getActiveSide().xmlView.insertNodeInto(newNodeName);
				break;
			case INSERT_BEFORE_LIST_INDEX:
				MainController.getActiveSide().xmlView.insertNodeBefore(newNodeName);
				break;
			case INSERT_AFTER_LIST_INDEX:
				MainController.getActiveSide().xmlView.insertNodeAfter(newNodeName);
				break;
			case INSERT_REPLACE_LIST_INDEX:
				CompoundEdit wrapperEdit = new NodeInsertWrapperUndoableEdit(newNodeName);
				DeltaSideController thisSideController = MainController.getActiveSideController();
				thisSideController.fireDirectUndoableEditHappened(wrapperEdit);
				DeltaSideController otherSideController = thisSideController.otherSideController;
				MainFrame.DeltaSide thisSide = thisSideController.thisSide, otherSide = otherSideController.thisSide;
				Set<Integer> deltaLinks = thisSide.xmlView.insertNodeReplace(newNodeName);
				if (!deltaLinks.isEmpty() && otherSide.deltaView.isLoaded()) {
					thisSide.deltaView.removeDELTALinksFromCollection(deltaLinks, true);   // remove links from gmtLinks which are in the XML view
					otherSide.deltaView.removeDELTALinksFromCollection(deltaLinks, false);  // remove links from gmtLinks which are not in the HTML view
					if (!deltaLinks.isEmpty())
						switch (MainController.removeDeadLinks) {
							case MainController.REMOVE_LINKS_ASK:
								if (JOptionPane.showConfirmDialog(MainController.mainFrame, "Do you want to remove " + deltaLinks.size()
																							+ " corresponding dead link(s) "
																							+ ((deltaLinks.size() > 10)
																							   ? "" : "(IDs=" + deltaLinks.toString() + ") ")
																							+ "in the opposite file?",
																  "Remove dead links", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
									== JOptionPane.NO_OPTION)
									break;
							case MainController.REMOVE_LINKS_YES:
								otherSideController.disableActions(true);
								otherSide.deltaView.setEnabled(false);
								otherSide.deltaView.removeDELTALinks(deltaLinks);
								otherSide.deltaView.setEnabled(true);
								otherSideController.enableActions();
								break;
						}
				}
				wrapperEdit.end();
				MainController.updateUndoActions();
				break;
			case INSERT_CONVERT_LIST_INDEX:
				MainController.getActiveSide().xmlView.insertNodeConvert(newNodeName);
				break;
			case INSERT_WRAP_LIST_INDEX:
				MainController.getActiveSide().xmlView.insertNodeWrap(newNodeName);
		}
	}

	private static final class NodeInsertWrapperUndoableEdit extends SideCompoundEdit {
		public NodeInsertWrapperUndoableEdit(String nodeName) {
			super(false);
			undoPresentationName = "Undo - " + "\"" + nodeName + "\" node replace";
			redoPresentationName = "Redo - " + "\"" + nodeName + "\" node replace";
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof MacroInsertUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}
    private static final class BorderPanel extends JPanel {
        BorderPanel(JComponent content) {
            super(new BorderLayout());
            setBorder(GlobalObjects.EMPTY_BORDER_5);
            add(content);
        }
    }
}
