package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.images.LazyResourceLoader;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.gui.xml.images.XMLImages;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.util.EnvironmentProperties;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.swing.ExtendedButtonGroup;
import at.ac.tuwien.asgaard.swing.SystemColorChooserPanel;
import at.ac.tuwien.asgaard.swing.models.FixedLengthDocument;
import at.ac.tuwien.asgaard.util.FileUtils;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Class PreferencesDialog.... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 */
public final class PreferencesDialog extends AbstractInputDialog {
	/** A return status code - returned if Cancel button has been pressed */
	public static final int RET_CANCEL = 0;
	/** A return status code - returned if OK button has been pressed */
	public static final int RET_OK = 1;

	private final Preferences preferences;

	private final Set<String> changedProperties;

	private static JColorChooser colorChooser;

	private Map oldPreferences;

	private JCheckBox jCheckBoxMacrosSearchOnlyForMacros;
	private JCheckBox jCheckBoxMacrosSearchIgnoreCase;
	private JCheckBox jCheckBoxMacrosSynchronizeStructureView;
	private JCheckBox jCheckBoxMacrosAutoSwitchInsertMode, jCheckBoxMacrosSwitchBackToDefault;

	// appearance settings
	private JCheckBox jCheckBoxAppearanceToolbarsRollover, jCheckBoxAppearanceToolbarsFloatable, jCheckBoxAppearanceDynamicLayout, jCheckBoxAppearanceSplitpanesContinuousLayout;

	private ColorView colorViewActivatedLinkBG, colorViewSelection, colorViewLinkHTMLOverlapping, colorViewHTMLLinkFG, colorViewLinkHTMLDelimiter, colorViewSearchResults, colorViewInvalidXMLNodesFG, colorViewInvalidXMLNodesBG, colorViewLinkFG, colorViewSideBorderActive, colorViewSideBorderInactive;

	private static final Color LABEL_COLOR = new Color(102, 102, 153);
	private static final Border PANEL_EMPTY_BORDER = new EmptyBorder(5, 10, 0, 5);

	private JSlider jSliderMacrosListWidth;
	private JSlider jSliderMacrosListHeight;
	private Action aRestoreDefaults, aRestoreSaved, aSaveNow, aSaveAs, aLoad, aChangePDFViewerPath;
	private JCheckBox jCheckBoxHTMLInsertDELTALinkDelimiter;
	private JTextField jTextFieldHTMLLinkDelimiter1, jTextFieldHTMLLinkDelimiter2, jTextFieldPDFViewerPath;
	private JCheckBox jCheckBoxXMLShowInsertableCount;
	private JSpinner jSpinnerRecentFilesLimit;

	/** Creates new form PreferencesDialog */
	public PreferencesDialog(Frame parent, final Preferences preferences) {
		super(parent, "Preferences", false, true, 530, 485, 517, 400, 0, 0);

		this.preferences = preferences;
		changedProperties = new HashSet<String>();

		installListeners();

		helpAction.putValue(HELP_LINK_KEY, "PreferencesDialog");

		loadPreferences();
	}

	// main for debugging
// public static void main(String[] args) {
//		new PreferencesDialog(new JFrame(), new Preferences()).showDialog();
//	}

	public boolean showDialog() {
		oldPreferences = null;
		changedProperties.clear();
		jCheckBoxClearRecentFileLists.setSelected(false);
		int state = preferences.getInt(Preferences.KEY_XML_DTD_STATE);
		if (state == 0) jCheckBoxDTDLoad.setSelected(false);
		else {
			jCheckBoxDTDLoad.setSelected(true);
			buttonGroupDTDState.setSelected(state - 1);
		}
		super.setVisible(true);
		return true;
	}

	protected boolean onOK() {
		savePreferences();
		return true;
	}
	protected boolean onCancel() {
    //    System.out.println(getSize()); return false;
		if (oldPreferences != null)
			preferences.putAll(oldPreferences);
		loadPreferences();
		changedProperties.clear();
		return true;
	}

	/** @return the return status of this dialog - one of RET_OK or RET_CANCEL */
	public int getReturnStatus() {
		return returnStatus;
	}

	public boolean hasChanged() {
		return !changedProperties.isEmpty();
	}

	public Set<String> getChangedProperties() {
		return changedProperties;
	}

	/**
	 * Method loadPreferences.
	 */
	private void loadPreferences() {
	// general
		jCheckBoxAdvancedMode.setSelected(preferences.getBoolean(Preferences.KEY_ADVANCED_MODE));
		buttonGroupAutoSave.setSelected(preferences.getInt(Preferences.KEY_AUTO_SAVE));
		jCheckBoxLoadLastFiles.setSelected(preferences.getBoolean(Preferences.KEY_OPEN_LAST_FILES));
		jCheckBoxLoadLastProject.setSelected(preferences.getBoolean(Preferences.KEY_OPEN_LAST_PROJECT));
		jSpinnerRecentFilesLimit.setValue(preferences.getInteger(Preferences.KEY_RECENT_FILES_MAX_COUNT));
		jCheckBoxBackupOnSave.setSelected(preferences.getBoolean(Preferences.KEY_BACKUP_ON_SAVE));
		((ExtendedButtonGroup)buttonGroupRemoveDeadLinks).setSelected(preferences.getInt(Preferences.KEY_REMOVE_DEAD_LINKS));

		jCheckBoxAskBeforeRemoveHTMLLinks.setSelected(preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_HTML_LINKS));
		jCheckBoxAskBeforeRemoveXMLNodes.setSelected(preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_NODES));
		jCheckBoxAskBeforeRemoveXMLLinks.setSelected(preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_LINKS));
		jCheckBoxAskBeforeRemoveXMLSubTree.setSelected(preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_SUBTREE));
		jCheckBoxAskBeforeRemoveXMLAttributes.setSelected(preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES));

	// HTML
		jCheckBoxHTMLInsertDELTALinkDelimiter.setSelected(preferences.getBoolean(Preferences.KEY_HTML_USE_LINK_SEPARATOR));
		jTextFieldHTMLLinkDelimiter1.setText(""+preferences.getChar(Preferences.KEY_HTML_LINK_SEPARATOR1, '['));
		jTextFieldHTMLLinkDelimiter2.setText(""+preferences.getChar(Preferences.KEY_HTML_LINK_SEPARATOR2, ']'));
		jTextFieldHTMLLinkDelimiter1.setEnabled(jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected());
		jTextFieldHTMLLinkDelimiter2.setEnabled(jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected());
		int state = preferences.getInt(Preferences.KEY_HTML_CHECK_LINKS);
		if (state == 0)
			jCheckBoxCheckLinks.setSelected(false);
		else {
			jCheckBoxCheckLinks.setSelected(true);
			buttonGroupHTMLCheckLinks.setSelected(state - 1);
		}
		jCheckBoxHTMLRemoveDELTALinksOnSave.setSelected(preferences.getBoolean(Preferences.KEY_HTML_REMOVE_DELTA_LINKS_ON_SAVE));

	// XML
        buttonGroupAutoChangeDocType.setSelected(preferences.getInt(Preferences.KEY_XML_AUTO_CHANGE_DOCTYPE));
		buttonGroupXMLLinkActivation.setSelected(preferences.getBoolean(Preferences.KEY_XML_LINK_SINGLECLICK) ? 0 : 1);
		state = preferences.getInt(Preferences.KEY_XML_DTD_STATE);
		if (state == 0)
			jCheckBoxDTDLoad.setSelected(false);
		else {
			jCheckBoxDTDLoad.setSelected(true);
			buttonGroupDTDState.setSelected(state - 1);
		}
		buttonGroupXMLAskAttrForEach.setSelected(preferences.getInt(Preferences.KEY_XML_ASK_ATTR_FOR_EACH));

		jCheckBoxXMLAttributesTableShowLinkedAttributesButton.setSelected(preferences.getBoolean(Preferences.KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON));
		jCheckBoxXMLAttributesTableShowEditButton.setSelected(preferences.getBoolean(Preferences.KEY_XML_TABLE_EDIT_BUTTON));
		jCheckBoxXMLTreeNodesExpandAllOnLoad.setSelected(preferences.getBoolean(Preferences.KEY_XML_TREE_EXPAND_ALL_ON_LOAD));
		jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode.setSelected(preferences.getBoolean(Preferences.KEY_XML_SELECT_ON_HIGHLIGHT));
		jCheckBoxXMLTreeNodesShowChildCount.setSelected(preferences.getBoolean(Preferences.KEY_XML_SHOW_CHILD_COUNT));
		jCheckBoxXMLTreeNodesShowAttributes.setSelected(preferences.getBoolean(Preferences.KEY_XML_SHOW_ATTRIBUTES));
		jCheckBoxXMLInsertEmptyAttr.setSelected(preferences.getBoolean(Preferences.KEY_XML_INSERT_EMPTY_ATTR));
		jCheckBoxXMLShowInsertableCount.setSelected(preferences.getBoolean(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT));
		jCheckBoxXMLTreeAutoCollapseOnExpand.setSelected(preferences.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND));
		jCheckBoxXMLTreeAutoCollapseOnSelection.setSelected(preferences.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_SELECT));
		jCheckBoxXMLTreeAutoExpandOnSelection.setSelected(preferences.getBoolean(Preferences.KEY_XML_AUTO_EXPAND_ON_SELECT));
		jCheckBoxXMLTreeAutoSelectOnFolding.setSelected(preferences.getBoolean(Preferences.KEY_XML_AUTO_SELECT_ON_FOLDING));
		jCheckBoxXMLTreeDragAndDrop.setSelected(preferences.getBoolean(Preferences.KEY_XML_TREE_DRAGNDROP));
		jCheckBoxXMLTreeAutoCollapseOnSelection.setEnabled(jCheckBoxXMLTreeAutoCollapseOnExpand.isSelected());

		// Macros
		jCheckBoxMacrosAutoSwitchInsertMode.setSelected(preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_AUTO_SWITCH));
		jCheckBoxMacrosSwitchBackToDefault.setSelected(preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_SWITCH_BACK));
		jCheckBoxMacrosSwitchBackToDefault.setEnabled(jCheckBoxMacrosAutoSwitchInsertMode.isSelected());
		jCheckBoxMacrosSynchronizeStructureView.setSelected(preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_SYNC_VIEWS));
		jCheckBoxMacrosSearchIgnoreCase.setSelected(preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_IGNORE_CASE));
		jCheckBoxMacrosSearchOnlyForMacros.setSelected(preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_ONLY_MACROS));
		jSliderMacrosListWidth.setValue(preferences.getInt(Preferences.KEY_MACROS_MIN_LIST_WIDTH));
		jSliderMacrosListHeight.setValue(preferences.getInt(Preferences.KEY_MACROS_MIN_LIST_HEIGHT));

	// appearance
		jCheckBoxRestoreLastLayoutOnStartup.setSelected(preferences.getBoolean(Preferences.KEY_LAYOUT_RESTORE));

		colorViewActivatedLinkBG.setColor(preferences.getColor(Preferences.KEY_COLOR_LINK_ACTIVATED_BG));
		colorViewSelection.setColor(preferences.getColor(Preferences.KEY_COLOR_SELECTION));
		colorViewInvalidXMLNodesFG.setColor(preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_LOAD));
		colorViewInvalidXMLNodesBG.setColor(preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE));
		colorViewLinkFG.setColor(preferences.getColor(Preferences.KEY_COLOR_LINK_FG));
		colorViewHTMLLinkFG.setColor(preferences.getColor(Preferences.KEY_COLOR_HTML_LINK_FG));
		colorViewLinkHTMLDelimiter.setColor(preferences.getColor(Preferences.KEY_COLOR_LINK_HTML_DELIM));
		colorViewSearchResults.setColor(preferences.getColor(Preferences.KEY_COLOR_SEARCH_RESULT));
		colorViewSideBorderActive.setColor(preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_ACTIVE));
		colorViewSideBorderInactive.setColor(preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_INACTIVE));

		jCheckBoxAppearanceXMLTreeRollOver.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_ROLLOVER_XML));
		jCheckBoxAppearanceDynamicLayout.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT));
		jCheckBoxAppearanceSplitpanesContinuousLayout.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT));
		jCheckBoxAppearanceToolbarsFloatable.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_FLOATABLE));
		jCheckBoxAppearanceToolbarsRollover.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER));
		jCheckBoxXMLTreeFishEyeView.setSelected(preferences.getBoolean(Preferences.KEY_APPEARANCE_XML_FISHEYE_VIEW));

		jTextFieldPDFViewerPath.setText(preferences.getString(Preferences.KEY_PATH_TO_PDF_VIEWER));
	}

	private void savePreferences() {
	// general
		if (preferences.getBoolean(Preferences.KEY_ADVANCED_MODE) != jCheckBoxAdvancedMode.isSelected()) {
			changedProperties.add(Preferences.KEY_ADVANCED_MODE);
			preferences.putBoolean(Preferences.KEY_ADVANCED_MODE, jCheckBoxAdvancedMode.isSelected());
		}
		int recentFilesLimit = ((Integer) jSpinnerRecentFilesLimit.getValue()).intValue();
		if (recentFilesLimit != preferences.getInt(Preferences.KEY_RECENT_FILES_MAX_COUNT)) {
			changedProperties.add(Preferences.KEY_RECENT_FILES_MAX_COUNT);
			preferences.putInt(Preferences.KEY_RECENT_FILES_MAX_COUNT, recentFilesLimit);
		}
		if (jCheckBoxClearRecentFileLists.isSelected())
			changedProperties.add(Preferences.KEY_CLEAR_RECENT_FILES);
		int state = buttonGroupAutoSave.getSelected();
		if (preferences.getInt(Preferences.KEY_AUTO_SAVE) != state) {
			changedProperties.add(Preferences.KEY_AUTO_SAVE);
			preferences.putInt(Preferences.KEY_AUTO_SAVE, state);
		}
		if (preferences.getBoolean(Preferences.KEY_OPEN_LAST_FILES) != jCheckBoxLoadLastFiles.isSelected()) {
			changedProperties.add(Preferences.KEY_OPEN_LAST_FILES);
			preferences.putBoolean(Preferences.KEY_OPEN_LAST_FILES, jCheckBoxLoadLastFiles.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_OPEN_LAST_PROJECT) != jCheckBoxLoadLastProject.isSelected()) {
			changedProperties.add(Preferences.KEY_OPEN_LAST_PROJECT);
			preferences.putBoolean(Preferences.KEY_OPEN_LAST_PROJECT, jCheckBoxLoadLastProject.isSelected());
		}

		if (preferences.getBoolean(Preferences.KEY_BACKUP_ON_SAVE) != jCheckBoxBackupOnSave.isSelected()) {
			changedProperties.add(Preferences.KEY_BACKUP_ON_SAVE);
			preferences.putBoolean(Preferences.KEY_BACKUP_ON_SAVE, jCheckBoxBackupOnSave.isSelected());
		}
		state = ((ExtendedButtonGroup)buttonGroupRemoveDeadLinks).getSelected();
		if (preferences.getInt(Preferences.KEY_REMOVE_DEAD_LINKS) != state) {
			changedProperties.add(Preferences.KEY_REMOVE_DEAD_LINKS);
			preferences.putInt(Preferences.KEY_REMOVE_DEAD_LINKS, state);
		}

		if (preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_HTML_LINKS) != jCheckBoxAskBeforeRemoveHTMLLinks.isSelected()) {
			changedProperties.add(Preferences.KEY_ASK_BEFORE_REMOVE_HTML_LINKS);
			preferences.putBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_HTML_LINKS, jCheckBoxAskBeforeRemoveHTMLLinks.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_NODES) != jCheckBoxAskBeforeRemoveXMLNodes.isSelected()) {
			changedProperties.add(Preferences.KEY_ASK_BEFORE_REMOVE_XML_NODES);
			preferences.putBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_NODES, jCheckBoxAskBeforeRemoveXMLNodes.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_LINKS) != jCheckBoxAskBeforeRemoveXMLLinks.isSelected()) {
			changedProperties.add(Preferences.KEY_ASK_BEFORE_REMOVE_XML_LINKS);
			preferences.putBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_LINKS, jCheckBoxAskBeforeRemoveXMLLinks.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_SUBTREE) != jCheckBoxAskBeforeRemoveXMLSubTree.isSelected()) {
			changedProperties.add(Preferences.KEY_ASK_BEFORE_REMOVE_XML_SUBTREE);
			preferences.putBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_SUBTREE, jCheckBoxAskBeforeRemoveXMLSubTree.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES) != jCheckBoxAskBeforeRemoveXMLAttributes.isSelected()) {
			changedProperties.add(Preferences.KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES);
			preferences.putBoolean(Preferences.KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES, jCheckBoxAskBeforeRemoveXMLAttributes.isSelected());
		}

	// HTML
		if (preferences.getBoolean(Preferences.KEY_HTML_USE_LINK_SEPARATOR) != jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected()) {
			changedProperties.add(Preferences.KEY_HTML_USE_LINK_SEPARATOR);
			preferences.putBoolean(Preferences.KEY_HTML_USE_LINK_SEPARATOR, jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected());
		}
		if (jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected()) {
			preferences.putString(Preferences.KEY_HTML_LINK_SEPARATOR1, jTextFieldHTMLLinkDelimiter1.getText());
			preferences.putString(Preferences.KEY_HTML_LINK_SEPARATOR2, jTextFieldHTMLLinkDelimiter2.getText());
		}
		state = (jCheckBoxCheckLinks.isSelected() ? buttonGroupHTMLCheckLinks.getSelected()+1 : 0);
		if (preferences.getInt(Preferences.KEY_HTML_CHECK_LINKS) != state) {
			changedProperties.add(Preferences.KEY_HTML_CHECK_LINKS);
			preferences.putInt(Preferences.KEY_HTML_CHECK_LINKS, state);
		}
		if (preferences.getBoolean(Preferences.KEY_HTML_REMOVE_DELTA_LINKS_ON_SAVE) != jCheckBoxHTMLRemoveDELTALinksOnSave.isSelected()) {
			changedProperties.add(Preferences.KEY_HTML_REMOVE_DELTA_LINKS_ON_SAVE);
			preferences.putBoolean(Preferences.KEY_HTML_REMOVE_DELTA_LINKS_ON_SAVE, jCheckBoxHTMLRemoveDELTALinksOnSave.isSelected());
		}

	// XML
        state = buttonGroupAutoChangeDocType.getSelected();
        if (preferences.getInt(Preferences.KEY_XML_AUTO_CHANGE_DOCTYPE) != state) {
            changedProperties.add(Preferences.KEY_XML_AUTO_CHANGE_DOCTYPE);
            preferences.putInt(Preferences.KEY_XML_AUTO_CHANGE_DOCTYPE, state);
        }
		if (preferences.getBoolean(Preferences.KEY_XML_LINK_SINGLECLICK) != (buttonGroupXMLLinkActivation.getSelected() == 0)) {
			changedProperties.add(Preferences.KEY_XML_LINK_SINGLECLICK);
			preferences.putBoolean(Preferences.KEY_XML_LINK_SINGLECLICK, (buttonGroupXMLLinkActivation.getSelected() == 0));
		}
		state = (jCheckBoxDTDLoad.isSelected() ? buttonGroupDTDState.getSelected()+1 : 0);
		if (preferences.getInt(Preferences.KEY_XML_DTD_STATE) != state) {
			changedProperties.add(Preferences.KEY_XML_DTD_STATE);
			preferences.putInt(Preferences.KEY_XML_DTD_STATE, state);
		}

		if (preferences.getBoolean(Preferences.KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON) != jCheckBoxXMLAttributesTableShowLinkedAttributesButton.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON);
			preferences.putBoolean(Preferences.KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON, jCheckBoxXMLAttributesTableShowLinkedAttributesButton.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_TABLE_EDIT_BUTTON) != jCheckBoxXMLAttributesTableShowEditButton.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_TABLE_EDIT_BUTTON);
			preferences.putBoolean(Preferences.KEY_XML_TABLE_EDIT_BUTTON, jCheckBoxXMLAttributesTableShowEditButton.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_TREE_EXPAND_ALL_ON_LOAD) != jCheckBoxXMLTreeNodesExpandAllOnLoad.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_TREE_EXPAND_ALL_ON_LOAD);
			preferences.putBoolean(Preferences.KEY_XML_TREE_EXPAND_ALL_ON_LOAD, jCheckBoxXMLTreeNodesExpandAllOnLoad.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_SELECT_ON_HIGHLIGHT) != jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_SELECT_ON_HIGHLIGHT);
			preferences.putBoolean(Preferences.KEY_XML_SELECT_ON_HIGHLIGHT, jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_SHOW_CHILD_COUNT) != jCheckBoxXMLTreeNodesShowChildCount.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_SHOW_CHILD_COUNT);
			preferences.putBoolean(Preferences.KEY_XML_SHOW_CHILD_COUNT, jCheckBoxXMLTreeNodesShowChildCount.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_SHOW_ATTRIBUTES) != jCheckBoxXMLTreeNodesShowAttributes.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_SHOW_ATTRIBUTES);
			preferences.putBoolean(Preferences.KEY_XML_SHOW_ATTRIBUTES, jCheckBoxXMLTreeNodesShowAttributes.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_INSERT_EMPTY_ATTR) != jCheckBoxXMLInsertEmptyAttr.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_INSERT_EMPTY_ATTR);
			preferences.putBoolean(Preferences.KEY_XML_INSERT_EMPTY_ATTR, jCheckBoxXMLInsertEmptyAttr.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT) != jCheckBoxXMLShowInsertableCount.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT);
			preferences.putBoolean(Preferences.KEY_XML_SHOW_INSERTABLE_COUNT, jCheckBoxXMLShowInsertableCount.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND) != jCheckBoxXMLTreeAutoCollapseOnExpand.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND);
			preferences.putBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND, jCheckBoxXMLTreeAutoCollapseOnExpand.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_SELECT) != jCheckBoxXMLTreeAutoCollapseOnSelection.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_AUTO_COLLAPSE_ON_SELECT);
			preferences.putBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_SELECT, jCheckBoxXMLTreeAutoCollapseOnSelection.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_AUTO_EXPAND_ON_SELECT) != jCheckBoxXMLTreeAutoExpandOnSelection.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_AUTO_EXPAND_ON_SELECT);
			preferences.putBoolean(Preferences.KEY_XML_AUTO_EXPAND_ON_SELECT, jCheckBoxXMLTreeAutoExpandOnSelection.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_AUTO_SELECT_ON_FOLDING) != jCheckBoxXMLTreeAutoSelectOnFolding.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_AUTO_SELECT_ON_FOLDING);
			preferences.putBoolean(Preferences.KEY_XML_AUTO_SELECT_ON_FOLDING, jCheckBoxXMLTreeAutoSelectOnFolding.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_XML_TREE_DRAGNDROP) != jCheckBoxXMLTreeDragAndDrop.isSelected()) {
			changedProperties.add(Preferences.KEY_XML_TREE_DRAGNDROP);
			preferences.putBoolean(Preferences.KEY_XML_TREE_DRAGNDROP, jCheckBoxXMLTreeDragAndDrop.isSelected());
		}

		state = buttonGroupXMLAskAttrForEach.getSelected();
		if (preferences.getInt(Preferences.KEY_XML_ASK_ATTR_FOR_EACH) != state) {
			changedProperties.add(Preferences.KEY_XML_ASK_ATTR_FOR_EACH);
			preferences.putInt(Preferences.KEY_XML_ASK_ATTR_FOR_EACH, state);
		}

	// appearance
		if (preferences.getBoolean(Preferences.KEY_LAYOUT_RESTORE) != jCheckBoxRestoreLastLayoutOnStartup.isSelected()) {
			changedProperties.add(Preferences.KEY_LAYOUT_RESTORE);
			preferences.putBoolean(Preferences.KEY_LAYOUT_RESTORE, jCheckBoxRestoreLastLayoutOnStartup.isSelected());
		}

		if (!colorViewActivatedLinkBG.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_LINK_ACTIVATED_BG))) {
			changedProperties.add(Preferences.KEY_COLOR_LINK_ACTIVATED_BG);
			preferences.putColor(Preferences.KEY_COLOR_LINK_ACTIVATED_BG, colorViewActivatedLinkBG.getColor());
		}
		if (!colorViewSelection.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_SELECTION))) {
			changedProperties.add(Preferences.KEY_COLOR_SELECTION);
			preferences.putColor(Preferences.KEY_COLOR_SELECTION, colorViewSelection.getColor());
		}
		if (!colorViewInvalidXMLNodesFG.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_LOAD))) {
			changedProperties.add(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_LOAD);
			preferences.putColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_LOAD, colorViewInvalidXMLNodesFG.getColor());
		}
		if (!colorViewInvalidXMLNodesBG.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE))) {
			changedProperties.add(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE);
			preferences.putColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE, colorViewInvalidXMLNodesBG.getColor());
		}
		if (!colorViewLinkFG.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_LINK_FG))) {
			changedProperties.add(Preferences.KEY_COLOR_LINK_FG);
			preferences.putColor(Preferences.KEY_COLOR_LINK_FG, colorViewLinkFG.getColor());
		}
		if (!colorViewHTMLLinkFG.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_HTML_LINK_FG))) {
			changedProperties.add(Preferences.KEY_COLOR_HTML_LINK_FG);
			preferences.putColor(Preferences.KEY_COLOR_HTML_LINK_FG, colorViewHTMLLinkFG.getColor());
		}
		if (!colorViewLinkHTMLDelimiter.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_LINK_HTML_DELIM))) {
			changedProperties.add(Preferences.KEY_COLOR_LINK_HTML_DELIM);
			preferences.putColor(Preferences.KEY_COLOR_LINK_HTML_DELIM, colorViewLinkHTMLDelimiter.getColor());
		}
		if (!colorViewSearchResults.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_SEARCH_RESULT))) {
			changedProperties.add(Preferences.KEY_COLOR_SEARCH_RESULT);
			preferences.putColor(Preferences.KEY_COLOR_SEARCH_RESULT, colorViewSearchResults.getColor());
		}
		if (!colorViewSideBorderActive.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_ACTIVE))) {
			changedProperties.add(Preferences.KEY_COLOR_SIDE_BORDER_ACTIVE);
			preferences.putColor(Preferences.KEY_COLOR_SIDE_BORDER_ACTIVE, colorViewSideBorderActive.getColor());
		}
		if (!colorViewSideBorderInactive.getColor().equals(preferences.getColor(Preferences.KEY_COLOR_SIDE_BORDER_INACTIVE))) {
			changedProperties.add(Preferences.KEY_COLOR_SIDE_BORDER_INACTIVE);
			preferences.putColor(Preferences.KEY_COLOR_SIDE_BORDER_INACTIVE, colorViewSideBorderInactive.getColor());
		}

		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_ROLLOVER_XML) != jCheckBoxAppearanceXMLTreeRollOver.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_ROLLOVER_XML);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_ROLLOVER_XML, jCheckBoxAppearanceXMLTreeRollOver.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT) != jCheckBoxAppearanceDynamicLayout.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_DYNAMIC_LAYOUT, jCheckBoxAppearanceDynamicLayout.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT) != jCheckBoxAppearanceSplitpanesContinuousLayout.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_CONTINUOUS_LAYOUT, jCheckBoxAppearanceSplitpanesContinuousLayout.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_FLOATABLE) != jCheckBoxAppearanceToolbarsFloatable.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_TOOLBARS_FLOATABLE);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_FLOATABLE, jCheckBoxAppearanceToolbarsFloatable.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER) != jCheckBoxAppearanceToolbarsRollover.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_TOOLBARS_ROLLOVER, jCheckBoxAppearanceToolbarsRollover.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_APPEARANCE_XML_FISHEYE_VIEW) != jCheckBoxXMLTreeFishEyeView.isSelected()) {
			changedProperties.add(Preferences.KEY_APPEARANCE_XML_FISHEYE_VIEW);
			preferences.putBoolean(Preferences.KEY_APPEARANCE_XML_FISHEYE_VIEW, jCheckBoxXMLTreeFishEyeView.isSelected());
		}


		// Macros
		if (preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_AUTO_SWITCH) != jCheckBoxMacrosAutoSwitchInsertMode.isSelected()) {
			changedProperties.add(Preferences.KEY_MACROS_INSERT_MODE_AUTO_SWITCH);
			preferences.putBoolean(Preferences.KEY_MACROS_INSERT_MODE_AUTO_SWITCH, jCheckBoxMacrosAutoSwitchInsertMode.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_MACROS_INSERT_MODE_SWITCH_BACK) != jCheckBoxMacrosSwitchBackToDefault.isSelected()) {
			changedProperties.add(Preferences.KEY_MACROS_INSERT_MODE_SWITCH_BACK);
			preferences.putBoolean(Preferences.KEY_MACROS_INSERT_MODE_SWITCH_BACK, jCheckBoxMacrosSwitchBackToDefault.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_SYNC_VIEWS) != jCheckBoxMacrosSynchronizeStructureView.isSelected()) {
			changedProperties.add(Preferences.KEY_MACROS_SEARCH_SYNC_VIEWS);
			preferences.putBoolean(Preferences.KEY_MACROS_SEARCH_SYNC_VIEWS, jCheckBoxMacrosSynchronizeStructureView.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_IGNORE_CASE) != jCheckBoxMacrosSearchIgnoreCase.isSelected()) {
			changedProperties.add(Preferences.KEY_MACROS_SEARCH_IGNORE_CASE);
			preferences.putBoolean(Preferences.KEY_MACROS_SEARCH_IGNORE_CASE, jCheckBoxMacrosSearchIgnoreCase.isSelected());
		}
		if (preferences.getBoolean(Preferences.KEY_MACROS_SEARCH_ONLY_MACROS) != jCheckBoxMacrosSearchOnlyForMacros.isSelected()) {
			changedProperties.add(Preferences.KEY_MACROS_SEARCH_ONLY_MACROS);
			preferences.putBoolean(Preferences.KEY_MACROS_SEARCH_ONLY_MACROS, jCheckBoxMacrosSearchOnlyForMacros.isSelected());
		}
		if (preferences.getInt(Preferences.KEY_MACROS_MIN_LIST_WIDTH) != jSliderMacrosListWidth.getValue()) {
			changedProperties.add(Preferences.KEY_MACROS_MIN_LIST_WIDTH);
			preferences.putInt(Preferences.KEY_MACROS_MIN_LIST_WIDTH, jSliderMacrosListWidth.getValue());
		}
		if (preferences.getInt(Preferences.KEY_MACROS_MIN_LIST_HEIGHT) != jSliderMacrosListHeight.getValue()) {
			changedProperties.add(Preferences.KEY_MACROS_MIN_LIST_HEIGHT);
			preferences.putInt(Preferences.KEY_MACROS_MIN_LIST_HEIGHT, jSliderMacrosListHeight.getValue());
		}

		preferences.putString(Preferences.KEY_PATH_TO_PDF_VIEWER, jTextFieldPDFViewerPath.getText());
	}

	private void installListeners() {
		jCheckBoxAskBeforeRemoveXMLNodes.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if (jCheckBoxAskBeforeRemoveXMLNodes.isSelected()) {
					jCheckBoxAskBeforeRemoveXMLLinks.setSelected(true);
					jCheckBoxAskBeforeRemoveXMLLinks.setEnabled(false);
					jCheckBoxAskBeforeRemoveXMLSubTree.setSelected(true);
					jCheckBoxAskBeforeRemoveXMLSubTree.setEnabled(false);
				} else {
					jCheckBoxAskBeforeRemoveXMLLinks.setEnabled(true);
					jCheckBoxAskBeforeRemoveXMLSubTree.setEnabled(true);
				}
			}
		});
		jCheckBoxCheckLinks.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Enumeration radios = buttonGroupHTMLCheckLinks.getElements();
				for (int i = buttonGroupHTMLCheckLinks.getButtonCount() - 1; i >= 0; i--)
					((JRadioButton) radios.nextElement()).setEnabled(jCheckBoxCheckLinks.isSelected());
			}
		});
		jCheckBoxDTDLoad.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				Enumeration radios = buttonGroupDTDState.getElements();
				for (int i = buttonGroupDTDState.getButtonCount() - 1; i >= 0; i--)
					((JRadioButton) radios.nextElement()).setEnabled(jCheckBoxDTDLoad.isSelected());
			}
		});

		jCheckBoxMacrosAutoSwitchInsertMode.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				jCheckBoxMacrosSwitchBackToDefault.setEnabled(jCheckBoxMacrosAutoSwitchInsertMode.isSelected());
			}
		});

		jCheckBoxHTMLInsertDELTALinkDelimiter.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				jTextFieldHTMLLinkDelimiter1.setEnabled(jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected());
				jTextFieldHTMLLinkDelimiter2.setEnabled(jCheckBoxHTMLInsertDELTALinkDelimiter.isSelected());
			}
		});
		jCheckBoxXMLTreeAutoCollapseOnExpand.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				jCheckBoxXMLTreeAutoCollapseOnSelection.setEnabled(jCheckBoxXMLTreeAutoCollapseOnExpand.isSelected());
				jCheckBoxXMLTreeAutoCollapseOnSelection.setSelected(false);
			}
		});

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				doClose(RET_CANCEL);
			}
		});
	}

	private void doClose(int retStatus) {
		returnStatus = retStatus;
		setVisible(false);
	}

	protected void createActions() {
		super.createActions();

		aRestoreDefaults = new SimpleAction("Restore Defaults", 'D', "restore default values") {
			protected void simpleActionPerformed() {
				PreferencesDialog.this.oldPreferences = (Map) preferences.clone();
				preferences.restoreDefaults();
				loadPreferences();
				changedProperties.add(Preferences.KEY_RESTORED_DEFAULT);
			}
		};
		aLoad = new SimpleAction("Load ...", null, 'L', "Load preferences from disk", false) {
			protected void simpleActionPerformed() {
				// todo
			}
		};
		aRestoreSaved = new SimpleAction("Restore to saved", null, 'E', "Load last saved preferences from disk", false) {
			protected void simpleActionPerformed() {
				// todo
			}
		};
		aSaveNow = new SimpleAction("Save now", null, 'S', "Save preferences to default file", false) {
			protected void simpleActionPerformed() {
				// todo
			}
		};
		aSaveAs = new SimpleAction("Save as ...", null, 'A', "Save preferences to another file", false) {
			protected void simpleActionPerformed() {
				// todo
			}
		};
		aChangePDFViewerPath = new SimpleAction(MiscImages.DOTS_ICON) {
			protected void simpleActionPerformed() {
				File file = DeltaFileChooser.showOpenFileChooser(PreferencesDialog.this, "Choose PDF Viewer executable", FileUtils.getDir(jTextFieldPDFViewerPath.getText()), EnvironmentProperties.OS_WINDOWS ? "exe" : null);
				if (file != null)
					jTextFieldPDFViewerPath.setText(FileUtils.getAbsolutePath(file));
			}
		};
	}

	protected Component createMainComponent() {
		JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane.insertTab("General", LazyResourceLoader.getIconRelativeToClass(LazyResourceLoader.APP_ICON), createGeneralPanel(), null, 0);
		tabbedPane.insertTab("HTML", MainImages.HTML_ICON, createHTMLPanel(), null, 1);
		tabbedPane.insertTab("XML", MainImages.XML_ICON, createXMLPanel(), null, 2);
		tabbedPane.insertTab("Macros", MainImages.MACRO_ICON, createMacrosPanel(), null, 3);
		tabbedPane.insertTab("Appearance", MainImages.VIEW_ICON, createAppearancePanel(), null, 4);
		tabbedPane.insertTab("Tools", XMLImages.PI_NODE_ICON, createToolsPanel(), null, 5);
		return tabbedPane;
	}

	private Component createGeneralPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(PANEL_EMPTY_BORDER);

		JPanel rowPanel = new JPanel(new BorderLayout());
		jCheckBoxAdvancedMode = new JCheckBox("Advanced Mode");
		jCheckBoxAdvancedMode.setToolTipText("enables much more features; DELT/A need to be restarted");
		rowPanel.add(jCheckBoxAdvancedMode, BorderLayout.WEST);
		panel.add(rowPanel);

		// auto save
		rowPanel = new JPanel(new BorderLayout());
		JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		rowPanel.add(new BlueLabel("Auto-save changes:"), BorderLayout.WEST);
		rowPanel.setToolTipText("Save changes on new / open / reload / revert / close / exit");
		buttonGroupAutoSave = new ExtendedButtonGroup();

		JRadioButton radioButton = new JRadioButton("yes");
		radioButton.setMnemonic(KeyEvent.VK_Y);
		rightPanel.add(radioButton);
		buttonGroupAutoSave.add(radioButton);

		radioButton = new JRadioButton("ask");
		radioButton.setMnemonic(KeyEvent.VK_A);
		rightPanel.add(radioButton);
		buttonGroupAutoSave.add(radioButton);

		radioButton = new JRadioButton("no");
		radioButton.setMnemonic(KeyEvent.VK_N);
		rightPanel.add(radioButton);
		buttonGroupAutoSave.add(radioButton);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		// load on startup
		rowPanel = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		rowPanel.add(new BlueLabel("load on Startup:"), BorderLayout.WEST);

		jCheckBoxLoadLastProject = new JCheckBox("last Project");
		rightPanel.add(jCheckBoxLoadLastProject);

		jCheckBoxLoadLastFiles = new JCheckBox("last Files");
		rightPanel.add(jCheckBoxLoadLastFiles);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		// recent files
		rowPanel = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		rowPanel.add(new BlueLabel("Recent Files Lists:"), BorderLayout.WEST);
		rightPanel.add(new JLabel("Limit: "));
		jSpinnerRecentFilesLimit = new JSpinner(new SpinnerNumberModel(10, 1, 20, 1));
		jSpinnerRecentFilesLimit.setEditor(new JSpinner.NumberEditor(jSpinnerRecentFilesLimit));  // todo: do I really need this?
		rightPanel.add(jSpinnerRecentFilesLimit);

		jCheckBoxClearRecentFileLists = new JCheckBox("Clear");
		rightPanel.add(jCheckBoxClearRecentFileLists);
		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		// backup
		rowPanel = new JPanel(new BorderLayout());
		jCheckBoxBackupOnSave = new JCheckBox("Create Backups when saving files");
		rowPanel.add(jCheckBoxBackupOnSave, BorderLayout.WEST);
		rowPanel.setBorder(new EmptyBorder(0, 0, 10, 0));  //new CompoundBorder(new MatteBorder(0, 0, 1, 0, Color.BLACK), new EmptyBorder(0, 0, 5, 0)));
		panel.add(rowPanel);

		// confirm
		rowPanel = new JPanel(new TableLayout(new double[][]{{20.0, TableLayout.PREFERRED, TableLayout.PREFERRED},
															 {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		rowPanel.setBorder(new TitledBorder(null, " Confirm remove of ... ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR ));
		panel.add(rowPanel);

		jCheckBoxAskBeforeRemoveHTMLLinks = new JCheckBox("HTML - DELT/A-Link(s)");
		rowPanel.add(jCheckBoxAskBeforeRemoveHTMLLinks, new TableLayoutConstraints(0, 0, 1, 0));
		jCheckBoxAskBeforeRemoveXMLNodes = new JCheckBox("any XML - Node");
		rowPanel.add(jCheckBoxAskBeforeRemoveXMLNodes, new TableLayoutConstraints(0, 1, 1, 1));

		jCheckBoxAskBeforeRemoveXMLLinks = new JCheckBox("XML - DELT/A-Link(s)");
		rowPanel.add(jCheckBoxAskBeforeRemoveXMLLinks, new TableLayoutConstraints(1, 2));
		jCheckBoxAskBeforeRemoveXMLSubTree = new JCheckBox("XML - Sub-Tree");
		rowPanel.add(jCheckBoxAskBeforeRemoveXMLSubTree, new TableLayoutConstraints(2, 2));

		jCheckBoxAskBeforeRemoveXMLAttributes = new JCheckBox("XML - Attributes");
		rowPanel.add(jCheckBoxAskBeforeRemoveXMLAttributes, new TableLayoutConstraints(0, 3, 1, 3));

		// auto remove dead links
		rowPanel = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		rowPanel.add(new BlueLabel("Auto-Remove dead DELT/A-Links: "), BorderLayout.WEST);
		rowPanel.setToolTipText("dead links are links that have no counterparts anymore");
		buttonGroupRemoveDeadLinks = new ExtendedButtonGroup();

		radioButton = new JRadioButton("yes");
		radioButton.setMnemonic(KeyEvent.VK_Y);
		rightPanel.add(radioButton);
		buttonGroupRemoveDeadLinks.add(radioButton);

		radioButton = new JRadioButton("ask");
		radioButton.setMnemonic(KeyEvent.VK_A);
		rightPanel.add(radioButton);
		buttonGroupRemoveDeadLinks.add(radioButton);

		radioButton = new JRadioButton("no");
		radioButton.setMnemonic(KeyEvent.VK_N);
		rightPanel.add(radioButton);
		buttonGroupRemoveDeadLinks.add(radioButton);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		jTextFieldPDFViewerPath = new JTextField();
		JPanel pdfViewerPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, 20.0}, {20.0}}));
		pdfViewerPanel.add(new JLabel("Path to PDF Viewer:  "), new TableLayoutConstraints(0, 0));
		pdfViewerPanel.add(jTextFieldPDFViewerPath, new TableLayoutConstraints(1, 0));
		pdfViewerPanel.add(new JButton(aChangePDFViewerPath), new TableLayoutConstraints(2, 0));

		panel.add(pdfViewerPanel);

		JPanel dummyPanel = new JPanel(new BorderLayout());
		dummyPanel.add(panel, BorderLayout.NORTH);
		return dummyPanel;
	}

	private Component createHTMLPanel() {
		JPanel htmlPanel = new JPanel();
		htmlPanel.setLayout(new BoxLayout(htmlPanel, BoxLayout.Y_AXIS));
		htmlPanel.setBorder(PANEL_EMPTY_BORDER);

		JPanel rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL}, {0.5, 0.5}}));
		jCheckBoxHTMLInsertDELTALinkDelimiter = new JCheckBox("Insert DELT/A-Link delimiter");
		rowPanel.add(jCheckBoxHTMLInsertDELTALinkDelimiter, new TableLayoutConstraints(0, 0));
		rowPanel.add(new JLabel("before text: "), new TableLayoutConstraints(1, 0, 1, 0, TableLayoutConstraints.RIGHT, TableLayoutConstraints.FULL));
		jTextFieldHTMLLinkDelimiter1 = new JTextField(new FixedLengthDocument(1), "[", 1);
		rowPanel.add(jTextFieldHTMLLinkDelimiter1, new TableLayoutConstraints(2, 0, 2, 0, TableLayoutConstraints.LEFT, TableLayoutConstraints.FULL));
		rowPanel.add(new JLabel("behind text: "), new TableLayoutConstraints(1, 1, 1, 1, TableLayoutConstraints.RIGHT, TableLayoutConstraints.FULL));
		jTextFieldHTMLLinkDelimiter2 = new JTextField(new FixedLengthDocument(1), "]", 1);
		rowPanel.add(jTextFieldHTMLLinkDelimiter2, new TableLayoutConstraints(2, 1, 2, 1, TableLayoutConstraints.LEFT, TableLayoutConstraints.FULL));
		htmlPanel.add(rowPanel);

		// check links
		rowPanel = new JPanel(new BorderLayout());
		JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		jCheckBoxCheckLinks = new JCheckBox("Check HTML Links: ");
		jCheckBoxCheckLinks.setForeground(LABEL_COLOR);
		rowPanel.add(jCheckBoxCheckLinks, BorderLayout.WEST);

		buttonGroupHTMLCheckLinks = new ExtendedButtonGroup();
		JRadioButton radioButton = new JRadioButton("On Link Activation");
		radioButton.setMnemonic(KeyEvent.VK_A);
		rightPanel.add(radioButton);
		buttonGroupHTMLCheckLinks.add(radioButton);

		radioButton = new JRadioButton("On Mouse over");
		radioButton.setMnemonic(KeyEvent.VK_O);
		rightPanel.add(radioButton);
		buttonGroupHTMLCheckLinks.add(radioButton);

		radioButton = new JRadioButton("On Load");
		radioButton.setMnemonic(KeyEvent.VK_L);
		rightPanel.add(radioButton);
		buttonGroupHTMLCheckLinks.add(radioButton);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		htmlPanel.add(rowPanel);

		// remove links
		rowPanel = new JPanel(new BorderLayout());
		jCheckBoxHTMLRemoveDELTALinksOnSave = new JCheckBox("Remove DELT/A-Links on Save and re-insert on Load");
		rowPanel.add(jCheckBoxHTMLRemoveDELTALinksOnSave, BorderLayout.WEST);
		htmlPanel.add(rowPanel);

		JPanel dummyPanel = new JPanel(new BorderLayout());
		dummyPanel.add(htmlPanel, BorderLayout.NORTH);
		return dummyPanel;
	}

	private Component createXMLPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(PANEL_EMPTY_BORDER);

        // auto change DocType
        JPanel rowPanel = new JPanel(new BorderLayout());
        JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        rowPanel.add(new BlueLabel("Auto-change DocType:"), BorderLayout.WEST);
        rowPanel.setToolTipText("Auto-change DocType entry on open");
        buttonGroupAutoChangeDocType = new ExtendedButtonGroup();

        JRadioButton radioButton = new JRadioButton("yes");
        radioButton.setMnemonic(KeyEvent.VK_Y);
        rightPanel.add(radioButton);
        buttonGroupAutoChangeDocType.add(radioButton);

        radioButton = new JRadioButton("ask");
        radioButton.setMnemonic(KeyEvent.VK_A);
        rightPanel.add(radioButton);
        buttonGroupAutoChangeDocType.add(radioButton);

        radioButton = new JRadioButton("no");
        radioButton.setMnemonic(KeyEvent.VK_N);
        rightPanel.add(radioButton);
        buttonGroupAutoChangeDocType.add(radioButton);

        rowPanel.add(rightPanel, BorderLayout.CENTER);
        panel.add(rowPanel);

		// check dtd
		rowPanel = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 0));
		jCheckBoxDTDLoad = new JCheckBox("Load DTD: ");
		jCheckBoxDTDLoad.setForeground(LABEL_COLOR);
		rowPanel.add(jCheckBoxDTDLoad, BorderLayout.WEST);

		buttonGroupDTDState = new ExtendedButtonGroup();
		radioButton = new JRadioButton("No Check");
		radioButton.setMnemonic(KeyEvent.VK_N);
		rightPanel.add(radioButton);
		buttonGroupDTDState.add(radioButton);

		radioButton = new JRadioButton("Simple Check");
		radioButton.setToolTipText("don't consider number of allowed children");
		radioButton.setMnemonic(KeyEvent.VK_C);
		rightPanel.add(radioButton);
		buttonGroupDTDState.add(radioButton);

		radioButton = new JRadioButton("Strict Check");
		radioButton.setToolTipText("ensures that the content stays valid after insertion");
		radioButton.setMnemonic(KeyEvent.VK_S);
		rightPanel.add(radioButton);
		buttonGroupDTDState.add(radioButton);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		// link activation
		rowPanel = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 0));
		rowPanel.add(new BlueLabel("DELT/A-Link Activation:"), BorderLayout.WEST);
		buttonGroupXMLLinkActivation = new ExtendedButtonGroup();

		radioButton = new JRadioButton("Single Click");
		radioButton.setMnemonic(KeyEvent.VK_S);
		rightPanel.add(radioButton);
		buttonGroupXMLLinkActivation.add(radioButton);

		radioButton = new JRadioButton("Double Click");
		radioButton.setMnemonic(KeyEvent.VK_D);
		rightPanel.add(radioButton);
		buttonGroupXMLLinkActivation.add(radioButton);

		rowPanel.add(rightPanel, BorderLayout.CENTER);
		panel.add(rowPanel);

		// treenodes
		rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED, TableLayout.PREFERRED}, {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		rowPanel.setBorder(new TitledBorder(null, " XML-Treenodes ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));

		jCheckBoxXMLTreeNodesShowChildCount = new JCheckBox("Show number of children");
		jCheckBoxXMLTreeNodesShowChildCount.setMnemonic(KeyEvent.VK_N);
		rowPanel.add(jCheckBoxXMLTreeNodesShowChildCount, new TableLayoutConstraints(0, 0));
		jCheckBoxXMLTreeNodesShowAttributes = new JCheckBox("Show attributes");
		jCheckBoxXMLTreeNodesShowAttributes.setMnemonic(KeyEvent.VK_A);
		rowPanel.add(jCheckBoxXMLTreeNodesShowAttributes, new TableLayoutConstraints(1, 0));

		jCheckBoxXMLTreeNodesExpandAllOnLoad = new JCheckBox("Expand all tree nodes on load");
		jCheckBoxXMLTreeNodesExpandAllOnLoad.setMnemonic(KeyEvent.VK_L);
		rowPanel.add(jCheckBoxXMLTreeNodesExpandAllOnLoad, new TableLayoutConstraints(0, 1));
		jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode = new JCheckBox("Select current highlighted node");
		jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode.setMnemonic(KeyEvent.VK_H);
		rowPanel.add(jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode, new TableLayoutConstraints(1, 1));

		jCheckBoxXMLTreeAutoCollapseOnExpand = new JCheckBox("Auto collapse previous node on expand");
		jCheckBoxXMLTreeAutoCollapseOnExpand.setMnemonic(KeyEvent.VK_C);
		rowPanel.add(jCheckBoxXMLTreeAutoCollapseOnExpand, new TableLayoutConstraints(0, 2));
		jCheckBoxXMLTreeAutoCollapseOnSelection = new JCheckBox("... also on selection");
		jCheckBoxXMLTreeAutoCollapseOnSelection.setMnemonic(KeyEvent.VK_L);
		rowPanel.add(jCheckBoxXMLTreeAutoCollapseOnSelection, new TableLayoutConstraints(1, 2));

		jCheckBoxXMLTreeAutoSelectOnFolding = new JCheckBox("Auto select on folding");
		jCheckBoxXMLTreeAutoSelectOnFolding.setMnemonic(KeyEvent.VK_S);
		rowPanel.add(jCheckBoxXMLTreeAutoSelectOnFolding, new TableLayoutConstraints(0, 3));
		jCheckBoxXMLTreeAutoExpandOnSelection = new JCheckBox("Auto expand on selection");
		jCheckBoxXMLTreeAutoExpandOnSelection.setMnemonic(KeyEvent.VK_E);
		rowPanel.add(jCheckBoxXMLTreeAutoExpandOnSelection, new TableLayoutConstraints(1, 3));

		jCheckBoxXMLTreeDragAndDrop = new JCheckBox("Drag-and-Drop of tree nodes");
		jCheckBoxXMLTreeDragAndDrop.setFont(jCheckBoxXMLTreeDragAndDrop.getFont().deriveFont(Font.ITALIC));
		jCheckBoxXMLTreeDragAndDrop.setMnemonic(KeyEvent.VK_D);
		rowPanel.add(jCheckBoxXMLTreeDragAndDrop, new TableLayoutConstraints(0, 4));
		rowPanel.add(new JLabel("... not yet finished - use with caution"), new TableLayoutConstraints(1, 4));
		panel.add(rowPanel);

		// attributes
		rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.FILL}, {TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		rowPanel.setBorder(new TitledBorder(null, " Attribute value input ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));

		JPanel rowPanel2 = new JPanel(new BorderLayout());
		rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 5, 0));
		rowPanel2.add(new BlueLabel("Ask attribute values for each ..."), BorderLayout.WEST);
		buttonGroupXMLAskAttrForEach = new ExtendedButtonGroup();

		radioButton = new JRadioButton("Attribute");
		radioButton.setMnemonic(KeyEvent.VK_A);
		rightPanel.add(radioButton);
		buttonGroupXMLAskAttrForEach.add(radioButton);

		radioButton = new JRadioButton("Element");
		radioButton.setMnemonic(KeyEvent.VK_E);
		rightPanel.add(radioButton);
		buttonGroupXMLAskAttrForEach.add(radioButton);

		radioButton = new JRadioButton("Macro");
		radioButton.setMnemonic(KeyEvent.VK_M);
		rightPanel.add(radioButton);
		buttonGroupXMLAskAttrForEach.add(radioButton);

		rowPanel2.add(rightPanel, BorderLayout.CENTER);
		rowPanel.add(rowPanel2, new TableLayoutConstraints(0, 0));

		jCheckBoxXMLInsertEmptyAttr = new JCheckBox("Insert attribute even if no value was entered");
		jCheckBoxXMLInsertEmptyAttr.setMnemonic(KeyEvent.VK_E);
		rowPanel.add(jCheckBoxXMLInsertEmptyAttr, new TableLayoutConstraints(0, 1));
		panel.add(rowPanel);

		// attr. table
		jCheckBoxXMLAttributesTableShowLinkedAttributesButton = new JCheckBox("Show \"Linked Attributes\" button in Attributes-Table");
		jCheckBoxXMLAttributesTableShowLinkedAttributesButton.setMnemonic(KeyEvent.VK_L);
		jCheckBoxXMLAttributesTableShowLinkedAttributesButton.setToolTipText("slows down performance of a larger XML file when \"Linked Attributes\" are defined for the current language");
		rowPanel = new JPanel(new BorderLayout());
		rowPanel.add(jCheckBoxXMLAttributesTableShowLinkedAttributesButton);
		panel.add(rowPanel);
		jCheckBoxXMLAttributesTableShowEditButton = new JCheckBox("Show \"Edit\" button in Attributes-Table");
		jCheckBoxXMLAttributesTableShowEditButton.setMnemonic(KeyEvent.VK_E);
		rowPanel = new JPanel(new BorderLayout());
		rowPanel.add(jCheckBoxXMLAttributesTableShowEditButton);
		panel.add(rowPanel);
		jCheckBoxXMLShowInsertableCount = new JCheckBox("Show count of insertable nodes in insertable dialog");
		jCheckBoxXMLShowInsertableCount.setMnemonic(KeyEvent.VK_I);
		rowPanel = new JPanel(new BorderLayout());
		rowPanel.add(jCheckBoxXMLShowInsertableCount);
		panel.add(rowPanel);

		JPanel dummyPanel = new JPanel(new BorderLayout());
		dummyPanel.add(panel, BorderLayout.NORTH);
		return dummyPanel;
	}

	private Component createMacrosPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.setBorder(PANEL_EMPTY_BORDER);

		JPanel insertPanel = new JPanel(new TableLayout(new double[][]{{20.0, TableLayout.PREFERRED}, {TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		insertPanel.setBorder(new TitledBorder(null, " Insert-Mode ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));
		jCheckBoxMacrosAutoSwitchInsertMode = new JCheckBox("Auto-switch Insert-Mode");
		jCheckBoxMacrosAutoSwitchInsertMode.setMnemonic(KeyEvent.VK_A);
		insertPanel.add(jCheckBoxMacrosAutoSwitchInsertMode, new TableLayoutConstraints(0, 0, 1, 0));
		panel.add(insertPanel);

		jCheckBoxMacrosSwitchBackToDefault = new JCheckBox("Always switch back to \"Insert Into\"-Mode");
		jCheckBoxMacrosSwitchBackToDefault.setMnemonic(KeyEvent.VK_I);
		insertPanel.add(jCheckBoxMacrosSwitchBackToDefault, new TableLayoutConstraints(1, 1));

		JPanel rowPanel = new JPanel(new BorderLayout());
		rowPanel.setBorder(new EmptyBorder(5, 0, 5, 0));
		jCheckBoxMacrosSynchronizeStructureView = new JCheckBox("Synchronize Structure View with Search View / Insertable View");
		jCheckBoxMacrosSynchronizeStructureView.setMnemonic(KeyEvent.VK_S);
		rowPanel.add(jCheckBoxMacrosSynchronizeStructureView);
		panel.add(rowPanel);

		rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED}, {TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		rowPanel.setBorder(new TitledBorder(null, " Macros Search ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));
		jCheckBoxMacrosSearchOnlyForMacros = new JCheckBox("Search only for Macros");
		jCheckBoxMacrosSearchOnlyForMacros.setMnemonic(KeyEvent.VK_M);
		rowPanel.add(jCheckBoxMacrosSearchOnlyForMacros, new TableLayoutConstraints(0, 0));
		jCheckBoxMacrosSearchIgnoreCase = new JCheckBox("Ignore Case");
		jCheckBoxMacrosSearchIgnoreCase.setMnemonic(KeyEvent.VK_C);
		rowPanel.add(jCheckBoxMacrosSearchIgnoreCase, new TableLayoutConstraints(0, 1));
		panel.add(rowPanel);

		rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}, {TableLayout.PREFERRED, TableLayout.PREFERRED}}, 5,5));
		rowPanel.setBorder(new TitledBorder(null, " Minimal Macros-List Dimension ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));
		rowPanel.add(new JLabel("Width:"), new TableLayoutConstraints(0, 0));
		jSliderMacrosListWidth = new JSlider(50, 301);
		jSliderMacrosListWidth.setExtent(10);
		jSliderMacrosListWidth.setLabelTable(jSliderMacrosListWidth.createStandardLabels(50, 50));
		jSliderMacrosListWidth.setMajorTickSpacing(50);
		jSliderMacrosListWidth.setMinorTickSpacing(10);
		jSliderMacrosListWidth.setPaintLabels(true);
		jSliderMacrosListWidth.setPaintTicks(true);
		rowPanel.add(jSliderMacrosListWidth, new TableLayoutConstraints(1, 0));
		final JLabel valueLabelWidth = new JLabel();
		jSliderMacrosListWidth.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				valueLabelWidth.setText(""+jSliderMacrosListWidth.getValue());
			}
		});
		rowPanel.add(valueLabelWidth, new TableLayoutConstraints(2, 0));
		rowPanel.add(new JLabel("Height:"), new TableLayoutConstraints(0, 1));
		jSliderMacrosListHeight = new JSlider(50, 301);
		jSliderMacrosListHeight.setExtent(10);
		jSliderMacrosListHeight.setLabelTable(jSliderMacrosListHeight.createStandardLabels(50, 50));
		jSliderMacrosListHeight.setMajorTickSpacing(50);
		jSliderMacrosListHeight.setMinorTickSpacing(10);
		jSliderMacrosListHeight.setPaintLabels(true);
		jSliderMacrosListHeight.setPaintTicks(true);
		rowPanel.add(jSliderMacrosListHeight, new TableLayoutConstraints(1, 1));
		final JLabel valueLabelHeight = new JLabel();
		jSliderMacrosListHeight.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent evt) {
				valueLabelHeight.setText("" + jSliderMacrosListHeight.getValue());
			}
		});
		rowPanel.add(valueLabelHeight, new TableLayoutConstraints(2, 1));
		panel.add(rowPanel);

		JPanel dummyPanel = new JPanel(new BorderLayout());
		dummyPanel.add(panel, BorderLayout.NORTH);
		return dummyPanel;
	}

	private Component createAppearancePanel() {
		JPanel panel = new JPanel(new TableLayout(new double[][]{{TableLayout.FILL},
																 {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		panel.setBorder(PANEL_EMPTY_BORDER);

		jCheckBoxRestoreLastLayoutOnStartup = new JCheckBox("Restore last layout on startup");
		jCheckBoxRestoreLastLayoutOnStartup.setToolTipText("Save the layout on quit and restore it on next startup");
		panel.add(jCheckBoxRestoreLastLayoutOnStartup, new TableLayoutConstraints(0, 0));

		JPanel colorsPanel = new JPanel(new TableLayout(new double[][]{{0.5, 0.5},
																	  {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 10, 5));
		colorsPanel.setBorder(new CompoundBorder(new TitledBorder(null, " Colors ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR), new EmptyBorder(0, 5, 2, 5)));
		panel.add(colorsPanel, new TableLayoutConstraints(0, 1));

		colorViewSelection = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewSelection, "Selection BG"), new TableLayoutConstraints(0, 0));

		colorViewSearchResults = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewSearchResults, "Search Results BG"), new TableLayoutConstraints(1, 0));


		colorViewLinkFG = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewLinkFG, "Link FG"), new TableLayoutConstraints(0, 1));

		colorViewActivatedLinkBG = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewActivatedLinkBG, "Activated Link BG"), new TableLayoutConstraints(1, 1));


		colorViewHTMLLinkFG = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewHTMLLinkFG, "HTML Link FG"), new TableLayoutConstraints(0, 2));

		colorViewLinkHTMLDelimiter = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewLinkHTMLDelimiter, "Link-Delimiter FG (HTML)"), new TableLayoutConstraints(1, 2));

//		colorViewLinkHTMLOverlapping = new ColorView();
//		colorsPanel.add(new ColorControlPanel(colorViewLinkHTMLOverlapping, "Overlapping Link FG (HTML)"), new TableLayoutConstraints(0,3, 1, 3, TableLayout.RIGHT, TableLayout.FULL));


		colorViewInvalidXMLNodesFG = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewInvalidXMLNodesFG, "Invalid XML-Nodes FG"), new TableLayoutConstraints(0, 3));

		colorViewInvalidXMLNodesBG = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewInvalidXMLNodesBG, "Invalid XML-Nodes BG"), new TableLayoutConstraints(1, 3));


		colorViewSideBorderActive = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewSideBorderActive, "Active Side-Border"), new TableLayoutConstraints(0, 4));

		colorViewSideBorderInactive = new ColorView();
		colorsPanel.add(new ColorControlPanel(colorViewSideBorderInactive, "Inactive Side-Border"), new TableLayoutConstraints(1, 4));

		JPanel rowPanel = new JPanel(new TableLayout(new double[][]{{TableLayout.PREFERRED, TableLayout.PREFERRED}, {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}));
		rowPanel.setBorder(new TitledBorder(null, " Misc. Appearance Settings ", TitledBorder.LEADING, TitledBorder.TOP, null, LABEL_COLOR));

		jCheckBoxAppearanceDynamicLayout = new JCheckBox("Use dynamic layout");
		jCheckBoxAppearanceDynamicLayout.setToolTipText("DELT/A need to be restarted");
		jCheckBoxAppearanceDynamicLayout.setMnemonic(KeyEvent.VK_D);
		rowPanel.add(jCheckBoxAppearanceDynamicLayout, new TableLayoutConstraints(0, 0));

		jCheckBoxAppearanceSplitpanesContinuousLayout = new JCheckBox("Use continuous layout for splitpanes");
		jCheckBoxAppearanceSplitpanesContinuousLayout.setToolTipText("DELT/A need to be restarted");
		jCheckBoxAppearanceSplitpanesContinuousLayout.setMnemonic(KeyEvent.VK_C);
		rowPanel.add(jCheckBoxAppearanceSplitpanesContinuousLayout, new TableLayoutConstraints(1, 0));

		jCheckBoxAppearanceToolbarsFloatable = new JCheckBox("Use floatable toolbars");
		jCheckBoxAppearanceToolbarsFloatable.setToolTipText("DELT/A need to be restarted");
		jCheckBoxAppearanceToolbarsFloatable.setMnemonic(KeyEvent.VK_F);
		rowPanel.add(jCheckBoxAppearanceToolbarsFloatable, new TableLayoutConstraints(0, 1));
		jCheckBoxAppearanceToolbarsRollover = new JCheckBox("Use rollover-effect on toolbars");
		jCheckBoxAppearanceToolbarsRollover.setToolTipText("DELT/A need to be restarted");
		jCheckBoxAppearanceToolbarsRollover.setMnemonic(KeyEvent.VK_T);
		rowPanel.add(jCheckBoxAppearanceToolbarsRollover, new TableLayoutConstraints(1, 1));
		jCheckBoxXMLTreeFishEyeView = new JCheckBox("Use \"FishEyeView\" in XML-View");
		jCheckBoxXMLTreeFishEyeView.setMnemonic(KeyEvent.VK_E);
		jCheckBoxXMLTreeFishEyeView.setToolTipText("current treenode will be displayed larger than other nodes");
		rowPanel.add(jCheckBoxXMLTreeFishEyeView, new TableLayoutConstraints(0, 2));
		jCheckBoxAppearanceXMLTreeRollOver = new JCheckBox("Use rollover-effect in XML tree(s)");
		jCheckBoxAppearanceXMLTreeRollOver.setMnemonic(KeyEvent.VK_X);
		jCheckBoxAppearanceXMLTreeRollOver.setToolTipText("highlights the treenode currently under or next to the mouse cursor");
		rowPanel.add(jCheckBoxAppearanceXMLTreeRollOver, new TableLayoutConstraints(1, 2));
		panel.add(rowPanel, new TableLayoutConstraints(0, 2));

		JPanel dummyPanel = new JPanel(new BorderLayout());
		dummyPanel.add(panel, BorderLayout.NORTH);
		return dummyPanel;
	}

	private Component createToolsPanel() {
		JPanel panel = new JPanel(new TableLayout(new double[][]{{0.35, 0.3, 0.35}, {0.5, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 0, 5));
		panel.add(new JButton(aRestoreDefaults), new TableLayoutConstraints(1, 1));
		panel.add(new JButton(aRestoreSaved), new TableLayoutConstraints(1, 2));
		panel.add(new JButton(aSaveNow), new TableLayoutConstraints(1, 3));
		panel.add(new JButton(aSaveAs), new TableLayoutConstraints(1, 4));
		panel.add(new JButton(aLoad), new TableLayoutConstraints(1, 5));
		return panel;
	}

	private static class BlueLabel extends JLabel {
		public BlueLabel(String text) {
			super(text);
			setForeground(LABEL_COLOR);
		}
	}
	private static class ColorView extends JPanel {
		private Color color;
		private JPanel colorPanel;
		public ColorView() {
			super(new BorderLayout());
			setBorder(new CompoundBorder(new EtchedBorder(Color.DARK_GRAY, Color.GRAY), GlobalObjects.EMPTY_BORDER_2));
			colorPanel = new JPanel();
			colorPanel.setBorder(new LineBorder(Color.BLACK));
			add(colorPanel);
			setPreferredSize(new Dimension(25, 21));
			color = Color.BLACK;
		}
		public Color getColor() {
			return color;
		}
		public void setColor(Color color) {
			this.color = color;
			colorPanel.setBackground(color);
		}
	}
	private static class ColorChangeAction extends AbstractAction {
		private final ColorView colorView;
		private final String name;
		public ColorChangeAction(ColorView colorView, String name) {
			super("", MiscImages.DOTS_ICON);
			this.colorView = colorView;
			this.name = name;
		}
		public void actionPerformed(ActionEvent evt) {
			Color oldColor = colorView.getColor();
			final Color[] color = new Color[1];
			if (colorChooser == null) {
				colorChooser = new JColorChooser();
				colorChooser.addChooserPanel(new SystemColorChooserPanel());
			}
			JDialog colorChooserDialog = JColorChooser.createDialog(colorView, "Choose a color for the "+name+" ...", true, colorChooser,
															new ActionListener() {
																public void actionPerformed(ActionEvent evt1) {
																	color[0] = colorChooser.getColor();
																}
															}, null);
			colorChooserDialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
			colorChooser.setColor(oldColor);
			colorChooserDialog.setVisible(true);
			if (color[0] != null)
				colorView.setColor(color[0]);
		}
	}
	private static class ColorControlPanel extends JPanel {
		public ColorControlPanel(ColorView colorView, String name) {
			super(new BorderLayout());
			add(new JLabel(name+":"), BorderLayout.WEST);
			JPanel rightPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, 2,0));
			rightPanel.add(colorView);
			rightPanel.add(new SmallButton(new ColorChangeAction(colorView, name)));
			add(rightPanel, BorderLayout.EAST);
		}
	}


    private ExtendedButtonGroup buttonGroupXMLAskAttrForEach;
    private JCheckBox jCheckBoxAskBeforeRemoveXMLLinks;
    private JCheckBox jCheckBoxXMLInsertEmptyAttr;
    private JCheckBox jCheckBoxDTDLoad;
    private JCheckBox jCheckBoxXMLTreeNodesShowChildCount;
    private JCheckBox jCheckBoxCheckLinks;
    private JCheckBox jCheckBoxXMLAttributesTableShowEditButton;
    private JCheckBox jCheckBoxXMLAttributesTableShowLinkedAttributesButton;
    private JCheckBox jCheckBoxAskBeforeRemoveXMLSubTree;
    private JCheckBox jCheckBoxAppearanceXMLTreeRollOver;
    private JCheckBox jCheckBoxAskBeforeRemoveHTMLLinks;
    private JCheckBox jCheckBoxRestoreLastLayoutOnStartup;
    private JCheckBox jCheckBoxHTMLRemoveDELTALinksOnSave;
    private JCheckBox jCheckBoxAdvancedMode;
    private JCheckBox jCheckBoxXMLTreeSelectCurrentHighlightedTreeNode, jCheckBoxXMLTreeNodesExpandAllOnLoad, jCheckBoxXMLTreeAutoCollapseOnExpand, jCheckBoxXMLTreeAutoCollapseOnSelection,
				jCheckBoxXMLTreeAutoSelectOnFolding, jCheckBoxXMLTreeDragAndDrop, jCheckBoxXMLTreeAutoExpandOnSelection, jCheckBoxXMLTreeFishEyeView;
    private JCheckBox jCheckBoxAskBeforeRemoveXMLNodes;
    private ExtendedButtonGroup buttonGroupXMLLinkActivation;
    private JCheckBox jCheckBoxClearRecentFileLists;
    private ButtonGroup buttonGroupRemoveDeadLinks;
    private JCheckBox jCheckBoxAskBeforeRemoveXMLAttributes;
    private ExtendedButtonGroup buttonGroupAutoSave, buttonGroupAutoChangeDocType;
//    private JComboBox jComboBoxRecentFilesLimit;
    private JCheckBox jCheckBoxBackupOnSave;
    private JCheckBox jCheckBoxLoadLastProject;
    private JCheckBox jCheckBoxLoadLastFiles;
    private ExtendedButtonGroup buttonGroupHTMLCheckLinks;
    private ExtendedButtonGroup buttonGroupDTDState;
    private JCheckBox jCheckBoxXMLTreeNodesShowAttributes;

	private int returnStatus = RET_CANCEL;
}
