/*
 * Project "Asbru Interpreter".
 * 
 * $Id$
 */
package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Process;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step;
import at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Steps;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.ProcessTemplate;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.swing.SimpleListCellRenderer;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import at.ac.tuwien.asgaard.swing.models.SimpleComboBoxModel;
import at.ac.tuwien.asgaard.swing.models.SimpleListModel;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;
import at.ac.tuwien.asgaard.util.FileUtils;

import info.clearthought.layout.SingleFiledLayout;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * Class ProcessPropertiesDialog implementes a dialog to view and edit the properties of a Transformation Process project.
 *
 * @author Peter Votruba
 * @version $Revision$
 */
public class ProcessPropertiesDialog extends AbstractInputDialog {
    private static ProcessPropertiesDialog sharedInstance;

    private Process deltaProcess;
	private Step currentStep;

	private final GUIFieldChangeListener processChangeListener, stepChangeListener;

    public static ProcessPropertiesDialog getSharedInstance() {
        if (sharedInstance == null)
            sharedInstance = new ProcessPropertiesDialog(MainController.mainFrame);
        return sharedInstance;
    }

    private ProcessPropertiesDialog(Frame parent) {
        super(parent, "Process Properties", false, true, 580, 480, 500, -1, 0, 0);

        helpAction.putValue(HELP_LINK_KEY, "ProcessPropertiesDialog");

		processChangeListener = new GUIFieldChangeListener();
		fieldProcessName.getDocument().addDocumentListener(processChangeListener);
		fieldProcessSourceFile.getDocument().addDocumentListener(processChangeListener);
		fieldProcessDescription.getDocument().addDocumentListener(processChangeListener);
		fieldProcessTemplate.addItemListener(processChangeListener);

		stepChangeListener = new GUIFieldChangeListener();
		fieldStepName.getDocument().addDocumentListener(stepChangeListener);
		fieldStepTargetFile.getDocument().addDocumentListener(stepChangeListener);
		fieldStepMacroFile.getDocument().addDocumentListener(stepChangeListener);
	}

    protected Component createMainComponent() {
        JPanel mainPanel = new JPanel(new BorderLayout(5,5));

        mainPanel.add(createGeneralPanel(), BorderLayout.NORTH);
        mainPanel.add(createStepsPanel(), BorderLayout.CENTER);

        return mainPanel;
    }

    private JComponent createGeneralPanel() {
        JPanel generalPanel = new JPanel(new TableLayout(
                new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}, // 120.0
                               {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED,
                                TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 2, 5));
        generalPanel.setBorder(new CompoundBorder(new TitledBorder(" General "), new EmptyBorder(0, 2, 2, 2)));

        JLabel label = new JLabel("Process-Name: ");
        fieldProcessName = new JTextField();
        label.setDisplayedMnemonic(KeyEvent.VK_N);
        label.setLabelFor(fieldProcessName);
        generalPanel.add(label, new TableLayoutConstraints(0, 0));
        generalPanel.add(fieldProcessName, new TableLayoutConstraints(1, 0, 2, 0));

        label = new JLabel("Process-File: ");
        fieldProcessFile = new JTextField();
        fieldProcessFile.setEditable(false);
        label.setDisplayedMnemonic(KeyEvent.VK_P);
        label.setLabelFor(fieldProcessFile);
        generalPanel.add(label, new TableLayoutConstraints(0, 1));
        generalPanel.add(fieldProcessFile, new TableLayoutConstraints(1, 1, 2, 1));

        label = new JLabel("Based on Template: ");
		processTemplateComboBoxModel = new SimpleComboBoxModel(templatesToNames());
		fieldProcessTemplate = new JComboBox(processTemplateComboBoxModel);
        fieldProcessTemplate.setEditable(false);
        label.setDisplayedMnemonic(KeyEvent.VK_T);
        label.setLabelFor(fieldProcessTemplate);
        generalPanel.add(label, new TableLayoutConstraints(0, 2));
        generalPanel.add(fieldProcessTemplate, new TableLayoutConstraints(1, 2));
		generalPanel.add(new SmallButton(actionShowTemplateDialog), new TableLayoutConstraints(2, 2));

		label = new JLabel("Process Source-File: ");
        fieldProcessSourceFile = new JTextField();
		fieldProcessSourceFile.setToolTipText("<html><b>source</b> file of the process (i.e. the source of the first step)");
		fieldProcessSourceFile.setEditable(false);
        label.setDisplayedMnemonic(KeyEvent.VK_S);
        label.setLabelFor(fieldProcessSourceFile);
        generalPanel.add(label, new TableLayoutConstraints(0, 3));
        generalPanel.add(fieldProcessSourceFile, new TableLayoutConstraints(1, 3));
        generalPanel.add(new SmallButton(actionChangeProcessSourceFile), new TableLayoutConstraints(2, 3));

        label = new JLabel("Highest Link-ID: ");
        fieldHighestLinkID = new JLabel();
		fieldHighestLinkID.setFont(fieldProcessName.getFont());
		label.setDisplayedMnemonic(KeyEvent.VK_L);
        label.setLabelFor(fieldHighestLinkID);
        generalPanel.add(label, new TableLayoutConstraints(0, 4));
        generalPanel.add(fieldHighestLinkID, new TableLayoutConstraints(1, 4, 2, 4));

        label = new JLabel("Description: ");
        fieldProcessDescription = new JTextField();
        label.setDisplayedMnemonic(KeyEvent.VK_D);
        label.setLabelFor(fieldProcessDescription);
        generalPanel.add(label, new TableLayoutConstraints(0, 5));
        generalPanel.add(fieldProcessDescription, new TableLayoutConstraints(1, 5));
        generalPanel.add(new SmallButton(actionEditDescription), new TableLayoutConstraints(2, 5));

        return generalPanel;
    }

	private Object[] templatesToNames() {
		java.util.List processTemplateList = MainController.processTemplates.getProcessTemplateList();
		Object[] result = new Object[processTemplateList.size()];
		int index=0;
		for (Object o : processTemplateList)
			result[index++] = o.toString();
		return result;
	}

	private JPanel createStepsPanel() {
		JPanel stepsPanel = new JPanel(new BorderLayout(5, 5));
		stepsPanel.setBorder(new CompoundBorder(new TitledBorder(" Process-Steps "), new EmptyBorder(0, 2, 2, 2)));

		JPanel northPanel = new JPanel(new TableLayout(
				new double[][]{{TableLayout.PREFERRED, TableLayout.FILL}, {TableLayout.PREFERRED}}, 2, 5)); // 120.0

		JLabel label = new JLabel("Active-Step #: ");
		fieldActiveStep = new JLabel();
		fieldActiveStep.setFont(fieldProcessName.getFont());
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(fieldActiveStep);
		northPanel.add(label, new TableLayoutConstraints(0, 0));
		northPanel.add(fieldActiveStep, new TableLayoutConstraints(1, 0));

		stepListModel = new SimpleListModel();
		JList stepList = new JList(stepListModel);
		stepListSelectionModel = new SingleListSelectionModel();
		stepList.setSelectionModel(stepListSelectionModel);
		stepList.addListSelectionListener(createStepListSelectionListener());
		stepList.setCellRenderer(new SimpleListCellRenderer() {
			protected String getLabel(final int index, Object value) {
				return (index+1) + ": " + value.toString();
			}
		});

		JPanel stepListPanel = new JPanel(new TableLayout(new double[][] {
				{TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, 5.0, TableLayout.PREFERRED, TableLayout.PREFERRED},
				{TableLayout.PREFERRED, TableLayout.FILL}}, 2, 5));
		stepListPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
		stepListPanel.add(new SmallButton(actionStepAdd), new TableLayoutConstraints(1, 0));
		stepListPanel.add(new SmallButton(actionStepRemove), new TableLayoutConstraints(2, 0));
		stepListPanel.add(new SmallButton(actionStepMoveUp), new TableLayoutConstraints(4, 0));
		stepListPanel.add(new SmallButton(actionStepMoveDown), new TableLayoutConstraints(5, 0));
		stepListPanel.add(new JScrollPane(stepList), new TableLayoutConstraints(0, 1, 5, 1));

		stepPanel = new JPanel(new TableLayout(
				new double[][]{{100.0, TableLayout.FILL, TableLayout.PREFERRED},
							   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED,
								TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 2, 5));
		stepsPanelTitleBorder = new TitledBorder(" Process-Step ");
		stepPanel.setBorder(new CompoundBorder(stepsPanelTitleBorder, new EmptyBorder(0, 2, 2, 2)));

		label = new JLabel("Step-Name: ");
		fieldStepName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_N);
		label.setLabelFor(fieldStepName);
		stepPanel.add(label, new TableLayoutConstraints(0, 0));
		stepPanel.add(fieldStepName, new TableLayoutConstraints(1, 0, 2, 0));

		JPanel langPanel = new JPanel(new SingleFiledLayout(SingleFiledLayout.ROW));
		fieldStepSourceLanguage = new JLabel();
		fieldStepSourceLanguage.setFont(fieldStepName.getFont());
		fieldStepSourceLanguage.setToolTipText("XML-Language of Source file");
		fieldStepTargetLanguage = new JLabel();
		fieldStepTargetLanguage.setFont(fieldStepName.getFont());
		fieldStepTargetLanguage.setToolTipText("XML-Language of Target file");
		langPanel.add(fieldStepSourceLanguage);
		langPanel.add(new JLabel(new ImageIcon(ArrowImage.DEFAULT_ARROW_IMAGE_RIGHT)));
		langPanel.add(fieldStepTargetLanguage);
		stepPanel.add(new JLabel("Transformation:"), new TableLayoutConstraints(0, 1));
		stepPanel.add(langPanel, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Source-File: ");
		fieldStepSourceFile = new JLabel();
		fieldStepSourceFile.setFont(fieldStepName.getFont());
		fieldStepSourceFile.setToolTipText("<html><b>source</b> file of this step,<br>will be shown in the <b>left</b> side pane");
		stepPanel.add(label, new TableLayoutConstraints(0, 2));
		stepPanel.add(fieldStepSourceFile, new TableLayoutConstraints(1, 2, 2, 2));

		label = new JLabel("Target-File: ");
		fieldStepTargetFile = new JTextField();
		fieldStepTargetFile.setToolTipText("<html><b>target</b> file of this step,<br>will be shown in the <b>right</b> side pane");
		fieldStepTargetFile.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_T);
		label.setLabelFor(fieldStepTargetFile);
		stepPanel.add(label, new TableLayoutConstraints(0, 3));
		stepPanel.add(fieldStepTargetFile, new TableLayoutConstraints(1, 3));
		stepPanel.add(new SmallButton(actionChangeStepTargetFile), new TableLayoutConstraints(2, 3));

		label = new JLabel("Macro-File: ");
		fieldStepMacroFile = new JTextField();
		fieldStepMacroFile.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_M);
		label.setLabelFor(fieldStepMacroFile);
		stepPanel.add(label, new TableLayoutConstraints(0, 4));
		stepPanel.add(fieldStepMacroFile, new TableLayoutConstraints(1, 4));
		stepPanel.add(new SmallButton(actionChangeStepMacroFile), new TableLayoutConstraints(2, 4));

		label = new JLabel("Description: ");
		fieldStepDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(fieldStepDescription);
		stepPanel.add(label, new TableLayoutConstraints(0, 5));
		stepPanel.add(fieldStepDescription, new TableLayoutConstraints(1, 5));
		stepPanel.add(new SmallButton(actionEditStepDescription), new TableLayoutConstraints(2, 5));

		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true, stepListPanel, stepPanel);
		splitPane.setOneTouchExpandable(true);
		stepsPanel.add(northPanel, BorderLayout.NORTH);
		stepsPanel.add(splitPane, BorderLayout.CENTER);
		splitPane.setBorder(null);

		return stepsPanel;
	}


    protected void createActions() {
        super.createActions();

		actionShowTemplateDialog = new AbstractAction("", MainImages.VIEW_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "View Process Template ...");
			}
			public void actionPerformed(ActionEvent evt) {
				Object selectedTemplate = processTemplateComboBoxModel.getSelectedItem();
				if (ProcessTemplatesDialog.getSharedInstance().showDialog(fieldProcessTemplate.getSelectedItem().toString())) {
					processTemplateComboBoxModel.init(templatesToNames(), selectedTemplate);
					MainController.buildProcessTemplatesMenu();
				}
			}
		};
		actionEditDescription = new AbstractAction("", MainImages.EDIT_ICON) {
            {
                putValue(StatusBar.TOOL_TIP, "Edit Process Description...");
            }
            private TextEditDialog editDialog;
            public void actionPerformed(ActionEvent evt) {
                if (editDialog == null)
                    editDialog = new TextEditDialog(ProcessPropertiesDialog.this, "Edit Process Description...");
                String newValue = editDialog.showEditDialog(fieldProcessDescription.getText());
                if (newValue != null) {
                    fieldProcessDescription.setText(newValue);
                    fieldProcessDescription.requestFocusInWindow();
                }
            }
        };
        actionChangeProcessSourceFile = new AbstractAction("", MiscImages.DOTS_ICON) {
            {
                putValue(StatusBar.TOOL_TIP, "Change Process Source File ...");
			}
            public void actionPerformed(ActionEvent evt) {
				ProcessTemplate currentProcessTemplate = MainController.currentProcessTemplate;
				if (currentProcessTemplate == null) {
					JOptionPane.showMessageDialog(ProcessPropertiesDialog.this, "Process template is not defined!");
					return;
				}
				String languageName = currentProcessTemplate.getSourceLanguage();
				XmlLanguage xmlLanguage = MainController.getXmlLanguageByName(languageName);
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser("Change '" + languageName + "' file (source) ...");
				else {
					fileChooser.setDialogTitle("Change '" + languageName + "' file (source) ...");
					fileChooser.resetChoosableFileFilters();
				}
				if (languageName.equalsIgnoreCase("HTML"))
					fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
				else
					fileChooser.setFileFilter(xmlLanguage.getFileFilter());
				File sourceFile = FileUtils.toExistingAbsoluteFile(fieldProcessSourceFile.getText(), deltaProcess.getFile());
				fileChooser.setSelectedFile(sourceFile);
				if (fileChooser.showOpenDialog(ProcessPropertiesDialog.this) == JFileChooser.APPROVE_OPTION)
					fieldProcessSourceFile.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(),
																		 deltaProcess.getFile()));
            }
        };
        actionChangeStepTargetFile = new AbstractAction("", MiscImages.DOTS_ICON) {
            {
                putValue(StatusBar.TOOL_TIP, "Change Step Target File ...");
			}
            public void actionPerformed(ActionEvent evt) {
				String languageName = fieldStepTargetLanguage.getText();
				XmlLanguage xmlLanguage = MainController.getXmlLanguageByName(languageName);
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser("Change '"+languageName + "' file (target) ...");
				else {
					fileChooser.setDialogTitle("Change '" + languageName + "' file (target) ...");
					fileChooser.resetChoosableFileFilters();
				}
				if (languageName.equalsIgnoreCase("HTML"))
					fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
				else
					fileChooser.setFileFilter(xmlLanguage.getFileFilter());
				File targetFile = FileUtils.toExistingAbsoluteFile(fieldStepTargetFile.getText(), deltaProcess.getFile());
				fileChooser.setSelectedFile(targetFile);
				if (fileChooser.showOpenDialog(ProcessPropertiesDialog.this) == JFileChooser.APPROVE_OPTION)
					fieldStepTargetFile.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(),
																		 deltaProcess.getFile()));
			}
        };
        actionChangeStepMacroFile = new AbstractAction("", MiscImages.DOTS_ICON) {
			private DeltaFileChooser fileChooser;
			{
				putValue(StatusBar.TOOL_TIP, "Change Step Macro File ...");
				fileChooser = new DeltaFileChooser("Choose Macro file");
			}
			public void actionPerformed(ActionEvent evt) {
				String xmlLanguageName = fieldStepTargetLanguage.getText();
				if (!xmlLanguageName.equals("HTML")) {
					fileChooser.setDialogTitle("Choose '"+ xmlLanguageName+"' Macro file");
					fileChooser.setFileFilter(new DeltaFileChooser.MacroFileFilter(xmlLanguageName));
					File macroFile = FileUtils.toExistingAbsoluteFile(fieldStepMacroFile.getText(), deltaProcess.getFile());
					fileChooser.setSelectedFile(macroFile);
					if (fileChooser.showOpenDialog(ProcessPropertiesDialog.this) == JFileChooser.APPROVE_OPTION)
						fieldStepMacroFile.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(),
																			deltaProcess.getFile()));
				}
			}
        };

		actionStepAdd = new AbstractAction("", MainImages.ADD_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Add new Process step with current values");
			}
			public void actionPerformed(ActionEvent evt) {
				currentStep = at.ac.tuwien.asgaard.delta.xmlBeans.processProject.Step.newInstance();
				setStepBeanFields(); // store current values in new step
				deltaProcess.getSteps().addStep(currentStep);
				stepListModel.add(currentStep);
				stepListSelectionModel.setSelectionIndex(stepListModel.getSize()-1);
				stepChangeListener.setFieldsChanged();
			}
		};
		actionStepRemove = new AbstractAction("", MainImages.REMOVE_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Remove selected Process Step");
			}
			public void actionPerformed(ActionEvent evt) {
				int index = stepListSelectionModel.getCurrentIndex();
				if (index >= 0) {
					if (stepChangeListener.fieldsHaveChanged())
						if (JOptionPane
								.showConfirmDialog(ProcessPropertiesDialog.this, "Changes of current Step will be lost. Really remove?",
												   "Remove Step ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
								!= JOptionPane.YES_OPTION)
							return;
					currentStep = null;
					stepListModel.remove(index);
					if (stepListModel.getSize() == 0)
						stepListSelectionModel.clearSelection();
					else if (index < stepListModel.getSize())
						stepListSelectionModel.setSelectionIndex(index);
					else
						stepListSelectionModel.setSelectionIndex(index - 1);
					stepChangeListener.setFieldsChanged();
				}
			}
		};
		actionStepMoveUp = new AbstractAction("", MainImages.SEARCH_UP_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Move selected Process Step up");
			}
			public void actionPerformed(ActionEvent evt) {
				int index = stepListSelectionModel.getCurrentIndex();
				if (index > 0) {
					stepListModel.remove(index);
					stepListModel.add(index - 1, currentStep);
					stepListSelectionModel.setSelectionIndex(index - 1);
					stepChangeListener.setFieldsChanged();
				}
			}
		};
		actionStepMoveDown = new AbstractAction("", MainImages.SEARCH_DOWN_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Move selected Process Step down");
			}
			public void actionPerformed(ActionEvent evt) {
				int index = stepListSelectionModel.getCurrentIndex();
				if (index < stepListModel.getSize() - 1) {
					stepListModel.remove(index);
					stepListModel.add(index + 1, currentStep);
					stepListSelectionModel.setSelectionIndex(index + 1);
					stepChangeListener.setFieldsChanged();
				}
			}
		};
		actionEditStepDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Edit Step Description...");
			}
			private TextEditDialog editDialog;
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(ProcessPropertiesDialog.this, "Edit Step Description...");
				String newValue = editDialog.showEditDialog(fieldStepDescription.getText());
				if (newValue != null) {
					fieldStepDescription.setText(newValue);
					fieldStepDescription.requestFocusInWindow();
				}
			}
		};
	}

    private ListSelectionListener createStepListSelectionListener() {
        return new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent evt) {
				if (currentStep != null) //(stepListSelectionModel.getPreviousIndex() != -1)
					setStepBeanFields();

                int stepIndex = stepListSelectionModel.getCurrentIndex();
				if (stepIndex >= 0) {
					stepsPanelTitleBorder.setTitle(" Process-Step #" + (stepIndex + 1) + " ");
					stepPanel.repaint();
					currentStep = deltaProcess.getSteps().getStep(stepIndex);
					fieldStepName.setText(currentStep.getName());
					if (stepIndex == 0)
						fieldStepSourceFile.setText(deltaProcess.getSourceFile());
					else
						fieldStepSourceFile.setText(deltaProcess.getSteps().getStep(stepIndex - 1).getTargetFile());
					fieldStepTargetFile.setText(currentStep.getTargetFile());
					fieldStepMacroFile.setText(currentStep.getMacroFile());
					fieldStepDescription.setText(currentStep.getDescription());

					ProcessTemplate currentProcessTemplate = MainController.currentProcessTemplate;
					if (currentProcessTemplate != null) {
						String sourceLanguageName;
						if (stepIndex == 0)
							sourceLanguageName = currentProcessTemplate.getSourceLanguage();
						else
							sourceLanguageName = currentProcessTemplate.getStepTemplate(stepIndex - 1).getTargetLanguage();
						String targetLanguageName = currentProcessTemplate.getStepTemplate(stepIndex).getTargetLanguage();
						fieldStepSourceLanguage.setText(sourceLanguageName);
						fieldStepTargetLanguage.setText(targetLanguageName);
					}
					stepChangeListener.reset();

					actionStepMoveUp.setEnabled(stepIndex > 0);
					actionStepMoveDown.setEnabled(stepIndex < stepListModel.getSize()-1);
				}
			}
        };
    }

	private void setStepBeanFields() {
		currentStep.setName(fieldStepName.getText());
		currentStep.setTargetFile(fieldStepTargetFile.getText());
		currentStep.setMacroFile(fieldStepMacroFile.getText());
		currentStep.setDescription(fieldStepDescription.getText());
	}


	protected boolean onOK() {
		if (processChangeListener.fieldsHaveChanged() || stepChangeListener.fieldsHaveChanged()) {
			deltaProcess.setName(fieldProcessName.getText());
			deltaProcess.setSourceFile(fieldProcessSourceFile.getText());
			deltaProcess.setTemplate(fieldProcessTemplate.getSelectedItem().toString());
			deltaProcess.setDescription(fieldProcessDescription.getText());
			setStepBeanFields();
			deltaProcess.getSteps().setStepList(stepListModel.getList());
			try {
				deltaProcess.marshal(deltaProcess.getFile());
			} catch (IOException exc) {
				System.err.println("Couldn't save DELT/A-Process file:\n  " + exc.getMessage());
	//            exc.printStackTrace();
				JOptionPane.showMessageDialog(ProcessPropertiesDialog.this, "Couldn't save DELT/A-Process file!\n\n" + exc.getMessage(), "I/O Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		return true;
	}

	protected boolean onCancel() {
		if (processChangeListener.fieldsHaveChanged() || stepChangeListener.fieldsHaveChanged()) {
			if (JOptionPane.showConfirmDialog(this, "Changes of current Process will be lost. Really cancel?",
											  "Template changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
				!= JOptionPane.YES_OPTION)
				return false;
			setGUIFields();
		}
		return true;
	}

    public Process getDeltaProcess() {
        return deltaProcess;
    }
    public void setDeltaProcess(final Process deltaProcess) {
        if (this.deltaProcess != deltaProcess) {
            this.deltaProcess = deltaProcess;
			setGUIFields();
		} else {
			// always update "highest link ID" and "active step"
			fieldHighestLinkID.setText(deltaProcess.getHighestLinkID());
			fieldActiveStep.setText(deltaProcess.getSteps().getActiveStep());
		}

        fieldProcessName.requestFocusInWindow();
		processChangeListener.reset();
		stepChangeListener.reset();
	}



	private void setGUIFields() {
		fieldProcessName.setText(deltaProcess.getName());
		fieldProcessTemplate.setSelectedItem(deltaProcess.getTemplate());
		fieldProcessFile.setText(deltaProcess.getFile().getAbsolutePath());
		fieldProcessDescription.setText(deltaProcess.getDescription());
		fieldHighestLinkID.setText(deltaProcess.getHighestLinkID());
		fieldProcessSourceFile.setText(deltaProcess.getSourceFile());

		Steps steps = deltaProcess.getSteps();
		fieldActiveStep.setText(steps.getActiveStep());
		stepListModel.init(steps.getStepList());
		stepListSelectionModel.setSelectionIndex(0);
	}

	private JTextField fieldProcessName;
    private JTextField fieldProcessDescription;
    private JComboBox fieldProcessTemplate; // not editable
    private JLabel fieldHighestLinkID; // not editable
    private JLabel fieldActiveStep; // not editable
    private JTextField fieldProcessSourceFile; // not editable
    private JTextField fieldProcessFile; // not editable

    private SimpleListModel stepListModel;
    private SingleListSelectionModel stepListSelectionModel;

    private JTextField fieldStepName;
    private JLabel fieldStepSourceFile; // not editable
    private JLabel fieldStepSourceLanguage; // not editable
    private JLabel fieldStepTargetLanguage; // not editable
    private JTextField fieldStepTargetFile; // not editable
    private JTextField fieldStepMacroFile; // not editable
	private JTextField fieldStepDescription;

	private Action actionShowTemplateDialog, actionEditDescription, actionChangeProcessSourceFile,
                actionChangeStepTargetFile, actionChangeStepMacroFile, actionEditStepDescription,
				actionStepAdd, actionStepRemove, actionStepMoveUp, actionStepMoveDown;

    private static DeltaFileChooser fileChooser;

    private TitledBorder stepsPanelTitleBorder;
    private JPanel stepPanel;

	private SimpleComboBoxModel processTemplateComboBoxModel;
}
