package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.xmlBeans.processTemplates.*;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.swing.SimpleListCellRenderer;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import at.ac.tuwien.asgaard.swing.models.SimpleComboBoxModel;
import at.ac.tuwien.asgaard.swing.models.SimpleListModel;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;
import at.ac.tuwien.asgaard.util.FileUtils;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

public class ProcessTemplatesDialog extends AbstractInputDialog {
	private static ProcessTemplatesDialog sharedInstance;

	private TemplatePropertiesDialog templatePropertiesDialog;

	public static ProcessTemplatesDialog getSharedInstance() {
		if (sharedInstance == null)
			sharedInstance = new ProcessTemplatesDialog(MainController.mainFrame);
		return sharedInstance;
	}

	private ProcessTemplatesDialog(Frame parent) {
		super(parent, "Process Templates", false, true, 180, 250, -1, -1, 0, 0);

		java.util.List xmlLanguageNames = new ArrayList(MainController.xmlLanguages.size() + 1);
		xmlLanguageNames.add("HTML");
		for (XmlLanguage xmlLanguage : MainController.xmlLanguages)
			xmlLanguageNames.add(xmlLanguage.getName());

		xmlLanguagesSourceModel = new SimpleComboBoxModel();
		xmlLanguagesTargetModel = new SimpleComboBoxModel();
		xmlLanguagesSourceModel.init(xmlLanguageNames);
		xmlLanguagesTargetModel.init(xmlLanguageNames);

		templateListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				int index = templateListSelectionModel.getCurrentIndex();
				actionTemplateEdit.setEnabled(index != -1);
				actionTemplateRemove.setEnabled(index != -1);
			}
		});

		if (templateListModel.getSize() > 0)
			templateListSelectionModel.setSelectionIndex(0);
	}


	public boolean showDialog(String templateName) {
		if (templateListModel.getSize() > 0) {
			int startIndex = 0;
			if (templateName != null)
				for (int i=0; i<templateListModel.getSize(); i++)
					if (templateListModel.getElementAt(i).toString().equalsIgnoreCase(templateName)) {
						startIndex = i;
						break;
					}
			templateListSelectionModel.setAnchorSelectionIndex(startIndex);
		}

		return super.showDialog();
	}

	protected Component createMainComponent() {
		templateListModel = new SimpleListModel(MainController.processTemplates.getProcessTemplateList());
		JList templateList = new JList(templateListModel);
		templateListSelectionModel = new SingleListSelectionModel();
		templateList.setSelectionModel(templateListSelectionModel);
		JPanel templatesPanel = new JPanel(new TableLayout(
				new double[][]{{TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED},
							   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 5));
		templatesPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
		templatesPanel.add(new JLabel("Templates:"), new TableLayoutConstraints(0, 0, 1, 0));
		templatesPanel.add(new SmallButton(actionTemplateAdd), new TableLayoutConstraints(1, 1));
		templatesPanel.add(new SmallButton(actionTemplateRemove), new TableLayoutConstraints(2, 1));
		templatesPanel.add(new SmallButton(actionTemplateEdit), new TableLayoutConstraints(3, 1));
		templatesPanel.add(new JScrollPane(templateList), new TableLayoutConstraints(0, 2, 3, 2));

		return templatesPanel;
	}

	protected boolean onOK() {
		MainController.processTemplates.setProcessTemplateList(templateListModel.getList());
		saveAsXML();
		return true;
	}


	protected boolean onCancel() {
		// re-set templates
		int index = templateListSelectionModel.getCurrentIndex();
		templateListModel.init(MainController.processTemplates.getProcessTemplateList());
		if (index < templateListModel.getSize())
			templateListSelectionModel.setSelectionIndex(index);
		else if (templateListModel.getSize() > 0)
			templateListSelectionModel.setSelectionIndex(0);
		return true;
	}

	private void saveAsXML() {
		try {
			MainController.processTemplates.marshal(MainController.PROCESS_TEMPLATES_FILE);
		} catch (IOException exc) {
			System.err.println("Couldn't save Process-Templates file:\n  " + exc.getMessage());
//            exc.printStackTrace();
			JOptionPane.showMessageDialog(ProcessTemplatesDialog.this,
										  "Couldn't save Process-Templates file '"+ MainController.PROCESS_TEMPLATES_FILE+"'!\n\n" + exc.getMessage(), "I/O Error",
										  JOptionPane.ERROR_MESSAGE);
		}
	}

	protected void createActions() {
		super.createActions();

		actionTemplateAdd = new AbstractAction("", MainImages.ADD_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Add new Process Template with current values");
			}
			public void actionPerformed(ActionEvent evt) {
				String templateName =
						JOptionPane.showInputDialog(ProcessTemplatesDialog.this, "Enter name of new template: ",
													"Add Template", JOptionPane.QUESTION_MESSAGE);
				if (templateName == null)
					return; // user cancel
				templateName = templateName.trim();
				if (templateName.length() == 0)
					return; // user cancel

				ProcessTemplate newTemplate = ProcessTemplate.newInstance();
				newTemplate.setName(templateName);
				if (templatePropertiesDialog == null)
					templatePropertiesDialog = new TemplatePropertiesDialog(ProcessTemplatesDialog.this);
				templatePropertiesDialog.setProcessTemplate(newTemplate);
				if (templatePropertiesDialog.showDialog()) {
					templateListModel.add(newTemplate);
					templateListSelectionModel.setSelectionIndex(templateListModel.getSize() - 1);
				}
			}
		};
		actionTemplateRemove = new AbstractAction("", MainImages.REMOVE_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Remove selected Process Template");
			}
			public void actionPerformed(ActionEvent evt) {
				int index = templateListSelectionModel.getCurrentIndex();
				if (index >= 0) {
						if (JOptionPane
								.showConfirmDialog(ProcessTemplatesDialog.this, "Really remove selected template?",
												   "Remove template ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
							!= JOptionPane.YES_OPTION)
							return;
					templateListModel.remove(index);
					if (templateListModel.getSize() == 0)
						templateListSelectionModel.clearSelection();
					else if (index < templateListModel.getSize())
						templateListSelectionModel.setSelectionIndex(index);
					else
						templateListSelectionModel.setSelectionIndex(index - 1);
				}
			}
		};
		actionTemplateEdit = new AbstractAction("", MainImages.EDIT_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Edit selected Template ...");
			}
			public void actionPerformed(ActionEvent evt) {
				if (templatePropertiesDialog == null)
					templatePropertiesDialog = new TemplatePropertiesDialog(ProcessTemplatesDialog.this);
				templatePropertiesDialog.setProcessTemplate((ProcessTemplate) templateListModel.getElementAt(templateListSelectionModel.getCurrentIndex()));
				templatePropertiesDialog.showDialog();
			}
		};
	}

	private SimpleListModel templateListModel;
	private SingleListSelectionModel templateListSelectionModel;

	private Action actionTemplateAdd, actionTemplateRemove, actionTemplateEdit;

	private SimpleComboBoxModel xmlLanguagesSourceModel, xmlLanguagesTargetModel,
			viewLeftSideModel, viewRightSideModel;

	private class TemplatePropertiesDialog extends AbstractInputDialog {
		private ProcessTemplate currentProcessTemplate;
		private final GUIFieldChangeListener templateChangeListener;

		private StepPropertiesDialog stepPropertiesDialog;
		private ViewPropertiesDialog viewPropertiesDialog;

		private TemplatePropertiesDialog(Window parent) {
			super(parent, "Template Properties", false, true, 500, 250, -1, -1, 0, 0);

			templateChangeListener = new GUIFieldChangeListener();
			fieldTemplateName.getDocument().addDocumentListener(templateChangeListener);
			fieldSourceLanguage.addItemListener(templateChangeListener);
			fieldTemplateDescription.getDocument().addDocumentListener(templateChangeListener);
			viewLeftSideModel = new SimpleComboBoxModel();
			viewRightSideModel = new SimpleComboBoxModel();

			stepListSelectionModel.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					int index = stepListSelectionModel.getCurrentIndex();
					actionStepEdit.setEnabled(index != -1);
					actionStepRemove.setEnabled(index != -1);
					actionStepMoveDown.setEnabled(index < stepListModel.getSize()-1);
					actionStepMoveUp.setEnabled(index > 0);
				}
			});
			viewListSelectionModel.addListSelectionListener(new ListSelectionListener() {
				public void valueChanged(ListSelectionEvent e) {
					int index = viewListSelectionModel.getCurrentIndex();
					actionViewEdit.setEnabled(index != -1);
					actionViewRemove.setEnabled(index != -1);
					actionViewMoveDown.setEnabled(index < viewListModel.getSize() - 1);
					actionViewMoveUp.setEnabled(index > 0);
				}
			});
		}

		protected Component createMainComponent() {
			return createTemplatePanel();
		}

		private Component createTemplatePanel() {
			JPanel templatePropertyPanel = new JPanel(new TableLayout(
					new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}, // 120.0
								   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 2, 5));
			templatePropertyPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
			JLabel label = new JLabel("Template-Name: ");
			fieldTemplateName = new JTextField();
			label.setDisplayedMnemonic(KeyEvent.VK_N);
			label.setLabelFor(fieldTemplateName);
			templatePropertyPanel.add(label, new TableLayoutConstraints(0, 0));
			templatePropertyPanel.add(fieldTemplateName, new TableLayoutConstraints(1, 0, 2, 0));

			label = new JLabel("Process Source-Language: ");
			fieldSourceLanguage = new JComboBox(xmlLanguagesSourceModel);
			label.setDisplayedMnemonic(KeyEvent.VK_L);
			label.setLabelFor(fieldSourceLanguage);
			templatePropertyPanel.add(label, new TableLayoutConstraints(0, 1));
			templatePropertyPanel.add(fieldSourceLanguage, new TableLayoutConstraints(1, 1, 2, 1));

			label = new JLabel("Description: ");
			fieldTemplateDescription = new JTextField();
			label.setDisplayedMnemonic(KeyEvent.VK_D);
			label.setLabelFor(fieldTemplateDescription);
			templatePropertyPanel.add(label, new TableLayoutConstraints(0, 2));
			templatePropertyPanel.add(fieldTemplateDescription, new TableLayoutConstraints(1, 2));
			templatePropertyPanel.add(new SmallButton(actionEditTemplateDescription), new TableLayoutConstraints(2, 2));

			JTabbedPane tabbedPane = new JTabbedPane();
			tabbedPane.addTab("Template", templatePropertyPanel);
			tabbedPane.addTab("Steps", createStepsTabPanel());
			tabbedPane.addTab("Views", createViewsTabPanel());

			return tabbedPane;
		}


		private Component createStepsTabPanel() {
			stepListModel = new SimpleListModel();
			JList stepList = new JList(stepListModel);
			stepListSelectionModel = new SingleListSelectionModel();
			stepList.setSelectionModel(stepListSelectionModel);
		//	stepList.addListSelectionListener(createStepListSelectionListener());
			stepList.setCellRenderer(new SimpleListCellRenderer() {
				protected String getLabel(final int index, Object value) {
					return (index + 1) + ": " + ((StepTemplate) value).getName();
				}
			});

			JPanel stepsPanel = new JPanel(new TableLayout(
					new double[][]{
							{TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED,
							 5.0, TableLayout.PREFERRED, TableLayout.PREFERRED},
							{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 5));
			stepsPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
			//	stepsPanel.add(new JLabel("Steps:"), new TableLayoutConstraints(0, 0, 1, 0));
			stepsPanel.add(new SmallButton(actionStepAdd), new TableLayoutConstraints(1, 1));
			stepsPanel.add(new SmallButton(actionStepRemove), new TableLayoutConstraints(2, 1));
			stepsPanel.add(new SmallButton(actionStepEdit), new TableLayoutConstraints(3, 1));
			stepsPanel.add(new SmallButton(actionStepMoveUp), new TableLayoutConstraints(5, 1));
			stepsPanel.add(new SmallButton(actionStepMoveDown), new TableLayoutConstraints(6, 1));
			stepsPanel.add(new JScrollPane(stepList), new TableLayoutConstraints(0, 2, 6, 2));

			return stepsPanel;
		}

		private Component createViewsTabPanel() {
			viewListModel = new SimpleListModel();
			JList viewList = new JList(viewListModel);
			viewListSelectionModel = new SingleListSelectionModel();
			viewList.setSelectionModel(viewListSelectionModel);
			//	viewList.addListSelectionListener(createViewListSelectionListener());
			viewList.setCellRenderer(new SimpleListCellRenderer());

			JPanel viewsPanel = new JPanel(new TableLayout(
					new double[][]{
							{TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, 5.0, TableLayout.PREFERRED,
							 TableLayout.PREFERRED},
							{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 5));
			viewsPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);
			//	stepsPanel.add(new JLabel("Steps:"), new TableLayoutConstraints(0, 0, 1, 0));
			viewsPanel.add(new SmallButton(actionViewAdd), new TableLayoutConstraints(1, 1));
			viewsPanel.add(new SmallButton(actionViewRemove), new TableLayoutConstraints(2, 1));
			viewsPanel.add(new SmallButton(actionViewEdit), new TableLayoutConstraints(3, 1));
			viewsPanel.add(new SmallButton(actionViewMoveUp), new TableLayoutConstraints(5, 1));
			viewsPanel.add(new SmallButton(actionViewMoveDown), new TableLayoutConstraints(6, 1));
			viewsPanel.add(new JScrollPane(viewList), new TableLayoutConstraints(0, 2, 6, 2));

			return viewsPanel;
		}

		public ProcessTemplate getCurrentProcessTemplate() {
			return currentProcessTemplate;
		}

		public void setProcessTemplate(ProcessTemplate currentProcessTemplate) {
			this.currentProcessTemplate = currentProcessTemplate;
			fieldTemplateName.setText(currentProcessTemplate.getName());
			fieldTemplateDescription.setText(currentProcessTemplate.getDescription());
			fieldSourceLanguage.setSelectedItem(currentProcessTemplate.getSourceLanguage());

			stepListModel.init(currentProcessTemplate.getStepTemplateList());
			if (stepListModel.getSize() > 0)
				stepListSelectionModel.setSelectionIndex(0);

			viewListModel.init(currentProcessTemplate.getViewList());
			if (viewListModel.getSize() > 0)
				viewListSelectionModel.setSelectionIndex(0);

			templateChangeListener.reset();
		}

		protected boolean onOK() {
			String name = fieldTemplateName.getText().trim();
			if (name.length() == 0) {
				JOptionPane.showMessageDialog(templatePropertiesDialog, "Template name has not been set!", "Invalid Fields",
											  JOptionPane.ERROR_MESSAGE);
				return false;
			}
			currentProcessTemplate.setName(name);
			currentProcessTemplate.setSourceLanguage((String) fieldSourceLanguage.getSelectedItem());
			currentProcessTemplate.setDescription(fieldTemplateDescription.getText());
			currentProcessTemplate.setStepTemplateList(stepListModel.getList());
			currentProcessTemplate.setViewList(viewListModel.getList());
			return true;
		}


		protected boolean onCancel() {
			if (templateChangeListener.fieldsHaveChanged())
				if (JOptionPane.showConfirmDialog(templatePropertiesDialog, "Changes of current Template will be lost. Really cancel?",
												  "Template changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
						!= JOptionPane.YES_OPTION)
					return false;

			// re-set step list
			stepListModel.init(currentProcessTemplate.getStepTemplateList());
			if (stepListModel.getSize() > 0)
				stepListSelectionModel.setSelectionIndex(0);

			// re-set view list
			viewListModel.init(currentProcessTemplate.getViewList());
			if (viewListModel.getSize() > 0)
				viewListSelectionModel.setSelectionIndex(0);

			return true;
		}

		protected void createActions() {
			super.createActions();

			actionEditTemplateDescription = new AbstractAction("", MainImages.EDIT_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Edit Template Description...");
				}

				private TextEditDialog editDialog;

				public void actionPerformed(ActionEvent evt) {
					if (editDialog == null)
						editDialog = new TextEditDialog(templatePropertiesDialog, "Edit Template Description...");
					String newValue = editDialog.showEditDialog(fieldTemplateDescription.getText());
					if (newValue != null) {
						fieldTemplateDescription.setText(newValue);
						fieldTemplateDescription.requestFocusInWindow();
					}
				}
			};

			actionStepAdd = new AbstractAction("", MainImages.ADD_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Add new Process step");
				}
				public void actionPerformed(ActionEvent evt) {
					String stepName =
							JOptionPane.showInputDialog(templatePropertiesDialog, "Enter name of new step: ",
														"Add Step", JOptionPane.QUESTION_MESSAGE);
					if (stepName == null)
						return; // user cancel
					stepName = stepName.trim();
					if (stepName.length() == 0)
						return; // user cancel

					StepTemplate newStep= StepTemplate.newInstance();
					newStep.setName(stepName);
					if (stepPropertiesDialog == null)
						stepPropertiesDialog = new StepPropertiesDialog(templatePropertiesDialog);

					String sourceLanguageName;
					int stepIndex = stepListModel.getSize();
					if (stepIndex == 0)
						sourceLanguageName = (String) fieldSourceLanguage.getSelectedItem();
					else
						sourceLanguageName = ((StepTemplate)stepListModel.getElementAt(stepIndex - 1)).getTargetLanguage();
					stepPropertiesDialog.setStepTemplate(newStep, sourceLanguageName);

					if (stepPropertiesDialog.showDialog()) {
						stepListModel.add(newStep);
						stepListSelectionModel.setSelectionIndex(stepListModel.getSize() - 1);
					}
				}
			};
			actionStepRemove = new AbstractAction("", MainImages.REMOVE_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Remove selected Process Step");
				}
				public void actionPerformed(ActionEvent evt) {
					int index = stepListSelectionModel.getCurrentIndex();
					if (index >= 0) {
							if (JOptionPane
									.showConfirmDialog(templatePropertiesDialog, "Really remove selected step?",
													   "Remove Step ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
								!= JOptionPane.YES_OPTION)
								return;
						stepListModel.remove(index);
						if (stepListModel.getSize() == 0)
							stepListSelectionModel.clearSelection();
						else if (index < stepListModel.getSize())
							stepListSelectionModel.setSelectionIndex(index);
						else
							stepListSelectionModel.setSelectionIndex(index - 1);
					}
				}
			};
			actionStepMoveUp = new AbstractAction("", MainImages.SEARCH_UP_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Move selected Process Step up");
				}

				public void actionPerformed(ActionEvent evt) {
					int index = stepListSelectionModel.getCurrentIndex();
					if (index > 0) {
						Object step = stepListModel.getElementAt(index);
						stepListModel.remove(index);
						stepListModel.add(index - 1, step);
						stepListSelectionModel.setSelectionIndex(index - 1);
					}
				}
			};
			actionStepMoveDown = new AbstractAction("", MainImages.SEARCH_DOWN_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Move selected Process Step down");
				}
				public void actionPerformed(ActionEvent evt) {
					int index = stepListSelectionModel.getCurrentIndex();
					if (index < stepListModel.getSize() - 1) {
						Object step = stepListModel.getElementAt(index);
						stepListModel.remove(index);
						stepListModel.add(index + 1, step);
						stepListSelectionModel.setSelectionIndex(index + 1);
					}
				}
			};
			actionStepEdit = new AbstractAction("", MainImages.EDIT_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Edit selected step");
				}
				public void actionPerformed(ActionEvent evt) {
					if (stepPropertiesDialog == null)
						stepPropertiesDialog = new StepPropertiesDialog(templatePropertiesDialog);
					int stepIndex = stepListSelectionModel.getCurrentIndex();
					StepTemplate currentStepTemplate = (StepTemplate) stepListModel.getElementAt(stepIndex);
					String sourceLanguageName;
					if (stepIndex == 0)
						sourceLanguageName = (String) fieldSourceLanguage.getSelectedItem();
					else
						sourceLanguageName = ((StepTemplate) stepListModel.getElementAt(stepIndex - 1)).getTargetLanguage();
					stepPropertiesDialog.setStepTemplate(currentStepTemplate, sourceLanguageName);
					stepPropertiesDialog.showDialog();
				}
			};

			actionViewAdd = new AbstractAction("", MainImages.ADD_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Add new View (with current values)");
				}

				public void actionPerformed(ActionEvent evt) {
					String viewName =
							JOptionPane.showInputDialog(templatePropertiesDialog, "Enter name of new view: ",
														"Add View", JOptionPane.QUESTION_MESSAGE);
					if (viewName == null)
						return; // user cancel
					viewName = viewName.trim();
					if (viewName.length() == 0)
						return; // user cancel

					View newView = View.newInstance();
					newView.setName(viewName);
					newView.setLeftFile(LeftFile.newInstance());
					newView.getLeftFile().setStepIndex("0");
					newView.setRightFile(RightFile.newInstance());
					newView.getRightFile().setStepIndex("0");
					if (viewPropertiesDialog == null)
						viewPropertiesDialog = new ViewPropertiesDialog(templatePropertiesDialog);
					viewPropertiesDialog.setView(newView);
					if (viewPropertiesDialog.showDialog()) {
						viewListModel.add(newView);
						viewListSelectionModel.setSelectionIndex(viewListModel.getSize() - 1);
					}
				}
			};

			actionViewEdit = new AbstractAction("", MainImages.EDIT_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Edit selected step");
				}
				public void actionPerformed(ActionEvent evt) {
					if (viewPropertiesDialog == null)
						viewPropertiesDialog = new ViewPropertiesDialog(templatePropertiesDialog);
					int viewIndex = viewListSelectionModel.getCurrentIndex();
					viewPropertiesDialog.setView((View) viewListModel.getElementAt(viewIndex));
					viewPropertiesDialog.showDialog();
				}
			};

			actionViewRemove = new AbstractAction("", MainImages.REMOVE_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Remove selected View");
				}

				public void actionPerformed(ActionEvent evt) {
					int index = viewListSelectionModel.getCurrentIndex();
					if (index >= 0) {
						if (JOptionPane
								.showConfirmDialog(templatePropertiesDialog, "Really remove selected view?",
												   "Remove View ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
							!= JOptionPane.YES_OPTION)
							return;
						viewListModel.remove(index);
						if (viewListModel.getSize() == 0)
							viewListSelectionModel.clearSelection();
						else if (index < viewListModel.getSize())
							viewListSelectionModel.setSelectionIndex(index);
						else
							viewListSelectionModel.setSelectionIndex(index - 1);
					}
				}
			};
			actionViewMoveUp = new AbstractAction("", MainImages.SEARCH_UP_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Move selected View up");
				}

				public void actionPerformed(ActionEvent evt) {
					int index = viewListSelectionModel.getCurrentIndex();
					if (index > 0) {
						Object view = viewListModel.getElementAt(index);
						viewListModel.remove(index);
						viewListModel.add(index - 1, view);
						viewListSelectionModel.setSelectionIndex(index - 1);
					}
				}
			};
			actionViewMoveDown = new AbstractAction("", MainImages.SEARCH_DOWN_ICON) {
				{
					putValue(StatusBar.TOOL_TIP, "Move selected View down");
				}
				public void actionPerformed(ActionEvent evt) {
					int index = viewListSelectionModel.getCurrentIndex();
					if (index < viewListModel.getSize() - 1) {
						Object view = viewListModel.getElementAt(index);
						viewListModel.remove(index);
						viewListModel.add(index + 1, view);
						viewListSelectionModel.setSelectionIndex(index + 1);
					}
				}
			};
		}

		private JTextField fieldTemplateName;
		private JComboBox fieldSourceLanguage;
		private JTextField fieldTemplateDescription;

		private SimpleListModel stepListModel;
		private SingleListSelectionModel stepListSelectionModel;

		private JPanel stepPanel;
		private TitledBorder stepsPanelTitleBorder;

		private SimpleListModel viewListModel;
		private SingleListSelectionModel viewListSelectionModel;

		private Action actionEditTemplateDescription, actionStepAdd, actionStepRemove, actionStepEdit, actionStepMoveUp, actionStepMoveDown,
						actionViewAdd, actionViewRemove, actionViewEdit, actionViewMoveUp, actionViewMoveDown;

		private class StepPropertiesDialog extends AbstractInputDialog {
			private StepTemplate currentStepTemplate;
			private final GUIFieldChangeListener stepChangeListener;
			private boolean checkTargetLanguageChange = true;

			private StepPropertiesDialog(Window parent) {
				super(parent, "Step Properties", false, true, 500, 250, -1, -1, 0, 0);

				stepChangeListener = new GUIFieldChangeListener();
				fieldStepName.getDocument().addDocumentListener(stepChangeListener);
				fieldStepDescription.getDocument().addDocumentListener(stepChangeListener);
				fieldStepMacroFile.getDocument().addDocumentListener(stepChangeListener);
				fieldStepTargetLanguage.addItemListener(stepChangeListener);

				fieldStepTargetLanguage.addItemListener(new ItemListener() {
					public void itemStateChanged(ItemEvent evt) {
						if (checkTargetLanguageChange && (evt.getStateChange() == ItemEvent.SELECTED)) {
							String newStepName = fieldStepSourceLanguage.getText() + " -> " + fieldStepTargetLanguage.getSelectedItem();
							if (JOptionPane.showConfirmDialog(viewPropertiesDialog, "Update step name to \""+newStepName+"\"?",
														  "Target Language changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
									== JOptionPane.YES_OPTION)
								fieldStepName.setText(newStepName);
							}
					}
				});
			}

			protected Component createMainComponent() {
				return createStepPanel();
			}

			private Component createStepPanel() {
				stepPanel = new JPanel(new TableLayout(
						new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}, // 120.0
									   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED,
										TableLayout.PREFERRED, TableLayout.PREFERRED}}, 2, 5));
				stepsPanelTitleBorder = new TitledBorder(" Process-Step ");
				stepPanel.setBorder(new CompoundBorder(stepsPanelTitleBorder, new EmptyBorder(0, 2, 2, 2)));

				JLabel label = new JLabel("Step-Name: ");
				fieldStepName = new JTextField();
				label.setDisplayedMnemonic(KeyEvent.VK_N);
				label.setLabelFor(fieldStepName);
				stepPanel.add(label, new TableLayoutConstraints(0, 0));
				stepPanel.add(fieldStepName, new TableLayoutConstraints(1, 0, 2, 0));

				label = new JLabel("Source-Language: ");
				fieldStepSourceLanguage = new JLabel();
				fieldStepSourceLanguage.setFont(fieldStepName.getFont());
				fieldStepSourceLanguage.setToolTipText(
						"<html><b>source</b> language of this step,<br>document based on this language will be shown in the <b>left</b> side pane");
				stepPanel.add(label, new TableLayoutConstraints(0, 1));
				stepPanel.add(fieldStepSourceLanguage, new TableLayoutConstraints(1, 1, 2, 1));

				label = new JLabel("Target-Language: ");
				fieldStepTargetLanguage = new JComboBox(xmlLanguagesTargetModel);
				fieldStepTargetLanguage.setToolTipText(
						"<html><b>target</b> language of this step,<br>document based on this language will be shown in the <b>right</b> side pane");
				label.setDisplayedMnemonic(KeyEvent.VK_T);
				label.setLabelFor(fieldStepTargetLanguage);
				stepPanel.add(label, new TableLayoutConstraints(0, 2));
				stepPanel.add(fieldStepTargetLanguage, new TableLayoutConstraints(1, 2, 2, 2));

				label = new JLabel("Macro-File: ");
				fieldStepMacroFile = new JTextField();
				fieldStepMacroFile.setEditable(false);
				label.setDisplayedMnemonic(KeyEvent.VK_M);
				label.setLabelFor(fieldStepMacroFile);
				stepPanel.add(label, new TableLayoutConstraints(0, 3));
				stepPanel.add(fieldStepMacroFile, new TableLayoutConstraints(1, 3));
				stepPanel.add(new SmallButton(actionChangeStepMacroFile), new TableLayoutConstraints(2, 3));

				label = new JLabel("Description: ");
				fieldStepDescription = new JTextField();
				label.setDisplayedMnemonic(KeyEvent.VK_D);
				label.setLabelFor(fieldStepDescription);
				stepPanel.add(label, new TableLayoutConstraints(0, 4));
				stepPanel.add(fieldStepDescription, new TableLayoutConstraints(1, 4));
				stepPanel.add(new SmallButton(actionEditStepDescription), new TableLayoutConstraints(2, 4));

				return stepPanel;
			}

			public StepTemplate getCurrentStepTemplate() {
				return currentStepTemplate;
			}

			public void setStepTemplate(StepTemplate currentStepTemplate, String sourceLanguageName) {
				this.currentStepTemplate = currentStepTemplate;
				fieldStepName.setText(currentStepTemplate.getName());
				fieldStepSourceLanguage.setText(sourceLanguageName);
				checkTargetLanguageChange = false;
				fieldStepTargetLanguage.setSelectedItem(currentStepTemplate.getTargetLanguage());
				checkTargetLanguageChange = true;
				fieldStepMacroFile.setText(currentStepTemplate.getMacroFile());
				fieldStepDescription.setText(currentStepTemplate.getDescription());

				stepChangeListener.reset();
			}


			protected boolean onOK() {
				if (fieldStepTargetLanguage.getSelectedItem() == null) {
					JOptionPane.showMessageDialog(stepPropertiesDialog, "Target language has not been set!", "Invalid Fields", JOptionPane.ERROR_MESSAGE);
					return false;
				}
				String name = fieldStepName.getText().trim();
				if (name.length() == 0) {
					JOptionPane.showMessageDialog(stepPropertiesDialog, "Step name has not been set!", "Invalid Fields",
												  JOptionPane.ERROR_MESSAGE);
					return false;
				}
				currentStepTemplate.setName(name);
				currentStepTemplate.setTargetLanguage(fieldStepTargetLanguage.getSelectedItem().toString());
				currentStepTemplate.setMacroFile(fieldStepMacroFile.getText());
				currentStepTemplate.setDescription(fieldStepDescription.getText());
				return true;
			}

			protected boolean onCancel() {
				if (stepChangeListener.fieldsHaveChanged())
					if (JOptionPane.showConfirmDialog(stepPropertiesDialog, "Changes of current Step will be lost. Really cancel?",
													  "Step changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
							!= JOptionPane.YES_OPTION)
						return false;
				return true;
			}

			protected void createActions() {
				super.createActions();

				actionChangeStepMacroFile = new AbstractAction("", MiscImages.DOTS_ICON) {
					private DeltaFileChooser fileChooser;

					{
						putValue(StatusBar.TOOL_TIP, "Change Step Macro File ...");
						fileChooser = new DeltaFileChooser("Choose Macro file");
					}

					public void actionPerformed(ActionEvent evt) {
						String xmlLanguageName = fieldStepTargetLanguage.getSelectedItem().toString();
						if (!xmlLanguageName.equals("HTML")) {
							fileChooser.setDialogTitle("Choose '" + xmlLanguageName + "' Macro file");
							fileChooser.setFileFilter(new DeltaFileChooser.MacroFileFilter(xmlLanguageName));
							File macroFile =
									FileUtils.toExistingAbsoluteFile(fieldStepMacroFile.getText(),
																	 MainController.processTemplates.getFile());
							fileChooser.setSelectedFile(macroFile);
							if (fileChooser.showOpenDialog(stepPropertiesDialog) == JFileChooser.APPROVE_OPTION)
								fieldStepMacroFile.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(),
																					MainController.processTemplates.getFile()));
						}
					}
				};
				actionEditStepDescription = new AbstractAction("", MainImages.EDIT_ICON) {
					{
						putValue(StatusBar.TOOL_TIP, "Edit Step Description...");
					}

					private TextEditDialog editDialog;

					public void actionPerformed(ActionEvent evt) {
						if (editDialog == null)
							editDialog = new TextEditDialog(stepPropertiesDialog, "Edit Step Description...");
						String newValue = editDialog.showEditDialog(fieldStepDescription.getText());
						if (newValue != null) {
							fieldStepDescription.setText(newValue);
							fieldStepDescription.requestFocusInWindow();
						}
					}
				};

			}

			private JTextField fieldStepName;
			private JTextField fieldStepMacroFile;
			private JTextField fieldStepDescription;
			private JLabel fieldStepSourceLanguage;
			private JComboBox fieldStepTargetLanguage;

			private Action 	actionChangeStepMacroFile, actionEditStepDescription;
		}

		private class ViewPropertiesDialog extends AbstractInputDialog {
			private View currentView;
			private final GUIFieldChangeListener viewChangeListener;

			private ViewPropertiesDialog(Window parent) {
				super(parent, "View Properties", false, true, 500, 200, -1, -1, 0, 0);

				viewChangeListener = new GUIFieldChangeListener();
				fieldViewName.getDocument().addDocumentListener(viewChangeListener);
				fieldViewDescription.getDocument().addDocumentListener(viewChangeListener);
				fieldViewLeftSide.addItemListener(viewChangeListener);
				fieldViewRightSide.addItemListener(viewChangeListener);
			}

			protected Component createMainComponent() {
				return createViewPanel();
			}

			private Component createViewPanel() {
				JPanel viewPanel = new JPanel(new TableLayout(
						new double[][]{{TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED}, // 120.0
									   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED,
										TableLayout.PREFERRED}}, 2, 5));
				viewPanel.setBorder(GlobalObjects.EMPTY_BORDER_2);

				JLabel label = new JLabel("View-Name: ");
				fieldViewName = new JTextField();
				label.setDisplayedMnemonic(KeyEvent.VK_N);
				label.setLabelFor(fieldViewName);
				viewPanel.add(label, new TableLayoutConstraints(0, 0));
				viewPanel.add(fieldViewName, new TableLayoutConstraints(1, 0, 2, 0));

				label = new JLabel("Left Side: ");
				fieldViewLeftSide = new JComboBox(viewLeftSideModel);
				fieldViewLeftSide.setRenderer(createViewListCellRenderer());
				label.setDisplayedMnemonic(KeyEvent.VK_L);
				label.setLabelFor(fieldViewLeftSide);
				viewPanel.add(label, new TableLayoutConstraints(0, 1));
				viewPanel.add(fieldViewLeftSide, new TableLayoutConstraints(1, 1, 2, 1));

				label = new JLabel("Right Side: ");
				fieldViewRightSide = new JComboBox(viewRightSideModel);
				fieldViewRightSide.setRenderer(createViewListCellRenderer());
				label.setDisplayedMnemonic(KeyEvent.VK_L);
				label.setLabelFor(fieldViewRightSide);
				viewPanel.add(label, new TableLayoutConstraints(0, 2));
				viewPanel.add(fieldViewRightSide, new TableLayoutConstraints(1, 2, 2, 2));

				label = new JLabel("Description: ");
				fieldViewDescription = new JTextField();
				label.setDisplayedMnemonic(KeyEvent.VK_D);
				label.setLabelFor(fieldViewDescription);
				viewPanel.add(label, new TableLayoutConstraints(0, 3));
				viewPanel.add(fieldViewDescription, new TableLayoutConstraints(1, 3));
				viewPanel.add(new SmallButton(actionEditViewDescription), new TableLayoutConstraints(2, 3));

				return viewPanel;
			}

			private ListCellRenderer createViewListCellRenderer() {
				return new SimpleListCellRenderer() {
					protected String getLabel(final int index, Object value) {
						if (value == null)
							return "";
						Integer stepIndex = (Integer) value;
						if (stepIndex == 0)
							return "Step #0: " + fieldSourceLanguage.getSelectedItem().toString();
						StepTemplate step = (StepTemplate) stepListModel.getElementAt(stepIndex - 1);
						return "Step #" + stepIndex + ": " + step.getTargetLanguage();
					}
				};
			}

			private java.util.List buildStepNameList() {
				java.util.List result = new LinkedList();
				result.add(new Integer(0));
				for (int i = 0; i < stepListModel.getSize(); i++) {
					result.add(i + 1);
				}
				return result;
			}

			public View getCurrentView() {
				return currentView;
			}

			public void setView(View currentView) {
				this.currentView = currentView;

				List names = buildStepNameList();
				viewLeftSideModel.init(names, viewLeftSideModel.getSelectedItem());
				viewRightSideModel.init(names, viewRightSideModel.getSelectedItem());

				if (currentView.getLeftFile() == null) {
					currentView.setLeftFile(LeftFile.newInstance());
					currentView.getLeftFile().setStepIndex("0");
				}
				if (currentView.getRightFile() == null) {
					currentView.setRightFile(RightFile.newInstance());
					currentView.getRightFile().setStepIndex("0");
				}
				fieldViewName.setText(currentView.getName());
				fieldViewLeftSide.setSelectedItem(Integer.parseInt(currentView.getLeftFile().getStepIndex()));
				fieldViewRightSide.setSelectedItem(Integer.parseInt(currentView.getRightFile().getStepIndex()));
				fieldViewDescription.setText(currentView.getDescription());

				viewChangeListener.reset();
			}


			protected boolean onOK() {
				String name = fieldViewName.getText().trim();
				if (name.length() == 0) {
					JOptionPane.showMessageDialog(viewPropertiesDialog, "View name has not been set!", "Invalid Fields",
												  JOptionPane.ERROR_MESSAGE);
					return false;
				}
				if (fieldViewLeftSide.getSelectedItem() == null) {
					JOptionPane.showMessageDialog(viewPropertiesDialog, "Left side step has not been set!", "Invalid Fields",
												  JOptionPane.ERROR_MESSAGE);
					return false;
				}
				if (fieldViewRightSide.getSelectedItem() == null) {
					JOptionPane.showMessageDialog(viewPropertiesDialog, "Right side step has not been set!", "Invalid Fields",
												  JOptionPane.ERROR_MESSAGE);
					return false;
				}
				currentView.setName(name);
				currentView.getLeftFile().setStepIndex(fieldViewLeftSide.getSelectedItem().toString());
				currentView.getRightFile().setStepIndex(fieldViewRightSide.getSelectedItem().toString());
				currentView.setDescription(fieldViewDescription.getText());
				return true;
			}

			protected boolean onCancel() {
				if (viewChangeListener.fieldsHaveChanged())
					if (JOptionPane.showConfirmDialog(viewPropertiesDialog, "Changes of current View will be lost. Really cancel?",
													  "View changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
						!= JOptionPane.YES_OPTION)
						return false;
				return true;
			}

			protected void createActions() {
				super.createActions();

				actionEditViewDescription = new AbstractAction("", MainImages.EDIT_ICON) {
					{
						putValue(StatusBar.TOOL_TIP, "Edit View Description...");
					}

					private TextEditDialog editDialog;

					public void actionPerformed(ActionEvent evt) {
						if (editDialog == null)
							editDialog = new TextEditDialog(viewPropertiesDialog, "Edit View Description...");
						String newValue = editDialog.showEditDialog(fieldViewDescription.getText());
						if (newValue != null) {
							fieldViewDescription.setText(newValue);
							fieldViewDescription.requestFocusInWindow();
						}
					}
				};
			}

			private JTextField fieldViewName;
			private JComboBox fieldViewLeftSide;
			private JComboBox fieldViewRightSide;
			private JTextField fieldViewDescription;

			private Action actionEditViewDescription;
		}
	}
}
