/*
 * ProjectPropertiesDialog.java
 *
 * Created on 03. J?nner 2002, 22:01
 */

package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideController;
import at.ac.tuwien.asgaard.delta.gui.GUIConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.xmlBeans.Project;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.swing.DocumentChangeAdapter;
import at.ac.tuwien.asgaard.swing.StatusBar;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import at.ac.tuwien.asgaard.util.FileUtils;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * Class ProjectPropertiesDialog ...
 * @author  Peter VOTRUBA
 */
public final class ProjectPropertiesDialog extends AbstractInputDialog {
	private Project project;
	private Action changeProjectFile, editDescription, changeLeftAction, changeRightAction, changeMacrosFileAction;
	private static DeltaFileChooser fileChooser;

	private File projectFile;
	private boolean fieldsHaveBeenChanged;
	public static final String MISSING_FILE_MSG = "File does not exist!";
	private static Border TEXTFIELD_BORDER;

	private static ProjectPropertiesDialog sharedInstance;

	public static ProjectPropertiesDialog getSharedInstance() {
		if (sharedInstance == null)
			sharedInstance = new ProjectPropertiesDialog(MainController.mainFrame);
		return sharedInstance;
	}

	/** Creates new form ProjectPropertiesDialog */
	private ProjectPropertiesDialog(Frame parent) {
		super(parent, "Project Properties", false, true, 500, 280, 350, -1, 0, -1);

		helpAction.putValue(HELP_LINK_KEY, "ProjectPropertiesDialog");
	}
	public Project getDELTAProject() {
		return project;
	}
	public void setDELTAProject(Project project) {
		this.project = project;
		projectFile = project.getProjectFile();
		jTextFieldName.setText(project.getName());
		jTextFieldProjectFile.setText(projectFile.getAbsolutePath());
		jTextFieldDescription.setText(project.getDescription());

		jTextFieldLeft.setText(project.getLeftSideFileName());
		if (project.getLeftSideFileName() == null)
			jTextFieldLeft.setBorder(new LineBorder(Color.BLACK));
		else if (project.getLeftSideFile() == null) {
			jTextFieldLeft.setBorder(new LineBorder(Color.RED));
			jTextFieldLeft.setToolTipText(MISSING_FILE_MSG);
		} else
			jTextFieldLeft.setBorder(TEXTFIELD_BORDER);

		jTextFieldRight.setText(project.getRightSideFileName());
		if (project.getRightSideFileName() == null)
			jTextFieldRight.setBorder(new LineBorder(Color.BLACK));
		else if (project.getRightSideFile() == null) {
			jTextFieldRight.setBorder(new LineBorder(Color.RED));
			jTextFieldRight.setToolTipText(MISSING_FILE_MSG);
		} else
			jTextFieldRight.setBorder(TEXTFIELD_BORDER);

		jTextFieldMacros.setText(project.getMacrosFileName());
		if (project.getMacrosFileName() == null)
			jTextFieldMacros.setBorder(new LineBorder(Color.BLACK));
		else if (project.getMacrosFile() == null) {
			jTextFieldMacros.setBorder(new LineBorder(Color.RED));
			jTextFieldMacros.setToolTipText(MISSING_FILE_MSG);
		} else
			jTextFieldMacros.setBorder(TEXTFIELD_BORDER);

		jTextFieldName.requestFocusInWindow();
		fieldsHaveBeenChanged = false; 
	}

	protected boolean onOK() {
	// if (fieldsHaveBeenChanged) {
			String name = jTextFieldName.getText().trim(), description = jTextFieldDescription.getText(),
					leftFileName = jTextFieldLeft.getText().trim(), rightFileName = jTextFieldRight.getText().trim(),
					macrosFileName = jTextFieldMacros.getText().trim();

		if (name.equals("")) {
				JOptionPane.showMessageDialog(this, "Enter a project name!", "Input Error", JOptionPane.ERROR_MESSAGE);
				jTextFieldName.requestFocusInWindow();
				return false;
			}

			File leftSideFile = null;
			if (!leftFileName.equals("")) {
				leftSideFile = FileUtils.toExistingAbsoluteFile(leftFileName, projectFile);
				if (leftSideFile == null) {
					JOptionPane.showMessageDialog(this, "Enter an existing file for the left side!", "Input Error", JOptionPane.ERROR_MESSAGE);
					jTextFieldLeft.requestFocusInWindow();
					return false;
				}
			}
			File rightSideFile = null;
			if (!rightFileName.equals("")) {
				rightSideFile = FileUtils.toExistingAbsoluteFile(rightFileName, projectFile);
				if (rightSideFile == null) {
					JOptionPane.showMessageDialog(this, "Enter an existing file for the right side!", "Input Error", JOptionPane.ERROR_MESSAGE);
					jTextFieldRight.requestFocusInWindow();
					return false;
				}
			}
			File macrosFile = null;
			if (!macrosFileName.equals("")) {
				macrosFile = FileUtils.toExistingAbsoluteFile(macrosFileName, projectFile);
				if (macrosFile == null) {
					JOptionPane.showMessageDialog(this, "Enter an existing macros file!", "Input Error", JOptionPane.ERROR_MESSAGE);
					jTextFieldRight.requestFocusInWindow();
					return false;
				}
			}
	// }
		project.setName(name);
		project.setLeftSideFile(leftSideFile);
		project.setRightSideFile(rightSideFile);
		project.setMacrosFile(macrosFile);
		project.setDescription(description);
		try {
			project.saveAs(projectFile);
		} catch (IOException exc) {
            System.err.println("Couldn't save DELT/A-Project File:\n  "+exc.getMessage());
//            exc.printStackTrace();
			JOptionPane.showMessageDialog(ProjectPropertiesDialog.this, "Couldn't save DELT/A-Project File!\n\n"+exc.getMessage(), "I/O Error", JOptionPane.ERROR_MESSAGE);
		}
		return true;
	}

	protected boolean onCancel() {
		if (fieldsHaveBeenChanged)
			return (JOptionPane.showConfirmDialog(this, "Fields have been changed!\nDo you really want to discard changes?", "Confirm Cancel", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION);
		return true;
	}

	public static boolean saveProjectAs(Project project, Component parentComponent) {
		if (fileChooser == null)
			fileChooser = new DeltaFileChooser(project.getProjectFile().getParentFile(), "Save Project File as ...");
		else fileChooser.setDialogTitle("Save Project File as ...");
		fileChooser.resetChoosableFileFilters();
		fileChooser.setFileFilter(DeltaFileChooser.PROJECT_FILE_FILTER);
		if (fileChooser.showSaveDialog(parentComponent) == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			try {
				project.setName(FileUtils.getFileName(file));
				project.saveAs(file);
				return true;
			} catch (IOException exc) {
				exc.printStackTrace();
				JOptionPane.showMessageDialog(parentComponent, "Couldn't save DELT/A-Project File under \"" + file.getAbsolutePath() + "\"!", "I/O Error", JOptionPane.ERROR_MESSAGE);
			}
		}
		return false;
	}

	protected void createActions() {
		super.createActions();

		changeProjectFile = new AbstractAction("", MiscImages.DOTS_ICON) {
            {putValue(StatusBar.TOOL_TIP, "Change Project File...");}
			public void actionPerformed(ActionEvent evt) {
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser(project.getProjectFile().getParentFile(), "Choose Project File ...");
				else
					fileChooser.setDialogTitle("Choose Project File ...");
				fileChooser.resetChoosableFileFilters();
				fileChooser.setFileFilter(DeltaFileChooser.PROJECT_FILE_FILTER);
				fileChooser.setCurrentDirectory(projectFile.getParentFile());
				fileChooser.setApproveButtonText("Select");
				fileChooser.setSelectedFile(projectFile);
				if (fileChooser.showSaveDialog(ProjectPropertiesDialog.this) == JFileChooser.APPROVE_OPTION) {
					File oldProjectFile = projectFile;
					projectFile = fileChooser.getSelectedFile();
					jTextFieldProjectFile.setText(projectFile.getAbsolutePath());
					jTextFieldName.setText(FileUtils.getFileName(projectFile));
                    System.out.println("... oldProjectFile: "+ oldProjectFile);
                    System.out.println("... projectFile: "+ projectFile);
					jTextFieldLeft.setText(FileUtils.toRelativeFile(FileUtils.toAbsoluteFile(jTextFieldLeft.getText(), oldProjectFile), projectFile));
					jTextFieldRight.setText(FileUtils.toRelativeFile(FileUtils.toAbsoluteFile(jTextFieldRight.getText(), oldProjectFile), projectFile));
					jTextFieldMacros.setText(FileUtils.toRelativeFile(FileUtils.toAbsoluteFile(jTextFieldMacros.getText(), oldProjectFile), projectFile));
					fieldsHaveBeenChanged = true;
				}
			}
		};
		editDescription = new AbstractAction("", MainImages.EDIT_ICON) {
            {putValue(StatusBar.TOOL_TIP, "Edit Project Description...");}
			private TextEditDialog editDialog;
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(ProjectPropertiesDialog.this, "Edit Project Description...");
				String newValue = editDialog.showEditDialog(jTextFieldDescription.getText());
				if (newValue != null) {
					jTextFieldDescription.setText(newValue);
					jTextFieldDescription.requestFocusInWindow();
				}
			}
		};
		changeLeftAction = new AbstractAction("", MiscImages.DOTS_ICON) {
            {putValue(StatusBar.TOOL_TIP, "Change Left File ...");}
			public void actionPerformed(ActionEvent evt) {
				File leftFile = project.getLeftSideFile();
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser("Choose Left file ...");
				else
					fileChooser.setDialogTitle("Choose Left file ...");
				if (leftFile == null)
					fileChooser.setCurrentDirectory(projectFile.getParentFile());
				else
					fileChooser.setSelectedFile(leftFile);

                fileChooser.resetChoosableFileFilters();

				DeltaSideController thisSideController = MainController.deltaSideController[GUIConstants.LEFT];
				for (XmlLanguage xmlLanguage : MainController.xmlLanguages)
                    if (xmlLanguage != thisSideController.currentXMLLanguage)
                        fileChooser.addChoosableFileFilter(xmlLanguage.getFileFilter()); //new DocTypeFileFilter(xmlLanguage));

                fileChooser.addChoosableFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
                fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
                fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
                if (thisSideController.isTypeHTML())
					fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
                else if (thisSideController.currentXMLLanguage != null)
                    fileChooser.setFileFilter(thisSideController.currentXMLLanguage.getFileFilter()); //new DocTypeFileFilter(thisSideController.currentXMLLanguage));

				if (fileChooser.showDialog(ProjectPropertiesDialog.this, "Select") == JFileChooser.APPROVE_OPTION) {
					jTextFieldLeft.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(), projectFile));
					jTextFieldLeft.setBorder(TEXTFIELD_BORDER);
					jTextFieldLeft.setToolTipText(null);
					fieldsHaveBeenChanged = true;
				// jTextFieldLeft.requestFocus();
				}
			}
		};
		changeRightAction = new AbstractAction("", MiscImages.DOTS_ICON) {
            {putValue(StatusBar.TOOL_TIP, "Change Right File ...");}
			public void actionPerformed(ActionEvent evt) {
				File rightFile = project.getRightSideFile();
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser("Choose Right file ...");
				else
					fileChooser.setDialogTitle("Choose Right file ...");
				if (rightFile == null)
					fileChooser.setCurrentDirectory(projectFile.getParentFile());
				else
					fileChooser.setSelectedFile(rightFile);

                fileChooser.resetChoosableFileFilters();
                fileChooser.addChoosableFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
                fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
                fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
                DeltaSideController thisSideController = MainController.deltaSideController[GUIConstants.RIGHT];

				for (XmlLanguage xmlLanguage : MainController.xmlLanguages)
                    if (xmlLanguage != thisSideController.currentXMLLanguage)
                        fileChooser.addChoosableFileFilter(xmlLanguage.getFileFilter()); //new DocTypeFileFilter(xmlLanguage));

				fileChooser.addChoosableFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
				fileChooser.addChoosableFileFilter(DeltaFileChooser.ANY_XML_FILE_FILTER);
				fileChooser.addChoosableFileFilter(DeltaFileChooser.XML_FILE_FILTER);
				if (thisSideController.isTypeHTML())
					fileChooser.setFileFilter(DeltaFileChooser.HTML_FILE_FILTER);
				else if (thisSideController.currentXMLLanguage != null)
                    fileChooser.setFileFilter(thisSideController.currentXMLLanguage.getFileFilter()); //new DocTypeFileFilter(thisSideController.currentXMLLanguage));

				if (fileChooser.showDialog(ProjectPropertiesDialog.this, "Select") == JFileChooser.APPROVE_OPTION) {
					jTextFieldRight.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(), projectFile));
					jTextFieldRight.setBorder(TEXTFIELD_BORDER);
					jTextFieldRight.setToolTipText(null);
					fieldsHaveBeenChanged = true;
				// jTextFieldRight.requestFocus();
				}
			}
		};
		changeMacrosFileAction = new AbstractAction("", MiscImages.DOTS_ICON) {
            {putValue(StatusBar.TOOL_TIP, "Change Macros File...");}
			public void actionPerformed(ActionEvent evt) {
				File macrosFile = project.getMacrosFile();
				if (fileChooser == null)
					fileChooser = new DeltaFileChooser("Choose Macros file ...");
				else
					fileChooser.setDialogTitle("Choose Macros file ...");
				if (macrosFile == null)
					fileChooser.setCurrentDirectory(projectFile.getParentFile());
				else
					fileChooser.setSelectedFile(macrosFile);

				fileChooser.resetChoosableFileFilters();
				fileChooser.setFileFilter(DeltaFileChooser.MACROS_FILE_FILTER);
				if (fileChooser.showDialog(ProjectPropertiesDialog.this, "Select") == JFileChooser.APPROVE_OPTION) {
					jTextFieldMacros.setText(FileUtils.toRelativeFile(fileChooser.getSelectedFile(), projectFile));
					jTextFieldMacros.setBorder(TEXTFIELD_BORDER);
					jTextFieldMacros.setToolTipText(null);
					fieldsHaveBeenChanged = true;
				// jTextFieldMacros.requestFocus();
				}
			}
		};
	}

	// for testing purposes only
	/*public static void main(String args[]) throws IOException, SAXException { // todo: remove main
		ProjectPropertiesDialog projectPropertiesDialog = new ProjectPropertiesDialog(new JFrame());
		projectPropertiesDialog.setDELTAProject(Project.load(new File(FileUtils.currentDirFile,args[0])));
		projectPropertiesDialog.setVisible(true);
	}*/

	protected Component createMainComponent() {  // todo: list for macros file(s) ???
		JPanel generalPanel = new JPanel(new TableLayout(
				new double[][]{{80.0, TableLayout.FILL, TableLayout.PREFERRED},
							   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED}}, 2, 5));
		generalPanel.setBorder(new CompoundBorder(new TitledBorder(" General "), new EmptyBorder(0, 2, 2, 2)));

		JLabel label = new JLabel("Name: ");
		jTextFieldName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_N);
		label.setLabelFor(jTextFieldName);
		generalPanel.add(label, new TableLayoutConstraints());
		generalPanel.add(jTextFieldName, new TableLayoutConstraints(1, 0, 2, 0));


		label = new JLabel("Project-File: ");
		jTextFieldProjectFile = new JTextField();
		jTextFieldProjectFile.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_P);
		label.setLabelFor(jTextFieldProjectFile);
		generalPanel.add(label, new TableLayoutConstraints(0, 1));
		generalPanel.add(jTextFieldProjectFile, new TableLayoutConstraints(1, 1));
		generalPanel.add(new SmallButton(changeProjectFile), new TableLayoutConstraints(2, 1));

		label = new JLabel("Description: ");
		jTextFieldDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldDescription);
		generalPanel.add(label, new TableLayoutConstraints(0, 2));
		generalPanel.add(jTextFieldDescription, new TableLayoutConstraints(1, 2));
		generalPanel.add(new SmallButton(editDescription), new TableLayoutConstraints(2, 2));

		JPanel filesPanel = new JPanel(new TableLayout(
				new double[][]{{80.0, TableLayout.FILL, TableLayout.PREFERRED},
							   {TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL}}, 2,5));
		filesPanel.setBorder(new CompoundBorder(new TitledBorder(" Files "), new EmptyBorder(0, 2, 0, 2)));

		label = new JLabel("Left: ");
		jTextFieldLeft = new JTextField();
		TEXTFIELD_BORDER = jTextFieldLeft.getBorder();
		jTextFieldLeft.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldLeft);
		filesPanel.add(label, new TableLayoutConstraints());
		filesPanel.add(jTextFieldLeft, new TableLayoutConstraints(1, 0));
		filesPanel.add(new SmallButton(changeLeftAction), new TableLayoutConstraints(2, 0));

		label = new JLabel("Right: ");
		jTextFieldRight = new JTextField();
		jTextFieldRight.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_R);
		label.setLabelFor(jTextFieldRight);
		filesPanel.add(label, new TableLayoutConstraints(0, 1));
		filesPanel.add(jTextFieldRight, new TableLayoutConstraints(1, 1));
		filesPanel.add(new SmallButton(changeRightAction), new TableLayoutConstraints(2, 1));

		label = new JLabel("Macros: ");
		jTextFieldMacros = new JTextField();
		jTextFieldMacros.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_M);
		label.setLabelFor(jTextFieldMacros);
		filesPanel.add(label, new TableLayoutConstraints(0, 2));
		filesPanel.add(jTextFieldMacros, new TableLayoutConstraints(1, 2));
		filesPanel.add(new SmallButton(changeMacrosFileAction), new TableLayoutConstraints(2, 2));

		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(generalPanel, BorderLayout.NORTH);
		mainPanel.add(filesPanel, BorderLayout.CENTER);

		jTextFieldName.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				fieldsHaveBeenChanged = true;
				String name = jTextFieldName.getText();
				String fileName = name.replaceAll("[^\\p{Alnum}-_]", "") + ".project";
				if (!fileName.equals("")) {
					projectFile = new File(projectFile.getParent(), fileName);
					jTextFieldProjectFile.setText(projectFile.getAbsolutePath());
				}
			}
		});
		jTextFieldDescription.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				fieldsHaveBeenChanged = true;
			}
		});

		return mainPanel;
	}

	private JTextField jTextFieldName, jTextFieldDescription, jTextFieldLeft, jTextFieldRight, jTextFieldMacros, jTextFieldProjectFile;
}
