package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.images.LazyResourceLoader;

import javax.swing.*;
import java.awt.*;

/**
 * SplashWindow
 * @author  Peter VOTRUBA
 * @version 2.7.3
 */
public final class SplashWindow extends Window {
	public static final int WIDTH = 372;
	public static final int HEIGHT = 204;

	public static final String LINE1 = "DELT/A v2.7.3 #452 (27.04.2006)";
	public static final String LINE2 = "- Document Exploration and Linking Tool (with Addons) -";
	public static final String LINE3 = "formerly known as \"Guideline Markup Tool (GMT)\"";
	public static final String LINE4 = "Information Engineering Group, Technical University, Vienna";

	private static int xLine1 = -1, xLine2 = -1, xLine3 = -1, xLine4 = -1;

	private static SplashWindow splashWindow;
	private Image splashImage;

	/** Creates new SplashWindow */
	private SplashWindow(Frame owner) {
		super(owner);

		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	//	URL url = MiscImages.class.getResource("SplashWindow.gif");
		splashImage = LazyResourceLoader.getImage(LazyResourceLoader.APP_LOGO); //Toolkit.getDefaultToolkit().getImage(url);
		MediaTracker mt = new MediaTracker(this);
		mt.addImage(splashImage, 0);
		try {
			mt.waitForID(0);
		} catch (InterruptedException ie) { }
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds((d.width - WIDTH) >> 1, (d.height - HEIGHT) >> 1, WIDTH, HEIGHT);
	}

	public void paint(Graphics g) {
		g.drawImage(splashImage, 0, 0, this);

		((Graphics2D) g).setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		Font defaultFont = g.getFont();
		if (xLine1 == -1)
			xLine1 = (int)((WIDTH - g.getFontMetrics().getStringBounds(LINE1, g).getWidth()) / 2);
		g.drawString(LINE1, xLine1, 18); // x=101 on Windows
		
		g.setFont(defaultFont.deriveFont(Font.BOLD));
		if (xLine2 == -1)
			xLine2 = (int)((WIDTH - g.getFontMetrics().getStringBounds(LINE2, g).getWidth()) / 2);
		g.drawString(LINE2, xLine2, 162); // x=28 on Windows

		g.setFont(defaultFont.deriveFont(Font.ITALIC));
		if (xLine3 == -1)
			xLine3 = (int)((WIDTH - g.getFontMetrics().getStringBounds(LINE3, g).getWidth()) / 2);
		g.drawString(LINE3, xLine3, 178);	// x=50 on Windows

		g.setFont(defaultFont);
		if (xLine4 == -1)
			xLine4 = (int)((WIDTH - g.getFontMetrics().getStringBounds(LINE4, g).getWidth()) / 2);
		g.drawString(LINE4, xLine4, 198);	// x=22 on Windows
	}

	public static void showSplash() {
		Frame owner = new Frame("Document Exploration and Linking Tool (with Addons)", null);
		owner.setIconImage(LazyResourceLoader.getImageRelativeToClass(LazyResourceLoader.APP_ICON));
		splashWindow = new SplashWindow(owner);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				splashWindow.setVisible(true);
			}
		});
	}
	public static void closeSplash() {
		splashWindow.dispose();
		splashWindow.splashImage = null;
	}

/* public static void main(String[] args) {
		final SplashWindow splashWindow = new SplashWindow();
		splashWindow.setVisible(true);
		splashWindow.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent evt) {
				splashWindow.close();
				System.exit(0);
			}
		});
	}*/
 }
