/*
 * XMLLanguageDialog.java
 *
 * Created on 11. April 2003, 09:47
 */

package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.GUIConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.actions.EmptyAction;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.gui.xml.TableCellEditorWithEditButton;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Transform;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.*;
import at.ac.tuwien.asgaard.swing.*;
import at.ac.tuwien.asgaard.swing.models.*;
import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.SortedList;
import at.ac.tuwien.asgaard.util.xml.DocTypeFileFilter;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelperXML4J;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.xml.transform.TransformerException;

/**
 * @author peter
 */
public final class XMLLanguageDialog extends AbstractInputDialog {
	private Action addNewLanguage, removeLanguage, editDescription, changeFileIcon, changeOriginalDTD, changeLinkDTD, convertDTD, changeRootElement,
		addNewTemplate, removeTemplate, changeTemplateFile, editTemplateDescription, changeTemplateFileIcon,
		addNewStylesheet, removeStylesheet, changeStylesheetFile, editStylesheetDescription, changeStylesheetFileIcon,
		addNewHighlightDef, removeHighlightDef, editHighlightDefXPathExpr, editHighlightDefDescription, changeHighlightDefIcon,
		addNewSummarizeDef, removeSummarizeDef, changeXSLFile, editSummarizeDefDescription, changeSummarizeDefFileIcon,
		addNewFoldingDef, removeFoldingDef, editFoldingDefXPathExpr, editFoldingDefDescription, changeFoldingDefFileIcon,
		addNewFilterDef, removeFilterDef, editFilterDefXPathExpr, editFilterDefDescription, changeFilterDefFileIcon,
		refreshIconsListAction, addLinkedAttribute, removeLinkedAttribute;
	private JTextField jTextFieldName, jTextFieldDescription, jTextFieldExtension,
		jTextFieldFileIcon, jTextFieldFile, jTextFieldOldDTD, jTextFieldNewDTD, jTextFieldRootElement,
		jTextFieldTemplateFileName, jTextFieldTemplateLabel, jTextFieldTemplateDescription, jTextFieldTemplateIconFile,
		jTextFieldStylesheetFileName, jTextFieldStylesheetLabel, jTextFieldStylesheetDescription, jTextFieldStylesheetIconFile,
		jTextFieldHighlightDefXPathExpr, jTextFieldHighlightDefLabel, jTextFieldHighlightDefDescription, jTextFieldHighlightDefIconFile,
		jTextFieldXSLFileName, jTextFieldSummarizeDefLabel, jTextFieldSummarizeDefDescription, jTextFieldSummarizeDefIconFile,
		jTextFieldFoldingDefXPathExpr, jTextFieldFoldingDefLabel, jTextFieldFoldingDefDescription, jTextFieldFoldingDefIconFile,
		jTextFieldFilterDefXPathExpr, jTextFieldFilterDefLabel, jTextFieldFilterDefDescription, jTextFieldFilterDefIconFile;

	private JTabbedPane tabbedPaneMain, tabbedPaneViews;

	private CardPanel cardPanel;

	private SimpleListModel xmlLanguagesListModel, templatesListModel, stylesheetsListModel, highlightDefsListModel, summarizeDefsListModel, foldingDefsListModel, filterDefsListModel;
	private JList jListXMLLanguages, jListXMLTemplates, jListStylesheets, jListHighlightDefs, jListSummarizeDefs, jListFoldingDefs, jListFilterDefs;
	private SingleListSelectionModel xmlLanguagesListSelectionModel, templatesListSelectionModel, stylesheetsListSelectionModel, highlightDefsListSelectionModel, summarizeDefsListSelectionModel, foldingDefsListSelectionModel, filterDefsListSelectionModel;
	private final GUIFieldChangeListener changeListener, templateChangeListener, stylesheetChangeListener, highlightDefChangeListener, summarizeDefChangeListener, foldingDefChangeListener, filterDefChangeListener;
	private JLabel xmlFileIconLabel, templateIconLabel, stylesheetIconLabel, highlightDefIconLabel, summarizeDefIconLabel, foldingDefIconLabel, filterDefIconLabel;

	private XmlLanguage currentXMLLanguage;
	private DTDHelperXML4J dtdHelper;
	private static final String NEW_LANGUAGE_BASE_NAME = "NewLanguage";
// private JCheckBox autoApplyCheckBox;

	private TextEditDialog editDialog;
	private XPathExpressionEditDialog editDialogXPath;

	private JFileChooser sharedFileChooser;
	public static final	Icon DEFAULT_LANGUAGE_ICON	 = MainImages.XML_LANGUAGE_ICON,
							 DEFAULT_TEMPLATE_ICON	 = MainImages.TEMPLATE_ICON,
							 DEFAULT_HIGHLIGHT_ICON	 = MainImages.HIGHLIGHT_ICON,
							 DEFAULT_FILTER_ICON	 = MainImages.FILTER_ICON,
							 DEFAULT_FOLDING_ICON	 = MainImages.TREE_VIEW_ICON,
							 DEFAULT_SUMMARIZE_ICON	 = MainImages.SUMMARIZE_ICON,
							 DEFAULT_STYLESHEET_ICON = MiscImages.XSL_FILE_ICON;

	public static final	int	TAB_INDEX_GENERAL	  =	0;
	public static final	int	TAB_INDEX_TEMPLATES	  =	1;
	public static final	int	TAB_INDEX_APPLICABLES =	2;
	public static final	int	TAB_INDEX_LINKEDATTRS =	3;
	public static final	int	TAB_INDEX_ICONS		  =	4;

	public static final	int	APPLICABLES_TAB_INDEX_XPATH		= 0;
	public static final	int	APPLICABLES_TAB_INDEX_FOLDING	= 1;
	public static final	int	APPLICABLES_TAB_INDEX_FILTER	= 2;
	public static final	int	APPLICABLES_TAB_INDEX_SUMMARIZE	= 3;
	public static final	int	APPLICABLES_TAB_INDEX_TRANSFORM	= 4;

	private RadioButtonsPanel filterTypeRadioButtons, foldingTypeRadioButtons;
	private JList jListIconSets, jListIcons;
	public static final String LINK_ENRICHED_DTD_SUFFIX = "_DELTA.dtd";
	private CombinedURLFileChooserDialog combinedURLFileChooserDialog;
	private JComboBox jComboboxStylesheetOutputType, jComboboxStylesheetOutputIn;

	private JTable linkAttrsTable;
	private SimpleTableModel linkAttrsTableModel;

	/**
	 * Creates new form XMLLanguageDialog
	 */
	public XMLLanguageDialog(Frame parent, List xmlLanguages) {
		super(parent, "XML Language Properties", false, true, 800, 370, 701, 350, 0, 0);

		changeListener			   = new GUIFieldChangeListener();
		templateChangeListener	   = new GUIFieldChangeListener();
		stylesheetChangeListener   = new GUIFieldChangeListener();
		highlightDefChangeListener = new GUIFieldChangeListener();
		summarizeDefChangeListener = new GUIFieldChangeListener();
		foldingDefChangeListener   = new GUIFieldChangeListener();
		filterDefChangeListener	   = new GUIFieldChangeListener();
		installListeners();

		xmlLanguagesListModel.init(xmlLanguages);
		if (!xmlLanguages.isEmpty()) {
			cardPanel.switchToCard("properties");
			xmlLanguagesListSelectionModel.setSelectionIndex(0);
		}
	}

	protected Component createMainComponent() {
		Box generalTab = Box.createVerticalBox();
		generalTab.add(createGeneralPanel());
		generalTab.add(createXMLPanel());

		tabbedPaneMain = new JTabbedPane();
		tabbedPaneMain.insertTab("General", MiscImages.LANG_FILE_ICON, generalTab, null, TAB_INDEX_GENERAL);
		tabbedPaneMain.insertTab("Templates", MainImages.TEMPLATE_ICON, createTemplatesPanel(), null, TAB_INDEX_TEMPLATES);
		tabbedPaneMain.insertTab("Applicable", MainImages.VIEW_ICON, createApplicablePanel(), "stuff that can be applied to the current XML document", TAB_INDEX_APPLICABLES);
		tabbedPaneMain.insertTab("Linked Attributes", MainImages.LINK_ICON, createLinkedAttributesPanel(), null, TAB_INDEX_LINKEDATTRS);
		tabbedPaneMain.insertTab("Icons", MiscImages.ICON_FILE_ICON, createIconsPanel(), null, TAB_INDEX_ICONS);

		cardPanel = new CardPanel();
		cardPanel.add(new JLabel(" <-- Create a new XML Language Definition ...", (Icon) addNewLanguage.getValue(Action.SMALL_ICON),
								 SwingConstants.CENTER), "empty");
		cardPanel.add(tabbedPaneMain, "properties");

		JPanel mainPanel = new JPanel(new TableLayout(new double[][]{{0.2, 0.8}, {28.0, TableLayout.FILL}}, 5, 5));
		mainPanel.add(new JLabel("XML Languages:"), new TableLayoutConstraints()); // 150.0, TableLayout.FILL
		mainPanel.add(createLanguagesPanel(), new TableLayoutConstraints(0, 1));
		mainPanel.add(cardPanel, new TableLayoutConstraints(1, 0, 1, 1));
		return mainPanel;
	}

	public int createNewXMLLanguage() {
		XmlLanguage previousXMLLanguage = currentXMLLanguage;
		addNewLanguage.actionPerformed(null);
		if (previousXMLLanguage == currentXMLLanguage) return -1;
		setVisible(true);
		return (approved ? xmlLanguagesListSelectionModel.getCurrentIndex() : -1);
	}

	public boolean showDialog(XmlLanguage xmlLanguage) {
		int langIndex = xmlLanguagesListModel.indexOf(xmlLanguage);
		xmlLanguagesListSelectionModel.setSelectionIndex(langIndex);
		return showDialog();
	}

	private javax.swing.filechooser.FileFilter lastFileFilter;

	private File askForFile(File dir, javax.swing.filechooser.FileFilter fileFilter,
							javax.swing.filechooser.FileFilter additionalFileFilter) {
		if (dir == null) dir = currentXMLLanguage.getFile().getParentFile();
		if (sharedFileChooser == null)
			sharedFileChooser = new DeltaFileChooser(dir);
		else
			sharedFileChooser.setCurrentDirectory(dir);

		if (fileFilter != lastFileFilter) {
			if (lastFileFilter != null)
				sharedFileChooser.resetChoosableFileFilters();
			if (additionalFileFilter != null)
				sharedFileChooser.addChoosableFileFilter(additionalFileFilter);
			sharedFileChooser.setFileFilter(fileFilter);
			lastFileFilter = fileFilter;
		}

		if (sharedFileChooser.showOpenDialog(XMLLanguageDialog.this) == JFileChooser.APPROVE_OPTION)
			return sharedFileChooser.getSelectedFile();
		return null;
	}

	private void installListeners() {
		xmlLanguagesListSelectionModel.addVetoableChangeListener(new VetoableChangeListener() {
			public void vetoableChange(PropertyChangeEvent evt) throws PropertyVetoException {
				if (!checkForChanges())
					throw new PropertyVetoException("Refuse index change", evt);
			}
		});
		xmlLanguagesListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				// jComboBoxRootElement.setEnabled(jComboBoxRootElement.getModel().getSize() > 0);
				convertDTD.setEnabled(false);
				// tabbedPaneMain.setSelectedIndex(0); // jump to first tab?
				int index = xmlLanguagesListSelectionModel.getCurrentIndex();
				if ((index != -1) && (index != xmlLanguagesListSelectionModel.getPreviousIndex())) {
					// checkForChanges();
					storeLinkedAttributes();
					currentXMLLanguage = (XmlLanguage) xmlLanguagesListModel.getElementAt(index);
					if (currentXMLLanguage.getFile() != null) {
						setGUIFields();
						convertDTD.setEnabled(!jTextFieldOldDTD.getText().trim().equals(""));
						removeLanguage.setEnabled(currentXMLLanguage.getFile() != null);
						jTextFieldName.requestFocusInWindow();
					}
				}
			}
		});

		jTextFieldName.getDocument().addDocumentListener(changeListener);
		jTextFieldDescription.getDocument().addDocumentListener(changeListener);
		jTextFieldExtension.getDocument().addDocumentListener(changeListener);
		jTextFieldFileIcon.getDocument().addDocumentListener(changeListener);
		jTextFieldOldDTD.getDocument().addDocumentListener(changeListener);
		jTextFieldNewDTD.getDocument().addDocumentListener(changeListener);

		jTextFieldOldDTD.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				if (!(evt.getOppositeComponent() instanceof JTextField))
					return;
				String oldFilePath = jTextFieldOldDTD.getText();
				String oldFilePathRelative = FileUtils.toRelativeFile(oldFilePath.trim(), currentXMLLanguage.getFile());
				if ((oldFilePathRelative != null) && !oldFilePath.equals(oldFilePathRelative)) {
					jTextFieldOldDTD.setText(oldFilePathRelative);
					convertDTD.setEnabled(true);
					changeRootElement.setEnabled(true);
				}
				String newFilePath = jTextFieldNewDTD.getText().trim();
				if (newFilePath.equals(""))
					jTextFieldNewDTD.setText(FileUtils.getFileName(oldFilePath) + LINK_ENRICHED_DTD_SUFFIX);
			}
		});
		jTextFieldNewDTD.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				String newFilePath = jTextFieldNewDTD.getText().trim();
				String newFilePathRelative = FileUtils.toRelativeFile(newFilePath, currentXMLLanguage.getFile());
				if ((newFilePathRelative != null) && !newFilePath.equals(newFilePathRelative)) {
					jTextFieldNewDTD.setText(newFilePath);
					String oldFilePath = jTextFieldOldDTD.getText().trim();
					if (oldFilePath.equals("") && newFilePath.endsWith(LINK_ENRICHED_DTD_SUFFIX))
						jTextFieldOldDTD.setText(newFilePath.substring(0, newFilePath.length() - LINK_ENRICHED_DTD_SUFFIX.length()) + ".dtd");
				}
				changeRootElement.setEnabled(true);
			}
		});

		jTextFieldFileIcon.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				URL url = FileUtils.toURL(jTextFieldFileIcon.getText().trim(), currentXMLLanguage.getFile());
				Icon icon = null;
				if (url != null) {
					icon = new ImageIcon(url);
					if (icon.getIconWidth() < 10) icon = null;
				}
				if (icon == null) {
					jTextFieldFileIcon.setText(null);
					icon = DEFAULT_LANGUAGE_ICON;
				}
				xmlFileIconLabel.setIcon(icon);
			}
		});

		jTextFieldName.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				String old = jTextFieldName.getText();
				String name = old.trim();
				if (name != old) jTextFieldName.setText(name);
				File dir = new File(MainController.XML_LANGUAGES_BASE_DIR, name);
				File file = new File(dir, name + ".lang");
				if (!file.exists())
					try {
						dir.mkdir();
						file.createNewFile();
						currentXMLLanguage.setFile(file);
						jTextFieldFile.setText(file.getPath());
					} catch (Exception e) {
						JOptionPane.showMessageDialog(XMLLanguageDialog.this, "Couldn't create \"" + file.getAbsolutePath() + "\"", "I/O Error",
													  JOptionPane.ERROR_MESSAGE);
						jTextFieldName.selectAll();
						jTextFieldName.requestFocusInWindow();
					}
			}
		});
		jTextFieldExtension.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				String ext = jTextFieldExtension.getText().trim();
				if (ext.equals("") || ext.equals("."))
					ext = ".xml";
				else if (ext.charAt(0) != '.') ext = "." + ext;
				jTextFieldExtension.setText(ext);
			}
		});
		jListIconSets.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = jListIconSets.getMinSelectionIndex();
				if (index == -1)
					jListIcons.setModel(EmptyListModel.sharedInstance);
				else {
					File dir = (File) jListIconSets.getSelectedValue();
					File[] files = dir.listFiles(new FileFilter() {
						public boolean accept(File file) {
							return file.getPath().endsWith(".gif");
						}
					});
					if (files != null) {
						Arrays.sort(files);
						jListIcons.setListData(files);
					} else
						jListIcons.setModel(EmptyListModel.sharedInstance);
				}
			}
		});

		templatesListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = templatesListSelectionModel.getCurrentIndex();
				int oldIndex = templatesListSelectionModel.getPreviousIndex();
				if (templateChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeTemplateProperties((Template) templatesListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					Template template = (Template) templatesListModel.getElementAt(index);
					jTextFieldTemplateFileName.setText(template.getFileName());
					jTextFieldTemplateLabel.setText(template.getLabel());
					jTextFieldTemplateLabel.setCaretPosition(0);
					jTextFieldTemplateDescription.setText(template.getDescription());
					jTextFieldTemplateDescription.setCaretPosition(0);
					jTextFieldTemplateIconFile.setText(template.getIconFileName());
					Icon icon = template.getIcon();
					if (icon == null) {
						icon = DEFAULT_TEMPLATE_ICON;
						template.setIcon(icon);
					}
					templateIconLabel.setIcon(icon);
					jListXMLTemplates.repaint();
					removeTemplate.setEnabled(true);
					templateChangeListener.reset();
				} else
					removeTemplate.setEnabled(false);
			}
		});
		jTextFieldTemplateFileName.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				String templateFilePath = jTextFieldTemplateFileName.getText().trim();
				addNewTemplate.setEnabled(FileUtils.toExistingAbsoluteFile(templateFilePath, currentXMLLanguage.getFile()) != null);
			}
		});
		jTextFieldTemplateFileName.getDocument().addDocumentListener(templateChangeListener);
		jTextFieldTemplateLabel.getDocument().addDocumentListener(templateChangeListener);
		jTextFieldTemplateDescription.getDocument().addDocumentListener(templateChangeListener);
		jTextFieldTemplateIconFile.getDocument().addDocumentListener(templateChangeListener);

		stylesheetsListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = stylesheetsListSelectionModel.getCurrentIndex();
				int oldIndex = stylesheetsListSelectionModel.getPreviousIndex();
				if (stylesheetChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeStylesheetProperties((Stylesheet) stylesheetsListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					Stylesheet stylesheet = (Stylesheet) stylesheetsListModel.getElementAt(index);
					jTextFieldStylesheetFileName.setText(stylesheet.getFileName());
					jComboboxStylesheetOutputType.setSelectedItem(stylesheet.getOutputType());
					jComboboxStylesheetOutputIn.setSelectedItem(stylesheet.getOutputIn());
					jTextFieldStylesheetLabel.setText(stylesheet.getLabel());
					jTextFieldStylesheetLabel.setCaretPosition(0);
					jTextFieldStylesheetDescription.setText(stylesheet.getDescription());
					jTextFieldStylesheetDescription.setCaretPosition(0);
					jTextFieldStylesheetIconFile.setText(stylesheet.getIconFileName());
					Icon icon = stylesheet.getIcon();
					if (icon == null) {
						icon = DEFAULT_STYLESHEET_ICON;
						stylesheet.setIcon(icon);
					}
					stylesheetIconLabel.setIcon(icon);
					jListStylesheets.repaint();
					removeStylesheet.setEnabled(true);
					stylesheetChangeListener.reset();
				} else
					removeStylesheet.setEnabled(false);
			}
		});
		jTextFieldStylesheetFileName.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				String stylesheetFilePath = jTextFieldStylesheetFileName.getText().trim();
				addNewStylesheet.setEnabled(FileUtils.toExistingAbsoluteFile(stylesheetFilePath, currentXMLLanguage.getFile()) != null);
			}
		});
		jTextFieldStylesheetFileName.getDocument().addDocumentListener(stylesheetChangeListener);
		jTextFieldStylesheetLabel.getDocument().addDocumentListener(stylesheetChangeListener);
		jTextFieldStylesheetDescription.getDocument().addDocumentListener(stylesheetChangeListener);
		jTextFieldStylesheetIconFile.getDocument().addDocumentListener(stylesheetChangeListener);
		jComboboxStylesheetOutputType.addItemListener(stylesheetChangeListener);
		jComboboxStylesheetOutputIn.addItemListener(stylesheetChangeListener);

		foldingDefsListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = foldingDefsListSelectionModel.getCurrentIndex();
				int oldIndex = foldingDefsListSelectionModel.getPreviousIndex();
				if (foldingDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeFoldingDefProperties((FoldingDef) foldingDefsListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					FoldingDef foldingDef = (FoldingDef) foldingDefsListModel.getElementAt(index);
					jTextFieldFoldingDefXPathExpr.setText(foldingDef.getXPathExpr());
					jTextFieldFoldingDefXPathExpr.setCaretPosition(0);
					foldingTypeRadioButtons.setSelected(foldingDef.getType().toLowerCase().equals("expand") ? 0 : 1);
					jTextFieldFoldingDefLabel.setText(foldingDef.getLabel());
					jTextFieldFoldingDefLabel.setCaretPosition(0);
					jTextFieldFoldingDefDescription.setText(foldingDef.getDescription());
					jTextFieldFoldingDefDescription.setCaretPosition(0);
					jTextFieldFoldingDefIconFile.setText(foldingDef.getIconFileName());
					Icon icon = foldingDef.getIcon();
					if (icon == null) {
						icon = DEFAULT_FOLDING_ICON;
						foldingDef.setIcon(icon);
					}
					foldingDefIconLabel.setIcon(icon);
					jListFoldingDefs.repaint();
					removeFoldingDef.setEnabled(true);
					foldingDefChangeListener.reset();
				} else
					removeFoldingDef.setEnabled(false);
			}
		});
		jTextFieldFoldingDefXPathExpr.getDocument().addDocumentListener(foldingDefChangeListener);
		foldingTypeRadioButtons.addChangeListener(foldingDefChangeListener);
		jTextFieldFoldingDefLabel.getDocument().addDocumentListener(foldingDefChangeListener);
		jTextFieldFoldingDefDescription.getDocument().addDocumentListener(foldingDefChangeListener);
		jTextFieldFoldingDefIconFile.getDocument().addDocumentListener(foldingDefChangeListener);

		highlightDefsListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = highlightDefsListSelectionModel.getCurrentIndex();
				int oldIndex = highlightDefsListSelectionModel.getPreviousIndex();
				if (highlightDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeHighlightDefProperties((HighlightDef) highlightDefsListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					HighlightDef highlightDef = (HighlightDef) highlightDefsListModel.getElementAt(index);
					jTextFieldHighlightDefXPathExpr.setText(highlightDef.getXPathExpr());
					jTextFieldHighlightDefXPathExpr.setCaretPosition(0);
					jTextFieldHighlightDefLabel.setText(highlightDef.getLabel());
					jTextFieldHighlightDefLabel.setCaretPosition(0);
					jTextFieldHighlightDefDescription.setText(highlightDef.getDescription());
					jTextFieldHighlightDefDescription.setCaretPosition(0);
					jTextFieldHighlightDefIconFile.setText(highlightDef.getIconFileName());
					Icon icon = highlightDef.getIcon();
					if (icon == null) {
						icon = DEFAULT_HIGHLIGHT_ICON;
						highlightDef.setIcon(icon);
					}
					highlightDefIconLabel.setIcon(icon);
					jListHighlightDefs.repaint();
					removeHighlightDef.setEnabled(true);
					highlightDefChangeListener.reset();
				} else
					removeHighlightDef.setEnabled(false);
			}
		});
		jTextFieldHighlightDefXPathExpr.getDocument().addDocumentListener(highlightDefChangeListener);
		jTextFieldHighlightDefLabel.getDocument().addDocumentListener(highlightDefChangeListener);
		jTextFieldHighlightDefDescription.getDocument().addDocumentListener(highlightDefChangeListener);
		jTextFieldHighlightDefIconFile.getDocument().addDocumentListener(highlightDefChangeListener);

		summarizeDefsListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = summarizeDefsListSelectionModel.getCurrentIndex();
				int oldIndex = summarizeDefsListSelectionModel.getPreviousIndex();
				if (summarizeDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeSummarizeDefProperties((SummarizeDef) summarizeDefsListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					SummarizeDef summarizeDef = (SummarizeDef) summarizeDefsListModel.getElementAt(index);
					jTextFieldXSLFileName.setText(summarizeDef.getXslFileName());
					jTextFieldXSLFileName.setCaretPosition(0);
					jTextFieldSummarizeDefLabel.setText(summarizeDef.getLabel());
					jTextFieldSummarizeDefLabel.setCaretPosition(0);
					jTextFieldSummarizeDefDescription.setText(summarizeDef.getDescription());
					jTextFieldSummarizeDefDescription.setCaretPosition(0);
					jTextFieldSummarizeDefIconFile.setText(summarizeDef.getIconFileName());
					Icon icon = summarizeDef.getIcon();
					if (icon == null) {
						icon = DEFAULT_SUMMARIZE_ICON;
						summarizeDef.setIcon(icon);
					}
					summarizeDefIconLabel.setIcon(icon);
					jListSummarizeDefs.repaint();
					removeSummarizeDef.setEnabled(true);
					summarizeDefChangeListener.reset();
				} else
					removeSummarizeDef.setEnabled(false);
			}
		});
		jTextFieldXSLFileName.getDocument().addDocumentListener(summarizeDefChangeListener);
		jTextFieldSummarizeDefLabel.getDocument().addDocumentListener(summarizeDefChangeListener);
		jTextFieldSummarizeDefDescription.getDocument().addDocumentListener(summarizeDefChangeListener);
		jTextFieldSummarizeDefIconFile.getDocument().addDocumentListener(summarizeDefChangeListener);

		filterDefsListSelectionModel.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				int index = filterDefsListSelectionModel.getCurrentIndex();
				int oldIndex = filterDefsListSelectionModel.getPreviousIndex();
				if (filterDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
					storeFilterDefProperties((FilterDef) filterDefsListModel.getElementAt(oldIndex));
				if ((index != -1)) { // && (index != oldIndex)) {
					FilterDef filterDef = (FilterDef) filterDefsListModel.getElementAt(index);
					jTextFieldFilterDefXPathExpr.setText(filterDef.getXPathExpr());
					jTextFieldFilterDefXPathExpr.setCaretPosition(0);
					filterTypeRadioButtons.setSelected(filterDef.getType().toLowerCase().equals("show") ? 0 : 1);
					jTextFieldFilterDefLabel.setText(filterDef.getLabel());
					jTextFieldFilterDefLabel.setCaretPosition(0);
					jTextFieldFilterDefDescription.setText(filterDef.getDescription());
					jTextFieldFilterDefDescription.setCaretPosition(0);
					jTextFieldFilterDefIconFile.setText(filterDef.getIconFileName());
					Icon icon = filterDef.getIcon();
					if (icon == null) {
						icon = DEFAULT_FILTER_ICON;
						filterDef.setIcon(icon);
					}
					filterDefIconLabel.setIcon(icon);
					jListFilterDefs.repaint();
					removeFilterDef.setEnabled(true);
					filterDefChangeListener.reset();
				} else
					removeFilterDef.setEnabled(false);
			}
		});
		jTextFieldFilterDefXPathExpr.getDocument().addDocumentListener(filterDefChangeListener);
		filterTypeRadioButtons.addChangeListener(filterDefChangeListener);
		jTextFieldFilterDefLabel.getDocument().addDocumentListener(filterDefChangeListener);
		jTextFieldFilterDefDescription.getDocument().addDocumentListener(filterDefChangeListener);
		jTextFieldFilterDefIconFile.getDocument().addDocumentListener(filterDefChangeListener);

		jTextFieldFilterDefXPathExpr.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				addNewFilterDef.setEnabled(
						checkXPathExprTextField(jTextFieldFilterDefXPathExpr) && (jTextFieldFilterDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldFoldingDefXPathExpr.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				addNewFoldingDef.setEnabled(
						checkXPathExprTextField(jTextFieldFoldingDefXPathExpr) && (jTextFieldFoldingDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldHighlightDefXPathExpr.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent evt) {
				addNewHighlightDef.setEnabled(
						checkXPathExprTextField(jTextFieldHighlightDefXPathExpr) && (jTextFieldHighlightDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldFilterDefLabel.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				addNewFilterDef.setEnabled(
						checkXPathExprTextField(jTextFieldFilterDefXPathExpr) && (jTextFieldFilterDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldFoldingDefLabel.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				addNewFoldingDef.setEnabled(
						checkXPathExprTextField(jTextFieldFoldingDefXPathExpr) && (jTextFieldFoldingDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldHighlightDefLabel.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				addNewHighlightDef.setEnabled(
						checkXPathExprTextField(jTextFieldHighlightDefXPathExpr) && (jTextFieldHighlightDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldXSLFileName.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				addNewSummarizeDef.setEnabled((FileUtils.toExistingAbsoluteFile(jTextFieldXSLFileName.getText().trim(),
																				currentXMLLanguage.getFile()) !=
												null)
											   && (jTextFieldSummarizeDefLabel.getText().trim().length() > 0));
			}
		});
		jTextFieldSummarizeDefLabel.getDocument().addDocumentListener(new DocumentChangeAdapter() {
			public void documentChange(DocumentEvent evt) {
				addNewSummarizeDef.setEnabled((FileUtils.toExistingAbsoluteFile(jTextFieldXSLFileName.getText().trim(),
																				currentXMLLanguage.getFile()) !=
												null)
											   && (jTextFieldSummarizeDefLabel.getText().trim().length() > 0));
			}
		});

		linkAttrsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				removeLinkedAttribute.setEnabled(linkAttrsTable.getSelectedRow() >= 0);
			}
		});
	}

	private boolean checkXPathExprTextField(JTextField textField) {
		String xpathExpr = textField.getText().trim();
		if (!xpathExpr.equals(""))
			try {
				XMLUtils.checkXPathExpression(xpathExpr);
				return true;
			} catch (Exception exc) {
				JOptionPane.showMessageDialog(XMLLanguageDialog.this, exc.getMessage(),
											  "Bad XPath-Expression!", JOptionPane.ERROR_MESSAGE);
				textField.selectAll();
				textField.requestFocusInWindow();
			}
		return false;
	}

	private void storeTemplateProperties(Template template) {
		templateChangeListener.reset();
		String fileName = jTextFieldTemplateFileName.getText().trim();
		template.setFileName(FileUtils.toRelativeFile(fileName, currentXMLLanguage.getFile()));
		String label = jTextFieldTemplateLabel.getText().trim();
		if (label.equals("")) label = FileUtils.getFileName(fileName);
		template.setLabel(label);
		template.setDescription(jTextFieldTemplateDescription.getText().trim());
		String s = jTextFieldTemplateIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(template.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_TEMPLATE_ICON;
			s = "";
		}
		template.setIconFileName(s);
		template.setIcon(icon);
		jListXMLTemplates.repaint();
	}

	private void clearTemplateGUIFields() {
		templatesListSelectionModel.clearSelection();
		jTextFieldTemplateFileName.setText("");
		jTextFieldTemplateLabel.setText("");
		jTextFieldTemplateDescription.setText("");
		jTextFieldTemplateIconFile.setText("");
		templateIconLabel.setIcon(null);
		removeTemplate.setEnabled(false);
		templateChangeListener.reset();
	}

	private void storeStylesheetProperties(Stylesheet stylesheet) {
		stylesheetChangeListener.reset();
		String fileName = jTextFieldStylesheetFileName.getText().trim();
		stylesheet.setFileName(FileUtils.toRelativeFile(fileName, currentXMLLanguage.getFile()));
		stylesheet.setOutputType(jComboboxStylesheetOutputType.getSelectedItem().toString());
		stylesheet.setOutputIn(jComboboxStylesheetOutputIn.getSelectedItem().toString());
		String label = jTextFieldStylesheetLabel.getText().trim();
		if (label.equals("")) label = FileUtils.getFileName(fileName);
		stylesheet.setLabel(label);
		stylesheet.setDescription(jTextFieldStylesheetDescription.getText().trim());

		String s = jTextFieldStylesheetIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(stylesheet.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 1) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_STYLESHEET_ICON;
			s = "";
		}
		stylesheet.setIconFileName(s);
		stylesheet.setIcon(icon);
		jListStylesheets.repaint();
	}

	private void clearStylesheetGUIFields() {
		stylesheetsListSelectionModel.clearSelection();
		jTextFieldStylesheetFileName.setText("");
		jTextFieldStylesheetLabel.setText("");
		jTextFieldStylesheetDescription.setText("");
		jTextFieldStylesheetIconFile.setText("");
		stylesheetIconLabel.setIcon(null);
		removeStylesheet.setEnabled(false);
		stylesheetChangeListener.reset();
	}


	private boolean checkForChanges() {
		if (changeListener.fieldsHaveChanged()) {
			changeListener.reset();
			if (validateFields())
				setCurrentXMLLanguageFields();
			else
				return false;
		}
		return true;
	}

	protected boolean onOK() {
		if (!validateFields())
			return false;
		if (currentXMLLanguage != null) {
			removeLanguage.setEnabled(true);
			setCurrentXMLLanguageFields();
		}
		return true;
	}

	protected boolean onCancel() {
		if (changeListener.fieldsHaveChanged())
			if (JOptionPane.showConfirmDialog(this, "Changes of XML language \"" + currentXMLLanguage.getName() +
													"\" will be lost. Really cancel?",
											  "XML language changed ...", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE)
				!= JOptionPane.YES_OPTION)
				return false;
		if (currentXMLLanguage != null)
			setGUIFields(); // reset Form-Fields to the old values
		return true;
	}

	private boolean validateFields() {
		if (currentXMLLanguage == null) return true; // todo
		boolean ok = true;
		String msg = "error"; // todo
		if (jTextFieldOldDTD.getText().trim().equals("")) {
			ok = false;
			msg = "Enter original DTD!";
		} else if (jTextFieldNewDTD.getText().trim().equals("")) {
			ok = false;
			msg = "Enter link-enriched DTD!";
		} else if (jTextFieldRootElement.getText().trim().equals("")) {
			ok = false;
			msg = "Choose root element!";
		}
		if (FileUtils.toExistingURL(jTextFieldNewDTD.getText().trim(), currentXMLLanguage.getFile()) == null) {
			ok = false;
			msg = "Convert original DTD to link-enriched DTD!";
		}
		if (!ok)
			JOptionPane.showMessageDialog(this, "\"" + msg + "\"\nTry again ...", "Input Error", JOptionPane.ERROR_MESSAGE);
		return ok;
	}

	private void setCurrentXMLLanguageFields() {
		currentXMLLanguage.setName(jTextFieldName.getText().trim());
		String s = jTextFieldExtension.getText().trim();
		if (s.charAt(0) != '.') s = "." + s;
		currentXMLLanguage.setExt(s);
		s = jTextFieldDescription.getText().trim();
		currentXMLLanguage.setDescription(s);

		s = jTextFieldFileIcon.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(currentXMLLanguage.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10)
					icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_LANGUAGE_ICON;
			s = "";
		}
		currentXMLLanguage.setIconFileName(s);
		currentXMLLanguage.setIcon(icon);
		xmlFileIconLabel.setIcon(icon);

		Dtd dtd = currentXMLLanguage.getDtd();
		if (dtd == null) {
			dtd = new Dtd();
			currentXMLLanguage.setDtd(dtd);
		}
		dtd.setOldFileName(jTextFieldOldDTD.getText().trim());
		dtd.setNewFileName(jTextFieldNewDTD.getText().trim());
		dtd.setRootElement(jTextFieldRootElement.getText());

		currentXMLLanguage.setFileFilter(new DocTypeFileFilter(currentXMLLanguage));

		int index = templatesListSelectionModel.getCurrentIndex();
		if (index != -1) storeTemplateProperties((Template) templatesListModel.getElementAt(index));
		index = stylesheetsListSelectionModel.getCurrentIndex();
		if (index != -1) storeStylesheetProperties((Stylesheet) stylesheetsListModel.getElementAt(index));
		index = foldingDefsListSelectionModel.getCurrentIndex();
		if (index != -1)
			storeFoldingDefProperties((FoldingDef) foldingDefsListModel.getElementAt(index));
		index = highlightDefsListSelectionModel.getCurrentIndex();
		if (index != -1)
			storeHighlightDefProperties((HighlightDef) highlightDefsListModel.getElementAt(index));
		index = summarizeDefsListSelectionModel.getCurrentIndex();
		if (index != -1)
			storeSummarizeDefProperties((SummarizeDef) summarizeDefsListModel.getElementAt(index));
		index = filterDefsListSelectionModel.getCurrentIndex();
		if (index != -1)
			storeFilterDefProperties((FilterDef) filterDefsListModel.getElementAt(index));
		index = stylesheetsListSelectionModel.getCurrentIndex();
		if (index != -1)
			storeStylesheetProperties((Stylesheet) stylesheetsListModel.getElementAt(index));

		storeLinkedAttributes();

		try {
			currentXMLLanguage.marshal(currentXMLLanguage.getFile());
		} catch (IOException exc) {
			JOptionPane.showMessageDialog(this, "Could not write to \"" + currentXMLLanguage.getFile().getAbsolutePath() + "\"!\n" + exc.getMessage(), "IO-Error",
										  JOptionPane.ERROR_MESSAGE);
		}
		jListXMLLanguages.repaint();
		changeListener.reset();
	}

	private void storeLinkedAttributes() {
		if (currentXMLLanguage == null) return;
		if (linkAttrsTable.getCellEditor() != null)
			linkAttrsTable.getCellEditor().stopCellEditing();
		int rows = linkAttrsTable.getRowCount();
		List<LinkedAttribute> linkedAttributeList = new ArrayList<LinkedAttribute>(rows);
		for (int i=0; i<rows; i++) {
			LinkedAttribute linkedAttribute = new LinkedAttribute();
			String s = linkAttrsTableModel.getValueAt(i, 0).toString().trim();
			if (s.length() == 0) continue;
			linkedAttribute.setTargetAttributeXPath(s);
			s = linkAttrsTableModel.getValueAt(i, 1).toString().trim();
			if (s.length() == 0) continue;
			linkedAttribute.setSourceXPath(s);
			linkedAttribute.setRestrictValues(((Boolean)linkAttrsTableModel.getValueAt(i, 2)).booleanValue() ? "yes" : "no");
			linkedAttributeList.add(linkedAttribute);
		}
		if (currentXMLLanguage.getLinkedAttributes() == null)
			currentXMLLanguage.setLinkedAttributes(new LinkedAttributes());
		currentXMLLanguage.getLinkedAttributes().setLinkedAttributeList(linkedAttributeList);
	}

	private void setGUIFields() {
		jTextFieldName.setText(currentXMLLanguage.getName());
		jTextFieldExtension.setText(currentXMLLanguage.getExt());
		jTextFieldDescription.setText(currentXMLLanguage.getDescription());
		xmlFileIconLabel.setIcon(currentXMLLanguage.getIcon());
		jTextFieldFileIcon.setText(currentXMLLanguage.getIconFileName());
		if (currentXMLLanguage.getFile() != null)
			jTextFieldFile.setText(currentXMLLanguage.getFile().getAbsolutePath());
		Dtd dtd = currentXMLLanguage.getDtd();
		if (dtd != null) {  // todo should'nt need this
			jTextFieldOldDTD.setText(dtd.getOldFileName());
			jTextFieldNewDTD.setText(dtd.getNewFileName());
			jTextFieldRootElement.setText(dtd.getRootElement());
			//    jComboBoxRootElement.getModel().setSelectedItem(dtd.getRootElement());
		}
		changeListener.reset();

		refreshIconsList();

		int oldIndex = templatesListSelectionModel.getCurrentIndex();
		if (templateChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeTemplateProperties((Template) templatesListModel.getElementAt(oldIndex));
		templatesListModel.init(currentXMLLanguage.getTemplates().getTemplateList());
		if (templatesListModel.isEmpty())
			clearTemplateGUIFields();
		else
			templatesListSelectionModel.setSelectionIndex(0);

		oldIndex = stylesheetsListSelectionModel.getCurrentIndex();
		if (stylesheetChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeStylesheetProperties((Stylesheet) stylesheetsListModel.getElementAt(oldIndex));
		if (currentXMLLanguage.getApplicables().getStylesheets() == null)
			currentXMLLanguage.getApplicables().setStylesheets(new Stylesheets());
		stylesheetsListModel.init(currentXMLLanguage.getApplicables().getStylesheets().getStylesheetList());
		if (stylesheetsListModel.isEmpty())
			clearStylesheetGUIFields();
		else
			stylesheetsListSelectionModel.setSelectionIndex(0);

		oldIndex = foldingDefsListSelectionModel.getCurrentIndex();
		if (foldingDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeFoldingDefProperties((FoldingDef) foldingDefsListModel.getElementAt(oldIndex));
		if (currentXMLLanguage.getApplicables().getFoldingDefs() == null)
			currentXMLLanguage.getApplicables().setFoldingDefs(new FoldingDefs());
		foldingDefsListModel.init(currentXMLLanguage.getApplicables().getFoldingDefs().getFoldingDefList());
		if (foldingDefsListModel.isEmpty())
			clearFoldingDefGUIFields();
		else
			foldingDefsListSelectionModel.setSelectionIndex(0);

		oldIndex = highlightDefsListSelectionModel.getCurrentIndex();
		if (highlightDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeHighlightDefProperties((HighlightDef) highlightDefsListModel.getElementAt(oldIndex));
		if (currentXMLLanguage.getApplicables().getHighlightDefs() == null)
			currentXMLLanguage.getApplicables().setHighlightDefs(new HighlightDefs());
		highlightDefsListModel.init(currentXMLLanguage.getApplicables().getHighlightDefs().getHighlightDefList());
		if (highlightDefsListModel.isEmpty())
			clearHighlightDefGUIFields();
		else
			highlightDefsListSelectionModel.setSelectionIndex(0);

		oldIndex = summarizeDefsListSelectionModel.getCurrentIndex();
		if (summarizeDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeSummarizeDefProperties((SummarizeDef) summarizeDefsListModel.getElementAt(oldIndex));
		if (currentXMLLanguage.getApplicables().getSummarizeDefs() == null)
			currentXMLLanguage.getApplicables().setSummarizeDefs(new SummarizeDefs());
		summarizeDefsListModel.init(currentXMLLanguage.getApplicables().getSummarizeDefs().getSummarizeDefList());
		if (summarizeDefsListModel.isEmpty())
			clearSummarizeDefGUIFields();
		else
			summarizeDefsListSelectionModel.setSelectionIndex(0);

		oldIndex = filterDefsListSelectionModel.getCurrentIndex();
		if (filterDefChangeListener.fieldsHaveChanged() && (oldIndex != -1))
			storeFilterDefProperties((FilterDef) filterDefsListModel.getElementAt(oldIndex));
		if (currentXMLLanguage.getApplicables().getFilterDefs() == null)
			currentXMLLanguage.getApplicables().setFilterDefs(new FilterDefs());
		filterDefsListModel.init(currentXMLLanguage.getApplicables().getFilterDefs().getFilterDefList());
		if (filterDefsListModel.isEmpty())
			clearFilterDefGUIFields();
		else
			filterDefsListSelectionModel.setSelectionIndex(0);

		linkAttrsTableModel.clear();
		if (currentXMLLanguage.getLinkedAttributes() != null) {
			java.util.List<LinkedAttribute> linkedAttributeList = currentXMLLanguage.getLinkedAttributes().getLinkedAttributeList();
			if (linkedAttributeList != null)
				for (LinkedAttribute linkedAttribute : linkedAttributeList)
					linkAttrsTableModel.addRow(new Object[]{linkedAttribute.getTargetAttributeXPath(), linkedAttribute.getSourceXPath(), linkedAttribute.getRestrictValues().equals("yes")});
		}
	}

	private void refreshIconsList() {
		File iconBaseDir = new File(currentXMLLanguage.getFile().getParent(), "icons");
		if (!iconBaseDir.exists() || !iconBaseDir.isDirectory()) {
		// iconBaseDir.mkdir();
			jListIconSets.setModel(EmptyListModel.sharedInstance);
		} else {
			File[] iconDirs = MainController.getIconDirs(iconBaseDir);
			if (MainController.isIconDir(iconBaseDir)) {
				List<File> list = new ArrayList<File>(Arrays.asList(iconDirs));
				list.add(0, iconBaseDir);
				jListIconSets.setListData(list.toArray());//setModel(new SimpleListModel(list));
			} else
				jListIconSets.setListData(iconDirs);
			if (jListIconSets.getModel().getSize() > 0)
				jListIconSets.setSelectedIndex(0);
		}
	}

	private void clearGUIFields() {
		jTextFieldName.setText("");

		jTextFieldExtension.setText("");
		jTextFieldDescription.setText("");
		jTextFieldFile.setText("");
		xmlFileIconLabel.setIcon(null);
		jTextFieldFileIcon.setText("");
		jTextFieldOldDTD.setText("");
		jTextFieldNewDTD.setText("");
		jTextFieldRootElement.setText("");

		clearTemplateGUIFields();
		clearStylesheetGUIFields();
		clearHighlightDefGUIFields();
		clearFoldingDefGUIFields();
		clearSummarizeDefGUIFields();
		clearFilterDefGUIFields();

		templatesListModel.clear();
		stylesheetsListModel.clear();
		highlightDefsListModel.clear();
		foldingDefsListModel.clear();
		summarizeDefsListModel.clear();
		filterDefsListModel.clear();
		linkAttrsTableModel.clear();

		jListIconSets.setModel(EmptyListModel.sharedInstance);
		changeListener.reset();
	}


	protected void createActions() {
		super.createActions();

		addNewLanguage = new SimpleAction(MainImages.NEW_ICON, "Adds a new XML Language") {
			protected void simpleActionPerformed() {
				if (!checkForChanges()) return;
				String newLangName = askForNewLangName();
				if (newLangName == null) return;
				currentXMLLanguage = XmlLanguage.newInstance();
				currentXMLLanguage.setName(newLangName);
				currentXMLLanguage.setExt(".xml");
				currentXMLLanguage.setIcon(DEFAULT_LANGUAGE_ICON);
				currentXMLLanguage.setDtd(new Dtd());
				currentXMLLanguage.setTemplates(new Templates());
				Applicables applicables = new Applicables();
				currentXMLLanguage.setApplicables(applicables);
				applicables.setFilterDefs(new FilterDefs());
				applicables.setFoldingDefs(new FoldingDefs());
				applicables.setHighlightDefs(new HighlightDefs());
				applicables.setSummarizeDefs(new SummarizeDefs());
				applicables.setStylesheets(new Stylesheets());

				cardPanel.switchToCard("properties");
				tabbedPaneMain.setSelectedIndex(0);
				tabbedPaneViews.setSelectedIndex(0);

				clearGUIFields();

				int newIndex = SortedList.getInsertIndex(xmlLanguagesListModel.getList(), currentXMLLanguage,
														 MainController.XML_LANGUAGE_COMPARATOR);
				xmlLanguagesListModel.add(newIndex, currentXMLLanguage);
				xmlLanguagesListSelectionModel.setSelectionIndex(newIndex);

				jTextFieldName.setText(newLangName);
				jTextFieldExtension.setText(".xml");
				xmlFileIconLabel.setIcon(DEFAULT_LANGUAGE_ICON);
			// convertDTD.setEnabled(false);
				changeRootElement.setEnabled(false);
				jTextFieldName.selectAll();
				jTextFieldName.requestFocusInWindow();
			}
		};
		removeLanguage = new AbstractAction("", MainImages.REMOVE_ICON) {
			{
				putValue(Action.SHORT_DESCRIPTION, "Removes selected XML Language");
				enabled = false;
			}

			public void actionPerformed(ActionEvent evt) {
				if (JOptionPane.showConfirmDialog(XMLLanguageDialog.this, "Do you really want to delete XML-Language \""
																		  + currentXMLLanguage.getName()
																		  + "\" ?", "Delete ?", JOptionPane.YES_NO_OPTION,
																					JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
					removeCurrentLanguage();
			}
		};
		editDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldDescription.getText(), "Edit Language description...");
				if (newValue != null)
					jTextFieldDescription.setText(newValue);
			}
		};
		changeFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					xmlFileIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldFileIcon.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// changeListener.fieldsChanged = true;
				}
				jTextFieldFileIcon.requestFocusInWindow();
			}
		};
		changeOriginalDTD = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				jTextFieldOldDTD.requestFocusInWindow();
				if (combinedURLFileChooserDialog == null)
					combinedURLFileChooserDialog = new CombinedURLFileChooserDialog(XMLLanguageDialog.this, DeltaFileChooser.DTD_FILE_FILTER);
				File dir = FileUtils.toExistingAbsoluteFile(currentXMLLanguage.getDtd().getOldFileName(), currentXMLLanguage.getFile());
				if (dir == null) dir = currentXMLLanguage.getFile();
				combinedURLFileChooserDialog.setCurrentDirectory(dir.getParentFile());
				if (combinedURLFileChooserDialog.showDialog()) {
					if (currentXMLLanguage.getFile() == null) {
						dir = new File(MainController.XML_LANGUAGES_BASE_DIR, currentXMLLanguage.getName());
						dir.mkdir();
						currentXMLLanguage.setFile(new File(dir, currentXMLLanguage.getName() + ".lang"));
					}
					if (combinedURLFileChooserDialog.hasFileResult()) {
						File file = combinedURLFileChooserDialog.getFile();
						String oldFilePath = FileUtils.toRelativeFile(file, currentXMLLanguage.getFile());
						if (oldFilePath != null) {
							jTextFieldOldDTD.setText(oldFilePath);
							String newFilePath = jTextFieldNewDTD.getText().trim();
							if (newFilePath.equals(""))
								jTextFieldNewDTD.setText(FileUtils.getFileName(oldFilePath) + LINK_ENRICHED_DTD_SUFFIX);
							convertDTD.setEnabled(true);
							changeRootElement.setEnabled(true);
						}
					} else { // result is an URL
						URL url = combinedURLFileChooserDialog.getURL();
						String oldFilePath = url.toExternalForm();
						jTextFieldOldDTD.setText(oldFilePath);
						String newFilePath = jTextFieldNewDTD.getText().trim();
						if (newFilePath.equals(""))
							jTextFieldNewDTD.setText(FileUtils.getFileName(url.getPath()) + LINK_ENRICHED_DTD_SUFFIX);
						convertDTD.setEnabled(true);
						changeRootElement.setEnabled(true);
					}
				}
			}
		};
		changeLinkDTD = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(FileUtils.toAbsoluteFile(currentXMLLanguage.getDtd().getNewFileName(), currentXMLLanguage.getFile()),
									   DeltaFileChooser.DTD_FILE_FILTER, null);
				if (file != null) {
					if (currentXMLLanguage.getFile() == null) {
						File dir = new File(MainController.XML_LANGUAGES_BASE_DIR, currentXMLLanguage.getName());
						dir.mkdir();
						currentXMLLanguage.setFile(new File(dir, currentXMLLanguage.getName() + ".lang"));
					}
					jTextFieldNewDTD.setText(FileUtils.toRelativeFile(file, currentXMLLanguage.getFile()));
					String oldFilePath = jTextFieldOldDTD.getText().trim();
					String newFileName = file.getName();
					if (oldFilePath.equals("") && newFileName.endsWith(LINK_ENRICHED_DTD_SUFFIX))
						jTextFieldOldDTD.setText(newFileName.substring(0, newFileName.length() - LINK_ENRICHED_DTD_SUFFIX.length()) + ".dtd");
				}
			}
		};
		convertDTD = new AbstractAction("Convert") { //, MiscImages.CONVERT_DTD_ICON) {
			{
				putValue(StatusBar.TOOL_TIP, "Convert original DTD to link-enriched DTD");
				putValue(MNEMONIC_KEY, new Integer('C'));
			}
			public void actionPerformed(ActionEvent evt) {
				final URL sourceURL = FileUtils.toExistingURL(jTextFieldOldDTD.getText(), currentXMLLanguage.getFile());
				if (sourceURL == null) {
					JOptionPane.showMessageDialog(XMLLanguageDialog.this, "Original DTD does not exist!", "Error", JOptionPane.ERROR_MESSAGE);
					return;
				}
				File targetFile;
				String newDTDFilePath = jTextFieldNewDTD.getText().trim();
				if (newDTDFilePath.equals("")) {
					targetFile = new File(currentXMLLanguage.getFile().getParentFile(), FileUtils.getFileName(sourceURL.getFile()) + LINK_ENRICHED_DTD_SUFFIX);
					jTextFieldNewDTD.setText(targetFile.getName());
				//	changeListener.fieldsChanged = true;
				} else {
					targetFile = FileUtils.toAbsoluteFile(newDTDFilePath, currentXMLLanguage.getFile());
					try {
						if (targetFile.toURI().toURL().equals(sourceURL)) {
							JOptionPane.showMessageDialog(XMLLanguageDialog.this, "Original DTD and target DTD must not be equal!", "Same File", JOptionPane.ERROR_MESSAGE);
							jTextFieldNewDTD.requestFocusInWindow();
							jTextFieldNewDTD.selectAll();
							return;
						}
					} catch (MalformedURLException e) { }  // could not happen
				}
				XMLLanguageDialog.this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				convertDTD.setEnabled(false);

				final File _targetFile = targetFile;
				new at.ac.tuwien.asgaard.swing.SwingWorker() {
					{   // so JavaWebStart also finds the XSL file
						ClassLoader loader = XMLLanguageDialog.this.getClass().getClassLoader();
						Thread.currentThread().setContextClassLoader(loader);
					}
					public Object construct() {
						try {
							Transform.convertDTD(sourceURL, _targetFile);
							return null;
						} catch (Exception exc) {
							return exc;
						}
					}
					public void finished() {
						XMLLanguageDialog.this.setCursor(Cursor.getDefaultCursor());
						convertDTD.setEnabled(true);
						Exception exc = (Exception) getValue();
						if (exc != null) {
							exc.printStackTrace();
							if (exc instanceof TransformerException)
								JOptionPane.showMessageDialog(XMLLanguageDialog.this,
															  "Original DTD could not be converted to links-enriched DTD!\n" + exc.toString(), "Transformation Error",
															  JOptionPane.ERROR_MESSAGE);
							else if (exc instanceof IOException)
								JOptionPane.showMessageDialog(XMLLanguageDialog.this,
															  "Original DTD could not be read OR links-enriched DTD could not be written!\n" + exc.toString(), "I/O Error",
															  JOptionPane.ERROR_MESSAGE);
						} else
							changeRootElement.setEnabled(true);
					}
				}.start();
			}
		};
		changeRootElement = new AbstractAction("", MiscImages.DOTS_ICON) {
			{ putValue(StatusBar.TOOL_TIP, "Change root element"); } // when mouse over
			public void actionPerformed(ActionEvent evt) {
				String dtdFilePath = jTextFieldOldDTD.getText();
				URL url = FileUtils.toExistingURL(dtdFilePath, currentXMLLanguage.getFile());
				if (url == null) {
					dtdFilePath = jTextFieldNewDTD.getText();
					url = FileUtils.toExistingURL(dtdFilePath, currentXMLLanguage.getFile());
				}
				if (url == null)
					JOptionPane.showMessageDialog(XMLLanguageDialog.this, "DTD \"" + dtdFilePath + "\" does not exist!", "Error",
												  JOptionPane.ERROR_MESSAGE);
				else {
					if (dtdHelper == null) dtdHelper = new DTDHelperXML4J();
					dtdHelper.loadDTDFile(url.toExternalForm());
					if (dtdHelper.getState() == DTDHelper.NO_DTD)
						JOptionPane.showMessageDialog(XMLLanguageDialog.this,
													  "DTD \"" + dtdFilePath + "\" could not be parsed!",
													  "Parse Error", JOptionPane.ERROR_MESSAGE);
					else {
						String rootElement = jTextFieldRootElement.getText();
						if (rootElement.equals("")) rootElement = dtdHelper.guessRootElement();
						rootElement =
						(String) JOptionPane.showInputDialog(XMLLanguageDialog.this, "Choose root element", "Root Element",
															 JOptionPane.QUESTION_MESSAGE, null, dtdHelper.getAllElements(), rootElement);
						if (rootElement != null)
							jTextFieldRootElement.setText(rootElement);
					}
				}
			}
		};
		refreshIconsListAction = new SimpleAction("Refresh", MainImages.RELOAD_ICON, 'R') {
			public void simpleActionPerformed() {
				refreshIconsList();
			}
		};

		addNewTemplate = new SimpleAction(MainImages.ADD_ICON, "Add a new XML Template", false) {
			public void simpleActionPerformed() {
				Template template = new Template();
				// currentXMLLanguage.getTemplates().addTemplate(template);
				storeTemplateProperties(template);
				templatesListModel.add(template); // templatesListModel refers to currentXMLLanguage.getTemplates().getTemplateList()
				templatesListSelectionModel.setSelectionIndex(templatesListModel.getSize() - 1);
			}
		};
		removeTemplate = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected XML Template", false) {
			public void simpleActionPerformed() {
				int index = templatesListSelectionModel.getCurrentIndex();
				templateChangeListener.reset();
				templatesListModel.remove(index);
				if (templatesListModel.isEmpty())
					clearTemplateGUIFields();
				else
					templatesListSelectionModel.setSelectionIndex(Math.min(index, templatesListModel.getSize() - 1));
			}
		};
		changeTemplateFile = new SimpleAction(MiscImages.DOTS_ICON, "Show filechooser to change template file") {
			public void simpleActionPerformed() {
				File file;
				if ((currentXMLLanguage.getDtd() != null)
					&& (currentXMLLanguage.getDtd().getNewFileName() != null))
					file = askForFile(null, currentXMLLanguage.getFileFilter(), null);
				else
					file = askForFile(null, DeltaFileChooser.XML_FILE_FILTER, DeltaFileChooser.ANY_XML_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					filePath = FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile());
					jTextFieldTemplateFileName.setText(filePath);
					// templateChangeListener.fieldsChanged = true;
				}
				jTextFieldTemplateFileName.requestFocusInWindow();
			}
		};
		editTemplateDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldTemplateDescription.getText(), "Edit Template description...");
				if (newValue != null) {
					jTextFieldTemplateDescription.setText(newValue);
					jTextFieldTemplateDescription.requestFocusInWindow();
				}
			}
		};
		changeTemplateFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					templateIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldTemplateIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// templateChangeListener.fieldsChanged = true;
				}
				// jTextFieldTemplateIconFile.requestFocus();
			}
		};

		addNewStylesheet = new SimpleAction(MainImages.ADD_ICON, "Add a new XSLT Stylesheet", false) {
			public void simpleActionPerformed() {
				Stylesheet stylesheet = new Stylesheet();
				storeStylesheetProperties(stylesheet);
				stylesheetsListModel.add(stylesheet);
				stylesheetsListSelectionModel.setSelectionIndex(stylesheetsListModel.getSize() - 1);
			}
		};
		removeStylesheet = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected XSLT Stylesheet") {
			public void simpleActionPerformed() {
				int index = stylesheetsListSelectionModel.getCurrentIndex();
				stylesheetChangeListener.reset();
				stylesheetsListModel.remove(index);
				if (stylesheetsListModel.isEmpty())
					clearStylesheetGUIFields();
				else
					stylesheetsListSelectionModel.setSelectionIndex(Math.min(index, stylesheetsListModel.getSize() - 1));
			}
		};
		changeStylesheetFile = new SimpleAction(MiscImages.DOTS_ICON, "Show filechooser to change XSLT stylesheet file") {
			public void simpleActionPerformed() {
				File file = askForFile(null, DeltaFileChooser.XSL_FILE_FILTER, null);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					filePath = FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile());
					jTextFieldStylesheetFileName.setText(filePath);
				}
				jTextFieldStylesheetFileName.requestFocusInWindow();
			}
		};
		editStylesheetDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldStylesheetDescription.getText(), "Edit Stylesheet description...");
				if (newValue != null) {
					jTextFieldStylesheetDescription.setText(newValue);
					jTextFieldStylesheetDescription.requestFocusInWindow();
				}
			}
		};
		changeStylesheetFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					stylesheetIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldStylesheetIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
				}
				jTextFieldStylesheetIconFile.requestFocusInWindow();
			}
		};

		addNewHighlightDef = new SimpleAction(MainImages.ADD_ICON, "Add a new Highlight Definition", false) {
			protected void simpleActionPerformed() {
				if (!jTextFieldHighlightDefXPathExpr.getText().equals("") &&
					!jTextFieldHighlightDefLabel.getText().equals("")) {
					HighlightDef highlightDef = new HighlightDef();
					storeHighlightDefProperties(highlightDef);
					// currentXMLLanguage.getViews().getHighlightDefs().addHighlightDef(highlightDef);
					highlightDefsListModel.add(highlightDef);
					highlightDefsListSelectionModel.setSelectionIndex(highlightDefsListModel.getSize() - 1);
				}
			}
		};
		removeHighlightDef = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected Highlight Definition", false) {
			protected void simpleActionPerformed() {
				highlightDefChangeListener.reset();
				int index = highlightDefsListSelectionModel.getCurrentIndex();
				highlightDefsListModel.remove(index);
				if (highlightDefsListModel.isEmpty())
					clearHighlightDefGUIFields();
				else
					highlightDefsListSelectionModel.setSelectionIndex(Math.min(index, highlightDefsListModel.getSize() - 1));
			}
		};
		editHighlightDefXPathExpr = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				String newValue = showXPathEditDialog("Edit Highlight-Def. X-Path...", jTextFieldHighlightDefXPathExpr.getText());
				if (newValue != null) {
					jTextFieldHighlightDefXPathExpr.setText(newValue);
					jTextFieldHighlightDefXPathExpr.requestFocusInWindow();
				}
			}
		};
		editHighlightDefDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldHighlightDefDescription.getText(),
															"Edit Highlight-Def. description...");
				if (newValue != null) {
					jTextFieldHighlightDefDescription.setText(newValue);
					jTextFieldHighlightDefDescription.requestFocusInWindow();
				}
			}
		};
		changeHighlightDefIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					highlightDefIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldHighlightDefIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// highlightDefChangeListener.fieldsChanged = true;
				}
			}
		};

		addNewSummarizeDef = new SimpleAction(MainImages.ADD_ICON, "Add a new Summarize Definition", false) {
			protected void simpleActionPerformed() {
				if (!jTextFieldXSLFileName.getText().equals("")) {
					SummarizeDef summarizeDef = new SummarizeDef();
					storeSummarizeDefProperties(summarizeDef);
					// currentXMLLanguage.getViews().getSummarizeDefs().addSummarizeDef(summarizeDef);
					summarizeDefsListModel.add(summarizeDef);
					summarizeDefsListSelectionModel.setSelectionIndex(summarizeDefsListModel.getSize() - 1);
				}
			}
		};
		removeSummarizeDef = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected Summarize Definition", false) {
			protected void simpleActionPerformed() {
				summarizeDefChangeListener.reset();
				int index = summarizeDefsListSelectionModel.getCurrentIndex();
				summarizeDefsListModel.remove(index);
				if (summarizeDefsListModel.isEmpty())
					clearSummarizeDefGUIFields();
				else
					summarizeDefsListSelectionModel.setSelectionIndex(Math.min(index, summarizeDefsListModel.getSize() - 1));
			}
		};
		changeXSLFile = new SimpleAction(MiscImages.DOTS_ICON, "Show filechooser to change XSL file") {
			protected void simpleActionPerformed() {
				File file = askForFile(null, DeltaFileChooser.XSL_FILE_FILTER, null);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					jTextFieldXSLFileName.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// templateChangeListener.fieldsChanged = true;
					jTextFieldXSLFileName.requestFocusInWindow();
				}
			}
		};
		editSummarizeDefDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldSummarizeDefDescription.getText(),
															"Edit Summarize-Def. description...");
				if (newValue != null) {
					jTextFieldSummarizeDefDescription.setText(newValue);
					jTextFieldSummarizeDefDescription.requestFocusInWindow();
				}
			}
		};
		changeSummarizeDefFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					summarizeDefIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldSummarizeDefIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// highlightDefChangeListener.fieldsChanged = true;
				}
			}
		};

		addNewFoldingDef = new SimpleAction(MainImages.ADD_ICON, "Add a new Folding Definition", false) {
			protected void simpleActionPerformed() {
				if (!jTextFieldFoldingDefXPathExpr.getText().equals("") &&
					!jTextFieldFoldingDefLabel.getText().equals("")) {
					FoldingDef foldingDef = new FoldingDef();
					storeFoldingDefProperties(foldingDef);
					// currentXMLLanguage.getViews().getFoldingDefs().addFoldingDef(foldingDef);
					foldingDefsListModel.add(foldingDef);
					foldingDefsListSelectionModel.setSelectionIndex(foldingDefsListModel.getSize() - 1);
				}
			}
		};
		removeFoldingDef = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected Folding Definition", false) {
			protected void simpleActionPerformed() {
				foldingDefChangeListener.reset();
				int index = foldingDefsListSelectionModel.getCurrentIndex();
				foldingDefsListModel.remove(index);
				if (foldingDefsListModel.isEmpty())
					clearFoldingDefGUIFields();
				else
					foldingDefsListSelectionModel.setSelectionIndex(Math.min(index, foldingDefsListModel.getSize() - 1));
			}
		};
		editFoldingDefXPathExpr = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				String newValue = showXPathEditDialog("Edit Folding-Def. X-Path...", jTextFieldFoldingDefXPathExpr.getText());
				if (newValue != null) {
					jTextFieldFoldingDefXPathExpr.setText(newValue);
					jTextFieldFoldingDefXPathExpr.requestFocusInWindow();
				}
			}
		};
		editFoldingDefDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldFoldingDefDescription.getText(), "Edit Folding-Def. description...");
				if (newValue != null) {
					jTextFieldFoldingDefDescription.setText(newValue);
					jTextFieldFoldingDefDescription.requestFocusInWindow();
				}
			}
		};
		changeFoldingDefFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					foldingDefIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldFoldingDefIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
					// highlightDefChangeListener.fieldsChanged = true;
				}
			}
		};

		addNewFilterDef = new SimpleAction(MainImages.ADD_ICON, "Add a new Filter Definition", false) {
			protected void simpleActionPerformed() {
				if (!jTextFieldFilterDefXPathExpr.getText().equals("") &&
					!jTextFieldFilterDefLabel.getText().equals("")) {
					FilterDef filterDef = new FilterDef();
					storeFilterDefProperties(filterDef);
					// currentXMLLanguage.getViews().getFilterDefs().addFilterDef(filterDef);
					filterDefsListModel.add(filterDef);
					filterDefsListSelectionModel.setSelectionIndex(filterDefsListModel.getSize() - 1);
				}
			}
		};
		removeFilterDef = new SimpleAction(MainImages.REMOVE_ICON, "Remove selected Filter Definition", false) {
			protected void simpleActionPerformed() {
				filterDefChangeListener.reset();
				int index = filterDefsListSelectionModel.getCurrentIndex();
				filterDefsListModel.remove(index);
				if (filterDefsListModel.isEmpty())
					clearFilterDefGUIFields();
				else
					filterDefsListSelectionModel.setSelectionIndex(Math.min(index, filterDefsListModel.getSize() - 1));
			}
		};
		editFilterDefXPathExpr = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				String newValue = showXPathEditDialog("Edit Filter-Def. X-Path...", jTextFieldFilterDefXPathExpr.getText());
				if (newValue != null) {
					jTextFieldFilterDefXPathExpr.setText(newValue);
					jTextFieldFilterDefXPathExpr.requestFocusInWindow();
				}
			}
		};
		editFilterDefDescription = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				if (editDialog == null)
					editDialog = new TextEditDialog(XMLLanguageDialog.this);
				String newValue = editDialog.showEditDialog(jTextFieldFilterDefDescription.getText(), "Edit Filter-Def. description...");
				if (newValue != null) {
					jTextFieldFilterDefDescription.setText(newValue);
					jTextFieldFilterDefDescription.requestFocusInWindow();
				}
			}
		};
		changeFilterDefFileIcon = new AbstractAction("", MiscImages.DOTS_ICON) {
			public void actionPerformed(ActionEvent evt) {
				File file = askForFile(null, DeltaFileChooser.GIF_FILE_FILTER, DeltaFileChooser.IMAGE_FILE_FILTER);
				if (file != null) {
					String filePath = file.getAbsolutePath();
					filterDefIconLabel.setIcon(new ImageIcon(filePath));
					jTextFieldFilterDefIconFile.setText(FileUtils.toRelativeFile(filePath, currentXMLLanguage.getFile()));
				}
			}
		};

		addLinkedAttribute = new SimpleAction(MainImages.ADD_ICON, "Add linked attribute definition", true) {
			protected void simpleActionPerformed() {
				linkAttrsTableModel.addRow(new Object[]{"","",Boolean.TRUE});
			}
		};
		removeLinkedAttribute = new SimpleAction(MainImages.REMOVE_ICON, "Remove linked attribute definition", false) {
			protected void simpleActionPerformed() {
				linkAttrsTableModel.removeRow(linkAttrsTable.getSelectedRow());
			}
		};
	}

	private String showXPathEditDialog(String title, String initialValue) {
		String result = null;
		String dtdFilePath = jTextFieldOldDTD.getText();
		URL url = FileUtils.toExistingURL(dtdFilePath, currentXMLLanguage.getFile());
		if (url == null) {
			dtdFilePath = jTextFieldNewDTD.getText();
			url = FileUtils.toExistingURL(dtdFilePath, currentXMLLanguage.getFile());
		}
		if (url == null)
			JOptionPane.showMessageDialog(XMLLanguageDialog.this, "DTD \"" + dtdFilePath + "\" does not exist!", "Error",
										  JOptionPane.ERROR_MESSAGE);
		else {
			if (dtdHelper == null) dtdHelper = new DTDHelperXML4J();
			dtdHelper.loadDTDFile(url.toExternalForm());
			if (dtdHelper.getState() == DTDHelper.NO_DTD)
				JOptionPane.showMessageDialog(XMLLanguageDialog.this,
											  "DTD \"" + dtdFilePath + "\" could not be parsed!",
											  "Parse Error", JOptionPane.ERROR_MESSAGE);
			else {
				if (editDialogXPath == null)
					editDialogXPath = new XPathExpressionEditDialog(XMLLanguageDialog.this);

				editDialogXPath.init(dtdHelper);
				result = editDialogXPath.showEditDialog(initialValue, title);
			}
		}
		return result;
	}

	private void removeCurrentLanguage() {
		File file = currentXMLLanguage.getFile();
		file.renameTo(new File(currentXMLLanguage.getFile().getParentFile(),
							   currentXMLLanguage.getFile().getName() + ".removed"));
		int index = xmlLanguagesListSelectionModel.getCurrentIndex();
		xmlLanguagesListModel.remove(index);
		if (xmlLanguagesListModel.isEmpty()) {
			cardPanel.switchToCard("empty");
			currentXMLLanguage = null;
			removeLanguage.setEnabled(false);
			//    clearGUIFields();
		} else {
			xmlLanguagesListSelectionModel.setSelectionIndex(Math.min(index, xmlLanguagesListModel.getSize() - 1));
			changeListener.reset();
		}
	}

	private void clearFilterDefGUIFields() {
		filterDefsListSelectionModel.clearSelection();
		jTextFieldFilterDefXPathExpr.setText("");
		filterTypeRadioButtons.setSelected(0);
		jTextFieldFilterDefLabel.setText("");
		jTextFieldFilterDefDescription.setText("");
		jTextFieldFilterDefIconFile.setText("");
		filterDefIconLabel.setIcon(null);
		removeFilterDef.setEnabled(false);
		filterDefChangeListener.reset();
	}

	private void storeFilterDefProperties(FilterDef filterDef) {
		filterDefChangeListener.reset();
		filterDef.setXPathExpr(jTextFieldFilterDefXPathExpr.getText().trim());
		filterDef.setType((filterTypeRadioButtons.getSelected() == 0) ? "show" : "hide");
		filterDef.setLabel(jTextFieldFilterDefLabel.getText().trim());
		filterDef.setDescription(jTextFieldFilterDefDescription.getText().trim());

		String s = jTextFieldFilterDefIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(filterDef.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_FILTER_ICON;
			s = "";
		}
		filterDef.setIconFileName(s);
		filterDef.setIcon(icon);

		jListFilterDefs.repaint();
	}

	private void clearHighlightDefGUIFields() {
		highlightDefsListSelectionModel.clearSelection();
		jTextFieldHighlightDefXPathExpr.setText("");
		jTextFieldHighlightDefLabel.setText("");
		jTextFieldHighlightDefDescription.setText("");
		jTextFieldHighlightDefIconFile.setText("");
		highlightDefIconLabel.setIcon(null);
		removeHighlightDef.setEnabled(false);
		highlightDefChangeListener.reset();
	}

	private void clearSummarizeDefGUIFields() {
		summarizeDefsListSelectionModel.clearSelection();
		jTextFieldXSLFileName.setText("");
		jTextFieldSummarizeDefLabel.setText("");
		jTextFieldSummarizeDefDescription.setText("");
		jTextFieldSummarizeDefIconFile.setText("");
		summarizeDefIconLabel.setIcon(null);
		removeSummarizeDef.setEnabled(false);
		summarizeDefChangeListener.reset();
	}

	private void clearFoldingDefGUIFields() {
		foldingDefsListSelectionModel.clearSelection();
		jTextFieldFoldingDefXPathExpr.setText("");
		foldingTypeRadioButtons.setSelected(0);
		jTextFieldFoldingDefLabel.setText("");
		jTextFieldFoldingDefDescription.setText("");
		jTextFieldFoldingDefIconFile.setText("");
		foldingDefIconLabel.setIcon(null);
		removeFoldingDef.setEnabled(false);
		foldingDefChangeListener.reset();
	}

	private void storeFoldingDefProperties(FoldingDef foldingDef) {
		foldingDefChangeListener.reset();
		foldingDef.setXPathExpr(jTextFieldFoldingDefXPathExpr.getText().trim());
		foldingDef.setType((foldingTypeRadioButtons.getSelected() == 0) ? "expand" : "collapse");
		foldingDef.setLabel(jTextFieldFoldingDefLabel.getText().trim());
		foldingDef.setDescription(jTextFieldFoldingDefDescription.getText().trim());

		String s = jTextFieldFoldingDefIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(foldingDef.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_FOLDING_ICON;
			s = "";
		}
		foldingDef.setIconFileName(s);
		foldingDef.setIcon(icon);

		jListFoldingDefs.repaint();
	}

	private void storeSummarizeDefProperties(SummarizeDef summarizeDef) {
		summarizeDefChangeListener.reset();
		summarizeDef.setXslFileName(jTextFieldXSLFileName.getText().trim());
		String label = jTextFieldSummarizeDefLabel.getText().trim();
		if (label.equals("")) label = FileUtils.getFileName(summarizeDef.getXslFileName());
		summarizeDef.setLabel(label);
		summarizeDef.setDescription(jTextFieldSummarizeDefDescription.getText().trim());

		String s = jTextFieldSummarizeDefIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(summarizeDef.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_SUMMARIZE_ICON;
			s = "";
		}
		summarizeDef.setIconFileName(s);
		summarizeDef.setIcon(icon);

		jListSummarizeDefs.repaint();
	}

	private void storeHighlightDefProperties(HighlightDef highlightDef) {
		highlightDefChangeListener.reset();
		highlightDef.setXPathExpr(jTextFieldHighlightDefXPathExpr.getText().trim());
		String label = jTextFieldHighlightDefLabel.getText().trim();
		highlightDef.setLabel(label);
		highlightDef.setDescription(jTextFieldHighlightDefDescription.getText().trim());

		String s = jTextFieldHighlightDefIconFile.getText().trim();
		Icon icon = null;
		if (!s.equals("")) {
			URL url = FileUtils.toURL(highlightDef.getIconFileName(), currentXMLLanguage.getFile());
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = null;
			}
		}
		if (icon == null) {
			icon = DEFAULT_HIGHLIGHT_ICON;
			s = "";
		}
		highlightDef.setIconFileName(s);
		highlightDef.setIcon(icon);

		jListHighlightDefs.repaint();
	}

	private final MessageFormat NEW_LANGUAGE_MSG_FORMAT = new MessageFormat(NEW_LANGUAGE_BASE_NAME + "_{0}");

	private String getNewLangName() {
		if (!searchForLanguageWithName(NEW_LANGUAGE_BASE_NAME))
			return NEW_LANGUAGE_BASE_NAME;
		boolean tryAgain = true;
		String newLangName = null;
		for (int i = 1; tryAgain; i++) {
			newLangName = NEW_LANGUAGE_MSG_FORMAT.format(new Object[]{new Integer(i)});
			tryAgain = searchForLanguageWithName(newLangName);
		}
		return newLangName;
	}

	private String askForNewLangName() {
		String newLangName = getNewLangName();
		do {
			newLangName = (String) JOptionPane.showInputDialog(this, "Enter name of new XML language:", "Add new XML language",
												 JOptionPane.QUESTION_MESSAGE, null, null, newLangName);
		} while ((newLangName != null) && searchForLanguageWithName(newLangName));
		return newLangName;
	}

	private boolean searchForLanguageWithName(String name) {
		for (int i = 0, size = xmlLanguagesListModel.getSize(); i < size; i++) {
			XmlLanguage lang = (XmlLanguage) xmlLanguagesListModel.getElementAt(i);
			if (lang.getName().equals(name)) return true;
		}
		return false;
	}

	private Component createLinkedAttributesPanel() {
		JPanel panel = new JPanel(new BorderLayout(2,0));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_2);

		linkAttrsTableModel = new SimpleTableModel(new String[]{"Target-Attribute XPath", "Source XPath", "Restrict"});

		linkAttrsTable = new JTable(linkAttrsTableModel);
		linkAttrsTable.getColumnModel().setSelectionModel(new SingleListSelectionModel());
		linkAttrsTable.setRowHeight(20); // rowHeight+4 - because the combobox-celleditor is bigger
		linkAttrsTable.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
		linkAttrsTable.setSurrendersFocusOnKeystroke(true);
		linkAttrsTable.putClientProperty("JTable.autoStartsEdit", Boolean.TRUE);
		linkAttrsTable.getActionMap().put("editCellAction", new AbstractAction() { // todo
			public void actionPerformed(ActionEvent e) {
				if (linkAttrsTable.getCellEditor() != null)
					linkAttrsTable.getCellEditor().stopCellEditing();
				else {
					int col = linkAttrsTable.getSelectedColumn();
					if (col < 2)
						linkAttrsTable.editCellAt(linkAttrsTable.getSelectionModel().getMinSelectionIndex(), col, null);
				}
			}
		});
		linkAttrsTable.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "editCellAction");
		setFocusTraversalKeysEnabled(false);

		linkAttrsTable.setDefaultRenderer(String.class, new DefaultTableCellRenderer() {
			public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
				super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
				JPanel panel = new JPanel(new BorderLayout(2, 0));
				JButton button = new JButton(new EmptyAction(MiscImages.DOTS_ICON, "Edit this value in an extra XPath dialog."));
				button.setMargin(new Insets(0, 0, 0, 0));
				button.setSize(25, 20);
				panel.add(this, BorderLayout.CENTER);
				panel.add(button, BorderLayout.EAST);
				setToolTipText(value.toString());
				return panel;
			}
		});
		linkAttrsTable.setDefaultEditor(String.class,
			new TableCellEditorWithEditButton(MiscImages.DOTS_ICON, "Edit this value in an extra XPath dialog.", linkAttrsTable, GlobalObjects.CELLEDITOR_BORDER) {
				protected String showEditDialog(String title, String initialValue) {
					return showXPathEditDialog(title, initialValue);
				}
			});
		SwingUtils.packTableColumn(linkAttrsTable, 2, 2);

		panel.add(new JScrollPane(linkAttrsTable));

		JToolBar toolBar = new JToolBar(JToolBar.VERTICAL);
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addLinkedAttribute));
		toolBar.add(new SmallButton(removeLinkedAttribute));

		panel.add(toolBar, BorderLayout.EAST);

		return panel;
	}

	private Component createStylesheetsPanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewStylesheet));
		toolBar.add(new SmallButton(removeStylesheet));

		stylesheetsListModel = new SimpleListModel();
		jListStylesheets = new JList(stylesheetsListModel);
		stylesheetsListSelectionModel = new SingleListSelectionModel();
		jListStylesheets.setSelectionModel(stylesheetsListSelectionModel);
		jListStylesheets.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				Stylesheet stylesheet = (Stylesheet) value;
				text = stylesheet.getLabel();
				String tooltip = stylesheet.getDescription();
				if ((tooltip == null) || (tooltip.length() == 0)) tooltip = text;
				setToolTipText(tooltip);
				Icon icon = stylesheet.getIcon();
				if (icon == null) icon = DEFAULT_STYLESHEET_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}

			protected String getLabel(final int index, Object value) {
				return text;
			}
		});

		JPanel stylesheetPanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																		   {22.0, 22.0, 22.0, 22.0, 22.0, 22.0}}, 5, 5));
		stylesheetPanel.setBorder(new CompoundBorder(new TitledBorder("Stylesheet Definition"), GlobalObjects.EMPTY_BORDER_5));
		JLabel label = new JLabel("Filename: ");
		jTextFieldStylesheetFileName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_F);
		label.setLabelFor(jTextFieldStylesheetFileName);
		stylesheetPanel.add(label, new TableLayoutConstraints());
		stylesheetPanel.add(jTextFieldStylesheetFileName, new TableLayoutConstraints(1, 0));
		stylesheetPanel.add(new SmallButton(changeStylesheetFile), new TableLayoutConstraints(2, 0));

		label = new JLabel("Output Type: ");
		jComboboxStylesheetOutputType =
		new JComboBox(new FixedComboBoxModel(new Object[]{"html", "text", "pdf", "ps", "svg", "xmlDoc", "xmlNodes"}));
		label.setDisplayedMnemonic(KeyEvent.VK_T);
		label.setLabelFor(jComboboxStylesheetOutputType);
		stylesheetPanel.add(label, new TableLayoutConstraints(0, 1));
		stylesheetPanel.add(jComboboxStylesheetOutputType, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Output in: ");
		jComboboxStylesheetOutputIn =
		new JComboBox(new FixedComboBoxModel(new Object[]{"sameSide", "otherSide", "extraWindow", "defaultApp", "newFile"}));
		label.setDisplayedMnemonic(KeyEvent.VK_I);
		label.setLabelFor(jComboboxStylesheetOutputIn);
		stylesheetPanel.add(label, new TableLayoutConstraints(0, 2));
		stylesheetPanel.add(jComboboxStylesheetOutputIn, new TableLayoutConstraints(1, 2, 2, 2));

		label = new JLabel("Label: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldStylesheetLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldStylesheetLabel);
		stylesheetPanel.add(label, new TableLayoutConstraints(0, 3));
		stylesheetPanel.add(jTextFieldStylesheetLabel, new TableLayoutConstraints(1, 3, 2, 3));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldStylesheetDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldStylesheetDescription);
		stylesheetPanel.add(label, new TableLayoutConstraints(0, 4));
		stylesheetPanel.add(jTextFieldStylesheetDescription, new TableLayoutConstraints(1, 4));
		stylesheetPanel.add(new SmallButton(editStylesheetDescription), new TableLayoutConstraints(2, 4));

		stylesheetIconLabel = new JLabel("Icon: ");
		stylesheetIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		stylesheetIconLabel.setIconTextGap(10);
		stylesheetIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldStylesheetIconFile = new JTextField();
		stylesheetIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		stylesheetIconLabel.setLabelFor(jTextFieldStylesheetIconFile);
		stylesheetPanel.add(stylesheetIconLabel, new TableLayoutConstraints(0, 5));
		stylesheetPanel.add(jTextFieldStylesheetIconFile, new TableLayoutConstraints(1, 5));
		stylesheetPanel.add(new SmallButton(changeStylesheetFileIcon), new TableLayoutConstraints(2, 5));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(stylesheetPanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListStylesheets), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		return panel;
	}

	private Component createIconsPanel() {
		JPanel panel = new JPanel(new TableLayout(
				new double[][]{{0.3, 0.7}, {TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED, TableLayout.FILL}}, 10, 10));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		panel.add(new JLabel("Icon-Sets:"), new TableLayoutConstraints());
		panel.add(new JLabel("Element-Icons:"), new TableLayoutConstraints(1, 0));
		jListIconSets = new JList();
		jListIconSets.setSelectionModel(new SingleListSelectionModel());
		jListIconSets.setCellRenderer(new SimpleListCellRenderer() {
			protected String getLabel(final int index, Object value) {
				File dir = (File) value;
				String dirName = FileUtils.getFileName(dir);
				if (dirName.equals("icons")) return "default";
				return dirName;
			}
		});
		panel.add(new JScrollPane(jListIconSets), new TableLayoutConstraints(0, 1));
		panel.add(new JButton(refreshIconsListAction), new TableLayoutConstraints(0, 2));

		jListIcons = new JList();
		jListIcons.setLayoutOrientation(JList.HORIZONTAL_WRAP);  //VERTICAL_WRAP); // since JDK 1.4
		jListIcons.setVisibleRowCount(-1);
		jListIcons.setSelectionModel(new SingleListSelectionModel());
		jListIcons.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index,
														  boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				String absolutePath = ((File) value).getAbsolutePath();
				setIcon(new ImageIcon(absolutePath));
				setToolTipText(absolutePath);
				return this;
			}
			protected String getLabel(final int index, Object value) {
				return FileUtils.getFileName((File) value);
			}
		});
		panel.add(new JScrollPane(jListIcons), new TableLayoutConstraints(1, 1, 1, 3));

		JTextArea infoArea = new JTextArea("Create the default icon-set by placing icon-files (*.gif, *.jpg, *.png) in the 'icons' sub-directory in the folder of this XML-language. For more icon-sets, create sub-directories in the 'icons' folder.");
		infoArea.setEditable(false);
		infoArea.setLineWrap(true);
		infoArea.setWrapStyleWord(true);
		infoArea.setBackground(panel.getBackground());
		panel.add(new JScrollPane(infoArea), new TableLayoutConstraints(0, 3));
		return panel;
	}

	private Component createGeneralPanel() {
		JPanel panel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																 {22.0, 22.0, 22.0, 22.0, 22.0}}, 5, 5));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		JLabel label = new JLabel("Name: ");
		jTextFieldName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_N);
		label.setLabelFor(jTextFieldName);
		panel.add(label, new TableLayoutConstraints());
		panel.add(jTextFieldName, new TableLayoutConstraints(1, 0, 2, 0));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldDescription);
		panel.add(label, new TableLayoutConstraints(0, 1));
		panel.add(jTextFieldDescription, new TableLayoutConstraints(1, 1));
		panel.add(new SmallButton(editDescription), new TableLayoutConstraints(2, 1));

		label = new JLabel("Extension: ");
		jTextFieldExtension = new JTextField("xml");
		Dimension size = jTextFieldExtension.getPreferredSize();
		size.width = 150;
		jTextFieldExtension.setPreferredSize(size);
		label.setDisplayedMnemonic(KeyEvent.VK_E);
		label.setLabelFor(jTextFieldExtension);
		panel.add(label, new TableLayoutConstraints(0, 2));
		panel.add(jTextFieldExtension, new TableLayoutConstraints(1, 2, 2, 2, TableLayout.LEFT, TableLayout.FULL));

		xmlFileIconLabel = new JLabel("Icon: ");
		xmlFileIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		xmlFileIconLabel.setIconTextGap(10);
		xmlFileIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldFileIcon = new JTextField();
		xmlFileIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		xmlFileIconLabel.setLabelFor(jTextFieldFileIcon);
		panel.add(xmlFileIconLabel, new TableLayoutConstraints(0, 3));
		panel.add(jTextFieldFileIcon, new TableLayoutConstraints(1, 3));
		panel.add(new SmallButton(changeFileIcon), new TableLayoutConstraints(2, 3));

		label = new JLabel("in file: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldFile = new JTextField();
		jTextFieldFile.setEditable(false);
		jTextFieldFile.setBackground(GlobalObjects.DEFAULT_BACKGROUND);
		panel.add(label, new TableLayoutConstraints(0, 4));
		panel.add(jTextFieldFile, new TableLayoutConstraints(1, 4, 2, 4));

		return panel;
	}

	private Component createXMLPanel() {
		JPanel panel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED, 70.0},
																 {22.0, 22.0, 22.0}}, 5, 5));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		JLabel label = new JLabel("Original DTD: ");
		jTextFieldOldDTD = new JTextField();
		panel.add(label, new TableLayoutConstraints());
		panel.add(jTextFieldOldDTD, new TableLayoutConstraints(1, 0));
		panel.add(new SmallButton(changeOriginalDTD), new TableLayoutConstraints(2, 0));

		label = new JLabel("DELT/A-DTD: ");
		jTextFieldNewDTD = new JTextField();
		panel.add(label, new TableLayoutConstraints(0, 1));
		panel.add(jTextFieldNewDTD, new TableLayoutConstraints(1, 1));
		panel.add(new SmallButton(changeLinkDTD), new TableLayoutConstraints(2, 1));

		JButton button = new SmallButton(convertDTD);
		panel.add(button, new TableLayoutConstraints(3, 0, 3, 1));

		label = new JLabel("Root Element: ");
		jTextFieldRootElement = new JTextField();
		jTextFieldRootElement.setEditable(false);
		label.setDisplayedMnemonic(KeyEvent.VK_R);
		label.setLabelFor(jTextFieldRootElement);
		panel.add(label, new TableLayoutConstraints(0, 2));
		panel.add(jTextFieldRootElement, new TableLayoutConstraints(1, 2));
		panel.add(new SmallButton(changeRootElement), new TableLayoutConstraints(2, 2));
		return panel;
	}

	private Component createTemplatesPanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewTemplate));
		toolBar.add(new SmallButton(removeTemplate));

		templatesListModel = new SimpleListModel();
		jListXMLTemplates = new JList(templatesListModel);
		templatesListSelectionModel = new SingleListSelectionModel();
		jListXMLTemplates.setSelectionModel(templatesListSelectionModel);
		jListXMLTemplates.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				Template template = (Template) value;
				text = template.getLabel();
				String tooltip = template.getDescription();
				if ((tooltip == null) || (tooltip.length() == 0)) tooltip = text;
				setToolTipText(tooltip);
				Icon icon = template.getIcon();
				if (icon == null) icon = DEFAULT_TEMPLATE_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}

			protected String getLabel(final int index, Object value) {
				return text;
			}
		});

		JPanel templatePanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																		 {22.0, 22.0, 22.0, 22.0}}, 5, 5));
		templatePanel.setBorder(new CompoundBorder(new TitledBorder("Template Definition"), GlobalObjects.EMPTY_BORDER_5));
		JLabel label = new JLabel("Filename: ");
		jTextFieldTemplateFileName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_F);
		label.setLabelFor(jTextFieldTemplateFileName);
		templatePanel.add(label, new TableLayoutConstraints());
		templatePanel.add(jTextFieldTemplateFileName, new TableLayoutConstraints(1, 0));
		templatePanel.add(new SmallButton(changeTemplateFile), new TableLayoutConstraints(2, 0));

		label = new JLabel("Label: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldTemplateLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldTemplateLabel);
		templatePanel.add(label, new TableLayoutConstraints(0, 1));
		templatePanel.add(jTextFieldTemplateLabel, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldTemplateDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldTemplateDescription);
		templatePanel.add(label, new TableLayoutConstraints(0, 2));
		templatePanel.add(jTextFieldTemplateDescription, new TableLayoutConstraints(1, 2));
		templatePanel.add(new SmallButton(editTemplateDescription), new TableLayoutConstraints(2, 2));

		templateIconLabel = new JLabel("Icon: ");
		templateIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		templateIconLabel.setIconTextGap(10);
		templateIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldTemplateIconFile = new JTextField();
		templateIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		templateIconLabel.setLabelFor(jTextFieldTemplateIconFile);
		templatePanel.add(templateIconLabel, new TableLayoutConstraints(0, 3));
		templatePanel.add(jTextFieldTemplateIconFile, new TableLayoutConstraints(1, 3));
		templatePanel.add(new SmallButton(changeTemplateFileIcon), new TableLayoutConstraints(2, 3));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(templatePanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListXMLTemplates), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		return panel;
	}

	private Component createHighlightsPanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewHighlightDef));
		toolBar.add(new SmallButton(removeHighlightDef));
		highlightDefsListModel = new SimpleListModel();
		jListHighlightDefs = new JList(highlightDefsListModel);
		highlightDefsListSelectionModel = new SingleListSelectionModel();
		jListHighlightDefs.setSelectionModel(highlightDefsListSelectionModel);
		jListHighlightDefs.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index,
														  boolean isSelected, boolean cellHasFocus) {
				HighlightDef highlightDef = (HighlightDef) value;
				text = highlightDef.getLabel();
				setToolTipText(highlightDef.getDescription());
				Icon icon = highlightDef.getIcon();
				if (icon == null) icon = DEFAULT_HIGHLIGHT_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}

			protected String getLabel(final int index, Object value) {
				return text;
			}
		});
		JPanel highlightDefPanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																			 {22.0, 22.0, 22.0, 22.0}}, 5, 5));
		highlightDefPanel.setBorder(new CompoundBorder(new TitledBorder("Highlight Definition"), GlobalObjects.EMPTY_BORDER_5));
		JLabel label = new JLabel("XPath Expr.: ");
		jTextFieldHighlightDefXPathExpr = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_X);
		label.setLabelFor(jTextFieldHighlightDefXPathExpr);
		highlightDefPanel.add(label, new TableLayoutConstraints());
		highlightDefPanel.add(jTextFieldHighlightDefXPathExpr, new TableLayoutConstraints(1, 0));
		highlightDefPanel.add(new SmallButton(editHighlightDefXPathExpr), new TableLayoutConstraints(2, 0));

		label = new JLabel("Label: ");
		jTextFieldHighlightDefLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldHighlightDefLabel);
		highlightDefPanel.add(label, new TableLayoutConstraints(0, 1));
		highlightDefPanel.add(jTextFieldHighlightDefLabel, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldHighlightDefDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldHighlightDefDescription);
		highlightDefPanel.add(label, new TableLayoutConstraints(0, 2));
		highlightDefPanel.add(jTextFieldHighlightDefDescription, new TableLayoutConstraints(1, 2));
		highlightDefPanel.add(new SmallButton(editHighlightDefDescription), new TableLayoutConstraints(2, 2));

		highlightDefIconLabel = new JLabel("Icon: ");
		highlightDefIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		highlightDefIconLabel.setIconTextGap(10);
		highlightDefIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldHighlightDefIconFile = new JTextField();
		highlightDefIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		highlightDefIconLabel.setLabelFor(jTextFieldHighlightDefIconFile);
		highlightDefPanel.add(highlightDefIconLabel, new TableLayoutConstraints(0, 3));
		highlightDefPanel.add(jTextFieldHighlightDefIconFile, new TableLayoutConstraints(1, 3));
		highlightDefPanel.add(new SmallButton(changeHighlightDefIcon), new TableLayoutConstraints(2, 3));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(highlightDefPanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListHighlightDefs), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		return panel;
	}

	private Component createFoldingPanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewFoldingDef));
		toolBar.add(new SmallButton(removeFoldingDef));
		foldingDefsListModel = new SimpleListModel();
		jListFoldingDefs = new JList(foldingDefsListModel);
		foldingDefsListSelectionModel = new SingleListSelectionModel();
		jListFoldingDefs.setSelectionModel(foldingDefsListSelectionModel);
		jListFoldingDefs.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index,
														  boolean isSelected, boolean cellHasFocus) {
				FoldingDef foldingDef = (FoldingDef) value;
				text = foldingDef.getLabel();
				setToolTipText(foldingDef.getDescription());
				Icon icon = foldingDef.getIcon();
				if (icon == null) icon = DEFAULT_FOLDING_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}

			protected String getLabel(final int index, Object value) {
				return text;
			}
		});
		JPanel templatePanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																		 {22.0, 22.0, 22.0, 22.0, 22.0}}, 5, 5));
		templatePanel.setBorder(new CompoundBorder(new TitledBorder("Folding Definition"), GlobalObjects.EMPTY_BORDER_5));

		JLabel label = new JLabel("XPath Expr.: ");
		jTextFieldFoldingDefXPathExpr = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_X);
		label.setLabelFor(jTextFieldFoldingDefXPathExpr);
		templatePanel.add(label, new TableLayoutConstraints());
		templatePanel.add(jTextFieldFoldingDefXPathExpr, new TableLayoutConstraints(1, 0));
		templatePanel.add(new SmallButton(editFoldingDefXPathExpr), new TableLayoutConstraints(2, 0));

		label = new JLabel("Type: ");
		foldingTypeRadioButtons = new RadioButtonsPanel();
		foldingTypeRadioButtons.add("Expand", 'E');
		foldingTypeRadioButtons.add("Collapse", 'C');
		templatePanel.add(label, new TableLayoutConstraints(0, 1));
		templatePanel.add(foldingTypeRadioButtons, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Label: ");
		jTextFieldFoldingDefLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldFoldingDefLabel);
		templatePanel.add(label, new TableLayoutConstraints(0, 2));
		templatePanel.add(jTextFieldFoldingDefLabel, new TableLayoutConstraints(1, 2, 2, 2));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldFoldingDefDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldFoldingDefDescription);
		templatePanel.add(label, new TableLayoutConstraints(0, 3));
		templatePanel.add(jTextFieldFoldingDefDescription, new TableLayoutConstraints(1, 3));
		templatePanel.add(new SmallButton(editFoldingDefDescription), new TableLayoutConstraints(2, 3));

		foldingDefIconLabel = new JLabel("Icon: ");
		foldingDefIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		foldingDefIconLabel.setIconTextGap(10);
		foldingDefIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldFoldingDefIconFile = new JTextField();
		foldingDefIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		foldingDefIconLabel.setLabelFor(jTextFieldFoldingDefIconFile);
		templatePanel.add(foldingDefIconLabel, new TableLayoutConstraints(0, 4));
		templatePanel.add(jTextFieldFoldingDefIconFile, new TableLayoutConstraints(1, 4));
		templatePanel.add(new SmallButton(changeFoldingDefFileIcon), new TableLayoutConstraints(2, 4));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(templatePanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListFoldingDefs), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		return panel;
	}

	private Component createSummarizePanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewSummarizeDef));
		toolBar.add(new SmallButton(removeSummarizeDef));
		summarizeDefsListModel = new SimpleListModel();
		jListSummarizeDefs = new JList(summarizeDefsListModel);
		summarizeDefsListSelectionModel = new SingleListSelectionModel();
		jListSummarizeDefs.setSelectionModel(summarizeDefsListSelectionModel);
		jListSummarizeDefs.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index,
														  boolean isSelected, boolean cellHasFocus) {
				SummarizeDef summarizeDef = (SummarizeDef) value;
				text = summarizeDef.getLabel();
				setToolTipText(summarizeDef.getDescription());
				Icon icon = summarizeDef.getIcon();
				if (icon == null) icon = DEFAULT_SUMMARIZE_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}
			protected String getLabel(final int index, Object value) {
				return text;
			}
		});
		JPanel summarizeDefPanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																			   {22.0, 22.0, 22.0, 22.0}}, 5, 5));
		summarizeDefPanel.setBorder(
				new CompoundBorder(new TitledBorder("Summarize Definition"), GlobalObjects.EMPTY_BORDER_5));
		JLabel label = new JLabel("XSL-Filename: ");
		jTextFieldXSLFileName = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_F);
		label.setLabelFor(jTextFieldXSLFileName);
		summarizeDefPanel.add(label, new TableLayoutConstraints());
		summarizeDefPanel.add(jTextFieldXSLFileName, new TableLayoutConstraints(1, 0));
		summarizeDefPanel.add(new SmallButton(changeXSLFile), new TableLayoutConstraints(2, 0));

		label = new JLabel("Label: ");
		jTextFieldSummarizeDefLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldSummarizeDefLabel);
		summarizeDefPanel.add(label, new TableLayoutConstraints(0, 1));
		summarizeDefPanel.add(jTextFieldSummarizeDefLabel, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldSummarizeDefDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldSummarizeDefDescription);
		summarizeDefPanel.add(label, new TableLayoutConstraints(0, 2));
		summarizeDefPanel.add(jTextFieldSummarizeDefDescription, new TableLayoutConstraints(1, 2));
		summarizeDefPanel.add(new SmallButton(editSummarizeDefDescription), new TableLayoutConstraints(2, 2));

		summarizeDefIconLabel = new JLabel("Icon: ");
		summarizeDefIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		summarizeDefIconLabel.setIconTextGap(10);
		summarizeDefIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldSummarizeDefIconFile = new JTextField();
		summarizeDefIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		summarizeDefIconLabel.setLabelFor(jTextFieldSummarizeDefIconFile);
		summarizeDefPanel.add(summarizeDefIconLabel, new TableLayoutConstraints(0, 3));
		summarizeDefPanel.add(jTextFieldSummarizeDefIconFile, new TableLayoutConstraints(1, 3));
		summarizeDefPanel.add(new SmallButton(changeSummarizeDefFileIcon), new TableLayoutConstraints(2, 3));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(summarizeDefPanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListSummarizeDefs), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		return panel;
	}

	private Component createFilterPanel() {
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewFilterDef));
		toolBar.add(new SmallButton(removeFilterDef));
		filterDefsListModel = new SimpleListModel();
		jListFilterDefs = new JList(filterDefsListModel);
		filterDefsListSelectionModel = new SingleListSelectionModel();
		jListFilterDefs.setSelectionModel(filterDefsListSelectionModel);
		jListFilterDefs.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index,
														  boolean isSelected, boolean cellHasFocus) {
				FilterDef filterDef = (FilterDef) value;
				text = filterDef.getLabel();
				setToolTipText(filterDef.getDescription());
				Icon icon = filterDef.getIcon();
				if (icon == null) icon = DEFAULT_FILTER_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}
			protected String getLabel(final int index, Object value) {
				return text;
			}
		});
		JPanel templatePanel = new JPanel(new TableLayout(new double[][]{{85.0, TableLayout.FILL, TableLayout.PREFERRED},
																		 {22.0, 22.0, 22.0, 22.0, 22.0}}, 5, 5));
		templatePanel.setBorder(new CompoundBorder(new TitledBorder("Filter Definition"), GlobalObjects.EMPTY_BORDER_5));
		JLabel label = new JLabel("XPath Expr.: ");
		jTextFieldFilterDefXPathExpr = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_X);
		label.setLabelFor(jTextFieldFilterDefXPathExpr);
		templatePanel.add(label, new TableLayoutConstraints());
		templatePanel.add(jTextFieldFilterDefXPathExpr, new TableLayoutConstraints(1, 0));
		templatePanel.add(new SmallButton(editFilterDefXPathExpr), new TableLayoutConstraints(2, 0));

		label = new JLabel("Type: ");
		filterTypeRadioButtons = new RadioButtonsPanel();
		filterTypeRadioButtons.add("Show", 'S');
		filterTypeRadioButtons.add("Hide", 'H');
		templatePanel.add(label, new TableLayoutConstraints(0, 1));
		templatePanel.add(filterTypeRadioButtons, new TableLayoutConstraints(1, 1, 2, 1));

		label = new JLabel("Label: ");
		jTextFieldFilterDefLabel = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_L);
		label.setLabelFor(jTextFieldFilterDefLabel);
		templatePanel.add(label, new TableLayoutConstraints(0, 2));
		templatePanel.add(jTextFieldFilterDefLabel, new TableLayoutConstraints(1, 2, 2, 2));

		label = new JLabel("Description: ");
		label.setFont(GlobalObjects.DEFAULT_FONT);
		jTextFieldFilterDefDescription = new JTextField();
		label.setDisplayedMnemonic(KeyEvent.VK_D);
		label.setLabelFor(jTextFieldFilterDefDescription);
		templatePanel.add(label, new TableLayoutConstraints(0, 3));
		templatePanel.add(jTextFieldFilterDefDescription, new TableLayoutConstraints(1, 3));
		templatePanel.add(new SmallButton(editFilterDefDescription), new TableLayoutConstraints(2, 3));

		filterDefIconLabel = new JLabel("Icon: ");
		filterDefIconLabel.setFont(GlobalObjects.DEFAULT_FONT);
		filterDefIconLabel.setIconTextGap(10);
		filterDefIconLabel.setHorizontalTextPosition(SwingConstants.LEADING);
		jTextFieldFilterDefIconFile = new JTextField();
		filterDefIconLabel.setDisplayedMnemonic(KeyEvent.VK_I);
		filterDefIconLabel.setLabelFor(jTextFieldFilterDefIconFile);
		templatePanel.add(filterDefIconLabel, new TableLayoutConstraints(0, 4));
		templatePanel.add(jTextFieldFilterDefIconFile, new TableLayoutConstraints(1, 4));
		templatePanel.add(new SmallButton(changeFilterDefFileIcon), new TableLayoutConstraints(2, 4));

		JPanel panel = new JPanel(
				new TableLayout(new double[][]{{TableLayout.FILL, 120.0}, {28.0, TableLayout.PREFERRED, TableLayout.FILL}}, 2, 0));
		panel.add(templatePanel, new TableLayoutConstraints(0, 1));
		panel.add(toolBar, new TableLayoutConstraints(1, 0));
		panel.add(new JScrollPane(jListFilterDefs), new TableLayoutConstraints(1, 1, 1, 2));
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);

		return panel;
	}

	private Component createApplicablePanel() {
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(GlobalObjects.EMPTY_BORDER_5);
		tabbedPaneViews = new JTabbedPane();
		tabbedPaneViews.insertTab("Highlight", 	DEFAULT_HIGHLIGHT_ICON,	 createHighlightsPanel(),  "if XPath expression result to\n   one node ... select and scroll to treenode\n   multiple nodes ... highlight treenodes and scroll to and select first\n   alue(s) ... display it", APPLICABLES_TAB_INDEX_XPATH);
		tabbedPaneViews.insertTab("Folding", 		DEFAULT_FOLDING_ICON, 	 createFoldingPanel(), 	   null, 																																														  APPLICABLES_TAB_INDEX_FOLDING);
		tabbedPaneViews.insertTab("Filter",			DEFAULT_FILTER_ICON, 	 createFilterPanel(), 	   null, 																																														  APPLICABLES_TAB_INDEX_FILTER);
		tabbedPaneViews.insertTab("Summarize", 		DEFAULT_SUMMARIZE_ICON,	 createSummarizePanel(),   null, 																																														  APPLICABLES_TAB_INDEX_SUMMARIZE);
		tabbedPaneViews.insertTab("Transformation",	DEFAULT_STYLESHEET_ICON, createStylesheetsPanel(), null, 																																														  APPLICABLES_TAB_INDEX_TRANSFORM);
		panel.add(tabbedPaneViews);
		return panel;
	}

	private Component createLanguagesPanel() {
		JPanel panel = new JPanel(new BorderLayout());
		panel.setBorder(new CompoundBorder(BorderFactory.createEtchedBorder(), GlobalObjects.EMPTY_BORDER_2));
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		toolBar.add(new SmallButton(addNewLanguage));
		toolBar.add(new SmallButton(removeLanguage));
		panel.add(toolBar, BorderLayout.NORTH);

		xmlLanguagesListModel = new SimpleListModel();
		jListXMLLanguages = new JList(xmlLanguagesListModel);
		xmlLanguagesListSelectionModel = new SingleListSelectionModel();
		jListXMLLanguages.setSelectionModel(xmlLanguagesListSelectionModel);
		jListXMLLanguages.setCellRenderer(new SimpleListCellRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				XmlLanguage lang = (XmlLanguage) value;
				text = lang.getName();
				if (MainController.deltaSideController[GUIConstants.LEFT].currentXMLLanguage == lang) text = text + " [LEFT]";
				if (MainController.deltaSideController[GUIConstants.RIGHT].currentXMLLanguage == lang) text = text + " [RIGHT]";
				setToolTipText(lang.getDescription());
				Icon icon = lang.getIcon();
				if (icon == null) icon = DEFAULT_LANGUAGE_ICON;
				setIcon(icon);
				return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			}
			protected String getLabel(final int index, Object value) {
				return text;
			}
		});
		panel.add(new JScrollPane(jListXMLLanguages), BorderLayout.CENTER);
		return panel;
	}

// todo: for debug purpose
//    public static void main(String args[]) {
//        new XMLLanguageDialog(new JFrame(), MainController.loadXMLLanguages()).setVisible(true);
//    }

	public boolean newTemplate(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_TEMPLATES);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearTemplateGUIFields();
		jTextFieldTemplateFileName.requestFocusInWindow();
		return showDialog();
	}

	public boolean newFilterDef(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_APPLICABLES);
		tabbedPaneViews.setSelectedIndex(APPLICABLES_TAB_INDEX_FILTER);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearFilterDefGUIFields();
		jTextFieldFilterDefXPathExpr.requestFocusInWindow();
		return showDialog();
	}

	public boolean newSummarizeDef(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_APPLICABLES);
		tabbedPaneViews.setSelectedIndex(APPLICABLES_TAB_INDEX_SUMMARIZE);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearSummarizeDefGUIFields();
		jTextFieldXSLFileName.requestFocusInWindow();
		return showDialog();
	}

	public boolean newHighlightDef(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_APPLICABLES);
		tabbedPaneViews.setSelectedIndex(APPLICABLES_TAB_INDEX_XPATH);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearHighlightDefGUIFields();
		jTextFieldHighlightDefXPathExpr.requestFocusInWindow();
		return showDialog();
	}

	public boolean newFoldingDef(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_APPLICABLES);
		tabbedPaneViews.setSelectedIndex(APPLICABLES_TAB_INDEX_FOLDING);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearFoldingDefGUIFields();
		jTextFieldFoldingDefXPathExpr.requestFocusInWindow();
		return showDialog();
	}


	public boolean newStylesheetDef(XmlLanguage currentXMLLanguage) {
		tabbedPaneMain.setSelectedIndex(TAB_INDEX_APPLICABLES);
		tabbedPaneViews.setSelectedIndex(APPLICABLES_TAB_INDEX_TRANSFORM);
		jListXMLLanguages.setSelectedValue(currentXMLLanguage, true);
		clearStylesheetGUIFields();
		jTextFieldStylesheetFileName.requestFocusInWindow();
		return showDialog();
	}
}
