/*
 * Part of "Guideline Markup Tool"
 * Created by peter, 08.10.2004, 15:50:34
 */

package at.ac.tuwien.asgaard.delta.gui.misc;

import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import at.ac.tuwien.asgaard.swing.models.FixedComboBoxModel;
import at.ac.tuwien.asgaard.util.BrowserControl;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

/**
 * Class XPathExpressionEditDialog ... todo
 * 
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class XPathExpressionEditDialog extends TextEditDialog {
	private FixedComboBoxModel elementsComboBoxModel;
	private FixedComboBoxModel attributesComboBoxModel;
	private DTDHelper dtdHelper;
    private SimpleAction insertAttributeAction;

    public XPathExpressionEditDialog(Component parentComponent) {
		this(parentComponent, null);
	}

	public XPathExpressionEditDialog(Component parentComponent, String title) {
		this(parentComponent, (title == null) ? "Edit XPath expression..." : title, false, false, 500, 200, 300, 150, 0, 0);
	}

	public XPathExpressionEditDialog(Component parentComponent, String title,
									 boolean useHelpButton, boolean useApplyButton, final int INIT_WIDTH, final int INIT_HEIGHT,
									 int minWidth, int minHeight, int maxWidth, int maxHeight) {
		super(parentComponent, title, useHelpButton, useApplyButton, INIT_WIDTH, INIT_HEIGHT, minWidth, minHeight, maxWidth, maxHeight);

		JPanel toolBar = new JPanel(new TableLayout(new double[][]{{TableLayout.FILL,TableLayout.PREFERRED, 2, TableLayout.FILL,TableLayout.PREFERRED, 5, TableLayout.FILL,TableLayout.PREFERRED, 10, TableLayout.PREFERRED, 5, TableLayout.PREFERRED},{TableLayout.PREFERRED}}));
		elementsComboBoxModel = new FixedComboBoxModel();
		final JComboBox elementsComboBox = new JComboBox(elementsComboBoxModel);
		elementsComboBox.setRenderer(new BasicComboBoxRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			// if (getPreferredSize().width > elementsComboBox.getPreferredSize().width)
					list.setToolTipText((String) value);
				return this;
			}
		});
		elementsComboBox.setToolTipText("elements");
		toolBar.add(elementsComboBox, new TableLayoutConstraints());
		toolBar.add(new SmallButton(new SimpleAction(MainImages.PASTE_ICON, "insert element name"){
			protected void simpleActionPerformed() {
				String elementName = elementsComboBoxModel.getSelectedItem().toString();
				if (textField.getText().length() == 0)
					textField.setText("//"+elementName);
				else
					textField.replaceSelection("/"+elementName);
				textField.requestFocusInWindow();
			}
		}), new TableLayoutConstraints(1,0));

		attributesComboBoxModel = new FixedComboBoxModel();
		final JComboBox attributesComboBox = new JComboBox(attributesComboBoxModel);
		attributesComboBox.setRenderer(new BasicComboBoxRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			// if (getPreferredSize().width > elementsComboBox.getPreferredSize().width)
					list.setToolTipText((String) value);
				return this;
			}
		});
		attributesComboBox.setToolTipText("attributes");
		toolBar.add(attributesComboBox, new TableLayoutConstraints(3, 0));
        insertAttributeAction = new SimpleAction(MainImages.PASTE_ICON, "insert attribute name") {
            protected void simpleActionPerformed() {
                textField.replaceSelection("@"+attributesComboBoxModel.getSelectedItem().toString());
                textField.requestFocusInWindow();
            }
        };
        toolBar.add(new SmallButton(insertAttributeAction), new TableLayoutConstraints(4,0));

		final FixedComboBoxModel axisComboboxModel = new FixedComboBoxModel(new Object[]{"ancestor","ancestor-or-self","descendant","descendant-or-self","following","following-sibling","parent","preceding","preceding-sibling","self"});
		JComboBox comboBox = new JComboBox(axisComboboxModel);
		comboBox.setRenderer(new BasicComboBoxRenderer() {
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				// if (getPreferredSize().width > elementsComboBox.getPreferredSize().width)
				list.setToolTipText(value.toString());
				return this;
			}
		});
		comboBox.setToolTipText("axes");
		toolBar.add(comboBox, new TableLayoutConstraints(6, 0));
		toolBar.add(new SmallButton(new SimpleAction(MainImages.PASTE_ICON, "insert attribute name") {
			protected void simpleActionPerformed() {
				textField.replaceSelection(axisComboboxModel.getSelectedItem().toString()+"::");
				textField.requestFocusInWindow();
			}
		}), new TableLayoutConstraints(7, 0));

		Action xpathClearAction = new SimpleAction(MainImages.CLEAR_ICON, "Clear XPath bar",
												   KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK)) {
			protected void simpleActionPerformed() {
				textField.setText("");
				textField.requestFocusInWindow();
			}
		};
		toolBar.add(new SmallButton(xpathClearAction), new TableLayoutConstraints(9, 0));

		Action xpathInfoAction = new SimpleAction(MainImages.WWW_HELP_ICON, "XML Path Language (XPath) 2.0 - W3C Working Draft",
												   KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0)) {
			protected void simpleActionPerformed() {
				BrowserControl.displayURL("http://www.w3.org/TR/xpath20/");
			}
		};
		toolBar.add(new SmallButton(xpathInfoAction), new TableLayoutConstraints(11, 0));

		ActionMap xpathBarTextActionMap = textField.getActionMap();
		InputMap xpathBarTextInputMap = textField.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		SwingUtils.addShortCut(xpathBarTextActionMap, xpathBarTextInputMap, xpathClearAction);
		SwingUtils.addShortCut(xpathBarTextActionMap, xpathBarTextInputMap, xpathInfoAction);

		contentPane.add(toolBar, BorderLayout.NORTH);

		elementsComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (dtdHelper != null) {
					Object selectedItem = elementsComboBoxModel.getSelectedItem();
					if (selectedItem != null) {
						String[] allAttributesForElement = dtdHelper.getAllAttributesForElement(selectedItem.toString());
						attributesComboBoxModel.init(allAttributesForElement, true);
						attributesComboBox.setEnabled(allAttributesForElement.length > 0);
                        insertAttributeAction.setEnabled(allAttributesForElement.length > 0);
					}
				}
			}
		});
	}

	public void init(DTDHelper dtdHelper) {
		this.dtdHelper = dtdHelper;
		elementsComboBoxModel.init(dtdHelper.getAllElements(), true);
	}

	protected boolean onOK() {  // check if this is a valid XPath 2 expression
		String xpathExpr = textField.getText().trim();
		if (!xpathExpr.equals(""))
			try {
				XMLUtils.checkXPathExpression(xpathExpr);
				return true;
			} catch (Exception exc) {
				JOptionPane.showMessageDialog(this, exc.getMessage(), "Bad XPath-Expression!", JOptionPane.ERROR_MESSAGE);
				textField.selectAll();
				textField.requestFocusInWindow();
			}
		return false;
	//	return XMLUtils.isValidXPathExpression(textField.getText());
	}
}
