/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 03.09.2003
 * Time: 12:16:00
 */
package at.ac.tuwien.asgaard.delta.gui.misc.undo;

import at.ac.tuwien.asgaard.delta.gui.*;

import javax.swing.undo.CompoundEdit;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.CannotRedoException;
import java.util.Iterator;

/**
 * Class SideCompoundEdit ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public abstract class SideCompoundEdit extends CompoundEdit implements SideUndoableEdit {
    protected int originSideIndex;
	protected final boolean affectsBothSides;
	protected String undoPresentationName, redoPresentationName;

	protected SideCompoundEdit(boolean affectsBothSides) {
		super();
		this.affectsBothSides = affectsBothSides;
		undoPresentationName = "UNDO"; redoPresentationName = "REDO"; // to overwrite in subclass
	}

	public boolean sideAffected(int sideIndex) {
		if (affectsBothSides || (this.originSideIndex == sideIndex))
			return true;

		Iterator iterator = edits.iterator();
		for (int i = edits.size(); i > 0; i--) {
			Object o = iterator.next();
			if (!(o instanceof SideUndoableEdit)) return false; // Todo: ClassCastException happended below
			SideUndoableEdit sideUndoableEdit = (SideUndoableEdit) o;
			if (sideUndoableEdit.sideAffected(sideIndex))
				return true;
		}
		return false;
	}

    public int getOriginSideIndex() {
        return originSideIndex;
    }

    public void setOriginSideIndex(int sideIndex) {
        this.originSideIndex = sideIndex;
    }

	public boolean affectsBothSides() {
		return affectsBothSides;
	}

	public void undo() throws CannotUndoException {
		MainController.setActiveSideIndex(originSideIndex);
		super.undo();
	}

    public void redo() throws CannotRedoException {
        MainController.setActiveSideIndex(originSideIndex);
        super.redo();
    }

	public boolean isSignificant() {
		return true;
	}

	public String getUndoPresentationName() {
		return undoPresentationName + " (" + GUIConstants.SIDE_NAMES[originSideIndex] + ")";
	}

	public String getRedoPresentationName() {
		return redoPresentationName + " (" + GUIConstants.SIDE_NAMES[originSideIndex] + ")";
	}
}
