package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.macros.MacrosListItem;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.CompleteMouseAdapter;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.models.SimpleMutableTreeNode;
import at.ac.tuwien.asgaard.swing.models.SimpleTreeModel;
import at.ac.tuwien.asgaard.swing.models.SingleTreeSelectionModel;
import at.ac.tuwien.asgaard.util.MiscUtils;

import org.w3c.dom.Node;

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.tree.*;

/**
 * SimpleXMLTree
 * @author  Peter VOTRUBA
 * @version 2.7.3
 */
public class SimpleXMLTree extends JTree {
    protected Preferences properties;
	protected boolean rollover;
	protected SimpleXMLTreeCellRenderer simpleXMLTreeCellRenderer;
	protected XMLTreeNode rootXMLTreeNode, currentlyHoveredXMLTreeNode, lastXMLTreeNode;
	protected SimpleTreeModel simpleTreeModel;
    protected boolean empty;
	protected Map<Node,XMLTreeNode> xmlNode2TreeNodeMap;

	protected boolean modeAutoCollapse, modeAutoCollapseOnSelection, modeAutoExpandOnSelection, modeAutoSelectOnFolding, modeFishEyeView;
	protected TreePath expandedTreePath;
	protected CompleteMouseAdapter xmlTreeMouseListener;

	private boolean collapsed = false;
	private boolean showChildCount;
	private boolean showAttributes;

	private boolean scrollSmoothly = false; // TODO

	/** Creates new SimpleXMLTree */
	public SimpleXMLTree(Preferences properties) {
		super(new SimpleTreeModel());
		this.properties = properties;
		simpleTreeModel = (SimpleTreeModel)treeModel;
		toggleClickCount = 0;
		empty = true;

		putClientProperty("JTree.lineStyle", "Angled"); // Possible values are "Angled", "Horizontal", and "None" (the default).
		ToolTipManager.sharedInstance().registerComponent(this);
		installKeyboardActions();

		addTreeWillExpandListener(new TreeWillExpandListener() {
			public void treeWillExpand(TreeExpansionEvent evt) { }
			public void treeWillCollapse(TreeExpansionEvent evt) throws ExpandVetoException {
				if (evt.getPath().getLastPathComponent() == rootXMLTreeNode)
					throw new ExpandVetoException(evt);
			}
		});
		addTreeExpansionListener(new TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent evt) {
				TreePath treePath = evt.getPath();
				if (modeAutoCollapse) {
					while (!expandedTreePath.isDescendant(treePath)) {
					//	boolean exp = isExpanded(expandedTreePath);  // TODO ???
						collapsePath(expandedTreePath);
					}
					expandedTreePath = treePath;
				}
				if (modeAutoSelectOnFolding && !((SingleTreeSelectionModel)selectionModel).isCurrentlySelecting())
					setSelectionPath(treePath);
			}
			public void treeCollapsed(TreeExpansionEvent evt) {
				collapsed = true;
				TreePath treePath = evt.getPath();
				expandedTreePath = treePath.getParentPath();
				if (modeAutoSelectOnFolding)
					setSelectionPath(treePath);
			}
		});
		addTreeSelectionListener(new TreeSelectionListener() {
			public void valueChanged(TreeSelectionEvent evt) {
				TreePath treePath = evt.getPath();

			// scrollPathToVisible(treePath); // todo only if treepath would be hidden

				if (modeFishEyeView) {
					simpleXMLTreeCellRenderer.setRowOfFocusedTreeNode(getRowForPath(treePath)); // if (treePath == null) getRowForPath() returns -1
					repaint();
				}

				if (treePath == null) return;
				if (collapsed && expandedTreePath.isDescendant(treePath)) {
					collapsed = false;
					return;
				}

				if (modeAutoExpandOnSelection && !getSelectedTreeNode().isLeaf() && !isExpanded(treePath))
					expandPath(treePath);
				else if (modeAutoCollapseOnSelection) {
					if ((treePath.getLastPathComponent() == rootXMLTreeNode) || (treePath.getParentPath().getLastPathComponent() == rootXMLTreeNode)
						|| (expandedTreePath.getLastPathComponent() == rootXMLTreeNode) || (expandedTreePath.getParentPath().getLastPathComponent() == rootXMLTreeNode)) return;
					if (!expandedTreePath.isDescendant(treePath)) {
						do {
							collapsePath(expandedTreePath);
						} while (!expandedTreePath.isDescendant(treePath));
						if (isExpanded(treePath))
							expandedTreePath = treePath;
						else
							expandedTreePath = treePath.getParentPath();
					}
				}
				collapsed = false;
			}
		});

		xmlTreeMouseListener = createXMLTreeMouseListener();
		addMouseListener(xmlTreeMouseListener);
		addMouseMotionListener(xmlTreeMouseListener);

		setEditable(false);
		setRootVisible(false);
		setCellRenderer(createXMLTreeCellRenderer());
		setSelectionModel(new SingleTreeSelectionModel());

		setProperties(false);

		setBackground(GlobalObjects.DEFAULT_BACKGROUND);
		setFocusTraversalKeysEnabled(false);

		addHierarchyListener(new HierarchyListener() {
			public void hierarchyChanged(HierarchyEvent evt) {
				if ((evt.getChangeFlags() & HierarchyEvent.DISPLAYABILITY_CHANGED) == HierarchyEvent.DISPLAYABILITY_CHANGED)
					getParent().getParent().addMouseWheelListener(xmlTreeMouseListener);
					//System.out.println("hierarchyChanged "+getParent());
			}
		});
	}

	/**
	 * @see javax.swing.plaf.basic.BasicTreeUI #createActionMap() / #loadActionMap
	 */
    protected void installKeyboardActions() {
        ActionMap actionMap = getActionMap();
        actionMap.remove("selectAll");
        actionMap.remove("clearSelection");
        actionMap.remove("postTip");
        actionMap.remove("hideTip");
        actionMap.remove("toggleSelectionPreserveAnchor");
        actionMap.remove("extendSelection");
		actionMap.remove("startEditing");

        actionMap.remove("selectPreviousLead");
        actionMap.remove("selectPreviousExtendSelection");
        actionMap.remove("selectNextChangeLead");
        actionMap.remove("selectNextExtendSelection");

        actionMap.remove("scrollUpChangeLead");
        actionMap.remove("scrollUpExtendSelection");
        actionMap.remove("scrollDownExtendSelection");
        actionMap.remove("scrollDownChangeLead");
        actionMap.remove("selectFirstExtendSelection");
        actionMap.remove("selectLastExtendSelection");

        InputMap inputMap = getInputMap();
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0), "toggle");
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, InputEvent.ALT_MASK),       "gotoParent");
		actionMap.put("gotoParent", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				gotoParentNode();
			}
		});
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, InputEvent.CTRL_MASK),     "gotoPreviousSibling");
        actionMap.put("gotoPreviousSibling", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				gotoPreviousSiblingAction();
			}
		});
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, InputEvent.CTRL_MASK),   "gotoNextSibling");
        actionMap.put("gotoNextSibling", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				gotoNextSiblingAction();
			}
		});
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_HOME, InputEvent.CTRL_MASK),   "gotoFirstSibling");
        actionMap.put("gotoFirstSibling", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				gotoFirstSiblingAction();
			}
		});
        inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_END, InputEvent.CTRL_MASK),    "gotoLastSibling");
        actionMap.put("gotoLastSibling", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				gotoLastSiblingAction();
			}
		});

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_E, InputEvent.CTRL_MASK | InputEvent.ALT_MASK), "expandAllDescendants");
		actionMap.put("expandAllDescendants", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				expandAllDescendants();
			}
		});
		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK | InputEvent.ALT_MASK), "collapseAllDescendants");
		actionMap.put("collapseAllDescendants", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				collapseAllDescendants();
			}
		});
    }

    public final void load(MacrosListItem mli) {
		if (mli == null) clear();
		else {
			if (xmlNode2TreeNodeMap == null)
				xmlNode2TreeNodeMap = new HashMap<Node, XMLTreeNode>();
			else
				xmlNode2TreeNodeMap.clear();
			rootXMLTreeNode = XMLTreeNode.getDummyTreeNode();
			buildTreeModel(rootXMLTreeNode, mli.getXMLElement());
			rootXMLTreeNode.setUserObject(mli);
			setBackground(Color.white);
			simpleTreeModel.setRoot(rootXMLTreeNode);
			expandedTreePath = simpleTreeModel.getTreePath(rootXMLTreeNode);
			setRootVisible(true);
			expandAll();
			empty = false;
		}
	}
	public final void load(Node newRootNode) {
		if (xmlNode2TreeNodeMap == null)
			xmlNode2TreeNodeMap = new HashMap<Node, XMLTreeNode>();
		else
			xmlNode2TreeNodeMap.clear();
		rootXMLTreeNode = createTreeNode(newRootNode);
		setBackground(Color.white);
		simpleTreeModel.setRoot(rootXMLTreeNode);
		expandedTreePath = simpleTreeModel.getTreePath(rootXMLTreeNode);
		setRootVisible(true);
		expandAll();
		empty = false;
	}

	public final void clear() {
        if (!empty) {
            empty = true;
		// selectionModel.setSelectionPath(null);
		//    setRootVisible(false);
		    simpleTreeModel.setRoot(null); //SimpleMutableTreeNode.getDummyTreeNode());
		    setBackground(GlobalObjects.DEFAULT_BACKGROUND);
        }
	}

	/** To be overridden */
	protected void processTreeNodeOnCreation(XMLTreeNode xmlTreeNode) { }
    protected boolean shouldInsertIntoTreeModel(Node xmlNode) {
        String text = xmlNode.getNodeValue();
        if (text == null) return true;
        return (text.trim().length() > 0);  // todo: do I need this?
    }

	protected final XMLTreeNode createTreeNode(Node xmlNode) {
		XMLTreeNode xmlTreeNode = new XMLTreeNode(xmlNode, false);
		xmlNode2TreeNodeMap.put(xmlNode, xmlTreeNode);
		processTreeNodeOnCreation(xmlTreeNode); // todo: do not need to call when inserting single element
        buildTreeModel(xmlTreeNode, xmlNode);
		xmlTreeNode.initStyleInfos();
		return xmlTreeNode;
	}

    protected final void buildTreeModel(SimpleMutableTreeNode parentXMLTreeNode, Node parentXMLNode) {
        for (Node childXMLNode = parentXMLNode.getFirstChild(); childXMLNode != null;
             childXMLNode = childXMLNode.getNextSibling()) {
            if (shouldInsertIntoTreeModel(childXMLNode))
				if (childXMLNode.getNodeType() == Node.ELEMENT_NODE) {
					XMLTreeNode childTreeNode = new XMLTreeNode(childXMLNode, false);
					xmlNode2TreeNodeMap.put(childXMLNode, childTreeNode);
					parentXMLTreeNode.add(childTreeNode);
					processTreeNodeOnCreation(childTreeNode);
					buildTreeModel(childTreeNode, childXMLNode);
					childTreeNode.initStyleInfos();
				} else {
					XMLTreeNode childTreeNode = new XMLTreeNode(childXMLNode);
					xmlNode2TreeNodeMap.put(childXMLNode, childTreeNode);
					parentXMLTreeNode.add(childTreeNode);
				}
            else {
				xmlNode2TreeNodeMap.remove(childXMLNode);
                buildTreeModel(parentXMLTreeNode, childXMLNode);
            }
        }
    }


// misc methods
	protected void setProperties(boolean refetch) {
		rollover = properties.getBoolean(Preferences.KEY_APPEARANCE_ROLLOVER_XML);
		showChildCount = properties.getBoolean(Preferences.KEY_XML_SHOW_CHILD_COUNT);
		simpleXMLTreeCellRenderer.setShowChildCount(showChildCount);
		showAttributes = properties.getBoolean(Preferences.KEY_XML_SHOW_ATTRIBUTES);
		simpleXMLTreeCellRenderer.setShowAttributes(showAttributes);
		simpleXMLTreeCellRenderer.setSelectionColor(properties.getColor(Preferences.KEY_COLOR_SELECTION));
		if (!modeAutoCollapse && properties.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND) && refetch && !empty) {
			TreePath selectedPath = getSelectionPath();
			boolean wasExpanded = isExpanded(selectedPath);
			selectionModel.setSelectionPath(new TreePath(rootXMLTreeNode));
			collapseAll();
			if (wasExpanded) expandPath(selectedPath);
			else expandPath(selectedPath.getParentPath());
			centerAndSelectTreePath(selectedPath);
			expandedTreePath = selectedPath;
		}
		modeAutoCollapse			= properties.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_EXPAND);
		modeAutoCollapseOnSelection	= properties.getBoolean(Preferences.KEY_XML_AUTO_COLLAPSE_ON_SELECT);
		modeAutoExpandOnSelection	= properties.getBoolean(Preferences.KEY_XML_AUTO_EXPAND_ON_SELECT);
		modeAutoSelectOnFolding		= properties.getBoolean(Preferences.KEY_XML_AUTO_SELECT_ON_FOLDING);
		modeFishEyeView				= properties.getBoolean(Preferences.KEY_APPEARANCE_XML_FISHEYE_VIEW);
		simpleXMLTreeCellRenderer.setRowOfFocusedTreeNode(modeFishEyeView ? getRowForPath(getSelectionPath()) : -1);
	}
	public void refetchProperties(Set<String> changedProperties) {
		setProperties(true);
		if (changedProperties.contains(Preferences.KEY_XML_SHOW_ATTRIBUTES) ||
				changedProperties.contains(Preferences.KEY_XML_SHOW_CHILD_COUNT))
			invalidate();
		repaint();
	}

	public final boolean isEmpty() {
		return empty;
	}

	public void expandAll() {
		expandAllDescendants(rootXMLTreeNode, new TreePath(rootXMLTreeNode));
	}
	public final void expandAllDescendants() {
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath == null) return;
		expandAllDescendants((TreeNode)selPath.getLastPathComponent(), selPath);
	}
	public final void expandAllDescendants(TreeNode treeNode) {
		expandAllDescendants(treeNode, simpleTreeModel.getTreePath(treeNode));
    }
	public final void expandAllDescendants(TreePath treePath) {
		expandAllDescendants((TreeNode)treePath.getLastPathComponent(), treePath);
	}
	protected final void expandAllDescendants(TreeNode treeNode, TreePath treePath) {
		if (treeNode.isLeaf()) return;
		boolean b = modeAutoSelectOnFolding;
		modeAutoSelectOnFolding = false;
		expandPath(treePath);
        int childCount = treeNode.getChildCount();
		if (modeAutoCollapse) childCount = Math.min(childCount, 1);
		for (int i = 0; i < childCount; i++) {
			TreeNode child = treeNode.getChildAt(i);
			expandAllDescendants(child, treePath.pathByAddingChild(child));
		}
		modeAutoSelectOnFolding = b;
	}

	public final void collapseAll() {
		collapseAllDescendants(rootXMLTreeNode, new TreePath(rootXMLTreeNode), false);
	}
	public final void collapseAllDescendants() {
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath == null) return;
		collapseAllDescendants((TreeNode) selPath.getLastPathComponent(), selPath, true);
	}
	public final void collapseAllDescendants(TreeNode treeNode) {
		collapseAllDescendants(treeNode, simpleTreeModel.getTreePath(treeNode), true);
    }
	public final void collapseAllDescendants(TreePath treePath) {
		collapseAllDescendants((TreeNode)treePath.getLastPathComponent(), treePath, true);
	}
	public final void collapseAllDescendants(TreePath treePath, boolean collapseItself) {
		collapseAllDescendants((TreeNode)treePath.getLastPathComponent(), treePath, collapseItself);
	}
	protected final void collapseAllDescendants(TreeNode treeNode, TreePath treePath, boolean collapseItself) {
		if (treeNode.isLeaf()) return;
		final int childCount = treeNode.getChildCount();
		for (int i = 0; i < childCount; i++) {
			TreeNode child = treeNode.getChildAt(i);
			collapseAllDescendants(child, treePath.pathByAddingChild(child), true);
		}
		if (collapseItself)
			collapsePath(treePath);
	}
	public final void collapseAllDescendantsOfChildren(TreeNode treeNode) {
		collapseAllDescendants(treeNode, simpleTreeModel.getTreePath(treeNode), false);
	}


	public final void reloadModel() {
		if (rootXMLTreeNode != null)
			if (modeAutoCollapse) {
				TreePath selectionPath = selectionModel.getSelectionPath();
				simpleTreeModel.reload();
				expandedTreePath = simpleTreeModel.getTreePath(rootXMLTreeNode);
				expandPath(expandedTreePath);
				selectionModel.setSelectionPath(selectionPath);
			} else {
				Enumeration expanded = getExpandedDescendants(new TreePath(rootXMLTreeNode));
				TreePath selectionPath = selectionModel.getSelectionPath();
				simpleTreeModel.reload();
				if (expanded != null)
					while (expanded.hasMoreElements())
						expandPath((TreePath) expanded.nextElement());
				selectionModel.setSelectionPath(selectionPath);
				expandedTreePath = simpleTreeModel.getTreePath(rootXMLTreeNode);
			}
	}

// private MouseEvent lastMouseEvent;
// event listener methods
	protected CompleteMouseAdapter createXMLTreeMouseListener() {
		return new CompleteMouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				if (!isEnabled()) return;
                if (SwingUtils.isMiddleMouseButton(evt)) // scroll to selected TreeNode on Middle-Mousebutton-Click
                    handleMiddleMouseButtonClicked(evt);
                else {
					if (currentlyHoveredXMLTreeNode != null) {
						TreePath treePath = simpleTreeModel.getTreePath(currentlyHoveredXMLTreeNode);
						selectionModel.setSelectionPath(treePath);
						ensureTreePathIsVisible(treePath); // don't need scrollPathToVisible(treePath); because parent of treePath is surely expanded
					}
					if (SwingUtilities.isLeftMouseButton(evt)) {
						handleLeftMouseButtonClick(evt);
						if (evt.getClickCount() == 2) // is this a double click?
							handleDoubleClickOnTreeNode();
					}
				}
				requestFocusInWindow();
			}
			public void mouseExited(MouseEvent evt) {
				if (currentlyHoveredXMLTreeNode != null) {
                    lastXMLTreeNode = currentlyHoveredXMLTreeNode;
                    currentlyHoveredXMLTreeNode = null;
				}
				hoveredTreeNodeChanged();
			}
            public void mouseEntered(MouseEvent evt) {
                lastXMLTreeNode = null;
				hoveredTreeNodeChanged();
			// mouseMoved(evt);
            }
			public void mouseMoved(MouseEvent evt) {
				if (!isEnabled()) return;
				TreePath path = getClosestPathForLocation(evt.getX(), evt.getY());
				if (path != null)
					currentlyHoveredXMLTreeNode = (XMLTreeNode) path.getLastPathComponent();
				else
					currentlyHoveredXMLTreeNode = null;
				if (lastXMLTreeNode != currentlyHoveredXMLTreeNode)  {
					hoveredTreeNodeChanged();
					lastXMLTreeNode = currentlyHoveredXMLTreeNode;
				}
			// lastMouseEvent = evt;
			}
			public void mouseWheelMoved(MouseWheelEvent evt) {
				mouseMoved(evt);
			}
		};
	}

	protected void handleLeftMouseButtonClick(MouseEvent evt) { }

	protected void handleMiddleMouseButtonClicked(MouseEvent evt) {
		centerTreePath(selectionModel.getSelectionPath());
	}

	protected void handleDoubleClickOnTreeNode() {
        TreePath treePath = currentlyHoveredXMLTreeNode.getTreePathTo(rootXMLTreeNode); //simpleTreeModel.getTreePath(currentlyHoveredXMLTreeNode);
        if (isExpanded(treePath))  // doesn't work if rootTreeNode has been set to another node
            collapseAllDescendants(currentlyHoveredXMLTreeNode, treePath, false);
        else
            expandAllDescendants(currentlyHoveredXMLTreeNode, treePath);
    }


    protected void hoveredTreeNodeChanged() {
		if (rollover)
			repaint();  // rollover
	}

	protected TreeCellRenderer createXMLTreeCellRenderer() {
		simpleXMLTreeCellRenderer = new SimpleXMLTreeCellRenderer();
		return simpleXMLTreeCellRenderer;
	}

	public final void centerTreePath(TreePath path) {
		if (path != null) {
			TreePath parentPath = path.getParentPath();
			if (parentPath != null) // Only expand if not leaf!
				if (!treeModel.isLeaf(parentPath.getLastPathComponent()))
					setExpandedState(parentPath, true);
			Rectangle bounds = ((TreeUI)ui).getPathBounds(this, path);
			if (bounds != null) {
				Rectangle vis = getVisibleRect();
				if (bounds.x > (vis.width - 50)) bounds.x -= (vis.width - 50);
				else bounds.x = 0;
				bounds.y = Math.max(0, bounds.y - (vis.height >> 1));
				bounds.width = vis.width;
				bounds.height = vis.height;

				if (scrollSmoothly) {
					JViewport viewPort = ((JViewport) getParent());
					Point viewPos = viewPort.getViewPosition();
					Rectangle visRect = viewPort.getViewRect();
					int targetY = bounds.y;
					int startY = viewPos.y;
				//	int amount = getScrollableUnitIncrement(visRect, SwingConstants.VERTICAL, targetY - startY);
					int amount = getScrollableBlockIncrement(visRect, SwingConstants.VERTICAL, targetY - startY) / 2;
				//	System.out.println("scroll from "+ startY + " to " + targetY + ", step="+ amount);
					bounds.y = startY;
					if (startY < targetY)
						while (bounds.y + amount < targetY) {
							bounds.y += amount;
							scrollRectToVisible(bounds);
							try { Thread.sleep(1); } catch (InterruptedException e) { }
						}
					else
						while (bounds.y - amount > targetY) {
							bounds.y -= amount;
							scrollRectToVisible(bounds);
							try { Thread.sleep(1); } catch (InterruptedException e) { }
						}
					bounds.y = targetY;
					scrollRectToVisible(bounds);
				} else
					scrollRectToVisible(bounds);
			}
		}
	}

	public void scrollToTreeNode(TreeNode treeNode) {
		centerTreePath(simpleTreeModel.getTreePath(treeNode));
	}

	public final boolean isSelectionEmpty() {
		return (selectionModel.getSelectionPath() == null);
	}

	public XMLTreeNode getSelectedTreeNode() {
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath == null) return null;
		return (XMLTreeNode) selPath.getLastPathComponent();
	}

	public void setSelectedTreeNode(TreeNode treeNode) {
		selectionModel.setSelectionPath(simpleTreeModel.getTreePath(treeNode));
	}

	public void centerAndSelectTreePath(final TreePath treePath) {
		if (treePath == null) return;
		selectionModel.setSelectionPath(treePath);
		centerTreePath(treePath);
	}
	public void centerAndSelectTreeNode(TreeNode treeNode) {
		centerAndSelectTreePath(simpleTreeModel.getTreePath(treeNode));
	}

	public final XMLTreeNode getTreeNode4XMLNode(Node xmlNode) {
		return xmlNode2TreeNodeMap.get(xmlNode);
	}

	public void gotoParentNode() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		centerAndSelectTreeNode(selectedTreeNode.getParent());
	}
	public void gotoPreviousSiblingAction() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		TreeNode parentTreeNode = selectedTreeNode.getParent();
		int index = parentTreeNode.getIndex(selectedTreeNode);
		if (index == 0) index = parentTreeNode.getChildCount();
		centerAndSelectTreeNode(parentTreeNode.getChildAt(index - 1));
	}
	public void gotoNextSiblingAction() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		TreeNode parentTreeNode = selectedTreeNode.getParent();
		int index = parentTreeNode.getIndex(selectedTreeNode);
		if (index == parentTreeNode.getChildCount() - 1) index = -1;
		centerAndSelectTreeNode(parentTreeNode.getChildAt(index + 1));
	}
	public void gotoFirstSiblingAction() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		TreeNode parentTreeNode = selectedTreeNode.getParent();
		centerAndSelectTreeNode(parentTreeNode.getChildAt(0));
	}
	public void gotoLastSiblingAction() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		TreeNode parentTreeNode = selectedTreeNode.getParent();
		centerAndSelectTreeNode(parentTreeNode.getChildAt(parentTreeNode.getChildCount() - 1));
	}

	public void nodeChanged(XMLTreeNode treeNode) {
		simpleTreeModel.nodeChanged(treeNode);
	}

	public String getToolTipText(MouseEvent event) {
        try {
            if (event != null) {
                Point p = event.getPoint();
                int selRow = getRowForLocation(p.x, p.y);
                TreeCellRenderer renderer = getCellRenderer();
                if (selRow != -1 && renderer != null) {
                    TreePath treePath = getPathForRow(selRow);
                    XMLTreeNode xmlTreeNode = (XMLTreeNode) treePath.getLastPathComponent();

                    if (!xmlTreeNode.getAttributesTextTruncated()) {
						if (xmlTreeNode.getErrorWrapper() != null)
							return xmlTreeNode.getErrorWrapper().message;

						JLabel label = (JLabel) renderer.getTreeCellRendererComponent(this, xmlTreeNode, isRowSelected(selRow),
                                        isExpanded(selRow), getModel().isLeaf(xmlTreeNode), selRow, true);

                        JViewport viewPort = ((JViewport) getParent());
                        Point viewPos = viewPort.getViewPosition();
                        JScrollPane scrollpane = (JScrollPane) viewPort.getParent();
                        Rectangle pathBounds = getPathBounds(treePath);
                        if (pathBounds.x + label.getPreferredSize().width - viewPos.x < scrollpane.getSize().width)
                            return null;
                        if (xmlTreeNode.getStyleInfos() == null)
							return label.getText();
                    }

					return getToolTipTextForTreeNode(xmlTreeNode, showAttributes, showChildCount);
				}
            }
            return null;
        } catch (Exception e) {     // should not happen, but since it happened already...
            e.printStackTrace();
            return null;
        }
    }

	public static String getToolTipTextForTreeNode(XMLTreeNode xmlTreeNode, boolean showAttributes, boolean showChildCount) {
	//	return xmlTreeNode.getLabel();
		StyleInfo[] styleInfos = xmlTreeNode.createStyleInfos(false);
		StringBuffer sb = new StringBuffer("<html>");
		for (int i=0; i< styleInfos.length; i++)
			if ((styleInfos[i] != null) &&
				(showChildCount || !styleInfos[i].isChildCount()) &&
				(showAttributes || !styleInfos[i].isAttribute())) {
			sb.append("<font style=\"color:").append(SwingUtils.colorToHex(styleInfos[i].getForeground(false)));
			if (styleInfos[i].getFont().isBold()) sb.append(";font-weight:bold");
			if (styleInfos[i].getFont().isItalic()) sb.append(";font-style:italic");
			sb.append("\">");
			sb.append(MiscUtils.wrapString(styleInfos[i].getText(), "<br>&nbsp;&nbsp;&nbsp;", 100, 0));
			sb.append("</font>");
		}
		return sb.toString();
	}


	public void ensureTreePathIsVisible(TreePath treePath) {
		Rectangle scrollBounds = getPathBounds(treePath);
		// System.out.println("scrollBounds = " + scrollBounds);
		scrollBounds.width = 50;
	// if (scrollBounds != null)
		scrollRectToVisible(scrollBounds);
	}
}

