/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 19, 2002
 * Time: 2:59:27 AM
 */
package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosListItem;
import at.ac.tuwien.asgaard.delta.gui.xml.images.XMLImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.SwingUtils;

import org.w3c.dom.Node;

import java.awt.*;

import javax.swing.*;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;
import javax.swing.tree.TreeCellRenderer;

/**
 * Class SimpleXMLTreeCellRenderer. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
class SimpleXMLTreeCellRenderer extends JLabel implements TreeCellRenderer {
	protected boolean showChildCount, showAttributes; //  set from properties
	protected Color rolloverColor;
	protected static final Color DEFAULT_SELECTION_BACKGROUND = UIManager.getColor("Tree.selectionBackground");
	protected Color backgroundColor;
	protected static final Font DEFAULT_FONT = GlobalObjects.PLAIN_FIXED_FONT;
	protected XMLTreeNode xmlTreeNode;
	protected int rowOfFocusedTreeNode, rowOfCurrentPaintedTreeNode;
	private static final float MAX_FONT_SIZE = GlobalObjects.BOLD_FIXED_FONT.getSize2D(); // + (float)2.0;
	private static final float MIN_FONT_SIZE = GlobalObjects.BOLD_FIXED_FONT.getSize2D() * (float)0.5;

	protected SimpleXMLTreeCellRenderer() {
		super();
		showChildCount = showAttributes = true;

		textSelectionColor = UIManager.getColor("Tree.selectionForeground");
		textNonSelectionColor = UIManager.getColor("Tree.textForeground");
		backgroundSelectionColor = DEFAULT_SELECTION_BACKGROUND;
	// backgroundNonSelectionColor = UIManager.getColor("Tree.textBackground");
		borderSelectionColor = Color.BLACK; // UIManager.getColor("Tree.selectionBorderColor");
		Object value = UIManager.get("Tree.drawsFocusBorderAroundIcon");
		drawsFocusBorderAroundIcon = (value != null && ((Boolean)value).booleanValue());
		rolloverColor = new Color(200, 188, 212);
		rowOfFocusedTreeNode = -1;
	}

	protected final Color getSelectionColor() {
		return backgroundSelectionColor;
	}

	protected final void setSelectionColor(Color selectionColor) {
		backgroundSelectionColor = selectionColor;
	}

	protected final boolean isShowAttributes() {
		return showAttributes;
	}

	protected final void setShowAttributes(boolean showAttributes) {
		this.showAttributes = showAttributes;
	}

	protected final boolean isShowChildCount() {
		return showChildCount;
	}

	protected final void setShowChildCount(boolean showChildCount) {
		this.showChildCount = showChildCount;
	}

	protected final Color getRolloverColor() {
		return rolloverColor;
	}

	protected final void setRolloverColor(Color rolloverColor) {
		this.rolloverColor = rolloverColor;
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value,
	                                              boolean selected, boolean expanded, boolean leaf,
	                                              int row, boolean hasFocus) {
		this.tree = (SimpleXMLTree) tree;
		this.hasFocus = hasFocus;
		this.selected = selected;
		rowOfCurrentPaintedTreeNode = row;
		setFont(DEFAULT_FONT);

		if (selected) {
			setForeground(textSelectionColor);
			backgroundColor = backgroundSelectionColor;
		} else {
			setForeground(textNonSelectionColor);
		// backgroundNonSelectionColor = (((SimpleXMLTree)tree).rollover && (value == ((SimpleXMLTree)tree).currentlyHoveredXMLTreeNode)) ? rolloverColor : tree.getBackground(); //GlobalConstants.TRANSPARENT;
			backgroundColor = tree.getBackground(); //backgroundNonSelectionColor;
		}

		setEnabled(tree.isEnabled());

		xmlTreeNode = (XMLTreeNode) value;
		if (xmlTreeNode.getClipBoardMode() != null)
			backgroundColor = SwingUtils.changeColor(backgroundColor, -20); // todo get from properties

		if (xmlTreeNode.getUserObject() == null) {
			if (xmlTreeNode.getStyleInfos() == null) {
				int childCount;
				if (showChildCount && ((childCount = xmlTreeNode.getChildCount()) > 0))
					setText(xmlTreeNode.getLabel() + " (" + childCount + ")");
				else
					setText(xmlTreeNode.getLabel());
			} else setText(null);
			if (xmlTreeNode.getXMLNode() != null)
				switch (xmlTreeNode.getXMLNode().getNodeType()) {
					case Node.DOCUMENT_NODE:
						setIcon(XMLImages.DOC_NODE_ICON);
						break;
					case Node.DOCUMENT_TYPE_NODE:
						setIcon(XMLImages.DOCTYPE_NODE_ICON);
						break;
					case Node.ELEMENT_NODE:
						if (xmlTreeNode.isDocumentElement())
							setIcon(XMLImages.OPENED_ROOT_NODE_ICON);
						else if (xmlTreeNode.getDELTALinkID() == LinkIDConstants.NO_LINK)
							setIcon(XMLImages.ELEMENT_ICON);
						else
							setIcon(XMLImages.LINK_ELEMENT_ICON);
						break;
					case Node.TEXT_NODE:
						setIcon(XMLImages.TEXT_NODE_ICON);
						break;
					case Node.CDATA_SECTION_NODE:
						setIcon(XMLImages.CDATA_NODE_ICON);
						setForeground(Color.gray);
						break;
					case Node.COMMENT_NODE:
						setIcon(XMLImages.COMMENT_NODE_ICON);
						setForeground(Color.gray);
						setFont(StyleInfo.COMMENT_FONT);
						break;
					case Node.PROCESSING_INSTRUCTION_NODE:
						setIcon(XMLImages.PI_NODE_ICON);
						setForeground(Color.gray);
						break;
				}
		} else {
			MacrosListItem mli = (MacrosListItem) xmlTreeNode.getUserObject();
			setText("<html>" + mli.getShortHTMLDescription()); // also <html> to have the same font
			setToolTipText(mli.getToolTipText());
			setIcon(XMLImages.OPENED_ROOT_NODE_ICON);
		}
		return this;
	}

	/**
	 * Paints the value.  The background is filled based on selected.
	 */
	public void paint(Graphics g) {
		Graphics2D g2d = (Graphics2D) g;
	//	g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		Dimension size = getPreferredSize();
		int imageOffset = 19;
		g.setColor( (((SimpleXMLTree) tree).rollover && (xmlTreeNode == ((SimpleXMLTree) tree).currentlyHoveredXMLTreeNode)) ?  SwingUtils.changeColor(backgroundColor, -15) : backgroundColor );
		g.fillRect(imageOffset, 0, size.width - 1 - imageOffset, size.height);
	//	g.fillRect(imageOffset, 0, getWidth() - 1 - imageOffset, getHeight());
		if (hasFocus) {
			if (drawsFocusBorderAroundIcon)
				imageOffset = 0;
			if (borderSelectionColor != null) {
				g.setColor(borderSelectionColor);
				g.drawRect(imageOffset, 0, size.width - 1 - imageOffset, size.height - 1);
			}
		}

		if ((getText() != null) || (xmlTreeNode.getStyleInfos() == null))//|| !isEnabled()) // (xmlTreeNode == null) ||
			super.paint(g);
		else {
			Graphics scratchGraphics = g.create();
			try {
				StyleInfo[] styleInfos = xmlTreeNode.getStyleInfos();
				Icon icon = (isEnabled()) ? getIcon() : getDisabledIcon();
				icon.paintIcon(this, scratchGraphics, 0, 0);

				float fontSize = getFontSize();
				int textY = (int) fontSize, //StyleInfo.BOLD_FONT.getSize(), //
						textX = 22; // = 16 + 4 = getIcon().getIconWidth() + getIconTextGap();
				for (int i = 0; i < styleInfos.length; i++) {
					StyleInfo styleInfo = styleInfos[i];
					if ((styleInfo != null) && (showChildCount || !styleInfo.isChildCount()) && (showAttributes || !styleInfo.isAttribute())) {
						Font font = styleInfo.getFont().deriveFont(fontSize); //...
						scratchGraphics.setFont(font);
						scratchGraphics.setColor(styleInfo.getForeground(selected));
						scratchGraphics.drawString(styleInfo.getText(), textX, textY);
						textX += getFontMetrics(font).stringWidth(styleInfo.getText()); //styleInfo.getWidth();//
					}
				}
			} finally {
				scratchGraphics.dispose();
			}
			if (xmlTreeNode.getClipBoardMode() == MainController.XMLClipBoard.ClipBoardMode.CUT) {
				g.setColor(Color.BLACK);
				g.drawLine(imageOffset, getHeight() / 2, size.width, size.height / 2);
			}
		}
	}

	private float getFontSize() {
		return ((rowOfFocusedTreeNode < 0) ? GlobalObjects.BOLD_FIXED_FONT.getSize() : (Math.max(MIN_FONT_SIZE, MAX_FONT_SIZE - ((float) Math.abs(rowOfFocusedTreeNode - rowOfCurrentPaintedTreeNode) / (float) 3.0))));
	}
	private Dimension DIMENSION = new Dimension(0, StyleInfo.getBoldFontHeight());
	/**
	 * Overrides <code>JComponent.getPreferredSize</code> to
	 * return slightly wider preferred size value.
	 */
	public final Dimension getPreferredSize() {
		StyleInfo[] styleInfos = xmlTreeNode.getStyleInfos();
		if (styleInfos == null) {
			Dimension retDimension = super.getPreferredSize();
			if (retDimension != null) retDimension.width += 3;
			return retDimension;
		} else {
			float fontSize = getFontSize();
			DIMENSION.width = 24; //getIcon().getIconWidth() + getIconTextGap() + 3;
			for (int i = 0; i < styleInfos.length; i++)
				if ((styleInfos[i] != null) &&
						(showChildCount || !styleInfos[i].isChildCount()) &&
						(showAttributes || !styleInfos[i].isAttribute())) {
					Font font = styleInfos[i].getFont().deriveFont(fontSize);
					DIMENSION.width += getFontMetrics(font).stringWidth(styleInfos[i].getText()); //styleInfos[i].getWidth();
				}
			return DIMENSION;
		}
	}
	/**
	 * @return getPreferredSize(c)
	 */
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}
	/**
	 * @return getPreferredSize(c)
	 */
	public Dimension getMaximumSize() {
		return getPreferredSize();
	}

	/**
	 * Subclassed to map <code>FontUIResource</code>s to null. If
	 * <code>font</code> is null, or a <code>FontUIResource</code>, this
	 * has the effect of letting the font of the JTree show
	 * through. On the other hand, if <code>font</code> is non-null, and not
	 * a <code>FontUIResource</code>, the font becomes <code>font</code>.
	 */
	public final void setFont(Font font) {
		if (font instanceof FontUIResource)
			font = null;
		super.setFont(font);
	}

	/**
	 * Gets the font of this component.
	 * @return this component's font; if a font has not been set
	 * for this component, the font of its parent is returned
	 */
	public final Font getFont() {
		Font font = super.getFont();

		if (font == null && tree != null) {
			// Strive to return a non-null value, otherwise the html support
			// will typically pick up the wrong font in certain situations.
			font = tree.getFont();
		}
		return font;
	}

	/**
	 * Subclassed to map <code>ColorUIResource</code>s to null. If
	 * <code>color</code> is null, or a <code>ColorUIResource</code>, this
	 * has the effect of letting the background color of the JTree show
	 * through. On the other hand, if <code>color</code> is non-null, and not
	 * a <code>ColorUIResource</code>, the background becomes
	 * <code>color</code>.
	 */
	public final void setBackground(Color color) {
		if (color instanceof ColorUIResource)
			color = null;
		super.setBackground(color);
	}

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void validate() { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void revalidate() { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void repaint(long tm, int x, int y, int width, int height) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void repaint(Rectangle r) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	protected final void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		// Strings get interned...
		if (propertyName.equals("text"))
			super.firePropertyChange(propertyName, oldValue, newValue);
	}

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, byte oldValue, byte newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, char oldValue, char newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, short oldValue, short newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, int oldValue, int newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, long oldValue, long newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, float oldValue, float newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, double oldValue, double newValue) { }

	/**
	 * Overridden for performance reasons.
	 * See the <a href="#override">Implementation Note</a>
	 * for more information.
	 */
	public final void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) { }

	/** Last tree the renderer was painted in. */
	protected SimpleXMLTree tree;

	/** Is the value currently selected. */
	protected boolean selected;
	/** True if has focus. */
	protected boolean hasFocus;
	/** True if draws focus border around icon as well. */
	protected final boolean drawsFocusBorderAroundIcon;

	// Colors
	/** Color to use for the foreground for selected nodes. */
	protected final Color textSelectionColor;
	/** Color to use for the foreground for non-selected nodes. */
	protected final Color textNonSelectionColor;
	/** Color to use for the background when a node is selected. */
	protected Color backgroundSelectionColor;
	/** Color to use for the background when the node isn't selected. */
	protected Color backgroundNonSelectionColor;
	/** Color to use for the focus indicator when the node has focus. */
	protected final Color borderSelectionColor;

	public int getRowOfFocusedTreeNode() {
		return rowOfFocusedTreeNode;
	}

	public void setRowOfFocusedTreeNode(int rowOfFocusedTreeNode) {
		this.rowOfFocusedTreeNode = rowOfFocusedTreeNode;
	}
}

