package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.SwingUtils;

import java.awt.*;

public class StyleInfo {
	public static final Color ELEMENT_COLOR = Color.BLACK,
			ATTRS_COLOR = Color.DARK_GRAY, SYMBOL_COLOR = Color.GRAY;
	private static final Component DUMMY_COMP = new Label();
	protected static final FontMetrics //PLAIN_FONTMETRICS = DUMMY_COMP.getFontMetrics(PLAIN_FONT),
			BOLD_FONTMETRICS = DUMMY_COMP.getFontMetrics(GlobalObjects.BOLD_FIXED_FONT);
//			ITALIC_FONTMETRICS = DUMMY_COMP.getFontMetrics(ITALIC_FONT),
//			BOLD_ITALIC_FONTMETRICS = DUMMY_COMP.getFontMetrics(BOLD_ITALIC_FONT);

//    public static final StyleInfo TAG_OPEN = new StyleInfo("<", SYMBOL_COLOR, PLAIN_FONT);
//    public static final StyleInfo TAG_CLOSE = new StyleInfo(">", SYMBOL_COLOR, PLAIN_FONT);

	private final int type;

	public int getType() {
		return type;
	}
	public boolean isAttribute() {
		return type == 1;
	}
	public boolean isChildCount() {
		return type == 2;
	}

	private final Font font;
	private final String text;
	private final Color foreground, foregroundSelected; //, background;

	private static Color colorLink, colorError, colorWarning, colorSummarizeText;

	protected static final Font COMMENT_FONT = GlobalObjects.PLAIN_FIXED_FONT.deriveFont(Font.ITALIC, GlobalObjects.DEFAULT_FONT_SIZE - 2);

	// todo : get all colors from preferences !

	private static Preferences properties;
	public static void fetchProperties(Preferences properties) {
		StyleInfo.properties = properties;
		colorLink = properties.getColor(Preferences.KEY_COLOR_LINK_FG);
		colorSummarizeText = properties.getColor(Preferences.KEY_COLOR_XML_SUMMARIZE);
		colorError = properties.getColor(Preferences.KEY_COLOR_XML_INVALID_NODES_ON_LOAD);
		colorWarning = SwingUtils.changeColor(colorError, +10);
	}

	public static StyleInfo createElementStyle(String text) {
		return new StyleInfo(text, ELEMENT_COLOR, GlobalObjects.BOLD_FIXED_FONT, 0);
	}
	public static StyleInfo createLinkElementStyle(String text) {
		return new StyleInfo(text, null, GlobalObjects.BOLD_FIXED_FONT, 0) {
			public Color getForeground(boolean selected) {
				return selected ? Color.BLACK : colorLink;
			}
		};
	}
	public static StyleInfo createRootElementStyle(String text) {
		return new StyleInfo(text, ELEMENT_COLOR, GlobalObjects.BOLD_FIXED_FONT, 0); // todo: other style for root-element?
	}
	public static StyleInfo createElementErrorStyle(String text) {
		return new StyleInfo(text, null, GlobalObjects.BOLD_ITALIC_FIXED_FONT, 0) {
			public Color getForeground(boolean selected) {
				return selected ? Color.BLACK : colorError;
			}
		};
	}
	public static StyleInfo createElementWarningStyle(String text) {
		return new StyleInfo(text, null, GlobalObjects.BOLD_ITALIC_FIXED_FONT, 0) {
			public Color getForeground(boolean selected) {
				return selected ? Color.BLACK : colorWarning;
			}
		};
	}
	public static StyleInfo createAttributeNameStyle(String text) {
		return new StyleInfo(text, ATTRS_COLOR, GlobalObjects.PLAIN_FIXED_FONT, 1);
	}
	public static StyleInfo createAttributeValueStyle(String text) {
		return new StyleInfo(text, ATTRS_COLOR, GlobalObjects.BOLD_ITALIC_FIXED_FONT, 1);
	}
	public static StyleInfo createAttributeSymbolStyle(String text) {
		return new StyleInfo(text, SYMBOL_COLOR, GlobalObjects.PLAIN_FIXED_FONT, 1);
	}
	public static StyleInfo createChildCountStyle(int childCount) {
		return new StyleInfo(((childCount == 0) ? "" : " ("+ childCount+")"), Color.GRAY, GlobalObjects.BOLD_FIXED_FONT, 2);
	}
	public static StyleInfo createSummarizeStyle(String info) {
		return new StyleInfo(" -- "+info, null, GlobalObjects.PLAIN_FIXED_FONT, 0) {
			public Color getForeground(boolean selected) {
				return selected ? Color.BLACK : colorSummarizeText;
			}
		};
	}
	public static StyleInfo createCommentStyle(String text) {
		return new StyleInfo(text, Color.GRAY, COMMENT_FONT, 0);
	}
	public static StyleInfo createDocumentStyle(String text) {
		return new StyleInfo(text, Color.BLACK, GlobalObjects.PLAIN_FIXED_FONT, 0);
	}

	private StyleInfo(String text, Color foreground, Font font, int type) {
		this(text, foreground, foreground, font, type);
	}

	private StyleInfo(String text, Color foreground, Color foregroundSelected, Font font, int type) {
		this.text = text;
		this.foreground = foreground;
		this.foregroundSelected = foregroundSelected;
		this.font = font;
		this.type = type;
	}

	public String getText() {
		return text;
	}

	public Color getForeground(boolean selected) {
		return selected ? foregroundSelected : foreground;
	}

	public Font getFont() {
		return font;
	}

//	public int getWidth() {
//		return fontMetrics.stringWidth(text);
//	}
//	public int getHeight() {
//		return fontMetrics.getHeight();
//	}
	public static int getBoldFontHeight() {
		return BOLD_FONTMETRICS.getHeight();
	}
}
