/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */
package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.swing.TextEditDialog;

import javax.swing.*;
import javax.swing.border.Border;
import javax.swing.table.TableCellEditor;
import java.awt.*;
import java.awt.event.*;

/**
 * Class TableCellEditorWithEditButton ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class TableCellEditorWithEditButton extends AbstractCellEditor implements TableCellEditor {
	private final JPanel editorComponent;
	private final CellTextField textField;
	public TableCellEditorWithEditButton(Icon buttonIcon, String buttonToolTip, final Component table, Border cellBorder) {
		editorComponent = new JPanel(new BorderLayout());
		textField = new CellTextField();
		textField.setBorder(null);
		textField.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) { // todo: enter in textfield doesn't work
				stopCellEditing();
				table.requestFocusInWindow();
			}
		});
		textField.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
					cancelCellEditing();
					table.requestFocusInWindow();
				}
			}
		});
		/*textField.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				stopCellEditing();
				table.requestFocusInWindow();
			}
		});*/
		JButton button = new JButton(buttonIcon);
		button.setMargin(new Insets(0, 0, 0, 0));
		button.setSize(25, 20);
		button.setToolTipText(buttonToolTip);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				String newValue = showEditDialog("Edit attribute value...", textField.getText());
				if (newValue != null) {
					textField.setText(newValue);
					stopCellEditing();
				} else
					cancelCellEditing();
				table.requestFocusInWindow();
			}
		});
		editorComponent.add(textField, BorderLayout.CENTER);
		editorComponent.add(button, BorderLayout.EAST);
		if (cellBorder != null)
			textField.setBorder(cellBorder);
	}
	public Component getTableCellEditorComponent(JTable table, Object value,
												 boolean isSelected, int row, int column) {
		textField.setText((String) value);
		textField.requestFocusInWindow();
	//	editorComponent.revalidate();
	//	editorComponent.repaint();
		return editorComponent;
	}
	public Object getCellEditorValue() {
		return textField.getText();
	}

	protected String showEditDialog(String title, String initialValue) {
		return getTextEditDialog(null, title).showEditDialog(initialValue);
	}
	private TextEditDialog editDialog;
	protected TextEditDialog getTextEditDialog(Component parentComponent, String title) {
		if (editDialog == null)
			editDialog = new TextEditDialog(parentComponent, title);
		return editDialog;
	}

	private static final class CellTextField extends JTextField {
		public boolean processKeyBinding(KeyStroke ks, KeyEvent e, int condition, boolean pressed) {
			return super.processKeyBinding(ks, e, condition, pressed);
		}
	}
}