package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;

import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.BorderLayout;
import java.awt.Component;
import java.util.*;

/**
 * Class XMLAttributesInputDialog. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
final class XMLAttributesInputDialog extends AbstractInputDialog {
	final XMLView xmlTree;
	final XMLAttributesTable table;
	final XMLAttributesTable xmlAttributesTable;

	XMLAttributesInputDialog(Preferences properties, DTDHelper dtdHelper, XMLView xmlTree, XMLAttributesTable xmlAttributesTable) {
		super(xmlTree, null, false, true, 420, 300, 300, 120, 0, 0);

		table = new XMLAttributesTable(properties, dtdHelper, false);
        table.setXmlView(xmlTree);
		contentPane.add(new JScrollPane(table), BorderLayout.CENTER);

		this.xmlTree = xmlTree;
		this.xmlAttributesTable = xmlAttributesTable;
	}

	protected Component createMainComponent() {
		return null;
	}

	public List<String> showDialog(final List nodeList, String title, boolean showElementColumn) {
		if ((nodeList == null) || nodeList.isEmpty() || (title == null))
			return null;

		int length = nodeList.size();

		Map elementsNumber = null;
		if (showElementColumn) {
			elementsNumber = new HashMap();
			Element oldElement = null;
			boolean equals = true;
			for (int i = 0; i < length; i++) {
				XMLView.AttributeInputWrapper wrapper = (XMLView.AttributeInputWrapper)nodeList.get(i);
				Element element = wrapper.element;
				if ((oldElement !=  null) && (element != oldElement)) equals = false;
				oldElement = element;
				String elementName = element.getNodeName();
				Map map = (HashMap)elementsNumber.get(elementName);
				if (map == null) {
					map = new HashMap();
					elementsNumber.put(elementName, map);
				}
				if (!map.containsKey(element))  // don't overwrite if one element has two or more attributes with %INPUT%
					map.put(element, new Integer(map.size()+1));
			}
			if (equals)
				showElementColumn = false;
		}

		ArrayList tableData = new ArrayList(length);
		for (int i = 0; i < length; i++) {
			XMLView.AttributeInputWrapper wrapper = (XMLView.AttributeInputWrapper)nodeList.get(i);
			Attr attribute = wrapper.attribute;
			if (showElementColumn) {
				Element element = wrapper.element;
				String elementName = element.getNodeName();
				HashMap map = (HashMap)elementsNumber.get(elementName);
				if (map.size() > 1) {
					Integer nr = (Integer)map.get(element);
					elementName = elementName + " (" + nr + ")";
				}
				tableData.add(new Object[]{elementName, attribute.getName(), attribute.getValue()});
			} else
				tableData.add(new Object[]{attribute.getName(), attribute.getValue()});
		}

		final String elementName = (showElementColumn ? null : ((XMLView.AttributeInputWrapper)nodeList.get(0)).element.getNodeName());
		final boolean _showElementColumn = showElementColumn;
		AbstractAttributesTableModel tableModel = new AbstractAttributesTableModel(tableData,  // todo sort tableData
				                                                (showElementColumn ? new String[]{"  Element  ", "  Attribute  ", "Value"} : new String[]{"Name", "Value"})) {
			Attr getAttribute(int rowIndex) {
				return ((XMLView.AttributeInputWrapper)nodeList.get(rowIndex)).attribute;
			}
			String getElementName(int rowIndex) {
				return (_showElementColumn ? ((XMLView.AttributeInputWrapper)nodeList.get(rowIndex)).element.getNodeName() : elementName);
			}
		};

		table.setModel(tableModel);
		ListSelectionListener listSelectionListener = new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent evt) {
				XMLView.AttributeInputWrapper wrapper = (XMLView.AttributeInputWrapper)nodeList.get(table.getSelectedRow());
				if (wrapper.treePath != null)
					xmlTree.centerAndSelectTreePath(wrapper.treePath);
				xmlAttributesTable.scrollToAndSelectRow(wrapper.attributeIndex);
			}
		};
		table.addSelectionListener(listSelectionListener);

		table.setSelectedRow(0);
        table.getColumnModel().getSelectionModel().setLeadSelectionIndex(showElementColumn ? 2 : 1);
		table.editCellAt(0, showElementColumn ? 2 : 1);

		setTitle(title);

		boolean ret = showDialog();

		table.removeSelectionListener(listSelectionListener);
		if (ret) {
			List<String> attributeValues = new LinkedList<String>();
			int valueColumn = (showElementColumn ? 2 : 1);
			int rows = tableModel.getRowCount();
			for (int i=0; i<rows; i++)
				attributeValues.add((String)tableModel.getValueAt(i, valueColumn));
			return attributeValues;
		} else
			return null;
	}

	public void resetProperties() {
		table.refetchProperties();
	}
}
