/*
 * XMLAttributesTabel.java
 *
 * Created on April 26, 2002, 1:29 PM
 */

package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideController;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.MainFrame;
import at.ac.tuwien.asgaard.delta.gui.actions.EmptyAction;
import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.gui.misc.images.MiscImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.LinkedAttribute;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.LinkedAttributes;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.autoComplete.AutoCompleteComboBoxCellEditor;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;

import org.apache.xerces.dom.CoreDocumentImpl;
import org.apache.xerces.util.XMLChar;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.event.UndoableEditListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;

/**
 * XMLAttributesTable
 *
 * @author Peter Votruba
 * @version 2.7.3
 * @since JDK 1.4.0
 */
public final class XMLAttributesTable extends JTable {
    private final Preferences properties;
    private final boolean inMainWindow;
    XMLView xmlView;
    DTDHelper dtdHelper;
    XMLTreeNode xmlTreeNode;

    private AbstractAttributesTableModel attributesTableModel;
    protected XMLAttributesTableModel xmlAttributesTableModel;

    private boolean showEditButton;
    private TableCellEditor defaultCellEditor, bigStringCellEditor;
    private AutoCompleteComboBoxCellEditor enumCellEditor, linkedAttrCellEditor;
    private SpecialAttributeCellEditor specialAttributeValueTypeCellEditor;
    private TableCellRenderer tableCellRenderer;

    private final Map<LinkedAttribute, List> linkedAttributeTarget2XPathResultsMap, linkedAttributeSource2XPathResultsMap;

    private int columnCount, attrNameColumnIndex, attrValueColumnIndex;
//	private	static final Color enumValueColumnBackground		 = new Color(210, 210, 210);
//	private	static final Color enumValueColumnSelectedBackground = new Color(210, 210, 240);
//	private	static final Color elementColumnBackground			 = new Color(190, 190, 190);
//	private	static final Color elementColumnSelectedBackground	 = new Color(190, 190, 210);
    private Color selectionColor;

    private final Font DEFAULT_VALUE_FONT;
    private final Font REQUIRED_ATTR_FONT;
    private final Font FIXED_ATTR_FONT;

    public XMLAttributesTable(Preferences properties, final DTDHelper dtdHelper, boolean inMainWindow) {
        super(new XMLAttributesTableModel(), null, new SingleListSelectionModel());
        Log.trace("enter: XMLAttributesTable()");
        this.properties = properties;
        this.dtdHelper = dtdHelper;
        this.inMainWindow = inMainWindow;

        linkedAttributeTarget2XPathResultsMap = new HashMap<LinkedAttribute, List>();
        linkedAttributeSource2XPathResultsMap = new HashMap<LinkedAttribute, List>();
        dataModel.addTableModelListener(new TableModelListener() {
            public void tableChanged(TableModelEvent evt) {
                linkedAttributeTarget2XPathResultsMap.clear();
                linkedAttributeSource2XPathResultsMap.clear();
            }
        });

        columnModel.setSelectionModel(new SingleListSelectionModel());

        showEditButton = this.properties.getBoolean(Preferences.KEY_XML_TABLE_EDIT_BUTTON);
        setRowHeight(20); // rowHeight+4 - because the combobox-celleditor is bigger

        addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) {
                if (SwingUtilities.isRightMouseButton(evt)) {
                    int row = rowAtPoint(evt.getPoint());
                    setSelectedRow(row);
                }
            }

            public void mouseClicked(MouseEvent evt) {
                if (SwingUtilities.isLeftMouseButton(evt) && (evt.getClickCount() == 2)) { // getCellRect(row, 0, true).contains(evt.getX(), evt.getY())
                    jumpToLinkedElement();
                }
            }
        });

        putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);
        setSurrendersFocusOnKeystroke(true);
        putClientProperty("JTable.autoStartsEdit", Boolean.TRUE);
        // todo: Enter should not jump to next cell
        getActionMap().put("editCellAction", new AbstractAction() { // todo
            public void actionPerformed(ActionEvent e) {
                if (cellEditor != null)
                    cellEditor.stopCellEditing();
                else {
                    int col = getSelectedColumn();
                    if (col == attrValueColumnIndex)
                        editCellAt(selectionModel.getMinSelectionIndex(), col, null);
                }
            }
        });
        getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "editCellAction");

        // setBackground(GlobalConstants.DEFAULT_BACKGROUND);
        ToolTipManager.sharedInstance().unregisterComponent(tableHeader);

        DEFAULT_VALUE_FONT = getFont().deriveFont(Font.ITALIC);
        REQUIRED_ATTR_FONT = getFont().deriveFont(Font.BOLD);
        FIXED_ATTR_FONT = REQUIRED_ATTR_FONT.deriveFont(Font.ITALIC);
        selectionColor = properties.getColor(Preferences.KEY_COLOR_SELECTION);
		setBackground(GlobalObjects.DEFAULT_BACKGROUND);
		setFocusTraversalKeysEnabled(false);

        Log.trace("exit: XMLAttributesTable()");
    }

    // todo chache linkedAttribute in getCellEditor(), only search for linkedAttribute if jumpToLinkedElement() was called on double-mouse-click
    private void jumpToLinkedElement() {
        int row = getSelectedRow();
        Attr attribute = attributesTableModel.getAttribute(row);
		String attrValue = attribute.getValue(); //dataModel.getValueAt(row, attrValueColumnIndex).toString();
		int attributeValueType = dtdHelper.getAttributeValueType(attribute);
        boolean foundTarget = false;
        Object source = null;
        if (MainController.getActiveSideController().currentXMLLanguage != null) {
            LinkedAttributes linkedAttributes = MainController.getActiveSideController().currentXMLLanguage.getLinkedAttributes();
            if (linkedAttributes != null) {
                List<LinkedAttribute> linkedAttributeList = linkedAttributes.getLinkedAttributeList();
                for (LinkedAttribute linkedAttribute : linkedAttributeList) {
                    List targets = linkedAttributeTarget2XPathResultsMap.get(linkedAttribute);
                    if (targets == null) {
                        targets = XMLUtils.evaluateXPathSilently(linkedAttribute.getTargetAttributeXPath(), attribute.getOwnerDocument());
                        linkedAttributeTarget2XPathResultsMap.put(linkedAttribute, targets);
                    }
                    if (targets.contains(attribute)) {
                        foundTarget = true;
                        List values = linkedAttributeSource2XPathResultsMap.get(linkedAttribute);
                        if (values == null) {
                            values = XMLUtils.evaluateXPathSilently(linkedAttribute.getSourceXPath(), attribute.getOwnerElement());
                            linkedAttributeSource2XPathResultsMap.put(linkedAttribute, values);
                        }
                        for (Object item : values)
							if (item instanceof Attr) {
								Attr sourceAttr = (Attr) item;
								if (sourceAttr.getNodeValue().equals(attrValue)) {
									XMLTreeNode treeNode = xmlView.getTreeNode4XMLNode(sourceAttr.getOwnerElement());
									if (treeNode == null)
										JOptionPane.showMessageDialog(this, "Corresponding element is currently hidden.",
																	  "Jump to corresponding element", JOptionPane.INFORMATION_MESSAGE);
									else
										xmlView.centerAndSelectTreeNode(treeNode);
									source = sourceAttr;
									break;
								}
							} else if (attrValue.equals(item))
								source = item;
                        if (source != null) break;
                    }
                }
                if (foundTarget && !(source instanceof Attr))
                    if (source == null)
                        JOptionPane.showMessageDialog(this, "Corresponding element was not found.",
                                "Cannot jump to corresponding element", JOptionPane.INFORMATION_MESSAGE);
                    else
                        JOptionPane.showMessageDialog(this, "Attribute value was found in list of pre-defined values.",
                                "Cannot jump to corresponding element", JOptionPane.INFORMATION_MESSAGE);
            }
        }
        if (!foundTarget) {
            if ((attributeValueType == DTDHelper.IDREF) || (attributeValueType == DTDHelper.IDREFS)) {
                Element idElement = ((CoreDocumentImpl) attribute.getOwnerDocument()).getIdentifier(attrValue);
                if (idElement == null)
                    JOptionPane.showMessageDialog(this, "Element with matching ID attribute was not found!",
                            "Cannot jump to corresponding element", JOptionPane.WARNING_MESSAGE);
                else {
                    XMLTreeNode treeNode = xmlView.getTreeNode4XMLNode(idElement);
                    if (treeNode == null)
                        JOptionPane.showMessageDialog(this, "Element with matching ID attribute is currently hidden.",
                                "Jump to corresponding element", JOptionPane.INFORMATION_MESSAGE);
                    else
                        xmlView.centerAndSelectTreeNode(treeNode);
                }
            }
        }
    }

    public void setModel(TableModel dataModel) {
        super.setModel(dataModel);
        attributesTableModel = (AbstractAttributesTableModel) dataModel;
        if (dataModel.getClass() == XMLAttributesTableModel.class) {
            xmlAttributesTableModel = (XMLAttributesTableModel) dataModel;
            xmlAttributesTableModel.postInit(this);
        } else
            xmlAttributesTableModel = null;
        columnCount = dataModel.getColumnCount();
        attrValueColumnIndex = columnCount - 1;
        attrNameColumnIndex = attrValueColumnIndex - 1;
    }

    public TableCellRenderer getCellRenderer(int row, int column) {
        if (tableCellRenderer == null)
            tableCellRenderer = new XMLAttributesTableCellRenderer();
        return tableCellRenderer;
    }

    public TableCellEditor getCellEditor(int row, int column) {
        String elementName = attributesTableModel.getElementName(row);
        Attr attribute = attributesTableModel.getAttribute(row);
        String attrName = attribute.getName();
        int attributeValueType = dtdHelper.getAttributeValueType(elementName, attrName);
        String[] comboBoxValues = null;

        if (MainController.getActiveSideController().currentXMLLanguage != null) {
            LinkedAttributes linkedAttributes = MainController.getActiveSideController().currentXMLLanguage.getLinkedAttributes();
            if (linkedAttributes != null) {
                List linkedAttributeList = linkedAttributes.getLinkedAttributeList();
                for (int i = 0, size = linkedAttributeList.size(); i < size; i++) {
                    LinkedAttribute linkedAttribute = (LinkedAttribute) linkedAttributeList.get(i);
                    List targets = linkedAttributeTarget2XPathResultsMap.get(linkedAttribute);
                    if (targets == null) {
                        targets = XMLUtils.evaluateXPathSilently(linkedAttribute.getTargetAttributeXPath(), attribute.getOwnerDocument());
                        linkedAttributeTarget2XPathResultsMap.put(linkedAttribute, targets);
                    }
                    if (targets.contains(attribute)) {
                        List values = linkedAttributeSource2XPathResultsMap.get(linkedAttribute);
                        if (values == null) {
                            values = XMLUtils.evaluateXPathSilently(linkedAttribute.getSourceXPath(), attribute.getOwnerElement());
                            linkedAttributeSource2XPathResultsMap.put(linkedAttribute, values);
                        }
                        TreeSet valueSet = new TreeSet(String.CASE_INSENSITIVE_ORDER);
                        XMLUtils.convertXPathResultListToCollection(values, valueSet);

                        comboBoxValues = (String[]) valueSet.toArray(new String[valueSet.size()]);

                        if (linkedAttrCellEditor == null)	// lazy init
                            linkedAttrCellEditor =
									inMainWindow ? new LinkedAttributesComboBoxCellEditor() : new AutoCompleteComboBoxCellEditor();
                        linkedAttrCellEditor.init(comboBoxValues, dataModel.getValueAt(row, attrValueColumnIndex));
                        linkedAttrCellEditor.setRestrictValues(linkedAttribute.getRestrictValues().equals("yes"));
                        return linkedAttrCellEditor;
                    }
                }
            }
        }

        if ((attributeValueType == DTDHelper.IDREF) || (attributeValueType == DTDHelper.IDREFS)) {
            Set set = MiscUtils.enumerationToSortedSet(((CoreDocumentImpl) attribute.getOwnerDocument()).getIdentifiers());
            if (!set.isEmpty()) {
                comboBoxValues = (String[]) set.toArray(new String[set.size()]);
                if (linkedAttrCellEditor == null) {	// lazy init
                    linkedAttrCellEditor = inMainWindow ? new LinkedAttributesComboBoxCellEditor() : new AutoCompleteComboBoxCellEditor();
                    linkedAttrCellEditor.setRestrictValues(true);
                }
                linkedAttrCellEditor.init(comboBoxValues, dataModel.getValueAt(row, attrValueColumnIndex));
                return linkedAttrCellEditor;
            }
        }

        comboBoxValues = dtdHelper.getEnumValues(elementName, attrName);
        if (comboBoxValues != null) {
            if (enumCellEditor == null) { // lazy init
                enumCellEditor = new AutoCompleteComboBoxCellEditor();
                enumCellEditor.setRestrictValues(true);
            }
            enumCellEditor.init(comboBoxValues, dataModel.getValueAt(row, attrValueColumnIndex));
            return enumCellEditor;
        }

        if ((attributeValueType != DTDHelper.CDATA) && (attributeValueType != DTDHelper.UNKNOWN)) {
            if (specialAttributeValueTypeCellEditor == null)
                specialAttributeValueTypeCellEditor = new SpecialAttributeCellEditor();
            specialAttributeValueTypeCellEditor.setAttributeType(attributeValueType);
            return specialAttributeValueTypeCellEditor;
        }
        if (showEditButton) {
            if (bigStringCellEditor == null)	// lazy init
                bigStringCellEditor = new TableCellEditorWithEditButton(MiscImages.DOTS_ICON,
                        "Edit this attribute value in an extra dialog.", this, GlobalObjects.CELLEDITOR_BORDER);
            return bigStringCellEditor;
        } else {
            if (defaultCellEditor == null)	// lazy init
                defaultCellEditor = new StringCellEditor();
            return defaultCellEditor;
        }
    }


    public boolean isCellEditable(int row, int column) {
        if (column < attrValueColumnIndex)
            return false; // attribute name column is never editable
        if ((xmlTreeNode != null) && xmlTreeNode.isDELTALinkElement())
            return !dataModel.getValueAt(row, attrNameColumnIndex).equals(XMLTreeNode.DELTA_LINK_ATTRIBUTE_NAME);
        return (dtdHelper.getAttributeDefaultType(attributesTableModel.getElementName(row),
                (String) dataModel.getValueAt(row, attrNameColumnIndex)) != DTDHelper.FIXED);
    }

    public Class<?> getColumnClass(int column) {
        return String.class;
    }

    public void refresh() {
        if (xmlAttributesTableModel.getRowCount() > 0) {
            clearSelection();
            xmlAttributesTableModel.refresh();
            selectionModel.setAnchorSelectionIndex(0);
            selectionModel.setLeadSelectionIndex(0);
        }
    }

    // todo cache attribute value list (LinkedAttributes, IDREF) for every attribute: array of List
    public void setXMLTreeNode(XMLTreeNode xmlTreeNode) {
        if (this.xmlTreeNode == xmlTreeNode) return;
        if (cellEditor != null)
            cellEditor.stopCellEditing();
        clearSelection();
        this.xmlTreeNode = xmlTreeNode;
        if (xmlTreeNode != null) {
            Node xmlNode = xmlTreeNode.getXMLNode();
            xmlAttributesTableModel.setXMLElement((Element) xmlNode);
            if (xmlAttributesTableModel.getRowCount() > 0)
                selectionModel.setAnchorSelectionIndex(0);
        } else
            xmlAttributesTableModel.setXMLElement(null);

        if (xmlAttributesTableModel.getRowCount() == 0)
            SwingUtils.resetTableColumnWidths(this, attrValueColumnIndex);
        else {
            SwingUtils.packTableColumn(this, 0, 5);
            if (attrNameColumnIndex == 1)
                SwingUtils.packTableColumn(this, 1, 5);
        }

        linkedAttributeTarget2XPathResultsMap.clear();
        linkedAttributeSource2XPathResultsMap.clear();
    }

    public XMLTreeNode getXMLTreeNode() {
        return xmlTreeNode;
    }

    public String getElementName() {
        return xmlTreeNode.getNodeName();
    }

    public boolean isCurrentAttributeRemoveable() {
        return isAttributeRemoveable(selectionModel.getMinSelectionIndex());
    }

    public boolean isAttributeRemoveable(int row) {
        if ((xmlTreeNode == null) || (row < 0) || (row >= dataModel.getRowCount())) return false;
        return dtdHelper.isAttributeRemovable(xmlAttributesTableModel.getXMLElement(),
                (String) dataModel.getValueAt(row, attrNameColumnIndex));
    }

    public void addAttribute(String attrName, String selectedText) {
        String value = xmlView.askForOneAttribute(xmlAttributesTableModel.getXMLElement(), attrName, selectedText, dtdHelper, true);
        if (value != null) {
            int newRow = xmlAttributesTableModel.addRow(attrName, value, false);
            scrollToAndSelectRow(newRow);
        }
        requestFocusInWindow();
    }

    public void removeAttribute(int row) {
        boolean removed = xmlAttributesTableModel.removeAttribute(row, false);
        if (removed) {
            if (!xmlAttributesTableModel.isEmpty()) {
                row = Math.min(row, xmlAttributesTableModel.getRowCount() - 1);
                scrollToAndSelectRow(row);
            }
        }
        requestFocusInWindow();
    }

    public boolean changeCurrentAttribute(String newAttrName) {  // todo undo !!!
        Element element = xmlAttributesTableModel.getXMLElement();
        int row = getSelectedRow();
        Attr attribute = xmlAttributesTableModel.getAttribute(row);
        String attrValue = attribute.getValue();
        boolean somethingChanged = false;
        if (dtdHelper.isValidAttribute(element, newAttrName, attrValue)) {
            element.removeAttribute(attribute.getName());
            element.setAttribute(newAttrName, attrValue);
            xmlAttributesTableModel.changeAttributeName(row, newAttrName);
            setSelectedRow(row);
            somethingChanged = true;
        }
        requestFocusInWindow();
        return somethingChanged;
    }

    public void scrollToAndSelectRow(int row) {
        scrollRowToVisible(row);
        selectionModel.setAnchorSelectionIndex(row);
    }

    public void setSelectedRow(int row) {
        selectionModel.setAnchorSelectionIndex(row);
    }

    public void scrollRowToVisible(int row) {
        if ((row < 0) || (row >= dataModel.getRowCount())) return;
        Rectangle cellRect = getCellRect(row, 0, true);
        scrollRectToVisible(cellRect);
    }

    public Object getValueAt(int row, int column) {
        return XMLUtils.unescapeCharacters(dataModel.getValueAt(row, column).toString());
    }

    public void setValueAt(Object aValue, int row, int column) {
        dataModel.setValueAt(aValue, row, column);
    }

    public int getRowCount() {
        return dataModel.getRowCount();
    }

    public void refetchProperties() {
        showEditButton = properties.getBoolean(Preferences.KEY_XML_TABLE_EDIT_BUTTON);
        selectionColor = properties.getColor(Preferences.KEY_COLOR_SELECTION);
    }

    public void setXmlView(XMLView xmlView) {
        this.xmlView = xmlView;
    }

    private static class StringCellEditor extends AbstractCellEditor implements TableCellEditor {
        protected final JTextField textField;

        StringCellEditor() {
            textField = new JTextField();
            textField.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    stopCellEditing();
                }
            });
            textField.setBorder(GlobalObjects.CELLEDITOR_BORDER);
        }

        public Object getCellEditorValue() {
            return textField.getText();
        }

        public Component getTableCellEditorComponent(JTable table, Object value,
                                                     boolean isSelected, int row, int column) {
            textField.setText(value.toString());
            return textField;
        }
    }

    private static class SpecialAttributeCellEditor extends StringCellEditor {
        private String value, oldValue;
        private int attributeType;
        private Node node;

        //    private DTDHelper dtdHelper;
        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
            this.value = null;
            oldValue = value.toString();
            textField.setBorder(GlobalObjects.CELLEDITOR_BORDER);
            textField.setText(oldValue);
            //node = ((XMLAttributesTable)table).xmlTreeNode.getXMLNode();//todo: when inserting macro getXMLNode returns null
            node = ((AbstractAttributesTableModel) table.getModel()).getAttribute(row).getOwnerElement();
//    dtdHelper = ((XMLAttributesTable) table).dtdHelper;
            return textField;
        }

        public void setAttributeType(int attributeType) {
            this.attributeType = attributeType;
        }

        public boolean stopCellEditing() {
            String text = textField.getText();
            String msg = null;
            switch (attributeType) {
                case DTDHelper.ID:
// case DTDHelper.IDS: // is missing in XML4J#DTD
// todo: if current ID-type attribute has not been changed, it is already in "ids" set
                    Set ids = MiscUtils.enumerationToSet(((CoreDocumentImpl) node.getOwnerDocument()).getIdentifiers());
                    if (!((text.equals(oldValue) || !ids.contains(text)) && XMLChar.isValidName(text))) msg = "New value is not a valid, unique ID!";
                case DTDHelper.NMTOKEN:
                case DTDHelper.NMTOKENS:
                    if (!XMLChar.isValidNmtoken(text)) msg = "New value is not a valid NMTOKEN!";
            }
            textField.setToolTipText(msg);
            if (msg == null/*verify(text)*/) {
                value = text;
                fireEditingStopped();
                return true;
            }
            //	textField.setToolTipText("Attribute value is not valid according to DTD!");
            textField.setBorder(new LineBorder(Color.red));
            return false;
        }

        public Object getCellEditorValue() {
            return value;
        }

/*		protected boolean verify(String text) {
			switch (attributeType) {
				case DTDHelper.ID:
			// case DTDHelper.IDS: // is missing in XML4J#DTD
					// todo: if current ID-type attribute has not been changed, it is already in "ids" set
					Set ids = MiscUtils.enumerationToSet(((CoreDocumentImpl) node.getOwnerDocument()).getIdentifiers());
					return ((text.equals(oldValue) || !ids.contains(text)) && XMLChar.isValidName(text));
				case DTDHelper.NMTOKEN:
				case DTDHelper.NMTOKENS:
					return XMLChar.isValidNmtoken(text);
			}
			return true;
		}*/
    }

    private final class LinkedAttributesComboBoxCellEditor extends AutoCompleteComboBoxCellEditor {
        private final JPanel panel;

        LinkedAttributesComboBoxCellEditor() {
            panel = new JPanel(new BorderLayout(2, 0));

            JButton button = new JButton(new SimpleAction(MainImages.BOOKMARK_ICON, "Jump to \"linked\" element") {
                protected void simpleActionPerformed() {
                    jumpToLinkedElement();
                }
            });
            button.setMargin(new Insets(0, 0, 0, 0));
            button.setSize(25, 20);

            panel.add(comboBox, BorderLayout.CENTER);
            panel.add(button, BorderLayout.EAST);
        }

        public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
            super.getTableCellEditorComponent(table, value, isSelected, row, column);
            return panel;
        }

        public Component getComponent() {
            return panel;
        }
    }

    /**
     * Adds a listener to the list that's notified each time a change
     * to the data model occurs.
     *
     * @param l the TableModelListener
     */
    public void addTableModelListener(TableModelListener l) {
        dataModel.addTableModelListener(l);
    }

    /**
     * Removes a listener from the list that's notified each time a
     * change to the data model occurs.
     *
     * @param l the TableModelListener
     */
    public void removeTableModelListener(TableModelListener l) {
        dataModel.removeTableModelListener(l);
    }

    /**
     * Add a listener to the table that's notified each time a change
     * to the selection occurs.
     *
     * @param listener the new ListSelectionListener
     * @see #removeSelectionListener
     */
    public void addSelectionListener(ListSelectionListener listener) {
        selectionModel.addListSelectionListener(listener);
    }

    /**
     * Remove a listener from the table that's notified each time a
     * change to the selection occurs.
     *
     * @see #addSelectionListener
     */
    public void removeSelectionListener(ListSelectionListener listener) {
        selectionModel.removeListSelectionListener(listener);
    }

    /**
     * Adds an undo listener for notification of any changes.
     *
     * @param listener the <code>UndoableEditListener</code> to add
     * @see #removeUndoableEditListener
     */
    public void addUndoableEditListener(UndoableEditListener listener) {
        xmlAttributesTableModel.addUndoableEditListener(listener);
    }

    /**
     * Removes an undo listener.
     *
     * @param listener the <code>UndoableEditListener</code> to remove
     * @see #addUndoableEditListener
     */
    public void removeUndoableEditListener(UndoableEditListener listener) {
        xmlAttributesTableModel.removeUndoableEditListener(listener);
    }

    private class XMLAttributesTableCellRenderer extends DefaultTableCellRenderer {
        public Component getTableCellRendererComponent(JTable table, Object value,
                                                       boolean isSelected, boolean hasFocus, int row, int column) {
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

			if (isSelected)
				setBackground(selectionColor);
			else
				setBackground(table.getBackground());

			if ((columnCount == 3) && (column == 0)) {
            //    setBackground(isSelected ? elementColumnSelectedBackground : elementColumnBackground);
                return this;
            }

			String attrName = (String) ((column == attrNameColumnIndex) ? value : dataModel.getValueAt(row, attrNameColumnIndex));
            String elementName = attributesTableModel.getElementName(row);
            int attrType = dtdHelper.getAttributeDefaultType(elementName, attrName);

            int attributeValueType = dtdHelper.getAttributeValueType(elementName, attrName);
            switch (attrType) {
                case DTDHelper.FIXED:
                    setToolTipText("fixed attribute value (" + DTDHelper.ATTRIBUTE_VALUE_TYPES[attributeValueType] + ")");
                    setBackground(getBackground().darker());
                    if (column == attrNameColumnIndex)
                        setFont(FIXED_ATTR_FONT);
                    break;
                case DTDHelper.REQUIRED:
                    if (column == attrNameColumnIndex)
                        setFont(REQUIRED_ATTR_FONT);
                    if (dtdHelper.getEnumValuesSize(elementName, attrName) == 0)
                        setToolTipText("required attribute (" +
                                DTDHelper.ATTRIBUTE_VALUE_TYPES[attributeValueType] + ")");
                    else {
                        setToolTipText("required attribute / enumeration");
//                        if (column == attrValueColumnIndex)
//                            setBackground(isSelected ? enumValueColumnSelectedBackground : enumValueColumnBackground);
                    }
                    break;
                case DTDHelper.IMPLIED:
                    setToolTipText("optional attribute (" + DTDHelper.ATTRIBUTE_VALUE_TYPES[attributeValueType] + ")");
                    break;
                case DTDHelper.ENUM:
                    setToolTipText("optional attribute / enumeration");
//                    if (column == attrValueColumnIndex)
//                        setBackground(isSelected ? enumValueColumnSelectedBackground : enumValueColumnBackground);
                    break;
            }

            Attr attribute = attributesTableModel.getAttribute(row);
            if (column == attrValueColumnIndex) {
                if ((attribute != null) && !attribute.getSpecified()) {
                    setFont(DEFAULT_VALUE_FONT);
                    setToolTipText(getToolTipText() + " - default value");
                }
            }

            // this is only to show the "Jump to corresponding Element"-Button - but it makes navigating through treenodes much slower
            if (inMainWindow && properties.getBoolean(Preferences.KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON)) {
                Component comp = XMLAttributesTable.this;
                while (!(comp instanceof MainFrame.DeltaSide))
                    comp = comp.getParent();
                DeltaSideController thisSideController = ((MainFrame.DeltaSide) comp).sideController;
                if ((column == 1) && (thisSideController.currentXMLLanguage != null)) {
                    boolean valuesAvailable = false;
                    LinkedAttributes linkedAttributes = thisSideController.currentXMLLanguage.getLinkedAttributes();
                    if (linkedAttributes != null) {
                        List linkedAttributeList = linkedAttributes.getLinkedAttributeList();
                        for (int i = 0, size = linkedAttributeList.size(); i < size; i++) {
                            LinkedAttribute linkedAttribute = (LinkedAttribute) linkedAttributeList.get(i);
                            List targets = linkedAttributeTarget2XPathResultsMap.get(linkedAttribute);
                            if (targets == null) {
                                targets = XMLUtils.evaluateXPathSilently(linkedAttribute.getTargetAttributeXPath(), attribute.getOwnerDocument());
                                linkedAttributeTarget2XPathResultsMap.put(linkedAttribute, targets);
                            }
                            if (targets.contains(attribute)) {
                                List values = linkedAttributeSource2XPathResultsMap.get(linkedAttribute);
                                if (values == null) {
                                    values = XMLUtils.evaluateXPathSilently(linkedAttribute.getSourceXPath(), attribute.getOwnerElement());
                                    linkedAttributeSource2XPathResultsMap.put(linkedAttribute, values);
                                }
                                if (values.isEmpty()) continue;
                                valuesAvailable = true;
                                break;
                            }
                        }
                    }
                    if (!valuesAvailable)
                        if ((attributeValueType == DTDHelper.IDREF) || (attributeValueType == DTDHelper.IDREFS)) {
                            Enumeration identifiers = ((CoreDocumentImpl) attribute.getOwnerDocument()).getIdentifiers();
                            valuesAvailable = identifiers.hasMoreElements();
                        }
                    if (valuesAvailable) {
                        JPanel panel = new JPanel(new BorderLayout(2, 0));
                        JButton button = new JButton(new EmptyAction(MainImages.BOOKMARK_ICON, "Jump to \"linked\" element"));
                        button.setMargin(new Insets(0, 0, 0, 0));
                        button.setSize(25, 20);
                        panel.add(this, BorderLayout.CENTER);
                        panel.add(button, BorderLayout.EAST);
                        return panel;
                    }
                }
            }

            return this;
        }
    }
}
