/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 19, 2002
 * Time: 3:00:58 AM
 */
package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.xml.images.XMLImages;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.util.xml.xerces.ErrorWrapper;

import java.awt.*;
import java.util.List;

import javax.swing.*;

/**
 * Class XMLTreeCellRenderer. ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3
 * @since JDK 1.3.1
 */
final class XMLTreeCellRenderer extends SimpleXMLTreeCellRenderer {

	private List<XMLTreeNode> highlightedTreeNodes;
	private Color highLightColor;
	private final Color linkParentColor;
	private final Color linkSubtreeColor;
	private final Color noLinkSubtreeColor;
	private final Color noElementNodeColor;
	private int linksVisualization;
	private ElementIconMap elementIconMap;

	XMLTreeCellRenderer(ElementIconMap elementIconMap) {
		super();
		this.elementIconMap = elementIconMap;
		linkParentColor = Color.cyan; // todo: take colors from prefs
		linkSubtreeColor = SwingUtils.changeColor(linkParentColor, -10);
		noLinkSubtreeColor = SwingUtils.changeColor(Color.lightGray, +45);
		noElementNodeColor = SwingUtils.changeColor(Color.lightGray, +55);
		linksVisualization = XMLView.NO_VISUALIZATION;
	}

	List getHighlightedTreeNodes() {
		return highlightedTreeNodes;
	}

	void setHighlightedTreeNodes(List<XMLTreeNode> highlightedTreeNodes) {
		this.highlightedTreeNodes = highlightedTreeNodes;
		if (highlightedTreeNodes != null)
			linksVisualization = XMLView.NO_VISUALIZATION;
	}

	Color getHighLightColor() {
		return highLightColor;
	}

	void setHighLightColor(Color highLightColor) {
		this.highLightColor = highLightColor;
	}

	int getLinksVisualization() {
		return linksVisualization;
	}

	void setLinksVisualization(int linksVisualization) {
		this.linksVisualization = linksVisualization;
		if (linksVisualization != XMLView.NO_VISUALIZATION)
			highlightedTreeNodes = null;
	}

	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected,
												  boolean expanded, boolean leaf, int row, boolean hasFocus) {
		super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
		ErrorWrapper errorWrapper = xmlTreeNode.getErrorWrapper();
		if (errorWrapper != null) {
			//		setToolTipText(errorWrapper.message);
			if (errorWrapper.error)
				setIcon(XMLImages.ERROR_ELEMENT_ICON);
			else
				setIcon(XMLImages.WARNING_ELEMENT_ICON);
		} else {
			//		setToolTipText(null);
			if (xmlTreeNode.isElement()) {
				Icon icon = elementIconMap.getIcon(xmlTreeNode.getXMLNode().getNodeName(), expanded, leaf);
				if (icon != null)
					setIcon(icon);
			}
		}

		if ((highlightedTreeNodes != null) && highlightedTreeNodes.contains(value))
			backgroundColor = highLightColor;
		else if (linksVisualization != XMLView.NO_VISUALIZATION)
			if (!xmlTreeNode.isElement())
				backgroundColor = noElementNodeColor;
			else if (xmlTreeNode.isDELTALinkElement())
				backgroundColor = highLightColor;
			else if (xmlTreeNode.hasDELTALink())
				backgroundColor = linkParentColor;
			else if ((linksVisualization == XMLView.LINK_VISUALIZATION_DOWN) ?
					 xmlTreeNode.hasDELTALinkAncestor() : xmlTreeNode.hasDELTALinkDescendant())
				backgroundColor = linkSubtreeColor;
			else backgroundColor = noLinkSubtreeColor;
		return this;
	}

	public void paint(Graphics g) {
		boolean _showAttributes = showAttributes;
		showAttributes = !xmlTreeNode.isSummarizeTextSet();
		super.paint(g);
		showAttributes = _showAttributes;
		if (selected && (backgroundColor != backgroundSelectionColor)) {
			int imageOffset = (drawsFocusBorderAroundIcon ? 0 : 19);
			g.setColor(borderSelectionColor);
			g.drawRect(imageOffset, 0, getWidth() - 1 - imageOffset, getHeight() - 1);
		//	g.drawRect(imageOffset + 1, 1, getWidth() - 3 - imageOffset, getHeight() - 3);
		}
	}
}