package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.swing.models.SimpleMutableTreeNode;
import at.ac.tuwien.asgaard.util.MiscUtils;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import at.ac.tuwien.asgaard.util.xml.xerces.ErrorWrapper;

import org.w3c.dom.*;

/**
 * Class XMLTreeNode.... todo
 *
 * @author Peter Votruba
 * @version 2.7.3
 * @since JDK 1.3.1
 */
public final class XMLTreeNode extends SimpleMutableTreeNode implements Comparable<XMLTreeNode> {

// compareable with NodeImpl.compareTreePosition(Node other)
	private int deltaLinkID;
	private Integer deltaLinkIDWrapper;

	private Node xmlNode;
	private String label;
	private String infoText, shortDescription;
	private StyleInfo styleInfos[];
	private ErrorWrapper errorWrapper;
	private String simpleXPath;
	private final boolean isDocumentElement;
	private MainController.XMLClipBoard.ClipBoardMode clipBoardMode;
	private boolean truncatedAttributesText;

	public static final	String DELTA_LINK_ELEMENT_NAME	 = "delta-link";
	public static final	String GMT_LINK_ELEMENT_NAME	 = "gmt-link";
	public static final	String DELTA_LINK_ATTRIBUTE_NAME = "link-id";

	private boolean summarizeTextSet = false;

	private XMLTreeNode() { isDocumentElement = false; }

	XMLTreeNode(Node xmlNode) {
		this(xmlNode, true);
	}

	XMLTreeNode(Node xmlNode, boolean initStyles) {
		super();
		this.xmlNode = xmlNode;
		setFields();
		if (initStyles) initStyleInfos();
		isDocumentElement = XMLUtils.isDocumentElement(xmlNode);
	}

	void initStyleInfos() {
		styleInfos = createStyleInfos(true);
	}

	StyleInfo[] createStyleInfos(boolean truncate) {
		StyleInfo styleInfos1[];
		if (truncate)
			truncatedAttributesText = false;
		switch (xmlNode.getNodeType()) {
			case Node.ELEMENT_NODE:
				NamedNodeMap attributes = xmlNode.getAttributes();
				int attributeCount = attributes.getLength();
				styleInfos1 = new StyleInfo[3 + attributeCount * 4];
				// styleInfos[0] = StyleInfo.TAG_OPEN;
				if (errorWrapper == null)
					if (isDocumentElement)
						styleInfos1[0] = StyleInfo.createRootElementStyle(label);
					else if (isDELTALinkElement())
						styleInfos1[0] = StyleInfo.createLinkElementStyle(label);
					else
						styleInfos1[0] = StyleInfo.createElementStyle(label);
				else if (errorWrapper.error)
					styleInfos1[0] = StyleInfo.createElementErrorStyle(label);
				else
					styleInfos1[0] = StyleInfo.createElementWarningStyle(label);
				styleInfos1[1] = StyleInfo.createChildCountStyle(getChildCount());
				// if (infoText != null) ... // TODO

				if (attributeCount > 0)
					for (int i = 0; i < attributeCount; i++) {
						Node attrNode = attributes.item(i);
						styleInfos1[3 + i * 4] = StyleInfo.createAttributeNameStyle(" " + attrNode.getNodeName());
						styleInfos1[3 + i * 4 + 1] = StyleInfo.createAttributeSymbolStyle("=\"");
						String attrValueStr = XMLUtils.unescapeCharacters(attrNode.getNodeValue());
						if (truncate) {
							int length = attrValueStr.length();
							attrValueStr = MiscUtils.truncateSring(attrValueStr);
							if (length != attrValueStr.length())
								truncatedAttributesText = true;
						}
						styleInfos1[3 + i * 4 + 2] = StyleInfo.createAttributeValueStyle(attrValueStr);
						styleInfos1[3 + i * 4 + 3] = StyleInfo.createAttributeSymbolStyle("\"");
					}
				//	attrsStyleInfo = StyleInfo.createAttributeNameStyle(attributesText);
				//	styleInfos[2] = attrsStyleInfo;
				// styleInfos[3 + indexAdd] = StyleInfo.TAG_CLOSE;
				break;
			case Node.TEXT_NODE:
			case Node.CDATA_SECTION_NODE:
			case Node.COMMENT_NODE:
			case Node.PROCESSING_INSTRUCTION_NODE:
				styleInfos1 = new StyleInfo[]{StyleInfo.createCommentStyle(label)};
				break;
			case Node.DOCUMENT_NODE:
			case Node.DOCUMENT_TYPE_NODE:
				styleInfos1 = new StyleInfo[]{StyleInfo.createDocumentStyle(label)};
				break;
			default:
				styleInfos1 = null;
		}
		return styleInfos1;
	}

	boolean getAttributesTextTruncated() {
		return truncatedAttributesText;
	}

	StyleInfo[] getStyleInfos() {
		return styleInfos;
	}

	public void setSummarizeString(String info) {
		if ((info == null) || info.equals("")) {
			styleInfos[2] = null;
			summarizeTextSet = false;
		} else {
			styleInfos[2] = StyleInfo.createSummarizeStyle(info);
			summarizeTextSet = true;
		}
	}

	public boolean isSummarizeTextSet() {
		return summarizeTextSet;
	}

	public boolean isDELTALinkElement() {
		return (deltaLinkID >= LinkIDConstants.FIRST_ID);
	}

	/**
	 * Returns the link ID of this <i>XMLTreeNode</i>.
	 *
	 * @return the link ID if this <i>XMLTreeNode</i> is a link node or
	 *         LinkIDConstants#NO_LINK if this <i>XMLTreeNode</i> is not a link node.
	 */
	public int getDELTALinkID() {
		return deltaLinkID;
	}

	/**
	 * Returns the link ID of this <i>XMLTreeNode</i>.
	 *
	 * @return the link ID if this <i>XMLTreeNode</i> is a link node or
	 *         LinkIDConstants#_NO_LINK if this <i>XMLTreeNode</i> is not a link node.
	 */
	public Integer getDELTALinkIDWrapper() {
		return deltaLinkIDWrapper;
	}

	public boolean hasDELTALink() {
		if (getChildCount() == 0) return false;
		XMLTreeNode child[] = children.toArray(new XMLTreeNode[children.size()]);
		for (int i = 0; i < child.length; i++)
			if (child[i].isDELTALinkElement()) return true;
		return false;
	}

	public boolean hasDELTALinkDescendant() {
		if (isDELTALinkElement()) return true;
		if (getChildCount() == 0) return false;
		XMLTreeNode child[] = children.toArray(new XMLTreeNode[children.size()]);
		for (int i = child.length - 1; i >= 0; i--)
			if (child[i].hasDELTALinkDescendant()) return true;
		return false;
	}

	public boolean hasDELTALinkAncestor() {
		// if (isDELTALinkElement()) return true;
		if ((parent == null) || !(parent instanceof XMLTreeNode)) return false;
//		if (((XMLTreeNode) parent).xmlNode.getNodeName().equals("plan")) return false;  // todo
		for (int i = parent.getChildCount() - 1; i >= 0; i--)
			if (((XMLTreeNode) parent.getChildAt(i)).isDELTALinkElement()) return true;
		return ((XMLTreeNode) parent).hasDELTALinkAncestor();
	}

	public Node getXMLNode() {
		return xmlNode;
	}

	public short getXMLNodeType() {
		return xmlNode.getNodeType();
	}

	public boolean isElement() {
		return (xmlNode.getNodeType() == Node.ELEMENT_NODE);
	}

	public boolean isDocumentElement() {
		return isDocumentElement;
	}

	public String getNodeName() {
		return xmlNode.getNodeName();
	}

	public String getLabel() {
		return label;
	}

	public String getInfoText() {
		return infoText;
	}

	public String getText() {
		return xmlNode.getNodeValue();
	}

	// DOCUMENT_NODE and DOCUMENT_TYPE_NODE nodes return null
	public String getSimpleXPath() {
		return simpleXPath;
	}

	public void reset() {
		setFields();
		initStyleInfos();
	}

	public String toString() {
		return label;
	}

	public boolean hasAttributes() {
		return ((xmlNode.getAttributes() != null) && (xmlNode.getAttributes().getLength() > 0));
	}

	private void setFields() {
		styleInfos = null;
		simpleXPath = null;
		label = "";
		infoText = "";

		deltaLinkIDWrapper = determineDELTALinkID(xmlNode);
		deltaLinkID = deltaLinkIDWrapper.intValue();

		switch (xmlNode.getNodeType()) {
			case Node.DOCUMENT_NODE:
				label = "Document node";
				infoText = "The document node of this XML file.";
				shortDescription = "DOCUMENT node";
				break;
			case Node.ELEMENT_NODE:
				label = xmlNode.getNodeName();
				infoText = null;
				shortDescription = "ELEMENT '" + label + "'";
				simpleXPath = "//" + label;
				break;
			case Node.TEXT_NODE:
			case Node.CDATA_SECTION_NODE:
			case Node.COMMENT_NODE:
			case Node.PROCESSING_INSTRUCTION_NODE:
				label = xmlNode.getNodeValue().trim();
				switch (xmlNode.getNodeType()) {
					case Node.TEXT_NODE:
						simpleXPath = "//text()";
						shortDescription = "TEXT node";
						break;
					case Node.CDATA_SECTION_NODE:
						simpleXPath = null;
						shortDescription = "CDATA node";
						break;
					case Node.COMMENT_NODE:
						simpleXPath = "//comment()";
						shortDescription = "COMMENT node";
						break;
					case Node.PROCESSING_INSTRUCTION_NODE:
						simpleXPath = "//processing-instruction()";
						shortDescription = "PROCESSING-INSTRUCTION node";
				}
				infoText = label;
				int pos = label.indexOf('\n');
				if (pos != -1) label = label.substring(0, pos);
				label = MiscUtils.replace(label, "\t", " ");
				break;
			case Node.DOCUMENT_TYPE_NODE:
				label = "DocType node";
				shortDescription = "DOCTYPE node";
				infoText = "The document type node of this XML file.";
				DocumentType docType = (DocumentType) xmlNode;
				infoText = infoText + "\n  Root element: \"" + docType.getName() + "\"";
				infoText = infoText + "\n  System ID: "
						   + ((docType.getSystemId() == null) ? "not specified" : "\"" + docType.getSystemId() + "\"");
				infoText = infoText + "\n  Public ID: "
						   + ((docType.getPublicId() == null) ? "not specified" : "\"" + docType.getPublicId() + "\"");
				break;
			default:
				System.err.println("Unknown Node Type: " + xmlNode.getNodeType()); // shouldn't happen
		}
		//    shortDescription = shortDescription + "node";
	}

	static Integer determineDELTALinkID(Node xmlNode) {
		if ((xmlNode.getNodeType() == Node.ELEMENT_NODE) &&
				(xmlNode.getNodeName().equals(DELTA_LINK_ELEMENT_NAME)||
				 xmlNode.getNodeName().equals(GMT_LINK_ELEMENT_NAME))) {
			String idStr = ((Element) xmlNode).getAttribute(DELTA_LINK_ATTRIBUTE_NAME);
			if ((idStr == null) || idStr.equals(""))
				return LinkIDConstants._BAD_LINK;
			else
				try {
					Integer integer = new Integer(idStr.substring(idStr.indexOf(':') + 1)); // ignore prefix
					if (integer.intValue() < 0) return LinkIDConstants._BAD_LINK;
					return integer;
				} catch (NumberFormatException exc) {
					return LinkIDConstants._BAD_LINK;
				}
		} else
			return LinkIDConstants._NO_LINK;
	}

	public int getDELTALinkPrefix() {
		if (!isDELTALinkElement()) return -1;
		String idStr = ((Element) xmlNode).getAttribute(DELTA_LINK_ATTRIBUTE_NAME);
		try {
			int prefix = Integer.parseInt(idStr.substring(0, idStr.indexOf(':')));
			if (prefix < 0) return 0;
			return prefix;
		} catch (Exception exc) {
			return 0;
		}
	}

	public void remove(int childIndex) {
		super.remove(childIndex);
		initStyleInfos();
	}

	ErrorWrapper getErrorWrapper() {
		return errorWrapper;
	}

	void setErrorWrapper(ErrorWrapper errorWrapper) {
		this.errorWrapper = errorWrapper;
	}

	/**
	 * getShortDescription()
	 */
	public String getShortDescription() {
		return shortDescription;
	}


	public String getXPath() {
		return XMLUtils.getXPathForNode(xmlNode);
	}

	public int compareTo(XMLTreeNode treeNode) {
		return XMLUtils.compareNodes(xmlNode, treeNode.xmlNode);
	}

//    private static XMLTreeNode dummyTreeNode;
	public static XMLTreeNode getDummyTreeNode() {
		return new XMLTreeNode();
	}


	public void setClipBoardMode(MainController.XMLClipBoard.ClipBoardMode clipBoardMode) {
		this.clipBoardMode = clipBoardMode;
		for (int i = 0, size = getChildCount(); i < size; i++) {
			XMLTreeNode xmlTreeNode = (XMLTreeNode) children.get(i);
			xmlTreeNode.setClipBoardMode(clipBoardMode);
		}
	}

	public MainController.XMLClipBoard.ClipBoardMode getClipBoardMode() {
		return clipBoardMode;
	}

	public static boolean isLinkElementName(String elementName) {
		return elementName.equals(DELTA_LINK_ELEMENT_NAME) || elementName.equals(GMT_LINK_ELEMENT_NAME);
	}
}
