/*
 * Copyright (c) 2004 Asgaard. All Rights Reserved.
 */
package at.ac.tuwien.asgaard.delta.gui.xml;

import at.ac.tuwien.asgaard.delta.gui.DeltaSideView;
import at.ac.tuwien.asgaard.delta.gui.LinkIDConstants;
import at.ac.tuwien.asgaard.delta.gui.MainController;
import at.ac.tuwien.asgaard.delta.gui.macros.MacrosListItem;
import at.ac.tuwien.asgaard.delta.gui.misc.ArrowImage;
import at.ac.tuwien.asgaard.delta.gui.misc.DeltaHyperlinkEvent;
import at.ac.tuwien.asgaard.delta.gui.misc.undo.*;
import at.ac.tuwien.asgaard.delta.util.Preferences;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.LinkedAttribute;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.LinkedAttributes;
import at.ac.tuwien.asgaard.delta.xmlBeans.xmlLanguage.XmlLanguage;
import at.ac.tuwien.asgaard.swing.SwingUtils;
import at.ac.tuwien.asgaard.swing.autoComplete.AutoCompleteComboBoxEditDialog;
import at.ac.tuwien.asgaard.swing.models.SimpleMutableTreeNode;
import at.ac.tuwien.asgaard.swing.models.SingleTreeSelectionModel;
import at.ac.tuwien.asgaard.util.*;
import at.ac.tuwien.asgaard.util.xml.FileURLEntityResolver;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper;
import at.ac.tuwien.asgaard.util.xml.dtd.DTDHelperXML4J;
import at.ac.tuwien.asgaard.util.xml.xerces.ErrorReportingDOMParser;
import at.ac.tuwien.asgaard.util.xml.xerces.ErrorWrapper;

import org.apache.xerces.dom.CoreDocumentImpl;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.DocumentTypeImpl;
import org.apache.xerces.impl.Constants;
import org.w3c.dom.*;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.TreeWalker;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.image.*;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.util.*;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.Timer;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.undo.*;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

/**
 * Class XMLView.... todo
 *
 * @version 2.7.3
 * @author Peter Votruba

 */
public final class XMLView extends SimpleXMLTree implements DeltaSideView, Autoscroll {
    public static final int ASK_ATTRVAL_FOR_EACH_ATTRIBUTE = 0;
    public static final int ASK_ATTRVAL_FOR_EACH_ELEMENT   = 1;
    public static final int ASK_ATTRVAL_FOR_EACH_MACRO     = 2;

    public static final int NO_VISUALIZATION        = 0;
    public static final int LINK_VISUALIZATION_DOWN = 1;
    public static final int LINK_VISUALIZATION_UP   = 2;

	private int askAttributeValueForEach;
	private static boolean insertEmptyAttributes;
	private XMLAttributesTable attributesTable;
	private XMLAttributesInputDialog attributesInputDialog;
	private static AutoCompleteComboBoxEditDialog textEditDialog, variableEditDialog;
	private File xmlFile;
	private boolean contentHasChanged;
	private ListMap<Integer,XMLTreeNode> deltaLinkMap;

	private int internalMaxLinkID;
	private int externalMaxLinkID;

	private int currentHighlightIndex;
	private boolean selectCurrentHighlightedTreeNode;
	private int fireXMLHyperLinkEventMouseClickCount;
	private Document xmlDocument;
	private ErrorReportingDOMParser domParser;
	private XMLTreeNode docTreeNode, docElemTreeNode;
	private DTDHelper dtdHelper;
	private XMLTreeCellRenderer xmlTreeCellRenderer;
	private boolean expandAllOnLoad;
	private FileURLEntityResolver entityResolver;
	private LinkedList<TreePath> historyBackList, historyForwardList;

	public static final String DATE_STRING = DateFormat.getDateInstance(DateFormat.SHORT).format(new Date()); // I assume that nobody will use the DELTA over midnight
	private static final DateFormat TIME_FORMAT = DateFormat.getTimeInstance(DateFormat.SHORT);

	private boolean addToHistory, isLoading;
	private boolean showDocumentNode, showCommentNodes, showCDATANodes, showTextNodes, showPINodes, showDocumentTypeNode;
	private List filterXMLNodes;
	private boolean filterShow;
	private ElementIconMap elementIconMap;

	private Color linkHighlightColor, searchHighlightColor;

	private Map<String, String> macroVariableMap;
	private Map<String, Integer> macroCounterMap;
	private TreeWalker searchTreeWalker;
	private boolean searchForward;
	private Node searchStartNode;
	private boolean searchFound;
	private final int thisSideIndex;
	private static final boolean COLLAPSE_AFTER_JUMP_TO_TREEPATH = false; // is used in jumpToTreePath() that is used by TreePathBar

	public XMLView(int sideIndex, Preferences properties, XMLAttributesTable attributesTable, DTDHelper dtdHelper) {
		super(properties);
		Log.trace("enter: XMLView()");

		thisSideIndex = sideIndex;
		this.dtdHelper = dtdHelper;

		this.properties = properties;
		this.attributesTable = attributesTable;
		internalMaxLinkID = externalMaxLinkID = LinkIDConstants.FIRST_ID - 1;
		contentHasChanged = false;
		addToHistory = true;
		isLoading = false;

		addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent evt) {
			/* if (!evt.isTemporary())
					if (lastXMLTreeNode != null) {
						currentlyHoveredXMLTreeNode = lastXMLTreeNode;
						lastXMLTreeNode = null;
						hoveredTreeNodeChanged();
					}*/
			}
			public void focusLost(FocusEvent evt) {
		//  fireXMLHyperLinkEvent(HyperlinkEvent.EventType.EXITED, -1);
			}
		});
		addMouseListener(new MouseAdapter() {
			public void mouseExited(MouseEvent evt) {
//  fireXMLHyperLinkEvent(HyperlinkEvent.EventType.EXITED, -1);
			    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			}
			public void mouseEntered(MouseEvent e) {
//  fireXMLHyperLinkEvent(HyperlinkEvent.EventType.EXITED, -1);
			}
		});
		addTreeExpansionListener(new TreeExpansionListener() {
			public void treeExpanded(TreeExpansionEvent evt) {
				if (xslTransformer == null) return;
				XMLTreeNode xmlTreeNode = (XMLTreeNode) evt.getPath().getLastPathComponent();
				if (xmlTreeNode.isElement()) {
					xmlTreeNode.setSummarizeString(null);
					simpleTreeModel.nodeChanged(xmlTreeNode);
				}
			}
			public void treeCollapsed(TreeExpansionEvent evt) {
				if (xslTransformer == null) return;
				XMLTreeNode xmlTreeNode = (XMLTreeNode) evt.getPath().getLastPathComponent();
				if (xmlTreeNode.isElement()) {
					Node xmlNode = xmlTreeNode.getXMLNode();
					String xslResult = summarizeNodeContent(xmlNode);
					xmlTreeNode.setSummarizeString(xslResult);
					simpleTreeModel.nodeChanged(xmlTreeNode);
				}
			}
		});
		Log.trace("exit: XMLView()");
	}

	protected void fireValueChanged(TreeSelectionEvent evt) {
		if ((evt.getNewLeadSelectionPath() != null)
				&& (evt.getOldLeadSelectionPath() != null)
				&& (evt.getNewLeadSelectionPath().getLastPathComponent() != evt.getOldLeadSelectionPath().getLastPathComponent())
				&& addToHistory) {
			historyBackList.addFirst(evt.getOldLeadSelectionPath());
			historyForwardList.clear();
		}
		super.fireValueChanged(evt);
	} 

	private void init() {
		macroVariableMap = new HashMap<String, String>();
		macroCounterMap = new HashMap<String, Integer>();

		if (domParser == null) {
			deltaLinkMap = new SortedListMap<Integer, XMLTreeNode>();
			historyBackList	   = new LinkedList<TreePath>();
			historyForwardList = new LinkedList<TreePath>();
			xmlNode2TreeNodeMap = new HashMap<Node, XMLTreeNode>();

			domParser = new ErrorReportingDOMParser();

			try {
				domParser.setFeature(Constants.SAX_FEATURE_PREFIX + Constants.NAMESPACES_FEATURE, false); // we dont use namespaces
				domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.DYNAMIC_VALIDATION_FEATURE, true);
				domParser.setFeature(Constants.SAX_FEATURE_PREFIX + Constants.VALIDATION_FEATURE, true); // DYNAMIC_VALIDATION_FEATURE alone produce a java.lang.NullPointerException at org.apache.xerces.impl.dtd.XMLDTDValidator.validateDTDattribute(XMLDTDValidator.java:1383)
				domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.WARN_ON_DUPLICATE_ATTDEF_FEATURE, true);
				domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.WARN_ON_DUPLICATE_ENTITYDEF_FEATURE, true);
				domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.CONTINUE_AFTER_FATAL_ERROR_FEATURE, true);
				domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.INCLUDE_IGNORABLE_WHITESPACE, false); // we dont need "ignorable whitespace"
			} catch (Exception exc) { // should not happen
				exc.printStackTrace();
				Log.trace("Couldn't set features of DOMParser: " + exc.getMessage());
			}
			entityResolver = new FileURLEntityResolver();
			domParser.setEntityResolver(entityResolver);
		} else {
			domParser.reinitialize();
			deltaLinkMap.clear();
			historyBackList.clear();
			historyForwardList.clear();
			xmlNode2TreeNodeMap.clear();
		}
	}

// I/O- and tree creation methods

    public void createNewEmptyXMLFile(File newXMLFile, String systemID, String rootElement, String XMLLangName, String userName)
			throws DOMException, IOException {
		Log.trace("enter: XMLView.createNewAs()");
		isLoading = true;
		setEnabled(false);

		init();

		try {
			xmlDocument = XMLUtils.createNewDocument(systemID, rootElement);
			xmlDocument.insertBefore(xmlDocument.createComment(XMLLangName + " document created by "+ userName +" using DELT/A on " + DATE_STRING),
									 xmlDocument.getDocumentElement());
			deltaLinkMap.clear();

		// showCommentNodes = showCDATANodes = showTextNodes = showDocumentTypeNode = showPINodes = showDocumentNode = true;
		// filterXMLNodes = null;

			internalMaxLinkID = externalMaxLinkID;

			Element documentElement = xmlDocument.getDocumentElement();
			dtdHelper.addRequiredContent(documentElement, null);
			docTreeNode = createTreeNode(xmlDocument);
			docElemTreeNode = getTreeNode4XMLNode(documentElement);

			xmlTreeCellRenderer.setHighlightedTreeNodes(null);
			xmlTreeCellRenderer.setLinksVisualization(NO_VISUALIZATION);
			historyBackList.clear();
			historyForwardList.clear();
			expandedTreePath = simpleTreeModel.getTreePath(docElemTreeNode);
			setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
			setRootVisible(true);
			if (expandAllOnLoad) expandAll();
			else expandPath(expandedTreePath);
			if (xmlFile == null) {
				setBackground(Color.white);
				refresh();
			}
			setEnabled(true);

			List<AttributeInputWrapper> attrsList = new LinkedList<AttributeInputWrapper>();
			buildAttrsList(documentElement, attrsList);

			// problem when XML is created on "Change process step"
			// requestAttributeValues(attrsList, "Enter attribute values for inserted elements...");

			empty = false;
		//	contentHasChanged(false);
			saveAs(newXMLFile);
		} finally {
			requestFocusInWindow();
			setEnabled(true);
			isLoading = false;
			Log.trace("exit: XMLView.createNewAs()");
		}
	}
	public final void expandAll() {
		expandAllDescendants(docElemTreeNode, simpleTreeModel.getTreePath(docElemTreeNode));
	}
	public void load(File xmlFile) throws Exception {
		load(xmlFile, new BufferedReader(new FileReader(xmlFile)));
	}

	public void load(File xmlFile, Reader reader) throws Exception {
		load(xmlFile, reader, null);
	}

  public void load(File xmlFile, Reader reader, String documentComment) throws Exception {
		Log.trace("XMLView: enter load() - xmlFile=" + xmlFile);
		isLoading = true;
		setEnabled(false);
		init();
		try {
			long fileSize = xmlFile.length();
//  Xerces DOM with deferred node expansion offers poor performance and
//  large memory size for small documents (0K-10K). Thus, for best performance
//  when using Xerces DOM with smaller documents you should disable the deferred
//  node expansion feature. For larger documents (~100K and higher) the deferred DOM
//  offers better performance than non-deferred DOM but uses a large memory size.
//  parser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.DEFER_NODE_EXPANSION_FEATURE, fileSize > 51200); // Current element node cannot be queried when node expansion is deferred!!!
			domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.DEFER_NODE_EXPANSION_FEATURE, false); // because DEFER_NODE_EXPANSION_FEATURE is true, if DTDConfiguration is used
//            domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.CREATE_ENTITY_REF_NODES_FEATURE, true);
//            domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.NOTIFY_CHAR_REFS_FEATURE, true);
//            domParser.setFeature(Constants.XERCES_FEATURE_PREFIX + Constants.NOTIFY_BUILTIN_REFS_FEATURE, true);
            domParser.setFeature("http://xml.org/sax/features/external-general-entities", true);
            domParser.setFeature("http://xml.org/sax/features/external-parameter-entities", true);

/*  Some tests indicate that a bigger buffer size can improve the parsing performance for relatively large files.
	The default buffer size in Xerces is 2K. This would give a good performance for small documents (less than 10K).
	For documents larger than 10K, specifying the buffer size to 4K or 8K will significantly improve the performance.
	But it's not recommended to set it to a value larger than 16K. For really tiny documents (1K, for example),
	you can also set it to a value less than 2K, to get the best performance. */
			int bufferSize = 2048;
			if (fileSize < 2048) bufferSize = 1024;
			else if (fileSize > 102400) bufferSize = 16384;
			else if (fileSize > 51200) bufferSize = 8192;
			else if (fileSize > 10240) bufferSize = 4096;
			domParser.setProperty(Constants.XERCES_PROPERTY_PREFIX + Constants.BUFFER_SIZE_PROPERTY, new Integer(bufferSize));
			entityResolver.setBaseFile(xmlFile);
			InputSource inputSource = new InputSource(reader);
			inputSource.setSystemId(xmlFile.getAbsolutePath());
			domParser.parse(inputSource);
			xmlDocument = domParser.getDocument();

			if (documentComment != null)
				xmlDocument.insertBefore(xmlDocument.createComment(documentComment), xmlDocument.getDocumentElement());

			((DTDHelperXML4J)dtdHelper).postInitHack(xmlDocument);

			showCommentNodes = showCDATANodes = showTextNodes = showDocumentTypeNode = showPINodes = showDocumentNode = true; // todo
			filterXMLNodes = null;

			internalMaxLinkID = LinkIDConstants.FIRST_ID - 1;

			docTreeNode = createTreeNode(xmlDocument);
			docElemTreeNode = getTreeNode4XMLNode(xmlDocument.getDocumentElement());

			xmlTreeCellRenderer.setHighlightedTreeNodes(null);
			xmlTreeCellRenderer.setLinksVisualization(NO_VISUALIZATION);

			expandedTreePath = simpleTreeModel.getTreePath(docElemTreeNode);
			setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
			setRootVisible(true);
			if (expandAllOnLoad && !modeAutoCollapse) expandAll();
			else expandPath(expandedTreePath);

			if (this.xmlFile == null) {  // first load() since global xmlFile has not been set
				setBackground(Color.white);
				refresh();
			}

			if (domParser.getElement2ErrorMap().isEmpty())
				centerAndSelectTreeNode(docElemTreeNode);
			else
				centerAndSelectTreeNode(xmlNode2TreeNodeMap.get(domParser.getElement2ErrorMap().keySet().iterator().next()));

			this.xmlFile = xmlFile;
			empty = false;
			contentHasChanged(false);
		} catch (SAXException saxExc) {
			saxExc.printStackTrace(new PrintWriter(System.err, true));
			throw new Exception(
					"XML-Error occured while parsing \"" + xmlFile.getName() + "\":\n" + saxExc.getMessage());
		} catch (UTFDataFormatException utfExc) {
			utfExc.printStackTrace(new PrintWriter(System.err, true));
			throw new Exception("UTF-Error: Found bad character in \"" + xmlFile.getName() + "\"!");
		} catch (FileNotFoundException fnfExc) {
			fnfExc.printStackTrace(new PrintWriter(System.err, true));
			if (fnfExc.getMessage().indexOf(".dtd") != -1)
				throw new Exception("DTD-File could not be found! (\"" + fnfExc.getMessage() + "\")");
			else if (fnfExc.getMessage().indexOf(xmlFile.getName()) != -1)
				throw new Exception("XML-File could not be found! (\"" + fnfExc.getMessage() + "\")");
			else
				throw new Exception("Required File could not be found! (\"" + fnfExc.getMessage() + "\")");
		} catch (IOException ioExc) {
			ioExc.printStackTrace(new PrintWriter(System.err, true));
			throw new Exception(
					"I/O-Error occured while reading \"" + xmlFile.getName() + "\":\n" + ioExc.getMessage());
		} catch (Throwable err) {
			err.printStackTrace(new PrintWriter(System.err, true));
			throw new Exception("Unexpected error: \"" + err.toString() + "\"!");
		} finally {
			reader.close();
			requestFocusInWindow();
			setEnabled(true);
			isLoading = false;
			Log.trace("XMLView: exit load()");
		}
	}

	public void reload() throws Exception {
		if (selectionModel.getSelectionPath() == null) { load(xmlFile); return; } // should not happen but happend -> nothing was selected somehow

		Object[] paths = selectionModel.getSelectionPath().getPath();

		load(xmlFile);

		if (domParser.getElement2ErrorMap().isEmpty() && (paths.length > 1)) {
			TreePath pathToSelect = new TreePath(rootXMLTreeNode);
			TreeNode newParentTreeNode = rootXMLTreeNode;
			for (int i = 0; i < paths.length - 1; i++) {
				TreeNode oldParentTreeNode = (TreeNode) paths[i];
				XMLTreeNode oldChildTreeNode = (XMLTreeNode) paths[i + 1];
				int index = oldParentTreeNode.getIndex(oldChildTreeNode);
				if ((index < 0) || (index >= newParentTreeNode.getChildCount())) break; // if file has been changed
				XMLTreeNode newChildTreeNode = (XMLTreeNode) newParentTreeNode.getChildAt(index);
				if (newChildTreeNode.getNodeName().equals(oldChildTreeNode.getNodeName())) {
					pathToSelect = pathToSelect.pathByAddingChild(newChildTreeNode);
					newParentTreeNode = newChildTreeNode;
				} else
					break;
			}
			super.centerAndSelectTreePath(pathToSelect);
		}
	}

	public void save() throws IOException {
		saveAs(xmlFile);
	}

	public void saveAs(File newXMLFile) throws IOException {
		DocumentType dt = xmlDocument.getDoctype();
		String dtdPath;
		if (dt == null) dtdPath = null;
		else dtdPath = dt.getSystemId();
		if (dtdPath != null) {
			File oldDTDFile = FileUtils.toExistingAbsoluteFile(dtdPath, newXMLFile);
			if (oldDTDFile != null) {
				String newDTDPath = FileUtils.toRelativeFile(oldDTDFile, newXMLFile);
				DocumentType newDocType = new DocumentTypeImpl((CoreDocumentImpl) xmlDocument, dt.getName(), dt.getPublicId(), newDTDPath);
				xmlDocument.removeChild(dt);
				xmlDocument.insertBefore(newDocType, xmlDocument.getDocumentElement());
			}
		}

		Writer out = null;
		try {
			out = new BufferedWriter(new FileWriter(newXMLFile));
			XMLUtils.serializeNode(xmlDocument, out, true);
			xmlFile = newXMLFile;
			((CoreDocumentImpl) xmlDocument).setDocumentURI(newXMLFile.toURI().toString());
			contentHasChanged(false);
		} finally {
			if (out != null) out.close();
		}
	}

	public void close() {
		if (!empty) {
			clear();
			xmlFile = null;
			xmlDocument = null;
			rootXMLTreeNode = null;
			docTreeNode = null;
			docElemTreeNode = null;
			deltaLinkMap.clear();
			historyBackList.clear();
			historyForwardList.clear();
         contentHasChanged(false);
		}
	}

	public boolean print() {
		// TODO test printing
		SimpleFilePrinter sfp = new SimpleFilePrinter(xmlFile);
		if (sfp.setupPageFormat()) {
			if (sfp.setupJobOptions()) {
				try {
					sfp.printFile();
					return true;
				} catch (Exception e) {
					System.err.println(e.toString());
					JOptionPane.showMessageDialog(this, "Printing failed due to: "+e);
				}
			}
		}
		return false;
	}

	public boolean isLoaded() {
		return !empty;
	}

	public boolean contentHasChanged() {
		return contentHasChanged;
	}

	/**
	 * Notify all listeners that have registered interest for
	 * notification contents changes.
	 *
	 * @see SwingUtils#fireChangeEvent(EventListenerList,Object)
	 */
	public void contentHasChanged(boolean hasChanged) {
		contentHasChanged = hasChanged;
		SwingUtils.fireChangeEvent(listenerList, this);
	}

	public String getSourceHTML() {
		return XMLUtils.transformDOM2String(xmlDocument);
	}

	public String getSourcePlain() {
		return XMLUtils.serializeNode(xmlDocument, true, true);
	}

	protected boolean shouldInsertIntoTreeModel(Node xmlNode) {
		short nodeType = xmlNode.getNodeType();
		switch (nodeType) {
			case Node.DOCUMENT_TYPE_NODE:
				return showDocumentTypeNode;
			case Node.ELEMENT_NODE:
				if (xmlNode.getParentNode().getNodeType() == Node.DOCUMENT_NODE)
					return true;
				if (filterXMLNodes != null) {
				// if (XMLTreeNode.determineDELTALinkID(xmlNode) != null) return true;
					if (xmlNode.getParentNode().getParentNode().getNodeType() == Node.DOCUMENT_NODE)
						return (filterShow == filterXMLNodes.contains(xmlNode));
					if (filterXMLNodes.contains(xmlNode))
						return filterShow;
					return (xmlNode2TreeNodeMap.get(xmlNode.getParentNode()) != null);
				}
				return true;
			default:
                if (xmlNode.getParentNode().getNodeType() != Node.DOCUMENT_NODE)
				if (filterXMLNodes != null) {
					if (xmlNode.getParentNode().getParentNode().getNodeType() == Node.DOCUMENT_NODE)
						return (filterShow == filterXMLNodes.contains(xmlNode));
					if (filterXMLNodes.contains(xmlNode))
						return filterShow;
					return (xmlNode2TreeNodeMap.get(xmlNode.getParentNode()) != null);
				}
				switch (nodeType) {
					case Node.COMMENT_NODE:
						if (!showCommentNodes) return false;
						break;
					case Node.CDATA_SECTION_NODE:
						if (!showCDATANodes) return false;
						break;
					case Node.TEXT_NODE:
						if (!showTextNodes) return false;
						break;
					case Node.PROCESSING_INSTRUCTION_NODE:
						if (!showPINodes) return false;
						break;
				}
		}
		return super.shouldInsertIntoTreeModel(xmlNode);
	}

	private void rebuildTreeModelAfterFilter() {
		setEnabled(false);
		Node currentXMLNode = getSelectedTreeNode().getXMLNode();
		deltaLinkMap.clear();
		internalMaxLinkID = LinkIDConstants.FIRST_ID - 1;
		docTreeNode.removeAll();
        xmlNode2TreeNodeMap.clear();
		xmlTreeCellRenderer.setLinksVisualization(NO_VISUALIZATION);
		xmlTreeCellRenderer.setHighlightedTreeNodes(null);

		buildTreeModel(docTreeNode, xmlDocument);
	// internalMaxLinkID = Math.max(internalMaxLinkID, externalMaxLinkID) + 1;
	// newLinkID = Math.max(internalMaxLinkID, externalMaxLinkID) + 1;

		docElemTreeNode = getTreeNode4XMLNode(xmlDocument.getDocumentElement());
		if (showDocumentNode) rootXMLTreeNode = docTreeNode;
		else rootXMLTreeNode = docElemTreeNode;
	// simpleTreeModel.setRoot(rootXMLTreeNode);
		simpleTreeModel.reload();

		if (expandAllOnLoad) expandAll();
		else expandPath(simpleTreeModel.getTreePath(docElemTreeNode));

		TreeNode treeNodeToSelect = xmlNode2TreeNodeMap.get(currentXMLNode);
		if (treeNodeToSelect == null) {
			for (Iterator iter = historyBackList.listIterator(0); iter.hasNext();) {
				TreePath treePath = (TreePath) iter.next();
				XMLTreeNode xmlTreeNode = (XMLTreeNode) treePath.getLastPathComponent();
				Node xmlNode = xmlTreeNode.getXMLNode();
				if (xmlNode2TreeNodeMap.containsKey(xmlNode)) {
					treeNodeToSelect = xmlNode2TreeNodeMap.get(xmlNode);
					break;
				}
			}
			if (treeNodeToSelect == null)
				treeNodeToSelect = rootXMLTreeNode;
		}
		centerAndReselectTreeNode(treeNodeToSelect);

		historyBackList.clear();
		historyForwardList.clear();

		setEnabled(true);
	}

	public void currentXMLTreeNodeChanged() {
		xmlTreeNodeChanged(getSelectedTreeNode());
	}

	public void xmlTreeNodeChanged(XMLTreeNode xmlTreeNode) {
		if (xmlTreeNode != null) {
			xmlTreeNode.reset();
			simpleTreeModel.nodeChanged(xmlTreeNode);
			contentHasChanged(true);
		}
	}

	public void commentCurrentElement() {

		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		Node xmlNode = selectedTreeNode.getXMLNode();
		String elementSourceCode = XMLUtils.serializeNode(xmlNode, false);
		Comment commentNode = xmlDocument.createComment(elementSourceCode);
		insertNodeReplace(commentNode);
	}
	public void unCommentCurrentElement() {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		Node xmlNode = selectedTreeNode.getXMLNode();
		String commentString = ((Comment) xmlNode).getData();
		Node uncommentedNode = XMLUtils.stringToElement(commentString);
		insertNodeReplace(uncommentedNode);
	}

	public void insertNodeInto(String newNodeName) {
		insertNodeInto(selectionModel.getSelectionPath(), newNodeName,
							  dtdHelper.getLastInsertIntoIndex(newNodeName), null);
	}

	public void insertNodeBefore(String newNodeName) {
		insertNodeInto(selectionModel.getSelectionPath().getParentPath(), newNodeName,
							  dtdHelper.getInsertBeforeIndex(newNodeName), null);
	}

	public void insertNodeAfter(String newNodeName) {
		insertNodeInto(selectionModel.getSelectionPath().getParentPath(), newNodeName,
							  dtdHelper.getInsertAfterIndex(newNodeName), null);
	}

	public Set<Integer> insertNodeReplace(String newNodeName) {
		TreePath selectionPath = selectionModel.getSelectionPath();
		XMLTreeNode xmlTreeNode = (XMLTreeNode) selectionPath.getLastPathComponent();
		int indexDOM = XMLUtils.getIndexOfChildNode(xmlTreeNode.getXMLNode());
		Set<Integer> result = removeTreeNode(xmlTreeNode, selectionPath);
		insertNodeInto(selectionPath.getParentPath(), newNodeName, indexDOM, null);
		return result;
	}

	public void insertNodeConvert(String newElementName) {
		TreePath treePath = selectionModel.getSelectionPath();
		XMLTreeNode xmlTreeNode = (XMLTreeNode) treePath.getLastPathComponent();
		Node xmlNode = xmlTreeNode.getXMLNode();
		String oldElementName = xmlNode.getNodeName();
		((org.apache.xerces.dom.CoreDocumentImpl) xmlDocument).renameNode(xmlNode, null, newElementName);
		resetTreeNode(xmlTreeNode);
		contentHasChanged(true);
		SwingUtils.fireUndoableEditHappened(new NodeConvertedUndoableEdit(xmlNode, oldElementName, newElementName), this, listenerList);
	}

	public void insertNodeWrap(String newElementName) {
		TreePath treePath = selectionModel.getSelectionPath();
		XMLTreeNode xmlTreeNode = (XMLTreeNode) treePath.getLastPathComponent();
		Node wrappedNode = xmlTreeNode.getXMLNode();

		int insertPositionDOM = XMLUtils.getIndexOfChildNode(wrappedNode);
		Element wrapperNode = xmlDocument.createElement(newElementName);
		dtdHelper.addRequiredAttributes(wrapperNode, null);

		Node parentXMLNode = wrappedNode.getParentNode();
		parentXMLNode.removeChild(wrappedNode);

		simpleTreeModel.removeNodeFromParent(xmlTreeNode);
		xmlNode2TreeNodeMap.remove(xmlTreeNode.getXMLNode());

		XMLUtils.insertNodeAt(parentXMLNode, wrapperNode, insertPositionDOM);
		wrapperNode.appendChild(wrappedNode);

		TreePath parentTreePath = treePath.getParentPath();
		if (shouldInsertIntoTreeModel(wrapperNode)) {
			XMLTreeNode parentTreeNode = (XMLTreeNode) parentTreePath.getLastPathComponent();
			boolean wasLeaf = parentTreeNode.isLeaf();
			XMLTreeNode wrapperXMLTreeNode = createTreeNode(wrapperNode);
			simpleTreeModel.insertNodeInto(wrapperXMLTreeNode, parentTreeNode,
										   convertDOMNodeIndexToTreeNodeIndex(parentTreeNode.getXMLNode(), insertPositionDOM));
			parentTreePath = parentTreePath.pathByAddingChild(wrapperXMLTreeNode);
			expandPath(parentTreePath); // expandAllDescendants(parentTreePath);
			centerAndSelectTreePath(parentTreePath);
			if (wasLeaf)
				simpleTreeModel.nodeChanged(parentTreeNode);
		} else
			centerAndSelectTreePath(parentTreePath);

		SwingUtils.fireUndoableEditHappened(new NodeWrapUndoableEdit(wrappedNode, wrapperNode), this, listenerList);
		contentHasChanged(true);
	}


	// called on "Element-Macro" Insert
	public void insertNode(String newElementName, int insertPosition) {
		switch (dtdHelper.getInsertMode()) {
			case DTDHelper.INSERT_MODE_REPLACE:
				//   removeCurrentTreeNode(); // has to be done outside
			case DTDHelper.INSERT_MODE_INTO:
				insertNodeInto(selectionModel.getSelectionPath(), newElementName, insertPosition, null);
			case DTDHelper.INSERT_MODE_BEFORE:
			case DTDHelper.INSERT_MODE_AFTER:
				insertNodeInto(selectionModel.getSelectionPath().getParentPath(), newElementName,
									  insertPosition, null);
		}
	}

	public void insertNodeInto(String newElementName, int insertPosition) {
		insertNodeInto(selectionModel.getSelectionPath(), newElementName, insertPosition, null);
	}

	private void insertNodeInto(TreePath treePath, String newNodeName, int insertPosition, String initialTextValue) {
		Node newXMLNode;
		if (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT) {
		// if (initialTextValue == null) initialTextValue = SwingUtils.getClipBoardContentAsString();
			initialTextValue = ((initialTextValue == null) ? "" : XMLUtils.escapeCharacters(initialTextValue));
			String value = askForTextValue("Comment-Node", "Enter comment value: ", initialTextValue);
			if ((value == null) || value.equals("")) return;
			newXMLNode = xmlDocument.createComment(value);
		} else if (newNodeName == DTDHelper.SPECIAL_NODE_NAME_TEXT) {
		// if (initialTextValue == null) initialTextValue = SwingUtils.getClipBoardContentAsString();
			initialTextValue = ((initialTextValue == null) ? "" : XMLUtils.escapeCharacters(initialTextValue));
			String value = askForTextValue("Text-Node", "Enter text value: ", XMLUtils.escapeCharacters(initialTextValue));
			if ((value == null) || value.equals("")) return;
			newXMLNode = xmlDocument.createTextNode(value);
		} else
			newXMLNode = dtdHelper.createMinimalValidElement(newNodeName, initialTextValue);
		insertNodeInto(treePath, newXMLNode, insertPosition);
		if (newXMLNode.getNodeType() == Node.ELEMENT_NODE) {
			TreePath selPath = selectionModel.getSelectionPath();
			List<AttributeInputWrapper> attrsList = new LinkedList<AttributeInputWrapper>();
			buildAttrsList(newXMLNode, attrsList); // todo: use selectedText of other side
			boolean cancel = requestAttributeValues(attrsList, "Enter attribute values for inserted elements...");
			centerAndSelectTreePath(selPath);
            if (cancel)
                MainController.aEditUndo.actionPerformed(null); // todo: not nice hack
		}
	}

	private void buildAttrsList(Node newXMLNode, List<AttributeInputWrapper> attrsList) {
		NamedNodeMap attributes = newXMLNode.getAttributes();
		if (attributes != null) {
			for (int i = 0, length = attributes.getLength(); i < length; i++) {
				Attr attr = (Attr) attributes.item(i);
				AttributeInputWrapper aiw = new AttributeInputWrapper(
						simpleTreeModel.getTreePath(getTreeNode4XMLNode(newXMLNode)), attr.getOwnerElement(), attr, i);
				attrsList.add(aiw);
			}
			for (Node childNode = newXMLNode.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
				buildAttrsList(childNode, attrsList);
		}
	}

	public TreePath insertNodeInto(Node newXMLNode) {
		return insertNodeInto(selectionModel.getSelectionPath(), newXMLNode,
							  dtdHelper.getLastInsertIntoIndex(newXMLNode));
	}

	public TreePath insertNodeBefore(Node newXMLNode) {
        return insertNodeInto(selectionModel.getSelectionPath().getParentPath(), newXMLNode,
							  dtdHelper.getInsertBeforeIndex(newXMLNode));
	}

	public TreePath insertNodeAfter(Node newXMLNode) {
		return insertNodeInto(selectionModel.getSelectionPath().getParentPath(), newXMLNode,
							  dtdHelper.getInsertAfterIndex(newXMLNode));
	}

	public Set insertNodeReplace(Node newXMLNode) {
		TreePath selectionPath = selectionModel.getSelectionPath();
		XMLTreeNode xmlTreeNode = (XMLTreeNode) selectionPath.getLastPathComponent();
		int indexDOM = XMLUtils.getIndexOfChildNode(xmlTreeNode.getXMLNode());
		Set result = removeTreeNode(xmlTreeNode, selectionPath);
		insertNodeInto(selectionPath.getParentPath(), newXMLNode, indexDOM);
		return result;
	}

	public TreePath insertNodeInto(TreePath parentTreePath, Node newXMLNode, int insertPositionDOM) {
		XMLTreeNode parentTreeNode = (XMLTreeNode) parentTreePath.getLastPathComponent();
		Node parentXMLNode = parentTreeNode.getXMLNode();
        if ((insertPositionDOM < 0) || (insertPositionDOM > parentXMLNode.getChildNodes().getLength()))
            throw new IndexOutOfBoundsException(insertPositionDOM + " is not a valid insert-position!");

		if (newXMLNode.getOwnerDocument() != xmlDocument)
			newXMLNode = xmlDocument.importNode(newXMLNode, true);
		XMLUtils.insertNodeAt(parentXMLNode, newXMLNode, insertPositionDOM);
		if (shouldInsertIntoTreeModel(newXMLNode)) {
			XMLTreeNode newXMLTreeNode = createTreeNode(newXMLNode);
			simpleTreeModel.insertNodeInto(newXMLTreeNode, parentTreeNode,
										   convertDOMNodeIndexToTreeNodeIndex(parentTreeNode.getXMLNode(), insertPositionDOM));
			expandAllDescendants(newXMLTreeNode);
			parentTreePath = parentTreePath.pathByAddingChild(newXMLTreeNode);
			centerAndSelectTreePath(parentTreePath);
		}

		parentTreeNode.reset();
		simpleTreeModel.nodeChanged(parentTreeNode);

        SwingUtils.fireUndoableEditHappened(new NodeInsertUndoableEdit(newXMLNode), this, listenerList);
		contentHasChanged(true);
        return parentTreePath;
	}

	public void duplicateCurrentNode() {
		TreePath treePath = selectionModel.getSelectionPath();
		XMLTreeNode sourceXMLTreeNode = (XMLTreeNode) treePath.getLastPathComponent();
		Node sourceXMLNode = sourceXMLTreeNode.getXMLNode();
		TreePath parentTreePath = treePath.getParentPath();
		Node newXMLNode = sourceXMLNode.cloneNode(true);
		Node parentXMLNode = sourceXMLNode.getParentNode();
		parentXMLNode.insertBefore(newXMLNode, sourceXMLNode.getNextSibling());
		if (shouldInsertIntoTreeModel(newXMLNode)) {
			XMLTreeNode newXMLTreeNode = createTreeNode(newXMLNode);
			simpleTreeModel.insertNodeInto(newXMLTreeNode, sourceXMLTreeNode.getSimpleMutableTreeNodeParent(),
										   convertDOMNodeIndexToTreeNodeIndex(parentXMLNode, XMLUtils.getIndexOfChildNode(newXMLNode)));
			treePath = parentTreePath.pathByAddingChild(newXMLTreeNode);
			expandAllDescendants(newXMLTreeNode, treePath);
			centerAndSelectTreePath(treePath);
		}
		SwingUtils.fireUndoableEditHappened(new NodeInsertUndoableEdit(newXMLNode), this, listenerList);
		contentHasChanged(true);
	}


	public int insertMacro(MacrosListItem.Macro macro, int insertPositionDOM, int selXMLLinkID,
						   String selText, Node selectedNodeOfOtherSide) {
		switch (dtdHelper.getInsertMode()) {
			case DTDHelper.INSERT_MODE_REPLACE:
				//  removeCurrentTreeNode(); // has to be done outside
			case DTDHelper.INSERT_MODE_INTO:
				return insertMacro(selectionModel.getSelectionPath(), macro, insertPositionDOM, selXMLLinkID,
								   selText, selectedNodeOfOtherSide);
			case DTDHelper.INSERT_MODE_BEFORE:
			case DTDHelper.INSERT_MODE_AFTER:
				return insertMacro(selectionModel.getSelectionPath().getParentPath(), macro,
								   insertPositionDOM, selXMLLinkID, selText, selectedNodeOfOtherSide);
		}
		return -1; // shouldn't happen
	}

	public int insertMacro(TreePath parentTreePath, MacrosListItem.Macro macro, int insertPositionDOM, int linkID,
						   String initialTextValue, Node selectedNodeOfOtherSide) {
		if (empty || (macro == null) || (insertPositionDOM == -1)) return -1;
		if (parentTreePath == null) return -1;  // is a treenode selected ?
		XMLTreeNode xmlTreeNode = (XMLTreeNode) parentTreePath.getLastPathComponent();
		Node parentNode = xmlTreeNode.getXMLNode();
		if (parentNode.getNodeType() != Node.ELEMENT_NODE) return -1;

		initialTextValue = ((initialTextValue == null) ? "" : XMLUtils.escapeCharacters(initialTextValue));
	// if (initialTextValue == null) initialTextValue = SwingUtils.getClipBoardContentsAsString();
	// initialTextValue = XMLUtils.escapeCharacters(initialTextValue);

		boolean wasLeaf = xmlTreeNode.isLeaf();

		Element macroXMLElement = (Element) xmlDocument.importNode(macro.getXMLElement(), true);
		List<XMLTreeNode> insertedLinks = new ArrayList<XMLTreeNode>();
		Node nextSibling;
		List<AttributeInputWrapper> attrsList = new LinkedList<AttributeInputWrapper>();
		Map<String, String> constantsMapLocal = new HashMap<String, String>();

		int insertPositionUI = convertDOMNodeIndexToTreeNodeIndex(parentNode, insertPositionDOM);
		TreePath firstInsertedTreePath = null;
		String macroName = macro.getName(); //macroXMLElement.getAttribute("name");
		if (macro.hasLink())
			if (linkID < LinkIDConstants.FIRST_ID)
				linkID = internalMaxLinkID = Math.max(internalMaxLinkID, externalMaxLinkID) + 1;
			else
				internalMaxLinkID = Math.max(internalMaxLinkID, linkID);

		Map<String, String> constantDefMap = macro.getConstantDefMap();
		if (constantDefMap != null)
			for (Iterator<Map.Entry<String, String>> iterator = constantDefMap.entrySet().iterator(); iterator.hasNext();) {
				Map.Entry<String, String> entry = iterator.next();
				String constantName = entry.getKey();
				String constantValue = entry.getValue();
				if (Character.isLowerCase(constantName.charAt(0)))
					constantsMapLocal.put(constantName, processMacroText(constantValue, initialTextValue, constantsMapLocal,
																		 parentNode, selectedNodeOfOtherSide, linkID));
				else
					if (!macroVariableMap.containsKey(constantName))
						macroVariableMap.put(constantName, processMacroText(constantValue, initialTextValue, constantsMapLocal,
																			parentNode, selectedNodeOfOtherSide, linkID));
			}

		for (Node childXMLNode = macroXMLElement.getFirstChild(); childXMLNode != null; childXMLNode = nextSibling) {
			nextSibling = childXMLNode.getNextSibling();
			//try {
			//     dtdHelper.addRequiredContentRecursive((Element)childXMLNode);  // todo ???
			XMLTreeNode newTreeNode = createTreeNode(childXMLNode);
			simpleTreeModel.insertNodeInto(newTreeNode, xmlTreeNode, insertPositionUI++);
			XMLUtils.insertNodeAt(parentNode, childXMLNode, insertPositionDOM++);
			TreePath treePath = parentTreePath.pathByAddingChild(newTreeNode);
			if (firstInsertedTreePath == null) firstInsertedTreePath = treePath;
			expandAllDescendants(treePath);
			processTreeNodesOnInsert(treePath, newTreeNode, insertedLinks, linkID, initialTextValue, attrsList,
									 constantsMapLocal, parentNode, selectedNodeOfOtherSide);

			SwingUtils.fireUndoableEditHappened(new NodeInsertUndoableEdit(childXMLNode), this, listenerList);
		}

		if (askAttributeValueForEach == ASK_ATTRVAL_FOR_EACH_MACRO)
			requestAttributeValues(attrsList, "Enter attribute values for Macro \"" + macroName + "\" ...");
		if (filterXMLNodes != null)
			activateFilter(filterXMLNodes, filterShow);
		if (xmlTreeNode.getChildCount() > 0) {
			centerAndSelectTreePath(firstInsertedTreePath);
			if (wasLeaf) simpleTreeModel.nodeChanged(xmlTreeNode);
		}
		contentHasChanged(true);
		if (insertedLinks.isEmpty())
			return LinkIDConstants.NO_LINK; // no link inserted
		SwingUtils.fireUndoableEditHappened(new MacroLinksInsertedUndoableEdit(linkID, insertedLinks), this, listenerList);

		deltaLinkMap.addAll(linkID, insertedLinks);
		return internalMaxLinkID;
	}

	private boolean requestAttributeValues(List<AttributeInputWrapper> attrsList, String title) {
		if (attrsList.isEmpty()) return false;
        boolean cancel;
		if (attrsList.size() == 1)
            cancel = doOneAttributeInput(attrsList.get(0));
		else {
			if (attributesInputDialog == null) // lazy init
				attributesInputDialog = new XMLAttributesInputDialog(properties, dtdHelper, this, attributesTable);
			List<String> attributeValueList = attributesInputDialog.showDialog(attrsList, title, true);
			updateAttributes(attrsList, attributeValueList);
            cancel = (attributeValueList == null);
		}
        return cancel;
	}

	static final class AttributeInputWrapper {
		final TreePath treePath;
		final Element element;
		final Attr attribute;
		final int attributeIndex;

		AttributeInputWrapper(TreePath treePath, Element element, Attr attribute, int attributeIndex) {
			this.treePath		= treePath;
			this.element		= element;
			this.attribute		= attribute;
			this.attributeIndex	= attributeIndex;
		}
	}

	private static final Pattern PATTERN_MACRO_INPUT  = Pattern.compile("%INPUT%");     //"%INPUT%(\\S*)" "%INPUT,?(.*?)%");
	private static final Pattern PATTERN_MACRO_CONSTANT_LOCAL = Pattern.compile("\\$([a-z0-9_]+?)(?:,(.+?))?\\$");
	private static final Pattern PATTERN_MACRO_CONSTANT_GLOBAL = Pattern.compile("\\$([A-Z0-9_]+?)(?:,(.+?))?\\$");
	private static final Pattern PATTERN_MACRO_COUNTER  = Pattern.compile("#([A-Z0-9_]*)?(?:-?(0+))?#"); // "#(\\p{Alpha}\\p{Alnum}_*)?(?:-?(0+))?#"
	private static final Pattern PATTERN_COUNTER = Pattern.compile("#([A-Z0-9_]*)?(?:-?([0-9]+))"); // "#(\\p{Alpha}\\p{Alnum}_*)?(?:-?([0-9]+))"
	private static final Pattern PATTERN_XPATH_SAME_FILE = Pattern.compile("\\{([^\\{\\}]+)\\}");
	private static final Pattern PATTERN_XPATH_OTHER_FILE = Pattern.compile("\\[([^\\[\\]]+)\\]");
	private static final Pattern PATTERN_MACRO_TEXT  = Pattern.compile("%TEXT%");
	private static final Pattern PATTERN_MACRO_DATE  = Pattern.compile("%DATE%");
	private static final Pattern PATTERN_MACRO_TIME  = Pattern.compile("%TIME%");
	private static final Pattern PATTERN_MACRO_USER  = Pattern.compile("%USER%");
	private static final Pattern PATTERN_MACRO_LINKID  = Pattern.compile("%LINK-ID%");

	// is used for attribute-ref, constant-def, text-node, comment-node
	private String processMacroText(String text, String textOfOtherSide, Map constantMapLocal,
									Node xpathRefNodeThisSide, Node xpathRefNodeOtherSide, int linkID) {
		// TODO: add constant and INPUT
		text = PATTERN_MACRO_LINKID.matcher(text).replaceAll(String.valueOf(linkID));
		text = PATTERN_MACRO_TEXT.matcher(text).replaceAll(textOfOtherSide);
		text = PATTERN_MACRO_DATE.matcher(text).replaceAll(DATE_STRING);
		text = PATTERN_MACRO_TIME.matcher(text).replaceAll(TIME_FORMAT.format(new Date()));
		text = PATTERN_MACRO_USER.matcher(text).replaceAll(MainController.getCurrentUserName());
		Matcher matcher = PATTERN_XPATH_SAME_FILE.matcher(text);
		while (matcher.find()) {
			String xpathExpr = matcher.group(1);
			String xpathResult = XMLUtils.evaluateXPathAsString(xpathExpr, xpathRefNodeThisSide);
			if (xpathResult == null) continue;
			// replace the current occurence; taken from Matcher.replaceFirst()
			StringBuffer sb = new StringBuffer();
			text = matcher.appendReplacement(sb, xpathResult).appendTail(sb).toString();
		}
		if (xpathRefNodeOtherSide != null) {
			matcher = PATTERN_XPATH_OTHER_FILE.matcher(text);
			while (matcher.find()) {
				String xpathExr = matcher.group(1);
				String xpathResult = XMLUtils.evaluateXPathAsString(xpathExr, xpathRefNodeOtherSide);
				if (xpathResult == null) continue;
				// replace the current occurence; taken from Matcher.replaceFirst()
				StringBuffer sb = new StringBuffer();
				text = matcher.appendReplacement(sb, xpathResult).appendTail(sb).toString();
			}
		}
		matcher = PATTERN_MACRO_COUNTER.matcher(text);
		while (matcher.find()) {
			String counterName = matcher.group(1);
			String counterDiv = "-";
			if (counterName == null) {
				counterName = "";
				counterDiv = "";
			}
			Integer i = macroCounterMap.get(counterName);
			int counterVal = (i == null) ? 1 :  i + 1;
			String counterPattern = matcher.group(2);
			if (counterPattern == null) counterPattern = "0";
			String counterText = new StringBuffer("#").append(counterName).append(counterDiv)
					.append(new DecimalFormat(counterPattern).format(counterVal)).toString();
			text = text.replaceAll(matcher.group(0), counterText);
			macroCounterMap.put(counterName, counterVal);
			SwingUtils.fireUndoableEditHappened(new MacroCounterInsertedUndoableEdit(counterName), this, listenerList);
		}
		return text;
	}

	private void processTreeNodesOnInsert(TreePath treePath, XMLTreeNode xmlTreeNode,
										  List<XMLTreeNode> linkTreeNodes, int linkID, String textOfOtherSide,
										  List<AttributeInputWrapper> attrsList, Map<String, String> constantMapLocal, Node xpathRefNodeThisSide, Node xpathRefNodeOtherSide) {
		Node xmlNode = xmlTreeNode.getXMLNode();
		switch (xmlNode.getNodeType()) {
			case Node.TEXT_NODE:
			case Node.COMMENT_NODE: {
				String text = processMacroText(xmlNode.getNodeValue(), textOfOtherSide, constantMapLocal,
											   xpathRefNodeThisSide, xpathRefNodeOtherSide, linkID);
				Matcher matcher = PATTERN_MACRO_INPUT.matcher(text);
				if (matcher.find()) {
					String input = askForTextValue("Text input", "Enter text: ", textOfOtherSide);
					text = matcher.replaceAll(input);
				}
				xmlNode.setNodeValue(text);
				resetTreeNode(xmlTreeNode);
				break;
			}
			case Node.ELEMENT_NODE:
				Element xmlElement = (Element) xmlNode;
				if (XMLTreeNode.isLinkElementName(xmlElement.getNodeName()))
					linkTreeNodes.add(xmlTreeNode);
			if (xmlElement.hasAttributes()) {
				NamedNodeMap attributes = xmlElement.getAttributes();
				boolean attributesChanged = false, notSelected = true;
				final int attributesCount = attributes.getLength();
				for (int attrIndex = 0; attrIndex < attributesCount; attrIndex++) {
					Attr attribute = (Attr) attributes.item(attrIndex);
					String value = attribute.getValue();

						String newValue;
					//	if (value.indexOf("%LINK-ID%") != -1) {
					//		newValue = String.valueOf(linkID);
					//		linkTreeNodes.add(xmlTreeNode);
					//	} else {
							newValue = processMacroText(value, textOfOtherSide, constantMapLocal,
														xpathRefNodeThisSide, xpathRefNodeOtherSide, linkID);

							Matcher matcher = PATTERN_MACRO_CONSTANT_GLOBAL.matcher(newValue);
							while (matcher.find()) {
								String varName = matcher.group(1);
								String varValue = macroVariableMap.get(varName);
								if (varValue == null) {
									attribute.setValue(newValue);
									xmlTreeNodeChanged(xmlTreeNode);
									attributesTable.refresh();
									if (notSelected) {
										centerTreePath(treePath);
										getSelectionModel().setSelectionPath(treePath);
										notSelected = false;
									}
									attributesTable.scrollRowToVisible(attrIndex);
									attributesTable.setSelectedRow(attrIndex);
                                    varValue = matcher.group(2);
                                    varValue = askForVariableValue(varName, (varValue == null) ? "" : varValue);
									if (varValue == null) varValue = "";
									else macroVariableMap.put(varName, varValue);
								}
								newValue = newValue.replaceAll("\\$" + varName + "\\$", varValue); //matcher.replaceAll(varValue);
							}
							matcher = PATTERN_MACRO_CONSTANT_LOCAL.matcher(newValue);
							while (matcher.find()) {
								String varName = matcher.group(1);
								String varValue = constantMapLocal.get(varName);
								if (varValue == null) {
									attribute.setValue(newValue);
									xmlTreeNodeChanged(xmlTreeNode);
									attributesTable.refresh();
									if (notSelected) {
										centerTreePath(treePath);
										getSelectionModel().setSelectionPath(treePath);
										notSelected = false;
									}
									attributesTable.scrollRowToVisible(attrIndex);
									attributesTable.setSelectedRow(attrIndex);
                                    varValue = matcher.group(2);
									varValue = askForVariableValue(varName, (varValue == null) ? "" : varValue);
									if (varValue == null)
										varValue = "";
									else
										constantMapLocal.put(varName, varValue);
								}
								newValue = newValue.replaceAll("\\$" +varName+ "\\$", varValue); // matcher.group(0) ... $var1$ //newValue = matcher.replaceAll(varValue);
							}

							matcher = PATTERN_MACRO_INPUT.matcher(newValue);
							if (matcher.find()) {
                                newValue = matcher.replaceAll("").trim();
                                attribute.setValue(newValue); //.substring(matcher.end() + 1).trim());
								if (askAttributeValueForEach == ASK_ATTRVAL_FOR_EACH_ATTRIBUTE) {
									if (notSelected) {
                                        centerAndSelectTreePath(treePath);
										notSelected = false;
									}
									xmlTreeNodeChanged(xmlTreeNode);
									attributesTable.refresh();
									attributesTable.scrollRowToVisible(attrIndex);
									attributesTable.setSelectedRow(attrIndex);
								// attribute.setValue("");
									newValue = askForOneAttribute(xmlElement, attribute.getName(), null, dtdHelper, false); //(matcher.groupCount() > 0) ? matcher.group(1) : textOfOtherSide, dtdHelper);
									attributesTable.refresh();
								// newValue = matcher.replaceAll(value);
								} else {
								// attribute.setValue(matcher.replaceAll((matcher.groupCount() > 0) ? matcher.group(1) : textOfOtherSide));
									attrsList.add(new AttributeInputWrapper(treePath, xmlElement, attribute, attrIndex));
								}
							}
					//	}
						if (!newValue.equals(value)) {
							attributesChanged = true;
							attribute.setValue(newValue);
						}
				} // for (int i = 0; i < attrCount; i++)
				if (attributesChanged) {
					xmlTreeNode.reset();
					simpleTreeModel.nodeChanged(xmlTreeNode);
					attributesTable.refresh();
				}
				if (askAttributeValueForEach == ASK_ATTRVAL_FOR_EACH_ELEMENT)
					requestAttributeValues(attrsList, "Enter attribute values for Element \"" + xmlElement.getNodeName() + "\" ...");
			}  // if (xmlElement.hasAttributes()) {
			if (askAttributeValueForEach == ASK_ATTRVAL_FOR_EACH_ELEMENT)
				attrsList.clear();

			List children = xmlTreeNode.getChildList();
			if ((children == null) || (children.size() == 0)) return;
			XMLTreeNode[] child = (XMLTreeNode[])children.toArray(new XMLTreeNode[children.size()]);
			for (int i = 0; i < child.length; i++)
				processTreeNodesOnInsert(treePath.pathByAddingChild(child[i]), child[i],
										 linkTreeNodes, linkID, textOfOtherSide, attrsList, constantMapLocal,
										 xpathRefNodeThisSide, xpathRefNodeOtherSide);
		}
	}

	private boolean doOneAttributeInput(AttributeInputWrapper wrapper) {
		Attr attribute = wrapper.attribute;
		if (wrapper.treePath != null)
			centerAndSelectTreePath(wrapper.treePath);
		attributesTable.getSelectionModel().setAnchorSelectionIndex(wrapper.attributeIndex);
		boolean cancel = (askForOneAttribute(wrapper.element, attribute.getName(), attribute.getValue(), dtdHelper, true) == null);
		attributesTable.refresh();
		if (wrapper.treePath != null)
			xmlTreeNodeChanged((XMLTreeNode) wrapper.treePath.getLastPathComponent());
		else
			contentHasChanged(true);
        return cancel;
	}

	private void updateAttributes(List<AttributeInputWrapper> attrsList, List<String> attributeValueList) {
		Iterator<AttributeInputWrapper> attrsIter = attrsList.iterator();
		Iterator<String> valuesIter = ((attributeValueList == null) ? null : attributeValueList.iterator());
		while (attrsIter.hasNext()) {
			AttributeInputWrapper wrapper = attrsIter.next();
			Attr attribute = wrapper.attribute;
			String newValue = ((valuesIter == null) ? "" : valuesIter.next());
			if (!insertEmptyAttributes && (newValue.length() == 0) &&
					(dtdHelper.getAttributeDefaultType(attribute) != DTDHelper.REQUIRED))
				wrapper.element.removeAttributeNode(attribute);
			else
				attribute.setValue(newValue);
			if (wrapper.treePath != null) {
				XMLTreeNode xmlTreeNode = (XMLTreeNode) wrapper.treePath.getLastPathComponent();
				xmlTreeNode.reset();
				simpleTreeModel.nodeChanged(xmlTreeNode);
			}
		}
		contentHasChanged(true);
		attributesTable.refresh();
	}

	public String askForOneAttribute(Element element, String attributeName, String initialValue,
									 DTDHelper dtdHelper, boolean allowCancel) {
		Object[] selectionValues = null;

		Attr attributeNode = element.getAttributeNode(attributeName);
		if (attributeNode == null) {
			element.setAttribute(attributeName, ""); // needed for XMLUtils.matchNodeWithXPath(attributeNode, linkedAttribute.getTargetAttributeXPath())
			attributeNode = element.getAttributeNode(attributeName);
		}
		XmlLanguage currentXMLLanguage = MainController.getActiveSideController().currentXMLLanguage;
        LinkedAttribute linkedAttribute = null;
		if (currentXMLLanguage != null) {
			LinkedAttributes linkedAttributes = currentXMLLanguage.getLinkedAttributes();
			if (linkedAttributes != null) {
				List linkedAttributeList = linkedAttributes.getLinkedAttributeList();
				for (int i = 0, size = linkedAttributeList.size(); i < size; i++) {
					linkedAttribute = (LinkedAttribute) linkedAttributeList.get(i);
					if (XMLUtils.matchNodeWithXPath(attributeNode, linkedAttribute.getTargetAttributeXPath())) {
						List<String> values = XMLUtils.evaluateXPathSilentlyAsStringList(linkedAttribute.getSourceXPath(), element.getOwnerDocument());
						if (values.isEmpty()) continue;
						Collections.sort(values, String.CASE_INSENSITIVE_ORDER);
						selectionValues = values.toArray(new String[values.size()]);
						break;
					}
				}
			}
		}

		int attributeValueType = dtdHelper.getAttributeValueType(element, attributeName);
        if (selectionValues == null)
	        if ((attributeValueType == DTDHelper.IDREF) || (attributeValueType == DTDHelper.IDREFS)) {
	            Set set = MiscUtils.enumerationToSortedSet(((CoreDocumentImpl) element.getOwnerDocument()).getIdentifiers());
	            selectionValues = set.toArray(new String[set.size()]);
	        } else
	            selectionValues = dtdHelper.getEnumValues(element, attributeName);
        if (initialValue == null) 
            initialValue = attributeNode.getNodeValue();//element.getAttribute(attributeName);

		String newValue;
		if (selectionValues != null)
			newValue = AutoCompleteComboBoxEditDialog.showEditDialog("Enter the value for the attribute \"" +
					attributeName + "\" of the Element \"" + element.getNodeName() + "\": ",
					"Enter attribute value ...", selectionValues, initialValue,
					(linkedAttribute == null) || linkedAttribute.getRestrictValues().equals("yes"));
		else
			for (;;) {
				newValue = askForTextValue("Enter attribute value ...",
										   "Enter the value for the attribute \"" +
											attributeName + "\" of the Element \"" + element.getNodeName() + "\": ",
											initialValue);
				if (newValue == null) { newValue = ""; break;}
				newValue = newValue.trim();
                String msg = dtdHelper.checkAttribute(attributeNode, newValue);
                if (msg == null) break;
                JOptionPane.showMessageDialog(this, msg, "Invalid Attribute", JOptionPane.ERROR_MESSAGE);
            }
		if ((newValue.length() == 0) && (allowCancel ||
				(!insertEmptyAttributes && dtdHelper.isAttributeRemovable(element, attributeName)))) { // user cancelled input
			element.removeAttribute(attributeName);
			return null;
		}
		newValue = XMLUtils.escapeCharacters(newValue);
	//	element.setAttribute(attributeName, newValue);
		attributeNode.setValue(newValue);
		if (attributeValueType == DTDHelper.ID) {
		    ((CoreDocumentImpl) element.getOwnerDocument()).putIdentifier(newValue, element);
		    dtdHelper.addIdentifier(newValue, element);
		}
		return newValue;
	}


	public Set<Integer> synchronizeSelectedDELTALinks(int newLinkID) {
		Set<Integer> result = new HashSet<Integer>(1);
		result.add(synchronizeCurrentDELTALink(newLinkID));
		return result;
	}

	public void setLinkMode(boolean linkMode) { }
	public boolean getLinkMode() { return true; }

	/**
	 * Change the ID of the currently selected DELTA-Link node
	 * @param newLinkId the new DELTA-Link ID
	 * @return the old DELTA-Link ID,
	 *   or -1, if the current Node is no link node,
	 *   or -2, if the old equals the new link id
	 */
	public int synchronizeCurrentDELTALink(int newLinkId) {
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		int oldLinkID = synchronizeDELTALink(selectedTreeNode, newLinkId);
		SwingUtils.fireUndoableEditHappened(new XMLLinkIDChangedUndoableEdit(selectedTreeNode, newLinkId, oldLinkID), this, listenerList);
		contentHasChanged(true);
		return oldLinkID;
	}
	/**
	 * Change the ID of the DELTA-Link node
	 * @param xmlTreeNode TreeNode of the DELTA-Link
	 * @param newLinkId the new DELTA-Link ID
	 * @return the old DELTA-Link ID,
	 *   or LinkIDConstants.NO_LINK, if the current Node is no link node,
	 *   or LinkIDConstants.BAD_LINK, if the old equals the new link id
	 */
	private int synchronizeDELTALink(XMLTreeNode xmlTreeNode, int newLinkId) {
		if (xmlTreeNode == null) return -1;
		int oldLinkID = xmlTreeNode.getDELTALinkID();
		if (oldLinkID < LinkIDConstants.FIRST_ID) return LinkIDConstants.NO_LINK;
		if (oldLinkID == newLinkId) return LinkIDConstants.BAD_LINK;
		Element xmlNode = (Element) xmlTreeNode.getXMLNode();
		Attr attr = xmlNode.getAttributeNode(XMLTreeNode.DELTA_LINK_ATTRIBUTE_NAME);
		deltaLinkMap.changeKey(oldLinkID, newLinkId, xmlTreeNode);
		attr.setValue(Integer.toString(newLinkId));
		xmlTreeNode.reset();
		simpleTreeModel.nodeChanged(xmlTreeNode);
		attributesTable.refresh();

		if (newLinkId > internalMaxLinkID)
			internalMaxLinkID = newLinkId;
		else if (oldLinkID == internalMaxLinkID)
			while (!deltaLinkMap.containsKey(internalMaxLinkID))
				internalMaxLinkID--;

		return oldLinkID;
	}

	/* overrides processElementTreeNodeOnCreation from SimpleXMLTree
will be called when new xml file is loaded */
	protected void processTreeNodeOnCreation(XMLTreeNode xmlTreeNode) {
		switch (xmlTreeNode.getXMLNodeType()) {
			case Node.ELEMENT_NODE:
				int id = xmlTreeNode.getDELTALinkID();
				if (id >= LinkIDConstants.FIRST_ID) {
					deltaLinkMap.add(xmlTreeNode.getDELTALinkIDWrapper(), xmlTreeNode);
					internalMaxLinkID = Math.max(internalMaxLinkID, id);
				}

				NamedNodeMap attributes = xmlTreeNode.getXMLNode().getAttributes();
				for (int i=0, size=attributes.getLength(); i<size; i++) {
					Attr attribute = (Attr) attributes.item(i);
					String value = attribute.getValue();
					Matcher matcher = PATTERN_COUNTER.matcher(value);
					while (matcher.find()) {
						String counterValStr = matcher.group(2);
						if (counterValStr != null) {
							try {
								int thisCounterVal = Integer.parseInt(counterValStr);
								String counterName = matcher.group(1);
								if (counterName == null) counterName = "";
								Integer val = macroCounterMap.get(counterName);
								int newCounterVal = thisCounterVal;
								if (val != null) newCounterVal = Math.max(val, thisCounterVal);
								macroCounterMap.put(counterName, newCounterVal);
							} catch (NumberFormatException e) { }
						}
					}
				}
			case Node.DOCUMENT_NODE:
			case Node.DOCUMENT_TYPE_NODE:
				ErrorWrapper errorWrapper = domParser.getElement2ErrorMap().get(xmlTreeNode.getXMLNode());
				xmlTreeNode.setErrorWrapper(errorWrapper);
		}
	}

	public boolean removeDELTALink(int gmtLinkID) {
		CompoundEdit compoundEdit = new XMLLinksRemovedUndoableEdit();
		if (removeDELTALink(gmtLinkID, compoundEdit)) {
            compoundEdit.end();
            SwingUtils.fireUndoableEditHappened(compoundEdit, this, listenerList);
            contentHasChanged(true);
            return true;
        }
        return false;
	}

	// through the usage of this method, it is guaranteed that gmtLinks is not empty and that all entries (link IDs) exists
	public void removeDELTALinks(Set<Integer> gmtLinks) {
		CompoundEdit compoundEdit = new XMLLinksRemovedUndoableEdit();
		Iterator<Integer> iter = gmtLinks.iterator();
		for (int i = gmtLinks.size() - 1; i >= 0; i--)
			removeDELTALink(iter.next(), compoundEdit);
		compoundEdit.end();
		SwingUtils.fireUndoableEditHappened(compoundEdit, this, listenerList);
		contentHasChanged(true);
	}

	private boolean removeDELTALink(Integer gmtLinkID, CompoundEdit compoundEdit) {
		Collection linkTreeNodes = deltaLinkMap.get(gmtLinkID);
		if ((linkTreeNodes == null) || linkTreeNodes.isEmpty()) return false;
		XMLTreeNode selectedTreeNode = getSelectedTreeNode();
		Iterator iter = linkTreeNodes.iterator();
		for (int i = linkTreeNodes.size() - 1; i >= 0; i--){
			XMLTreeNode xmlTreeNode = (XMLTreeNode) iter.next();
			MutableTreeNode parentTreeNode = (MutableTreeNode) xmlTreeNode.getParent();
			int index = parentTreeNode.getIndex(xmlTreeNode);
			Node xmlNode = xmlTreeNode.getXMLNode();

			NodeRemovedUndoableEdit treeNodeRemovedUndoableEdit = new NodeRemovedUndoableEdit(xmlNode);
			compoundEdit.addEdit(treeNodeRemovedUndoableEdit);

			xmlNode.getParentNode().removeChild(xmlNode);
			simpleTreeModel.removeNode(parentTreeNode, index);
			if (selectedTreeNode == xmlTreeNode)
				setSelectedTreeNode(parentTreeNode);
		}
		deltaLinkMap.remove(gmtLinkID);
        return true;
	}

	public Set<Integer> removeCurrentTreeNode() {
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath == null) return null;
		XMLTreeNode xmlTreeNode = (XMLTreeNode) selPath.getLastPathComponent();
		return removeTreeNode(xmlTreeNode, selPath);
	}

	public Set<Integer> removeTreeNode(TreePath treePath) {
        return removeTreeNode((XMLTreeNode) treePath.getLastPathComponent(), treePath);
    }
	public Set<Integer> removeTreeNode(XMLTreeNode xmlTreeNode) {
		return removeTreeNode(xmlTreeNode, simpleTreeModel.getTreePath(xmlTreeNode));
	}

	private Set<Integer> removeTreeNode(XMLTreeNode xmlTreeNode, TreePath treePath) {
		Set<Integer> result = new TreeSet<Integer>();
		Node xmlNode = xmlTreeNode.getXMLNode();
		TreePath treePathToSelect;
        TreeNode parentTreeNode = xmlTreeNode.getParent();
        int childCount = parentTreeNode.getChildCount();
        TreePath parentPath = treePath.getParentPath();
        if (childCount == 1)
            treePathToSelect = parentPath;
        else {
            int index = parentTreeNode.getIndex(xmlTreeNode);
            if (index == childCount-1) treePathToSelect = parentPath.pathByAddingChild(parentTreeNode.getChildAt(index - 1));
            else treePathToSelect = parentPath.pathByAddingChild(parentTreeNode.getChildAt(index + 1));
        }

        createRemovedDELTALinkSet(xmlTreeNode, result);
        UndoableEdit undoableEdit = new NodeRemovedUndoableEdit(xmlNode);

		xmlNode.getParentNode().removeChild(xmlNode);
		simpleTreeModel.removeNodeFromParent(xmlTreeNode);
		centerAndSelectTreePath(treePathToSelect);
		historyBackList.remove(treePath);
		historyForwardList.remove(treePath);

        SwingUtils.fireUndoableEditHappened(undoableEdit, this, listenerList);
		contentHasChanged(true);
		return result;
	}

	private void createRemovedDELTALinkSet(XMLTreeNode xmlTreeNode, Set<Integer> links) {
		xmlNode2TreeNodeMap.remove(xmlTreeNode.getXMLNode());
		if (xmlTreeNode.getXMLNodeType() == Node.ELEMENT_NODE) {
			if (xmlTreeNode.isDELTALinkElement()) {
				Integer id = xmlTreeNode.getDELTALinkIDWrapper();
				deltaLinkMap.remove(id, xmlTreeNode);
				if (deltaLinkMap.isEmpty())
					internalMaxLinkID = LinkIDConstants.FIRST_ID - 1;
				else
					while (!deltaLinkMap.containsKey(internalMaxLinkID))
						internalMaxLinkID--;
				if (links != null)
					links.add(id);
			}
			List children = xmlTreeNode.getChildList();
			if ((children == null) || (children.size() == 0)) return;
			for (int i = 0, size = children.size(); i < size; i++) {
				XMLTreeNode child = (XMLTreeNode) children.get(i);
				createRemovedDELTALinkSet(child, links);
			}
		}
	}

	public boolean containsDELTALink(int deltaLinkID) {
		return empty ? false : deltaLinkMap.containsKey(deltaLinkID);
	}

	public int getDELTALinkCount(int deltaLinkID) {
		return empty ? 0 : deltaLinkMap.getListSize(deltaLinkID);
	}

	public String getDELTALinkContext(int deltaLinkID) {
		List<XMLTreeNode> treeNodes = deltaLinkMap.get(deltaLinkID);
		if ((treeNodes == null) || treeNodes.isEmpty()) return null;
		int index = (xmlTreeCellRenderer.getHighlightedTreeNodes() != treeNodes) ? 0 : currentHighlightIndex;
		return XMLUtils.getXPathForNode(treeNodes.get(index).getXMLNode().getParentNode());
	}

	public Set getAllLinkIDs() {
		return deltaLinkMap.keySet();
	}

	public void removeDELTALinksFromCollection(Collection<Integer> deltaLinkIDs, boolean contained) {
		if (empty) return;
		Iterator<Integer> iter = deltaLinkIDs.iterator();
		for (int i = deltaLinkIDs.size() - 1; i >= 0; i--)
			if (contained == deltaLinkMap.containsKey(iter.next()))
				iter.remove();
	}

	public void jumpToDELTALink(int xmlLinkID) {
		if (empty) return;
		if (xmlLinkID < LinkIDConstants.FIRST_ID)
			removeHighlights();
		else
			highlightTreeNodes(deltaLinkMap.get(xmlLinkID), linkHighlightColor);
	}

	public Set<Integer> getSelectedLinkIDs() {
		XMLTreeNode xmlTreeNode = getSelectedTreeNode();
		try {
		// List xmlNodes = XPATH_SUBTREE_LINKS.selectNodes(xmlTreeNode.getXMLNode());
			List xmlNodes = XMLUtils.evaluateXPath("descendant-or-self::" + XMLTreeNode.DELTA_LINK_ELEMENT_NAME + " | descendant-or-self::" + XMLTreeNode.GMT_LINK_ELEMENT_NAME, xmlTreeNode.getXMLNode());

			Set<Integer> result = new TreeSet<Integer>();
			for (int i = 0, size = xmlNodes.size(); i < size; i++) {
				Node node = (Node) xmlNodes.get(i);
				Integer gmtLinkID = XMLTreeNode.determineDELTALinkID(node);
				if (gmtLinkID != null) result.add(gmtLinkID);
			}
			return result;
		} catch (Exception exc) {
			return null;
		}
	}

	public void highlightDELTALinks(Collection<Integer> selectedLinkIDs) {
		List<XMLTreeNode> allTreeNodes = new ArrayList<XMLTreeNode>();
		for (Integer gmtLinkID : selectedLinkIDs) {
			List<XMLTreeNode> treeNodes = deltaLinkMap.get(gmtLinkID);
			if ((treeNodes != null) && !treeNodes.isEmpty())
				allTreeNodes.addAll(treeNodes);
		}
		if (!allTreeNodes.isEmpty())
			highlightTreeNodes(allTreeNodes, linkHighlightColor);
	}

	public void highlightAllDELTALinks() {
		highlightTreeNodes(deltaLinkMap.getAll(), linkHighlightColor);
	}

	public void highlightSameDELTALinks() {
		highlightTreeNodes(deltaLinkMap.getList(getCurrentDELTALinkID(true)), linkHighlightColor);
	}

	public void highlightDeadDELTALinks(Collection<Integer> linksOfOtherSide) {
		Set<Integer> keys = new HashSet<Integer>(deltaLinkMap.keySet());
		keys.removeAll(linksOfOtherSide);
		highlightDELTALinks(keys);
	}

	public void highlightTreeNodes(List<XMLTreeNode> treeNodes) {
		highlightTreeNodes(treeNodes, searchHighlightColor);
	}

	public void highlightTreeNodes(List<XMLTreeNode> treeNodes, Color highlightColor) {
		if ((treeNodes != null) && !treeNodes.isEmpty()) {
			boolean newList = (xmlTreeCellRenderer.getHighlightedTreeNodes() != treeNodes);
			if (newList)
				currentHighlightIndex = 0;
			else
				currentHighlightIndex = (currentHighlightIndex + 1) % treeNodes.size();
			if (newList) {
			//	System.out.println(treeNodes.size() + " tree nodes are highlighted.");
				xmlTreeCellRenderer.setHighlightedTreeNodes(treeNodes);
				xmlTreeCellRenderer.setHighLightColor(highlightColor);
                for (int i = 0, size = treeNodes.size(); i < size; i++)
                    makeVisible(simpleTreeModel.getTreePath(treeNodes.get(i)));
				repaint();
			}
            TreeNode treeNode = treeNodes.get(currentHighlightIndex);
            TreePath treePath = simpleTreeModel.getTreePath(treeNode);
			if (treePath.getPathComponent(0) != rootXMLTreeNode)
				setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
            centerTreePath(treePath);
            if (selectCurrentHighlightedTreeNode)
                selectionModel.setSelectionPath(treePath);
			requestFocusInWindow();
		} else
			removeHighlights();
	}


	public void expandTreeNodes(List treeNodes) {
		if ((treeNodes != null) && !treeNodes.isEmpty()) {
			TreePath treePath = selectionModel.getSelectionPath();
			collapseAll();
			for (int i = 0, size = treeNodes.size(); i < size; i++)
				expandPath(simpleTreeModel.getTreePath((TreeNode) treeNodes.get(i)));
			while (!isVisible(treePath)) treePath = treePath.getParentPath();
				centerAndSelectTreePath(treePath);
		}
	}

	public void collapseTreeNodes(List treeNodes) {
		if ((treeNodes != null) && !treeNodes.isEmpty()) {
			TreePath selPath = selectionModel.getSelectionPath();
			expandAll();
			for (int i = 0, size = treeNodes.size(); i < size; i++)
				collapsePath(simpleTreeModel.getTreePath((TreeNode) treeNodes.get(i)));
			if (!isCollapsed(selPath))
				centerAndSelectTreePath(selPath);
		}
	}

	public void removeHighlights() {
		if ((xmlTreeCellRenderer.getHighlightedTreeNodes() != null)) {
			xmlTreeCellRenderer.setHighlightedTreeNodes(null);
			repaint();
		}
	}

	protected void installKeyboardActions() {
		super.installKeyboardActions();
		ActionMap actionMap = getActionMap();
		InputMap inputMap = getInputMap();

		inputMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "removeHighlights");
		actionMap.put("removeHighlights", new AbstractAction() {
			public void actionPerformed(ActionEvent evt) {
				removeHighlights();
				centerTreePath(selectionModel.getSelectionPath());
			}
		});
	}

	public void centerAndSelectTreePath(TreePath treePath) {
		if (treePath.getPathComponent(0) != rootXMLTreeNode)
			setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
		removeHighlights();
		super.centerAndSelectTreePath(treePath);
		requestFocusInWindow();
	}
	public void centerAndReselectTreePath(TreePath treePath) {
		removeHighlights();
		centerTreePath(treePath);
		reselectTreePath(treePath);
	}
	public void centerAndReselectTreeNode(TreeNode treeNode) {
		centerAndSelectTreePath(simpleTreeModel.getTreePath(treeNode));
	}

	public Document getDocumentNode() {
		return xmlDocument;
	}

	public void reselectCurrentTreeNode() {
		TreePath selectionPath = selectionModel.getSelectionPath();
		reselectTreePath(selectionPath);
	}
	public void reselectTreeNode(TreeNode treeNode) {
		TreePath treePath = simpleTreeModel.getTreePath(treeNode);
		reselectTreePath(treePath);
	}
	public void reselectTreePath(TreePath treePath) {
		if (treePath != null) {
			addToHistory = false;
			selectionModel.setSelectionMode(SingleTreeSelectionModel.RESELECTION_TEMPORARILY_ENABLED);
			selectionModel.setSelectionPath(treePath);
			addToHistory = true;
		}
	}

	// called from TreePathBar
	public void jumpToTreePath(TreePath treePath) {
		centerAndSelectTreePath(treePath);
		if (COLLAPSE_AFTER_JUMP_TO_TREEPATH)
			collapseAllDescendants(treePath, false);
	}


// setter/getter methods
	public File getFile() {
		return xmlFile;
	}

	public String getFilePath() {
		return ((xmlFile == null) ? null : xmlFile.getAbsolutePath());
	}

	public String getDTDURLString() {
		if (xmlDocument == null) return null;
		DocumentType dt = xmlDocument.getDoctype();
		if (dt == null) return null;
		String dtdPath = dt.getSystemId();
		if (dtdPath == null) return null;
		if (!dtdPath.startsWith("/") && (dtdPath.indexOf(':') == -1)) // relative!
			try {
				URL u = xmlFile.toURI().toURL();
				dtdPath = new URL(u, dtdPath).toExternalForm();
			} catch (MalformedURLException e) { }
		return dtdPath;
	}

	public URL getDTDURL() {
		if (xmlDocument == null) return null;
		DocumentType dt = xmlDocument.getDoctype();
		if (dt == null) return null;
		return FileUtils.toURL(dt.getSystemId(), xmlFile);
	}

	public boolean isShowDocumentNode() {
		return showDocumentNode;
	}

	public void setShowDocumentNode(boolean showDocumentNode) {
		this.showDocumentNode = showDocumentNode;
		setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
		if (expandAllOnLoad) expandAll();
		else expandPath(simpleTreeModel.getTreePath(docElemTreeNode));
	}

	public void setCurrentNodeAsRootTreeNode() {
		setRootTreeNode(getSelectedTreeNode());
	}

	private void setRootTreeNode(XMLTreeNode rootXMLTreeNode) {
		this.rootXMLTreeNode = rootXMLTreeNode;
		simpleTreeModel.setRoot(rootXMLTreeNode);
		expandedTreePath = simpleTreeModel.getTreePath((TreeNode) expandedTreePath.getLastPathComponent());
		selectionModel.setSelectionPath(new TreePath(rootXMLTreeNode));
		expandAll();
	}

	public Color getCurrentHighLightColor() {
		return xmlTreeCellRenderer.getHighLightColor();
	}

	public void setCurrentHighLightColor(Color highlightColor) {
		xmlTreeCellRenderer.setHighLightColor(highlightColor);
		if (xmlTreeCellRenderer.getHighlightedTreeNodes() != null)
			repaint();
	}

	public Color getSelectionColor() {
		return xmlTreeCellRenderer.backgroundSelectionColor;
	}

	public void setSelectionColor(Color defaultSelectionColor) {
		xmlTreeCellRenderer.backgroundSelectionColor = defaultSelectionColor;
		repaint();
	}

	public void setDELTALinksVisualization(int gmtLinksVisualization) {
		if (gmtLinksVisualization != xmlTreeCellRenderer.getLinksVisualization()) {
			xmlTreeCellRenderer.setLinksVisualization(gmtLinksVisualization);
			xmlTreeCellRenderer.setHighlightedTreeNodes(null);
			xmlTreeCellRenderer.setHighLightColor(linkHighlightColor);
			repaint();
		}
	}

	public int getDELTALinksVisualization() {
		return xmlTreeCellRenderer.getLinksVisualization();
	}

	public boolean isFireXMLHyperLinkEventSingleClick() {
		return (fireXMLHyperLinkEventMouseClickCount == 1);
	}

	public void setFireXMLHyperLinkEventSingleClick(boolean fireXMLHyperLinkEventSingleClick) {
		fireXMLHyperLinkEventMouseClickCount = (fireXMLHyperLinkEventSingleClick ? 1 : 2);
	}

	protected void setProperties(boolean refetch) {
		super.setProperties(refetch);
		linkHighlightColor					 = properties.getColor(Preferences.KEY_COLOR_LINK_ACTIVATED_BG);
		searchHighlightColor				 = properties.getColor(Preferences.KEY_COLOR_SEARCH_RESULT);
		fireXMLHyperLinkEventMouseClickCount = (properties.getBoolean(Preferences.KEY_XML_LINK_SINGLECLICK)	? 1	: 2);
		expandAllOnLoad						 = properties.getBoolean(Preferences.KEY_XML_TREE_EXPAND_ALL_ON_LOAD);
		showDocumentNode					 = properties.getBoolean(Preferences.KEY_XML_FILTER_ROOT_ELEMENT[thisSideIndex]);
		showDocumentTypeNode				 = properties.getBoolean(Preferences.KEY_XML_FILTER_DOCTYPE_NODE[thisSideIndex]);
		showCommentNodes					 = properties.getBoolean(Preferences.KEY_XML_FILTER_COMMENT_NODES[thisSideIndex]);
		showTextNodes						 = properties.getBoolean(Preferences.KEY_XML_FILTER_TEXT_NODES[thisSideIndex]);
		showCDATANodes						 = properties.getBoolean(Preferences.KEY_XML_FILTER_CDATA_NODES[thisSideIndex]);
		showPINodes							 = properties.getBoolean(Preferences.KEY_XML_FILTER_PI_NODES[thisSideIndex]);
		askAttributeValueForEach			 = properties.getInt(Preferences.KEY_XML_ASK_ATTR_FOR_EACH,	ASK_ATTRVAL_FOR_EACH_ELEMENT);
		insertEmptyAttributes				 = properties.getBoolean(Preferences.KEY_XML_INSERT_EMPTY_ATTR);
		selectCurrentHighlightedTreeNode	 = properties.getBoolean(Preferences.KEY_XML_SELECT_ON_HIGHLIGHT);
		setDragAndDrop(properties.getBoolean(Preferences.KEY_XML_TREE_DRAGNDROP));
	}

	public void refetchProperties(Set<String> changedProperties) {
		super.refetchProperties(changedProperties);
		if (attributesInputDialog != null)
			attributesInputDialog.resetProperties();
	}

	public void storeProperties() {
		properties.putBoolean(Preferences.KEY_XML_FILTER_ROOT_ELEMENT[thisSideIndex], showDocumentNode);
		properties.putBoolean(Preferences.KEY_XML_FILTER_DOCTYPE_NODE[thisSideIndex], showDocumentTypeNode);
		properties.putBoolean(Preferences.KEY_XML_FILTER_COMMENT_NODES[thisSideIndex], showCommentNodes);
		properties.putBoolean(Preferences.KEY_XML_FILTER_TEXT_NODES[thisSideIndex], showTextNodes);
		properties.putBoolean(Preferences.KEY_XML_FILTER_CDATA_NODES[thisSideIndex], showCDATANodes);
		properties.putBoolean(Preferences.KEY_XML_FILTER_PI_NODES[thisSideIndex], showPINodes);
	}

	/**
	 * Returns the link ID of the currently selected TreeNode.
	 * @return the link ID, or
	 *   LinkIDConstants.NO_LINK if current element is not a link node, or no treenode is selected
	 * @param useCache
	 */
	public int getCurrentDELTALinkID(boolean useCache) {
		XMLTreeNode xmlTreeNode = getSelectedTreeNode();
		if (xmlTreeNode == null) return LinkIDConstants.NO_LINK;
		return xmlTreeNode.getDELTALinkID();
	}


// event listener methods
	public void addHyperlinkListener(HyperlinkListener listener) {
		listenerList.add(HyperlinkListener.class, listener);
	}

	public void removeHyperlinkListener(HyperlinkListener listener) {
		listenerList.remove(HyperlinkListener.class, listener);
	}

	/**
	 * Notify all listeners that have registered interest for
	 * notification on this event type.
	 */
	private void fireXMLHyperLinkEvent(HyperlinkEvent.EventType eventType, int gmtLinkID) {
		DeltaHyperlinkEvent evt = new DeltaHyperlinkEvent(this, eventType, gmtLinkID);
		fireXMLHyperLinkEvent(evt);

	}

	private void fireXMLHyperLinkEvent(DeltaHyperlinkEvent evt) {
		Object[] listeners = listenerList.getListenerList();

		// Process the listeners last to first, notifying those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2)
			if (listeners[i] == HyperlinkListener.class)
				((HyperlinkListener) listeners[i + 1]).hyperlinkUpdate(evt);
	}

	/**
	 * Adds a change listener for notification of any changes.
	 *
	 * @param listener the change listener
	 */
	public void addChangeListener(ChangeListener listener) {
		listenerList.add(ChangeListener.class, listener);
	}

	/**
	 * Removes a change listener.
	 *
	 * @param listener the change listener
	 */
	public void removeChangeListener(ChangeListener listener) {
		listenerList.remove(ChangeListener.class, listener);
	}

	protected void handleLeftMouseButtonClick(MouseEvent evt) {
		removeHighlights();
		if (evt.getClickCount() == fireXMLHyperLinkEventMouseClickCount) {
		    if (currentlyHoveredXMLTreeNode == null) return; // should not happen, but happened !?!
			int id = currentlyHoveredXMLTreeNode.getDELTALinkID();
			if (id >= LinkIDConstants.FIRST_ID) // link node?
				fireXMLHyperLinkEvent(new DeltaHyperlinkEvent(this, HyperlinkEvent.EventType.ACTIVATED, id, currentlyHoveredXMLTreeNode.getDELTALinkPrefix()));
		}
	}

	protected void handleMiddleMouseButtonClicked(MouseEvent evt) {
		if (xmlTreeCellRenderer.getHighlightedTreeNodes() == null)
			super.handleMiddleMouseButtonClicked(evt);
		else {
			if ((evt.getModifiers() & InputEvent.SHIFT_MASK) == InputEvent.SHIFT_MASK)
				jumpToPreviousHighlight();
			else
				jumpToNextHighlight();
		}
	}

	protected void handleDoubleClickOnTreeNode() {
		if (!currentlyHoveredXMLTreeNode.isLeaf()) {
			if (currentlyHoveredXMLTreeNode == rootXMLTreeNode)
				if (isExpanded(simpleTreeModel.getTreePath(docElemTreeNode)))
					collapseAll();
				else
					expandAll();
			else
				super.handleDoubleClickOnTreeNode();
		}
	}

	protected void hoveredTreeNodeChanged() {
		super.hoveredTreeNodeChanged();
		if (lastXMLTreeNode != null) {
			int id = lastXMLTreeNode.getDELTALinkID();
			if (id != LinkIDConstants.NO_LINK)
				fireXMLHyperLinkEvent(HyperlinkEvent.EventType.EXITED, id);
		}
		if (currentlyHoveredXMLTreeNode != null) {
			int id = currentlyHoveredXMLTreeNode.getDELTALinkID();
			if (id != LinkIDConstants.NO_LINK)
				fireXMLHyperLinkEvent(HyperlinkEvent.EventType.ENTERED, id);
		}
	}

	protected TreeCellRenderer createXMLTreeCellRenderer() {
		elementIconMap = new ElementIconMap();
		simpleXMLTreeCellRenderer = xmlTreeCellRenderer = new XMLTreeCellRenderer(elementIconMap);
		return xmlTreeCellRenderer;
	}

	public void jumpToNextHighlight() {
		List treeNodes = xmlTreeCellRenderer.getHighlightedTreeNodes();
		if (treeNodes != null) {
			currentHighlightIndex = (currentHighlightIndex >= treeNodes.size()-1) ? 0 : currentHighlightIndex+1;
			TreeNode treeNode = (TreeNode) treeNodes.get(currentHighlightIndex);
			TreePath treePath = simpleTreeModel.getTreePath(treeNode);

			if (treePath.getPathComponent(0) != rootXMLTreeNode)
				setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
			centerTreePath(treePath);
			if (selectCurrentHighlightedTreeNode)
				selectionModel.setSelectionPath(treePath);
		}
	}

	public void jumpToPreviousHighlight() {
		List treeNodes = xmlTreeCellRenderer.getHighlightedTreeNodes();
		if (treeNodes != null) {
			currentHighlightIndex = (currentHighlightIndex > 0) ? currentHighlightIndex-1 : treeNodes.size() - 1;
			TreeNode treeNode = (TreeNode) treeNodes.get(currentHighlightIndex);
			TreePath treePath = simpleTreeModel.getTreePath(treeNode);

			if (treePath.getPathComponent(0) != rootXMLTreeNode)
				setRootTreeNode(showDocumentNode ? docTreeNode : docElemTreeNode);
			centerTreePath(treePath);
			if (selectCurrentHighlightedTreeNode)
				selectionModel.setSelectionPath(treePath);
		}
	}

	public ElementIconMap getElementIconMap() { return elementIconMap; }

	public boolean canGoBack() {
		return (!historyBackList.isEmpty());
	}

	public boolean canGoForward() {
		return (!historyForwardList.isEmpty());
	}

	public boolean goBack() {
		if (historyBackList.isEmpty()) return false;
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath != null)
			historyForwardList.addFirst(selPath);
		TreePath path = historyBackList.removeFirst();
		addToHistory = false;
		centerAndSelectTreePath(path);
		addToHistory = true;
		return (!historyBackList.isEmpty());
	}

	public boolean goForward() {
		if (historyForwardList.isEmpty()) return false;
		TreePath selPath = selectionModel.getSelectionPath();
		if (selPath != null)
			historyBackList.addFirst(selPath);
		TreePath path = historyForwardList.removeFirst();
		addToHistory = false;
		centerAndSelectTreePath(path);
		addToHistory = true;
		return (!historyForwardList.isEmpty());
	}

	public int getMaxDELTALinkID() {
		return internalMaxLinkID;
	}

	public void setExternalMaxDELTALinkID(int externalMaxLinkID) {
		this.externalMaxLinkID = externalMaxLinkID;
	}

	public void insertNewDELTALink(int newDELTALinkID) {
		String linkElementName = XMLTreeNode.DELTA_LINK_ELEMENT_NAME;
		int insertPosition = dtdHelper.getLastInsertIntoIndex(XMLTreeNode.DELTA_LINK_ELEMENT_NAME);
		if (insertPosition < 0) {
			insertPosition = dtdHelper.getLastInsertIntoIndex(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
			linkElementName = XMLTreeNode.GMT_LINK_ELEMENT_NAME;
		}
		TreePath selPath = selectionModel.getSelectionPath();
		XMLTreeNode currentXMLTreeNode = (XMLTreeNode) selPath.getLastPathComponent();
		if (newDELTALinkID < LinkIDConstants.FIRST_ID)
			newDELTALinkID = internalMaxLinkID = Math.max(internalMaxLinkID, externalMaxLinkID) + 1;
		else
			internalMaxLinkID = Math.max(internalMaxLinkID, newDELTALinkID);
		boolean wasLeaf = currentXMLTreeNode.isLeaf();
		Element newLinkElement = xmlDocument.createElement(linkElementName);
		newLinkElement.setAttribute(XMLTreeNode.DELTA_LINK_ATTRIBUTE_NAME, String.valueOf(newDELTALinkID)); // without prefix and without leading 0
		XMLTreeNode newXMLTreeNode = new XMLTreeNode(newLinkElement);
		xmlNode2TreeNodeMap.put(newLinkElement, newXMLTreeNode);
		Node parentXMLNode = currentXMLTreeNode.getXMLNode();
		XMLUtils.insertNodeAt(parentXMLNode, newLinkElement, insertPosition);
		simpleTreeModel.insertNodeInto(newXMLTreeNode, currentXMLTreeNode, insertPosition);
		selPath = selPath.pathByAddingChild(newXMLTreeNode);
		centerAndSelectTreePath(selPath);
		if (wasLeaf) simpleTreeModel.nodeChanged(currentXMLTreeNode);

		deltaLinkMap.add(newDELTALinkID, newXMLTreeNode);
		SwingUtils.fireUndoableEditHappened(new XMLLinkInsertUndoableEdit(parentXMLNode, newLinkElement, newDELTALinkID), this, listenerList);
		contentHasChanged(true);
	}

	public boolean acceptsNewDELTALink(boolean activeSide, boolean includeHTMLView) {
		XMLTreeNode xmlTreeNode = getSelectedTreeNode();
		if (xmlTreeNode == null) return false;
		return (!activeSide && xmlTreeNode.isDELTALinkElement()) ||
			   dtdHelper.isNodeInsertableInto(XMLTreeNode.DELTA_LINK_ELEMENT_NAME) ||
			   dtdHelper.isNodeInsertableInto(XMLTreeNode.GMT_LINK_ELEMENT_NAME);
	}

	public void refresh() {
		revalidate();
		repaint();
	}

	public String getSelectedText() {
		XMLTreeNode treeNode = getSelectedTreeNode();
		if (treeNode != null) return treeNode.getText();
		return null;
	}

	public Node getSelectedNode() {
		XMLTreeNode treeNode = getSelectedTreeNode();
		if (treeNode != null) return treeNode.getXMLNode();
		return null;
	}

	private Transformer xslTransformer;

	public void unLoadSummarizeXSLFile() {
		if (xslTransformer != null) {
			xslTransformer = null;
			resetAllCollapsedNodes(rootXMLTreeNode, new TreePath(rootXMLTreeNode));
			repaint();
		}
	}

	public void applySummarizeXSLFile(File xslFile) throws IOException {
		try {
			Source source = new StreamSource(new BufferedReader(new FileReader(xslFile)), "");
			xslTransformer = XMLUtils.getTransformerFactory().newTransformer(source);
			resetAllCollapsedNodes(rootXMLTreeNode, new TreePath(rootXMLTreeNode));
		} catch (TransformerConfigurationException exc) {
			xslTransformer = null;
			throw new IOException(exc.getMessage());
		}
	}

	private void resetAllCollapsedNodes(SimpleMutableTreeNode xmlTreeNode, TreePath treePath) {
		List children = xmlTreeNode.getChildList();
		if (children != null)
		for (int i = 0, size = children.size(); i < size; i++) {
			XMLTreeNode childXMLTreeNode = (XMLTreeNode) children.get(i);
			TreePath newTreePath = treePath.pathByAddingChild(childXMLTreeNode);
			resetAllCollapsedNodes(childXMLTreeNode, newTreePath);
			if (!childXMLTreeNode.isLeaf() && childXMLTreeNode.isElement()) {
				Node xmlNode = childXMLTreeNode.getXMLNode();
				childXMLTreeNode.setSummarizeString(summarizeNodeContent(xmlNode));
				simpleTreeModel.nodeChanged(childXMLTreeNode);
			}
		}
	}

	private String summarizeNodeContent(Node node) {
		if (xslTransformer == null) return null;
		try {
			StringWriter sw = new StringWriter();
			xslTransformer.transform(new DOMSource(node), new StreamResult(sw));
			return sw.toString().trim();
		} catch (TransformerException exc) {
			Log.error("Could not summarize node \"" + node.getNodeName() + "\": " + exc.getMessage());
            exc.printStackTrace();
			return null;
		}
	}

	public boolean isShowCommentNodes() { return showCommentNodes; }

	public void setShowCommentNodes(boolean showCommentNodes) {
		if (this.showCommentNodes != showCommentNodes) {
			this.showCommentNodes = showCommentNodes;
			rebuildTreeModelAfterFilter();
		}
	}

	public boolean isShowCDATANodes() { return showCDATANodes; }

	public void setShowCDATANodes(boolean showCDATANodes) {
		if (this.showCDATANodes != showCDATANodes) {
			this.showCDATANodes = showCDATANodes;
			rebuildTreeModelAfterFilter();
		}
	}

	public boolean isShowTextNodes() { return showTextNodes; }

	public void setShowTextNodes(boolean showTextNodes) {
		if (this.showTextNodes != showTextNodes) {
			this.showTextNodes = showTextNodes;
			rebuildTreeModelAfterFilter();
		}
	}

	public boolean isShowPINodes() { return showPINodes; }

	public void setShowPINodes(boolean showPINodes) {
		if (this.showPINodes != showPINodes) {
			this.showPINodes = showPINodes;
			rebuildTreeModelAfterFilter();
		}
	}

	public boolean isShowDocumentTypeNode() { return showDocumentTypeNode; }

	public void setShowDocumentTypeNode(boolean showDocumentTypeNode) {
		if (this.showDocumentTypeNode != showDocumentTypeNode) {
			this.showDocumentTypeNode = showDocumentTypeNode;
			rebuildTreeModelAfterFilter();
		}
	}

	public void activateFilter(List filterXMLNodes, boolean show) {
		this.filterXMLNodes = filterXMLNodes;
		filterShow = show;
		rebuildTreeModelAfterFilter();
	}

	/**
	 * Removes any previously set custom Filter.
	 */
	public void showAllElementTreeNodes() {
		filterXMLNodes = null;
		rebuildTreeModelAfterFilter();
	}

	/**
	 * Removes any previously set Filter.
	 */
	public void showAllTreeNodes() {
		filterXMLNodes = null;
		showCommentNodes = showCDATANodes = showTextNodes = showDocumentTypeNode = showPINodes = showDocumentNode = true;
		rebuildTreeModelAfterFilter();
	}

	public boolean isLoading() {
		return isLoading;
	}

	public void addUndoableEditListener(UndoableEditListener undoableEditListener) {
		listenerList.add(UndoableEditListener.class, undoableEditListener);
	}

	public void removeUndoableEditListener(UndoableEditListener undoableEditListener) {
		listenerList.remove(UndoableEditListener.class, undoableEditListener);
	}

	private String askForTextValue(String title, String label, String initialValue) {
		if (textEditDialog == null)
			textEditDialog = new AutoCompleteComboBoxEditDialog(null, title, label,
													   properties.getStringList(Preferences.KEY_COMBOBOX_EDIT_TEXT));
		return textEditDialog.showEditDialog(title, label, initialValue);
	}

	private String askForVariableValue(String varName, String initialValue) {
		if (variableEditDialog == null)
			variableEditDialog = new AutoCompleteComboBoxEditDialog(null, "", "", properties.getStringList(Preferences.KEY_COMBOBOX_EDIT_TEXT));
		return variableEditDialog.showEditDialog("Variable", "Enter value for variable \"" + varName + "\": ", initialValue);
	}

	public void changeTextValueOfCurrentTreeNode() {
		XMLTreeNode xmlTreeNode = getSelectedTreeNode();
		Node xmlNode = xmlTreeNode.getXMLNode();
		String oldValue = XMLUtils.unescapeCharacters(xmlNode.getNodeValue()), newValue = null;
		switch (xmlNode.getNodeType()) {
			case Node.COMMENT_NODE:
				newValue = askForTextValue("Comment-Node", "Enter comment value: ", oldValue);
				break;
			case Node.TEXT_NODE:
				newValue = askForTextValue("Text-Node", "Enter text value: ", oldValue);
				break;
			case Node.CDATA_SECTION_NODE:
				newValue = askForTextValue("CDATA-Node", "Enter value: ", oldValue);
				break;
		}
		if ((newValue != null) && !oldValue.equals(newValue)) {
			xmlNode.setNodeValue(XMLUtils.escapeCharacters(newValue));
			resetTreeNode(xmlTreeNode);
			contentHasChanged(true);
			SwingUtils.fireUndoableEditHappened(new NodeValueChangedUndoableEdit(xmlNode, oldValue, newValue), this, listenerList);
		}
	}

	public void addRequiredContentToCurrentNode() {
		XMLTreeNode xmlTreeNode = getSelectedTreeNode();
		Node xmlNode = xmlTreeNode.getXMLNode();
		if (xmlNode.getNodeType() == Node.ELEMENT_NODE) {
			if (dtdHelper.addRequiredContent((Element) xmlNode, null)) {
				xmlTreeNode.removeAll();
				buildTreeModel(xmlTreeNode, xmlNode);
				xmlTreeNode.reset();
				simpleTreeModel.nodeStructureChanged(xmlTreeNode);
				reselectCurrentTreeNode();
				expandAllDescendants(xmlTreeNode);
				contentHasChanged(true);
			}
		}
	}

	public void resetTreeNode(XMLTreeNode xmlTreeNode) {
		xmlTreeNode.reset();
		simpleTreeModel.nodeChanged(xmlTreeNode);
		reselectTreeNode(xmlTreeNode);
	}

	public List<XMLTreeNode> convertXMLNodeListToTreeNodeList(final List nodeList) {
		Set<Node> set = new HashSet<Node>();
		final int length = nodeList.size();
		List<XMLTreeNode> result = new ArrayList<XMLTreeNode>(length);
		for (Object item : nodeList) {
            if (item instanceof Node) {
                Node xmlNode = (Node) item;
                if (xmlNode.getNodeType() == Node.ATTRIBUTE_NODE)
                    xmlNode = ((Attr) xmlNode).getOwnerElement();
                if (!set.contains(xmlNode)) {
                    set.add(xmlNode);
                    XMLTreeNode xmlTreeNode = getTreeNode4XMLNode(xmlNode);
                    if (xmlTreeNode != null)
                        result.add(xmlTreeNode);
				// else iter.remove();
                }
            }
		}
		return result;
	}
	private void search(NodeFilter nodeFilter, int whereToSearch, boolean entireDoc, boolean forward) {
		searchFound = false;
		searchForward = entireDoc || forward;
		searchStartNode = (entireDoc ? xmlDocument : getSelectedTreeNode().getXMLNode());
		searchTreeWalker = ((DocumentTraversal) xmlDocument).createTreeWalker(xmlDocument, whereToSearch, nodeFilter, false);
		searchTreeWalker.setCurrentNode(searchStartNode); // ???
		findNext();
	}
	public void search(String searchText, final int whereToSearch, boolean entireDoc, boolean forward,
					   final boolean caseSensitive, final boolean similar, final boolean regEx) {
		searchText = (caseSensitive ? searchText : searchText.toLowerCase());
		NodeFilter nodeFilter;
		if (similar)
			nodeFilter = new SimilarNodeFilter(searchText, whereToSearch, caseSensitive, true);
		else if (regEx)
			nodeFilter = new RegExNodeFilter(searchText, whereToSearch, caseSensitive); // todo: catch ParseException -> endsWith("ext") ... \A*ext
		else
			nodeFilter = new DefaultNodeFilter(searchText, whereToSearch, caseSensitive);
		search(nodeFilter, whereToSearch, entireDoc, forward);
	}
	public void searchForNextMacroInsertionPosition(Element macroElement, boolean entireDoc, boolean forward) { // todo node
		search(new MacroInsertableNodeFilter(macroElement), NodeFilter.SHOW_ELEMENT, entireDoc, forward);
	}
	public void searchForNextNodeInsertionPosition(String newNodeName, boolean entireDoc, boolean forward) { // todo node
		search(new NodeInsertableNodeFilter(newNodeName), NodeFilter.SHOW_ELEMENT, entireDoc, forward);
	}

    public void findNext() {
        searchAgain(searchForward);
    }
    public void findPrevious() {
        searchAgain(!searchForward);
    }
	public void searchAgain(boolean searchForward) {
		if (searchTreeWalker == null) return; // shouldn't happen
		Node result = (searchForward ? searchTreeWalker.nextNode() : searchTreeWalker.previousNode());
		TreeNode treeNode = null;
		if ((result == null) && searchFound) {
			searchTreeWalker.setCurrentNode(searchStartNode);
			result = (searchForward ? searchTreeWalker.nextNode() : searchTreeWalker.previousNode());
		}
		if (result != null)
			treeNode = getTreeNode4XMLNode(result);
		if (treeNode != null) {
			searchFound = true;
			centerAndSelectTreeNode(treeNode);
		} else
			if (result != null) findNext();
			else {
				searchFound = false;
				JOptionPane.showMessageDialog(this.getParent(), "Couldn't find matching Node!", "No Search Result",
											  JOptionPane.INFORMATION_MESSAGE);
				if (searchTreeWalker.getFilter() instanceof MacroInsertableNodeFilter)
					reselectCurrentTreeNode();
			}
	}

	public boolean canSearchAgain() {
		return searchFound;
	}

	public void searchHighlightResults(String searchText, int whereToSearch, boolean entireDoc, boolean caseSensitive,
									   boolean similar, boolean regEx) {
		searchText = (caseSensitive ? searchText : searchText.toLowerCase());
		NodeFilter nodeFilter;
		if (similar)
			nodeFilter = new SimilarNodeFilter(searchText, whereToSearch, caseSensitive, true);
		else if (regEx)
			nodeFilter = new RegExNodeFilter(searchText, whereToSearch, caseSensitive); // todo: catch ParseException -> endsWith("ext") ... \A*ext
		else
			nodeFilter = new DefaultNodeFilter(searchText, whereToSearch, caseSensitive);
		TreeWalker treeWalker = ((DocumentImpl) xmlDocument).createTreeWalker(
				(entireDoc ? xmlDocument : getSelectedTreeNode().getXMLNode()),
				whereToSearch, nodeFilter, false);
		List<XMLTreeNode> results = new ArrayList<XMLTreeNode>();
		for (Node result = treeWalker.nextNode(); result != null; result = treeWalker.nextNode())
			results.add(getTreeNode4XMLNode(result));
		if (results.isEmpty())
			JOptionPane.showMessageDialog(this, "Couldn't find matching Nodes.", "No Search Result",
										  JOptionPane.INFORMATION_MESSAGE);
		else
			highlightTreeNodes(results, searchHighlightColor);
	}

	private int convertDOMNodeIndexToTreeNodeIndex(Node parentDOMNode, int domNodeIndex) {
		return convertDOMNodeIndexToTreeNodeIndex(domNodeIndex, parentDOMNode.getChildNodes().item(domNodeIndex));
	}
    private int convertDOMNodeIndexToTreeNodeIndex(final int domNodeIndex, Node childDOMNode) {
        int treeNodeIndex = domNodeIndex;
        if (childDOMNode != null) {
            childDOMNode = childDOMNode.getPreviousSibling();
            while (childDOMNode != null) {
                if (!xmlNode2TreeNodeMap.containsKey(childDOMNode))
                    treeNodeIndex--;
                childDOMNode = childDOMNode.getPreviousSibling();
            }
        }
        return treeNodeIndex;
    }

	public void jumpToLineNumber(int lineNumber) {
		Node node = domParser.getLineNumberToNodeMap().get(lineNumber);
		if (node == null)
			JOptionPane.showMessageDialog(this, "There is element on line #"+lineNumber+" in the current XML document!", "Bad line number", JOptionPane.ERROR_MESSAGE);
		else {
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.get(node);
			if (xmlTreeNode != null)
				centerAndSelectTreeNode(xmlTreeNode);
			else
				System.err.println(""+node);
		}
	}
	public int getLineNumber() {
		Node selectedNode = getSelectedNode();
		if (selectedNode == null) return -1;
		Integer lineNumber = domParser.getNodeToLineNumberMap().get(selectedNode);
		if (lineNumber == null) return -1;
		return lineNumber.intValue();
	}

//========================= Drag'n'Drop =========================== (taken from JavaWorld Tip 114)
	private TreeDragAndDropHandler treeDragAndDropHandler;
	private DropTarget dropTarget;
	private DragGestureRecognizer dragGestureRecognizer;
	public void setDragAndDrop(boolean enabled) {
		if (enabled) {
			if (treeDragAndDropHandler == null)
				treeDragAndDropHandler = new TreeDragAndDropHandler();
			// Make this JTree a drag source
			DragSource dragSource = DragSource.getDefaultDragSource(); //new DragSource(); //
			dragGestureRecognizer = dragSource.createDefaultDragGestureRecognizer(this,
																	  DnDConstants.ACTION_COPY | DnDConstants.ACTION_MOVE | DnDConstants.ACTION_LINK, treeDragAndDropHandler);
			dragSource.addDragSourceListener(treeDragAndDropHandler);
			// Also, make this JTree a drag target
			dropTarget = new DropTarget(this, treeDragAndDropHandler);
			dropTarget.setDefaultActions(DnDConstants.ACTION_COPY | DnDConstants.ACTION_MOVE | DnDConstants.ACTION_LINK);
			// this improves the speed of expandAll() of large XML files (> ~50K)
			setLargeModel(false);
			setRowHeight(-1);    // problem with DND - getClosestPathForLocation returns not the same TreePath for same TreeNode when using FixedHeightLayoutCache
		} else {
			if (treeDragAndDropHandler == null) return;
			DragSource dragSource = DragSource.getDefaultDragSource();
			dragGestureRecognizer.removeDragGestureListener(treeDragAndDropHandler);
			dragSource.removeDragSourceListener(treeDragAndDropHandler);
			dropTarget.removeDropTargetListener(treeDragAndDropHandler);
			// this improves the speed of expandAll() of large XML files (> ~50K)
			setLargeModel(true);
			setRowHeight(18);    // problem with DND - getClosestPathForLocation returns not the same TreePath for same TreeNode when using FixedHeightLayoutCache
		}
	}

	// Implementation of Autoscroll-Interface
	private static final int AUTOSCROLL_MARGIN = 12;
	/** the tree needs to scroll because the mouse cursor is in the "scroll zone". */
	public void autoscroll(Point pt) {
		// Figure out which row we are on.
		int row = getRowForLocation(pt.x, pt.y);

		// If we are not on a row then ignore this autoscroll request
		if (row < 0) return;
		Rectangle outerBounds = getBounds();
        System.out.print("autoscroll: old row="+row);
		// Now decide if the row is at the top of the screen or at the
		// bottom. We do this to make the previous row (or the next
		// row) visible as appropriate. If we are at the absolute top or
		// bottom, just return the first or last row respectively.
		row = (pt.y + outerBounds.y <= AUTOSCROLL_MARGIN) ? // Is row at top of screen?
			   ((row == 0) ? 0 : (row - 1)) : // Yes, scroll up one row
			   (row < (getRowCount() - 1) ? (row + 1) : row); // No, scroll down one row
        System.out.println("- new row="+row);
		scrollRowToVisible(row);
	}
	/** Calculate the insets for the JTree, not the viewport the tree is in. */
	public Insets getAutoscrollInsets() {
		Rectangle outerBounds = getBounds();
		Rectangle innerBounds = getParent().getBounds();
		return new Insets(innerBounds.y - outerBounds.y + AUTOSCROLL_MARGIN, innerBounds.x - outerBounds.x + AUTOSCROLL_MARGIN,
						  outerBounds.height - innerBounds.height - innerBounds.y + outerBounds.y + AUTOSCROLL_MARGIN,
						  outerBounds.width - innerBounds.width - innerBounds.x + outerBounds.x + AUTOSCROLL_MARGIN);
	}

	private static final Color dragImageGradientColor = new Color(255, 255, 255, 0);
	private static final Color DND_CUELINE_COLOR = new Color(SystemColor.controlShadow.getRed(),
														 SystemColor.controlShadow.getGreen(),
														 SystemColor.controlShadow.getBlue(), 64);
	
	private class TreeDragAndDropHandler implements DropTargetListener, DragGestureListener, DragSourceListener {
		private TreePath lastTreePath = null;
		private Rectangle2D cueLineBounds = new Rectangle2D.Float();
		private Rectangle2D dragImageBounds = new Rectangle2D.Float();

		private Point lastLocation = new Point();
		private Timer hoverTimer;
		private int nLeftRight = 0; // Cumulative left/right mouse movement
		private int direction = 0; // -1 ... left, +1 ... right

		private TreeDragAndDropHandler() {
			// Set up a hover timer, so that a node will be automatically expanded or collapsed
			// if the user lingers on it for more than a short time
			hoverTimer = new Timer(2000, new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					nLeftRight = 0; // Reset left/right movement trend
					if (lastTreePath.getParentPath() == null) return; // Do nothing if we are hovering over the root node
					if (isExpanded(lastTreePath)) collapsePath(lastTreePath);
					else expandPath(lastTreePath);
				}
			});
			hoverTimer.setRepeats(false); // Set timer to one-shot mode
		}

		
		private TreePath dndTreePathSource;  // The path being dragged
		private BufferedImage dndDragImage;  // The 'drag image'
		private final Point dndDragImageOffset = new Point(); // Where, in the drag image, the mouse was clicked
		
		// DND - DragGestureListener
		public void dragGestureRecognized(DragGestureEvent dge) {
			System.out.println("XMLView[" + xmlFile + "].dragGestureRecognized");
			int dragAction = dge.getDragAction();
			System.out.println("  DragAction = "
							   + ((dragAction == DnDConstants.ACTION_COPY) ? "COPY" : ((dragAction == DnDConstants.ACTION_MOVE) ? "MOVE" : "LINK"))
                               + "\nmodifiers: "+ InputEvent.getModifiersExText(dge.getTriggerEvent().getModifiersEx()));

			Point dragOrigin = dge.getDragOrigin();
			TreePath treePath = getPathForLocation(dragOrigin.x, dragOrigin.y); //getClosestPathForLocation(dragOrigin.x, dragOrigin.y);
			if (treePath == null) return; // no treenode under cursor
			TreeNode treeNode = (TreeNode) treePath.getLastPathComponent();

			System.out.println("  DragSourceTreeNode = " + treeNode);

			if (!isTreeNodeDragable(treeNode, dragAction)) return;

			// Work out the offset of the drag point in the TreePath bounding rectangle
			Rectangle treeNodeBounds = getPathBounds(treePath);
			dndDragImageOffset.setLocation(dragOrigin.x - treeNodeBounds.x, dragOrigin.y - treeNodeBounds.y);

			// Get the cell renderer (which is a JLabel) for the path being dragged
			JLabel lbl = (JLabel) getCellRenderer().getTreeCellRendererComponent(XMLView.this, treeNode, false, isExpanded(treePath), getModel().isLeaf(treeNode), 0, false);
			lbl.setSize((int) treeNodeBounds.getWidth(), (int) treeNodeBounds.getHeight());
			// Get a buffered image of the selection for dragging a ghost image
			dndDragImage = new BufferedImage(treeNodeBounds.width, treeNodeBounds.height, BufferedImage.TYPE_INT_ARGB_PRE);
			Graphics2D g2 = dndDragImage.createGraphics();

			// Ask the cell renderer to paint itself into the BufferedImage
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC, 0.5f));  // Make the image ghostlike
			lbl.paint(g2);

			// Now paint a gradient UNDER the ghosted JLabel text (but not under the icon if any)
			// Note: this will need tweaking if your icon is not positioned to the left of the text
			Icon icon = lbl.getIcon();
			int nStartOfText = (icon == null) ? 0 : icon.getIconWidth() + lbl.getIconTextGap();
		// System.out.println("nStartOfText = " + nStartOfText);
			g2.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_OVER, 0.5f)); // Make the gradient ghostlike

			g2.setPaint(new GradientPaint(nStartOfText, 0, SystemColor.controlShadow, getWidth(), 0, dragImageGradientColor));
			g2.fillRect(nStartOfText, 0, getWidth(), dndDragImage.getHeight());
			g2.dispose();

			setSelectionPath(treePath); // Select this path in the tree
			dndTreePathSource = treePath;
			dge.startDrag(null, dndDragImage, new Point(5, 5), new TransferableTreePath(treePath), this);
		}

		private boolean isTreeNodeDragable(TreeNode treeNode, int dragAction) {
			return (treeNode.getParent() != null);  //todo
		}

		// DND - DragSourceListener
		public void dragEnter(DragSourceDragEvent dsde) {
			//todo
		// System.out.println("XMLView[" + xmlFile + "].DragSourceListener.dragEnter");
		}
		public void dragOver(DragSourceDragEvent dsde) {
		//todo
		// System.out.println("XMLView["+xmlFile+"].DragSourceListener.dragOver");
		}
		public void dropActionChanged(DragSourceDragEvent dsde) {
			//todo
			System.out.println("XMLView[" + xmlFile + "].DragSourceListener.dropActionChanged");
			int dragAction = dsde.getDropAction();
			System.out.println("  DropAction = "
							   + ((dragAction == DnDConstants.ACTION_COPY) ? "COPY" : ((dragAction == DnDConstants.ACTION_MOVE) ? "MOVE" : "LINK")));
		}
		public void dragExit(DragSourceEvent dse) {
			//todo
		// System.out.println("XMLView[" + xmlFile + "].DragSourceListener.dragExit");
		}
		public void dragDropEnd(DragSourceDropEvent dsde) {
            if (dndTreePathSource == null) return; // public void dragDropEnd(DragSourceDropEvent dsde)  is called twice !?!
			//todo
			System.out.println("XMLView[" + xmlFile + "].DragSourceListener.dragDropEnd");
            System.out.println("  DropSuccess = "+dsde.getDropSuccess());
            int dragAction = dsde.getDropAction();
            System.out.println("  DropAction = "
                    + ((dragAction == DnDConstants.ACTION_COPY) ? "COPY" : ((dragAction == DnDConstants.ACTION_MOVE) ? "MOVE" : "LINK")));
            System.out.println("  dndTreePathSource = " + dndTreePathSource);

            if (dsde.getDropSuccess()) {
                if (dragAction == DnDConstants.ACTION_MOVE)
                    removeTreeNode(dndTreePathSource);
            }
            dndTreePathSource = null;
		}
		public void dragEnter(DropTargetDragEvent dtde) {
			//todo
		// System.out.println("XMLView[" + xmlFile + "].DropTargetListener.dragEnter");
		}

		public void dragOver(DropTargetDragEvent dtde) {
			// System.out.println("XMLView[" + xmlFile + "].DropTargetListener.dragOver");

			// Even if the mouse is not moving, this method is still invoked 10 times per second
			Point pt = dtde.getLocation();
			if (pt.equals(lastLocation)) return;

			// Try to determine whether the user is flicking the cursor right or left
			int deltaLeftRight = pt.x - lastLocation.x;
			/*System.out.println("direction = " + direction);
			System.out.println();
			System.out.println("pt = " + pt);
			System.out.println("lastLocation = " + lastLocation);
			System.out.println("deltaLeftRight = " + deltaLeftRight);
			System.out.println("nLeftRight = " + nLeftRight);*/
			if (((nLeftRight > 0) && (deltaLeftRight < 0)) || ((nLeftRight < 0) && (deltaLeftRight > 0)))
				nLeftRight = 0;
			nLeftRight += deltaLeftRight;
			lastLocation = pt;
			Graphics2D g2 = (Graphics2D) getGraphics();

			// If a drag image is not supported by the platform, then draw my own drag image
			if (!DragSource.isDragImageSupported()) {
				paintImmediately(dragImageBounds.getBounds()); // Rub out the last ghost image and cue line
				// And remember where we are about to draw the new ghost image
				dragImageBounds.setRect(pt.x - dndDragImageOffset.x, pt.y - dndDragImageOffset.y, dndDragImage.getWidth(), dndDragImage.getHeight());
				g2.drawImage(dndDragImage, AffineTransform.getTranslateInstance(dragImageBounds.getX(), dragImageBounds.getY()), null);
			} else // Just rub out the last cue line
				paintImmediately(cueLineBounds.getBounds());
			TreePath treePath = getClosestPathForLocation(pt.x, pt.y);
			if (treePath != lastTreePath) {
				nLeftRight = 0;  // We've moved up or down, so reset left/right movement trend
				System.out.println("treepath changed from \"" + lastTreePath + "\"\n  to \"" + treePath + "\"");
				lastTreePath = treePath;
				hoverTimer.restart();
			}

			// In any case draw (over the ghost image if necessary) a cue line indicating where a drop will occur
			Rectangle raPath = getPathBounds(treePath);
			cueLineBounds.setRect(0, raPath.y + (int) raPath.getHeight(), getWidth(), 2); // below treePath
			g2.setColor(DND_CUELINE_COLOR);
			g2.fill(cueLineBounds);

			// Now superimpose the left/right movement indicator if necessary
			if (nLeftRight > 20) {
				g2.drawImage(ArrowImage.DEFAULT_ARROW_IMAGE_RIGHT, AffineTransform.getTranslateInstance(pt.x - dndDragImageOffset.x, pt.y - dndDragImageOffset.y), null);
				direction = +1;
			} else if (nLeftRight < -20) {
				g2.drawImage(ArrowImage.DEFAULT_ARROW_IMAGE_LEFT, AffineTransform.getTranslateInstance(pt.x - dndDragImageOffset.x, pt.y - dndDragImageOffset.y), null);
				direction = -1;
			} else
				direction = 0;

			// And include the cue line in the area to be rubbed out next time
			dragImageBounds = dragImageBounds.createUnion(cueLineBounds);
		}

		public void dropActionChanged(DropTargetDragEvent dtde) {
			//todo
			System.out.println("XMLView[" + xmlFile + "].DropTargetListener.dropActionChanged");
			int dragAction = dtde.getDropAction();
			System.out.println("  DropAction = "
							   + ((dragAction == DnDConstants.ACTION_COPY) ? "COPY" : ((dragAction == DnDConstants.ACTION_MOVE) ? "MOVE" : "LINK")));
		}

		public void dragExit(DropTargetEvent dte) {
			//todo
		// System.out.println("XMLView[" + xmlFile + "].DropTargetListener.dragExit");
			repaint();
		}

		public void drop(DropTargetDropEvent dtde) { //todo
            hoverTimer.stop(); // Prevent hover timer from doing an unwanted expandPath or collapsePath

            try {
                Point dropPoint = dtde.getLocation();
                TreePath pathTarget = getClosestPathForLocation(dropPoint.x, dropPoint.y);
                TreePath pathSource = (TreePath) dtde.getTransferable().getTransferData(TransferableTreePath.TREEPATH_FLAVOR);
                if (isExpanded(pathTarget)) direction = (direction + 1) % 2;

                if (!isDropAcceptable(pathSource, pathTarget, direction)) {
                    dtde.rejectDrop();
                    return;
                }
                dtde.acceptDrop(dtde.getDropAction());

                /*System.out.println("XMLView[" + xmlFile + "].DropTargetListener.drop");
                System.out.println("  pathSource: " + pathSource.getLastPathComponent());
                System.out.println("  pathTarget = " + pathTarget.getLastPathComponent());
                System.out.println("  direction = " + direction); // -1 ... as sibling of parent, 0...as sibling, +1...as child
*/
                XMLTreeNode xmlTreeNode = (XMLTreeNode) pathSource.getLastPathComponent();
                Node nodeSource = xmlTreeNode.getXMLNode();
             //   if (dtde.getDropAction() == DnDConstants.ACTION_COPY)
                    nodeSource = nodeSource.cloneNode(true);

                TreePath newTreePath=null;
                switch (direction) {
                    case -1:
                        selectionModel.setSelectionPath(pathTarget.getParentPath());
                        newTreePath = insertNodeAfter(nodeSource);
                        break;
                    case  0:
                        selectionModel.setSelectionPath(pathTarget);
                        newTreePath = insertNodeAfter(nodeSource);
                        break;
                    case +1:
                        selectionModel.setSelectionPath(pathTarget);
                        newTreePath = insertNodeInto(nodeSource);
                        break;
                }

                dtde.dropComplete(true);
                if ((dtde.getDropAction() == DnDConstants.ACTION_MOVE) &&
                        (nodeSource.getOwnerDocument() == xmlDocument))
                    centerAndSelectTreePath(newTreePath);
            } catch (Exception ufe) {
             //   System.out.println(ufe);
                dtde.dropComplete(false);
                return;
            }

            repaint();
		}

		private boolean isDropAcceptable(TreePath pathSource, TreePath pathTarget, int direction) {
			return true;  //todo
		}
	}

	// taken from JavaWorld Tip 114
	private static final class TransferableTreePath implements Transferable {
		public static final DataFlavor TREEPATH_FLAVOR = new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType, "TreePath");
		private static final DataFlavor[] flavors = {TREEPATH_FLAVOR};
		private TreePath path;

		/**
		 * Constructs a transferrable tree path object for the specified path.
		 */
		public TransferableTreePath(TreePath path) {
			this.path = path;
		}

		public DataFlavor[] getTransferDataFlavors() {
			return flavors;
		}

		public boolean isDataFlavorSupported(DataFlavor flavor) {
			return (flavor == TREEPATH_FLAVOR);
		}

		public synchronized Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
			if (flavor == TREEPATH_FLAVOR)
				return path;
			else
				throw new UnsupportedFlavorException(flavor);
		}
	}
// ================================


	private final class NodeRemovedUndoableEdit extends SideAbstractUndoableEdit
			implements MacroInsertUndoableEdit {
		protected final int domNodeIndex;
        protected Node removedXMLNode, parentXMLNode;

        // is called before removedXMLNode is removed
		NodeRemovedUndoableEdit(Node removedXMLNode) {
            this(removedXMLNode, XMLUtils.getIndexOfChildNode(removedXMLNode));
        }
        NodeRemovedUndoableEdit(Node removedXMLNode, int domNodeIndex) {
			super(false);
			this.removedXMLNode = removedXMLNode;
            this.domNodeIndex = domNodeIndex;
            parentXMLNode = removedXMLNode.getParentNode();
        }

		public void undo() throws CannotUndoException {
			super.undo(); // undo child-edits
			Log.trace("enter: NodeRemovedUndoableEdit.undo()");
            XMLUtils.insertNodeAt(parentXMLNode, removedXMLNode, domNodeIndex);
			if (shouldInsertIntoTreeModel(removedXMLNode)) {
                int treeNodeIndex = convertDOMNodeIndexToTreeNodeIndex(domNodeIndex, removedXMLNode);
                XMLTreeNode parentTreeNode = xmlNode2TreeNodeMap.get(parentXMLNode);
				boolean wasLeaf = parentTreeNode.isLeaf();
                XMLTreeNode newXMLTreeNode = createTreeNode(removedXMLNode);
                simpleTreeModel.insertNodeInto(newXMLTreeNode, parentTreeNode, treeNodeIndex);
				TreePath newTreePath = simpleTreeModel.getTreePath(newXMLTreeNode);
				expandAllDescendants(newXMLTreeNode, newTreePath);
				centerAndReselectTreePath(newTreePath);
				if (wasLeaf) simpleTreeModel.nodeChanged(parentTreeNode);
			}
		}

		public void redo() throws CannotRedoException {
			super.redo();
			Log.trace("enter: NodeRemovedUndoableEdit.redo()");

            XMLTreeNode treeNodeToRemove = xmlNode2TreeNodeMap.get(removedXMLNode);
            removedXMLNode.getParentNode().removeChild(removedXMLNode);
            if (treeNodeToRemove != null) {
				createRemovedDELTALinkSet(treeNodeToRemove, null); //, null);
				TreePath treePath = simpleTreeModel.getTreePath(treeNodeToRemove);
                TreePath parentTreePath = treePath.getParentPath();
				centerAndReselectTreePath(parentTreePath);
                simpleTreeModel.removeNodeFromParent(treeNodeToRemove);
                historyBackList.remove(treePath);
                historyForwardList.remove(treePath);
            }
		}

		public void die() {
			super.die();
			Log.trace("enter: NodeRemovedUndoableEdit.die()");
			removedXMLNode = null;
            parentXMLNode = null;
		}
	}

	private final class MacroLinksInsertedUndoableEdit extends AbstractUndoableEdit implements MacroInsertUndoableEdit {
		private int deltaLinkID;
		private List<XMLTreeNode> linkTreeNodes;

		MacroLinksInsertedUndoableEdit(int gmtLinkID, List<XMLTreeNode> linkTreeNodes) {
			this.linkTreeNodes = linkTreeNodes;
			this.deltaLinkID = gmtLinkID;
		}

		public void undo() throws CannotUndoException {
			super.undo();
			Log.trace("enter: LinksRemovedFromMapUndoableEdit.undo()");
			deltaLinkMap.removeAll(deltaLinkID, linkTreeNodes);
			if (deltaLinkMap.isEmpty())
				internalMaxLinkID = LinkIDConstants.FIRST_ID - 1;
			else
				while (!deltaLinkMap.containsKey(internalMaxLinkID))
					internalMaxLinkID--;
		}

		public void redo() throws CannotRedoException {
			super.redo();
			Log.trace("enter: LinksRemovedFromMapUndoableEdit.redo()");
			deltaLinkMap.addAll(deltaLinkID, linkTreeNodes);
			internalMaxLinkID = Math.max(internalMaxLinkID, deltaLinkID);
		}

		public void die() {
			super.die();
			Log.trace("enter: LinksRemovedFromMapUndoableEdit.die()");
			linkTreeNodes = null;
		}
	}
	
	private final class MacroCounterInsertedUndoableEdit extends AbstractUndoableEdit implements MacroInsertUndoableEdit {
		private final String counterName;
		
		MacroCounterInsertedUndoableEdit(final String counterName) {
			super();
			this.counterName = counterName;
		}
		public void undo() throws CannotUndoException {
			super.undo();
			int val = macroCounterMap.get(counterName);
			macroCounterMap.put(counterName, (val-1));
		}
		public void redo() throws CannotRedoException {
			super.redo();
			int val = macroCounterMap.get(counterName);
			macroCounterMap.put(counterName, (val+1));
		}
}

	private final class XMLLinksRemovedUndoableEdit extends SideCompoundEdit implements LinksRemovedUndoableEdit {
		private XMLLinksRemovedUndoableEdit() {
			super(false);
		}

		public boolean addEdit(UndoableEdit anEdit) {
			if (anEdit instanceof NodeRemovedUndoableEdit)
				return super.addEdit(anEdit);
			return false;
		}
	}

	private final class NodeValueChangedUndoableEdit extends SideAbstractUndoableEdit {
		private Node xmlNode;
		private String newValue, oldValue;

		NodeValueChangedUndoableEdit(Node xmlNode, String oldValue, String newValue) {
			super(false);
			this.xmlNode = xmlNode;
			this.newValue = newValue;
			this.oldValue = oldValue;
			undoPresentationName = "Undo - Node value change";
			redoPresentationName = "Redo - Node value change";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			xmlNode.setNodeValue(oldValue);
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.get(xmlNode);
			if (xmlTreeNode != null) {
				resetTreeNode(xmlTreeNode);
				scrollToTreeNode(xmlTreeNode);
			}
		}

		public void redo() throws CannotRedoException {
			super.redo();
			xmlNode.setNodeValue(newValue);
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.get(xmlNode);
			if (xmlTreeNode != null) {
				resetTreeNode(xmlTreeNode);
				scrollToTreeNode(xmlTreeNode);
			}
		}

		public void die() {
			Log.trace("enter: NodeValueChangedUndoableEdit.die()");
			super.die();
			oldValue = null;
			newValue = null;
			xmlNode = null;
		}
	}

    private final class NodeInsertUndoableEdit extends SideAbstractUndoableEdit implements MacroInsertUndoableEdit {
        private Node xmlNode, parentXMLNode;
        private final int domNodeIndex;

        public NodeInsertUndoableEdit(Node xmlNode) {
			super(false);
			this.xmlNode = xmlNode;
            parentXMLNode = xmlNode.getParentNode();
            domNodeIndex = XMLUtils.getIndexOfChildNode(xmlNode);

			String shortDescription = XMLUtils.getShortDescriptionForNode(xmlNode);
			undoPresentationName = "Undo - \"" + shortDescription + "\" insertion";
            redoPresentationName = "Redo - \"" + shortDescription + "\" insertion";
        }

        public void undo() throws CannotUndoException {
            super.undo();
            Log.trace("enter: NodeInsertUndoableEdit.undo()");
            XMLTreeNode treeNodeToRemove = xmlNode2TreeNodeMap.remove(xmlNode);
            xmlNode.getParentNode().removeChild(xmlNode);
            if (treeNodeToRemove != null) {
                TreePath treePath = simpleTreeModel.getTreePath(treeNodeToRemove);
				centerAndReselectTreePath(treePath.getParentPath());
                simpleTreeModel.removeNodeFromParent(treeNodeToRemove);
                historyBackList.remove(treePath);
                historyForwardList.remove(treePath);
            } else
				centerAndReselectTreePath(simpleTreeModel.getTreePath(xmlNode2TreeNodeMap.remove(xmlNode.getParentNode())));
        }

        public void redo() throws CannotRedoException {
            super.redo();
            Log.trace("enter: NodeInsertUndoableEdit.redo()");
            XMLUtils.insertNodeAt(parentXMLNode, xmlNode, domNodeIndex);
            if (shouldInsertIntoTreeModel(xmlNode)) {
                int treeNodeIndex = convertDOMNodeIndexToTreeNodeIndex(domNodeIndex, xmlNode);
                XMLTreeNode parentTreeNode = xmlNode2TreeNodeMap.get(parentXMLNode);
                boolean wasLeaf = parentTreeNode.isLeaf();
                XMLTreeNode newXMLTreeNode = createTreeNode(xmlNode);
                simpleTreeModel.insertNodeInto(newXMLTreeNode, parentTreeNode, treeNodeIndex);
                TreePath newTreePath = simpleTreeModel.getTreePath(newXMLTreeNode);
                expandAllDescendants(newXMLTreeNode, newTreePath);
				centerAndReselectTreePath(newTreePath);
                if (wasLeaf) simpleTreeModel.nodeChanged(parentTreeNode);
            } else
				centerAndReselectTreePath(simpleTreeModel.getTreePath(xmlNode2TreeNodeMap.remove(xmlNode.getParentNode())));
        }

        public void die() {
            super.die();
            xmlNode = null;
            parentXMLNode = null;
        }
    }

	private final class NodeConvertedUndoableEdit extends SideAbstractUndoableEdit {
		private Node xmlNode;
		private String oldElementName, newElementName;

		public NodeConvertedUndoableEdit(Node xmlNode, String oldElementName, String newElementName) {
			super(false);
			this.newElementName = newElementName;
			this.oldElementName = oldElementName;
			this.xmlNode = xmlNode;
			undoPresentationName = "Undo - Convert \"" + oldElementName + "\" to \"" + newElementName + "\"";
			redoPresentationName = "Redo - Convert \"" + oldElementName + "\" to \"" + newElementName + "\"";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			((org.apache.xerces.dom.CoreDocumentImpl) xmlDocument).renameNode(xmlNode, null, oldElementName); // renameNode - DOM Level 3, cast to CoreDocumentImpl needed before Java1.5
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.get(xmlNode);
			if (xmlTreeNode != null)
				resetTreeNode(xmlTreeNode);
		}

		public void redo() throws CannotRedoException {
			super.redo();
			((org.apache.xerces.dom.CoreDocumentImpl) xmlDocument).renameNode(xmlNode, null, newElementName);
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.get(xmlNode);
			if (xmlTreeNode != null)
				resetTreeNode(xmlTreeNode);
		}

		public void die() {
			super.die();
			newElementName = null;
			oldElementName = null;
			xmlNode = null;
		}
	}
	private final class NodeWrapUndoableEdit extends SideAbstractUndoableEdit {
		private Node wrappedNode, wrapperNode;

		public NodeWrapUndoableEdit(Node wrappedNode, Node wrapperNode) {
			super(false);
			this.wrappedNode = wrappedNode;
			this.wrapperNode = wrapperNode;

			String shortDescription = XMLUtils.getShortDescriptionForNode(wrappedNode);
			undoPresentationName = "Undo - Wrap \"" + shortDescription + "\"";
			redoPresentationName = "Redo - Wrap \"" + shortDescription + "\"";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			Node parentXMLNode = wrapperNode.getParentNode();
			parentXMLNode.replaceChild(wrappedNode, wrapperNode);

			XMLTreeNode treeNodeToRemove = xmlNode2TreeNodeMap.remove(wrapperNode);
			if (treeNodeToRemove != null) {
				TreePath treePath = simpleTreeModel.getTreePath(treeNodeToRemove);
				simpleTreeModel.removeNodeFromParent(treeNodeToRemove);
				historyBackList.remove(treePath);
				historyForwardList.remove(treePath);
			}
			XMLTreeNode treeNodeToInsert = xmlNode2TreeNodeMap.get(wrappedNode);
			if (treeNodeToInsert != null) {
				SimpleMutableTreeNode parentTreeNode = xmlNode2TreeNodeMap.get(parentXMLNode);
				int treeNodeIndex = convertDOMNodeIndexToTreeNodeIndex(XMLUtils.getIndexOfChildNode(wrappedNode), wrappedNode);
				boolean wasLeaf = parentTreeNode.isLeaf();
				simpleTreeModel.insertNodeInto(treeNodeToInsert, parentTreeNode, treeNodeIndex);
				TreePath newTreePath = simpleTreeModel.getTreePath(treeNodeToInsert);
				expandPath(newTreePath);
				centerAndReselectTreePath(newTreePath);
				if (wasLeaf) simpleTreeModel.nodeChanged(parentTreeNode);
			} else
				centerAndReselectTreePath(simpleTreeModel.getTreePath(xmlNode2TreeNodeMap.remove(parentXMLNode)));
		}

		public void redo() throws CannotRedoException {
			super.redo();
			int insertPositionDOM = XMLUtils.getIndexOfChildNode(wrappedNode);
			Node parentXMLNode = wrappedNode.getParentNode();
			parentXMLNode.removeChild(wrappedNode);
			XMLTreeNode wrappedTreeNode = xmlNode2TreeNodeMap.remove(wrappedNode);
			if (wrappedTreeNode != null)
				simpleTreeModel.removeNodeFromParent(wrappedTreeNode);

			XMLUtils.insertNodeAt(parentXMLNode, wrapperNode, insertPositionDOM);
			wrapperNode.appendChild(wrappedNode);

			if (shouldInsertIntoTreeModel(wrapperNode)) {
				XMLTreeNode parentTreeNode = xmlNode2TreeNodeMap.get(parentXMLNode);
				TreePath parentTreePath = simpleTreeModel.getTreePath(parentTreeNode);
				boolean wasLeaf = parentTreeNode.isLeaf();
				XMLTreeNode wrapperXMLTreeNode = createTreeNode(wrapperNode);
				simpleTreeModel.insertNodeInto(wrapperXMLTreeNode, parentTreeNode,
						convertDOMNodeIndexToTreeNodeIndex(parentTreeNode.getXMLNode(), insertPositionDOM));
				parentTreePath = parentTreePath.pathByAddingChild(wrapperXMLTreeNode);
				expandPath(parentTreePath);
				centerAndSelectTreePath(simpleTreeModel.getTreePath(wrapperXMLTreeNode));
				if (wasLeaf) simpleTreeModel.nodeChanged(parentTreeNode);
			} else
				centerAndSelectTreeNode(xmlNode2TreeNodeMap.get(parentXMLNode));
		}

		public void die() {
			super.die();
			wrappedNode = null;
			wrapperNode = null;
		}
	}

	private final class XMLLinkInsertUndoableEdit extends SideAbstractUndoableEdit
			implements LinkInsertedUndoableEdit {
		private Node parentNode, linkElement;
		private int linkId;

		public XMLLinkInsertUndoableEdit(Node parentNode, Node linkElement, int linkId) {
			super(false);
			this.parentNode = parentNode;
			this.linkElement = linkElement;
			this.linkId = linkId;
			undoPresentationName = "Undo - Link insertion";
			redoPresentationName = "Redo - Link insertion";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			XMLTreeNode xmlTreeNode = xmlNode2TreeNodeMap.remove(linkElement);
			if (xmlTreeNode != null) {
				deltaLinkMap.remove(linkId, xmlTreeNode);
				centerAndReselectTreeNode(xmlTreeNode.getParent());
				simpleTreeModel.removeNodeFromParent(xmlTreeNode);
			}
			parentNode.removeChild(linkElement);
			if (deltaLinkMap.isEmpty())
				internalMaxLinkID = LinkIDConstants.FIRST_ID - 1;
			else
				while (!deltaLinkMap.containsKey(internalMaxLinkID))
					internalMaxLinkID--;
		}

		public void redo() throws CannotRedoException {
			super.redo();
			XMLTreeNode parentTreeNode = xmlNode2TreeNodeMap.get(parentNode);
			parentNode.insertBefore(linkElement, parentNode.getFirstChild());
			if (parentTreeNode != null) {
				XMLTreeNode treeNode = createTreeNode(linkElement);
				boolean wasLeaf = parentTreeNode.isLeaf();
				simpleTreeModel.insertNodeInto(treeNode, parentTreeNode, 0);
				TreePath treePath = simpleTreeModel.getTreePath(treeNode);
				centerAndReselectTreePath(treePath);
				if (wasLeaf) simpleTreeModel.nodeChanged(parentTreeNode);
			}
			internalMaxLinkID = Math.max(internalMaxLinkID, linkId);
		}
	}

	private final class XMLLinkIDChangedUndoableEdit extends SideAbstractUndoableEdit implements LinkIDChangedUndoableEdit {
		private XMLTreeNode linkTreeNode;
		private int newLinkID, oldLinkID;

		public XMLLinkIDChangedUndoableEdit(XMLTreeNode linkTreeNode, int newLinkID, int oldLinkID) {
			super(false);
			this.linkTreeNode = linkTreeNode;
			this.newLinkID = newLinkID;
			this.oldLinkID = oldLinkID;
			undoPresentationName = "Undo - Link Synchronization";
			redoPresentationName = "Redo - Link Synchronization";
		}

		public void undo() throws CannotUndoException {
			super.undo();
			synchronizeDELTALink(linkTreeNode, oldLinkID);
		}

		public void redo() throws CannotRedoException {
			super.redo();
			synchronizeDELTALink(linkTreeNode, newLinkID);
		}
	}

	public static final	int	SEARCH_ALL			   = NodeFilter.SHOW_ALL;
	public static final	int	SEARCH_ELEMENT_NAME	   = NodeFilter.SHOW_ELEMENT;
	public static final	int	SEARCH_ATTRIBUTE_NAME  = 0x00001000;
	public static final	int	SEARCH_ATTRIBUTE_VALUE = 0x00002000;
	public static final	int	SEARCH_COMMENT		   = NodeFilter.SHOW_COMMENT;
	public static final	int	SEARCH_TEXT			   = NodeFilter.SHOW_TEXT;


	private static class DefaultNodeFilter implements NodeFilter {
		protected final String searchText;
		protected final int whereToSearch;
		protected final boolean caseSensitive;

		DefaultNodeFilter(String searchText, int whereToSearch, boolean caseSensitive) {
			this.searchText = searchText;
			this.whereToSearch = whereToSearch;
			this.caseSensitive = caseSensitive;
		}

		public final short acceptNode(Node xmlNode) {
			switch (xmlNode.getNodeType()) {
				case Node.ELEMENT_NODE:
					String nodeName = (caseSensitive ? xmlNode.getNodeName() : xmlNode.getNodeName().toLowerCase());
					if (matches(nodeName))
						return NodeFilter.FILTER_ACCEPT;
					if ((whereToSearch & 0xFFFFF000) != 0) {
						NamedNodeMap attrs = xmlNode.getAttributes();
						for (int i = 0, size = attrs.getLength(); i < size; i++) {
							Attr attr = (Attr) attrs.item(i);
							String attrName = (caseSensitive ? attr.getName() : attr.getName().toLowerCase());
							if (((whereToSearch & SEARCH_ATTRIBUTE_NAME) != 0) && matches(attrName))
								return NodeFilter.FILTER_ACCEPT;
							String attrValue = (caseSensitive ? attr.getValue() : attr.getValue().toLowerCase());
							if (((whereToSearch & SEARCH_ATTRIBUTE_VALUE) != 0) && matches(attrValue))
								return NodeFilter.FILTER_ACCEPT;
						}
					}
					return NodeFilter.FILTER_SKIP;
				case Node.COMMENT_NODE:
				case Node.TEXT_NODE:
					String nodeValue = (caseSensitive ? xmlNode.getNodeValue() : xmlNode.getNodeValue().toLowerCase());
					return (matches(nodeValue) ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_SKIP);
			}
			return NodeFilter.FILTER_SKIP;
		}

		boolean matches(String matchAgainstString) {
			return (matchAgainstString.indexOf(searchText) > -1);// searchText.equals(matchAgainstString);
		}
	}

	private static final class SimilarNodeFilter extends DefaultNodeFilter {
		private final boolean splitIntoWords;
		SimilarNodeFilter(String searchText, int whereToSearch, boolean caseSensitive, boolean splitIntoWords) {
			super(searchText, whereToSearch, caseSensitive);
			this.splitIntoWords = splitIntoWords;
		}

		boolean matches(String s) {
			if (!splitIntoWords)
				return WLD.defaultMatch(searchText, s);
			String[] words = s.split("[\\s-]");
			for (String word : words) {
				if (WLD.defaultMatch(searchText, word))
					return true;
			}
			return false;
		}
	}

	private static final class RegExNodeFilter extends DefaultNodeFilter {
        private final Pattern regexPattern;

		RegExNodeFilter(String regexSearchText, int whereToSearch, boolean caseSensitive) {
			super(regexSearchText, whereToSearch, caseSensitive);
            regexPattern = Pattern.compile(regexSearchText);
		}

		boolean matches(String matchAgainstString) {
            return regexPattern.matcher(matchAgainstString).matches();
		}
	}

	private final class MacroInsertableNodeFilter implements NodeFilter {
		private Element macroElement;

		public MacroInsertableNodeFilter(Element macroElement) {
			this.macroElement = macroElement;
		}

		public short acceptNode(Node node) {
			dtdHelper.setTargetElement(node);
			return (dtdHelper.isMacroInsertable(macroElement) ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_SKIP);
		}
	}
	private final class NodeInsertableNodeFilter implements NodeFilter {
		private String newNodeName;

		public NodeInsertableNodeFilter(String newNodeName) {
			this.newNodeName = newNodeName;
		}

		public short acceptNode(Node node) {
			dtdHelper.setTargetElement(node);
			return (dtdHelper.isNodeInsertable(newNodeName) ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_SKIP);
		}
	}
}
