package at.ac.tuwien.asgaard.delta.util;

import java.io.File;

/**
 * Class EnvironmentProperties - todo javadoc.
 *
 * @author Peter Votruba
 * @version 2.5.3
 */
public class EnvironmentProperties {

	/**
	 * indicates whether this runs with Java Web Start or not
	 */
	private static Boolean RUN_UNDER_WEB_START;
	public static final String JAVA_VERSION = System.getProperty("java.version");
	public static final boolean JAVA_1_5;
	public static final boolean JAVA_1_4;
	/**
	 * Boolean indicating if DELTA is running under Windows accomplished by the easiest way checking File.separatorChar
	 */
	public static final boolean OS_WINDOWS;
	private static Boolean IS_JHELP_AVAILABLE;
	private static Boolean IS_FOP_AVAILABLE;
	private static Boolean IS_BATIK_AVAILABLE;
	/**
	 * Line separator string.  This is the value of the line.separator
	 * property at the moment that this class was initialized.
	 */
	public static final String LINE_SEPARATOR;

	static {
		LINE_SEPARATOR = System.getProperty("line.separator");

		JAVA_1_5 = ((JAVA_VERSION.charAt(0) == '1') && (JAVA_VERSION.charAt(2) == '5'));
		JAVA_1_4 = ((JAVA_VERSION.charAt(0) == '1') && (JAVA_VERSION.charAt(2) == '4'));
		OS_WINDOWS = (File.separatorChar == '\\'); // String os = System.getProperty("os.name"); ((os != null) && os.startsWith("Windows"))
	}

	private EnvironmentProperties() { } // this class should not be instantiated

	public static boolean isRunUnderWebStart() {
		if (RUN_UNDER_WEB_START == null) {
			Object serviceNames;
			try {
				serviceNames = javax.jnlp.ServiceManager.getServiceNames();
			} catch (Throwable t) {
				serviceNames = null;
			}
			RUN_UNDER_WEB_START = (serviceNames != null);
		}
		return RUN_UNDER_WEB_START;
	}

	public static boolean isIsJhelpAvailable() {
		if (IS_JHELP_AVAILABLE == null)
			try {
				new javax.help.HelpSet();
				IS_JHELP_AVAILABLE = true;
			} catch (Throwable t) {
				IS_JHELP_AVAILABLE = false;
			}
		return IS_JHELP_AVAILABLE;
	}

	public static boolean isIsFopAvailable() {
		if (IS_FOP_AVAILABLE == null) {
			try {
				new org.apache.fop.apps.Driver();
				IS_FOP_AVAILABLE = isIsBatikAvailable();
			} catch (Throwable t) {
				IS_FOP_AVAILABLE = false;
			}
		}
		return IS_FOP_AVAILABLE;
	}

	public static boolean isIsBatikAvailable() {
		if (IS_BATIK_AVAILABLE == null)
			try {
				new org.apache.batik.swing.JSVGCanvas();
				IS_BATIK_AVAILABLE = true;
			} catch (Throwable t) {
				IS_BATIK_AVAILABLE = false;
			}
		return IS_BATIK_AVAILABLE;
	}
}
