package at.ac.tuwien.asgaard.delta.util;

import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.swing.SwingUtils;

import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.List;

/**
 * Class Preferences.... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
public final class Preferences extends AbstractDELTAProperties {
	public static final String DEFAULT_USER_PREFERENCES_FILE  = "delta.preferences";
	public static final String STATE_PROPERTIES_FILE   = "delta.state";

	private static final String LIST_SEPARATOR = "|";

	public static final String KEY_ADVANCED_MODE  = "general.advanced_mode";

	public static final String KEY_HTML_CHECK_LINKS  = "html.check.links";
	public static final String KEY_HTML_REMOVE_DELTA_LINKS_ON_SAVE  = "html.onsave.remove.gmt_links";
	public static final String KEY_HTML_USE_LINK_SEPARATOR   = "html.use.link.separator";
	public static final String KEY_HTML_LINK_SEPARATOR1   = "html.link.separator1";
	public static final String KEY_HTML_LINK_SEPARATOR2  = "html.link.separator2";

	public static final String KEY_XML_LINK_SINGLECLICK     = "xml.link.singleClick";
	public static final String KEY_XML_DTD_STATE      = "xml.dtd.state";

	public static final String KEY_XML_FILTER_DOCTYPE_NODE[] = {"xml.left.tree.filter.doctype_node","xml.right.tree.filter.doctype_node"};
	public static final String KEY_XML_FILTER_ROOT_ELEMENT[] = {"xml.left.tree.filter.root_element","xml.right.tree.filter.root_element"};
	public static final String KEY_XML_FILTER_COMMENT_NODES[]  = {"xml.left.tree.filter.comment_nodes","xml.right.tree.filter.comment_nodes"};
	public static final String KEY_XML_FILTER_CDATA_NODES[]  = {"xml.left.tree.filter.cdata_nodes","xml.right.tree.filter.cdata_nodes"};
	public static final String KEY_XML_FILTER_TEXT_NODES[]   = {"xml.left.tree.filter.text_nodes","xml.right.tree.filter.text_nodes"};
	public static final String KEY_XML_FILTER_PI_NODES[]     = {"xml.left.tree.filter.pi_nodes","xml.right.tree.filter.pi_nodes"};

	public static final String KEY_XML_SHOW_XPATH_BAR[]  = {"xml.left.show.xpath_bar", "xml.right.show.xpath_bar"};
	public static final String KEY_XML_SHOW_TREEPATH_BAR[] = {"xml.left.show.treepath_bar", "xml.right.show.treepath_bar"};
	public static final String KEY_XML_SHOW_AUTOXPATH_BAR[] = {"xml.left.show.autoxpath_bar", "xml.right.show.autoxpath_bar"};

	public static final String KEY_XML_TREE_DRAGNDROP     = "xml.tree.drag_and_drop";
	public static final String KEY_XML_TREE_EXPAND_ALL_ON_LOAD = "xml.tree.expandall.onload";
	public static final String KEY_XML_SHOW_CHILD_COUNT     = "xml.tree.show.childcount";
	public static final String KEY_XML_SHOW_ATTRIBUTES     = "xml.tree.show.attributes";
	public static final String KEY_XML_ASK_ATTR_FOR_EACH    = "xml.ask.attribute.for.each";
	public static final String KEY_XML_INSERT_EMPTY_ATTR    = "xml.insert.empty.attribute";
	public static final String KEY_XML_TABLE_EDIT_BUTTON    = "xml.table.show.edit_button";
	public static final String KEY_XML_TABLE_SHOW_LINKED_ATTRIBUTES_BUTTON   = "xml.table.show.linked_attrs_button";
	public static final String KEY_XML_SELECT_ON_HIGHLIGHT    = "xml.tree.select_on_highlight";
	public static final String KEY_XML_SHOW_INSERTABLE_COUNT   = "xml.show.insertable_count";

	public static final String KEY_XML_AUTO_COLLAPSE_ON_EXPAND     = "xml.tree.auto.collapse.on.expand";
	public static final String KEY_XML_AUTO_COLLAPSE_ON_SELECT     = "xml.tree.auto.collapse.on.select";
	public static final String KEY_XML_AUTO_SELECT_ON_FOLDING     = "xml.tree.auto.select.on.folding";
	public static final String KEY_XML_AUTO_EXPAND_ON_SELECT     = "xml.tree.auto.expand.on.select";

	public static final String KEY_APPEARANCE_XML_FISHEYE_VIEW     = "appearance.xml.tree.fisheyeview";
	public static final String KEY_APPEARANCE_PREVIOUS_LAF = "swing.previousLaF";

	public static final	String KEY_COLOR_LINK_ACTIVATED_BG			   = "appearance.color.link.activated.bg";
	public static final	String KEY_COLOR_LINK_FG					   = "appearance.color.link.fg";
	public static final	String KEY_COLOR_HTML_LINK_FG				   = "appearance.color.html.link.fg";
	public static final	String KEY_COLOR_LINK_HTML_OVERLAPPING_FG	   = "appearance.color.link.html.overlapping.fg";
	public static final	String KEY_COLOR_LINK_HTML_DELIM			   = "appearance.color.link.html.delimiter.fg";
	public static final	String KEY_COLOR_SELECTION					   = "appearance.color.selection.bg";
	public static final	String KEY_COLOR_XML_INVALID_NODES_ON_VALIDATE = "appearance.color.xml.invalid_nodes.on_validate.bg";
	public static final	String KEY_COLOR_XML_INVALID_NODES_ON_LOAD	   = "appearance.color.xml.invalid_nodes.on_load.fg";
	public static final	String KEY_COLOR_XML_SUMMARIZE				   = "appearance.color.xml.summarize.fg";
	public static final	String KEY_COLOR_SEARCH_RESULT				   = "appearance.color.search_result.bg";
	public static final	String KEY_COLOR_SIDE_BORDER_ACTIVE			   = "appearance.color.side_border.active";
	public static final	String KEY_COLOR_SIDE_BORDER_INACTIVE		   = "appearance.color.side_border.inactive";

	public static final	String KEY_APPEARANCE_ROLLOVER_XML		 = "appearance.xml.tree.rollover";
	public static final	String KEY_APPEARANCE_DYNAMIC_LAYOUT	 = "appearance.dynamic_layout";
	public static final	String KEY_APPEARANCE_CONTINUOUS_LAYOUT	 = "appearance.splitpane.continuous_layout";
	public static final	String KEY_APPEARANCE_TOOLBARS_FLOATABLE = "appearance.toolbars.floatable";
	public static final	String KEY_APPEARANCE_TOOLBARS_ROLLOVER	 = "appearance.toolbars.rollover";

	public static final String KEY_ASK_BEFORE_REMOVE_HTML_LINKS = "general.confirm.remove.html.links";
	public static final String KEY_ASK_BEFORE_REMOVE_XML_LINKS  = "general.confirm.remove.xml.links";
	public static final String KEY_ASK_BEFORE_REMOVE_XML_NODES  = "general.confirm.remove.xml.nodes";
	public static final String KEY_ASK_BEFORE_REMOVE_XML_SUBTREE  = "general.confirm.remove.xml.subtree";
	public static final String KEY_ASK_BEFORE_REMOVE_XML_ATTRIBUTES = "general.confirm.remove.xml.attribute";

    public static final String KEY_XML_AUTO_CHANGE_DOCTYPE = "xml.change.docType";
    public static final int CHANGE_DOCTYPE_NO  = 2;
    public static final int CHANGE_DOCTYPE_ASK = 1;
    public static final int CHANGE_DOCTYPE_YES = 0;

	public static final	String	 KEY_AUTO_SAVE		   = "general.autosave";
	public static final	String	 KEY_REMOVE_DEAD_LINKS = "general.remove.dead.links";
	public static final	String	 KEY_BACKUP_ON_SAVE	   = "general.backup.on.save";
	public static final	String	 KEY_OPEN_LAST_FILES   = "general.openLast.files";
	public static final	String	 KEY_OPEN_LAST_PROJECT = "general.openLast.project";
	public static final	String	 KEY_LAST_MACROS_FILE  = "general.last.macrosFile";
	public static final	String	 KEY_LAST_PROJECT_FILE = "general.last.project";
	public static final	String	 KEY_LAST_LEFT_FILE	   = "general.last.leftFile";
	public static final	String	 KEY_LAST_RIGHT_FILE   = "general.last.rightFile";
	public static final	String[] KEY_LAST_FILE		   = {KEY_LAST_LEFT_FILE, KEY_LAST_RIGHT_FILE};

	public static final int PROJECT_CHANGE_UNSET = 0, PROJECT_CHANGE_CLOSE = 1, PROJECT_CHANGE_ADJUST = 2, PROJECT_CHANGE_SAVEAS = 3, PROJECT_CHANGE_ASK = 4;
	public static final String KEY_PROJECT_ON_CHANGE  = "general.project.change";

	public static final String KEY_RECENT_FILES_MAX_COUNT   = "general.recentFiles";
	public static final String KEY_RECENT_XHTML_FILES  = "general.recentFiles.xhtml";
	public static final String KEY_RECENT_MACROS_FILES  = "general.recentFiles.macros";
	public static final String KEY_RECENT_PROJECT_FILES = "general.recentFiles.project";
	public static final String KEY_RECENT_DIRS   = "general.recentDirs";

	public static final String KEY_LAST_ACTIVE_SIDE  = "general.last.activeside";

	public static final String KEY_LAYOUT_WINDOW_POS_X   = "layout.window.position.x";
	public static final String KEY_LAYOUT_RESTORE   = "appearance.layout.restore";
	public static final String KEY_LAYOUT_WINDOW_POS_Y   = "layout.window.position.y";
	public static final String KEY_LAYOUT_WINDOW_SIZE_WIDTH  = "layout.window.size.width";
	public static final String KEY_LAYOUT_WINDOW_SIZE_HEIGHT  = "layout.window.size.height";
	public static final String KEY_LAYOUT_SPLIT_MAIN_DIV_LOC  = "layout.split.main.divider.location";
	public static final String KEY_LAYOUT_SPLIT_MAIN_DIV_LOC_LAST  = "layout.split.main.divider.location.last";

	public static final String KEY_LAYOUT_SPLIT_SIDES_DIV_LOC   = "layout.split.sides.divider.location";
	public static final String KEY_LAYOUT_SPLIT_SIDES_DIV_LOC_LAST = "layout.split.sides.divider.location.last";
	public static final String KEY_LAYOUT_SPLIT_LEFT_DIV_LOC   = "layout.split.left.divider.location";
	public static final String KEY_LAYOUT_SPLIT_LEFT_DIV_LOC_LAST = "layout.split.left.divider.location.last";
	public static final String KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC   = "layout.split.right.divider.location";
	public static final String KEY_LAYOUT_SPLIT_RIGHT_DIV_LOC_LAST = "layout.split.right.divider.location.last";

	public static final String KEY_LAYOUT_SPLIT_MACROS_DIV_LOC  = "layout.split.macros.divider.location";
	public static final String KEY_LAYOUT_SPLIT_MACROS_DIV_LOC_LAST = "layout.split.macros.divider.location.last";

	public static final String KEY_LAYOUT_WINDOW_PROJECT_POS_X   = "layout.window.project.position.x";
	public static final String KEY_LAYOUT_WINDOW_PROJECT_POS_Y   = "layout.window.project.position.y";
	public static final String KEY_LAYOUT_WINDOW_PROJECT_SIZE_WIDTH  = "layout.window.project.size.width";
	public static final String KEY_LAYOUT_WINDOW_PROJECT_SIZE_HEIGHT  = "layout.window.project.size.height";

	public static final String KEY_LAYOUT_WINDOW_LANG_POS_X   = "layout.window.lang.position.x";
	public static final String KEY_LAYOUT_WINDOW_LANG_POS_Y   = "layout.window.lang.position.y";
	public static final String KEY_LAYOUT_WINDOW_LANG_SIZE_WIDTH  = "layout.window.lang.size.width";
	public static final String KEY_LAYOUT_WINDOW_LANG_SIZE_HEIGHT  = "layout.window.lang.size.height";


	public static final String KEY_MACROS_INSERT_MODE_AUTO_SWITCH = "macros.insert_mode.auto_switch";
	public static final String KEY_MACROS_INSERT_MODE_SWITCH_BACK = "macros.insert_mode.switch.back";
	public static final String KEY_MACROS_MIN_LIST_WIDTH    = "macros.list.min.width";
	public static final String KEY_MACROS_MIN_LIST_HEIGHT    = "macros.list.min.height";

	public static final String KEY_CLEAR_RECENT_FILES = "clearRecentFiles";
// public static final String KEY_RESTORE_LAYOUT  = "restoreLayout";
	public static final String KEY_RESTORED_DEFAULT  = "restoredDefault";

    public static final String KEY_MACROS_SEARCH_ONLY_MACROS  = "macros.search.only.macros";
	public static final String KEY_MACROS_SEARCH_IGNORE_CASE  = "macros.search.ignore.case";
	public static final String KEY_MACROS_SEARCH_SYNC_VIEWS  = "macros.search.sync.views";
	
	public static final String KEY_COMBOBOX_EDIT_TEXT   = "combobox.xml.edit.text";
	public static final String KEY_COMBOBOX_FIND  =   "combobox.find";
    public static final String KEY_COMBOBOX_XPATH           = "combobox.xml.xpath";

    public static final String KEY_PATH_TO_PDF_VIEWER = "path.pdf.viewer";
	public static final String KEY_USER_NAME = "user.name";


	public Preferences() {  // for testing only
		super(DEFAULT_USER_PREFERENCES_FILE, true, true, true);
	}
	public Preferences(String propertiesFilename, boolean useDefaultProperties, boolean load) {
		super(propertiesFilename, useDefaultProperties, load, true);
	}

	protected String getPropertiesHeader() {
		return "Preferences file for the Guideline Markup Tool";
	}

	protected Properties createDefaultProperties() {
		Properties defaultProperties = new Properties();
		try {
			defaultProperties.load(getClass().getResourceAsStream("default.properties"));
		} catch (IOException e) {
			e.printStackTrace();
		}
		return defaultProperties;
	}

	public void restoreDefaults() {
		clear();
		putAll(defaultProperties);
	}

	public String getProperty(String key) {
		if (key == null) return null;
		String ret = System.getProperty(key);
		if (ret == null)
			ret = super.getProperty(key);
		return ret;
	}

	public Color getColor(String key) {
		Color color = SwingUtils.stringToColor(getProperty(key));
		if (color == null) {
			color = new Color(getInt(key)); // try old format
			putColor(key, color);
		}
		return color;
	}

	public void putColor(String key, Color color) {
	//	putInt(key, color.getRGB());
		putString(key, SwingUtils.colorToHex(color));
	}

	public List<String> getStringList(String key) {
		List<String> list = new ArrayList<String>();
        String text;
        if ((key == null) || ((text = getString(key)) == null))
			return list;
		StringTokenizer st = new StringTokenizer(text, LIST_SEPARATOR);
		while (st.hasMoreTokens())
			list.add(st.nextToken());
		return list;
	}

	public void putStringList(String key, List list) {
		if (key == null) return;
		StringBuffer sb = new StringBuffer();
		if (list != null) {
			Iterator iter = list.iterator();
			for (int i = list.size() - 1; i >= 0; i--) {
				sb.append((String)iter.next());
				if (iter.hasNext()) sb.append(LIST_SEPARATOR);
			}
		}
		putString(key, sb.toString());
	}

	public String getFilePath(String key) {
		String filePath = super.getString(key);
		if (filePath == null) return null;
		if (FileUtils.toExistingAbsoluteFile(filePath, propertiesFile.getParentFile()) != null)
			return filePath;
		else {
			putString(key, "");  // remove property if file doesn't exist
			return null;
		}
	}

	public List<String> getFilePathList(String key) {
		String files;
		boolean haveToReset = false;
		Set<String> set = new LinkedHashSet<String>();
		if ((key == null) || ((files = getString(key)) == null))
			return new ArrayList<String>();
		StringTokenizer st = new StringTokenizer(files, LIST_SEPARATOR);
		while (st.hasMoreTokens()) {
			String filePath = st.nextToken();
			if (FileUtils.toExistingAbsoluteFile(filePath, propertiesFile.getParentFile()) != null)
				set.add(filePath);
			else
				haveToReset = true;
		}
		List<String> list = new ArrayList<String>(set);
		if (haveToReset) putStringList(key, list);
		return list;
	}

	public File getFile(String key) {
		String filePath = getString(key);
		return FileUtils.toExistingAbsoluteFile(filePath, propertiesFile.getParentFile());
	}
	public void putFile(String key, File file, boolean storeRelativePath) {
		if ((key == null) || (file == null)) return;
		if (storeRelativePath)
			setProperty(key, FileUtils.toRelativeFile(file, propertiesFile));
		else
			setProperty(key, file.getAbsolutePath());
	}

	public List<File> getFileList(String key) {
		String files;
		boolean haveToReset = false;
		Set<File> set = new LinkedHashSet<File>();
		if ((key == null) || ((files = getString(key)) == null))
			return new ArrayList<File>();
		StringTokenizer st = new StringTokenizer(files, LIST_SEPARATOR);
		while (st.hasMoreTokens()) {
			File file = FileUtils.toExistingAbsoluteFile(st.nextToken(), propertiesFile.getParentFile());
			if (file == null) haveToReset = true;
			else set.add(file);
		}
		List<File> list = new ArrayList<File>(set);
		if (haveToReset) putFileList(key, list, true);
		return list;
	}

	public void putFileList(String key, Collection<File> list, boolean storeRelativePaths) {
		if (key == null) return;
		if (list != null) {
			list = new LinkedHashSet<File>(list);
			StringBuffer sb = new StringBuffer();
			Iterator iter = list.iterator();
			for (int i = list.size() - 1; i >= 0; i--) {
				File file = ((File)iter.next());
				if (storeRelativePaths)
					sb.append(FileUtils.toRelativeFile(file));
				else
					sb.append(file.getAbsolutePath());
				if (iter.hasNext())
					sb.append(LIST_SEPARATOR);
			}
			putString(key, sb.toString());
		} else
			putString(key, "");
	}

	public Integer getInteger(String key) {
		String property = getProperty(key);
		if (property == null) return null;
		return new Integer(property);
	}
}
