/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */

package at.ac.tuwien.asgaard.delta.util;

import at.ac.tuwien.asgaard.util.xml.XMLUtils;

import javax.xml.transform.*;
import javax.xml.transform.stream.StreamSource;
import java.io.*;
import java.net.URL;

/**
 * Class Transform ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.4.0
 */
public class Transform {
	public static Transformer convertDTDTransformer;

	public static void convertDTD(URL sourceURL, File targetFile) throws TransformerException, IOException {
		if (convertDTDTransformer == null) {
			Source xslSource = new StreamSource(new BufferedReader(new InputStreamReader(Transform.class.getResourceAsStream("convertDTD.xsl"))));
			convertDTDTransformer = XMLUtils.getTransformerFactory().newTransformer(xslSource);
		}
		XMLUtils.transformDTD(sourceURL, new BufferedWriter(new FileWriter(targetFile)), convertDTDTransformer);
	}

	public static void main(String[] args) {
		if ((args.length < 1) || ((args.length == 1) && (args[0].charAt(0) == '-'))) {
			System.out.println("Usage: java at.ac.tuwien.asgaard.delta.util.Transform [-L|-D|-S] <original DTD> [<target-file>]");
			System.out.println("  -L ... transform DTD to link-enriched DTD   (default option)");
			System.out.println("  -D ... transform DTD to HTML documentation");
			System.out.println("  -S ... transform DTD to 'syntax-highlighted' source (HTML)");
			System.out.println("Usage: java at.ac.tuwien.asgaard.delta.util.Transform -T <sourceXML> <targetX(HT)ML> <XSL file>");
			System.out.println("  ... transform <sourceXML> to <targetXML> using <XSL file>");
            System.out.println("Usage: java at.ac.tuwien.asgaard.delta.util.Transform -X <sourceXML> <targetHTML>");
            System.out.println("  ... transform XML file <sourceXML> to formatted source <targetHTML>");
			return;
		}

		char type = 'L';
		int firstArgIndex = 0;
		if (args[0].charAt(0) == '-') {
			firstArgIndex = 1;
			type = args[0].charAt(1);
		}

		String sourceFilePath = args[firstArgIndex];
		File sourceFile = new File(sourceFilePath); //FileUtils.toFile(sourceFilePath); //

		File targetFile;
		if (args.length > firstArgIndex+1) {
			targetFile = new File(args[firstArgIndex + 1]); //FileUtils.toFile(args[firstArgIndex + 1]); //
			if (sourceFile.equals(targetFile)) {
				System.out.println("<source-file> must NOT match <target-file> !");
				return;
			}
		} else {
			String sourceFileName = getFileName(sourceFilePath); // FileUtils.
			String suffix = (type == 'L' ? "_DELTA.dtd" : (type == 'D' ? "-dtd_doc.html" : "-dtd_src.html"));
			targetFile = new File(sourceFile.getParentFile(), sourceFileName + suffix);
		}

		System.out.println("Transforming \""+sourceFile+"\"\n  to \""+targetFile+"\" ...");

		try {
			switch (type) {
				case 'L' : convertDTD(sourceFile.toURI().toURL(), targetFile); break;
				case 'D' : XMLUtils.transformDTD2Report(sourceFile.toURI().toURL(), new BufferedWriter(new FileWriter(targetFile))); break;
				case 'S' : XMLUtils.transformDTD2Source(sourceFile.toURI().toURL(), new BufferedWriter(new FileWriter(targetFile))); break;
				case 'T' : XMLUtils.transform(sourceFile, new File(args[3]), targetFile); break;
				case 'X' :
                    XMLUtils.transform(new FileReader(sourceFile), new InputStreamReader(XMLUtils.class.getResourceAsStream("xml2html.xsl")), new FileWriter(targetFile));
                    break;
			}
			System.out.println("Done.");
		} catch (IOException exc) {
			System.err.println("I/O error occured: "+exc.getMessage());
		} catch (TransformerException exc) {
			System.err.println("XSL-Transformation error occured: "+exc.getMessageAndLocation());
		} catch (Exception exc) {
                        System.err.println("Error occured: "+exc);
                }
	}

	// instead of using FileUtils#getFileName(), so FileUtils don't need to be included in transformDTD.jar
	private static String getFileName(String filePath) {
		int lastSepPos = filePath.lastIndexOf(File.separatorChar);
		if ((lastSepPos == -1) && (File.separatorChar != '/')) lastSepPos = filePath.lastIndexOf('/');
		if ((lastSepPos == -1)) lastSepPos = filePath.lastIndexOf(':');
		String fileNameExt = filePath.substring(lastSepPos + 1);
		int dotPos = fileNameExt.lastIndexOf('.');
		if (dotPos < 1) return fileNameExt;
		return fileNameExt.substring(0, dotPos);
	}
}
