/*
 * Project: DELTA - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 09.07.2003
 * Time: 16:23:27
 */
package at.ac.tuwien.asgaard.delta.xmlBeans;

import at.ac.tuwien.asgaard.util.FileUtils;
import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.MiscUtils;

import net.sf.saxon.trans.XPathException;
import net.sf.saxon.xpath.XPathEvaluator;
import net.sf.saxon.xpath.XPathExpressionImpl;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import java.io.File;
import java.net.URL;

import javax.swing.*;
import javax.xml.transform.TransformerException;
import javax.xml.transform.dom.DOMSource;

/**
 * Class Definition ... todo
 *
 * @author Peter Votruba
 * @version 2.7.3.#452
 * @since JDK 1.3.0
 */
public abstract class AbstractXMLBean {
	protected final Document documentNode;
	protected final Element rootElement;

	protected String name;
	protected String iconFileName;
	protected Icon icon;
	protected String description;

//    protected static org.apache.xpath.XPath xpathDescription, xpathDescriptionText;
	protected final XPathEvaluator xpeval;
	protected final XPathExpressionImpl xpathDescription, xpathDescriptionText;

	protected AbstractXMLBean(Document documentNode) {
		this.documentNode = documentNode;
		rootElement = documentNode.getDocumentElement();

		try {
			xpeval = new XPathEvaluator(new DOMSource(rootElement));
			xpathDescription = xpeval.createExpression("description");
			xpathDescriptionText = xpeval.createExpression("description/text()");
		} catch (XPathException exc) {
			throw new RuntimeException(exc); // should not happen
		}

		/*if (xpathDescription == null)
			try {
				xpathDescription        = new org.apache.xpath.XPath("description", null, null, org.apache.xpath.XPath.SELECT, null);//new DOMXPath("description");
				xpathDescriptionText    = new org.apache.xpath.XPath("description/text()", null, null, org.apache.xpath.XPath.SELECT, null);//new DOMXPath("description/text()");
			} catch (TransformerException exc) {
				throw new RuntimeException(exc); // should not happen
			}*/
	}

	public String getName() {
		if (name == null)
			name = rootElement.getAttribute("name");
		return name;
	}

	public void setName(String name) {
		this.name = name;
		rootElement.setAttribute("name", name);
	}

	public void setIconFileName(String iconFileName) {
		if (!MiscUtils.equals(this.iconFileName, iconFileName)) {
			this.iconFileName = iconFileName;
			rootElement.setAttribute("iconFileName", iconFileName);
			icon = null;
			icon = getIcon();
		}
	}

	public String getIconFileName() {
		if (iconFileName == null)
			iconFileName = rootElement.getAttribute("iconFileName");
		return iconFileName;
	}

	public abstract Icon getIcon();

	protected Icon getIcon(File relativeTo) {
		if (icon == null) {
			URL url = FileUtils.toURL(getIconFileName(), relativeTo);
			if (url != null) {
				icon = new ImageIcon(url);
				if (icon.getIconWidth() < 10) icon = getDefaultIcon();
			} else
				icon = getDefaultIcon();
		}
		return icon;
	}

	public abstract Icon getDefaultIcon();

	public String getDescription() {
		if (description == null)
			try {
				Text textNode = (Text) xpathDescriptionText.evaluateSingle();
				// Text textNode = (Text) selectSingleNode(rootElement, xpathDescriptionText);//xpathDescriptionText.selectSingleNode(rootElement);
				if (textNode == null) return null;
				description = textNode.getData();
			} catch (TransformerException exc) {
				return null; // should not happen
			}
		return description;
	}

	public void setDescription(String description) {
		if (description != null) {
			description = description.trim();
			if (description.equals("")) description = null;
		}

		if (!MiscUtils.equals(this.description, description)) {
			this.description = description;
			try {
				Text textNode = (Text) xpathDescriptionText.evaluateSingle();
				// Text textNode = (Text) selectSingleNode(rootElement, xpathDescriptionText);//xpathDescriptionText.selectSingleNode(rootElement);
				if (textNode == null) {
					if (description != null)
						try {
							Element element = (Element) xpathDescription
									.evaluateSingle(); //selectSingleNode(rootElement, xpathDescription);//xpathDescription.selectSingleNode(rootElement);
							if (element == null) {
								element = documentNode.createElement("description");
								rootElement.insertBefore(element, rootElement.getFirstChild());
							}
							textNode = documentNode.createTextNode(description);
							element.appendChild(textNode);
						} catch (Exception exc) {
							Log.trace("Couldn't set project description: " + exc.getMessage());
						}
				} else if (description != null)
					textNode.setData(description);
				else
					rootElement.removeChild(textNode.getParentNode());
			} catch (TransformerException exc) {
				throw new RuntimeException(exc); // should not happen
			}
		}
	}

	public String toString() {
		return getName();
	}

/* public static Node selectSingleNode(Node contextNode, org.apache.xpath.XPath xpath) throws TransformerException {
		XPathContext xpathSupport = new XPathContext();
		int ctxtNode = xpathSupport.getDTMHandleFromNode(contextNode);
		XObject xObject = xpath.execute(xpathSupport, ctxtNode, null);
		NodeIterator nodeIterator = xObject.nodeset();
		return nodeIterator.nextNode();
	}*/
}
