/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 16, 2002
 * Time: 3:46:25 PM
 */
package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.util.EnvironmentProperties;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.util.JHelpUtils;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Class AbstractInputDialog. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public abstract class AbstractInputDialog extends JDialog {
	/**
	 * The key used for storing a URL to a help page to be used in the <code>actionPerformed</code> method
	 * for the <code>helpAction</code> action.
	 */
	public static final String HELP_LINK_KEY = "HelpLinkKey";

	protected boolean approved;
	protected final boolean useApplyButton, useCancelButton, useHelpButton;
	protected Action okAction, applyAction, cancelAction, helpAction;
	protected final JPanel contentPane;

	protected final int MIN_WIDTH, MIN_HEIGHT, MAX_WIDTH, MAX_HEIGHT;

    protected AbstractInputDialog(Component parentComponent, String title,
								  boolean useApplyButton, boolean useHelpButton,
								  final int INIT_WIDTH, final int INIT_HEIGHT,
								  int minWidth, int minHeight, int maxWidth, int maxHeight) {
        this(parentComponent, title, useApplyButton, true, useHelpButton, INIT_WIDTH, INIT_HEIGHT, minWidth, minHeight, maxWidth, maxHeight, null);
    }

	/** if minWidth/minHeight arg < 0 then it will become the same as INIT_WIDTH/INIT_HEIGHT
	  * if maxWidth/maxHeight arg < 0 then it will be the same as INIT_WIDTH/INIT_HEIGHT
	  * if maxWidth/maxHeight arg < minWidth/minHeight then the max. window size will be the same as the screen size
	  * for example: new AbstractInputDialog(parent, title, false, false, 200, 100, -1, -1, -1, -1) will create a non-resizeable dialog */
	protected AbstractInputDialog(Component parentComponent, String title,
								  boolean useApplyButton, boolean useCancelButton, boolean useHelpButton,
								  final int INIT_WIDTH, final int INIT_HEIGHT,
								  int minWidth, int minHeight, int maxWidth, int maxHeight,
								  String okButtonText) {
		super(JOptionPane.getFrameForComponent(parentComponent), title, true);

		this.useHelpButton	 = useHelpButton;
		this.useApplyButton	 = useApplyButton;
		this.useCancelButton = useCancelButton;

		createActions();
		useHelpButton = (useHelpButton && (helpAction.getValue(HELP_LINK_KEY) != null)); // only show help button if HELP_LINK_KEY is set

		if (okButtonText == null) okButtonText = "OK";
        okAction.putValue(Action.NAME, okButtonText);

		contentPane = new JPanel(new BorderLayout(0, 5));
		contentPane.setBorder(GlobalObjects.EMPTY_BORDER_5);

		JPanel controlPanel = new JPanel(new TableLayout(
				new double[][]{{TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED},  // TableLayout.PREFERRED, TableLayout.FILL, TableLayout.PREFERRED, TableLayout.PREFERRED, TableLayout.PREFERRED},
							   {TableLayout.PREFERRED}}, 5, 0));

		JButton okButton, applyButton = null, cancelButton = null, helpButton = null;
		Insets margin = new Insets(2, 5, 2, 5);
		okButton = new JButton(okAction);
		okButton.setMargin(margin);
		if (!useApplyButton && !useCancelButton && !useHelpButton)
			controlPanel.add(okButton, new TableLayoutConstraints(4, 0));
		else
			controlPanel.add(okButton, new TableLayoutConstraints(0, 0));

		if (useApplyButton) {
			applyButton = new JButton(applyAction);
			applyButton.setMargin(margin);
			controlPanel.add(applyButton, new TableLayoutConstraints(1, 0));  // 3, 0
		//	controlPanel.add(cancelButton, new TableLayoutConstraints(2, 0));
		} //else
		//	controlPanel.add(cancelButton, new TableLayoutConstraints(1, 0));
		if (useCancelButton) {
			cancelButton = new JButton(cancelAction);
			cancelButton.setMargin(margin);
			controlPanel.add(cancelButton, new TableLayoutConstraints(2, 0));
		}
		if (useHelpButton) {
			helpButton = new JButton(helpAction);
			helpButton.setMargin(margin);
			controlPanel.add(helpButton, new TableLayoutConstraints(4, 0)); // 0,0
		}

		final Component mainComponent = createMainComponent();
		if (mainComponent != null) // if createMainComponent() couldn't be implemented
			contentPane.add(mainComponent, BorderLayout.CENTER);
		contentPane.add(controlPanel, BorderLayout.SOUTH);
		setContentPane(contentPane);

		okButton.requestFocusInWindow();
		rootPane.setDefaultButton(okButton);

		rootPane.getActionMap().put("cancelAction", cancelAction);
		rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancelAction");
		rootPane.getActionMap().put("helpAction", helpAction);
		rootPane.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0), "helpAction");

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	// setBounds((screenSize.width - INIT_WIDTH) >> 1, ((screenSize.height - INIT_HEIGHT) >> 1) - 20, INIT_WIDTH, INIT_HEIGHT);
		setSize(INIT_WIDTH, INIT_HEIGHT);
		setLocationRelativeTo(parentComponent); // todo

		Dimension preferredSize = okButton.getPreferredSize();
		int width = preferredSize.width; //okButton.getWidth();
		if (useCancelButton)
			width = Math.max(width, cancelButton.getPreferredSize().width);
		if (useHelpButton)
			width = Math.max(width, helpButton.getPreferredSize().width);
		preferredSize.width = width;
		okButton.setPreferredSize(preferredSize);
		if (useCancelButton)
			cancelButton.setPreferredSize(preferredSize);
		if (useApplyButton)
			applyButton.setPreferredSize(preferredSize);
		if (useHelpButton)
			helpButton.setPreferredSize(preferredSize);

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent evt) {
				if (onCancel()) {
					approved = false;
					setVisible(false);
				}
			}
		});

        // workaround for setResizeable(false); which would hide the titlebar icon - BUG #4295864
		MIN_WIDTH = (minWidth < 0) ? INIT_WIDTH : minWidth;
		MIN_HEIGHT = (minHeight < 0) ? INIT_HEIGHT : minHeight;
		MAX_WIDTH = (maxWidth < 0) ? INIT_WIDTH : (maxWidth < MIN_WIDTH) ? screenSize.width : maxWidth;
		MAX_HEIGHT = (maxHeight < 0) ? INIT_HEIGHT : (maxHeight < MIN_HEIGHT) ? screenSize.height : maxHeight;
		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				if (evt.getID() == ComponentEvent.COMPONENT_RESIZED) {
					final int width = getWidth(), height = getHeight();
					if ((width < MIN_WIDTH) && (height < MIN_HEIGHT))
						setSize(MIN_WIDTH, MIN_HEIGHT);
					else if (width < MIN_WIDTH)
						setSize(MIN_WIDTH, height);
					else if (height < MIN_HEIGHT)
						setSize(width, MIN_HEIGHT);
					if ((width > MAX_WIDTH) && (height < MAX_HEIGHT))
						setSize(MAX_WIDTH, MAX_HEIGHT);
					else if (width > MAX_WIDTH)
						setSize(MAX_WIDTH, height);
					else if (height > MAX_HEIGHT)
						setSize(width, MAX_HEIGHT);
				}
			}
		});
	}

	protected Component createMainComponent() {
        return null;
    }

	protected void createActions() {
		okAction = new AbstractAction("OK") {
			{ // Constructor
				putValue(MNEMONIC_KEY, new Integer('O'));
			}
			public void actionPerformed(ActionEvent evt) {
				if (onOK()) {
					approved = true;
					setVisible(false);
				}
			}
		};
		if (useApplyButton)
			applyAction  = new SimpleAction("Apply", 'A') {
				public void simpleActionPerformed() {
					onOK();
				}
			};
		if (useCancelButton)
			cancelAction = new SimpleAction("Cancel", 'C') {
				public void simpleActionPerformed() {
					if (onCancel()) {
						approved = false;
						setVisible(false);
					}
				}
			};
		if (useHelpButton)
			helpAction = new AbstractAction("Help", MainImages.HELP_ICON) {
				{ // Constructor
					putValue(MNEMONIC_KEY, new Integer('H'));
					putValue(ACCELERATOR_KEY, new Integer(KeyEvent.VK_F1));
				//	putValue(HELP_LINK_KEY, "index"); // if not set in subclass, help button will not be shown
				}
				public void actionPerformed(ActionEvent evt) {
				// BrowserControl.displayURL((String)getValue(HELP_LINK_KEY));
					if (!EnvironmentProperties.isIsJhelpAvailable())
						JOptionPane.showMessageDialog(AbstractInputDialog.this, "jhelp.jar is missing!\n\nDownload DELT/A-Help package or use user manual instead.", "DELT/A Help",
								JOptionPane.INFORMATION_MESSAGE);
					else if (JHelpUtils.getHelpBroker() == null)
						JOptionPane.showMessageDialog(AbstractInputDialog.this, "delta-help.jar is missing!\n\nDownload DELT/A-Help package or use user manual instead.", "DELT/A Help",
								JOptionPane.INFORMATION_MESSAGE);
					else
						JHelpUtils.showHelpOutOfModalDialog((String) getValue(HELP_LINK_KEY), contentPane);
				}
			};
	}

	/** return true if this dialog should be closed on clicking the ok button, false if it should stay open */
	protected boolean onOK() {
		return true;
	}

	/** return true if this dialog should be closed on clicking the cancel button, false if it should stay open */
	protected boolean onCancel() {
		return true;
	}

	public boolean showDialog() {
		setVisible(true);
		return approved;
	}

	public static final class SmallButton extends JButton {
		public SmallButton(Action a) {
			super(a);
			setMargin(GlobalObjects.INSETS_ZERO);
            setFocusable(false);
		}
	}
}

