/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 21.07.2003
 * Time: 10:05:07
 */
package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.swing.models.FixedComboBoxModel;
import at.ac.tuwien.asgaard.swing.selectable.SelectableAction;

import java.awt.*;
import java.awt.event.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * Class BlockComboBox ... todo
 * Taken from <i>http://www2.gol.com/users/tame/swing/examples/JComboBoxExamples1.html / BlockComboBoxExample</i>.
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class BlockComboBox extends JComboBox {
    public static final String SEPARATOR = "SEPARATOR";

    public BlockComboBox(final Object[] items) {
        this(new FixedComboBoxModel(items));
    }
    public BlockComboBox(final List items) {
        this(new FixedComboBoxModel(items.toArray()));
    }

    public BlockComboBox(final ComboBoxModel model) {
		super(model);
        setRenderer(new ComboBoxRenderer());
    }

	// Flag to ensure the we don't get multiple ActionEvents on item selection.
	private boolean selectingItem = false;

	public void setSelectedIndex(int newIndex) {
		if (newIndex == -1)
			setSelectedItem(null);
		else if ((newIndex < -1) || (newIndex >= dataModel.getSize()))
			throw new IllegalArgumentException("setSelectedIndex: " + newIndex + " out of bounds");
		else {
			Object newSelection = dataModel.getElementAt(newIndex);
			Object oldSelection = selectedItemReminder;

		// if ((oldSelection == null) || !oldSelection.equals(newSelection))
			if (newSelection != oldSelection) {
				// Must toggle the state of this flag since this method call may result in ListDataEvents being fired.
				selectingItem = true;
				if (SEPARATOR == newSelection)
					dataModel.setSelectedItem(oldSelection);
				else {
					dataModel.setSelectedItem(newSelection);
					if (newSelection instanceof SelectableAction) {
						((SelectableAction) oldSelection).setSelected(false);
						((SelectableAction) newSelection).setSelected(true);
					} else if (newSelection instanceof ActionListener)
						((ActionListener) newSelection).actionPerformed(null);
				}
				selectingItem = false;
		// }
			fireActionEvent();  // inside or outside if-block? fire ActionEvent on re-selection too?
			}
		}
	}

	public void contentsChanged(ListDataEvent evt) {
		Object oldSelection = selectedItemReminder;
		Object newSelection = dataModel.getSelectedItem();
	// if ((oldSelection == null) || !oldSelection.equals(newSelection)) {
		if (newSelection != oldSelection) {
			selectedItemChanged();
			if (!selectingItem)
				fireActionEvent();
		}
	}

    public int getSelectedIndexTrimmed() {
        Object sObject = dataModel.getSelectedItem();

        for (int index = -1, i = 0, size = dataModel.getSize(); i < size; i++) {
            Object obj = dataModel.getElementAt(i);
            if (obj != SEPARATOR) {
                index++;
                if (obj == sObject)
                    return index;
            }
        }
        return -1;
    }

    private static class ComboBoxRenderer extends JLabel implements ListCellRenderer {
        private static final JSeparator separator = new JSeparator(JSeparator.HORIZONTAL);

        ComboBoxRenderer() {
            setOpaque(true);
            setBorder(new EmptyBorder(1, 1, 1, 1));
        }

        public Component getListCellRendererComponent(JList list,
                                                      Object value, int index, boolean isSelected, boolean cellHasFocus) {
            if (SEPARATOR == value) return separator;

            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            setFont(list.getFont());
            if (value instanceof Action) {
                Action action = (Action) value;
                setText(action.getValue(Action.NAME).toString());
                setToolTipText(action.getValue(Action.SHORT_DESCRIPTION).toString());
				setIcon((Icon)action.getValue(Action.SMALL_ICON));
				setEnabled(action.isEnabled());
            } else {
                setText((value == null) ? "" : value.toString());
				setEnabled(true);
			}
            return this;
        }
    }
}
