package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.delta.gui.actions.SimpleAction;
import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

/**
 * Created by IntelliJ IDEA. User: peter Date: 28.03.2005 Time: 20:40:24 To change this template use File | Settings |
 * File Templates.
 */
public class ButtonScrollPanel extends JPanel {
	protected static final int SCROLL_X = 50;

	protected final JScrollPane scrollPane;
	protected final Action scrollLeft;
	protected final Action scrollRight;
	protected JComponent contentComponent;

	public ButtonScrollPanel(JComponent content) {
		super(new BorderLayout(2, 0));
		this.contentComponent = (content != null) ? content : createContentComponent();

		//setBorder(new CompoundBorder(BorderFactory.createLoweredBevelBorder(), new EmptyBorder(0, 2, 0, 2)));

		//setBorder(GlobalConstants.EMPTY_BORDER_0);
		//setFloatable(false);
		//putClientProperty("JToolBar.isRollover", Boolean.TRUE);
		//setMargin(GlobalConstants.INSETS_ZERO);

		setBorder(GlobalObjects.EMPTY_BORDER_0);

		scrollLeft = new SimpleAction(MainImages.BACK_ICON, "Scroll left") {
			protected void simpleActionPerformed() {
				JViewport viewport = scrollPane.getViewport();
				Point viewPosition = viewport.getViewPosition();
				viewPosition.x -= SCROLL_X;
				if (viewPosition.x < SCROLL_X) {
					viewPosition.x = 0;
					scrollLeft.setEnabled(false);
				}
				scrollRight.setEnabled(true);
				viewport.setViewPosition(viewPosition);
			}
		};
		add(createButton(scrollLeft), BorderLayout.WEST);

		scrollPane = new JScrollPane(contentComponent, JScrollPane.VERTICAL_SCROLLBAR_NEVER,
									 JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		add(scrollPane, BorderLayout.CENTER);

		scrollRight = new SimpleAction(MainImages.FORWARD_ICON, "Scroll right") {
			protected void simpleActionPerformed() {
				JViewport viewport = scrollPane.getViewport();
				Point viewPosition = viewport.getViewPosition();
				viewPosition.x += SCROLL_X;
				int rightestX = contentComponent.getWidth() - viewport.getWidth();
				if (viewPosition.x > (rightestX - SCROLL_X)) {
					viewPosition.x = rightestX;
					scrollRight.setEnabled(false);
				}
				scrollLeft.setEnabled(true);
				viewport.setViewPosition(viewPosition);
			}
		};
		add(createButton(scrollRight), BorderLayout.EAST);
		setPreferredSize(new Dimension(Integer.MAX_VALUE, 28));

		addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent evt) {
				resetScrollButtons();
			}
		});
	}

	public JComponent createContentComponent() {
		return contentComponent;
	}

	protected void resetScrollButtons() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JViewport viewport = scrollPane.getViewport();
				int contentPanelWidth = contentComponent.getWidth();
				int viewportWidth = viewport.getWidth();
				if (contentPanelWidth <= viewportWidth) {
					scrollLeft.setEnabled(false);
					scrollRight.setEnabled(false);
				} else {
					viewport.setViewPosition(new Point(contentPanelWidth - viewportWidth, 0));
					scrollLeft.setEnabled(true);
					scrollRight.setEnabled(false);
				}
			}
		});
	}

	protected JButton createButton(Action a) {
		JButton b = new AbstractInputDialog.SmallButton(a);
		//	b.setBorder(rolloverBorder);
		b.setRolloverEnabled(true);
		return b;
	}
}
