/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 05.05.2003
 * Time: 17:28:29
 */
package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.delta.gui.misc.DeltaFileChooser;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.util.FileUtils;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileView;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Class CombinedURLFileChooserDialog ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class CombinedURLFileChooserDialog extends AbstractInputDialog {
	private JTabbedPane tabbedPane;
	private JFileChooser fileChooser;
	private JTextField jTextFieldURL;
	private File resultFile;
	private URL resultURL;

	public CombinedURLFileChooserDialog(Component parentComponent) {
		this(parentComponent, (String)null);
	}

	public CombinedURLFileChooserDialog(Component parentComponent, String okButtonText) {
        super(parentComponent, "Enter URL or choose local File ...", false, true, false, 477, 346, 350, 300, 0, 0, okButtonText);
        fileChooser.addActionListener(okAction);
	}
	public CombinedURLFileChooserDialog(Component parentComponent, FileFilter fileFilter) {
        this(parentComponent, (String) null);
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.setFileFilter(fileFilter);
	}
	public CombinedURLFileChooserDialog(Component parentComponent, String okButtonText, FileFilter fileFilter) {
		this(parentComponent, okButtonText);
		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.setFileFilter(fileFilter);
	}

	protected Component createMainComponent() {
		tabbedPane = new JTabbedPane();

		JPanel panelURL = new JPanel(new TableLayout(new double[][]{{40.0, TableLayout.FILL},{TableLayout.PREFERRED}}));
		panelURL.setBorder(GlobalObjects.EMPTY_BORDER_5);
		JLabel label = new JLabel("URL: ");
		jTextFieldURL = new JTextField();
		jTextFieldURL.getKeymap().addActionForKeyStroke(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), okAction);
		label.setDisplayedMnemonic(KeyEvent.VK_U);
		label.setLabelFor(jTextFieldURL);
		panelURL.add(label, new TableLayoutConstraints());
		panelURL.add(jTextFieldURL, new TableLayoutConstraints(1, 0));
		panelURL.addComponentListener(new ComponentAdapter() {
			public void componentShown(ComponentEvent e) {
				jTextFieldURL.requestFocusInWindow();
			}
		});

		fileChooser = new DeltaFileChooser(FileUtils.currentDirFile);
		fileChooser.setFileHidingEnabled(true);
		fileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		fileChooser.setControlButtonsAreShown(false);

		tabbedPane.insertTab("Local File", null, fileChooser, "Choose local File ...", 0);
		tabbedPane.insertTab("URL", null, panelURL, "Enter URL ...", 1);

		return tabbedPane;
	}

	protected boolean onOK() {
		if (tabbedPane.getSelectedIndex() == 0) {
			resultFile = fileChooser.getSelectedFile();
			resultURL = null;
			if (resultFile == null) return false;
			if (resultFile.isFile()) return true;
			fileChooser.setCurrentDirectory(resultFile);
			return false;
		}
		try {
			resultFile = null;
			resultURL = new URL(jTextFieldURL.getText());
            InputStream in = resultURL.openStream();
            if (in.read() == -1) {
                JOptionPane.showMessageDialog(this, "\"" + resultURL.toString() + "\" denotes an empty file!\nTry again ...", "I/O Error", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            in.close();
			return true;
		} catch (MalformedURLException exc) {
			JOptionPane.showMessageDialog(this, "\""+exc.getMessage()+"\"\nTry again ...", "Malformed URL", JOptionPane.ERROR_MESSAGE);
			return false;
		} catch (IOException exc) {
            JOptionPane.showMessageDialog(this, "\"" + exc.getMessage() + "\"\nTry again ...", "I/O Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }
	}

	protected boolean onCancel() {
		resultFile = null;
		resultURL = null;
		return super.onCancel();
	}

	public void setFileFilter(FileFilter filter) {
		fileChooser.setFileFilter(filter);
	}
	public void setFileView(FileView fileView) {
		fileChooser.setFileView(fileView);
	}
	public void setCurrentDirectory(File dir) {
		fileChooser.setCurrentDirectory(dir);
	}
	
	public void setOKButtonText(String okButtonText) {
		if ((okButtonText == null) || okButtonText.equals(""))
			okAction.putValue(Action.NAME, "OK");
		else
			okAction.putValue(Action.NAME, okButtonText);
	}

	public String getFileURLString() {
		if (resultFile != null)
			return resultFile.getAbsolutePath();
		return jTextFieldURL.getText();
	}

	public Object getFileURL() {
		if (resultFile != null) return resultFile;
		return resultURL;
	}
	public File getFile() {
		return resultFile;
	}
	public URL getURL() {
		return resultURL;
	}
	public boolean hasFileResult() {
		return (resultFile != null);
	}
	public boolean hasURLResult() {
		return (resultURL != null);
	}
}
