package at.ac.tuwien.asgaard.swing;

import javax.swing.*;
import java.util.*;

public final class ExtendedButtonGroup extends ButtonGroup {
	private final Map<ButtonModel, Integer> buttonModelMap;
	/**
	 * The current selection.
	 */
	private ButtonModel selectedButtonModel;

	public ExtendedButtonGroup() {
		selectedButtonModel = null;
		buttonModelMap = new HashMap<ButtonModel, Integer>();
	}

	public void setSelected(int nr) {
		setSelected(buttons.get(nr).getModel(), true);
	}

	public int getSelected() {
		return buttonModelMap.get(selectedButtonModel);
	}

	public void unSelect() {
		if (selectedButtonModel != null) {
			int selectedIndex = buttonModelMap.get(selectedButtonModel);
			selectedButtonModel = null;
			buttons.get(selectedIndex).setSelected(false);
		}
	}

	public void add(AbstractButton b) {
		if (b == null) return;
		buttons.addElement(b);

		if (b.isSelected())
			if (selectedButtonModel == null)
				selectedButtonModel = b.getModel();
			else
				b.setSelected(false);

		b.getModel().setGroup(this);
		buttonModelMap.put(b.getModel(), buttonModelMap.size());
	}

	public ButtonModel getSelection() {
		return selectedButtonModel;
	}

	public boolean isSelected(ButtonModel m) {
		return (m == selectedButtonModel);
	}

	public void remove(AbstractButton b) {
		if (b == null) return;
		buttons.removeElement(b);
		if (b.getModel() == selectedButtonModel)
			selectedButtonModel = null;
		b.getModel().setGroup(null);
	}

	public void setSelected(ButtonModel m, boolean b) {
		if (b && (m != null) && (m != selectedButtonModel)) {
			ButtonModel oldSelection = selectedButtonModel;
			selectedButtonModel = m;
			if (oldSelection != null)
				oldSelection.setSelected(false);
			selectedButtonModel.setSelected(true);
		}
	}

    public void removeAll() {
        buttons.removeAllElements();
        buttonModelMap.clear();
		selectedButtonModel = null;
    }
}
