/*
 * Copyright (c) 2004 Asgaard. All Rights Reserved.
 */
package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.swing.models.FixedListModel;
import at.ac.tuwien.asgaard.swing.models.SingleListSelectionModel;

import javax.swing.JList;
import javax.swing.ListCellRenderer;
import java.awt.event.*;
import java.util.Collection;


public final class ExtendedList extends JList {
	private FixedListModel listModel;

	public ExtendedList() {
		this(new SimpleListCellRenderer());
	}

	public ExtendedList(ListCellRenderer listCellRenderer) {
		super();
		listModel = new FixedListModel();
		setModel(listModel);
		setSelectionModel(new SingleListSelectionModel());
		setCellRenderer(listCellRenderer);
		addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent evt) {
				if (evt.getClickCount() == 2)
					SwingUtils.fireActionEvent(listenerList, ExtendedList.this);
			}
		});
		addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent evt) {
				if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
					SwingUtils.fireActionEvent(listenerList, ExtendedList.this);
					evt.consume();
				}
			}
		});
	}

	public void init(Collection data) {
		init(data.toArray());
	}

	public void init(Object[] data) {
		listModel.init(data);
		if (data.length > 0) {
			setSelectedIndex(0);
			ensureIndexIsVisible(0);
			repaint();  /** taken from JList: FIX-ME setSelectedIndex does not redraw all the time with the basic l&f**/
		} else
			clearSelection();
		requestFocusInWindow();
	}

	public void addActionListener(ActionListener listener) {
		listenerList.add(ActionListener.class, listener);
	}

	public void removeActionListener(ActionListener listener) {
		listenerList.remove(ActionListener.class, listener);
	}
}
