/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Oct 7, 2002
 * Time: 6:13:48 PM
 */
package at.ac.tuwien.asgaard.swing;

import java.awt.*;
import java.awt.event.*;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.event.*;

/**
 * Class PopupMenuHandler. ... todo
 * reacts on right mouse button
 *
 * @version 2.7.3
 * @author Peter Votruba
 */
public class PopupMenuHandler extends MouseAdapter {
	public static final	int	SHIFT_MASK	   = InputEvent.SHIFT_DOWN_MASK; 	 //SHIFT_MASK; //
	public static final	int	CTRL_MASK	   = InputEvent.CTRL_DOWN_MASK;		 //CTRL_MASK; //
	public static final	int	META_MASK	   = InputEvent.META_DOWN_MASK;		 //META_MASK; //
	public static final	int	ALT_MASK	   = InputEvent.ALT_DOWN_MASK; 		 //ALT_MASK; //
	public static final	int	ALT_GRAPH_MASK = InputEvent.ALT_GRAPH_DOWN_MASK; //ALT_GRAPH_MASK; //

	protected java.util.List<Wrapper> wrappers;
	protected JPopupMenu defaultPopupMenu;

	public PopupMenuHandler(JPopupMenu defaultPopupMenu) {
		wrappers = new LinkedList<Wrapper>();
		this.defaultPopupMenu = defaultPopupMenu;
		setPopupMenuInvoker(defaultPopupMenu);
	}

	public void addPopupMenu(final JPopupMenu popupMenu, int keyMask){
		 setPopupMenuInvoker(popupMenu);
		 wrappers.add(new Wrapper(popupMenu, keyMask));
	}

	private static void setPopupMenuInvoker(final JPopupMenu popupMenu) {
		if ((popupMenu != null) && (popupMenu.getInvoker() instanceof JMenu)) {
			final JMenu popupMenuInvoker = (JMenu) popupMenu.getInvoker();
			popupMenu.addPopupMenuListener(new PopupMenuListener() {
				public void popupMenuWillBecomeVisible(PopupMenuEvent evt) {
				}
				public void popupMenuWillBecomeInvisible(PopupMenuEvent evt) {
					popupMenu.setInvoker(popupMenuInvoker);
				}
				public void popupMenuCanceled(PopupMenuEvent evt) { }
			});
		}
	}

	public void mousePressed(MouseEvent evt) {
		maybeShowPopup(evt);
	}
	public void mouseReleased(MouseEvent evt) {
		maybeShowPopup(evt);
	}

	private void maybeShowPopup(MouseEvent evt) {
		if (evt.isPopupTrigger()) {
			for (Wrapper wrapper : wrappers)
				if (wrapper.popupMenu.isEnabled() && wrapper.isMatchingEvent(evt.getModifiersEx())) {
					showPopupMenu(evt, wrapper.popupMenu);
					return;
				}
			if (defaultPopupMenu.isEnabled())
				showPopupMenu(evt, defaultPopupMenu);
		}
	}

	private static void showPopupMenu(MouseEvent evt, JPopupMenu popupMenu) {
		Point p = getSuitableLocation(evt.getPoint(), popupMenu.getPreferredSize(), evt.getComponent());
		// if this popupMenu belongs to a JMenu, JPopupMenu#getInvoker() returns this JMenu
		if ((popupMenu.getInvoker() == null) || popupMenu.getInvoker().isEnabled())
			popupMenu.show(evt.getComponent(), p.x, p.y);
	}

	// Get a suitable location for a JPopupMenu to make it lightweight
	public static Point getSuitableLocation(Point point, Dimension d, Component c) {
		if (c instanceof JButton) {
			point.x = 0;
			point.y = c.getHeight();
		}
		// First locate the parent JFrame or JDialog
		Component topLevel = c;
		while (topLevel != null && !(topLevel instanceof JFrame) && !(topLevel instanceof JDialog)) {
			topLevel = topLevel.getParent();
		}

		//Dimension scrnSize = Toolkit.getDefaultToolkit().getScreenSize();
		Rectangle scrnSize = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();

		int newX;  // New proposed x co-ordinate
        int newY;  // New proposed y co-ordinate
        // We have a top-level parent
        Rectangle parentBounds = topLevel.getBounds();  // Parent bounds, screen-relative

        // Get proposed location, relative to the screen
        SwingUtilities.convertPointToScreen(point, c);

        int maxX = Math.min(parentBounds.x + parentBounds.width, scrnSize.width);
        int maxY = Math.min(parentBounds.y + parentBounds.height, scrnSize.height);

        if ((point.x + d.width) > maxX) {
            // Popup overhangs to the right
            newX = maxX - d.width;
        } else
            newX = point.x;

        if ((point.y + d.height) > maxY) {
            // Popup ends below frame
            newY = maxY - d.height;
        } else
            newY = point.y;

        // Change location only if necessary AND if
        // we can make the popupMenu fit inside the frame
        if ((newX >= 0) && (newY >= 0)) {
            point.x = newX;
            point.y = newY;
        }

        // Convert back to relative co-ordinates
        SwingUtilities.convertPointFromScreen(point, c);

	// if (yChanged && !xChanged) point.x = c.getWidth(); // ???

		return point;
	}

	private static class Wrapper {
		JPopupMenu popupMenu;
		private int keyMask;

		Wrapper(JPopupMenu popupMenu, int keyMask){
			this.keyMask = keyMask;
			this.popupMenu = popupMenu;
		}

		boolean isMatchingEvent(int modifiers) {
			return modifiers == keyMask; // only this key(s) have to be pressed, no other
		}
	}
}
