package at.ac.tuwien.asgaard.swing;

import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Vector;

import javax.swing.*;

public final class StatusBar extends JLabel {
	public static final	String TOOL_TIP			  =	Action.SHORT_DESCRIPTION;
	public static final	String DESCRIPTION_HOVER  =	"DescriptionHover";
	public static final	String DESCRIPTION_ACTIVE =	Action.LONG_DESCRIPTION;

	private static final String defaultMessage   = ""; // "Ready.";

	private final List<String> messages;

	public StatusBar() {
		messages = new Vector<String>(); // do not use ArrayList() because this need to be thread-safe
	//	setBorder(BorderFactory.createLoweredBevelBorder());
		setText(" " + defaultMessage);
	}

	public void setMessage(String message) {
//  Log.trace("message = " + message);
		setText(" " + (message != null ? message : defaultMessage));
	// paintImmediately(0, 0, getWidth()-1, getHeight()-1);
		revalidate();
		repaint();
	}
	public void clear() {
		setMessage(null);
	}
    public void clearAll() {
        messages.clear();
        setMessage(null);
    }
	public void popMessage() {
        String message = null;
		int length = messages.size();
        if (length > 0) {
            messages.remove(length - 1);
            if (length > 1)
                message = messages.get(length - 2);
        }
        setMessage(message);
	}

	public void pushMessage(String message) {
		if (message != null)
			messages.add(message);
		setMessage(message);
	}

	public void popMessage(String message) {
		if (message != null)
			if (messages.remove(message)) {
				message = null;
				int length = messages.size();
				if (length != 0)
					message = messages.get(length - 1);
				setMessage(message);
			}
	}

	public void setActiveMessageFrom(Action action) {
		popMessage((String)action.getValue(DESCRIPTION_HOVER));
		pushMessage((String)action.getValue(DESCRIPTION_ACTIVE));
	}
	public void unsetActiveMessageFrom(Action action) {
		popMessage((String)action.getValue(DESCRIPTION_ACTIVE));
	}

	public void setHoverMessageFrom(Action action) {
		pushMessage((String)action.getValue(DESCRIPTION_HOVER));
	}
	public void unsetHoverMessageFrom(Action action) {
		popMessage((String)action.getValue(DESCRIPTION_HOVER));
	}

	public void attachToAbstractButton(AbstractButton button) {
		final Action action = button.getAction();
		if (action != null) {
	//  button.removeMouseListener(this);
			button.addMouseListener(new MouseAdapter() {
                public void mousePressed(MouseEvent evt) {
                    unsetHoverMessageFrom(action);
                }
                public void mouseEntered(MouseEvent e) {
                    setHoverMessageFrom(action);
                }
                public void mouseExited(MouseEvent e) {
                    unsetHoverMessageFrom(action);
                }
            });
			String msg = (String)action.getValue(DESCRIPTION_HOVER);
            if (msg != null)
				messages.remove(msg);
		}
	}

	public void attachToMenuBar(JMenuBar menuBar) {
		int count = menuBar.getMenuCount();
		for (int i = 0; i < count; i++) {
			JMenu menu = menuBar.getMenu(i);
			if (menu != null) attachToMenuElement(menu);
		}
	}

	public void attachToMenuElement(MenuElement menuElement) {
		Component component = menuElement.getComponent();
		if (component instanceof AbstractButton)
			attachToAbstractButton((AbstractButton) component);
		MenuElement[] elements = menuElement.getSubElements();
		for (int i = 0; i < elements.length; i++) {
			MenuElement element = elements[i];
			if ((element instanceof JMenu) || (element instanceof JPopupMenu))
				attachToMenuElement(element);
			else if (element instanceof AbstractButton)
				attachToAbstractButton((AbstractButton)element);
		}
	}

	public void attachToToolBar(Container cont) {
		Component[] components = cont.getComponents();
		for (int i = 0; i < components.length; i++) {
			Component comp = components[i];
			if (comp instanceof AbstractButton)
				attachToAbstractButton((AbstractButton)comp);
			else if ((comp instanceof Container) &&
			        ((Container)comp).getComponentCount() > 0)
				attachToToolBar((Container)comp);
		}
	}

	public void mouseClicked(MouseEvent evt) {}
}
