/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 24.04.2003
 * Time: 14:43:40
 */
package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.swing.plaf.LocalAcceleratorMenuItemUI;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.io.BufferedReader;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.undo.*;

/**
 * Class SwingUtils ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class SwingUtils {
	private SwingUtils() {  } // should not be instantiated

	public static void addShortCut(JComponent comp, Action a) {
		addShortCut(comp, a, JComponent.WHEN_FOCUSED);
	}
	public static void addShortCut(JComponent comp, Action a, KeyStroke ks) {
		addShortCut(comp, a, JComponent.WHEN_FOCUSED, ks);
	}

	public static void addShortCut(JComponent comp, Action a, int condition) {
		addShortCut(comp.getActionMap(), comp.getInputMap(condition), a);
	}
	public static void addShortCut(JComponent comp, Action a, int condition, KeyStroke ks) {
		addShortCut(comp.getActionMap(), comp.getInputMap(condition), a, ks);
	}

  public static void addShortCut(ActionMap actionMap, InputMap inputMap, Action action) {
		addShortCut(actionMap, inputMap, action, (KeyStroke) action.getValue(LocalAcceleratorMenuItemUI.LOCAL_ACCELERATOR_KEY));
	}
	public static void addShortCut(ActionMap actionMap, InputMap inputMap, Action action, KeyStroke ks) {
		Object name = action.getValue(Action.NAME);
        if (name == null) name = action.getValue(Action.SHORT_DESCRIPTION);
        if (name == null) name = action;
		actionMap.put(name, action);
		inputMap.put(ks, name);
	}

	public static String getClipBoardContentAsString() {
		Transferable contents = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
		DataFlavor dataFlavor = DataFlavor.selectBestTextFlavor(contents.getTransferDataFlavors());
		try {
			BufferedReader bufferedReader = new BufferedReader(dataFlavor.getReaderForText(contents));
			StringBuffer result = new StringBuffer();

			String s = bufferedReader.readLine();
			while (s != null) {
				result.append(s);
				s = bufferedReader.readLine();
			}
			return result.toString();
		} catch (Exception exc) {
			return null;
		}
	}

	/**
	 * Converts a type Color to a hex string
	 * in the format "#RRGGBB"
	 */
	public static String colorToHex(Color color) { // taken from javax.swing.text.html.CSS (because its not public there)
		String colorstr = new String("#");

		// Red
		String str = Integer.toHexString(color.getRed());
		if (str.length() > 2)
			colorstr +=  str.substring(0, 2);
		else if (str.length() < 2)
			colorstr += "0" + str;
		else
			colorstr += str;

		// Green
		str = Integer.toHexString(color.getGreen());
		if (str.length() > 2)
			colorstr +=  str.substring(0, 2);
		else if (str.length() < 2)
			colorstr += "0" + str;
		else
			colorstr += str;

		// Blue
		str = Integer.toHexString(color.getBlue());
		if (str.length() > 2)
			colorstr +=  str.substring(0, 2);
		else if (str.length() < 2)
			colorstr += "0" + str;
		else
			colorstr += str;
		return colorstr;
	}
	/**
	 * Convert a "#FFFFFF" hex string to a Color.
	 * If the color specification is bad, an attempt
	 * will be made to fix it up.
	 */
	public static final Color hexToColor(String value) {
		String digits;
		int n = value.length();
		if (n == 0) return null;
		if (value.startsWith("#"))
			digits = value.substring(1, Math.min(n, 7));
		else
			digits = value;
		String hstr = "0x" + digits;
		Color c;
		try {
			c = Color.decode(hstr);
		} catch (NumberFormatException nfe) {
			c = null;
		}
		return c;
	}
	/**
	 * Convert a color string such as "RED" or "#NNNNNN" or "rgb(r, g, b)"
	 * to a Color.
	 */
	public static Color stringToColor(String str) {
		Color color;
		if (str.length() == 0)
			color = Color.black;
//		else if (str.startsWith("rgb(")) {
//			color = parseRGB(str); }
		else if (str.charAt(0) == '#')
			color = hexToColor(str);
		else if (str.equalsIgnoreCase("Black"))
			color = hexToColor("#000000");
		else if (str.equalsIgnoreCase("Silver"))
			color = hexToColor("#C0C0C0");
		else if (str.equalsIgnoreCase("Gray"))
			color = hexToColor("#808080");
		else if (str.equalsIgnoreCase("White"))
			color = hexToColor("#FFFFFF");
		else if (str.equalsIgnoreCase("Maroon"))
			color = hexToColor("#800000");
		else if (str.equalsIgnoreCase("Red"))
			color = hexToColor("#FF0000");
		else if (str.equalsIgnoreCase("Purple"))
			color = hexToColor("#800080");
		else if (str.equalsIgnoreCase("Fuchsia"))
			color = hexToColor("#FF00FF");
		else if (str.equalsIgnoreCase("Green"))
			color = hexToColor("#008000");
		else if (str.equalsIgnoreCase("Lime"))
			color = hexToColor("#00FF00");
		else if (str.equalsIgnoreCase("Olive"))
			color = hexToColor("#808000");
		else if (str.equalsIgnoreCase("Yellow"))
			color = hexToColor("#FFFF00");
		else if (str.equalsIgnoreCase("Navy"))
			color = hexToColor("#000080");
		else if (str.equalsIgnoreCase("Blue"))
			color = hexToColor("#0000FF");
		else if (str.equalsIgnoreCase("Teal"))
			color = hexToColor("#008080");
		else if (str.equalsIgnoreCase("Aqua"))
			color = hexToColor("#00FFFF");
		else
			color = hexToColor(str); // sometimes get specified without leading #
		return color;
	}

	public static void switchToCard(JPanel panel, String cardName) {
		((CardLayout) panel.getLayout()).show(panel, cardName);
	}


	// taken from "The Java Developers Almanac 1.4", "e950. Packing a Column of a JTable Component", http://javaalmanac.com/egs/javax.swing.table/PackCol.html
	// Sets the preferred width of the visible column specified by vColIndex. The column
	// will be just wide enough to show the column head and the widest cell in the column.
	// margin pixels are added to the left and right
	// (resulting in an additional width of 2*margin pixels).
	public static void packTableColumn(JTable table, int vColIndex, int margin) {
		int rowCount = table.getRowCount();
	//	if (rowCount == 0) return;

		TableColumn column = table.getColumnModel().getColumn(vColIndex);

		// Get width of column header
		TableCellRenderer renderer = column.getHeaderRenderer();
		if (renderer == null)
			renderer = table.getTableHeader().getDefaultRenderer();

		int width;
		Component comp = renderer.getTableCellRendererComponent(table, column.getHeaderValue(), false, false, 0, 0);
		width = comp.getPreferredSize().width;

		// Get maximum width of column data
		for (int r = 0; r < rowCount; r++) {
			renderer = table.getCellRenderer(r, vColIndex);
			comp = renderer.getTableCellRendererComponent(table, table.getValueAt(r, vColIndex), false, false, r, vColIndex);
			width = Math.max(width, comp.getPreferredSize().width);
		}

		// Add margin
		width += 2 * margin;

		// Set the width
		column.setMinWidth(width);
		column.setMaxWidth(width);
		column.setPreferredWidth(width);
		column.setWidth(width);
	}
	public static void resetTableColumnWidths(JTable table, int columns) {
		TableColumnModel columnModel = table.getColumnModel();
		int columnCount = columnModel.getColumnCount();
		if (columns == 0) columns = columnCount;
		else columns = Math.min(columns, columnCount);
		int totalColumnWidth = columnModel.getTotalColumnWidth();
		int newColumnWidth = totalColumnWidth / columnCount;
		for (int i=0; i< columns; i++) {
			TableColumn column = columnModel.getColumn(i);
			// Set the width
			column.setMinWidth(newColumnWidth);
			column.setMaxWidth(newColumnWidth);
			column.setPreferredWidth(newColumnWidth);
			column.setWidth(newColumnWidth);
		}
	}

	/**
	 * Returns true if the mouse event specifies only the middle mouse button.
	 *
	 * @param evt a MouseEvent object
	 * @return true if the middle mouse button was active
	 */
	public static boolean isMiddleMouseButton(MouseEvent evt) {
		return ((evt.getModifiersEx() & MouseEvent.BUTTON2_DOWN_MASK) == MouseEvent.BUTTON2_DOWN_MASK);
	}
	/**
	 * Returns true if the mouse event specifies only the right mouse button.
	 *
	 * @param evt a MouseEvent object
	 * @return true if the right mouse button was active
	 */
	public static boolean isRightMouseButton(MouseEvent evt) {
		return ((evt.getModifiersEx() & MouseEvent.BUTTON3_DOWN_MASK) == MouseEvent.BUTTON3_DOWN_MASK);
	}

    /** Notify all listeners that have registered interest for
     notification on this event type. */
    public static void fireActionEvent(EventListenerList listenerList, Object source) {
        fireActionEvent(listenerList, new ActionEvent(source, ActionEvent.ACTION_PERFORMED, null));
    }
    public static void fireActionEvent(EventListenerList listenerList, ActionEvent evt) {
        Object[] listeners = listenerList.getListenerList();
		for (int i = listeners.length - 2; i >= 0; i -= 2)
			if (listeners[i] == ActionListener.class)
				((ActionListener) listeners[i + 1]).actionPerformed(evt);
	}

	public static void fireActionEvent(ActionListener[] listeners, ActionEvent evt) {
		for (int i = listeners.length - 1; i >= 0; i--)
			listeners[i].actionPerformed(evt);
	}

	/** Notify all listeners that have registered interest for
     notification on this event type. */
    public static void fireChangeEvent(EventListenerList listenerList, Object source) {
        Object[] listeners = listenerList.getListenerList();
	    if (listeners.length > 0) {
		    ChangeEvent evt = new ChangeEvent(source);
			for (int i = listeners.length - 2; i >= 0; i -= 2)
				if (listeners[i] == ChangeListener.class)
					((ChangeListener) listeners[i + 1]).stateChanged(evt);
		}
    }


	public static void fireUndoableEditHappened(UndoableEdit undoableEdit, Object source, EventListenerList listenerList) {
		UndoableEditEvent evt = new UndoableEditEvent(source, undoableEdit);
		fireUndoableEditHappened(evt, listenerList);
	}

	public static void fireUndoableEditHappened(UndoableEditEvent evt, EventListenerList listenerList) {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == UndoableEditListener.class)
				((UndoableEditListener) listeners[i + 1]).undoableEditHappened(evt);
		}
	}

	public static Color changeColor(Color c, int delta) {
		if (delta == 0) return c;
		return changeColor(c, delta, delta, delta);
	}
	public static Color changeColor(Color c, int deltaRed, int deltaGreen, int deltaBlue) {
		int red = Math.max(0, Math.min(255, c.getRed() + deltaRed));
		int green = Math.max(0, Math.min(255, c.getGreen() + deltaGreen));
		int blue = Math.max(0, Math.min(255, c.getBlue() + deltaBlue));
		return new Color(red, green, blue, c.getAlpha());
	}
}
