package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.swing.models.*;

import javax.swing.*;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import javax.swing.event.*;
import java.awt.*;

public final class SystemColorChooserPanel extends AbstractColorChooserPanel implements ListSelectionListener {
	private Color currentColor;
	private JList list;
	private static final String[] labels = {
		"<Custom>",
		"black",
		"blue",
		"cyan",
		"darkGray",
		"gray",
		"green",
		"lightGray",
		"magenta",
		"orange",
		"pink",
		"red",
		"white",
		"yellow",
		"activeCaption",
		"activeCaptionBorder",
		"activeCaptionText",
		"control",
		"controlDkShadow",
		"controlHighlight",
		"controlLtHighlight",
		"controlShadow",
		"controlText",
		"desktop",
		"inactiveCaption",
		"inactiveCaptionBorder",
		"inactiveCaptionText",
		"info",
		"infoText",
		"menu",
		"menuText",
		"scrollbar",
		"text",
		"textHighlight",
		"textHighlightText",
		"textInactiveText",
		"textText",
		"window",
		"windowBorder",
		"windowText"};

	private static final Color[] colors = {null,
	                                 Color.black,
	                                 Color.blue,
	                                 Color.cyan,
	                                 Color.darkGray,
	                                 Color.gray,
	                                 Color.green,
	                                 Color.lightGray,
	                                 Color.magenta,
	                                 Color.orange,
	                                 Color.pink,
	                                 Color.red,
	                                 Color.white,
	                                 Color.yellow,
	                                 SystemColor.activeCaption,
	                                 SystemColor.activeCaptionBorder,
	                                 SystemColor.activeCaptionText,
	                                 SystemColor.control,
	                                 SystemColor.controlDkShadow,
	                                 SystemColor.controlHighlight,
	                                 SystemColor.controlLtHighlight,
	                                 SystemColor.controlShadow,
	                                 SystemColor.controlText,
	                                 SystemColor.desktop,
	                                 SystemColor.inactiveCaption,
	                                 SystemColor.inactiveCaptionBorder,
	                                 SystemColor.inactiveCaptionText,
	                                 SystemColor.info,
	                                 SystemColor.infoText,
	                                 SystemColor.menu,
	                                 SystemColor.menuText,
	                                 SystemColor.scrollbar,
	                                 SystemColor.text,
	                                 SystemColor.textHighlight,
	                                 SystemColor.textHighlightText,
	                                 SystemColor.textInactiveText,
	                                 SystemColor.textText,
	                                 SystemColor.window,
	                                 SystemColor.windowBorder,
	                                 SystemColor.windowText};

	// Change combo box to match color, if possible
	private void setColor(Color newColor) {
		if (newColor.equals(currentColor)) return;
		int position = findColorPosition(newColor);
		list.setSelectedIndex(position);
	}

	// Given a color, find the position whose color matches
	// This could result in a position different from original if two are equal
	// Since color is same, this is considered to be okay
	private static int findColorPosition(Color color) {
		int position = 0; //colors.length - 1;
		// Cannot use equals() to compare Color and SystemColor
		int colorRGB = color.getRGB();
		for (int i = 0,n = colors.length; i < n; i++)
			if ((colors[i] != null) && (colorRGB == colors[i].getRGB())) {
				position = i;
				break;
			}
		return position;
	}

	public void valueChanged(ListSelectionEvent evt) {
		if (!((SingleListSelectionModel)list.getSelectionModel()).selectionChanged()) return;
		int position = list.getSelectedIndex();
		if (position != -1) {
			currentColor = colors[position];
			getColorSelectionModel().setSelectedColor(currentColor);
		}
	}

	public String getDisplayName() {
		return "System Colors";
	}

	public Icon getSmallDisplayIcon() {
		return null;
	}

	public Icon getLargeDisplayIcon() {
		return null;
	}

	protected void buildChooser() {
		list = new JList(labels);
		list.setSelectionModel(new SingleListSelectionModel());
		list.addListSelectionListener(this);
		list.setBackground(Color.lightGray);
		list.setCellRenderer(new ColorRenderer());
		add(new JScrollPane(list));
	}

	public void updateChooser() {
		setColor(getColorFromModel());
	}

	private static final class ColorRenderer extends DefaultListCellRenderer {
		private final ColorFillIcon icon = new ColorFillIcon();
		public Component getListCellRendererComponent(JList list,
		                                              Object value, // value to display
		                                              int index, // cell index
		                                              boolean isSelected, // is the cell selected
		                                              boolean cellHasFocus) {    // the list and the cell have the focus
			super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
			Color c = colors[index];
			if (c != null) {
				icon.setFillColor(c);
				setIcon(icon);
			}
			return this;
		}
	}
	private static final class ColorFillIcon implements Icon {
		ColorFillIcon(Color fill, int width, int height) {
			this.fillColor = fill;
			this.borderSize = BORDER_SIZE;
			this.width = width;
			this.height = height;
			this.shadow = Color.black;
			this.fillWidth = width - 2 * borderSize;
			this.fillHeight = height - 2 * borderSize;
		}

		ColorFillIcon(Color fill, int size) {
			this(fill, size, size);
		}

		ColorFillIcon(Color fill) {
			this(fill, DEFAULT_SIZE, DEFAULT_SIZE);
		}
		ColorFillIcon() {
			this(Color.white, DEFAULT_SIZE, DEFAULT_SIZE);
		}
		// Set non-default shadow color
		public void setShadow(Color c) {
			shadow = c;
		}

		// Change the main color
		public void setFillColor(Color c) {
			fillColor = c;
		}

		// The Icon interface
		public int getIconWidth() {
			return width;
		}

		public int getIconHeight() {
			return height;
		}

		public void paintIcon(Component comp, Graphics g, int x, int y) {
			Color c = g.getColor();

			// Draw the border
			if (borderSize > 0) {
				g.setColor(shadow);
				for (int i = 0; i < borderSize; i++) {
					g.drawRect(x + i, y + i, width - 2 * i - 1, height - 2 * i - 1);
				}
			}

			// Fill the remainder of the icon
			g.setColor(fillColor);
			g.fillRect(x + borderSize, y + borderSize, fillWidth, fillHeight);

			g.setColor(c);

		}

		// Icon state
		final int width;  // Color fill width
		final int height;  // Color fill height
		Color fillColor; // Color to fill with
		Color shadow;  // Shadow color
		final int borderSize; // Border size in pixels
		final int fillHeight; // Height of area to fill
		final int fillWidth; // Width of area to fill

		// GUIConstants
		private static final int BORDER_SIZE = 1;
		private static final int DEFAULT_SIZE = 16;
	}
}


