package at.ac.tuwien.asgaard.swing;

import at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode;
import at.ac.tuwien.asgaard.delta.gui.xml.XMLView;
import at.ac.tuwien.asgaard.delta.util.GlobalObjects;
import at.ac.tuwien.asgaard.swing.models.SimpleMutableTreeNode;
import at.ac.tuwien.asgaard.util.MiscUtils;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

public class TreePathBar extends ButtonScrollPanel {
    private TreePath treePath;
	private JToolBar contentPanel; // JToolBar

	public TreePathBar() {
        super(null);
    }

	public JComponent createContentComponent() {
		contentPanel = new JToolBar(); //new JPanel(new SingleFiledLayout(SingleFiledLayout.ROW, SingleFiledLayout.CENTER, 5)); //
		contentPanel.setMargin(GlobalObjects.INSETS_ZERO);
		contentPanel.setBorder(GlobalObjects.EMPTY_BORDER_0);
		contentPanel.setFloatable(false);
		contentPanel.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
		setToolTipText("Treepath Bar ...");
		contentPanel.setToolTipText("Treepath Bar ...");
		return contentPanel;
	}

	public TreePath getTreePath() {
        return treePath;
    }

	private static final Font SLASH_FONT = GlobalObjects.BOLD_FIXED_FONT.deriveFont(GlobalObjects.BOLD_FIXED_FONT.getSize2D()+(float)2.0);
    public void setTreePath(TreePath treePath) {
		if (treePath == null) return;

		this.treePath = treePath;
		contentPanel.removeAll();

        Object[] paths = treePath.getPath();
        for (int i = 0; i < paths.length-1; i++) {
			JLabel jLabel = new JLabel("/");
			jLabel.setFont(SLASH_FONT);
			contentPanel.add(jLabel);
            Component button = contentPanel.add(new TreePathBarButton(new TreePathBarButtonAction(paths[i], i, paths.length)));
			button.addMouseListener(new PopupMenuHandler(createPopupMenu((SimpleMutableTreeNode) paths[i], i)));
        }

		contentPanel.revalidate();
		contentPanel.repaint();
		revalidate();
		repaint();

		resetScrollButtons();
    }

    private JPopupMenu createPopupMenu(SimpleMutableTreeNode currentNode, final int index) {
        final JPopupMenu currentMenu = new JPopupMenu();
		for (int i = 0, count = currentNode.getChildCount(); i < count; i++) {
            final XMLTreeNode node = (XMLTreeNode) currentNode.getChildAt(i);
			JMenuItem menuItem;
            if (node.getChildCount() == 0)
				menuItem = currentMenu.add(new PopupMenuItemAction(node));
            else {
				final JMenu menu = new JMenu(new PopupMenuItemAction(node));
				currentMenu.add(menu);
				menu.addMouseListener(new MouseAdapter() {
					public void mouseClicked(MouseEvent evt) {
						currentMenu.setVisible(false);
						menu.getAction().actionPerformed(null);
					}
				});
				menu.addChangeListener(new ChangeListener() {
					public void stateChanged(ChangeEvent e) {
						if ((node.getChildCount() > 0) && (menu.getMenuComponentCount() == 0))
							fillPopupMenu(node, menu, index+1);
					}
				});
				menuItem = menu;
			}
			if (index+1 >= treePath.getPathCount())
				menuItem.setFont(GlobalObjects.PLAIN_FIXED_FONT);
			else
				menuItem.setFont((node == treePath.getPathComponent(index + 1)) ? GlobalObjects.BOLD_FIXED_FONT : GlobalObjects.PLAIN_FIXED_FONT);
        }
        return currentMenu;
    }

    private void fillPopupMenu(SimpleMutableTreeNode currentNode, final JMenu currentMenu, final int index) {
        for (int i=0,count= currentNode.getChildCount(); i<count;i++) {
            final XMLTreeNode node = (XMLTreeNode) currentNode.getChildAt(i);
			JMenuItem menuItem;
			if (node.getChildCount() == 0)
				menuItem = currentMenu.add(new PopupMenuItemAction(node));
			else {
				final JMenu menu = new JMenu(new PopupMenuItemAction(node));
				currentMenu.add(menu);
				menu.addMouseListener(new MouseAdapter() {
					public void mouseClicked(MouseEvent evt) {
						currentMenu.setVisible(false);
						menu.getAction().actionPerformed(null);
					}
				});
				menu.addChangeListener(new ChangeListener() {
					public void stateChanged(ChangeEvent e) {
						if ((node.getChildCount() > 0) && (menu.getMenuComponentCount() == 0))
							fillPopupMenu(node, menu, index+1);
					}
				});
				menuItem = menu;
			}
			if (index + 1 >= treePath.getPathCount())
				menuItem.setFont(GlobalObjects.PLAIN_FIXED_FONT);
			else
				menuItem.setFont((node == treePath.getPathComponent(index + 1)) ? GlobalObjects.BOLD_FIXED_FONT :
								 GlobalObjects.PLAIN_FIXED_FONT);
        }
    }

    public void addActionListener(ActionListener listener) {
        listenerList.add(ActionListener.class, listener);
    }


	private class TreePathBarButtonAction extends AbstractAction {
        private final int index, pathCount;

        public TreePathBarButtonAction(Object node, int index, int pathCount) {
            super(node.toString());
            this.index = index;
            this.pathCount = pathCount;
			putValue(Action.SHORT_DESCRIPTION, "<html>Jumps to treenode <i>" + node.toString() + "</i><br>... use right mouse-button to show list of children");
        }
        public void actionPerformed(ActionEvent evt) {
            for (int j = pathCount; j > index+1; j--)
                treePath = treePath.getParentPath();
            SwingUtils.fireActionEvent(listenerList, TreePathBar.this);
        }
    }

    private class PopupMenuItemAction extends AbstractAction {
        private final SimpleMutableTreeNode node;
        public PopupMenuItemAction(XMLTreeNode node) {
            super(MiscUtils.truncateSring(node.toString(), 20, 5));
            this.node = node;
            putValue(Action.SHORT_DESCRIPTION, //"Jumps to treenode \"" + node.toString() + "\"");
					 XMLView.getToolTipTextForTreeNode(node, true, true));
        }
        public void actionPerformed(ActionEvent evt) {
            treePath = node.getTreePath();
            SwingUtils.fireActionEvent(listenerList, TreePathBar.this);
        }
    }

	private static class TreePathBarButton extends JButton {
		public TreePathBarButton(Action a) {
			super(a);
			setMargin(GlobalObjects.INSETS_ZERO);
			setFocusable(false);
			setRolloverEnabled(true);
		}
		public Dimension getPreferredSize() {
			return new Dimension(super.getPreferredSize().width, 21);
		}
	}
}
