package at.ac.tuwien.asgaard.swing.autoComplete;

import at.ac.tuwien.asgaard.delta.gui.images.MainImages;
import at.ac.tuwien.asgaard.swing.AbstractInputDialog;
import at.ac.tuwien.asgaard.swing.TextEditDialog;
import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstraints;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.Collection;
import java.util.List;

/**
 * Class StringEditDialog. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 */
public class AutoCompleteComboBoxEditDialog extends AbstractInputDialog {
	private AutoCompleteComboBox comboBox;
	private Action editText;
	private JLabel jLabel;

	public AutoCompleteComboBoxEditDialog(Component parentComponent, String title, String label, Collection data) {
		super(parentComponent, title, false, false, 400, 125, 250, 120, 0, 125);
        comboBox = new AutoCompleteComboBox(data);
		init(label);
	}
	public AutoCompleteComboBoxEditDialog(Component parentComponent, String title, String label, Object[] data) {
		super(parentComponent, title, false, false, 400, 125, 250, 120, 0, 125);
		comboBox = new AutoCompleteComboBox(data);
		init(label);
	}

	private void init(String label) {
		comboBox.registerKeyboardAction(okAction, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JComponent.WHEN_FOCUSED);

		JPanel panel = new JPanel(new TableLayout(new double[][]{{TableLayout.FILL, TableLayout.PREFERRED}, {24.0, 24.0}}, 2, 0));
		jLabel = new JLabel(label);
		panel.add(jLabel, new TableLayoutConstraints(0,0, 1,0));
		panel.add(comboBox,  new TableLayoutConstraints(0, 1));
		panel.add(new SmallButton(editText), new TableLayoutConstraints(1, 1));
		contentPane.add(panel, BorderLayout.CENTER);
	}

	protected boolean onOK() {
		if (comboBox.getSelectedItem() == null) return false;
        comboBox.storeInput();
        return true;
    }

	protected void createActions() {
		super.createActions();
		editText = new AbstractAction("", MainImages.EDIT_ICON) {
			public void actionPerformed(ActionEvent evt) {
				editDialog = getTextEditDialog(AutoCompleteComboBoxEditDialog.this, AutoCompleteComboBoxEditDialog.this.getTitle());
				String newValue = editDialog.showEditDialog(comboBox.getText());
				if (newValue != null) 
					comboBox.setSelectedItem(newValue);
				comboBox.requestFocus();
			}
		};
	}

	public String showEditDialog(String initialValue) {
        if (initialValue != null)
        	comboBox.setText(initialValue);
		comboBox.requestFocusInWindow();
		return (showDialog() ? comboBox.getSelectedItem().toString().trim() : null);
	}
	public String showEditDialog(String title, String label, String initialValue) {
		setTitle(title);
		jLabel.setText(label);
		return showEditDialog(initialValue);
	}

	public static String showEditDialog(String title, String label, Object[] items, String initialValue, boolean restrictValues) {
		AutoCompleteComboBoxEditDialog dialog = new AutoCompleteComboBoxEditDialog(null, title, label, items);
		dialog.comboBox.setRestrictValues(restrictValues);
		return dialog.showEditDialog(initialValue);
	}
    public List getElements() {
        return comboBox.getElements();
    }

	private TextEditDialog editDialog;
	protected TextEditDialog getTextEditDialog(Component parentComponent, String title) {
		if (editDialog == null)
			editDialog = new TextEditDialog(parentComponent, title);
		return editDialog;
	}

	public void setAutoAddValue(boolean autoAddValue) {
		comboBox.setAutoAddValue(autoAddValue);
	}

	public void setIgnoreCase(boolean ignoreCase) {
		comboBox.setIgnoreCase(ignoreCase);
	}

	public void setMaximumMatch(boolean maximumMatch) {
		comboBox.setMaximumMatch(maximumMatch);
	}

	public void setRestrictValues(boolean restrictValues) {
		comboBox.setRestrictValues(restrictValues);
	}

	public void setShowPopupOnAnyKey(boolean showPopupOnAnyKey) {
		comboBox.setShowPopupOnAnyKey(showPopupOnAnyKey);
	}

	public boolean isShowPopupOnAnyKey() {
		return comboBox.isShowPopupOnAnyKey();
	}

	public boolean isRestrictValues() {
		return comboBox.isRestrictValues();
	}

	public boolean isMaximumMatch() {
		return comboBox.isMaximumMatch();
	}

	public boolean isIgnoreCase() {
		return comboBox.isIgnoreCase();
	}

	public boolean isAutoAddValue() {
		return comboBox.isAutoAddValue();
	}
}
