/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 19, 2002
 * Time: 2:17:37 AM
 */
package at.ac.tuwien.asgaard.swing.models;

import at.ac.tuwien.asgaard.util.MiscUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.*;

/**
 * Class SimpleComboBoxModel. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
public class SimpleComboBoxModel extends AbstractListModel implements MutableComboBoxModel {
	private List objects;
	private Object selectedObject;

	/**
	 * Constructs an empty SimpleComboBoxModel object.
	 */
	public SimpleComboBoxModel() {
		objects = new ArrayList();
	}

	public SimpleComboBoxModel(Object[] items) {
		this(items, true);
	}

	/**
	 * Constructs a SimpleComboBoxModel object initialized with
	 * an array of objects.
	 *
	 * @param items  an array of Object objects
	 */
	public SimpleComboBoxModel(Object[] items, boolean selectFirstItem) {
		if (items == null)
			objects = new ArrayList();
		else {
			objects = new ArrayList(items.length);
			for (int i = 0, c = items.length; i < c; i++)
				objects.add(items[i]);
			if (selectFirstItem && (items.length > 0))
				selectedObject = objects.get(0);
		}
	}

	/**
	 * Constructs a SimpleComboBoxModel object initialized with
	 * a vector.
	 *
	 * @param items  a List object ...
	 */
    public SimpleComboBoxModel(Collection items) {
        this(items, true);
    }
	public SimpleComboBoxModel(Collection items, boolean selectFirstItem) {
		if (items == null)
			objects = new ArrayList();
//        else if (items instanceof List)
//			objects = (List)items;
        else
			objects = new ArrayList(items);
		if (selectFirstItem && !objects.isEmpty())
			selectedObject = objects.get(0);
	}
    public void init(Collection items) {
        init(items, true);
    }
    public void init(Collection items, boolean selectFirstItem) {
	    if (items == null)
			objects = new ArrayList();
//        else if (items instanceof List)
//			objects = (List)items;
        else
			objects = new ArrayList(items);
		if (objects.size() > 0) {
		    selectedObject = selectFirstItem ? objects.get(0) : null;
//		    fireContentsChanged(this, 0, objects.size()-1);
	    } else
		    selectedObject = null;
	    fireContentsChanged(this, -1, -1);
    }
	public void init(Collection items, Object selection) {
		if (items == null)
			objects = new ArrayList();
//        else if (items instanceof List)
//			objects = (List)items;
        else
			objects = new ArrayList(items);
		if (objects.contains(selection))
			selectedObject = selection;
		else selectedObject = null;
//		if (objects.size() > 0)
//			fireContentsChanged(this, 0, objects.size()-1);
		fireContentsChanged(this, -1, -1);
	}

	public void init(Object[] items) {
		init(items, true);
	}

	public void init(Object[] items, boolean selectFirstItem) {
		if (items == null)
			objects = new ArrayList();
		else {
			objects = new ArrayList(items.length);
			for (int i = 0, c = items.length; i < c; i++)
				objects.add(items[i]);
		}
		if (objects.size() > 0) {
			selectedObject = selectFirstItem ? objects.get(0) : null;
//		    fireContentsChanged(this, 0, objects.size()-1);
		} else
			selectedObject = null;
		fireContentsChanged(this, -1, -1);
	}

	public void init(Object[] items, Object selection) {
		if (items == null)
			objects = new ArrayList();
		else {
			objects = new ArrayList(items.length);
			for (int i = 0, c = items.length; i < c; i++)
				objects.add(items[i]);
		}
		if (objects.contains(selection))
			selectedObject = selection;
		else
			selectedObject = null;
//		if (objects.size() > 0)
//			fireContentsChanged(this, 0, objects.size()-1);
		fireContentsChanged(this, -1, -1);
	}

	// implements javax.swing.ComboBoxModel
	/**
	 * Set the value of the selected item. The selected item may be null.
	 * <p>
	 * @param selection The combo box value or null for no selection.
	 */
	public void setSelectedItem(Object selection) {
		if (!MiscUtils.equals(selectedObject, selection)) {
			selectedObject = selection;
			fireContentsChanged(this, -1, -1);
		}
	}

	// implements javax.swing.ComboBoxModel
	public Object getSelectedItem() {
		return selectedObject;
	}

	// implements javax.swing.ListModel
	public int getSize() {
		return objects.size();
	}

	// implements javax.swing.ListModel
	public Object getElementAt(int index) {
		if ((index >= 0) && (index < objects.size()))
			return objects.get(index);
		else
			return null;
	}

	/**
	 * Returns the index-position of the specified object in the list.
	 *
	 * @param anObject
	 * @return an int representing the index position, where 0 is
	 *         the first position
	 */
	public int getIndexOf(Object anObject) {
		return objects.indexOf(anObject);
	}

	// implements javax.swing.MutableComboBoxModel
	public void addElement(Object anObject) {
		objects.add(anObject);
		fireIntervalAdded(this, objects.size() - 1, objects.size() - 1);
		setSelectedItem(anObject);	// always select the new item
	}

	// implements javax.swing.MutableComboBoxModel
	public void insertElementAt(Object anObject, int index) {
		objects.add(index, anObject);
		fireIntervalAdded(this, index, index);
		setSelectedItem(anObject);	// always select the new item
	}

	// implements javax.swing.MutableComboBoxModel
	public void removeElementAt(int index) {
		if ((index >= 0) && (index < objects.size())) {
			if (objects.get(index) == selectedObject) {
				if (index == 0)
					setSelectedItem(objects.size() == 1 ? null : objects.get(1));
				else
					setSelectedItem(objects.get(index - 1));
			}
			objects.remove(index);
			fireIntervalRemoved(this, index, index);
		}
	}

	// implements javax.swing.MutableComboBoxModel
	public void removeElement(Object anObject) {
		int index = objects.indexOf(anObject);
		if (index != -1)
			removeElementAt(index);
	}

	/**
	 * Empties the list.
	 */
	public void clear() {
		if (objects.size() > 0) {
			int lastIndex = objects.size() - 1;
			objects.clear();
			selectedObject = null;
			fireIntervalRemoved(this, 0, lastIndex);
		}
	}

    public List getElements() {
        return objects;
    }
}
