/*
 * Project "Guideline Markup Tool"
 * Author: Peter
 * Date: Sep 14, 2002
 * Time: 3:29:57 PM
 */
package at.ac.tuwien.asgaard.swing.models;

import java.util.ArrayList;

import javax.swing.event.*;
import javax.swing.table.*;

/**
 * Class SimpleTableModel. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.1
 */
public class SimpleTableModel extends AbstractTableModel {
	protected java.util.List tableData;
	protected String[] columnNames;
	protected int columnCount;
	protected boolean editable;

	/**
	 * Creates a new SimpleTableModel.
	 * @param tableData a ArrayList of an Object-Array
	 * @param columnNames a String-Array of column names
	 */
	public SimpleTableModel(java.util.List tableData, String[] columnNames) {
		if ((tableData == null) ||
		        (columnNames == null) || (columnNames.length == 0) ||
		        (!tableData.isEmpty() && (((Object[])tableData.get(0)).length != columnNames.length)))
			throw new IllegalArgumentException("Bad table data!");
		this.tableData = tableData;
		this.columnNames = columnNames;
		columnCount = columnNames.length;
		editable = true;
	}
	public SimpleTableModel(String[] columnNames) {
		this(new ArrayList(), columnNames);
	}
	// to be compatible with DefaultTableModel
	public SimpleTableModel(String[] columnNames, int rowCount) {
		this(new ArrayList(rowCount), columnNames);
	}

	public int getColumnCount() {
		return columnCount;
	}

	public int getRowCount() {
		return ((tableData == null) ? 0 : tableData.size());
	}
	public boolean isEmpty() {
		return ((tableData == null) ? true : (tableData.size() == 0));
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return ((Object[])tableData.get(rowIndex))[columnIndex];
	}

	public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
		Object[] rowData = (Object[])tableData.get(rowIndex);
		if (!aValue.equals(rowData[columnIndex])) {
			rowData[columnIndex] = aValue;
			fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE));
		}
	}

	public String getColumnName(int columnIndex) {
		return columnNames[columnIndex];
	}

	public Class getColumnClass(int columnIndex) {
		if (getRowCount() > 0) return getValueAt(0, columnIndex).getClass();
		return Object.class;
	}

	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return editable;
	}

	public int addRow(Object[] row) {
		if ((row == null) || (row.length != columnCount)) return -1;
		tableData.add(row);
		int newRowIndex = tableData.size()-1;
		fireTableChanged(new TableModelEvent(this, newRowIndex, newRowIndex,
						 TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
		return newRowIndex;
	}
	public int addRow(Object[] row, int newRowIndex) {
		if ((row == null) || (row.length != columnCount)) return -1;
		tableData.add(newRowIndex, row);
		fireTableChanged(new TableModelEvent(this, newRowIndex, newRowIndex,
											 TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
		return newRowIndex;
	}
	// to be compatible with DefaultTableModel
	public void insertRow(int newRowIndex, Object[] row) {
		addRow(row, newRowIndex);
	}

	public void removeRow(int rowIndex) {
		tableData.remove(rowIndex);
		fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex,
		                 TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
	}

	public void clear() {
		tableData.clear();
		fireTableDataChanged();
	}

	public boolean isEditable() {
		return editable;
	}

	public void setEditable(boolean editable) {
		this.editable = editable;
	}
}
