/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 15.05.2003
 * Time: 11:14:45
 */
package at.ac.tuwien.asgaard.swing.models;

import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;

/**
 * Class SimpleTreeModel ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public class SimpleTreeModel implements TreeModel {
	/** Root of the tree. */
	protected TreeNode root;
	protected final List<TreeModelListener> listenerList;

	public SimpleTreeModel() {
		root = null; //SimpleMutableTreeNode.getDummyTreeNode();
		listenerList = new ArrayList<TreeModelListener>(4);
	}

	public SimpleTreeModel(TreeNode root) {
		this.root = root;
		listenerList = new ArrayList<TreeModelListener>(4);
	}

	public Object getChild(Object parent, int index) {
		return ((TreeNode) parent).getChildAt(index);
	}

	public int getChildCount(Object parent) {
		return ((TreeNode) parent).getChildCount();
	}

	public int getIndexOfChild(Object parent, Object child) {
		if ((parent == null) || (child == null))
			return -1;
		return ((TreeNode) parent).getIndex((TreeNode) child);
	}

	public boolean isLeaf(Object node) {
		return ((TreeNode) node).isLeaf();
	}

	public Object getRoot() {
		return root;
	}
	/**
	 * Sets the root to <code>root</code>. A null <code>root</code> implies
	 * the tree is to display nothing, and is legal.
	 */
	public void setRoot(TreeNode root) {
		TreeNode oldRoot = this.root;
		if (oldRoot != root) {
			this.root = root;
			TreeModelEvent evt;
			if (root == null)
				evt = new TreeModelEvent(this, new TreePath(oldRoot)); //(TreePath)null);
			else
				evt = new TreeModelEvent(this, new TreePath(root));
			for (int i = listenerList.size()-1; i >= 0; i--)
				listenerList.get(i).treeStructureChanged(evt);
		}
	}

	public TreePath getTreePath(TreeNode node) {
		if (node == null) return null;
        TreeNode parent = node.getParent();
		if (parent == null) //(node == root) 
            return new TreePath(node);
        return getTreePath(parent).pathByAddingChild(node);
	}

	/**
	 * Invoked this to insert newChild at location index in parents children.
	 * This will then message nodesWereInserted to create the appropriate
	 * event. This is the preferred way to add children as it will create
	 * the appropriate event.
	 */
	public void insertNodeInto(MutableTreeNode newChild,
							   MutableTreeNode parent, int index) {
		parent.insert(newChild, index);

		TreeModelEvent evt = new TreeModelEvent(this, getTreePath(parent),
												new int[]{index}, new Object[]{newChild});
		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeNodesInserted(evt);
	}
	/**
	 * Message this to remove node from its parent. This will message
	 * nodesWereRemoved to create the appropriate event. This is the
	 * preferred way to remove a node as it handles the event creation
	 * for you.
	 */
	public void removeNodeFromParent(MutableTreeNode node) {
		MutableTreeNode parent = (MutableTreeNode) node.getParent();
		int index = parent.getIndex(node);
		parent.remove(index);

		TreeModelEvent evt = new TreeModelEvent(this, getTreePath(parent), new int[]{index}, new Object[]{node});
		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeNodesRemoved(evt);
	}
	public void removeNode(MutableTreeNode parent, int index) {
		TreeNode node = parent.getChildAt(index);
		parent.remove(index);

		TreeModelEvent evt = new TreeModelEvent(this, getTreePath(parent), new int[]{index}, new Object[]{node});
		for (int i = listenerList.size() - 1; i >= 0; i--)
			listenerList.get(i).treeNodesRemoved(evt);
	}

    public void replaceNode(MutableTreeNode newChild, MutableTreeNode oldChild, MutableTreeNode parent) {
        int index = parent.getIndex(oldChild);
        parent.remove(index);
        parent.insert(newChild, index);

        TreeModelEvent evt = new TreeModelEvent(this, getTreePath(parent),
                new int[]{index}, new Object[]{parent});
        for (int i = listenerList.size() - 1; i >= 0; i--)
            listenerList.get(i).treeStructureChanged(evt);
    }


	/**
	 * Invoke this method if you've modified the TreeNodes upon which this
	 * model depends.  The model will notify all of its listeners that the
	 * model has changed.
	 */
	public void reload() {
		nodeStructureChanged(root);
	}
	/**
	 * Invoke this method if you've totally changed the children of
	 * node and its childrens children...  This will post a
	 * treeStructureChanged event.
	 */
	public void nodeStructureChanged(TreeNode node) {
		TreeModelEvent evt = new TreeModelEvent(this, getTreePath(node));
		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeStructureChanged(evt);
	}

	/**
	 * Invoke this method after you've changed how node is to be
	 * represented in the tree.
	 */
	public void nodeChanged(TreeNode node) {
		TreeNode parent = node.getParent();
		TreeModelEvent evt;
		if (parent != null)
			evt = new TreeModelEvent(this, getTreePath(parent),
									 new int[]{parent.getIndex(node)}, new Object[]{node});
		else //if (node == root)
			evt = new TreeModelEvent(this, getTreePath(node), null, null);

		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeNodesChanged(evt);
	}

	public void valueForPathChanged(TreePath path, Object newValue) {
		MutableTreeNode aNode = (MutableTreeNode) path.getLastPathComponent();

		aNode.setUserObject(newValue);
		nodeChanged(aNode);
	}


	public void addTreeModelListener(TreeModelListener l) {
		listenerList.add(l);
	}
	public void removeTreeModelListener(TreeModelListener l) {
		listenerList.remove(l); // can be dangerous if called during firing an event, better use Vector or remove this method
	}

	protected void fireTreeNodesChanged(TreeModelEvent evt) {
		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeNodesChanged(evt);
	}

	protected void fireTreeStructureChanged(TreeModelEvent evt) {
		for (int i = listenerList.size()-1; i >= 0; i--)
			listenerList.get(i).treeStructureChanged(evt);
	}
}
