/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * User: peter
 * Date: 13.05.2003
 * Time: 10:53:26
 */
package at.ac.tuwien.asgaard.swing.selectable;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;

import javax.swing.*;
import javax.swing.event.ChangeEvent;

/**
 * Class RadioButtonAbstractAction ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public abstract class RadioButtonAbstractAction extends AbstractAction implements SelectableAction {

	/**
	 * Specifies whether this action is selected; the default is false.
	 */
	protected boolean selected = false;

	protected RadioButtonAbstractAction() { }

    protected RadioButtonAbstractAction(String name, char mnemonicKey) {
        super(name);
        putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
    }
	protected RadioButtonAbstractAction(boolean enabled, boolean selected) {
		this.enabled = enabled;
		this.selected = selected;
	}

	protected RadioButtonAbstractAction(String name, Icon icon) {
		super(name, icon);
	}
    protected RadioButtonAbstractAction(String name, Icon icon, char mnemonicKey) {
        super(name, icon);
        putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
    }
    protected RadioButtonAbstractAction(String name, Icon icon, char mnemonicKey, String toolTip) {
        super(name, icon);
        putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
        putValue(SHORT_DESCRIPTION, toolTip);
    }
	protected RadioButtonAbstractAction(String name, Icon icon, boolean selected) {
		super(name, icon);
		this.selected = selected;
	}
	protected RadioButtonAbstractAction(String name, Icon icon, char mnemonicKey, String toolTip, boolean selected) {
		super(name, icon);
		putValue(MNEMONIC_KEY, new Integer(mnemonicKey));
		putValue(SHORT_DESCRIPTION, toolTip);
		this.selected = selected;
	}

	public final boolean isSelected() {
		return selected;
	}

	public final void setSelected(boolean newValue) {
		boolean oldValue = selected;
		if (oldValue != newValue) {
			selected = newValue;
			firePropertyChange(SelectableAction.SELECTED_KEY, Boolean.valueOf(oldValue), Boolean.valueOf(newValue));
			if (selected)  // <-- the only difference to CheckBoxAbstractAction
				stateChanged(new ChangeEvent(this));
			else
				deselected();
		}
	}

	public final void setSelectedState(boolean newValue) {
		boolean oldValue = this.selected;
		if (oldValue != newValue) {
			this.selected = newValue;
			firePropertyChange(SelectableAction.SELECTED_KEY, Boolean.valueOf(oldValue), Boolean.valueOf(newValue));
		}
	}

	public void actionPerformed(ActionEvent evt) {
		if ((evt == null) || (evt.getModifiers() != InputEvent.BUTTON1_MASK))
			setSelected(true);
	}

	protected void deselected() { }
}
