/*
 * Created by Peter for Project "Guideline Markup Tool".
 * Date: Dec 13, 2002
 * Time: 1:34:29 AM
 */
package at.ac.tuwien.asgaard.util;

import java.util.*;

/**
 * Class MiscUtils. ... todo
 *
 * @version 2.7.3
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class MiscUtils {
	private static final String WRAP_CHARS = " -/,;.:!?";

	private MiscUtils() {} // this class should not be instantiated

	public static String replace(String source, String from, String to) { // todo replace by replaceAll
		if (source == null) return null;
		if ((from == null) || (to == null)) return source;
		StringBuffer sb = new StringBuffer(source);
		int len = from.length(), pos;
		while ((pos = sb.toString().indexOf(from)) != -1)
			sb.replace(pos, pos + len, to);
		return sb.toString();
	}

	public static String replace(String source, int from, int to, String replace) {
		return source.substring(0, from) + replace + source.substring(to+1);
	}

	public static String defaultValue(String value, String defValue) {
		return ((value == null) ? defValue : value);
	}

	/**
	 * <code>equals</code> that considers <code>null</code> arguments
	 * @param o1 first argument
	 * @param o2 second argument
	 * @return if one (or both) argument is null returns (o1 == o2), else returns <code>o1.equals(o2)</code>
	 */
	public static boolean equals(Object o1, Object o2) {
		if ((o1 == null) || (o2 == null)) return (o1 == o2);
		return o1.equals(o2);
	}

	public static Set enumerationToSet(Enumeration enumeration) {
		Set result = new HashSet();
		while (enumeration.hasMoreElements())
			result.add(enumeration.nextElement());
		return result;
	}
	public static SortedSet enumerationToSortedSet(Enumeration enumeration) {
		SortedSet result = new TreeSet();
		while (enumeration.hasMoreElements())
			result.add(enumeration.nextElement());
		return result;
	}

	public static final char[] WORD_SEPARATORS = {' ', '\t', '\n', '\r', '\f',
												  '.', ',', ':', ';', '-', '_', '!', '?',
												  '(', ')', '[', ']', '{', '}', '<', '>', '/', '|', '\\', '\'', '\"'};
	public static String WORD_SEPARATORS_STRING = new String(WORD_SEPARATORS);
	public static boolean isWordSeparator(char ch) {
		return (WORD_SEPARATORS_STRING.indexOf(ch) != -1);
	}
	public static boolean containsWordSeparators(String s) {
		for (int i=0, size=s.length(); i<size; i++)
			if (isWordSeparator(s.charAt(i))) return true;
		return false;
	}


	/**
	 * Wraps a string by inserting "lineBreak"'s obeying word boundaries.
	 * @param source The source string.
	 * @param lineBreak This will be inserted as a line break, e.g. "<br>" or "\n"
	 * @param maxLineLength The maximum length of each line.
	 * @param firstLinePrefixLength if greater than 0, "indents" the first line.
	 * @return new string with line breaks
	 */
	public static String wrapString(String source, String lineBreak, int maxLineLength, int firstLinePrefixLength) {
		StringBuffer sb = new StringBuffer(source);
		int start = 0,
				end = Math.min(source.length(), maxLineLength - firstLinePrefixLength) - 1;
		while (end < sb.length() - 1) {
			int foundAt = -1;
			for (int i = end; (i >= start) && (foundAt == -1); i--) {
				char ch = sb.charAt(i);
				if (WRAP_CHARS.indexOf(ch) != -1) {
					foundAt = i;
					break;
				}
			}
			if (foundAt == -1) foundAt = start + maxLineLength - 1;
			if (foundAt < sb.length() - 5)
				sb.insert(foundAt + 1, lineBreak);
			start = foundAt + 1 + lineBreak.length();
			end = start + maxLineLength;
		}
		return sb.toString();
	}

	public static String truncateSring(String text) {
		return truncateSring(text, 30, 7);
	}
	public static String truncateSring(String text, int truncPos, int searchForSpaceRange) {
		if (text.length() <= truncPos) return text;
		int foundTruncPos = -1;
		for (int i=0; i<= searchForSpaceRange; i++) {
			if ((truncPos + i == text.length()-1)) return text;
			if (!Character.isLetterOrDigit(text.charAt(truncPos + i))) {
				foundTruncPos = truncPos + i; break;
			}
			if ((truncPos - i >= 0) && !Character.isLetterOrDigit(text.charAt(truncPos - i))) {
				foundTruncPos = truncPos - i;
				break;
			}
		}
		if (foundTruncPos == -1) foundTruncPos = truncPos;
		StringBuffer sb = new StringBuffer(text.substring(0, foundTruncPos + 1));
		if (sb.charAt(foundTruncPos) != ' ') sb.append(' ');
		sb.append("...");
		return sb.toString();
	}
}
