/*
 * Asgaard-Project (Peter Votruba), http://www.asgaard.tuwien.ac.at/
 * Vienna University of Technology (Austria)
 * Institute of Software Technology and Interactive Systems
 * Information & Software Engineering Group
 * --------------------------------------------------------------------
 * in cooperation with the Protocure-Project, http://www.protocure.org/
 */
package at.ac.tuwien.asgaard.util;

import java.util.*;

public class SortedList<V> extends ArrayList<V> {
	protected final Comparator comp;

	public SortedList(Comparator<V> comp) {
		this.comp = comp;
	}

	public SortedList(Collection<V> c, Comparator<V> comp) {
		super(c);
		this.comp = comp;
	}

	public SortedList(int initialCapacity, Comparator<V> comp) {
		super(initialCapacity);
		this.comp = comp;
	}

	public boolean add(V value) {
		if (comp == null)
			super.add(value);
		else {
			int length = size();
			if (length == 0)
				super.add(value);
			else if (comp.compare(value, get(0)) < 0)
				super.add(0, value);
			else if (comp.compare(value, get(length - 1)) > 0)
				super.add(value);
			else
				for (int i = length - 2; i >= 0; i--) {
					int diff = comp.compare(value, get(i));
					if (diff >= 0) {
						if (diff == 0)
							super.set(i, value); // replace old value if its the same again
						else
							super.add(i + 1, value);
						break;
					}
				}
		}
		return true;
	}

	public boolean addAll(int index, Collection<? extends V> c) {
		Iterator<? extends V>  iterator = c.iterator();
		int sizeOther = c.size();
		for (int i = 0; i < sizeOther; i++) {
			V o = iterator.next();
			add(o);
		}
		return (sizeOther != 0);
	}

	public static int getInsertIndex(List list, Object value, Comparator comp) {
		int length = list.size();
		if (comp == null)
			return length;
		else {
			if (length == 0)
				return 0;
			else if (comp.compare(value, list.get(0)) < 0)
				return 0;
			else if (comp.compare(value, list.get(length - 1)) > 0)
				return length;
			else
				for (int i = length - 2; i >= 0; i--) {
					int diff = comp.compare(value, list.get(i));
					if (diff >= 0) return i+1;
				}
			return length;
		}
	}
}
