// taken from a posting in the de.comp.lang.java newsgroup

package at.ac.tuwien.asgaard.util;

/* "One of the most usual distance is the Weighted Levenshtein Distance (WLD), that indicates 
the minimum number of replacements r(i), insertions i(i) and deletions d(i) of symbols that 
have to be made in a chain to be converted in other one." */

public class WLD {
	public static final WLD DEFAULT_WLD = new WLD(2, 2, 1, true);
	
	protected int mP; // INSERT
	protected int mQ; // DELETE
	protected int mR; // REPLACE
	protected boolean ignoreCase;

	public WLD(int p, int q, int r, boolean ignoreCase) {
		mP = p;
		mQ = q;
		mR = r;
		this.ignoreCase = ignoreCase;
	}

	public WLD() {
		this(1, 1, 1, false);
	}

	/**
	 * Berechnet die Distanz von src->dst.
	 *
	 * Liefert die gewichtete Summe von Inserts/Deletes/Replaces
	 * zurck, die den String src nach String dst transformiert.
	 *
	 */
	public int distance(String src, String dst) {
		if (ignoreCase) {
			src = src.toLowerCase();
			dst = dst.toLowerCase();
		}

		// we need two rows of the matrix
		int[] m0 = new int[1 + src.length()];
		int[] m1 = new int[1 + src.length()];

		int a1;
		int a2;
		int a3;

		// init first row
		for (int i = 0; i <= src.length(); ++i)
			m0[i] = i * mQ;

		for (int j = 0; j < dst.length(); ++j) {
			m1[0] = m0[0] + mP;

			for (int i = 0; i < src.length(); ++i) {
				a1 = m0[i + 1] + mP;
				a2 = m1[i] + mQ;
				a3 = m0[i] + (src.charAt(i) == dst.charAt(j) ? 0 : mR);

				m1[i + 1] = Math.min(a1, Math.min(a2, a3));
			}

			// one row processed
			int[] tmp = m0;
			m0 = m1;
			m1 = tmp;
		}

		// Were done!
		return m0[src.length()];

	}

	public static boolean defaultMatch(String a, String b) {
		return match(a, b, DEFAULT_WLD);
	}
	public static boolean match(String a, String b, WLD wld) {
		return (wld.distance(a, b) <= (Math.max(a.length(), b.length()) >> 1));
	}

// public static void main(String[] args) {
//  WLD wld = null;
//  if (args.length >= 5)
//  wld = new WLD(Integer.parseInt(args[2]), Integer.parseInt(args[3]), Integer.parseInt(args[4]));
//  else
//  wld = new WLD();
//  System.out.println("distance(" + args[0] + ", " + args[1] + ") = " + wld.distance(args[0], args[1]));
// }
}
