package at.ac.tuwien.asgaard.util.xml;

import at.ac.tuwien.asgaard.util.FileUtils;

import org.apache.xerces.parsers.DTDConfiguration;
import org.apache.xerces.parsers.SAXParser;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import javax.swing.filechooser.*;

public class AnyXMLFileFilter extends FileFilter {
	protected final DTDConfiguration config;
	protected final SAXParser parser;
	protected static final int MAX_PARSE_STEPS = 10;
	protected boolean continueParse, successful;

	// Files with these extensions are never XML files
	protected static final String[] DEFAULT_NO_XML_FILE_EXTENSIONS = {".html", ".htm", ".dtd", ".css", ".exe", ".dll", ".zip", ".class",".jar",".java",".gz",".tar",".txt", ".gif", ".jpg", ".jpeg", ".png", ".jar", ".java", ".class", ".properties"};
	protected final Set<String> xmlFileExtensionsSet;
	protected final Set<String> noXmlFileExtensionsSet;

	public AnyXMLFileFilter() {
		this(new HashSet<String>(), new HashSet<String>(Arrays.asList(DEFAULT_NO_XML_FILE_EXTENSIONS)));
	}
	public AnyXMLFileFilter(Set<String> xmlFileExtensionsSet, Set<String> noXmlFileExtensionsSet) {
		this.xmlFileExtensionsSet	= xmlFileExtensionsSet;
		this.noXmlFileExtensionsSet	= noXmlFileExtensionsSet;

		config = new DTDConfiguration();
		parser = new SAXParser(config);
		parser.setContentHandler(getContentHandler());
		try {
			parser.setFeature("http://apache.org/xml/features/validation/dynamic", false);
			parser.setFeature("http://apache.org/xml/features/nonvalidating/load-dtd-grammar", false);
			parser.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
		} catch (SAXException e) {
			e.printStackTrace();
		}
	}

	protected ContentHandler getContentHandler() {
		return new AnyXMLContentHandler();
	}

	public boolean accept(File file) {
		if (file.isDirectory()) return true;
		String ext = FileUtils.getFileExt(file).toLowerCase();

		if (xmlFileExtensionsSet.isEmpty()) {
			if (ext.equals(".xml"))
				return true; // lets trust that this is an well-formed xml file
		} else if (!xmlFileExtensionsSet.contains(ext) || noXmlFileExtensionsSet.contains(ext))
			return false; // do not accept file

		continueParse = true;
		successful = false;
		try {
			String systemId = file.toURI().toString();  //.getAbsolutePath();
			Reader reader = new BufferedReader(new FileReader(file));
			config.setInputSource(new XMLInputSource(null, systemId, systemId, reader, null));
			parser.reset();
			boolean goOn = true;
			for (int i = 0; goOn && (i < MAX_PARSE_STEPS); i++) {
				boolean moreToParse = config.parse(false);
				goOn = continueParse && moreToParse;
			}
			reader.close();
			return successful; // if the loop wasn't stopped through startElement/continueParse=false it's not an XML file
		} catch (Exception exc) {
			return false;
		} finally {
			config.cleanup();
		}
	}

	public String getDescription() {
		return "Any XML File";
	}

	protected class AnyXMLContentHandler extends DefaultHandler {
		// parse always until the first element
		public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
			successful = !localName.equalsIgnoreCase("html");
			continueParse = false; // stop parsing at the first element
		}
	}
}

