package at.ac.tuwien.asgaard.util.xml.dtd;

import at.ac.tuwien.asgaard.util.Log;
import at.ac.tuwien.asgaard.util.xml.XMLUtils;
import com.ibm.xml.parser.*;
import org.apache.xerces.dom.CoreDocumentImpl;
import org.apache.xerces.util.XMLChar;
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;
import org.xml.sax.InputSource;

import java.util.*;
import java.io.IOException;
import java.io.Reader;
import java.net.URL;



/**
 * Class DTDHelperXML4J.... todo
 * 
 * @author Peter Votruba
 * @version 2.7.3
 */
public final class DTDHelperXML4J extends DTDHelper {
	private DTD dtd;
	private Hashtable targetTable, parentTable;
	private Element parentElement;
	private Map element2TableMap;

	private static final Map dtdMap = new Hashtable(), dtd2TablesMap = new Hashtable();
	private static final ParserDummy parserDummy = new ParserDummy();

	public static DTDHelper getDTDHelper(String dtdFilename) {
		DTDHelperXML4J dtdHelper = new DTDHelperXML4J();
		dtdHelper.loadDTDFile(dtdFilename);
		return dtdHelper;
	} 

	public DTDHelperXML4J() {
		this(STRICT_CHECK);
	}

	public DTDHelperXML4J(int state) {
		targetElement = null;
		targetTable = parentTable = null;
		dtd = null;
		this.state = state; // newState =
	}

	public void loadDTDFile(String dtdFilename) {
		loadDTDFile(dtdFilename, false);
	}

	public void loadDTDFile(String dtdFilename, boolean dontUseCachedDTD) {
		Log.trace("enter: DTDHelperXML4J.loadDTDFile() - " + dtdFilename);
	// state = newState;
		parentNode = targetNode = targetElement = parentElement = null;
		targetTable = parentTable = null;
		if (state == NO_DTD)
			Log.trace("DTD will not be loaded!");
		else if (dtdFilename == null) {
			dtd = null;
			Log.trace("No DTD specified!");
		} else {
			element2TableMap = (Map) dtd2TablesMap.get(dtdFilename);
			if (element2TableMap == null) {
				element2TableMap = new HashMap();
				dtd2TablesMap.put(dtdFilename, element2TableMap);
			}
			if (!dontUseCachedDTD)
			    dtd = (DTD) dtdMap.get(dtdFilename);
			if (dontUseCachedDTD || (dtd == null))
				try {
					Parser parser = new Parser(null, parserDummy, parserDummy);
					parser.parseDTD(new InputSource(dtdFilename));
					dtd = ((TXDocument) parser.getDocument()).getDTD();
					Enumeration elements = dtd.getElementDeclarations(); // because it happened, that dtd != null, but readDTDStream didn't succed
					if (!elements.hasMoreElements()) {
						dtd = null;
					//	state = NO_DTD; // do not do this without resetting it to original value
						System.err.println("No element definitions found in DTD \"" + dtdFilename + "\"!");
					} else {
						dtdMap.put(dtdFilename, dtd);
						element2TableMap.clear();
					}
				} catch (Throwable exc) {
					dtd = null;
					//	state = NO_DTD; // do not do this without resetting it to original value
					System.err.println("Couldn't read DTD \"" + dtdFilename + "\":\n  " + exc);

					if (exc instanceof org.xml.sax.SAXException)
						((org.xml.sax.SAXException)exc).getException().printStackTrace();
					else exc.printStackTrace();
				}
		}
		Log.trace("exit:  DTDHelperXML4J.loadDTDFile() - " + dtdFilename);
	}

	public void postInitHack(Document document) {
		if (dtd != null) {
			clearIdentifiers();
			Enumeration identifiers = ((CoreDocumentImpl) document).getIdentifiers();
			while (identifiers.hasMoreElements()) {
				String id = (String) identifiers.nextElement();
				dtd.registID(((CoreDocumentImpl) document).getIdentifier(id), id);
			}
		}
	}
	public void addIdentifier(String id, Element element) {
		if (dtd != null)
	    	dtd.registID(element, id);
	}

	public void removeIdentifier(String id) {
		if (dtd != null)
			dtd.unregistID(id);
	}

	private void clearIdentifiers() {
		if (dtd != null)
			for (Enumeration ids = dtd.IDs(); ids.hasMoreElements();)
				dtd.unregistID((String) ids.nextElement());
	}

	public boolean isValidAttribute(String elementName, String attrName, String newAttrValue, String oldAttrValue) {
		switch (getAttributeValueType(elementName, attrName)) {
			case DTDHelper.ID:
                //case DTDHelper.IDS: // is missing in XML4J#DTD
				return (newAttrValue.equals(oldAttrValue) || ((dtd.checkID(newAttrValue) == null) && org.apache.xerces.util.XMLChar.isValidName(newAttrValue)));
			case DTDHelper.NMTOKEN:
			case DTDHelper.NMTOKENS:
				return org.apache.xerces.util.XMLChar.isValidNmtoken(newAttrValue);
            case IDREF:
            case IDREFS:
                return (dtd.checkID(newAttrValue) != null);
		}
		return true;
	}
    public String checkAttribute(String elementName, String attrName, String newAttrValue, String oldAttrValue) {
        String msg = null;
        switch (getAttributeValueType(elementName, attrName)) {
            case DTDHelper.ID:
                //case DTDHelper.IDS: // is missing in XML4J#DTD
                if (!((newAttrValue.equals(oldAttrValue) || (dtd.checkID(newAttrValue) == null)) && XMLChar.isValidName(newAttrValue)))
                    msg = "New value is not a valid, unique ID!";
                break;
            case DTDHelper.NMTOKEN:
            case DTDHelper.NMTOKENS:
                if (!org.apache.xerces.util.XMLChar.isValidNmtoken(newAttrValue))
                    msg = "New value is not a valid NMTOKEN!";
                break;
            case IDREF:
            case IDREFS:
                if (dtd.checkID(newAttrValue) == null)
                    msg = "Corresponding ID attribute does not exist for this IDREF attribute!";
                break;
        }
        return msg;
    }

	public void setTargetElement(Node newTargetNode) {
		if (targetNode != newTargetNode) {
			parentNode = targetNode = targetElement = parentElement = null;
			targetTable = parentTable = null;
            targetNode = newTargetNode;
            if (newTargetNode != null) {
                parentNode = targetNode.getParentNode();
                if ((dtd != null) && (state >= NO_CHECK)) {
                    if (newTargetNode.getNodeType() == Node.ELEMENT_NODE) {
                        targetElement = new TXWrapper(newTargetNode);
                        String newNodeName = newTargetNode.getNodeName();
                        targetTable = (Hashtable) element2TableMap.get(newNodeName);
                        if (targetTable == null) {
                            targetTable = dtd.prepareTable(newNodeName);
                            element2TableMap.put(newNodeName, targetTable);
                        }
                    }
                    if (parentNode != null) {
                        parentElement = new TXWrapper(parentNode);
                        String parentNodeName = parentElement.getNodeName();
                        parentTable = (Hashtable) element2TableMap.get(parentNodeName);
                        if (parentTable == null) {
                            parentTable = dtd.prepareTable(parentNodeName);
                            element2TableMap.put(parentNodeName, parentTable);
                        }
                    }
                }
            }
        }
    }

	/**
	 * Try guessing the root element and return its name.
	 *
	 * @return If there is only one element that cannot be the child of any other element, return the name of this element.
	 * If there are more such elements (which should not happen), return the name of the first one.
	 * If there is no such element (i.e. each element can be the child of another element), return null.
	 */
	public String guessRootElement() {
		if (dtd != null) {
			Enumeration elements1 = dtd.getElementDeclarations();
			if (elements1 != null)
				while (elements1.hasMoreElements()) {
					String elemName1 = ((ElementDecl) elements1.nextElement()).getName();
					Enumeration elements2 = dtd.getElementDeclarations();
					boolean found = false;
					while (elements2.hasMoreElements() && !found) {
						String elemName2 = ((ElementDecl) elements2.nextElement()).getName();
						CMNode contentModelNode = dtd.getContentModel(elemName2).getContentModelNode();
						if (contentModelNode == null)
							continue;
						String contentModelString = contentModelNode.toString();
						found = (contentModelString.indexOf(elemName1) != -1);
					}
					if (!found) return elemName1;
				}
		}
		return null;
	}

    public Set getAllElementsSet() {
        Set set = new TreeSet();
		if (dtd != null) {
			Enumeration elements = dtd.getElementDeclarations();
			if (elements != null)
				while (elements.hasMoreElements())
					set.add(((ElementDecl) elements.nextElement()).getName());
		} else {
			Document document = targetNode.getOwnerDocument();
			DocumentTraversal trav = (DocumentTraversal) document;
			NodeIterator nodeIterator = trav.createNodeIterator(document.getDocumentElement(), NodeFilter.SHOW_ELEMENT, null, false);
			Node node;
			while ((node = nodeIterator.nextNode()) != null)
				set.add(node.getNodeName());
		}
        return set;
    }

	public SortedSet getInsertableIntoElementSet() {
        SortedSet set = new TreeSet();
		if ((dtd != null) && (targetTable != null)) {
			int oldState = state;
			if ((state == STRICT_CHECK) && !isValid(targetElement)) state = CHECK;
			for (int index = targetElement.getChildNodes().getLength(); index >= 0; index--) {
				if (state == STRICT_CHECK)
					targetTable = dtd.getInsertableElementsForValidContent(targetElement, index, targetTable);
				else
					targetTable = dtd.getInsertableElements(targetElement, index, targetTable);
				for (Enumeration elements = targetTable.elements(); elements.hasMoreElements();) {
					InsertableElement insertable = (InsertableElement) elements.nextElement();
					if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) && insertable.status)
						set.add(insertable.name);
				}
			}
			state = oldState;
		}
		if (XMLUtils.isBranchNode(targetNode))
			set.add(SPECIAL_NODE_NAME_COMMENT);
		return set;

	}

	public SortedSet getInsertableBeforeElementSet() {
		SortedSet set = new TreeSet();
		if ((dtd != null) && (parentElement != null)) {
			if (parentTable != null) {
				int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
				for (int index = indexOfCurrentNode; index >= 0; index--) {
					if (state == STRICT_CHECK)
						parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
					else
						parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
					for (Enumeration elements = parentTable.elements(); elements.hasMoreElements();) {
						InsertableElement insertable = (InsertableElement) elements.nextElement();
						if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) &&
							insertable.status)
							set.add(insertable.name);
					}
				}
			}
			if (targetNode.getNodeType() != Node.DOCUMENT_TYPE_NODE) set.add(SPECIAL_NODE_NAME_COMMENT); // comment is valid before any node, if targetNode is not the document node
		}
		return set;
	}

	public SortedSet getInsertableAfterElementSet() {
		SortedSet set = new TreeSet();
		if ((dtd != null) && (parentElement != null)) {
			if (parentTable != null) {
				int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
				for (int index = indexOfCurrentNode + 1, lastIndex = parentElement.getChildNodes().getLength();
					 index <= lastIndex; index++) {
					if (state == STRICT_CHECK)
						parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
					else
						parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
					for (Enumeration elements = parentTable.elements(); elements.hasMoreElements();) {
						InsertableElement insertable = (InsertableElement) elements.nextElement();
						if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) &&
							insertable.status)
							set.add(insertable.name);
					}
				}
			}
			set.add(SPECIAL_NODE_NAME_COMMENT); // comment is valid after any node, if targetNode is not the document node
		}
		return set;
	}

	public SortedSet getInsertableReplaceElementSet() {
		SortedSet result = new TreeSet();
		if ((dtd != null) && (parentElement != null)) {
			if (parentTable != null) {
				Node beforeNode = targetNode.getNextSibling();
				int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
				parentElement.removeChild(targetNode);
				if (state == STRICT_CHECK)
					parentTable = dtd.getInsertableElementsForValidContent(parentElement, indexOfCurrentNode, parentTable);
				else
					parentTable = dtd.getInsertableElements(parentElement, indexOfCurrentNode, parentTable);
				for (Enumeration elements = parentTable.elements(); elements.hasMoreElements();) {
					InsertableElement insertable = (InsertableElement) elements.nextElement();
					if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) && insertable.status)
						result.add(insertable.name);
				}
				if (isValid(parentElement))
					result.add(SPECIAL_NODE_NAME_COMMENT);
				parentNode.insertBefore(targetNode, beforeNode);
			}
		}
		return result;
	}

	public SortedSet getInsertableConvertElementSet() {
		SortedSet result = new TreeSet();
		if ((dtd != null) && (parentElement != null) && (targetElement != null) && (parentTable != null)) {
			Node beforeNode = targetNode.getNextSibling();
			int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
			parentElement.removeChild(targetNode);
			if (state == STRICT_CHECK)
				parentTable = dtd.getInsertableElementsForValidContent(parentElement, indexOfCurrentNode, parentTable);
			else
				parentTable = dtd.getInsertableElements(parentElement, indexOfCurrentNode, parentTable);
			for (Enumeration elements = parentTable.elements(); elements.hasMoreElements();) {
				InsertableElement insertable = (InsertableElement) elements.nextElement();
				if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) && insertable.status && _isElementConvertable(insertable.name))
					result.add(insertable.name);
			}
			parentNode.insertBefore(targetNode, beforeNode);
		}
		return result;
	}

	public boolean canBeConvertedToElement(String newElementName) {
		return canBeReplacedByNode(newElementName) && (targetNode.getNodeType() == Node.ELEMENT_NODE) && _isElementConvertable(newElementName);
	}
	private boolean _isElementConvertable(String newElementName) {
		if (newElementName.equals(targetNode.getNodeName())) return false;

		Set currentAttributeNames = XMLUtils.getAttributeNames(targetElement);
		Set requiredAttributeNamesForNewElement = getRequiredAttributeNames(newElementName);

		boolean result = currentAttributeNames.equals(requiredAttributeNamesForNewElement);

		if (result) {
			Element newElement = targetNode.getOwnerDocument().createElement(newElementName);
			Iterator iter = currentAttributeNames.iterator();
			for (int i = currentAttributeNames.size(); i > 0; i--) {
				String attrName = (String) iter.next();
				String attrValue = targetElement.getAttribute(attrName);
				newElement.setAttribute(attrName, attrValue);
			}

			if (targetElement.getFirstChild() == null)
				result = isValid(newElement);
			else {
				Node currentNode = targetNode;
				Element currentElement = targetElement;
				Hashtable currentTable = targetTable;
				targetNode = newElement;
				targetElement = new TXWrapper(newElement);
				targetTable = (Hashtable) element2TableMap.get(newElementName);
				if (targetTable == null) {
					targetTable = dtd.prepareTable(newElementName);
					element2TableMap.put(newElementName, targetTable);
				}
				result = (getLastInsertIntoIndexForMacro(currentElement) != -1);
				targetNode = currentNode;
				targetElement = currentElement;
				targetTable = currentTable;
			}
		}

		return result;
	}

	public SortedSet getInsertableWrapElementSet() {
		SortedSet result = new TreeSet();
		if ((dtd != null) && (parentElement != null) && (targetElement != null) && (parentTable != null)) {
			Node beforeNode = targetNode.getNextSibling();
			int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
			parentElement.removeChild(targetNode);
			if (state == STRICT_CHECK)
				parentTable = dtd.getInsertableElementsForValidContent(parentElement, indexOfCurrentNode, parentTable);
			else
				parentTable = dtd.getInsertableElements(parentElement, indexOfCurrentNode, parentTable);
			for (Enumeration elements = parentTable.elements(); elements.hasMoreElements();) {
				InsertableElement insertable = (InsertableElement) elements.nextElement();
				if ((insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) && insertable.status && _isElementWrapable(insertable.name))
					result.add(insertable.name);
			}
			parentNode.insertBefore(targetNode, beforeNode);
		}
		return result;
	}
	public boolean canBeWrappedByElement(String newElementName) {
		return canBeReplacedByNode(newElementName) && (targetNode.getNodeType() == Node.ELEMENT_NODE) && _isElementWrapable(newElementName);
	}
	private boolean _isElementWrapable(String newElementName) {
		List possibleChildren = dtd.makeContentElementList(newElementName);
		if (possibleChildren == null) return false;
		return possibleChildren.contains(targetNode.getNodeName());
	}

	public List getInsertablePositions(Node newNode) {
		List result;
		int oldState = state;
		if (newNode.getNextSibling() != null) // if newNode has siblings -> do not strict check
			state = Math.min(state, CHECK);
		result = getInsertablePositions(newNode.getNodeName());
		state = oldState;
		return result;
	}

	public List getInsertablePositions(String newNodeName) {
		List<Integer> result = new LinkedList<Integer>();
		if (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT)
			result.add(0);
		else if ((dtd != null) && (targetTable != null) && (state >= CHECK))
			for (int index = 0, count = targetElement.getChildNodes().getLength(); index <= count; index++) {
				if (state == STRICT_CHECK)
					targetTable = dtd.getInsertableElementsForValidContent(targetElement, index, targetTable);
				else
					targetTable = dtd.getInsertableElements(targetElement, index, targetTable);
				InsertableElement insertable = (InsertableElement) targetTable.get(newNodeName);
				if ((insertable != null) && insertable.status)
					result.add(index);
			}
		return result;
	}


	public int getLastInsertIntoIndex(String newNodeName) {
		if ((targetElement == null) || (newNodeName == null)) return -1;
		if (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT) return 0;
		try {
			if ((dtd == null) || (targetTable == null) || (state < CHECK))
				return targetElement.getChildNodes().getLength();
			for (int index = targetElement.getChildNodes().getLength(); index >= 0; index--) {
//				if (state == STRICT_CHECK)
//					targetTable = dtd.getInsertableElementsForValidContent(targetElement, index, targetTable);
//				else
					targetTable = dtd.getInsertableElements(targetElement, index, targetTable);
				InsertableElement insertable = (InsertableElement) targetTable.get(newNodeName);
				if ((insertable != null) && insertable.status) return index;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return -1;
	}

	public int getInsertBeforeIndex(String newNodeName) {
		if ((targetNode == null) || (newNodeName == null)) return -1;

        int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
        if ((dtd == null) || (state < CHECK) || (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT))
            return indexOfCurrentNode;

		if ((parentTable == null) || (parentElement == null))
			return -1;

		for (int index = indexOfCurrentNode; index >= 0; index--) {
//			if (state == STRICT_CHECK)
//				parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
//			else
				parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
			InsertableElement insertable = (InsertableElement) parentTable.get(newNodeName);
			if ((insertable != null) && insertable.status)
				return index;
		}
		return -1;
	}

	public int getInsertAfterIndex(String newNodeName) {
		if ((targetNode == null) || (newNodeName == null)) return -1;

        int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
        if ((dtd == null) || (state < CHECK) || (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT))
            return indexOfCurrentNode + 1;

		if ((parentElement == null) || (parentTable == null)) return -1;

		for (int index = indexOfCurrentNode + 1, lastIndex = parentElement.getChildNodes().getLength(); index <= lastIndex; index++) {
//			if (state == STRICT_CHECK)
//				parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
//			else
				parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
			InsertableElement insertable = (InsertableElement) parentTable.get(newNodeName);
			if ((insertable != null) && insertable.status) return index;
		}
		return -1;
	}

	public boolean canBeReplacedByNode(String newNodeName) {
		if ((targetNode == null) || (newNodeName == null)) return false;
		if (state < CHECK) return true;
        if (dtd == null) return false;
		if  (targetNode.getNodeType() == Node.ELEMENT_NODE) {
				if (XMLUtils.isDocumentElement(targetNode)) return false;
		} else
			return isNodeRemoveable();
		if ((parentElement == null) || (parentTable == null))
			return false;

		int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
		Node beforeNode = targetNode.getNextSibling();
		parentElement.removeChild(targetNode);

		boolean replaceable;
		if (newNodeName == DTDHelper.SPECIAL_NODE_NAME_COMMENT)
			replaceable = isValid(parentElement);
		else {
			if (state == STRICT_CHECK)
				parentTable = dtd.getInsertableElementsForValidContent(parentElement, indexOfCurrentNode, parentTable);
			else
				parentTable = dtd.getInsertableElements(parentElement, indexOfCurrentNode, parentTable);
			InsertableElement insertable = (InsertableElement) parentTable.get(newNodeName);
			replaceable = ((insertable != null) && insertable.status);
		}

		parentElement.insertBefore(targetNode, beforeNode);
		return replaceable;
	}


	public boolean isNodeRemoveable() {
		if (targetNode == null) return false;

		switch (targetNode.getNodeType()) {
			case Node.ELEMENT_NODE:
				if (XMLUtils.isDocumentElement(targetNode)) return false;
				break;
			case Node.TEXT_NODE:
			case Node.CDATA_SECTION_NODE:
			case Node.COMMENT_NODE:
			case Node.PROCESSING_INSTRUCTION_NODE:
				return true;
			default:
				return false;
		}

        if ((dtd == null) || (state < CHECK)) return true;
		if ((parentElement == null) || (parentTable == null))
			return false;

		Node beforeNode = targetNode.getNextSibling();
		parentElement.removeChild(targetNode);

		boolean removeable = isValid(parentElement);

		parentElement.insertBefore(targetNode, beforeNode);
		return removeable;
	}


	public int getLastInsertIntoIndexForMacro(Element macroElement) {
		if ((macroElement == null) || (targetElement == null) || (targetTable == null)) return -1;
		NodeList children = targetNode.getChildNodes();
        if ((dtd == null) || (state < CHECK))
			return children.getLength();
		if (macroElement.getFirstChild() == null) // macro is empty!
			return children.getLength(); // instead of -1; because getLastInsertIntoIndexForMacro is used also in isElementConvertable

		Node iteratorChild = null;
		Node[] importedNewChildNode = null;
		Document ownerDocument = targetNode.getOwnerDocument();
		for (int index = children.getLength(); index >= 0; index--) {
			if (iteratorChild == null)
				iteratorChild = children.item(index);

			if (importedNewChildNode == null) {
				importedNewChildNode = new Node[macroElement.getChildNodes().getLength()];
				int i = 0;
				for (Node childNode = macroElement.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
					if (childNode.getNodeType() != Node.COMMENT_NODE)
						importedNewChildNode[i++] = ownerDocument.importNode(childNode, false);
			}

			Node beforeChild = iteratorChild;
			for (int i = importedNewChildNode.length - 1; i >= 0; i--)
				if (importedNewChildNode[i] != null)
					beforeChild = targetNode.insertBefore(importedNewChildNode[i], beforeChild);

			boolean isValidAfterInsert = isValid(targetNode);

			for (int i = 0; i < importedNewChildNode.length; i++)
				if (importedNewChildNode[i] != null)
					targetElement.removeChild(importedNewChildNode[i]);

			if (isValidAfterInsert)
				return index;

			if (iteratorChild != null)
				iteratorChild = iteratorChild.getPreviousSibling();
		}

		return -1;
	}

	public int getInsertBeforeIndexForMacro(Element macroElement) {
		if ((macroElement == null) || (targetNode == null) || (parentElement == null) || (parentTable == null)) return -1;

		int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
        if ((dtd == null) || (state < CHECK))
			return indexOfCurrentNode;

		Node firstNewChildNode = macroElement.getFirstChild();
		if (firstNewChildNode == null) return -1; // macro is empty!
		while ((firstNewChildNode != null) && (firstNewChildNode.getNodeType() == Node.COMMENT_NODE))
			firstNewChildNode = firstNewChildNode.getNextSibling();
		if (firstNewChildNode == null) return indexOfCurrentNode; // macro has only comment-node

		String firstNewChildNodeName = getNodeName(firstNewChildNode);

		Node iteratorChild = targetNode;
		Node[] importedNewChildNode = null;
		Document ownerDocument = targetNode.getOwnerDocument();
		for (int index = indexOfCurrentNode; index >= 0; index--) {
			if (state == STRICT_CHECK)
				parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
			else
				parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
			InsertableElement insertable = (InsertableElement) parentTable.get(firstNewChildNodeName);
			if ((insertable != null) && insertable.status) {
				if (importedNewChildNode == null) {
					importedNewChildNode = new Node[macroElement.getChildNodes().getLength()];
					int i = 0;
					for (Node childNode = macroElement.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
						if (childNode.getNodeType() != Node.COMMENT_NODE)
							importedNewChildNode[i++] = ownerDocument.importNode(childNode, false);
				}

				Node beforeChild = iteratorChild;
				for (int i = importedNewChildNode.length - 1; i >= 0; i--)
					if (importedNewChildNode[i] != null)
						beforeChild = parentElement.insertBefore(importedNewChildNode[i], beforeChild);

				boolean isValidAfterInsert = isValid(parentElement);

				for (int i = 0; i < importedNewChildNode.length; i++)
					if (importedNewChildNode[i] != null)
						parentElement.removeChild(importedNewChildNode[i]);

				if (isValidAfterInsert)
					return index;
			}
			iteratorChild = iteratorChild.getPreviousSibling();
		}

		return -1;
	}

	public int getInsertAfterIndexForMacro(Element macroElement) {
		if ((macroElement == null) || (targetNode == null) || (parentElement == null) || (parentTable == null)) return -1;

		int indexOfCurrentNode = XMLUtils.getIndexOfChildNode(targetNode);
        if ((dtd == null) || (state < CHECK))
			return indexOfCurrentNode+1;

		Node firstNewChildNode = macroElement.getFirstChild();
		if (firstNewChildNode == null) return -1; // macro is empty!
		while ((firstNewChildNode != null) && (firstNewChildNode.getNodeType() == Node.COMMENT_NODE))
			firstNewChildNode = firstNewChildNode.getNextSibling();
		if (firstNewChildNode == null) return indexOfCurrentNode+1; // macro has only comment-node
		String firstNewChildNodeName = getNodeName(firstNewChildNode);

		Node iteratorChild = targetNode.getNextSibling();
		Node[] importedNewChildNode = null;
		Document ownerDocument = targetNode.getOwnerDocument();
		for (int index = indexOfCurrentNode + 1, lastIndex = parentElement.getChildNodes().getLength();
			 index <= lastIndex; index++) {
			if (state == STRICT_CHECK)
				parentTable = dtd.getInsertableElementsForValidContent(parentElement, index, parentTable);
			else
				parentTable = dtd.getInsertableElements(parentElement, index, parentTable);
			InsertableElement insertable = (InsertableElement) parentTable.get(firstNewChildNodeName);
			if ((insertable != null) && insertable.status) {
				if (importedNewChildNode == null) {
					importedNewChildNode = new Node[macroElement.getChildNodes().getLength()];
					int i = 0;
					for (Node childNode = macroElement.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
						if (childNode.getNodeType() != Node.COMMENT_NODE)
							importedNewChildNode[i++] = ownerDocument.importNode(childNode, false);
				}

				Node beforeChild = iteratorChild;
				for (int i = importedNewChildNode.length - 1; i >= 0; i--)
					if (importedNewChildNode[i] != null)
						beforeChild = parentNode.insertBefore(importedNewChildNode[i], beforeChild);

				boolean isValidAfterInsert = isValid(parentElement);

				for (int i = 0; i < importedNewChildNode.length; i++)
					if (importedNewChildNode[i] != null)
						parentNode.removeChild(importedNewChildNode[i]);

				if (isValidAfterInsert)
					return index;
			}
			if (iteratorChild != null)
				iteratorChild = iteratorChild.getNextSibling();
		}

		return -1;
	}

	public boolean canBeReplacedByMacro(Element macroElement) {
		try {
			if ((macroElement == null) || !XMLUtils.isRemoveable(targetNode)) return false;
			if (state < CHECK) return true;
			if (dtd == null) return false;
			if ((targetNode == null) || (parentElement == null) || (parentTable == null))
				return false;

			Node beforeNode = targetNode.getNextSibling();
			parentNode.removeChild(targetNode);

			Document ownerDocument = targetNode.getOwnerDocument();
			Node[] importedNewChildNode = new Node[macroElement.getChildNodes().getLength()];
			int i = 0;
			for (Node childNode = macroElement.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
				importedNewChildNode[i++] = ownerDocument.importNode(childNode, false);

			Node beforeChildTemp = beforeNode;
			for (i = importedNewChildNode.length - 1; i >= 0; i--)
				beforeChildTemp = parentNode.insertBefore(importedNewChildNode[i], beforeChildTemp);

			boolean isValidAfterReplace = isValid(parentElement);

			for (i = 0; i < importedNewChildNode.length; i++)
				parentNode.removeChild(importedNewChildNode[i]);

			parentNode.insertBefore(targetNode, beforeNode);
			return isValidAfterReplace;
		} catch (DOMException e) {	 // TODO
			return false;
		}
	}


	public boolean isNodeInsertableAt(Node newNode, int index) {
		String newNodeName;
		switch (newNode.getNodeType()) {
			case Node.COMMENT_NODE:
				return true;
			case Node.TEXT_NODE:
				newNodeName = DTD.CM_PCDATA;
				break;
			case Node.ELEMENT_NODE:
				newNodeName = newNode.getNodeName();
				break;
			default:
				return false;
		}
		int oldState = state;
		if (newNode.getNextSibling() == null) state = CHECK;
		boolean insertable = isNodeInsertableAt(newNodeName, index);
		state = oldState;
		return insertable;
	}

	public boolean isNodeInsertableAt(String newNodeName, int index) {
		if ((dtd == null) || (targetTable == null) || (state < CHECK)) return true;

		if (state == STRICT_CHECK)
			targetTable = dtd.getInsertableElementsForValidContent(targetElement, index, targetTable);
		else
			targetTable = dtd.getInsertableElements(targetElement, index, targetTable);
		InsertableElement insertable = (InsertableElement) targetTable.get(newNodeName);
		return ((insertable != null) && insertable.status);
	}


	public Set getNewAttributes() {
		Set result = new TreeSet();
		if ((dtd != null) && (state >= NO_CHECK) && (targetElement != null)) {
			Enumeration attrs = dtd.getAttributeDeclarations(targetElement.getNodeName());
			while (attrs.hasMoreElements()) {
				AttDef attr = (AttDef) attrs.nextElement();
				if (!targetElement.hasAttribute(attr.getName()))
					result.add(attr.getName());
			}
		}
		return result;
	}

	public Set getAllAttributesForElementSet(String elementName) {
		Set result = new TreeSet();
		if (dtd != null) {
			Enumeration attrs = dtd.getAttributeDeclarations(elementName);
			while (attrs.hasMoreElements()) {
				AttDef attr = (AttDef) attrs.nextElement();
				result.add(attr.getName());
			}
		}
		return result;
	}

	public boolean acceptsAttributes(String elemName, Collection attributes) {
        if ((dtd == null) || (state < CHECK)) return true;
		for (Iterator iter = attributes.iterator(); iter.hasNext();) {
			String attrName = (String) iter.next();
			if (dtd.getAttributeDeclaration(elemName, attrName) == null) return false;
		}
		return true;
	}

	public Set getRequiredAttributeNames(String elemName) {
		Set result = new HashSet();
		Enumeration attributeDecls = dtd.getAttributeDeclarations(elemName);
		while (attributeDecls.hasMoreElements()) {
			AttDef attDef = (AttDef) attributeDecls.nextElement();
			if (attDef.getDefaultType() == AttDef.REQUIRED)
				result.add(attDef.getName());
		}
		return result;
	}

	public int getAttributeDefaultType(String elemName, String attrName) {
		if (dtd == null) return UNKNOWN;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return UNKNOWN;
		return attDef.getDefaultType();
	}

	public int getAttributeValueType(String elemName, String attrName) {
		if (dtd == null) return UNKNOWN;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return UNKNOWN;
		return attDef.getDeclaredType();
	}

	public int getEnumValuesSize(String elemName, String attrName) {
		if (dtd == null) return 0;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return 0;
		return attDef.size();
	}

	public String[] getEnumValues(String elemName, String attrName) {
		if (dtd == null) return null;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return null;
		int size = attDef.size();
		if (size == 0) return null;
		String[] result = new String[size];
		for (int i = 0; i < size; i++)
			result[i] = attDef.elementAt(i);
		Arrays.sort(result, String.CASE_INSENSITIVE_ORDER);
		return result;
	}

	public String getAttributeDefaultValue(String elemName, String attrName) {
		if (dtd == null) return null;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return null;
		return attDef.getDefaultStringValue();
	}

	public boolean isAttributeRemovable(String elemName, String attrName, String attrValue) {
		if ((dtd == null) || (state < CHECK)) return true;
		AttDef attDef = dtd.getAttributeDeclaration(elemName, attrName);
		if (attDef == null) return true; // if this attribute is not valid, it may be removed
		int attrType = attDef.getDefaultType();
		String defaultStringValue = attDef.getDefaultStringValue();
		return ((attrType != FIXED) && (attrType != REQUIRED) &&
				((defaultStringValue == null) || !defaultStringValue.equals(attrValue)));
	}

	public void setState(int state) {
	// if (!((dtd == null) && (state > 0)))
		this.state = state;
	// newState = state;
	}

	public Node validate(Node node) {
		if (node.getNodeType() != Node.ELEMENT_NODE) return null;
		if ((dtd == null) || (state == NO_DTD)) return null;

		if (node instanceof TXElement)
			node = dtd.validate((TXElement) node);
		else
			node = dtd.validate(new TXWrapper(node));
		if (node == null) return null;

		if (node instanceof TXWrapper)
			return ((TXWrapper) node).delegateNode;
		if (node.getNodeType() == Node.ATTRIBUTE_NODE) {
			Document ownerDocument = ((Attr) node).getOwnerElement().getOwnerDocument();
			switch (getAttributeValueType((Attr) node)) {
				case IDREF:
					if (ownerDocument.getElementById(node.getNodeValue()) != null) node = null;
					break;
				case IDREFS:
					StringTokenizer tokenizer = new StringTokenizer(node.getNodeValue());
					boolean allFound = true;
					while (tokenizer.hasMoreTokens()) {
						String idref = tokenizer.nextToken();
						if (ownerDocument.getElementById(idref) == null) {
							allFound = false;
							break;
						}
					}
					if (allFound) node = null;
			}
		}
		return node;
	}

	public Node validateDeep(Node branch) {
		if ((dtd == null) || (state == NO_DTD)) return null;
		if (branch.getNodeType() == Node.DOCUMENT_NODE) branch = ((Document) branch).getDocumentElement();
		if (branch.getNodeType() != Node.ELEMENT_NODE) return null;
		Node invalidNode = validate(branch);
		if (invalidNode != null) return invalidNode;
		Node nextNode = branch.getFirstChild();
		if (nextNode != null) {
			invalidNode = validateDeep(nextNode);
			if (invalidNode != null) return invalidNode;
		}
		// todo only step down at the first level
		nextNode = branch.getNextSibling();
		if (nextNode != null)
			invalidNode = validateDeep(nextNode);
		return invalidNode;
	}

	public List getAllInvalidNodes(Node branch) {
		List result = new LinkedList();
		if ((dtd != null) && (state != NO_DTD)) {
			if (branch.getNodeType() == Node.DOCUMENT_NODE)
				branch = ((Document) branch).getDocumentElement();
			Node invalidNode = validate(branch);
			if (invalidNode != null)
				result.add(invalidNode);
			Node node = branch.getFirstChild();
			if (node != null)
				getAllInvalidNodes(node, result);
		}
		return result;
	}

	private void getAllInvalidNodes(Node branch, List result) {
		Node invalidNode = validate(branch);
		if (invalidNode != null)
			result.add(invalidNode);
		Node node = branch.getFirstChild();
		if (node != null)
			getAllInvalidNodes(node, result);
		node = branch.getNextSibling();
		if (node != null)
			getAllInvalidNodes(node, result);
	}

	private static Node convertNodeToTX(final Node node) {
		if (node == null) return null;
		if (node instanceof TXWrapper) return node;
		if (node.getNodeType() == Node.TEXT_NODE)
			return new TXText(node.getNodeValue()) {
				public Node getNextSibling() {
					return convertNodeToTX(node.getNextSibling());
				}
				public Node getPreviousSibling() {
					return convertNodeToTX(node.getPreviousSibling());
				}
				public Node getParentNode() {
					return convertNodeToTX(node.getParentNode());
				}
				public boolean getIsIgnorableWhitespace() {
					return getText().trim().length() == 0;
				}
			};
		return new TXWrapper(node);
	}


	public static final class TXWrapper extends TXElement {//implements Element {
		Node delegateNode;
		Element delegateElement;

		public TXWrapper(Node delegateNode) {
			super(delegateNode.getNodeName());
			this.delegateNode = delegateNode;
			if (delegateNode.getNodeType() == Node.ELEMENT_NODE)
				delegateElement = (Element) delegateNode;
		}

		public String toString() {
			return delegateNode.toString();
		}

		public String getNodeName() {
			return delegateNode.getNodeName();
		}

		public String getNodeValue()
				throws DOMException {
			return delegateNode.getNodeValue();
		}

		public void setNodeValue(String nodeValue)
				throws DOMException {
			delegateNode.setNodeValue(nodeValue);
		}

		public short getNodeType() {
			return delegateNode.getNodeType();
		}

		public Node getParentNode() {
			return delegateNode.getParentNode(); // todo: do I need TXWrapper?
		}

		public NodeList getChildNodes() {
			return delegateNode.getChildNodes(); // todo: do I need TXWrapper?
		}

		public Node getFirstChild() {
			return convertNodeToTX(delegateNode.getFirstChild());
		}

		public Node getLastChild() {
			return convertNodeToTX(delegateNode.getLastChild());
		}

		public Node getPreviousSibling() {
			return convertNodeToTX(delegateNode.getPreviousSibling());
		}

		public Node getNextSibling() {
			return convertNodeToTX(delegateNode.getNextSibling());
		}

		public NamedNodeMap getAttributes() {
			return delegateNode.getAttributes();
		}

		public Document getOwnerDocument() {
			return delegateNode.getOwnerDocument();
		}

		public Node insertBefore(Node newChild, Node refChild) throws DOMException {
			return delegateNode.insertBefore(newChild, refChild);
		}

		public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
			return delegateNode.replaceChild(newChild, oldChild);
		}

		public Node removeChild(Node oldChild) throws DOMException {
			try {
				return delegateNode.removeChild(oldChild);
			} catch (DOMException e) {
				return oldChild; // todo
			}
		}

		public Node appendChild(Node newChild) throws DOMException {
			return delegateNode.appendChild(newChild);
		}

		public boolean hasChildNodes() {
			return delegateNode.hasChildNodes();
		}

		public Node cloneNode(boolean deep) {
			return delegateNode.cloneNode(deep); // todo: do I need TXWrapper?
		}

		public void normalize() {
			delegateNode.normalize();
		}

		public boolean isSupported(String feature, String version) {
			return delegateNode.isSupported(feature, version);
		}

		public String getNamespaceURI() {
			return delegateNode.getNamespaceURI();
		}

		public String getPrefix() {
			return delegateNode.getPrefix();
		}

		public void setPrefix(String prefix)
				throws DOMException {
			delegateNode.setPrefix(prefix);
		}

		public String getLocalName() {
			return delegateNode.getLocalName();
		}

		public boolean hasAttributes() {
			return delegateNode.hasAttributes();
		}

		public String getTagName() {
			return delegateElement.getTagName();
		}

		public String getAttribute(String name) {
			return delegateElement.getAttribute(name);
		}

		public void setAttribute(String name,
								 String value)
				throws DOMException {
			delegateElement.setAttribute(name, value);
		}

		public void removeAttribute(String name)
				throws DOMException {
			delegateElement.removeAttribute(name);
		}

		public Attr getAttributeNode(String name) {
			return delegateElement.getAttributeNode(name);
		}

		public Attr setAttributeNode(Attr newAttr)
				throws DOMException {
			return delegateElement.setAttributeNode(newAttr);
		}

		public Attr removeAttributeNode(Attr oldAttr)
				throws DOMException {
			return delegateElement.removeAttributeNode(oldAttr);
		}

		public NodeList getElementsByTagName(String name) {
			return delegateElement.getElementsByTagName(name);
		}

		public String getAttributeNS(String namespaceURI,
									 String localName) {
			return delegateElement.getAttributeNS(namespaceURI, localName);
		}

		public void setAttributeNS(String namespaceURI,
								   String qualifiedName,
								   String value)
				throws DOMException {
			delegateElement.setAttributeNS(namespaceURI, qualifiedName, value);
		}

		public void removeAttributeNS(String namespaceURI,
									  String localName)
				throws DOMException {
			delegateElement.removeAttributeNS(namespaceURI, localName);
		}

		public Attr getAttributeNodeNS(String namespaceURI,
									   String localName) {
			return delegateElement.getAttributeNodeNS(namespaceURI, localName);
		}

		public Attr setAttributeNodeNS(Attr newAttr)
				throws DOMException {
			return delegateElement.setAttributeNodeNS(newAttr);
		}

		public NodeList getElementsByTagNameNS(String namespaceURI,
											   String localName) {
			return delegateElement.getElementsByTagNameNS(namespaceURI, localName);
		}

		public boolean hasAttribute(String name) {
			return delegateElement.hasAttribute(name);
		}

		public boolean hasAttributeNS(String namespaceURI,
									  String localName) {
			return delegateElement.hasAttributeNS(namespaceURI, localName);
		}
	}

	/* (non-Javadoc)
	 * @see at.ac.tuwien.asgaard.util.xml.dtd.DTDHelper#createMinimalValidElement(java.lang.String)
	 */
	public Element createMinimalValidElement(String newElementName, String defaultAttributeValue) {
        if (targetNode == null)
            return null;  // debug
		Element newElement = targetNode.getOwnerDocument().createElement(newElementName);

		if ((dtd != null))
			if ((state >= CHECK))
				addRequiredContent(newElement, defaultAttributeValue);
			else
				addRequiredAttributes(newElement, defaultAttributeValue);

//  ContentModel contentModel = dtd.getContentModel(newElementName);
//  CMNode contentModelNode = contentModel.getContentModelNode();
//  System.out.println("mincc = "+getMinimalChildrenCount(contentModelNode));

		return newElement;
	}

/* private int getMinimalChildrenCount(CMNode cmNode) {
		if (cmNode instanceof CM2op) {
			CM2op cm2opNode = (CM2op) cmNode;
			if (cm2opNode.getType() == ',')
				return getMinimalChildrenCount(cm2opNode.getLeft()) + getMinimalChildrenCount(cm2opNode.getRight());
			return Math.min(getMinimalChildrenCount(cm2opNode.getLeft()), getMinimalChildrenCount(cm2opNode.getRight()));
		}
		if (cmNode instanceof CM1op) {
			CM1op cm1opNode = (CM1op) cmNode;
			if (cm1opNode.getType() == '+')
				return getMinimalChildrenCount(cm1opNode.getNode());
			return 0;
		}
		System.out.print(((CMLeaf)cmNode).getName()+ " ");
		return 1;
	}*/

	public void addRequiredAttributes(Element element, String defaultAttributeValue) {
		if (dtd == null) return;
		Enumeration attrs = dtd.getAttributeDeclarations(element.getNodeName());
		Enumeration identifiers = null;
		while (attrs.hasMoreElements()) {
			AttDef attDef = (AttDef) attrs.nextElement();
			if ((attDef.getDefaultType() == REQUIRED) && !element.hasAttribute(attDef.getName()))
				switch (attDef.getDeclaredType()) {
					case AttDef.IDREF:
					case AttDef.IDREFS:
						if (identifiers == null)
							identifiers = ((CoreDocumentImpl) element.getOwnerDocument()).getIdentifiers();
						if (identifiers.hasMoreElements())
							element.setAttribute(attDef.getName(), (String) identifiers.nextElement());
						else
							element.setAttribute(attDef.getName(), "undefined"); // todo: invalid!
						break;
					case AttDef.NAME_TOKEN_GROUP:
						Enumeration tokens = attDef.elements();
						if (tokens.hasMoreElements())
							element.setAttribute(attDef.getName(), (String) tokens.nextElement());
						else
							element.setAttribute(attDef.getName(), "undefined"); // todo: should not happen
						break;
					default:
						element.setAttribute(attDef.getName(), "undefined"); // todo
				}
		}
	}

	public boolean addRequiredContent(Element element, String defaultAttributeValue) {
		addRequiredContentInternal(element, new HashSet<String>(), defaultAttributeValue);
		return (element.hasAttributes() || element.hasChildNodes());
	}

/* public void addRequiredContentRecursive(Element element) {
		for (Node childNode = element.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
			addRequiredContentRecursive((Element) childNode);
		addRequiredContent(element);
	}*/

	private void addRequiredContentInternal(Element currentElement, Set<String> nodeNames, String defaultAttributeValue) {
		addRequiredAttributes(currentElement, defaultAttributeValue);

        if (currentElement.hasChildNodes()) return; // TODO: doesn't work currently for an Element with children (needs getInsertableElements)

		String nodeName = currentElement.getNodeName();
		if (nodeNames.contains(nodeName)) return;
		nodeNames.add(nodeName);
//  if ((validate(currentElement) == null)) return; // todo: if validate returns an Attr-Node (-> there were invalid
		if (dtd.checkContent(new TXWrapper(currentElement))) 
		    return;

        int result = 0, width = 1;
		Hashtable table = dtd.prepareTable(nodeName);
        if (table == null) // if currentElement is not definded in current DTD
            result = -1;
		while (result == 0)
			result = tryAddNextChild(currentElement, table, width++);
		if (result == -1) return;
		for (Node childNode = currentElement.getFirstChild(); childNode != null; childNode = childNode.getNextSibling())
			addRequiredContentInternal((Element) childNode, new HashSet<String>(nodeNames), defaultAttributeValue);
	}

	private int tryAddNextChild(Element currentElement, Hashtable table, int width) {
		table = dtd.getAppendableElements(currentElement, table); 

		List insertableElements = convertAppendableTableToList(table);
		int size = insertableElements.size();
		if (size == 0) return -1;
		Document ownerDocument = currentElement.getOwnerDocument();
		for (int i = 0; i < size; i++) {
			String newNodeName = (String) insertableElements.get(i);
			Node newNode;
			if (newNodeName == SPECIAL_NODE_NAME_TEXT)
				newNode = ownerDocument.createTextNode("???"); // todo: text value?
			else
				newNode = ownerDocument.createElement(newNodeName);
			currentElement.appendChild(newNode);
		// if (isValid(currentElement) && dtd.checkContent(new TXWrapper(newNode))) return 1;
			if (dtd.checkContent(new TXWrapper(currentElement)) 
			        && dtd.checkContent(new TXWrapper(newNode))) 
			    return 1;
			currentElement.removeChild(newNode);
		}
		for (int i = 0; i < size; i++) {
			String newNodeName = (String) insertableElements.get(i);
			Node newNode = ownerDocument.createElement(newNodeName);
			currentElement.appendChild(newNode);
			if (isValid(currentElement)) return 1;
			if (width > 1) {
				int result = tryAddNextChild(currentElement, table, width - 1);
				if (result == 1) return 1;
			}
			currentElement.removeChild(newNode);
		}
		return 0;
	}

	private static List convertAppendableTableToList(Hashtable table) {
		List result = new ArrayList();
		Random random = new Random();
		int size = -1;
		for (Enumeration elements = table.elements(); elements.hasMoreElements();) {
			InsertableElement insertable = (InsertableElement) elements.nextElement();
			if (!insertable.name.equals(at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode.DELTA_LINK_ELEMENT_NAME) &&
					!insertable.name.equals(at.ac.tuwien.asgaard.delta.gui.xml.XMLTreeNode.GMT_LINK_ELEMENT_NAME) &&
					(insertable.name != DTD.CM_EOC) && (insertable.name != DTD.CM_ERROR) && insertable.status)
				result.add((++size == 0) ? 0 : random.nextInt(size), insertable.name);
		}
		return result;
	}

	public boolean isLoaded() {
		return (state != NO_DTD) && (dtd != null);
	}

	private static class ParserDummy implements ErrorListener, StreamProducer {
		public int error(String fileName, int lineNo, int charOffset, Object key, String msg) {
			System.err.println("Error occured while parsing DTD \""+ fileName +"\" at "+lineNo+"/"+(charOffset-1)+":\n    "+key+" - "+msg);
			return 1; // return-value is never used
		}

		public void closeInputStream(Source source) { }
		public Source getInputStream(String name, String publicID, String systemID) throws IOException {
			System.out.println("DTDHelper: resolving entity - systemID =  "+ systemID);
			return new Source(new URL(systemID).openStream()); //new BufferedReader(new FileReader(systemID)));
		}
		public void loadCatalog(Reader reader) throws IOException { }
	}
}
