/*
 * Project: GMT - Guideline Markup Tool / Version: 2.7.3.#452
 * Author: Peter Votruba
 * Date: 27.08.2003
 * Time: 13:37:04
 */
package at.ac.tuwien.asgaard.util.xml.xerces;

import at.ac.tuwien.asgaard.util.Log;
import org.apache.xerces.impl.dtd.XMLDTDValidator;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.DTDConfiguration;
import org.apache.xerces.xni.*;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.util.HashMap;
import java.util.Map;

/**
 * Class ErrorReportingDOMParser ... todo
 *
 * @version 2.7.3.#452
 * @author Peter Votruba
 * @since JDK 1.3.0
 */
public final class ErrorReportingDOMParser extends DOMParser {
	// todo: this solution is very unaesthetic ...

	Node currentEmptyElement;
	private ErrorReportingXMLDTDValidator xmlDTDValidator;

	public ErrorReportingDOMParser() {
		super(new ErrorReportingDTDConfiguration());
		xmlDTDValidator = ((ErrorReportingDTDConfiguration)fConfiguration).getXMLDTDValidator();
		xmlDTDValidator.setDomParser(this);
		setErrorHandler(xmlDTDValidator);
	}

	public void emptyElement(QName element, XMLAttributes attributes, Augmentations augs) throws XNIException {
		startElement(element, attributes, augs);
		currentEmptyElement = fCurrentNode;
		endElement(element, augs);
	} // emptyElement(QName,XMLAttributes)

	Node getCurrentNode() {
		return fCurrentNode;
	}

	public Map<Integer, Node> getLineNumberToNodeMap() {
		return xmlDTDValidator.getLineNumberToNodeMap();
	}
	public Map<Node, Integer> getNodeToLineNumberMap() {
		return xmlDTDValidator.getNodeToLineNumberMap();
	}
	public Map<Node, ErrorWrapper> getElement2ErrorMap() {
		return xmlDTDValidator.getElement2ErrorMap();
	}

	public void reinitialize() {
//		xmlDTDValidator.setLastParseError(null, true);
		xmlDTDValidator.getElement2ErrorMap().clear();
		xmlDTDValidator.getLineNumberToNodeMap().clear();
		xmlDTDValidator.getNodeToLineNumberMap().clear();
	}

	private static class ErrorReportingDTDConfiguration extends DTDConfiguration {
		protected XMLDTDValidator createDTDValidator() {
			return new ErrorReportingXMLDTDValidator();
		} // createDTDValidator():XMLDTDValidator

		ErrorReportingXMLDTDValidator getXMLDTDValidator() {
			return (ErrorReportingXMLDTDValidator)fDTDValidator;
		}
	}

	/**
	 * Class ErrorReportingXMLDTDValidator ... todo
	 *
	 * @author Peter Votruba
	 * @version 2.7.3.#452
	 * @since JDK 1.4.0
	 */
	private static final class ErrorReportingXMLDTDValidator extends XMLDTDValidator
			implements ErrorHandler {
		private final Map<Node, ErrorWrapper> element2ErrorMap;
		private final Map<Integer, Node> lineNumberToNodeMap;
		private final Map<Node, Integer> nodeToLineNumberMap;
		private ErrorReportingDOMParser domParser;

//		private SAXParseException lastParseError;
//		private boolean lastParseErrorType; // true ... error, false ... warning
		private Node lastStartElement;

		ErrorReportingXMLDTDValidator() {
			element2ErrorMap = new HashMap<Node, ErrorWrapper>();
			lineNumberToNodeMap = new HashMap<Integer, Node>();
			nodeToLineNumberMap = new HashMap<Node, Integer>();
		}

		void setDomParser(ErrorReportingDOMParser domParser) {
			this.domParser = domParser;
		}

		public Map<Node, ErrorWrapper> getElement2ErrorMap() {
			return element2ErrorMap;
		}

		public Map<Node, Integer> getNodeToLineNumberMap() {
			return nodeToLineNumberMap;
		}

		public Map<Integer, Node> getLineNumberToNodeMap() {
			return lineNumberToNodeMap;
		}

/*		void setLastParseError(SAXParseException lastParseError, boolean lastParseErrorType) {
			this.lastParseError = lastParseError;
			this.lastParseErrorType = lastParseErrorType;
		}*/


		public void startElement(QName element, XMLAttributes attributes, Augmentations augs)
				throws XNIException {
			super.startElement(element, attributes, augs);
			Node currentNode = domParser.getCurrentNode();
			int lineNumber = fDocLocation.getLineNumber();
			lineNumberToNodeMap.put(lineNumber, currentNode);
			nodeToLineNumberMap.put(currentNode, lineNumber);
/*			if (lastParseError != null) {
				element2ErrorMap.put(currentNode, new ErrorWrapper(lastParseErrorType, currentNode, lastParseError));
				lastParseError = null;
			}*/
			lastStartElement = currentNode;
		} // startElement(QName,XMLAttributes)

		protected void handleEndElement(QName element, Augmentations augs, boolean isEmpty) throws XNIException {
			super.handleEndElement(element, augs, isEmpty);
			Node currentNode = domParser.getCurrentNode();
			if (lastStartElement.getParentNode() == currentNode)
				currentNode = lastStartElement;
			lineNumberToNodeMap.put(fDocLocation.getLineNumber(), currentNode);
/*			if (lastParseError != null) {
				element2ErrorMap.put(currentNode, new ErrorWrapper(lastParseErrorType, currentNode, lastParseError));
				lastParseError = null;
			}*/
		} // handleEndElement(QName,boolean)

		public void emptyElement(QName element, XMLAttributes attributes, Augmentations augs)
				throws XNIException {
			boolean removed = handleStartElement(element, attributes, augs);
			fDocumentHandler.emptyElement(element, attributes, augs);
			Node currentNode = domParser.currentEmptyElement;
			int lineNumber = fDocLocation.getLineNumber();
			lineNumberToNodeMap.put(lineNumber, currentNode);
			nodeToLineNumberMap.put(currentNode, lineNumber);
			/*if (lastParseError != null) {
				element2ErrorMap.put(currentNode, new ErrorWrapper(lastParseErrorType, currentNode, lastParseError));
				lastParseError = null;
			}*/
			if (!removed) {
				super.handleEndElement(element, augs, true);
/*				if (lastParseError != null) {
					element2ErrorMap.put(currentNode,
										 new ErrorWrapper(lastParseErrorType, currentNode, lastParseError));
					lastParseError = null;
				}*/
			}
		} // emptyElement(QName,XMLAttributes)

		// ErrorHandler methods

		public void warning(SAXParseException exc) {
			Log.trace("XML-Parser - Warning in " + exc.getSystemId() + "(line #" + exc.getLineNumber() + ", col #" + exc.getColumnNumber() +
					  "): " + exc.getMessage());
//			setLastParseError(exc, false);
			element2ErrorMap.put(domParser.fCurrentNode, new ErrorWrapper(false, domParser.fCurrentNode, exc));
		}

		public void error(SAXParseException exc) {
			Log.error("XML-Parser - Error in " + exc.getSystemId() + "(line #" + exc.getLineNumber() + ", col #" + exc.getColumnNumber() +
					  "): " + exc.getMessage());
//			setLastParseError(exc, true);
			element2ErrorMap.put(domParser.fCurrentNode, new ErrorWrapper(true, domParser.fCurrentNode, exc));
		}

		public void fatalError(SAXParseException exc) throws SAXException {
			Log.error("XML-Parser - fatal Error in " + exc.getSystemId() + "(line #" + exc.getLineNumber() + ", col #" +
					  exc.getColumnNumber() + "): " + exc.getMessage());
			throw exc;
		}
	}
}
